vansetup_exists := isbound (`vansetup, nil);

vcFac.Create ("vcFlagEdit", vscmVT);

define_enum ("vssOverwriteVT", list ("Only specified design unit",
                                     "Any file",
                                     "No files"));
overwrite_flags := ["overwriteself", "overwrite", "nooverwrite"];
default_overwrite := 0;

define_enum ("vssShowBuildVT", list ("Compile only",
                                     "Show design units first, then compile",
                                     "Show design units only"));
show_build_flags := ["noshowbuild", "showbuild", "showbuildonly"];
default_show_build := 0;

the_analyzer := nil;

function run_analyzer(vsi, cmd, src) {
    shell_prefs := list (env <- SYS_ENV, kill_on_close <- TRUE);
    function exit_handler(mode, code) {
        if (the_analyzer != nil) {
            the_analyzer.pending = cdr (the_analyzer.pending);
            if (the_analyzer.pending != nil) {
                cmd_src := car (the_analyzer.pending);
                vcShell.Restart (the_analyzer.shell, car (cmd_src),
                                 shell_prefs);
                setup_subprocess ();
            } /* if */
            else {
                display_queue ();
                vcWin.Modified (the_analyzer.window, FALSE);
                vcRegion.Enable (the_analyzer.dismiss_button, TRUE, TRUE);
                if (vsi != nil)
                    (update_browser (vsi)) (mode, code);
            } /* else */
        } /* if */
     } /* function exit_handler */

    function display_queue() {
        vcWin.ClearStatus (the_analyzer.window);
        first := TRUE;
        title := "Vantage Analyzer (idle)";
        foreach (cmd_src; vscript.list_iter (the_analyzer.pending)) {
            src := convert (vlStringVT, cdr (cmd_src));
            if (src[0] != '-') {
                parsed := vscript.vsParse (src);
                src = vscript.vsMerge ("", "", caddr (parsed),
                                       cadddr (parsed));
            } /* if */
            if (first) {
                first = FALSE;
                title = "Vantage Analyzer: " + src;
                vcWin.AddStatus (the_analyzer.window, "Pending compiles: ");
            } /* if */
            else
                vcWin.AddStatus (the_analyzer.window, ", ");
            vcWin.AddStatus (the_analyzer.window, src);
        } /* foreach */
        vcWin.SetTitle (the_analyzer.window, title);
    } /* function display_queue */

    function setup_subprocess() {
        vcWin.Modified (the_analyzer.window, TRUE);
        vcRegion.Enable (the_analyzer.dismiss_button, FALSE, TRUE);
        sys := vcShell.Sys (the_analyzer.shell);
        vcSys.CallOnExit (sys, exit_handler);
        if (vcMac.IsReplayingOn ())
            vcMac.SubprocessHold (sys, 0);
        display_queue ();
    } /* function setup_subprocess */

    function show_errors() {
        stattext := vcRegion.Create ("", vcStaticText.fac,
                                     vtx.GetBuffer (the_analyzer.shell));
        vcRegion.Font (stattext, vgFont.small);
        buttons := vcRegion.Create ("", vcButnRow.fac,
                                    list (list ("dismiss", "Dismiss",
                                                vcWin.wclose, nil)));
        scroll := vcRegion.Create ("Error Log", vcScroll.fac, stattext,
                                   size <- vgSize.Create (100, 300,
                                                          700, 500,
                                                          1000, 1000));
        table := vcTable.Make ("", 2, 1, scroll, buttons);
        vcWin.Create (vcRegion.Window (the_analyzer.shell), table);
    } /* function show_errors */

    if (the_analyzer == nil) {
        shell := vcRegion.Create1 ("", vcShell.fac, cmd, shell_prefs);
        buttons := vcRegion.Create ("", vcButnRow.fac,
                                    list (list ("dismiss", "Dismiss",
                                                vcWin.wclose, nil),
                                          list ("show",
                                                "Show errors in small font",
                                                show_errors, nil)));
           
        t := vcTable.Make ("Vantage Analyzer", 2, 1, shell, buttons);
        the_analyzer = struct {
            shell = shell;
            dismiss_button = vcButnRow.Get (buttons, 0);
            window = vcWin.Create (t);
            pending = list (cons (cmd, src))
        };
        vcWin.DestroyNotify (the_analyzer.window,
                             func(w) the_analyzer = nil;);
        vcWin.SetConfirmMessage (the_analyzer.window, "

 If you close the analyzer window, the analyzer will be
 non-gracefully terminated, and all of the queued analyze
 commands will be discarded.

");
        setup_subprocess ();
    } /* if */
    else {
        if (the_analyzer.pending == nil) {
            the_analyzer.pending = list (cons (cmd, src));
            vcShell.Restart (the_analyzer.shell, cmd, shell_prefs);
            setup_subprocess ();
        } /* if */
        else {
            for (p := the_analyzer.pending; cdr (p) != nil; p = cdr (p))
                ;
            set_cdr (p, list (cons (cmd, src)));
            display_queue ();
        } /* else */
    } /* else */
} /* function run_analyzer */

function build_command_line(libs, overwrite, showbuild, fromline,
                            maxerr, maxdesunits, flags, listing)
{
    cmd := "";
    foreach (flag; string_loop (flags, "\n"))
        cmd += " " + flag;

    overwrite = convert (vlIntVT, overwrite);
    if (overwrite != default_overwrite)
        cmd += " -" + overwrite_flags[overwrite + 0];
    showbuild = convert (vlIntVT, showbuild);
    if (showbuild != default_show_build)
        cmd += " -" + show_build_flags[showbuild + 0];
    if (fromline != 1)  cmd += sprintf (" -fromline %d", fromline);
    if (!string_eq (maxerr, "")) cmd += " -maxerr " + maxerr;
    if (!string_eq (maxdesunits, "")) cmd += " -maxdesunits " + maxdesunits;
    if (!string_eq (listing, "")) cmd += " -lst " + listing;

    cmd + " " + libs;
} /* function build_command_line */

function analyze_without_setup(w, e, src, flags, overwrite, showbuild,
                               fromline, maxerr, maxdesunits, listing)
{
    cmd := "analyze -src " + src +
        build_command_line ("-libfile vsslib.ini", overwrite, showbuild,
                            fromline, maxerr, maxdesunits, flags, listing);

    run_analyzer (nil, cmd, src);
} /* function analyze_without_setup */

common_flags :=
    list (list ("Debug", "Enable VHDL source level debugging",
                "-dbg 2", "-dbg 0"),
          list ("Range Check", "Produce code for runtime range checking",
                nil, "-nobndchk"),
          list ("Dependencies", "Compile out-of-date dependencies",
                nil, "-nodep"),
          list ("Halt on error",
"Stop compiling design units as soon as one is found to contain syntax errors",
                nil, "-nohaltonerr"),
          list ("Fail on warning", 
                "Compiler emit a nonzero return code if a warning is found",
                nil, "-nowarnrc"));

check_flag := 
    list ("Check only", "Do not write any new files to the design library",
          "-chkonly", nil);

force_flag :=
    list ("Force", "Force recompilation even if time stamps are okay",
          "-force", nil);

flags_with_check :=
    reverse_destructive (cons (check_flag, reverse (common_flags)));
flags_with_force :=
    reverse_destructive (cons (force_flag, reverse (common_flags)));

/*
 * There are two analyze commands, most of whose options are the same,
 * except for the first couple.  We use this list and the non-var-args
 * form of vcWv.Cmd: vcCmd.Create, where it's easier to form the list
 * of args with a call to append.
 */
common_params :=
    list (vcWv.Param ("Overwrite", vssOverwriteVT, vcWv.Sticky,
                      default_overwrite,
                      vcWv.Secondary,
                      "Specify whether compiler should overwrite existing design library files"),
          vcWv.Param ("Show Build", vssShowBuildVT, vcWv.Sticky,
                      default_show_build,
                      vcWv.Secondary,
                      "Specify whether analyzer should compile and/or show which units require compilation"),
          vcWv.Param ("First line", vlPositiveIntVT, vcWv.Sticky, 1,
                      vcWv.Secondary,
                      "Specifies the line in the source file where compilation is to commence."),
          vcWv.Param ("Maximum Errors", vlStringVT, vcWv.Sticky, "",
                      vcWv.Secondary,
                      "Specifies the number of syntax errors at which the compiler will terminate."),
          vcWv.Param ("Maximum Design Units", vlStringVT, vcWv.Sticky, "",
                      vcWv.Secondary,
                      "Specifies the maximum number of design units to be compiled."),
          vcWv.Param ("Listing Filename", vlStringVT, vcWv.Sticky, "",
                      vcWv.Secondary,
                      "Specifies a compilation listing file."));


/*
 * This little bit of magic does this:
 *      cons2 (a, b, c, list (d, e, f)) -> list (a, b, c, d, e, f)
 */
function cons2(args...) {
    if (args == nil)
        nil;
    else if (cdr (args) == nil)
        car (args);
    else
        cons (car (args), apply (cons2, cdr (args)));
} /* function cons2 */

flg_str := "flags_with_check";

if (vansetup_exists) {
    lib_flag_table := nil;

    function get_obj_libs(vsi) {
        if (lib_flag_table == nil) {
            lib_flag_table = vlSym.StrCreate (3);
            lib_flag_table[vsMerge ("", SYS_PATH, "ieee", "lib")] = "-libieee";
            lib_flag_table[vsMerge ("", SYS_PATH, "math", "lib")] = "-libmath";
            lib_flag_table[vsMerge ("", SYS_PATH, "vantage_logic", "lib")] =
                    "-libvantage_logic";
        } /* if */

        function lib_cmd_spec(obj) {
            p := browse.obj_pathname (obj);
            flag := lib_flag_table[p];
            if (flag == nil)
                "-lib " + vsRelPath (p);
            else
                flag;
        } /* function lib_cmd_spec */

        this_lib := browse.obj_lib (current_obj (vsi));
        lib_str := lib_cmd_spec (this_lib);
        foreach (lib; current_library_objs (vsi))
            if (this_lib != lib)
                lib_str += " " + lib_cmd_spec (lib);
        lib_str;
    } /* function get_obj_libs */

    function analyze_src(vsi, e, src, flags, overwrite, showbuild, fromline,
                         maxerr, maxdesunits, listing)
    {
        if (vsi.is_modified)
            save_ini (vsi, e);

        if (isbound (`vsRelPath, nil))
            src = vsRelPath (src);

        cmd := "analyze -src " + src +
            build_command_line ("-libfile " + vsi.init_filename, overwrite,
                                showbuild, fromline, maxerr, maxdesunits,
                                flags, listing);
        run_analyzer (vsi, cmd, src);
    } /* function analyze_src */

    function analyze_obj(vsi, e, flags, overwrite, showbuild,
                         fromline, maxerr, maxdesunits, listing)
    {
        if (vsi.is_modified)
            save_ini (vsi, e);

        src_info := browse.design_unit_string (current_obj (vsi));
        cmd := "analyze " + src_info +
            build_command_line (get_obj_libs (vsi), overwrite, showbuild,
                                fromline, maxerr, maxdesunits, flags,
                                listing);

        run_analyzer (vsi, cmd, src_info);
    } /* function analyze_obj */

    function confirm_analyze(args...) {
        vsi := car (args);
        if (vsi.is_modified)
    "The library initialization file, " + vsi.init_filename + ",
    has been modified.  It will be automatically saved before
    analysis begins.

    Save " + vsi.init_filename + " and run the analyzer?";
        else
            FALSE;
    } /* function confirm_analyze */

    analyze_src_cmd :=
        apply (vcWv.CmdWithConfirm,
               cons2 (vansetup.fac, analyze_src, confirm_analyze, TRUE,
                      "analyze_src", "Analyze VHDL Source File",
                      vcWv.Param ("VHDL Source File", vlFileReadVT,
                                  vcWv.Sticky, vcWv.NoDefault,
                                  vcFileSelect.fac, filter <- "*.vhd*"),
                      vcWv.Param ("Flags", vlStringVT,
                                  vcWv.Sticky, list ("-dbg 2"),
                                  vcWv.Secondary, vcFlagEdit.fac,
                                  flags <- "vansetup.flags_with_check"),
                      common_params));
    vcCmd.SetHelp (analyze_src_cmd,
                "Analyze a VHDL source file into the current working library");
    vcCmd.EnableApply (analyze_src_cmd, TRUE);

    analyze_obj_cmd :=
        apply (vcWv.CmdWithConfirm,
               cons2 (vansetup.fac, analyze_obj, confirm_analyze,
                      TRUE, "analyze_obj", "Analyze VHDL Design Unit",
                      vcWv.Param ("Flags", vlStringVT,
                                  vcWv.Sticky, list ("-dbg 2"),
                                  vcWv.Secondary, vcFlagEdit.fac,
                                  flags <- "vansetup.flags_with_force"),
                      common_params));
    vcCmd.SetHelp (analyze_obj_cmd,
                   "Re-analyze the selected VHDL design object");
    vcCmd.EnableApply (analyze_obj_cmd, TRUE);
    flg_str := "vansetup.flags_with_check";
} /* if */

analyze_without_setup_cmd :=
    vcCmd.Create (vcWin.fac, analyze_without_setup, "analyze_src",
                  "Analyze VHDL Source File",
                  cons2 (vcWv.Param ("VHDL Source File",
                                     vlStringVT, vcWv.Sticky, "",
                                     vcFileSelect.fac,
                                     filter <- "*.vhd*",
                                     read_now <- TRUE),
                         vcWv.Param ("Flags", vlStringVT,
                                     vcWv.Sticky, "-dbg 2",
                                     vcWv.Secondary, vcFlagEdit.fac,
                                     flags <- flg_str),
                         common_params));
vcCmd.SetHelp (analyze_without_setup_cmd,
               "Analyze a VHDL source file into the current working library");
vcCmd.EnableApply (analyze_without_setup_cmd, TRUE);

