/*
 * Name:      VDRAW.VS
 * Facility:  Viewdraw
 * 
 * Description: 
 *    Viewscript file for Viewdraw.  (Copied initially from vplace.vs)
 *
 * Environment:
 *     SunOS 4.1 & CORE 5.0
 *     
 * Author:  
 *   Ron Maxwell, February 12, 1991  (vplace.vs)
 *
 * Modified by: 
 * Who            Version   When      What
 *
 * PJS
 * 
 * (c) Copyright 1991, 1992 by Viewlogic Systems, Inc.
 */

/* Symbols for default parameter strategies */
STICKY   := vcWv.Sticky;
CONSTANT := vcWv.Constant;
DYNAMIC  := vcWv.Dynamic;

/* Symbols for snaping point to grid or not */
NOSNAP := 0;
SNAP   := 1;

/* Symbols for initmove callback */
FROM_MOVE   := 0;
FROM_COPY   := 1;
FROM_CREATE := 2;
FROM_PASTE  := 3;

from_where  := FROM_MOVE;  /* global variable used by initmove to deterimine */
                           /* where the move operation is from */

/* ON OFF symbols */
ON  := "on";
OFF := "off";

/*
 * COLOR symbols
 *
 * BLACK        := "Black";
 * BLUE         := "Blue";
 * GREEN        := "Green";
 * CYAN         := "Cyan";
 * RED          := "Red";
 * MAGENTA      := "Magenta";
 * BROWN        := "Brown";
 * LT_GREY      := "LightGrey";
 * DARK_GREY    := "DarkGrey";
 * LT_BLUE      := "LightBlue";
 * LT_GREEN     := "LightGreen";
 * LT_CYAN      := "LightCyan";
 * LT_RED       := "LightRed";
 * LT_MAGENTA   := "LightGagenta";
 * YELLOW       := "Yellow";
 * WHITE        := "White";
 */

/* YES NO symbols */
YES := convert (vlYesNoVT, "yes");
NO  := convert (vlYesNoVT, "no");

/*
 * Data types - vqDataType
 *
 * "Schematic"
 * "Symbol"
 */

/* Objects */

OBJ_COMPONENT  := "component";
OBJ_NET        := "net";

/*
 * Sheet sizes
 *
 * ASIZE   := "asize";
 * BSIZE   := "bsize";
 * CSIZE   := "csize";
 * DSIZE   := "dsize";
 * ESIZE   := "esize";
 * A4_SIZE := "a4_size";
 * A3_SIZE := "a3_size";
 * A2_SIZE := "a2_size";
 * A1_SIZE := "a1_size";
 * A0_SIZE := "a0_size";
 *
 * ZSIZE   := "zsize";
 */

/* Values may be modified by user in vdrawbt.vs */

enable_fast_mode    := 1;
zoom_in_and_center  := YES;
auto_scroll_enabled := YES;

add_net_mode        := 4;        /* 4[.x] or 5[.0] */
add_line_mode       := 4;        /* 4[.x] or 5[.0] */
move_mode           := 4;        /* 4[.x] or 5[.0] */

move_warp_cursor    := 5;        /* 5 - Center cursor in bounding box. */
copy_warp_cursor    := move_warp_cursor;
add_warp_cursor     := 0;   /* 3 */
bpaste_warp_cursor  := 0;   /* 3 */
rotate_warp_cursor  := -1;  /* If set to 0, macro playback will be diff. */
scale_warp_cursor   := 5;
pan_warp_cursor     := NO;

enable_gestures     := 1;  /* -1 for middle, 1 for shift middle.  0 for off. */

/* IniFileName := "viewdraw.ini"; */

/* ---------------------------------------------------------------------- */

function status_msg(w, str) {

    vcWin.ClearStatus (w);

    if (str != nil)
        vcWin.AddStatus (w, str);

} /* function status_msg */


/* ---------------------------------------------------------------------- */

function file_lock_str (display, event)
{
    declare libcompext_name;

    data_type := getblockdatatype (display);
    blkname := getblkfullname (display);
    owner := getlockowner (display);
    id := getlockpid (display);
    host := getlockhost (display);
    date := getlockdate (display);

    if (data_type == vqDataType.Schematic)
        prompt := " Schematic ";
    else
        prompt := " Symbol ";


    str := "
Unable to obtain lock for " + prompt + blkname + "

File is locked by:

     Username = " + owner + "
     Date/time = " + date + "
     Process id = " + id + "
     Host name = " + host;
}    

/* Builds and displays the dialog box giving options for handling 
 * condition where the user is attempting to execute a command which 
 * will modify the data, but a lock could not be obtained for this
 * block.  Options are presented in buttons which execute the 
 * desired command.
 */
function file_lock_options (display, event, proc)
{
    declare popup;

    str := file_lock_str (display, event);

    table := vcTable.Create ("File Locked!", 2, 1);
    vcTable.Add (table, text := vcRegion.Create1 ("msg", vcStaticText.fac, str, nil));

    buttons := vcRegion.Create ("Buttons", vcButnRow.fac,
                        list (list ("Cancel", "Cancel",
                                    func()
                                        vcWin.Close (popup);
                                    , list (macro_on <- TRUE,
                                          close_window <- TRUE)),
                              list ("WriteTo", "WriteTo",
                                    func() {
                                        vcWin.Close (popup);
                                        vcCmd.Run (writeto_cmd, display,
                                                   event, nil, TRUE);
                                    },
                                    nil)));

    vcTable.Add (table, buttons);
    popup = vcWin.Popup (vcEvent.Window (event), table, text, 7);

} /* file_lock_options */

function file_mod_str (display, event)
{
    declare libcompext_name;

    data_type := getblockdatatype (display);
    blkname := getblkfullname (display);

    if (data_type == vqDataType.Schematic)
        prompt := " Schematic ";
    else
        prompt := " Symbol ";


    str := prompt + blkname + " has been modified on disk 

";
}    

/* Builds and displays the dialog box giving options for handling 
 * condition where the user is attempting to execute a command which 
 * will modify the data, but the file on disk has been modified since
 * being opened.  Options are presented in buttons which execute the 
 * desired command.
 */
function file_mod_options (display, event, proc)
{
    declare popup;

    str := file_mod_str (display, event);

    table := vcTable.Create ("File Modified", 2, 1);
    vcTable.Add (table, text := vcRegion.Create1 ("msg", vcStaticText.fac, str, nil));

    buttons := vcRegion.Create ("Buttons", vcButnRow.fac,
                        list (list ("Read", "Read file from disk",
                                    func() {
                                        vcCmd.Run (_read_cmd, display, 
                                                   event, nil, TRUE);
                                        vcWin.Close (popup);
                                    },
                                    nil),
                              list ("Continue", "Overwrite file and continue",
                                    func() {
                                        lockfile (display);
                                        vcCmd.Run (_save_cmd, display, 
                                                   event, nil, TRUE);
                                        proc(display, event);
                                        vcWin.Close (popup);
                                    },
                                    nil),
                              list ("Cancel", "Cancel",
                                    func()
                                        vcWin.Close (popup);
                                    , list (macro_on <- TRUE,
                                          close_window <- TRUE))));

    vcTable.Add (table, buttons);
    popup = vcWin.Popup (vcEvent.Window (event), table, text, 7);

} /* file_mod_options */

function PROC0(proc)
{
    tmp := func (d, e)
           {
               proc (d, e);
           };

    return (tmp);
}

function PROC1(proc, p1)
{
    tmp := func (d, e)
           {
               proc (d, e, p1);
           };

    return (tmp);
}

function PROC2(proc, p1, p2)
{
    tmp := func (d, e)
           {
               proc (d, e, p1, p2);
           };

    return (tmp);
}

function PROC3(proc, p1, p2, p3)
{
    tmp := func (d, e)
           {
               proc (d, e, p1, p2, p3);
           };

    return (tmp);
}

function PROC4(proc, p1, p2, p3, p4)
{
    tmp := func (d, e)
           {
               proc (d, e, p1, p2, p3, p4);
           };

    return (tmp);
}

function PROC5(proc, p1, p2, p3, p4, p5)
{
    tmp := func (d, e)
           {
               proc (d, e, p1, p2, p3, p4, p5);
           };

    return (tmp);
}

function PROC6(proc, p1, p2, p3, p4, p5, p6)
{
    tmp := func (d, e)
           {
               proc (d, e, p1, p2, p3, p4, p5, p6);
           };

    return (tmp);
}

function FILE_CHECK0(proc)
{
    tmp := func (d, e)
           {
               /* if the file being worked on is not currently locked,
                * attempt to put a lock on it.
                */
               if (!islocked (d) || !ismylock (d))
                   lockfile (d);

               if (islocked (d) && !ismylock (d))
                   file_lock_options (d, e, PROC0 (proc));
               else if (filemodified (d))
                   file_mod_options (d, e, PROC0 (proc));
               else 
                   proc (d, e);
           };

    return (tmp);
}

function FILE_CHECK1(proc)
{
    tmp := func (d, e, p1)
           {
               if (!islocked (d) || !ismylock (d))
                   lockfile (d);

               if (islocked (d) && !ismylock (d))
                   file_lock_options (d, e, PROC1 (proc, p1));
               else if (filemodified (d))
                   file_mod_options (d, e, PROC1 (proc, p1));
               else
                   proc (d, e, p1);
           };

    return (tmp);
}

function FILE_CHECK2(proc)
{
    tmp := func (d, e, p1, p2)
           {
               if (!islocked (d) || !ismylock (d))
                   lockfile (d);

               if (islocked (d) && !ismylock (d))
                   file_lock_options (d, e, PROC2 (proc, p1, p2));
               else if (filemodified (d))
                   file_mod_options (d, e, PROC2 (proc, p1, p2));
               else
                   proc (d, e, p1, p2);
           };

    return (tmp);
}

function FILE_CHECK3(proc)
{
    tmp := func (d, e, p1, p2, p3)
           {
               if (!islocked (d) || !ismylock (d))
                   lockfile (d);

               if (islocked (d) && !ismylock (d))
                   file_lock_options (d, e, PROC3 (proc, p1, p2, p3));
               else if (filemodified (d))
                   file_mod_options (d, e, PROC3 (proc, p1, p2, p3));
               else
                   proc (d, e, p1, p2, p3);
           };

    return (tmp);
}

function FILE_CHECK4(proc)
{
    tmp := func (d, e, p1, p2, p3, p4)
           {
               if (!islocked (d) || !ismylock (d))
                   lockfile (d);

               if (islocked (d) && !ismylock (d))
                   file_lock_options (d, e, PROC4 (proc, p1, p2, p3, p4));
               else if (filemodified (d))
                   file_mod_options (d, e, PROC4 (proc, p1, p2, p3, p4));
               else
                   proc (d, e, p1, p2, p3, p4);
           };

    return (tmp);
}

function FILE_CHECK5(proc)
{
    tmp := func (d, e, p1, p2, p3, p4, p5)
           {
               if (!islocked (d) || !ismylock (d))
                   lockfile (d);

               if (islocked (d) && !ismylock (d))
                   file_lock_options (d, e, PROC5 (proc, p1, p2, p3, p4, p5));
               else if (filemodified (d))
                   file_mod_options (d, e, PROC5 (proc, p1, p2, p3, p4, p5));
               else
                   proc (d, e, p1, p2, p3, p4, p5);
           };

    return (tmp);
}

function FILE_CHECK6(proc)
{
    tmp := func (d, e, p1, p2, p3, p4, p5, p6)
           {
               if (!islocked (d) || !ismylock (d))
                   lockfile (d);

               if (islocked (d) && !ismylock (d))
                   file_lock_options6 (d, e, PROC6 (proc, p1, p2, p3, p4, p5, p6));
               else if (filemodified (d))
                   file_mod_options6 (d, e, PROC6 (proc, p1, p2, p3, p4, p5, p6));
               else
                   proc (d, e, p1, p2, p3, p4, p5, p6);
           };

    return (tmp);
}

/* ---------------------------------------------------------------------- */

function maybe_run_cmd(title, str, command, display, event, arglist)
{
    w := vcEvent.Window (event);

    table := vcTable.Create (title, 2, 1);
    vcTable.Add (table, vcRegion.Create ("msg", vcStaticText.fac, str));

    buttons := vcTable.Create ("buttons", 1, 2);

    yes_button :=
        vcWv.LabelButtonCreate ("yes", "Yes",
                                nil,
                                func()
                                    vcCmd.Run (command, display,
                                               event, arglist, TRUE);
                                , list (close_window <- TRUE));

    vcTable.Add (buttons, yes_button);

    no_button :=
        vcWv.LabelButtonCreate ("no", "No",
                                nil, func() {;},
                                list (close_window <- TRUE));

    vcTable.Add (buttons, no_button);

    vcTable.Add (table, buttons);
    popup := vcWin.Popup (w, table, no_button,
                          vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);

    vcWin.Forward (w, popup);

} /* function maybe_run_cmd */


/* Create a command that does nothing */
noop_cmd := vcWv.Cmd (fac, func(d, e);, "NoOperation");
vcCmd.EnableMacro (noop_cmd, "off");

noopmode_cmd := vcWv.Cmd (fac, func(d, e)
                              status_msg (vcEvent.Window (e),
                                          "No current operation mode");
                          , "None");
vcCmd.EnableMacro (noopmode_cmd, "off");


/* Create binding table that can be used instead of the standard table */
bt := vcBind.Create ();

/* Clears the binding table "bt" */
function clear_binding_table() {vcBind.clear (bt);}

/* Unbind the current binding table from all current displays */
unbind_cmd := vcWv.Cmd (fac, func(d, e) {unbind (d);});

/* Bind a command to the middle button */

cmd_sch := noop_cmd;
cmd_sym := noop_cmd;

def_gesture_cmd := nil;

function bind_middle_button(cmd)
{
    if (enable_gestures == 1)  /* 1 == Shift middle is used for gestures. */
        vcBind.Set (bindings, '\middledown', cmd);

    else if (enable_gestures == -1) {

        def_gesture_cmd = cmd;
        vcGesture.Set (gestures, vcGesture.Default, cmd);
    }
    else
        vcBind.Set (bindings, '\middledown', cmd);
}


function get_middle_button_cmd()
{
    declare mb_cmd;

    if (enable_gestures == 1)  /* 1 == Shift middle is used for gestures. */
        mb_cmd = vcBind.Get (bindings, '\middledown');

    else if (enable_gestures == -1)
        mb_cmd = def_gesture_cmd;

    else
        mb_cmd = vcBind.Get (bindings, '\middledown');        

    mb_cmd;

} /* function get_middle_button_cmd */

function get_middle_button_cmd()
    /*
     * Note that the return value of this function is in the else clauses
     * of the if statement.
     */
    if (enable_gestures == 1)  /* 1 == Shift middle is used for gestures. */
        vcBind.Get (bindings, '\middledown');

    else if (enable_gestures == -1)
        def_gesture_cmd;

    else
        vcBind.Get (bindings, '\middledown');        

    /*
     * DON'T ADD ANYTHING HERE
     */


/* Bind a command to the right button */
function bind_right_button(cmd)
    vcBind.Set (bindings, '\rightdown', cmd);


/*
 * This function returns a command which when executed binds the command (cmd)
 * passed in to the middle button. This is used for commands in menus and
 * typed commands which need to wait for a point to be specified before 
 * execution can go on.  After that the middle button is bound to that command 
 * until another command is executed from the menu which rebinds it.
 */

function make_middle_button_cmd(cmd, name)
    button_cmd := vcWv.Cmd (fac, func(d, e)
                                bind_middle_button (cmd);
                            , name);

    /* If you uncomment this, add curly braces and return button_cmd. */
    /* vcCmd.EnableMacro (button_cmd, "off"); */

 /* function make_middle_button_cmd */


/* This function returns another function that gets the value of the 
 * specified mode
 */
/*
function get_mode(mode) func(mode) getmode(mode);;
*/

/* This function returns another function that gets the value of the 
 * specified value
 */
function get_value(value) func(display) getvalue (value);;

/* This function returns another function that gets the value of the 
 * specified value
 */
function get_valuei(value) func(display) getvaluei (value);;

/* This function returns another function that gets the visibility of the 
 * specified object type
 */
/* function get_obj_vis(obj) func(display) getobjtypevis(obj);; */

/* This function returns another function that gets the color of the 
 * specified object type
 */
function get_obj_color(obj) func(display) getcolor (obj);;

/* This function returns another function that gets the linestyle of the 
 * specified object type
 */
function get_obj_lstyle(obj) func(display) getlstyle (obj);;

/* This function returns another function that gets the fillstyle of the 
 *  specified object type
 */
function get_obj_fstyle(obj) func(display) getfstyle (obj);;



/*
 *
 */

function getblkfullname(display)
{
    lib   := getblklib (display);
    name  := getblkname (display);
    sheet := getblksheet (display);

    if (!string_eq (lib, ""))
        lib + ":" + name + "." + convert (string, sheet);
    else
        name + "." + convert (string, sheet);

} /* function getblkfullname */


function getblknameext(display)
{
    name  := getblkname (display);
    sheet := getblksheet (display);

    name + "." + convert (string, sheet);

} /* function getblknameext */


/*
 * This function returns an event which contains the designated origin point
 * of the selected objects MBB.  This function is used by numerous routines,
 * such as move, to begin an operation from a menu entry or keyboard command
 * when there is no initial point specified.
 */

function set_starting_point(display, event, origin)
{
    /* origin values   1  4  7
     *                 2  5  8
     *                 3  6  9
     *
     * Use  0 if CURRENT cursor position is desired (no warping).
     * Use -1 if no change is desired.
     */

    declare e;
    x := 0; y := 0;

    /* if something selected then calculate center point */

    if (   (origin > 0)
        && (isnumselobj (display, vqObject.Any, vqObjectCount.AtLeastOne))) {

        /* get MBB of selected objects */
        mbb := mbb_selected (display);

        x = 0;
        y = 0;

        /* calculate point */
        if ((origin == 1) || (origin == 2) || (origin == 3))
            x = vgRect.Xmin (mbb);

        if ((origin == 4) || (origin == 5) || (origin == 6))
            x = vgRect.Xmin (mbb) + ((vgRect.Xmax(mbb) - vgRect.Xmin(mbb))/2);

        if ((origin == 7) || (origin == 8) || (origin == 9))
            x = vgRect.Xmax (mbb);

        if ((origin == 1) || (origin == 4) || (origin == 7))
            y = vgRect.Ymax (mbb);

        if ((origin == 2) || (origin == 5) || (origin == 8))
            y := vgRect.Ymin(mbb) + ((vgRect.Ymax(mbb) - vgRect.Ymin(mbb))/2);

        if ((origin == 3) || (origin == 6) || (origin == 9))
            y = vgRect.Ymin (mbb);

        /* convert to pixel coordinates */
        vgU.UserToPixel (getviewport (display), &x, &y);

        /* Test x, y to make sure that they are within Viewdraw window. 
         */

        r := vgU.Region (getviewport (display));

        if (x < vgRect.Xmin (r))
            x = vgRect.Xmin (r);

        if (vgRect.Xmax (r) < x)
            x = vgRect.Xmax (r);

        if (y < vgRect.Ymin (r))
            y = vgRect.Ymin (r);

        if (vgRect.Ymax (r) < y)
            y = vgRect.Ymax (r);

        /* create new event */
        e = vcEvent.Create (x, y,
                            vcEvent.Ch (event),
                            /* vcEvent.String (event), */
                            "",
                            vcEvent.Window (event),
                            vcEvent.Region (event), nil);
    }

    /* The cursor has probably moved.  Get current cursor position */

    else if (   (origin == 0)
             && (vcMac.IsReplayingOn () == FALSE)
             && (vgU.QueryCursor (getviewport (display), &x, &y) != FALSE)) {

        vgU.UserToPixel (getviewport (display), &x, &y);
            
        e = vcEvent.Create (x, y,
                            vcEvent.Ch (event),
                            /* vcEvent.String (event), */
                            "",
                            vcEvent.Window (event),
                            vcEvent.Region (event), nil);
    }
    else 
        /* Cursor not in window or origin < 0. just return original event. */
        e = vcEvent.Duplicate (event);

    e;

} /* function set_starting_point */


function vdraw_quit()
{
    count := 0;
    blklist := "";

    foreach (blkname; lsblocks (0)) {

        ++count;
        blklist = blklist + "     " + blkname + "\n";
    }

    if (count > 0)

        blklist = "
 The following Schematics and Symbols have been edited.   

" + blklist + "
 You may discard edits now and quit, or cancel the request  
 to quit and go back and write your changes to disk.";

    blklist;
}
vcQuit.AddFunction (vdraw_quit);


function vdraw_wclose(w)
{
    /* Generic message for now.  Later list modified data that could
     * be lost, if any.  If none, just close it.
     */

    blklist := "";

    foreach (blkname; lswinblocks (w, 0))
        blklist = blklist + "      " + blkname + "\n";

    if (string_eq (blklist, "") == FALSE) {

        str := "
 The following Schematics and Symbols have been edited.

" + blklist + "
 You may discard edits now and close the window, or cancel  
 the request to close and go back and write your changes to  
 disk.

";
        table := vcTable.Create ("Close?", 2, 1);
        vcTable.Add (table, vcRegion.Create ("msg", vcStaticText.fac, str));

        buttons := vcTable.Create ("buttons", 1, 2);

        default_button := vcWv.LabelButtonCreate ("cancel",
                                                  "Cancel, do NOT Close",
                                                  nil, func();,
                                                  list (macro_on <- TRUE,
                                                        close_window <- TRUE));
        vcTable.Add (buttons, default_button);

        close_button := vcWv.LabelButtonCreate ("close",
                                                "Discard changes, then Close",
                                                nil, func() vcWin.Destroy (w);,
                                                list (macro_on <- TRUE,
                                                      close_window <- TRUE));
        vcTable.Add (buttons, close_button);
        
        vcTable.Add (table, buttons);
        vcWin.Popup (w, table, default_button,
                     vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
    }
    else
        vcWin.Destroy (w);

} /* function vdraw_wclose */

/******************************************************************************
   readRecoveryAlert() - If problems were found in the database file then
                         this routine is called to alert the user and block
                         them from ViewDraw until they acknowledge this 
                         message. (VFE)
******************************************************************************/
function readRecoveryAlert(vdrawWindow)
{
declare alertWindow;

   /***************************************************************************
      recoverContinue - Callback for the continue button on the recovery 
                        alert window.  Kill the alert window.
   ***************************************************************************/
   function recoverContinue()
   {
      if (alertWindow != nil)
         vcWin.Destroy(alertWindow);
   }

   /******************* begin readRecoveryAlert  ****************************/

   /* Create a table for a message row and a button row */
   table := vcTable.Create("Recovery Alert", 2, 1);

   message := vcTable.Create("Recovery Message", 10, 1);

   vcTable.Add(message, vcRegion.Create("msg1", vcStaticText.fac,
                        "   "));

   vcTable.Add(message, vcRegion.Create("msg2", vcStaticText.fac,
                        "Some problems were found in the database."));
                         
   vcTable.Add(message, vcRegion.Create("msg3", vcStaticText.fac,
                        "It is important to review these in the alert box."));

   vcTable.Add(message, vcRegion.Create("msg4", vcStaticText.fac,
                        "   "));

   vcTable.Add(message, vcRegion.Create("msg5", vcStaticText.fac,
                        "Recovery has taken place to read this database into ViewDraw."));

   vcTable.Add(message, vcRegion.Create("msg6", vcStaticText.fac,
                        "But these changes have not been saved persistently."));

   vcTable.Add(message, vcRegion.Create("msg7", vcStaticText.fac,
                        "   "));

   vcTable.Add(message, vcRegion.Create("msg8", vcStaticText.fac,
                        "To save these changes select File=>Write within ViewDraw."));

   vcTable.Add(message, vcRegion.Create("msg9", vcStaticText.fac,
                        "If this is not acceptable then close the window or quit ViewDraw."));

   vcTable.Add(message, vcRegion.Create("msg10", vcStaticText.fac,
                        "   "));

   /* Add the message to the table */
   vcTable.Add(table, message);

   /* Create the labeled button */
   button := vcWv.LabelButtonCreate("RecoverContinue", "Continue", nil,
                                    recoverContinue,
                                    list (close_window <- TRUE));

   /* Add the button to the table */
   vcTable.Add(table, button);

   if (vdrawWindow != nil) {
      alertWindow = vcWin.Popup(vdrawWindow, table, button, 
                             vcWin.NoCmdline+vcWin.NoStatus+vcWin.NoBindings);

      vcWin.Forward(vdrawWindow, alertWindow);
      }
}

/* ======================================================================
 *
 * File Commands.  Open, close, read, etc. 
 *
 * Need to use $LIBRARY and $PROJECT as original defaults.
 */

function contains_wildcards(str)
    !((lindex (str, '*') == FALSE) && (lindex (str, '?') == FALSE));

 /* function contains_wildcards */


/* File Open Schematic.  This command run from File Open dialog box. */

function window_open_schematic(e, name)
{
    if (e != nil)
        vcWin.Busy (vcEvent.Window (e), TRUE);

    window := schematic (name);

    if (window != nil) {

        /* If there were any problems with the database then create a
           recovery alert window. (VFE) */

        if (readRecoveryDone() == 1)  
           readRecoveryAlert(window);

        vcWin.CallOnClose (window, vdraw_wclose);
        }

    if (e != nil)
        vcWin.Busy (vcEvent.Window (e), FALSE);

} /* function window_open_schematic */

sch_cmd := vcWv.Cmd (vcWin.fac, func(d, e, name)
                     {
                         window_open_schematic (e, name);
                         if (!fopen_is_apply)
                             if (fopen_window != nil)
                                 vcWin.Close (fopen_window);
                     }, "_schematic", "Schematic",
                     vcWv.Param ("Schematic Name",
                                 string, STICKY, ""));

function open_sch(display, event, str)
    if (string_eq (str, "") || contains_wildcards (str))
        build_fopen_dialog (vcEvent.Window (event), vqDataType.Schematic);
    else
        vcCmd.Run (sch_cmd, display, event, list (str), TRUE);

schematic_cmd := vcWv.Cmd (fac, open_sch, "schematic", "Schematic",
                           vcWv.Param ("Schematic name",
                                       vlStringRestOfLineVT, STICKY, "*"));

vcCmd.EnableMacro (schematic_cmd, "off");


/* File Open Symbol.  This command run from File Open dialog box. */

function window_open_symbol(e, name)
{
    if (e != nil)
        vcWin.Busy (vcEvent.Window (e), TRUE);

    window := symbol (name);

    /* If there were any problems with the database then create a
       recovery alert window. (VFE) */

    if (readRecoveryDone() == 1) 
       readRecoveryAlert(window);

    if (window != nil)
        vcWin.CallOnClose (window, vdraw_wclose);

    if (e != nil)
        vcWin.Busy (vcEvent.Window (e), FALSE);

} /* function window_open_symbol */

sym_cmd := vcWv.Cmd (fac, func(d, e, name)
                         window_open_symbol (e, name);
                     , "_symbol", "Symbol",
                     vcWv.Param ("Symbol Name",
                                 string, DYNAMIC, getblknameext));

function open_sym(display, event, str)
{
    if (string_eq (str, "") || contains_wildcards (str))
        build_fopen_dialog (vcEvent.Window (event), vqDataType.Symbol);
    else
        vcCmd.Run (sym_cmd, display, event, list (str), TRUE);
}
symbol_cmd := vcWv.Cmd (fac, open_sym, "symbol", "Symbol",
                        vcWv.Param ("Symbol name",
                                    vlStringRestOfLineVT, STICKY, "*"));

vcCmd.EnableMacro (symbol_cmd, "off");

fileopen_menu_cmd := vcWv.Cmd (fac, func(d, e)
                                   /* Default to schematic. */
                                   open_sch (d, e, "*");
                                /* No name.  Not meant to be typed */ );

/* vcCmd.EnableMacro (fileopen_menu_cmd, "off"); */

function file_open(data_type, name)
{
    /* This function is called from Viewdraw's main routine on startup.
     * The name - "file_open" is hard-coded in the Viewdraw executable.
     */

    if (string_eq (name, ""))
        build_fopen_dialog (nil, data_type);

    else if (data_type == vqDataType.Schematic)
        window_open_schematic (nil, name);

    else
        window_open_symbol (nil, name);

} /* function file_open */


/* Window Copy */
wcopy_cmd := vcWv.Cmd (fac, wcopy, "wcopy");

/* ======================================================================
 *
 * Toggle auto-scrolling while moving objects or creating nets/buses
 */

function toggle_auto_scroll(d, e)
    auto_scroll_enabled = !auto_scroll_enabled;

 /* function toggle_auto_scroll */

toggle_auto_scroll_cmd := vcWv.Cmd (fac, toggle_auto_scroll);


/* ======================================================================
 *
 * View changes while moving objects or creating nets/buses
 */


function add_drag_bindings(bt, drag_continue, enable_auto_scroll)
{
    auto_scroll_enabled = enable_auto_scroll;

    function pan_drag(display, event)
    {
        pan (display, event, TRUE);
        drag_continue (display, event);
    }
    pan_drag_cmd := vcWv.Cmd (fac, pan_drag);

    function in_drag(display, event)
    {
        view (display, event, 1.4, zoom_in_and_center);
        drag_continue (display, event);
    }
    in_drag_cmd := vcWv.Cmd (fac, in_drag);

    function out_drag(display, event)
    {
        view (display, event, 0.6, NO);
        drag_continue (display, event);
    }
    out_drag_cmd := vcWv.Cmd (fac, out_drag);

    function scroll_drag(display, event)
    {
        scroll (display, event, 0.75, TRUE);
        drag_continue (display, event);
    }
    scroll_drag_cmd := vcWv.Cmd (fac, scroll_drag);

    function refresh_drag(display, event)
    {
        vrefresh (display);
        drag_continue (display, event);
    }
    refresh_drag_cmd := vcWv.Cmd (fac, refresh_drag);

    function full_drag(display, event)
    {
        full (display, event);
        drag_continue (display, event);
    }
    full_drag_cmd := vcWv.Cmd (fac, full_drag);

    /* vcBind.Set (bt, '\F4',  scroll_drag_cmd); */  /* Never get F4's */

    vcBind.Set (bt, '\F5', refresh_drag_cmd); 
    vcBind.Set (bt, '\F6', pan_drag_cmd); 
    vcBind.Set (bt, '\F7', in_drag_cmd); 
    vcBind.Set (bt, '\F8', out_drag_cmd); 
    vcBind.Set (bt, '\F0', full_drag_cmd);    /* F-10 */

    vcBind.Set (bt, 's',   toggle_auto_scroll_cmd);

} /* function add_drag_bindings */


/* These command are for dragging a box in a window. */
drag_box_nosnap_cmd := vcWv.Cmd (fac, func(d, e)
                                 {
                                     contbox (d, e, NOSNAP);
                                     auto_scroll (d, e);
                                 });

drag_box_snap_cmd   := vcWv.Cmd (fac, func(d, e)
                                 {
                                     contbox (d, e, SNAP);
                                     auto_scroll (d, e);
                                 });

cancel_drag_box_cmd := vcWv.Cmd (fac, func(d, e)
                                 {
                                     canbox (d, e);
                                     unbind (d);

                                 }, "DragBoxCancel");

/* create a pop-up menu for cancelling box drag*/
popup_box_menu := vcWv.MakeMenu ("box",
                                  list ("Cancel", cancel_drag_box_cmd));

popup_box_menu_cmd := vcWv.Cmd (fac, func(d, e)
                                vcMenuPop.Create (popup_box_menu, e););

function make_select(cmdfunc, cmdname, add)
{
    select_bt := vcBind.Create ();

    /* This function ends the select operation */
    function end_select(display, event)
    {
        finbox (display, event);
        unbind (display);

        if (add == nil)
            cmdfunc (display, event);
        else
            cmdfunc (display, event, add);

        enable_cmds (display);
    }
    end_select_cmd := vcWv.Cmd(fac, end_select, cmdname + "Execute");

    /* bind the move key to continue dragging box */
    vcBind.Set (select_bt, '\move',     drag_box_nosnap_cmd);
    vcBind.Set (select_bt, '\^\move',   drag_box_nosnap_cmd);
    vcBind.Set (select_bt, '\#\move',   drag_box_nosnap_cmd);

    /* bind the leftup and shift-leftup to the end select command */
    vcBind.Set (select_bt, '\leftup',   end_select_cmd);
    vcBind.Set (select_bt, '\^\leftup', end_select_cmd);
    vcBind.Set (select_bt, '\#\leftup', end_select_cmd);

    add_drag_bindings (select_bt, func(d, e) { contbox (d, e, NOSNAP); }, YES);

    /* Initialize for box drag without snapping to grid */
    function start_select (display, event)
        if (initbox (display, event, "White", "Solid", NOSNAP) == 0)
            bind (display, select_bt);

    vcWv.Cmd (fac, start_select, cmdname);
}
select_cmd    := make_select (select,   "select",    OFF);
addselect_cmd := make_select (select,   "addselect", ON);
unselect_cmd  := make_select (unselect, "unselect",  nil);


function selectarea(display, event, llx, lly, urx, ury, add_select)
{
    setarea (display, llx, lly, urx, ury);
    select (display, event, add_select);

    enable_cmds (display);
}
selectarea_cmd := vcWv.Cmd (fac, selectarea, "selectarea", "Region",
                            vcWv.Param ("Lower Left X",
                                        vqBlockWidthVT, STICKY, 0),
                            vcWv.Param ("Lower Left Y",
                                        vqBlockHeightVT, STICKY, 0),
                            vcWv.Param ("Upper Right X",
                                        vqBlockWidthVT, STICKY, 100),
                            vcWv.Param ("Upper Right Y",
                                        vqBlockHeightVT, STICKY, 100),
                            vcWv.Param ("Add", vlOnOffVT, STICKY, OFF));

/*-*/

/* ======================================================================
 *
 * View commands.  Zoom, refresh, full, etc.
 */

/* This function makes commands for the zoom operation */
function make_zoom()
{
    function end_zoom(display, event)
    {
        finbox (display, event);
        unbind (display);

        zoom (display, event);
    }
    end_zoom_cmd := vcWv.Cmd (fac, end_zoom, "ZoomExecute");

    function cancel_zoom(display, event)
    {
        finbox (display, event);
        unbind (display);
    }
    cancel_zoom_cmd := vcWv.Cmd (fac, cancel_zoom, "ZoomCancel");

    /* create the pop-up menu for a move operation and the command to pop it */
    popup_zoom_menu := vcWv.MakeMenu ("zoom",
                                       list ("Cancel", cancel_zoom_cmd));

    popup_zoom_menu_cmd := vcWv.Cmd (fac, func(d, e)
                                         vcMenuPop.Create (popup_zoom_menu, e);
                                     );
    
    function start_zoom(display, event)
    {
        if (   (vcEvent.String (event) != nil)
            && (!string_eq (vcEvent.String (event), "")))

            bind_middle_button (zoom_cmd);

        else if (initbox (display, event, "White", "Solid", NOSNAP) == 0) {

            /* Initialize for box dragging */
            status_msg (vcEvent.Window (event), "Specify area for zoom");

            /* bind keys for dragging box */
            clear_binding_table ();

            vcBind.Set (bt, '\move',       drag_box_nosnap_cmd);
            vcBind.Set (bt, '\middleup',   end_zoom_cmd);
            vcBind.Set (bt, '\middledown', end_zoom_cmd);
            vcBind.Set (bt, '\leftdown',   end_zoom_cmd);
            vcBind.Set (bt, '\^[',         cancel_zoom_cmd);

            if (move_mode == 4)
                vcBind.Set (bt, '\rightdown',  cancel_zoom_cmd);
            else
                vcBind.Set (bt, '\rightdown',  popup_zoom_menu_cmd);

            add_drag_bindings (bt,
                               func(d, e) { contbox (d, e, NOSNAP); }, YES);

            /* binds the binding table */
            bind (display, bt);
        }
    }
    vcWv.Cmd (fac, start_zoom, "zoom");
}
zoom_cmd := make_zoom ();


/* viewcontext_cmd := vcWv.Cmd (fac, viewcontext, "viewcontext"); */

in_and_center_cmd := vcWv.Cmd (fac, func(d, e)
                                   view (d, e, 1.6, zoom_in_and_center);
                               ,
                               "in_and_center");

function view_scroll(display, event)
{
    if (   (vcEvent.String (event) != nil)
        && (!string_eq (vcEvent.String (event), "")))

        bind_middle_button (scroll_cmd);

    else
        scroll (display, event, 0.33, TRUE);   /* Used to be .75 */

}
scroll_cmd := vcWv.Cmd (fac, view_scroll, "scroll");


function view_zselect(display, event, str)
{
    declare factor;

    if (string_eq (str, ""))
        factor = 1.6;
    else
        factor = convert (float, str);

    zselect (display, event, factor);
}
zselect_cmd := vcWv.Cmd (fac, view_zselect, "zselect", "ViewZoomSelect",
                         vcWv.Param ("Factor",
                                     vlStringRestOfLineVT, STICKY, "2.0"));

zselect_menu_cmd := vcWv.Cmd (fac, func(d, e)
                                  view_zselect (d, e, "2.0");
                              , "_zselect-menu");

view_cmd := vcWv.Cmd (fac, view, "view", "View",
                      vcWv.Param ("Zoom Factor", vlPositiveFloatVT,
                                  STICKY, 1.0),
                      vcWv.Param ("Center region", vlYesNoVT,
                                  CONSTANT, "yes"));

full_cmd := vcWv.Cmd (fac, full, "full");


function view_in(display, event, str)
{
    declare factor;

    if (string_eq (str, ""))
        factor = 1.6;
    else
        factor = convert (float, str);

    view (display, event, factor, NO);   /* Could use zoom_in_and_center */
}
in_cmd := vcWv.Cmd (fac, func(d, e, f)
                        view_in (d, e, f);
                    , "in", "Zoom Factor",
                    vcWv.Param ("Zoom Factor", vlStringRestOfLineVT,
                                STICKY, "1.6"));

in_menu_cmd := vcWv.Cmd (fac, func(d, e)
                         { view_in (d, e, "1.6"); }, "_in-menu");

function view_out(display, event, str)
{
    declare factor;

    if (string_eq (str, ""))
        factor = 0.6;
    else
        factor = convert (float, str);

    view (display, event, factor, NO);   /* Could use zoom_in_and_center */
}
out_cmd := vcWv.Cmd (fac, func(d, e, f)
                         view_out (d, e, f);
                     , "out", "Zoom Factor",
                     vcWv.Param ("Zoom Factor", vlStringRestOfLineVT,
                                 STICKY, "0.6"));

out_menu_cmd := vcWv.Cmd (fac, func(d, e)
                           view_out (d, e, "0.6");, "_out-menu");

refresh_cmd := vcWv.Cmd (fac, func(d, e) vrefresh (d);, "refresh");


/* ======================================================================
 *
 * Edit commands
 */

/* Make the command for the move operation */
function make_move()
{
    gspace := 0;

    function end_move(display, event)
        if (finmove (display, event) == 0) {

            unbind (display);
            enable_cmds (display);
        }

    end_move_cmd := vcWv.Cmd (fac, end_move, "MoveExecute");

    function cancel_move(display, event)
        if (canmove (display, event) == 0) {

            unbind (display);
            enable_cmds (display);
        }

    cancel_move_cmd := vcWv.Cmd (fac, cancel_move, "MoveCancel");
    
    /* Toggle detail command */
    tdetail_cmd := vcWv.Cmd (fac, tdetail, "MoveDetail");

    /* continue move command */
    cont_move_cmd := vcWv.Cmd (fac, func(d, e)
                               {
                                   contmove (d, e);
                                   auto_scroll (d, e);
                               });

    function move_one_gspace(display, event)
    {
        vp := getviewport (display);

        x := vcEvent.X (event);
        y := vcEvent.Y (event);

        vgU.PixelToUser (vp, &x, &y);

        if (vcEvent.Ch (event) == '\left')
            x = x - gspace;

        else if (vcEvent.Ch (event) == '\right')
            x = x + gspace;

        else if (vcEvent.Ch (event) == '\up')
            y = y + gspace;

        else if (vcEvent.Ch (event) == '\down')
            y = y - gspace;

        else if (vcEvent.Ch (event) == '\#\left')
            x = x - (10 * gspace);

        else if (vcEvent.Ch (event) == '\#\right')
            x = x + (10 * gspace);

        else if (vcEvent.Ch (event) == '\#\up')
            y = y + (10 * gspace);

        else if (vcEvent.Ch (event) == '\#\down')
            y = y - (10 * gspace);

        vgU.MoveCursor (vp, x, y);
    }
    move_one_grid_cmd := vcWv.Cmd (fac, move_one_gspace);


    /* Rotate objects while moving */
    rot_move_cmd := vcWv.Cmd (fac, func(d, e)
                              {
                                  rotmove (d, e);
                                  auto_scroll (d, e);
                              }, "MoveRotate");

    /* create the pop-up menu for a move operation and the command to pop it */

    popup_move_menu := vcWv.MakeMenu ("move",
                                       list ("Detail", tdetail_cmd),
                                       list ("End",    end_move_cmd),
                                       list ("Cancel", cancel_move_cmd));

    popup_move_menu_cmd := vcWv.Cmd (fac, func(d, e)
                                         vcMenuPop.Create (popup_move_menu, e);
                                     );

    /* This function starts the move operation */
    function start_move(display, event)
    {
        gspace = ggspacing (display);        

        if (initmove (display, event, from_where) == 0) {

            from_where = FROM_MOVE;  /* set it back for move operation */

            clear_binding_table ();
            
            /* Move binding keys */

            vcBind.Set (bt, '\move',       cont_move_cmd);

            vcBind.Set (bt, '\left',       move_one_grid_cmd);
            vcBind.Set (bt, '\right',      move_one_grid_cmd);
            vcBind.Set (bt, '\up',         move_one_grid_cmd);
            vcBind.Set (bt, '\down',       move_one_grid_cmd);

            vcBind.Set (bt, '\middledown', end_move_cmd);
            vcBind.Set (bt, '\^[',         cancel_move_cmd);  /* Escape */

            if (move_mode == 4) {

                vcBind.Set (bt, '\leftdown',   tdetail_cmd);
                vcBind.Set (bt, '\rightdown',  cancel_move_cmd);
            }
            else {

                vcBind.Set (bt, '\leftdown',   end_move_cmd);
                vcBind.Set (bt, '\rightdown',  popup_move_menu_cmd);
            }

            add_drag_bindings (bt, contmove, YES);

            vcBind.Set (bt, '\F9', rot_move_cmd);

            /* binds the binding table */
            bind (display, bt);
        }
    }
    vcWv.Cmd (fac, FILE_CHECK0 (start_move), "do_move");
}
do_move_cmd := make_move ();

/* 
 * This function does a move operation using the specified origin point
 * of the MBB of the selected objects as the initial point.
 */

function move_center(display, event)
{
    if (   (vcEvent.String (event) != nil)
        && (!string_eq (vcEvent.String (event), ""))) {
        
        bind_middle_button (move_cmd);
        event = set_starting_point (display, event, move_warp_cursor);
    }

    vcCmd.Run (do_move_cmd, display, event, nil, FALSE);
}
move_cmd := vcWv.Cmd (fac, move_center, "move");


/* ----------------------------------------------------------------------
/* This function is called to move an object after creation
 */

function move_created_object(display, event)
{
    from_where = FROM_CREATE;
    vcCmd.Run (do_move_cmd, display,
               set_starting_point (display, event, add_warp_cursor),
               nil, FALSE);

} /* function move_created_object */


/* ----------------------------------------------------------------------
 * This function does a copy operation using the specified origin point
 * of the MBB of the selected objects as the initial point.
 */

function copy_center(display, event)
{
    if (   (vcEvent.String (event) != nil)
        && (!string_eq (vcEvent.String (event), ""))) {

        bind_middle_button (copy_cmd);
        event = set_starting_point (display, event, copy_warp_cursor);
    }

    if (copy (display, event) == 0) {

        from_where = FROM_COPY;
        vcCmd.Run (do_move_cmd, display, event, nil, FALSE);
    }
}
copy_cmd := vcWv.Cmd (fac, FILE_CHECK0 (copy_center), "copy");


/* ----------------------------------------------------------------------
 * This function makes commands for the stretch operation
 */
function make_stretch()
{
    /* command to end the stretch operation */
    end_stretch_cmd := vcWv.Cmd (fac, func(d, e)
                                     if (finstrch (d, e) == 0)
                                         unbind (d);
                                 , "StretchExecute");

    /* command to cancel the stretch operation */
    cancel_stretch_cmd := vcWv.Cmd (fac, func(d, e)
                                        if (canstrch (d, e) == 0)
                                            unbind (d);
                                    , "StretchCancel");

    /*  command to continue stretch operation */
    function cont_stretch(d, e)
        contstrch (d, e);
        /* auto_scroll (d, e);  add curlies when uncommenting this line */

    cont_stretch_cmd := vcWv.Cmd (fac, cont_stretch);

    /* create a pop-up menu for cancelling stretch */
    popup_stretch_menu :=
        vcWv.MakeMenu ("stretch", list ("Cancel", cancel_stretch_cmd));

    popup_stretch_menu_cmd :=
        vcWv.Cmd (fac, func(d, e)
                      vcMenuPop.Create (popup_stretch_menu, e); );

    /* This function starts the stretch operation, the first call
     * of contstrch looks for a point to stretch
     */

    function start_stretch(display, event)
    {
        if (contstrch (display, event) == 0) {

            clear_binding_table ();

            vcBind.Set (bt, '\move',       cont_stretch_cmd);
            vcBind.Set (bt, '\middledown', end_stretch_cmd);
            vcBind.Set (bt, '\leftdown',   end_stretch_cmd);
            vcBind.Set (bt, '\^[',         cancel_stretch_cmd);  /* Escape */

            if (move_mode == 4)
                vcBind.Set (bt, '\rightdown',  cancel_stretch_cmd);
            else
                vcBind.Set (bt, '\rightdown',  popup_stretch_menu_cmd);

            /* add_drag_bindings (bt, contstrch); */

            bind (display, bt);
        }
    }

    /* This function inits the stretch operation */
    function init_stretch(display, event)
        if (   (vcEvent.String (event) != nil)
            && (!string_eq (vcEvent.String (event), "")))
        
            bind_middle_button (stretch_cmd);

        else if (initstrch (display) == 0)

            /* Initialize stretch operation */
            start_stretch (display, event);

    vcWv.Cmd (fac, FILE_CHECK0 (init_stretch), "stretch");
}
stretch_cmd := make_stretch ();


/* ======================================================================
 * [Edit] Buffer command.  Cut, copy, append and paste.
 */

function bpaste_objects(display, event)
    if (bpaste (display, event) == 0) {

        from_where = FROM_PASTE;
        vcCmd.Run (do_move_cmd, display,
                   set_starting_point (display, event, bpaste_warp_cursor),
                   nil, FALSE);
    }

bpaste_cmd := vcWv.Cmd (fac, FILE_CHECK0 (bpaste_objects), "bpaste");

bcut_cmd :=
    vcWv.Cmd (fac, 
              FILE_CHECK0 (func(d, e) { bcut (d, e);   enable_cmds (d); }), 
              "bcut");

bcopy_cmd :=
    vcWv.Cmd (fac, func(d, e) { bcopy (d, e);   enable_cmds (d); }, "bcopy");

bappend_cmd :=
    vcWv.Cmd (fac, func(d, e) { bappend (d, e); enable_cmds (d); }, "bappend");

bclear_cmd :=
    vcWv.Cmd (fac, func(d, e) { bclear (d);     enable_cmds (d); }, "bclear");


/* ---------------------------------------------------------------------- */

function yes_no_box(w, title, question)
{
    declare answer;

    table := vcTable.Create (title, 2, 1);
    vcTable.Add (table, vcRegion.Create ("msg", vcStaticText.fac, question));

    buttons := vcTable.Create ("buttons", 1, 2);

    yes_button := vcWv.LabelButtonCreate ("yes",
                                          "Yes",
                                          nil, func() answer = "yes"; ,
                                          list (close_window <- TRUE));
    vcTable.Add (buttons, yes_button);

    no_button := vcWv.LabelButtonCreate ("no",
                                         "No",
                                         nil, func() answer = "no"; ,
                                         list (close_window <- TRUE));
    vcTable.Add (buttons, no_button);

    vcTable.Add (table, buttons);
    popup := vcWin.Popup (w, table, no_button,
                          vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);

    vcWin.Forward (w, popup);

    answer;

} /* function yes_no_box */

/* ---------------------------------------------------------------------- */


/* ======================================================================
 * Add commands.  Attributes, etc.
 */

/*
 * IMPORTANT INFORMATION:
 *
 * The operations for creating text, attributes, labels and components need 
 * only one command that can be bound to a menu entry and a mouse button.  
 * In either case the user is prompted for a text string first and then thrown 
 * into a move operation.
 */

function get_new_attr_vis(display)
    if ((vis := getvalue (vqValue.NewAttrVis)) == 0)
        "Invisible";
    else if (vis == 1)
        "Visible";
    else if (vis == 2)
        "Name";
    else
        "Value";

 /* function get_new_attr_vis */


function add_attr_check(display)
{
    num_comp    := numselobj (display, vqObject.Component);
    num_net     := numselobj (display, vqObject.Net);
    num_pin     := numselobj (display, vqObject.Pin);
    num_comppin := numselobj (display, vqObject.ComponentPin);

    total       := num_comp + num_net + num_pin;

    /*
     * Note that the return value of this function is defined by the values
     * inside the if and else clauses of this statement.
     */
    if (total > 1) {

        /* All must be the same type object */

        if (   (   (num_comp > 0)
                && (num_net + num_pin == 0)
                && ((num_comp == num_comppin) || (num_comppin == 0)))
            || ((num_net > 0) && (num_pin  + num_comp == 0))
            || ((num_pin > 0) && (num_comp + num_net  == 0))) {

            total;
        }
        else
            -1;
    }
    else
        total;

    /* DON'T PUT ANY CODE HERE */
} /* function add_attr_check */

bind_attr_command := vlTrueFalse.True;

function make_add_attribute()
{
    declare lstr, lvis;

    function attributeobject(display, event, vis, expand, cycle, str)
    {
        if (bind_attr_command)
            bind_middle_button (attr_kbd_cmd);

        bind_attr_command = vlTrueFalse.True;

        if (attr (display, event, vis, expand, cycle, str) == 0) {

            /* Only go into move mode if 1 attribute is being placed and
             * it does not have a default position (i.e. not a instance
             * override.
             */

            enable_cmds (display);
            move_created_object (display, event);
        }

    } /* function attributeobject */
    attrobj_cmd := vcWv.Cmd (fac, attributeobject,
                             "_attributeobject", "Add attribute",
/* RSM - 12/14/93 changed parameter to get the new_attr_vis value */
                             vcWv.Param ("Visibility", vqAttrVisibilityVT,
                                         DYNAMIC, get_new_attr_vis, 
					 vcWv.Secondary),
                             vcWv.Param ("Expand", vlYesNoVT, CONSTANT, "yes"),
                             vcWv.Param ("If strings run out",
                                         vqApplyStringModeVT,
                                         STICKY, "Apply_Full_String"),
                             vcWv.Param ("Attribute", vqAttributeStringVT,
                                         STICKY, ""));

    function prompt_expand(display, event, expand, cycle)
    {
        vcCmd.Run (attrobj_cmd,
                   display, event, list (lvis, expand, cycle, lstr), TRUE);

        /* Should the attribute command be bound to middle button if there
         * are strings in the expand buffer?
         */

        if (expand)
            bind_middle_button (attr_kbd_cmd);
    }
    prompt_expand_cmd := vcWv.Cmd (fac, prompt_expand, nil, "Expand attribute",
                                   vcWv.Param ("Expand attribute", vlYesNoVT,
                                               CONSTANT, "yes"),
                                   vcWv.Param ("If strings run out",
                                               vqApplyStringModeVT,
                                               STICKY, "Apply_Full_String"));

    function attribute_full(display, event, str, vis)
    {
        total := add_attr_check (display);

        if (total > 1)
            vcCmd.Run (attrobj_cmd, display, event,
                       list (vis, "no", "End_Command", str), TRUE);

        else if (total == 1) {

            if (labelwidth (display, str) > 1) {

                lstr = str;
                lvis = vis;

                vcCmd.Run (prompt_expand_cmd, display, event, nil, FALSE);
            }
            else
                vcCmd.Run (attrobj_cmd, display, event,
                           list (vis, "no", "End_Command", str), TRUE);
        }
        else
            vcCmd.Run (attrobj_cmd, display, event,
                       list (vis, "no", "End_Command", str), TRUE);
    }
    vcWv.Cmd (fac, FILE_CHECK2 (attribute_full), "attribute", "Add Attribute",
              vcWv.Param ("Attribute", vqAttributeStringVT,
                          STICKY, ""),
/* RSM - 12/14/93 changed parameter to get the new_attr_vis value */
              vcWv.Param ("Visibility", vqAttrVisibilityVT,
                          DYNAMIC,get_new_attr_vis, vcWv.Secondary));

} /* function make_add_attribute */
attr_cmd := make_add_attribute ();  /* Typed "attribute" command */
vcCmd.EnableMacro (attr_cmd, "off");

function attr_from_buffer(display, event)
    if (nextattr (display, event) == 0) {

        enable_cmds (display);
        move_created_object (display, event);
    }

attr_from_buffer_cmd :=
    vcWv.Cmd (fac, FILE_CHECK0 (attr_from_buffer), "_attr-from-buffer");

function kbd_add_attr(display, event)
{
    bind_attr_command = vlTrueFalse.False;

    if (   (emptystring (display, "add-attr") == vlTrueFalse.True)
        || (canapplystring (display, "add-attr") == vlTrueFalse.False))

        vcCmd.Run (attr_cmd, display, event, nil, TRUE);
    else
        vcCmd.Run (attr_from_buffer_cmd, display, event, nil, TRUE);

} /* function kbd_add_attr */

/* Accelerator */
attr_kbd_cmd := vcWv.Cmd (fac, FILE_CHECK0 (kbd_add_attr), "next-attribute");
vcCmd.EnableMacro (attr_kbd_cmd, "off");


/* #if (ENH_OAT) */

bind_oattr_command := TRUE;

function oattributeobject(display, event, expand, cycle, str)
    if (oattr (display, event, expand, cycle, str) == 0)
        enable_cmds (display);

 /* function oattributeobject */
oattrobj_cmd := vcWv.Cmd (fac, FILE_CHECK3 (oattributeobject),
                         "oattributeobject", "Add oat",
                         vcWv.Param ("Expand", vlYesNoVT, CONSTANT, "yes"),
                         vcWv.Param ("If strings run out", vqApplyStringModeVT,
                                     STICKY, "Apply_Full_String"),
                         vcWv.Param ("Attribute", vqAttributeStringVT,
                                     STICKY, ""));

function make_add_oattribute()
{
    declare lstr;

    function prompt_expand(display, event, expand, cycle)
    {
        vcCmd.Run (oattrobj_cmd,
                   display, event, list (expand, cycle, lstr), TRUE);

        /* Should the oattribute command be bound to middle button if there
         * are strings in the expand buffer?
         */
        /* 
         * if (y_n == YES)
         *     bind_middle_button (oattr_kbd_cmd);
         */
    }
    prompt_expand_cmd := vcWv.Cmd (fac, prompt_expand, nil, "Expand attribute",
                                   vcWv.Param ("Expand attribute", vlYesNoVT,
                                               CONSTANT, "yes"),
                                   vcWv.Param ("If strings run out",
                                               vqApplyStringModeVT,
                                               STICKY, "Apply_Full_String"));

    function oattribute_full(display, event, str)
    {
        if (bind_oattr_command == TRUE)
            bind_middle_button (oattr_kbd_cmd);

        bind_oattr_command = TRUE;

        total := add_attr_check (display);

        if (total > 1)
            vcCmd.Run (oattrobj_cmd, display, event,
                       list ("no", "End_Command", str), TRUE);

        else if (total == 1) {

            if (labelwidth (display, str) > 1) {

                lstr = str;

                vcCmd.Run (prompt_expand_cmd, display, event, nil, FALSE);
            }
            else
                vcCmd.Run (oattrobj_cmd, display, event,
                           list ("no", "End_Command", str), TRUE);
        }
        else
            vcCmd.Run (oattrobj_cmd, display, event,
                       list ("no", "End_Command", str), TRUE);
    }
    vcWv.Cmd (fac, FILE_CHECK1 (oattribute_full), "oattribute", "Add Oat",
              vcWv.Param ("Attribute", vqAttributeStringVT,
                          STICKY, ""));

} /* function make_add_oattribute */
oattr_cmd := make_add_oattribute ();  /* Typed "oattribute" command */
vcCmd.EnableMacro (oattr_cmd, "off");

function oattr_from_buffer(display, event)
    if (nextoattr (display, event) == 0)
        enable_cmds (display);

oattr_from_buffer_cmd :=
    vcWv.Cmd (fac, oattr_from_buffer, "oattr-from-buffer");

function kbd_add_oattr(display, event)
{
    bind_oattr_command = FALSE;

    if (   (emptystring (display, "add-oattr") == vlTrueFalse.True)
        || (canapplystring (display, "add-oattr") == vlTrueFalse.False))

        vcCmd.Run (oattr_cmd, display, event, nil, TRUE);
    else
        vcCmd.Run (oattr_from_buffer_cmd, display, event, nil, TRUE);

} /* function kbd_add_oattr */

/* Accelerator */
oattr_kbd_cmd := vcWv.Cmd (fac, kbd_add_oattr, "next-oat");
vcCmd.EnableMacro (oattr_kbd_cmd, "off");

/* #endif */

/*-*/

/*
 * These functions determine which label command to call depending on whether
 * or not the internal label string buffer is empty. If the buffer is empty
 * then we want to execute a command that prompts the user for a label string.
 * Otherwise we want to simply add the next label from the expand buffer.
 */

bind_label_command := vlTrueFalse.True;

function make_add_label()
{
    declare lstr, lvis, lscope, lsense;

    function labelobject(display, event, str, expand, vis, scope, sense)
    {
        if (bind_label_command == vlTrueFalse.True)
            bind_middle_button (label_kbd_cmd);

        bind_label_command = vlTrueFalse.True;

        if (labelobj (display, event, str, expand, vis, scope, sense) == 0) {

            enable_cmds (display);
            move_created_object (display, event);
        }
    }
    labelobj_cmd := vcWv.Cmd (fac, labelobject, "_labelobject", "Add Label",
                              vcWv.Param ("Label String", vqLabelStringVT,
                                          STICKY, ""),
                              vcWv.Param ("Expand", vlYesNoVT,
                                          CONSTANT, "yes"),
                              vcWv.Param ("Visibility", vqLabelVisibilityVT,
                                          CONSTANT, "Visible",
                                          vcWv.Secondary),
                              vcWv.Param ("Scope", vqLabelScopeVT,
                                          STICKY, "Local",
                                          vcWv.Secondary),
                              vcWv.Param ("Sense", vqLabelSenseVT,
                                          STICKY, "NotInverted",
                                          vcWv.Secondary));

    function prompt_expand(display, event, expand)
    {
        vcCmd.Run (labelobj_cmd, display, event,
                   list (lstr, expand, lvis, lscope, lsense), TRUE);

        /* Should the label command be bound to middle button if there
         * are labels in the expand buffer?
         */

        if (expand)
            bind_middle_button (label_kbd_cmd);
    }
    prompt_expand_cmd := vcWv.Cmd (fac, prompt_expand, nil, "Expand Label",
                                   vcWv.Param ("Expand label", vlYesNoVT,
                                               CONSTANT, "yes"));

    /* We only want to prompt if labeling a pin.  For BUSes, we never expand
     * the label and for all other objects we always expand.
     */

    function label_full(display, event, str, vis, scope, sense)
    {
        /* if compound label */
        if (labelwidth (display, str) > 1) {
            if (maybeexpand (display, "add-label") == vlTrueFalse.True) {

                lstr = str;
                lvis = vis;
                lscope = scope;
                lsense = sense;

                /* prompt for expansion before running the command */
                vcCmd.Run (prompt_expand_cmd, display, event, nil, FALSE);
            }
            else {
                /* expansion assumed, but bind command to middle button
                   only if not a bus */

                if (isnumselobj (display, vqObject.BusSegment,
                     vqObjectCount.AtLeastOne) == vlTrueFalse.False)

                    bind_middle_button (label_kbd_cmd);

                vcCmd.Run (labelobj_cmd, display, event,
                           list (str, "no", vis, scope, sense), TRUE);
            }
        }
        else
            vcCmd.Run (labelobj_cmd, display, event,
                       list (str, "no", vis, scope, sense), TRUE);
    }
    vcWv.Cmd (fac, FILE_CHECK4 (label_full), "label", "Add Label",
              vcWv.Param ("Label String", vqLabelStringVT,
                          STICKY, ""),
              vcWv.Param ("Visibility", vqLabelVisibilityVT,
                          CONSTANT, "Visible",
                          vcWv.Secondary),
              vcWv.Param ("Scope", vqLabelScopeVT,
                          STICKY, "Local",
                          vcWv.Secondary),
              vcWv.Param ("Sense", vqLabelSenseVT,
                          STICKY, "NotInverted",
                          vcWv.Secondary));

} /* function make_add_label */

label_cmd := make_add_label ();          /* Typed "label" command */
vcCmd.EnableMacro (label_cmd, "off");

function label_from_buffer(display, event)
    if (nextlabel (display, event) == 0) {

        enable_cmds (display);
        move_created_object (display, event);
    }

label_from_buffer_cmd :=
    vcWv.Cmd (fac, FILE_CHECK0 (label_from_buffer), "_label-from-buffer");

function kbd_label(display, event)
{
    bind_label_command = vlTrueFalse.False;

    if (   (emptystring (display, "add-label") == vlTrueFalse.True)
        || (canapplystring (display, "add-label") == vlTrueFalse.False))

        vcCmd.Run (label_cmd, display, event, nil, TRUE);
    else
        vcCmd.Run (label_from_buffer_cmd, display, event, nil, TRUE);

} /* function kbd_label */

label_kbd_cmd := vcWv.Cmd (fac, FILE_CHECK0 (kbd_label), 
                           "next-label");  /* Accelerator */
vcCmd.EnableMacro (label_kbd_cmd, "off");


/* ---------------------------------------------------------------------- */

/* ---------------------------------------------------------------------- */

function get_string(obj_type)
    func(display) getstring (display, obj_type);;

function get_kanji_string()
    func(display) getkanjistring (display);;

function add_text(display, event, str)
{
    /* Bind text command to middle button if typed or picked from menu. */

    if (vcEvent.String (event) != nil)
        bind_middle_button (text_cmd);

    if (text (display, event, str) == 0) {

        enable_cmds (display);
        move_created_object (display, event);
    }
}
text_cmd := vcWv.Cmd (fac, FILE_CHECK1 (add_text), "text", "Add Text",
                      vcWv.Param ("Text String", vlStringRestOfLine1VT,
                                  DYNAMIC, get_string (vqObject.Text)));

if (vcWv.OS (wv) == vcOs.Unix) {
    function add_kanji_text(display, event, str)
    {
        bind_middle_button (ktext_cmd);
        
        if (ktext (display, event, str) == 0) {
                
            enable_cmds (display);
            move_created_object (display, event);
        }
    }
    ktext_cmd := vcWv.Cmd (fac, FILE_CHECK1 (add_kanji_text), 
                           "kanji", "Add Japanese",
                           vcWv.Param ("Text String",
                                       string, STICKY, "", vjEditor.fac));
}

function add_text_from_file(display, event, fname)
    if (textfromfile (display, event, fname) == 0) {

        enable_cmds (display);
        move_created_object (display, event);
    }

textfromfile_cmd := vcWv.Cmd(fac, FILE_CHECK1 (add_text_from_file), 
                             "textfromfile", "Add Text (File)",
                             vcWv.Param ("File name", string, STICKY, ""));



/* ----------------------------------------------------------------------
 * These command are used for line, pin and arc creation
 */

drag_line_cmd := vcWv.Cmd (fac, func(d, e)
                           {
                               contline (d, e, 0);
                               auto_scroll (d, e);
                           });

cancel_line_cmd := vcWv.Cmd (fac, func(d, e)
                             {
                                 unbind (d);
                                 while (canline (d, e) != 1)
                                     ;
                             },
                             "LineCancel");

/* This function creates a pin object */
function make_pin(cmdname, special)
{
    end_pin_cmd := vcWv.Cmd (fac, func (d, e)
                             {
                                 pin (d, e, special);
                                 unbind (d);
                                 enable_cmds (d);
                             },
                             cmdname + "Add");

    cancel_pin_cmd := vcWv.Cmd (fac, func(d, e)
                                {
                                    canline (d, e);
                                    unbind (d);
                                },
                                cmdname + "Cancel");

    /* create a pop-up menu for cancelling arc line create */
    popup_pin_menu := vcWv.MakeMenu ("pin",
                                      list ("Cancel", cancel_pin_cmd));

    popup_pin_menu_cmd := vcWv.Cmd (fac, func(d, e)
                                    vcMenuPop.Create (popup_pin_menu, e););

    function start_pin(display, event)
    {
        if (   (vcEvent.String (event) != nil)
            && (!string_eq (vcEvent.String (event), ""))) {

            if (special == 0)
                bind_middle_button (pin_cmd);
            else
                bind_middle_button (spin_cmd);
        }
        /* add first point of pin */
        else if (initpin (display, event, getcolor (vqObject.Pin),
                          getlstyle (vqObject.Pin)) == 0) {

            status_msg (vcEvent.Window (event), "Specify two points for pin");

            clear_binding_table ();

            vcBind.Set (bt, '\move',       drag_line_cmd);
            vcBind.Set (bt, '\middledown', end_pin_cmd);
            vcBind.Set (bt, '\leftdown',   end_pin_cmd);
            vcBind.Set (bt, '\^[',         cancel_pin_cmd);  /* Escape */

            /* "Double type" 'p' to start pin at cursor and snap to
             * bounding box.
             */
            vcBind.Set (bt, 'p',           end_pin_cmd);

            if (move_mode == 4)
                vcBind.Set (bt, '\rightdown',  cancel_pin_cmd);
            else
                vcBind.Set (bt, '\rightdown',  popup_pin_menu_cmd);

            add_drag_bindings (bt, func(d, e) contline (d, e, 0);, YES);
            
            bind (display, bt);
        }
    }
    vcWv.Cmd (fac, FILE_CHECK0 (start_pin), cmdname);
}
pin_cmd  := make_pin ("pin", 0);
spin_cmd := make_pin ("spin", 1);


/* This function makes commands for the box object creation operation */
function make_box()
{
    function end_box(display, event)
    {
        finbox (display, event); /* finish dragging */
        unbind (display);

        box (display);           /* create the actual box objects */
        enable_cmds (display);
    }
    end_box_cmd := vcWv.Cmd (fac, end_box, "BoxAdd");

    /* Need a cancel box function? */

    function start_box (display, event)
        if (   (vcEvent.String (event) != nil)
            && (!string_eq (vcEvent.String (event), "")))

            bind_middle_button (box_cmd);

        else if (initbox (display, event, getcolor (vqObject.Box),
                          getlstyle (vqObject.Box), SNAP) == 0) {

            status_msg (vcEvent.Window (event), "Specify points for box");

            clear_binding_table ();

            vcBind.Set (bt, '\move',       drag_box_nosnap_cmd);
            vcBind.Set (bt, '\middledown', end_box_cmd);
            vcBind.Set (bt, '\leftdown',   end_box_cmd);
            vcBind.Set (bt, '\^[',         cancel_drag_box_cmd);

            if (move_mode == 4)
                vcBind.Set (bt, '\rightdown',  cancel_drag_box_cmd);
            else
                vcBind.Set (bt, '\rightdown',  popup_box_menu_cmd);

            add_drag_bindings (bt,
                               func(d, e) { contbox (d, e, NOSNAP); }, YES);

            bind (display, bt);
        }
    vcWv.Cmd (fac, FILE_CHECK0 (start_box), "box");
}
box_cmd := make_box ();


/* This function makes commands for the circle operation */
function make_circle()
{
    /* This command ends the circle operation */
    function end_circle(d, e)
    {
        fincirc (d, e);
        unbind (d);
        enable_cmds (d);
    }
    end_circle_cmd := vcWv.Cmd (fac, end_circle, "CircleAdd");

    /* This command cancels the circle operation */
    function cancel_circle(d, e)
    {
        cancirc (d, e);
        unbind (d);
    }
    cancel_circle_cmd := vcWv.Cmd (fac, cancel_circle, "CircleCancel");

    /* This command drags the circle */
    drag_circle_cmd := vcWv.Cmd (fac, func(d, e)
                                 {
                                     contcirc (d, e);
                                     auto_scroll (d, e);
                                 });

    /* create a pop-up menu for cancelling circle create */
    popup_circle_menu := vcWv.MakeMenu ("circle",
                                         list ("Cancel", cancel_circle_cmd));

    popup_circle_menu_cmd :=
        vcWv.Cmd (fac, func(d, e) vcMenuPop.create (popup_circle_menu, e););

    function start_circle(display, event)
        if (   (vcEvent.String (event) != nil)
            && (!string_eq (vcEvent.String (event), "")))

            bind_middle_button (circle_cmd);

        else if (initcirc (display, event) == 0) {

            clear_binding_table ();

            vcBind.Set (bt, '\move',       drag_circle_cmd);
            vcBind.Set (bt, '\middledown', end_circle_cmd);
            vcBind.Set (bt, '\leftdown',   end_circle_cmd);
            vcBind.Set (bt, '\^[',         cancel_circle_cmd);

            if (move_mode == 4)
                vcBind.Set (bt, '\rightdown',  cancel_circle_cmd);
            else
                vcBind.Set (bt, '\rightdown',  popup_circle_menu_cmd);

            add_drag_bindings (bt, contcirc, YES);
            
            bind (display, bt);
        }
    vcWv.Cmd (fac, FILE_CHECK0 (start_circle), "circle");
}
circle_cmd := make_circle ();


/* This function makes commands for the line operation */
function make_line()
{
    end_line_cmd := vcWv.Cmd (fac, func(d, e)
                              {
                                  unbind (d);
                                  finline (d, e, 0);
                                  enable_cmds (d);
                              },
                              "LineAdd");

    undo_point_cmd := vcWv.Cmd (fac, func(d, e)
                                    if (canline (d, e) == 1) {

                                        unbind (d);
                                        enable_cmds (d);
                                    },
                                "LineUndoPoint");

    add_point_cmd := vcWv.Cmd (fac, func(d, e)
                                   contline (d, e, 1);,
                               "LineAddPoint");
    
    /* create pop-up menu for line operation and the command to pop it */
    popup_line_menu := vcWv.MakeMenu ("line",
                                       list ("End",        end_line_cmd),
                                       list ("Cancel",     cancel_line_cmd),
                                       list ("Undo Point", undo_point_cmd));

    popup_line_menu_cmd := vcWv.Cmd (fac, func(d, e)
                                     vcMenuPop.Create (popup_line_menu, e););
    
    function start_line(display, event)
        if (   (vcEvent.String (event) != nil)
            && (!string_eq (vcEvent.String (event), "")))

            bind_middle_button (line_cmd);

        else if (initline (display, event,
                           getcolor (vqObject.Line),
                           getlstyle (vqObject.Line)) == 0) {

            status_msg (vcEvent.Window (event), "Specify points for line");

            clear_binding_table ();

            vcBind.Set (bt, '\move',       drag_line_cmd);
            vcBind.Set (bt, '\middledown', add_point_cmd);
            vcBind.Set (bt, '\^[',         cancel_line_cmd);

            if (add_line_mode == 4) {

                vcBind.Set (bt, '\leftdown',   undo_point_cmd);
                /* vcBind.Set (bt, '\rightdown',  cancel_line_cmd); */
                vcBind.Set (bt, '\rightdown',  end_line_cmd);
            }
            else {

                vcBind.Set (bt, '\leftdown',   add_point_cmd);
                vcBind.Set (bt, '\rightdown',  popup_line_menu_cmd);
            }

            add_drag_bindings (bt, func(d, e) { contline (d, e, 0); }, YES);

            bind (display, bt);
        }

    vcWv.Cmd (fac, FILE_CHECK0 (start_line), "line");
}
line_cmd := make_line ();


/* This function makes commands for the arc creation operation */
function make_arc()
{
    function end_arc(d, e)
    {
        finarc (d, e);
        unbind (d);
        enable_cmds (d);
    }
    end_arc_cmd := vcWv.Cmd (fac, end_arc, "ArcAdd");

    function cancel_arc(d, e)
    {
        canarc (d, e);
        unbind (d);
    }
    cancel_arc_cmd := vcWv.Cmd (fac, cancel_arc, "ArcCancel");

    function drag_arc(d, e)
    {
        contarc (d, e);
        auto_scroll (d, e);
    }
    drag_arc_cmd := vcWv.Cmd (fac, drag_arc);

    /* This function ends the arc line creation */

    function end_arc_line(display, event)
    {
        /* create a pop-up menu for cancelling arc create */
        popup_arc_menu :=
            vcWv.MakeMenu ("arc", list ("Cancel", cancel_arc_cmd));

        popup_arc_menu_cmd :=
            vcWv.Cmd (fac, func(d, e) vcMenuPop.create (popup_arc_menu, e););

        contline (display, event, 1);   /* add second point */

        if (initarc (display, event) == 0) {       /* init arc */

            clear_binding_table ();

            vcBind.Set (bt, '\move',       drag_arc_cmd);
            vcBind.Set (bt, '\middledown', end_arc_cmd);
            vcBind.Set (bt, '\leftdown',   end_arc_cmd);
            vcBind.Set (bt, '\^[',         cancel_arc_cmd);

            if (move_mode == 4)
                vcBind.Set (bt, '\rightdown',  cancel_arc_cmd);
            else
                vcBind.Set (bt, '\rightdown',  popup_arc_menu_cmd);

            add_drag_bindings (bt, contarc, YES);
            
            bind (display, bt);
        }
    }
    end_arc_line_cmd := vcWv.Cmd (fac, end_arc_line, "ArcLineExecute");

    function start_arc(display, event)
    {
        /* create a pop-up menu for cancelling arc line create */
        popup_arc_menu :=
            vcWv.MakeMenu ("arcline", list ("Cancel", cancel_line_cmd));

        popup_arc_menu_cmd :=
            vcWv.Cmd (fac, func(d, e) vcMenuPop.create (popup_arc_menu, e););

        /* add first point of arc */

        if (   (vcEvent.String (event) != nil)
            && (!string_eq (vcEvent.String (event), "")))

            bind_middle_button (arc_cmd);

        else if (initline (display, event,
                           getcolor (vqObject.Arc),
                           getlstyle (vqObject.Arc)) == 0) {

            status_msg (vcEvent.Window (event), "Specify arc end points");

            clear_binding_table ();

            vcBind.Set (bt, '\move',       drag_line_cmd);
            vcBind.Set (bt, '\middledown', end_arc_line_cmd);
            vcBind.Set (bt, '\leftdown',   end_arc_line_cmd);
            vcBind.Set (bt, '\^[',         cancel_line_cmd);   /* Escape */

            if (move_mode == 4)
                vcBind.Set (bt, '\rightdown',  cancel_line_cmd);
            else
                vcBind.Set (bt, '\rightdown',  popup_arc_menu_cmd);

            add_drag_bindings (bt, func(d, e) { contline (d, e, 0); }, YES);

            bind (display, bt);
        }
    }
    vcWv.Cmd (fac, FILE_CHECK0 (start_arc), "arc");
}
arc_cmd := make_arc ();


/* This function makes commands for the net creation operation */
function make_net(cmdname, bus_flag)
{
    last_x := 0;  last_y := 0;  moved := TRUE;

    undo_netjoint_cmd := vcWv.Cmd (fac, undonetjoint, cmdname + "UndoJoint");

    /* This command ends the adding of pins to a net */

    end_net_cmd := vcWv.Cmd (fac, func(d, e)
                             {
                                 finnet (d, e);
                                 unbind (d);
                                 status_msg (vcEvent.Window (e), nil);
                             },
                             cmdname + "Add");

    cont_net_cmd := vcWv.Cmd (fac, func(d, e)
                              {
                                  contnet (d, e);
                                  auto_scroll (d, e);
                                  moved = TRUE;
                              });

    cancel_net_cmd := vcWv.Cmd (fac, func(d, e)
                                {
                                    cannet (d, e);
                                    unbind (d);
                                },
                                cmdname + "Cancel");

    /* create a pop-up menu for cancelling net create */
    popup_net_menu := vcWv.MakeMenu ("net",
                                      list ("End",        end_net_cmd),
                                      list ("Undo Point", undo_netjoint_cmd),
                                      list ("Cancel",     cancel_net_cmd));
    
    popup_net_menu_cmd := vcWv.Cmd (fac, func(d, e)
                                        vcMenuPop.Create (popup_net_menu, e);
                                    );

    /* This functions anchors the net (creates a joint) at the cursor pos. */
    function add_netjoint(display, event)
    {
        x := vcEvent.X (event);
        y := vcEvent.Y (event);

        /* Should be using ABS function and selection distance value from
         * viewdraw initialization file or preference sheet.
         */

        if (   (   (moved == FALSE)
                && (x - last_x <= 2) && (-2 <= (x - last_x))
                && (y - last_y <= 2) && (-2 <= (y - last_y)))
            || (addnetjoint (display, event) == 0)) {

            finnet (display, event);
            unbind (display);

            status_msg (vcEvent.Window (event), nil);
        }
        else {

            last_x = x;
            last_y = y;
            moved = FALSE;
        }
    }
    add_netjoint_cmd := vcWv.Cmd (fac, add_netjoint, cmdname + "AddJoint");

    /* This function starts the net operation */
    function start_net(display, event)
        if (   (vcEvent.String (event) != nil)
            && (!string_eq (vcEvent.String (event), ""))) {

            if (bus_flag == 0)
                bind_middle_button (net_cmd);
            else
                bind_middle_button (bus_cmd);
        }
        else if (initnet (display, event, bus_flag) == 0) {

            last_x = vcEvent.X (event) + 10;  /* + anything > distance      */
            last_y = vcEvent.Y (event) + 10;  /* used in add_netjoint above */

            clear_binding_table ();

            vcBind.Set (bt, '\move',       cont_net_cmd);
            vcBind.Set (bt, '\middledown', add_netjoint_cmd);

            if (add_net_mode == 4) {

                vcBind.Set (bt, '\leftdown',   undo_netjoint_cmd);
                vcBind.Set (bt, '\rightdown',  cancel_net_cmd);
            }
            else {

                vcBind.Set (bt, '\leftdown',   add_netjoint_cmd);
                vcBind.Set (bt, '\rightdown',  popup_net_menu_cmd);
            }

            add_drag_bindings (bt, contnet, YES);

            bind (display, bt);
        }

    vcWv.Cmd (fac, FILE_CHECK0 (start_net), cmdname);
}
net_cmd := make_net ("net", 0);
bus_cmd := make_net ("bus", 1);


/*-*/

tthres_cmd := vcWv.Cmd (fac, tthres, "threshold", "TextThreshold",
                        vcWv.Param ("Text Threshold",
                                    vlPositiveIntVT,
                                    DYNAMIC,
                                    get_value (vqValue.TextThreshold)));

function pan_view(display, event)
{
    declare vp, r, x, y;

    if (   (vcEvent.String (event) != nil)
        && (!string_eq (vcEvent.String (event), "")))

        bind_middle_button (pan_cmd);

    else if (pan_warp_cursor == YES) {

        pan (display, event, FALSE);

        vp = getviewport (display);
        r = vgU.Region (getviewport (display));

        x = (vgRect.Xmax (r) + vgRect.Xmin (r)) / 2;
        y = (vgRect.Ymax (r) + vgRect.Ymin (r)) / 2;

        vgU.PixelToUser (vp, &x, &y);
        vgU.MoveCursor (vp, x, y);

        vrefresh (display);
    }
    else
        pan (display, event, TRUE);

} /* function pan_view */
pan_cmd := vcWv.Cmd (fac, pan_view, "pan");


/** Rotate selected objects about their center point **/
/** Viewdraw 4.1 rotated about current cursor location (option 0) **/

function edit_xform_rotate(display, event)
    if (   (vcEvent.String (event) != nil)
        && (!string_eq (vcEvent.String (event), "")))

        bind_middle_button (rotate_cmd);

    else {
        rotate (display,
                set_starting_point (display, event, rotate_warp_cursor));
        enable_cmds (display);
    }

rotate_cmd := vcWv.Cmd (fac, FILE_CHECK0 (edit_xform_rotate), "rotate");


/** Scale selected objects about their center point **/
scale_cmd :=
    vcWv.Cmd (fac, FILE_CHECK1 (func(d, e, s)
                  scale (d, set_starting_point (d, e, scale_warp_cursor), s);)
              , "scale", "Scale",
              vcWv.Param ("Scale Factor",
                          vlPositiveFloatVT, STICKY, 1.0));

/* This function does reflect command */
function make_reflect()
{
    end_reflect_cmd := vcWv.Cmd (fac, func (d, e)
                                 {
                                     reflect (d, e);
                                     unbind (d);
                                     enable_cmds (d);
                                 },
                                 "ReflectFinish");

    cancel_reflect_cmd := vcWv.Cmd (fac, func(d, e)
                                    {
                                        canline (d, e);
                                        unbind (d);
                                    },
                                    "ReflectCancel");

    /* create a pop-up menu for cancelling arc line create */
    popup_reflect_menu :=
        vcWv.MakeMenu ("reflect", list ("Cancel", cancel_line_cmd));

    popup_reflect_menu_cmd :=
        vcWv.Cmd (fac, func(d, e)
                      vcMenuPop.Create (popup_reflect_menu, e); );

    function start_reflect(display, event)
    {
        /* get first point of reflect axis */

        /* initline2 does not deselect first.  Nothing to reflect if deselect
         * done first.
         */

        if (   (vcEvent.String (event) != nil)
            && (!string_eq (vcEvent.String (event), "")))

            bind_middle_button (reflect_cmd);

        else if (initline2 (display, event, "White", "Solid") == 0) {

            clear_binding_table ();

            vcBind.Set (bt, '\move',       drag_line_cmd);
            vcBind.Set (bt, '\middledown', end_reflect_cmd);
            vcBind.Set (bt, '\leftdown',   end_reflect_cmd);
            vcBind.Set (bt, '\^[',         cancel_reflect_cmd);   /* Escape */

            if (move_mode == 4)
                vcBind.Set (bt, '\rightdown',  cancel_reflect_cmd);
            else
                vcBind.Set (bt, '\rightdown',  popup_reflect_menu_cmd);

            add_drag_bindings (bt, func(d, e) { contline (d, e, 0); }, YES);

            bind (display, bt);
        }
    }
    vcWv.Cmd (fac, FILE_CHECK0 (start_reflect), "reflect");
}
reflect_cmd := make_reflect ();


/* GRID Spacing command */
gspacing_cmd := vcWv.Cmd (fac, 
                          FILE_CHECK1 (func (d, e, p1) gspacing (d, e, p1);),
                          "gspacing", "Grid Spacing",
                          vcWv.Param ("Grid Spacing", vlUnsignedIntVT,
                                      DYNAMIC, ggspacing));
pinnum_cmd := vcWv.Cmd (fac, 
                        FILE_CHECK1 (func (d, e, p1) pinnum (d, e, p1);),
                        "pinnum", "Pin Number",
                        vcWv.Param ("Pin number", string, CONSTANT, ""));

refdes_cmd := vcWv.Cmd (fac, 
                        FILE_CHECK1 (func (d, e, p1) refdes (d, e, p1);),
                        "refdes", "Refdes",
                        vcWv.Param ("Refdes", string, CONSTANT, ""));

vcWv.MakeEnum("vqNewBlockTypeVT", list("Composite", "Module",
                                       "Annotate", "Pin"));

/* This stuff with the new block type enum is done to allow for the
 * addition of the block types of equation (obsolete) and fub (not yet
 * released) to the original enum definition (in qcbacks.h).  These are 
 * needed to support all possible block types via vpi, but are not valid 
 * Viewdraw blocktypes at this point.  This stuff should be removed when
 * all the types specified in the vqBlockTypeVT enumeration are supported
 * in Viewdraw.
 */
function newgetblocktype(d) {
    curval := getblocktype(d);  
    intval := convert(vlIntVT, curval);
    if (intval > 1)
        /* There was a "hole" in the enumeration for old type "equation".  
         * This decrement "covers" it.
         */
        intval = intval - 1;

    convert(vqNewBlockTypeVT, intval);
}

btype_cmd := vcWv.Cmd (fac, 
                       FILE_CHECK1 (func(d, e, t) {
                           value := convert(vlIntVT, t);
                           /* There was a "hole" in the enumeration for old
                            * type "equation".  This increment "skips over" 
                            * it.
                            */
                           if (value > 1)
                               value = value + 1;
                           blocktype (d, e, value);
                           }),
                       "btype", "BType",
                       vcWv.Param ("Block type",
                             vqNewBlockTypeVT, DYNAMIC, newgetblocktype));
bcomposite_cmd :=
    vcWv.Cmd (fac, FILE_CHECK0 (func(d, e) blocktype (d, e, "Composite");),
              "bcomposite");

bmodule_cmd :=
    vcWv.Cmd (fac, FILE_CHECK0 (func(d, e) blocktype (d, e, "Module");),
              "bmodule");

bannotate_cmd :=
    vcWv.Cmd (fac, FILE_CHECK0 (func(d, e) blocktype (d, e, "Annotate");),
              "bannotate");

bpin_cmd :=
    vcWv.Cmd (fac, FILE_CHECK0 (func(d, e) blocktype (d, e, "Pin");),
              "bpin");

route_cmd := vcWv.Cmd (fac, func(d, e, t){route (d, e, t);},
                       "route", "Route Mode",
                       vcWv.Param ("Route mode",
                                   vqRouteModeVT, STICKY, "Avoidance"));

aroute_cmd := vcWv.Cmd (fac, func(d, e) route(d, e, "Avoidance");,  "aroute");
sroute_cmd := vcWv.Cmd (fac, func(d, e) route(d, e, "Straight");,   "sroute");
oroute_cmd := vcWv.Cmd (fac, func(d, e) route(d, e, "Orthogonal");, "oroute");


/*
 * Continue Viewdraw U/I loading
 */

/* NOTE:  Any vs file with menu definitions must be loaded before vdrawmn.vs */
load ("vdraw2.vs");
load ("vdfopen.vs");
load ("vdcomp.vs");
load ("vdlpush.vs");    /* level push source implementation */
load ("vdvpi.vs");      /* Support layer for Viewdraw/API */
load ("mfont.vs");
load ("cprobe.vs");     /* cross-probing specific commands */
load ("vdrawmn.vs");    /* Main menu and pop-up menus */
load ("vdrawbt.vs");    /* Mouse, keyboard, gesture bindings. */
load ("vdr_nav.vs");    /* vdraw addendums for navigator */
load ("vdrcload.vs");   /* vdraw addendums for on-line DRC */
