/* This file contains the following major sections:
   - parameter functions
   - command generator functions
   - file and create functions
   - ten blocks of specific dialog box functions
*/

/*** PARAMETER FUNCTIONS ***/

function p_selected () {
   vcWv.Param ("Selected element", vlStringVT, vcWv.Dynamic,
      func (d) { g_selel; },
      vcStaticText.fac, "Apply options to this element"); }

function p_additional (prefname) {
   vcWv.Param ("Additional arguments", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, prefname);,
      "Additional command line arguments (optional)"); }

function p_pldarch (prefname, archlist, title, help) {
   vcWv.Param (title, vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, prefname);, vfpgaArchPick.fac, help,
      funclist <- archlist); }


/*** COMMAND GENERATOR FUNCTIONS ***/


function cg_vdes (callback, title, name) {
   vcWv.Cmd (fac, callback, title, name,
      p_selected (),
      vcWv.Param ("Technology", vlStringVT, vcWv.Dynamic,
         func (d) cb_prefGet (g_selel, "technology");,
         vfpgaPopPick.fac, "FPGA target technology",
            funclist <- "fl_techlist"),
      vcWv.Param ("Optimization", vfpgaVdesOptVT, vcWv.Dynamic,
         func (d) cb_prefGet (g_selel, "vdes_opt");,
         "Strategy: short runtime; minimum area; maximum speed"),
      vcWv.Param ("Maximum delay (ns)", vlStringVT, vcWv.Dynamic,
         func (d) cb_prefGet (g_selel, "vdes_max");,
        "Maximum delay for area optimization"),
      vcWv.Param ("Constraint file", vlStringVT, vcWv.Dynamic,
         func (d) cb_prefGet (g_selel, "vdes_file");,
         "Name of timing constraint file"),
      vcWv.Param ("Make viewsim netlist", vlYesNoVT, vcWv.Dynamic,
         func (d) cb_prefGet (g_selel, "vdes_vsm");, vcPickEnum.fac,
         "Run viewsim netlister for gate level simulation"),
      vcWv.Param ("Generate schematic", vlYesNoVT, vcWv.Dynamic,
         func (d) cb_prefGet (g_selel, "vdes_vgen");, vcPickEnum.fac,
         "Run viewgen to create schematic of synthesized gates"),
      p_additional ("vdes_args")); }


function cg_filebox (extension, typename) {
   vcWv.Cmd (fac,
      func (d, e, filename)
         statDoload (string_lower(caddr (vscript.vsParse (filename))), 
                     typename, FALSE);,
      "Load" + typename, "Specify " + typename + " filename",
      vcWv.Param ("Filename", string, vcWv.Sticky, "", vcFileSelect.fac,
         "Enter the name of the " + typename + " file", read_now <- TRUE,
         filter <- "*" + extension, dir_columns <- 0, edit_filter <- FALSE)); }


function cg_part (callback, title, name) {
   vcWv.Cmd (fac, callback, title, name,
      p_selected (),
      vcWv.Param ("Output name", vlStringVT, vcWv.Dynamic,
                  func (d) { str := cb_prefGet (g_selel, "part_out"); },
                  vfpgaNotEmptyString.fac,
                  "Base filename for new partitions"), 
      p_pldarch ("part_auto", "fl_partlist", "Partitioning device",
         "PLD device (architecture) for new partitions"),
      vcWv.Param ("Strategy", vlStringVT, vcWv.Dynamic,
         func (d) cb_prefGet (g_selel, "part_strat");,
         vfpgaVertPick.fac, "Source of guide information, output style",
            funclist <- "fl_partstrat"),
      p_additional ("part_args")); }

function cg_devsel (callback, title, name, el_name_func) {
   vcWv.Cmd (fac, callback, title, name,
             vcWv.Param ("Selected element", vlStringVT, 
                         vcWv.Dynamic, func(d) el_name_func();,
                         vcStaticText.fac, "Apply options to this element"),
             vcWv.Param ("Manufacturer", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_man");,
                         vfpgaPopPick.fac, "PLD manufacturer", 
                         exclusive <- FALSE, funclist <- "fl_manlist"),
             vcWv.Param ("Device", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "device");,
                         vfpgaArchPick.fac, "PLD device (architecture) type", 
                         funclist <- "fl_devlist"),
             vcWv.Param ("Temperature", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_temp");,
                         vfpgaPopPick.fac, "Temperature grade", 
                         exclusive <- FALSE, funclist <- "fl_seltemp"),
             vcWv.Param ("Technology", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_tech");,
                         vfpgaPopPick.fac, "Fabrication technology", 
                         exclusive <- FALSE, funclist <- "fl_seltech"),
             vcWv.Param ("Package", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_pkg");,
                         vfpgaPopPick.fac, "Package style", 
                         exclusive <- FALSE, funclist <- "fl_selpack"),
             vcWv.Param ("Preload", vfpgaSelJamVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_jam");, 
                         "Register preload capability"),
             vcWv.Param ("Erasable", vfpgaSelEraVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_era");, 
                         "Method to erase device"), 
             vcWv.Param ("Pins", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_pins");, 
                         vfpgaNumComp.fac, "Device package pins"),
             vcWv.Param ("Speed (ns)", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_spd");, 
                         vfpgaNumComp.fac, "Device speed"),
             vcWv.Param ("Setup time (ns)", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_set");, 
                         vfpgaNumComp.fac, "Register setup time"),
             vcWv.Param ("Hold time (ns)", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_hold");, 
                         vfpgaNumComp.fac, "Register hold time"),
             vcWv.Param ("Price ($ per 100)", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_cost");, 
                         vfpgaNumComp.fac, "User supplied cost"),
             vcWv.Param ("Power (mW)", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_pow");, 
                         vfpgaNumComp.fac, "Device power usage"),
             vcWv.Param ("User 1", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_u1");, 
                         vfpgaNumComp.fac, "User defined value 1"),
/*             vcWv.Param ("User 2", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_u2");, 
                         vfpgaNumComp.fac, "User defined value 2")); */
             vcWv.Param ("Sort Order", vlStringVT, vcWv.Dynamic,
                         func (d) cb_prefGet (el_name_func(), "sel_sort");,
                         vfpgaPopPick.fac, "Search result sort order", 
                         exclusive <- FALSE, funclist <- "fl_selsort"));
}




/* String manipulation operations to create a list like:
    list (list ("Element", "Device"),
          list ("binbcd",  "P22V10"),
          list ("foo",     "P16R8")); */
function fl_jed_elements() {
   l_part := vscript.Sort (cb_loadPartlist (g_selel), string_lt);
   for (l_spread:=nil; !null(l_part); l_part=cdr(l_part)) {
      name := car (l_part);
      l_spread = cons (list (name, cb_prefGet (name, "device")), l_spread); }
   cons (list ("Element", "Device"), l_spread); }


/* String manipulation operations to create a list like:
    list (list ("Element", "Device", "Exact Partname"),
          list ("binbcd",  "P22V10", "AMD PAL22V10-10PC PLD"),
          list ("foo",     "P16L8",  "AMD P16L8-5PC PAL")); */
function fl_tim_elements() {
   l_part := vscript.Sort (cb_loadPartlist (g_selel), string_lt);
   for (l_spread:=nil; !null(l_part); l_part=cdr(l_part)) {
      name := car (l_part);
      l_spread = cons (list (name,
         cb_prefGet (name, "device"),
         cb_prefGet (name, "part")),
         l_spread); }
   cons (list ("Element","Device","Exact Partname"), l_spread); }


/*** FILE, CREATE FUNCTIONS ***/


/* Because of core bug #7891, the "sticky" parameter to the dialog box
   must be maintained with the global variable g_lastsch */
g_lastsch := "";
g_schfilter := vscript.vsDnpath ("", "sch") + "*.1";
c_file_sch := vcWv.Cmd (fac,
      func (d, e, filename) {
         rootname := caddr (vscript.vsParse (filename));
         statDoload (string_lower(rootname), "Schematic", FALSE);
         g_lastsch = rootname + ".1"; },
      "LoadSchematic", "Load Schematic",
      vcWv.Param ("Schematic name", string, vcWv.Dynamic,
         func (d) g_lastsch;, vcFileSelect.fac,
         "Enter the name of the schematic", filter <- g_schfilter,
         read_now <- TRUE, dir_columns <- 0, edit_filter <- FALSE));

c_file_vhdl := cg_filebox (".vhd", "VHDL");
c_file_abel := cg_filebox (".abl", "ABEL");
c_file_jed  := cg_filebox (".jed", "JEDEC");

c_file_edit := vcWv.Cmd (fac,
   func (d, e) {
      type := cb_loadGettype (g_selel, g_ispartsel);

      if (string_eqi ("sch", substring (type, 0, 3)))
         cb_itcEditSch (g_selel);
      else {
         fullname := "";
         if (string_eqi (type, "VHDL"))
            fullname = g_selel + ".vhd";
         else if (string_eqi (type, "ABEL"))
            fullname = g_selel + ".abl";
         else if (string_eqi (type, "JEDEC"))
            fullname = g_selel + ".jed";

         if (string_eq (fullname, ""))
            cb_mainErr (1013, "Error", nil,
               "Unable to edit file type " + type);
         else
            vcWin.Create (vcRegion.Create (fullname, vtx.fac, fullname,
               load_file <- TRUE, height <- 20)); } },
   "EditSelected", "Edit Selected File");

c_opt_ui := vcWv.Cmd (fac, 
   func (d, e, ignore, xprobe, alerts) {
       vlPref.Setval (prefs, "ignore_timestamps", ignore);
       vlPref.Setval (prefs, "crossprobe", xprobe);
       vlPref.Setval (prefs, "warnalerts", alerts); },
   "UIOptions", "User Interface Options",                         
   vcWv.Param ("Ignore Timestamps", vlYesNoVT, vcWv.Dynamic,
      func (d) vlPref.Val (prefs, "ignore_timestamps");, vcPickEnum.fac,
      "Force tool invocation by ignoring timestamps"),
   vcWv.Param ("Vdraw CrossProbe", vfpgaXprobeVT, vcWv.Dynamic,
      func (d) vlPref.Val (prefs, "crossprobe");,
      "Selection of elements broadcast between vfpga and vdraw"),
   vcWv.Param ("Popup Warnings", vlYesNoVT, vcWv.Dynamic,
      func (d) vlPref.Val (prefs, "warnalerts");, vcPickEnum.fac,
      "Popup alerts before overwriting files"));

c_cre_func := vcWv.Cmd (fac,
   func (d, e) {
      if ((vlPref.Val (prefs, "warnalerts")) && (cb_loadIstop (g_selel)) &&
         (string_eqi (cb_loadGettype (g_selel, g_ispartsel), "ABEL")) &&
         (cb_cfgFileExists ("cmd", g_selel)) &&
         (cb_prefGet (g_selel, "a2p_tmv")))
            vfpgaConfirm ("OverwriteCmd",
               "ViewSim command file " + g_selel + ".cmd already exists.\n" +
               "OK to write over this file?",
               func () vcWin.WithBusy(vcRegion.Window(gr_log),
                         func() cb_makeDochain("funcsim",g_selel,nil); ); );
      else vcWin.WithBusy(vcRegion.Window(gr_log),
              func () cb_makeDochain ("funcsim", g_selel, nil);); },
   "CreateFuncModel", "Create functional model");


c_cre_fpga := cg_vdes (
   func (d, e, selel, tech, opt, max, file, vsm, vgen, args) {
      cb_prefPut (g_selel, "technology", tech, FALSE);
      cb_prefPut (g_selel, "vdes_opt", opt, FALSE);
      cb_prefPut (g_selel, "vdes_max", max, FALSE);
      cb_prefPut (g_selel, "vdes_file", file, FALSE);
      cb_prefPut (g_selel, "vdes_vsm", vsm, FALSE);
      cb_prefPut (g_selel, "vdes_vgen", vgen, FALSE);
      cb_prefPut (g_selel, "vdes_args", args, TRUE);

      if ((vlPref.Val (prefs, "warnalerts")) &&
         (string_eq (cb_prefGet (g_selel, "technology"), "")))
            vfpgaConfirm ("NoTechnology",
               "OK to synthesize without specifying technology?",
               func () cb_makeDochain ("fpganet", g_selel, nil););
      else cb_makeDochain ("fpganet", g_selel, nil); },
   "CreateFPGA", "Create FPGA WIR file");


function f_cre_part (d, e, selel, out, auto, strat, args) {
      cb_prefPut (g_selel, "part_out", out, FALSE);
      cb_prefPut (g_selel, "part_auto", auto, FALSE);
      cb_prefPut (g_selel, "part_strat", strat, FALSE);
      cb_prefPut (g_selel, "part_args", args, TRUE);

      /* FixFor52: a cleaner way to do the check for a string is to use
         the typespec vlStringNotEmptyVT in the dialog box */
      /* Produce error if no output name given */
      if (string_eq (out, ""))
         cb_mainErr (1425, "Error", nil, "No name given for output schematic");
      else cb_makeDochain ("partition", g_selel, nil); }


c_cre_part := cg_part (f_cre_part, "CreatePLDPart",
   "Create PLD partitioning schematic");


/*** SYMBOL CREATION FUNCTIONS ***/


function f_cre_sym () {
   if ((vlPref.Val (prefs, "warnalerts")) &&
       (cb_cfgFileExists ("sym", g_selel)))
         vfpgaConfirm ("OverwriteSym",
            "Symbol " + g_selel + " already exists.\n" +
            "OK to rearrange pins on symbol?",
            func () cb_makeDochain ("symbol", g_selel, nil););
   else cb_makeDochain ("symbol", g_selel, nil); }

function f_cre_sym_dio (d, e, selel, type, libsym, pinno, space, 
                        len, biside) {
    cb_prefPut (g_selel, "sym_type", type, FALSE);
    cb_prefPut (g_selel, "sym_libsym", libsym, FALSE);
    cb_prefPut (g_selel, "sym_pinno", pinno, FALSE);
    cb_prefPut (g_selel, "sym_pinspace", space, FALSE);
    cb_prefPut (g_selel, "sym_pinlen", len, FALSE);
    cb_prefPut (g_selel, "sym_biside", biside, FALSE);
    f_cre_sym (); }

function f_cre_sym_vs (d, e, selel, pinspace, pinlen) {
   cb_prefPut (g_selel, "sym_pinspace", pinspace, FALSE);
   cb_prefPut (g_selel, "sym_pinlen", pinlen, FALSE);
   f_cre_sym (); }

c_cre_sym_dio := vcWv.Cmd (fac, f_cre_sym_dio,
   "CreateDIOSymbol", "Create Symbol",
   p_selected (),
   vcWv.Param ("Symbol type", vfpgaSymTypeVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "sym_type");,
      "Generate automatic, modify user, or copy from library"),
   vcWv.Param ("Library symbol name", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "sym_libsym");,
      "Enter name of symbol if library symbol type"),
   vcWv.Param ("Pin numbers", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "sym_pinno");, vcPickEnum.fac,
      "Put device pin numbers on symbol pins"),
   vcWv.Param ("Pin spacing", vlPositiveIntVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "sym_pinspace");,
      "Spacing between pins of automatic symbol in grid units"),
   vcWv.Param ("Pin length", vlPositiveIntVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "sym_pinlen");,
      "Length of pins on automatic symbol in grid units"),
   vcWv.Param ("Bidirectional pin side", vfpgaSideVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "sym_biside");,
      "Put bidirectional pins on this side of automatic symbol"));

c_cre_sym_vs := vcWv.Cmd (fac, f_cre_sym_vs,
   "CreateVSSymbol", "Create Symbol",
   p_selected (),
   vcWv.Param ("Pin spacing", vlPositiveIntVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "sym_pinspace");,
      "Spacing between automatic pins in grid units"),
   vcWv.Param ("Pin length", vlPositiveIntVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "sym_pinlen");,
      "Length of automatic pins in grid units"));

/* The symbol creation command does nothing but decide which of two commands
   to call: c_cre_sym_vs or c_cre_sym_dio.  Therefore, it must not be macro
   recorded, or the macro will create the symbol twice. */
c_cre_sym := vcWv.Cmd (fac,
   func (d, e) {
      type := cb_loadGettype (g_selel, g_ispartsel);
      if ((string_eqi (type, "Partition")) || (string_eqi (type, "ABEL")) ||
         (string_eqi (type, "JEDEC")))
            vcCmd.Run (c_cre_sym_dio, d, e, nil, FALSE);
      else if (string_eqi ("sch", substring (type, 0, 3)) ||
         (string_eqi (type, "VHDL")))
            vcCmd.Run (c_cre_sym_vs, d, e, nil, FALSE); },
   "CreateSymbol", "Create symbol");
vcCmd.EnableMacro (c_cre_sym, FALSE);


/*** ABEL COMPILER FUNCTIONS ***/


function f_ahdl2pla (d, e, selel, retain, tmv, defs, args) {
   cb_prefPut (g_selel, "a2p_retain", retain, FALSE);
   cb_prefPut (g_selel, "a2p_tmv",    tmv, FALSE);
   cb_prefPut (g_selel, "a2p_defs",   defs, FALSE);
   cb_prefPut (g_selel, "a2p_args",   args, TRUE); }

c_opt_ahdl2pla := vcWv.Cmd (fac, f_ahdl2pla, "CompileABELOptions",
   "Compile ABEL options",
   p_selected (),
   vcWv.Param ("Retain redundant", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "a2p_retain");, vcPickEnum.fac,
      "Retain redundant logic equations (hazard protection)"),
   vcWv.Param ("Translate vectors", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "a2p_tmv");, vcPickEnum.fac,
      "Translate test vectors for top level file"),
   vcWv.Param ("Definitions", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "a2p_defs");,
      "Values for substitution into source"),
   p_additional ("a2p_args"));


/*** PLA OPTIMIZER FUNCTIONS ***/


function f_plaopt (d, e, selel, strategy, exact) {
   cb_prefPut (g_selel, "po_strat", strategy, FALSE);
   cb_prefPut (g_selel, "po_exact", exact, TRUE); }

c_opt_plaopt := vcWv.Cmd (fac, f_plaopt, "OptimizePLAOptions",
   "Optimize PLA options",
   p_selected (),
   vcWv.Param ("Strategy", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "po_strat");,
      vfpgaVertPick.fac, "Strategy for reduction, polarity",
         funclist <- "fl_plastrat"),
   vcWv.Param ("Exact optimization", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "po_exact");, vcPickEnum.fac,
      "Use Quine-McCluskey algorithm"));


/*** VHDL ANALYZER FUNCTIONS ***/


function f_vhdl (d, e, selel, err) {
   cb_prefPut (g_selel, "vhdl_err", err, FALSE);
}

c_opt_vhdl := vcWv.Cmd (fac, f_vhdl, "AnalyzeVHDLOptions",
   "Analyze VHDL options",
   p_selected (),
   vcWv.Param ("Error level", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "vhdl_err");,
      vfpgaVertPick.fac, "Maximum error severity reported by analyzer",
         funclist <- "fl_vhdlerr"));


/*** FITTER FUNCTIONS ***/


function f_fit (d, e, selel, arch, fitrun, fitpin, args) {
   cb_prefPut (g_selel, "device", arch, FALSE);
   cb_prefPut (g_selel, "fit_run", fitrun, FALSE);
   cb_prefPut (g_selel, "fit_pin", fitpin, FALSE);
   cb_prefPut (g_selel, "fit_args", args, TRUE); }

c_opt_fit := vcWv.Cmd (fac, f_fit, "AssignPinsOptions",
   "Assign pins options",
   p_selected (),
   p_pldarch ("device", "fl_fitlist", "Device",
      "PLD device (architecture) type"),
   vcWv.Param ("Run fitter", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "fit_run");, vcPickEnum.fac,
      "Should fitter be run"),
   vcWv.Param ("Previous pin assignment", vfpgaFitPinVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "fit_pin");,
      "Strategy for pre-existing pin assignments"),
   p_additional ("fit_args"));


/*** TIMING MODELLER FUNCTIONS ***/


function f_j2vhdl (d, e, selel, out, arch, delay, utf) {
   cb_prefPut (selel, "device", arch, FALSE);
   cb_prefPut (selel, "j2v_delay", delay, FALSE);
   cb_prefPut (selel, "j2v_utf", utf, TRUE); 
   if ((string_eqi (cb_loadGettype (selel, FALSE), "vhdl")) &&
       ((string_eqi (out, selel)) ||
        (string_eq (out, "")))) 
       vfpgaConfirmCmdline ("JedecFilename",
                   "To avoid filename collision, the jedec and timing\n" +
                   "model must have a different name than the source\n" +
                   "VHDL file\n\n" +
                   "Please enter the name to call the timing model:",
                              "Jedec Filename",
                      func (str) cb_prefPut (selel, "jed_out", str, TRUE); );
   else
       cb_prefPut (selel, "jed_out", out, FALSE);
}

c_opt_j2vhdl := vcWv.Cmd (fac, f_j2vhdl, "PLDTimingModelOptions",
   "PLD timing model options",
   p_selected (),
   vcWv.Param ("VHDL file name", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_out");,
               vfpgaNotEmptyString.fac,
               "Name of jedec and full timing vhdl file"),
   p_pldarch ("device", "fl_j2vlist", "Device",
      "PLD device (architecture) type"),
   vcWv.Param ("Delay model", vfpgaJ2VDelayVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "j2v_delay");,
      "Use minimum, typical or maximum delay values"),
   vcWv.Param ("User timing file", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "j2v_utf");, vcPickEnum.fac,
      "Read timing values from user-created timing file"));


/*** PARTITIONER FUNCTIONS ***/


function f_part (d, e, selel, out, auto, strat, args) {
   cb_prefPut (g_selel, "part_out", out, FALSE);
   cb_prefPut (g_selel, "part_auto", auto, FALSE);
   cb_prefPut (g_selel, "part_strat", strat, FALSE);
   cb_prefPut (g_selel, "part_args", args, TRUE); }

c_opt_part := cg_part (f_part, "PartitionOptions", "PLD partitioning options");


/*** SYNTHESIZER FUNCTIONS ***/


function f_vdes (d, e, selel, tech, opt, max, file, vsm, vgen, args) {
   cb_prefPut (g_selel, "technology", tech, FALSE);
   cb_prefPut (g_selel, "vdes_opt", opt, FALSE);
   cb_prefPut (g_selel, "vdes_max", max, FALSE);
   cb_prefPut (g_selel, "vdes_file", file, FALSE);
   cb_prefPut (g_selel, "vdes_vsm", vsm, FALSE);
   cb_prefPut (g_selel, "vdes_vgen", vgen, FALSE);
   cb_prefPut (g_selel, "vdes_args", args, TRUE); }

c_opt_vdes := cg_vdes (f_vdes, "SynthesizeOptions", "Synthesize options");


/*** CREATE JEDEC FUNCTIONS ***/


function f_fuseasm (d, e, selel, arch, chksum, docfmt, plcc, blow,
   turbo, miser, lock, xval, fmt, ues) {

   cb_prefPut (g_selel, "device", arch, FALSE);
   cb_prefPut (g_selel, "jed_chksum", chksum, FALSE);
   cb_prefPut (g_selel, "jed_docfmt", docfmt, FALSE);
   cb_prefPut (g_selel, "jed_plcc", plcc, FALSE);
   cb_prefPut (g_selel, "jed_blow", blow, FALSE);
   cb_prefPut (g_selel, "jed_turbo", turbo, FALSE);
   cb_prefPut (g_selel, "jed_miser", miser, FALSE);
   cb_prefPut (g_selel, "jed_lock", lock, FALSE);
   cb_prefPut (g_selel, "jed_xval", xval, FALSE);
   cb_prefPut (g_selel, "jed_fmt", fmt, FALSE);
   cb_prefPut (g_selel, "jed_ues", ues, TRUE); }

c_opt_fuseasm := vcWv.Cmd (fac, f_fuseasm, "CreateJEDECOptions",
   "Create JEDEC options",
   p_selected (),
   p_pldarch ("device", "fl_devlist", "Device",
      "PLD device (architecture) type"),
   vcWv.Param ("Checksum", vfpgaJedChksumVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_chksum");,
      "Type of checksum to insert in programmer file"),
   vcWv.Param ("Report format", vfpgaJedDocfmtVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_docfmt");,
      "Short or detailed output report format"),
   vcWv.Param ("PLCC chip diagram", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_plcc");, vcPickEnum.fac,
      "Create pinout diagram using PLCC package"),
   vcWv.Param ("Blow product terms", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_blow");, vcPickEnum.fac,
      "Blow fuses for unused product terms"),
   vcWv.Param ("Turbo", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_turbo");, vcPickEnum.fac,
      "Enable Turbo mode for certain architectures"),
   vcWv.Param ("Miser", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_miser");, vcPickEnum.fac,
      "Enable Miser mode for certain architectures"),
   vcWv.Param ("Lock", vlYesNoVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_lock");, vcPickEnum.fac,
      "Enable security mode for certain architectures"),
   vcWv.Param ("X value", vfpgaJedXvalVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_xval");,
      "Value to program for don't care terms"),
   vcWv.Param ("JEDEC format", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_fmt");,
      vfpgaPopPick.fac, "Format for device programming file",
         funclist <- "fl_jedfmtlist"),
   vcWv.Param ("User electronic signature", vlStringVT, vcWv.Dynamic,
      func (d) cb_prefGet (g_selel, "jed_ues");,
      "User signature word"));


/*** SELECT PLD DEVICE FUNCTIONS ***/
function cg_datasheet(el_name, select_action, accept_action) {
    vcWv.Cmd (fac, accept_action, "DeviceDatasheet", 
              "Device Datasheet", 
              vcWv.Param ("Device", vscmVT, vcWv.Constant,
                          select_action, vfpgaDatasheet.fac,
                          "Datasheet for exact part", name <- el_name)); 
}

function f_devsel(d, e, selel, man, dev, temp, tech, pkg, jam, era,
/*                  pins, spd, set, hold, cost, pow, u1, u2) { */
                  pins, spd, set, hold, cost, pow, u1, sort) {
    cb_prefPut (selel, "sel_man", man, FALSE);
    cb_prefPut (selel, "device", dev, FALSE);
    cb_prefPut (selel, "sel_temp", temp, FALSE);
    cb_prefPut (selel, "sel_tech", tech, FALSE);
    cb_prefPut (selel, "sel_pkg", pkg, FALSE);
    cb_prefPut (selel, "sel_jam", jam, FALSE);
    cb_prefPut (selel, "sel_era", era, FALSE); 
    cb_prefPut (selel, "sel_pins", pins, FALSE);
    cb_prefPut (selel, "sel_spd", spd, FALSE);
    cb_prefPut (selel, "sel_set", set, FALSE);
    cb_prefPut (selel, "sel_hold", hold, FALSE);
    cb_prefPut (selel, "sel_cost", cost, FALSE);
    cb_prefPut (selel, "sel_pow", pow, FALSE);
    cb_prefPut (selel, "sel_u1", u1, FALSE);
/*    cb_prefPut (selel, "sel_u2", u2, TRUE); } */
    cb_prefPut (selel, "sel_sort", sort, TRUE); }

c_opt_devsel := cg_devsel(f_devsel, "SelectPLDDeviceOptions",
                          "Set PLD Device Selector options",
                          func(){g_selel;});

/* This function gets called when the devsel dialog box is "accept"-ed.
   It first sets the preferences using f_devsel, then checks to see if
   all device selector prefs (except device) were nil.  If they're all
   nil, we may not want to run the device selector at all, so throw an
   alert.  If some device selector prefs are not nil, or the user said
   run it anyway, we run the device selector and build the datasheet 
   dialog box.  We've defined some highlight and accept action routines
   here for the datasheet dialog box. */
function run_devsel(d, e, selel, man, dev, temp, tech, pkg, jam,
/*                    era, pins, spd, set, hold, cost, pow, u1, u2) { */
                    era, pins, spd, set, hold, cost, pow, u1, sort) {

    function get_part_string(p) {
        declare dev, mfg, part, type;
        p = string_token (p, &dev, " ");
        p = string_token (p, &mfg, " ");
        p = string_token (p, &part, " ");
        p = string_token (p, &type, " ");
        partstr :=  mfg + " " + part + " " + type;
    }

    function do_devsel() {
        declare dev;
        cb_makeDochain ("devsel", selel, func() {
            vcCmd.Run(cg_datasheet(selel, 
                          func(d) { partstr := get_part_string(d);
                                    string_token(d, &dev, " ");
                                    vfpgaSpreadsheet.SetCmdline(selel,
                                                list(dev, partstr)); },
                          func(d,e,p) { partstr :=  get_part_string(p);
                                      cb_prefPut(selel, "part", 
                                                 partstr, FALSE); } ),
                      d, e, nil, FALSE); });
    }

    f_devsel(d, e, selel, man, dev, temp, tech, pkg, jam,
/*             era, pins, spd, set, hold, cost, pow, u1, u2); */
             era, pins, spd, set, hold, cost, pow, u1, sort);
    vfpgaSpreadsheet.SetCmdline(selel, list(dev, cb_prefGet(selel, "part")));
    if ((string_eq(man,"")) && (string_eq(temp,"")) && 
        (string_eq(tech,"")) && (string_eq(pkg,"")) && 
/*	(string_eq(jam,"")) && (string_eq(era,"")) && */
        (string_eq(pins,"")) && (string_eq(spd,"")) && 
        (string_eq(set,"")) && (string_eq(hold,"")) && 
        (string_eq(cost,"")) && (string_eq(pow,"")) &&
/*        (string_eq(u1,"")) && (string_eq(u2 ,""))) { */
        (string_eq(u1,"")) && (string_eq(sort ,""))) {
        if (vlPref.Val (prefs, "warnalerts"))
            vfpgaConfirm ("NoDevselValues",
                          "No device selector values specified.\n" +
                          "Still want to run device selector?",
                          do_devsel);
    }
    else do_devsel();
}

function cg_jedtim(el_name) {
    cg_devsel(run_devsel, "RunPLDDeviceSelector", 
              "Run PLD Device Selector for " + el_name, func(){el_name;});
}

function cg_jedec (callback, title, name) {

    vcWv.Cmd (fac, callback, title, name,
          vcWv.Param ("Set Device", vscmVT, vcWv.Constant,
                      cg_jedtim, vfpgaSpreadsheet.fac,
                      "PLD device architecture",
                      el_list <- "fl_jed_elements")); }


function cg_timing (callback, title, name) {

    vcWv.Cmd (fac, callback, title, name,
          vcWv.Param ("Set Device", vscmVT, vcWv.Constant,
                      cg_jedtim, vfpgaSpreadsheet.fac,
                      "PLD exact part",
                      el_list <- "fl_tim_elements")); }


c_cre_jedec := cg_jedec (
   func (d, e, el_list) {
       declare element, el;
       function set_pref_and_run () {
           el = cdr(el);
           if (!null(el)) {
               element = car(el);
               cb_prefPut(car(element), "device", cadr(element), TRUE);
               cb_makeDochain ("jedec", car(element), set_pref_and_run);
           }
       }
       el = el_list;
       element = car(el);
       cb_prefPut(car(element), "device", cadr(element), TRUE);
       cb_makeDochain ("jedec", car(element), set_pref_and_run);
       
   }, "DeviceSpreadsheet", "Create JEDEC");


c_cre_timing := cg_timing (
   func (d, e, el_list) {
       declare element, el; no_exact := TRUE;

       function set_pref_and_run () {
           el = cdr(el);
           if (!null(el)) {
               element = car(el);
               cb_prefPut(car(element), "device", cadr(element), FALSE);
               cb_prefPut(car(element), "part", caddr(element), TRUE);
               cb_makeDochain ("timing", car(element), set_pref_and_run);
           }
           else {
               x := cb_prefGet (g_selel, "part_out");
               if (!string_eq (x, "")) {
                   cb_prefPut (g_selel, "p2s_sym", vlYesNo.No, TRUE);
                   cb_makeDochain ("timsch", g_selel, func () {
                       cb_prefPut (g_selel, "p2s_sym", vlYesNo.Yes, TRUE);});
               }
           }
       }

       function chainelements () {
           el = el_list;
           element = car(el);
           elname := car(element);
           cb_prefPut(elname, "device", cadr(element), FALSE);
           cb_prefPut(elname, "part", caddr(element), TRUE);
           jedname := cb_prefGet (elname, "jed_out");
           if ((string_eqi (cb_loadGettype (elname, FALSE), "vhdl")) &&
               ((string_eqi (jedname, elname)) ||
                (string_eqi (jedname, "")))) 
               vfpgaConfirmCmdline ("JedecFilename",
                   "To avoid filename collision, the jedec and timing\n" +
                   "model must have a different name than the source\n" +
                   "VHDL file\n\n" +
                   "Please enter the name to call the timing model:",
                              "Jedec Filename",
                              func (out) {
                                  cb_prefPut (elname, "jed_out", out, TRUE); 
                                  cb_makeDochain ("timing", elname, 
                                                  set_pref_and_run); 
                              });
           else
               cb_makeDochain ("timing", elname, set_pref_and_run);
       }
       
       for (el:=el_list; !null(el); el=cdr(el)) {
           element = car(el);
           if (!string_eq (caddr (element), "")) no_exact = FALSE;
       }
       if (no_exact && (vlPref.Val (prefs, "warnalerts")))
           vfpgaConfirm ("NoPartnames",
               "No exact partnames were specified.\n" +
               "OK to create unit delay models?",
               chainelements);
       else chainelements (); 
   }, "PartSpreadsheet", "Create PLD timing model");
