/* This file contains the functions to build tool command lines and errors */

/*** PLA2VIEW tool command lines ***/


function a_p2v (name) {
   str := "pla2view " + name;
   if ((cb_loadIstop (name)) && (cb_prefGet (name, "a2p_tmv")))
      str = str + " -tmv";
   str; }

function a_p2v_dio (name) { a_p2v (name) + " -wir"; }
function a_p2v_diosch (name) { a_p2v (name) + " -sch"; }
function a_p2v_vli (name) { a_p2v (name) + " -sim"; }
function add_sym_prefs (name, str) { 
    x := cb_prefGet(name, "sym_type");
    if (x == vfpgaSymType.Library) {
        y := cb_prefGet(name, "sym_libsym");
        str = str + " -lsym " + y; }
    else if (x == vfpgaSymType.User) str = str + " -usym";
    if (cb_prefGet(name, "sym_pinno")) str = str + " -pcb";
    x := cb_prefGet(name, "sym_pinspace");
    if (x != 10) str = str + " -pinspacing " + number2string(x);
    x = cb_prefGet(name, "sym_pinlen");
    if (x != 20) str = str + " -pinlength " + number2string(x);
    x = cb_prefGet(name, "sym_biside");
    if (x != vfpgaSide.Right) str = str + " -bidirside left";
    x = cb_loadGettype(name, FALSE);
    if (string_eqi (x, "partition"))
        str = str + " -symtype pla";
    else if (string_eqi (x, "abel"))
        str = str + " -symtype abel";
    else if (string_eqi (x, "vhdl"))
        str = str + " -symtype vhdl";
    str;
}

function a_p2v_sym (name) { 
    str := "pla2view " + name + " -sym -novhdl";
    str = add_sym_prefs(name, str);
}

function a_p2v_pcb (name) { 
    str := "pla2view " + name + " -pcb -novhdl";
    str = add_sym_prefs (name, str);
}

function a_p2v_pin (name) { 
    str := "pla2view " + name + " -pin -novhdl";
    str = add_sym_prefs (name, str);
}

function a_timp2v (name) { 
    jedname := cb_prefGet (name, "jed_out");
    str := "pla2view " + jedname + " -pcb -novhdl -pla " + name;
    str = add_sym_prefs (name, str);
}


/*** DIO tool command lines  ***/


function a_ahdl2pla (name) {
   str := "ahdl2blf " + name;
   if (cb_prefGet (name, "a2p_retain"))
      str = str + " -retain";
   s := cb_prefGet (name, "a2p_defs");
   if (!string_eq (s, ""))
      str = str + " -args " + s;
   s = cb_prefGet (name, "a2p_args");
   if (!string_eq (s, ""))
      str = str + " " + s;
   str; }


function a_ahdl2prt (name) { a_ahdl2pla (name) + " -oprt " + name + ".prt"; }

function a_p2a (name) { "pla2eqn " + name + ".tt2 -lang abel"; }

/* Caution: this function changes one underscore into a space!
   There are matching cautions in def.vs:fl_plastrat and fac.vs:VertPick */
function a_plaopt (name) {
   str := "blifopt " + name + " -pla";

   exact := cb_prefGet (name, "po_exact");
   strat := cb_prefGet (name, "po_strat");
   index := lindex (strat, '_');
   if (index != FALSE) string_set (strat, index, ' ');

   if (exact) {
      str = str + " -reduce exact";
      if (!string_eq (strat, "default"))
         str = str + " " + strat; }
   else if (!string_eq (strat, "default"))
      str = str + " -reduce " + strat;

   str; }

function a_mergett (name) {
    str := "blifopt " + name + " -reduce none -pla";
}


function a_fit (name) {

   if (cb_prefGet (name, "fit_run")) {
       str := "fitshell " + name;
       
       x := cb_prefGet (name, "fit_pin");
       if (x == vfpgaFitPin.Keep) str = str + " -pre keep";
       else if (x == vfpgaFitPin.Ignore) str = str + " -pre ignore";

       x = cb_prefGet (name, "device");
       if (!string_eq (x, "")) str = str + " -dev " + x;
       x = cb_prefGet (name, "fit_args");
       if (!string_eq (x, "")) str = str + " " + x;
       str; 
   }
   else {
       /* Just copy .tt2 to .tt3 */
       CallCopytt2(name);
       str := "";
   }
}


function a_j2v (name, jedname) {
   str := "j2vhdl " + jedname + " -vhdl";
   x := cb_loadGettype(name, FALSE);
   if (!string_eqi (x, "jedec")) str = str + " -ord";
   else str = str + " -sym";
   dev := cb_prefGet (name, "device");
   if (!string_eq (dev, "")) str = str + " -dev " + dev;

   part := cb_prefGet (name, "part");
   delay := cb_prefGet (name, "j2v_delay");
   utf := cb_prefGet (name, "j2v_utf");

   if ((!string_eq (part, "")) || utf || (delay != vfpgaJ2VDelay.Typ))
      str = str + " -dly";
   if (!string_eq (part, ""))
      str = str + " " + part;
   else if (utf)
      str = str + " local";
   if (delay == vfpgaJ2VDelay.Min)
      str = str + " min";
   else if (delay == vfpgaJ2VDelay.Max)
      str = str + " max";

   str; }

function a_j2vhdl (name) { a_j2v(name, name); }
function a_timj2v (name) { 
    jedname := cb_prefGet (name, "jed_out");
    a_j2v(name, jedname); 
}



function a_wir2abl (name) { "wir2abl " + name + " -o " + name + ".abl"; }


function a_jed2ahdl (name) {
   str := "jed2ahdl " + name;
   x := cb_prefGet (name, "device");
   if (!string_eq (x, "")) str = str + " -dev " + x;
   str; }


function a_fuse (name, jedname) {
   str := "fuseasm " + name;

   /* String options */
   x := cb_prefGet (name, "device");
   if (!string_eq (x, "")) str = str + " -dev " + x;
   x = cb_prefGet (name, "jed_ues");
   if (!string_eq (x, "")) str = str + " -ues " + x;

   /* File format */
   s := ".jed";
   x = cb_prefGet (name, "jed_fmt");
   if (string_eq  (x, "pof")) s   = ".pof";
   if (!string_eq (x, "default"))
      if (string_ref (x, 0) == '8') s   = ".p" + x;
   str = str + " -o " + jedname + s;
   if (!string_eq (x, "default")) str = str + " -format " + x;

   /* Config options */
   s = "";
   if (cb_prefGet (name, "jed_turbo")) s = s + " turbo";
   if (cb_prefGet (name, "jed_miser")) s = s + " miser";
   if (cb_prefGet (name, "jed_blow"))  s = s + " ptintact";
   if (cb_prefGet (name, "jed_lock"))  s = s + " lock";
   if (!string_eq (s, "")) str = str + " -config" + s;

   /* Other options */
   x = cb_prefGet (name, "jed_xval");
   if (x != vfpgaJedXval.Zero) str = str + " -x 1 ";
   x = cb_prefGet (name, "jed_chksum");
   if (x == vfpgaJedChksum.Dummy) str = str + " -check dummy ";
   else if (x == vfpgaJedChksum.None) str = str + " -check none ";
   s = "";
   if (cb_prefGet (name, "jed_plcc")) s = s + " plcc";
   if (cb_prefGet (name, "jed_docfmt") == vfpgaJedDocfmt.Long)
      s = s + " long";
   if (!string_eq (s, "")) str = str + " -doc" + s;

   str; }

function a_fuseasm (name) { a_fuse(name, name); }
function a_timfuse (name) { 
   jedname := cb_prefGet (name, "jed_out");
   if (string_eq (jedname, "")) jedname = name;
   a_fuse(name, jedname); 
}


function a_autopart (name) {
   str := "autopart " + name + ".prt -o " + name + ".pr1";
   str = str + " -ana " + name + ".ana -gen";

   outname := cb_prefGet (name, "part_out");
   if (!string_eq (outname, "")) str = str + " -base " + outname;

   x := cb_prefGet (name, "part_auto");
   y := cb_prefGet (name, "part_strat");
   if ((!string_eq (x, "")) && (!string_eq (y, "newnone")))
       str = str + " -auto -dev " + x;
   x = cb_prefGet (name, "part_args");
   if (!string_eq (x, "")) str = str + " " + x;

   str; }


function a_devsel (name) {
   str := "devsel " + name + " -log " + name + ".chp -o " + name + ".sel";

   /* String options */
   x := cb_prefGet (name, "device");
   if ((!string_eq (x, "")) && (!string_eqi (x, "ANY")))
         str = str + " -dev " + x;
   x = cb_prefGet (name, "sel_man");
   if (!string_eq (x, "")) str = str + " -man " + x;
   x = cb_prefGet (name, "sel_temp");
   if (!string_eq (x, "")) str = str + " -spc " + x;
   x = cb_prefGet (name, "sel_tech");
   if (!string_eq (x, "")) str = str + " -tech " + x;
   x = cb_prefGet (name, "sel_pkg");
   if (!string_eq (x, "")) str = str + " -package " + x;

   /* Numeric options */
   x = cb_prefGet (name, "sel_pins");
   if (!string_eq (x, "")) str = str + " -pins " + x;
   x = cb_prefGet (name, "sel_spd");
   if (!string_eq (x, "")) str = str + " -speed " + x;
   x = cb_prefGet (name, "sel_set");
   if (!string_eq (x, "")) str = str + " -setup " + x;
   x = cb_prefGet (name, "sel_hold");
   if (!string_eq (x, "")) str = str + " -hold " + x;
   x = cb_prefGet (name, "sel_cost");
   if (!string_eq (x, "")) str = str + " -price " + x;
   x = cb_prefGet (name, "sel_pow");
   if (!string_eq (x, "")) str = str + " -power " + x;
   x = cb_prefGet (name, "sel_u1");
   if (!string_eq (x, "")) str = str + " -uf1 " + x;
/*   x = cb_prefGet (name, "sel_u2");
   if (!string_eq (x, "")) str = str + " -uf2 " + x; */
   x = cb_prefGet (name, "sel_sort");
   if (!string_eq (x, "")) str = str + " -sort " + x;

   /* Other options */
   x = cb_prefGet (name, "sel_jam");
   if (x == vfpgaSelJam.No) str = str + " -jam no";
   else if (x == vfpgaSelJam.Yes) str = str + " -jam yes";
   x = cb_prefGet (name, "sel_era");
   if (x == vfpgaSelEra.No) str = str + " -era no";
   else if (x == vfpgaSelEra.Yes) str = str + " -era yes";
   else if (x == vfpgaSelEra.EE) str = str + " -era ee";
   else if (x == vfpgaSelEra.UV) str = str + " -era uv";
   
   str; }


/*** VHDL tool command lines  ***/


function vdes_func (name, isdio) {
   declare x, m;

   /* Function arguments */
   str := "vhdldes " + name;
   if (isdio) str = str + " -tec=dio";
   else {
      x = cb_prefGet (name, "technology");
      if (!string_eq (x, "")) str = str + " -tec=" + x; }

   /* Optimization argument and max_delay (don't use max_delay unless area) */
   x = cb_prefGet (name, "vdes_opt");
   if (x == vfpgaVdesOpt.Speed) str = str + " -fast";
   else if (x == vfpgaVdesOpt.None) str = str + " -nooptimize";
   m = cb_prefGet (name, "vdes_max");
   if ((!string_eq (m, "")) && (x == vfpgaVdesOpt.Area))
      str = str + " -max_delay=" + m;

   /* Other arguments */
   x = cb_prefGet (name, "vdes_file");
   if (!string_eq (x, "")) str = str + " -timing=" + x;
   x = cb_prefGet (name, "vdes_args");
   if (!string_eq (x, "")) str = str + " " + x;

   str; }


function a_vdes_vhdl (name) { vdes_func (name, FALSE); }
function a_vdes_dio (name)  { vdes_func (name, TRUE); }
/* Do not run vsm after synthesis except upon top level design */
function a_vdes_viewsim (name)  { 
    if ((cb_loadIstop (name)) && (cb_prefGet (name, "vdes_vsm")))
       str := "vsm " + name + " vhdl";
    else str := "";
    str;
}
function a_vdes_viewgen (name)  { 
    if (cb_prefGet (name, "vdes_vgen")) "viewgen " + name;
    else "";
}


function a_vhdl_ana (name) {
   str := "vhdl " + name + " -v";
   x := cb_prefGet (name, "vhdl_err");
   if (!string_eq (x, "1")) str = str + " -e=" + x;

   str; }

function a_vhdl2sym (name) {
   str := "vhdl2sym " + name + " -c";
   n := cb_prefGet (name, "sym_pinspace");
   if (n != 10) str = str + sprintf (" -s %d", n);
   n := cb_prefGet (name, "sym_pinlen");
   if (n != 20) str = str + sprintf (" -p %d", n);
   str; }


/*** Other tool command lines  ***/


function a_sch2prt (name) { 
    if (!string_eq(cb_prefGet(name, "part_strat"), "oldauto"))
        str := "";
    else {
        x := cb_prefGet (name, "part_out");
        str := "sch2prt " + x + " -o " + name;
        x = cb_prefGet (name, "part_auto");
        if (!string_eq(x, "")) str = str + " -auto " + x;
    }
    str;
}

function a_prt2sch (name) { 
   str := "prt2sch " + name + ".pr1";
   outname := cb_prefGet (name, "part_out");
   if (!string_eq(outname, "")) str = str + " -o " + outname; 
   if (!cb_prefGet (name, "p2s_sym")) str = str + " -nosym";
   str;
}

function a_viewgen (name) { "viewgen " + name; }
function a_vsm     (name) { "vsm " + name + " vhdl"; }

function a_vg_sym  (name) {
   str := "viewgen " + name + " -makesym -noschem";
   n := cb_prefGet (name, "sym_pinspace");
   if (n != 10) str = str + sprintf (" -sympinspc %d", n);
   n := cb_prefGet (name, "sym_pinlen");
   if (n != 20) str = str + sprintf (" -sympinlth %d", n);
   str; }

function a_prtcheck (name) { "check -s " + cb_prefGet (name, "part_out"); }


/*** ERROR FUNCTIONS ***/


function errWrap (errno, text, name) {
   cb_mainErr (errno, "Error", nil, text + " failed on " + name); }


function errInfile (errno, text, name, extension) {
   /* Should "locate" file to find distant VHDL, for example */
   cb_mainErr (errno, "Error", "file " + name + "." + extension,
      text + " failed"); }


/* Set up a regular expression to track just a filename, no line number */
if ((vcWv.UI(wv) == vcUi.Motif) || (vcWv.UI(wv) == vcUi.OpenLook))
    vcAlert.AddVisit ("^file ([^,]*)$", func (src, sev, num, loc, data) {
        name := vscript.re_getmatch (vcAlert.last_regexp, 1);
        vcWin.Create(vcRegion.Create (name, vtx.fac, name, load_file <- TRUE));
        nil; } );


/*** PLA2VIEW tool errors ***/


function e_p2v_dio (name) { errWrap (1400, "PLA2VIEW translator", name); }
function e_p2v_vli (name) { errWrap (1401, "PLA2VIEW translator", name); }
function e_p2v_sym (name) { errWrap (1402, "PLA2VIEW translator", name); }
function e_p2v_diosch (name) { errWrap (1400, "PLA2VIEW translator", name); }


/*** DIO tool errors ***/


function e_ahdl2pla (name) { errInfile (1404, "ABEL compiler", name, "abl"); }
function e_ahdl2prt (name) { errInfile (1404, "ABEL compiler", name, "abl"); }
function e_plaopt (name) { errWrap (1406, "PLA optimizer", name); }
function e_autopart (name) { errWrap (1407, "Automatic partitioner", name); }
function e_fit (name) { errWrap (1408, "Device fitter", name); }
function e_fuseasm (name) { errWrap (1409, "JEDEC assembler", name); }
function e_j2vhdl (name) { errWrap (1410, "Timing modeller", name); }
function e_devsel (name) { errWrap (1411, "Device selector", name); }
function e_jed2ahdl (name) { errWrap (1412, "JEDEC translator", name); }
function e_wir2abl (name) { errWrap (1413, "Netlist to ABEL translator",name); }


/*** VHDL tool errors ***/


function e_vhdl_ana (name) { errInfile (1417, "VHDL analyzer", name, "vhd"); }
function e_vdes_dio (name) { errInfile (1416, "VHDL translator", name, "vhd"); }
function e_vhdl2sym (name) { errInfile (1418, "VHDL symbol builder",
   name, "vhd"); }

/* Special case error with no technology specified */
function e_vdes_vhdl (name) {
   x := cb_prefGet (name, "technology");
   if (string_eq (x, "")) cb_mainErr (1424, "Error", nil,
      "No synthesis technology specified for " + name);
   else errInfile (1415, "VHDL synthesizer", name, "vhd"); }


/*** Other tool errors ***/


function e_sch2prt (name) { errWrap (1419, "Schematic translator", name); }
function e_prt2sch (name) { errWrap (1420, "Schematic writer", name); }
function e_vsm (name) { errWrap (1423, "Viewsim netlister", name); }
function e_viewgen (name) {
   cb_mainErr (1421, "Error", nil,
      "Viewgen failed to create schematic for " + name); }
function e_vg_sym (name) {
   cb_mainErr (1422, "Error", nil,
      "Viewgen failed to create symbol for " + name); }
