/*
 *  $Revision: 1.21 $
 *
 *  Viewscript UI definitions for the Kanji editor.
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

/* This file is most easily edited with an editor with 103 columns or more */


/* process valid input keys pressed */
chr_cmd := vcWv.Cmd (fac, chr);

modified_cmdline_bindings := vcBind.Duplicate (vcCmdline.bindings);

foreach (c; vcWv.Printable ()) {
    vcBind.Set (bindings, c, chr_cmd);
    vcBind.Set (modified_cmdline_bindings, c, nil);
} /* foreach */
for (c := 128; c <= 255; ++c) {
    vcBind.Set (bindings, int2char (c), chr_cmd);
    vcBind.Set (modified_cmdline_bindings, int2char (c), nil);
} /* for */

vcBind.Set (modified_cmdline_bindings, '\e', vcCmdline.DelAllCmd);

SetModifiedCmdlineBindings (modified_cmdline_bindings);

cvt := cons (nil, nil);
edit    := cons (nil, nil);
/* mode    := cons (nil, nil); */
kbd     := cons (nil, nil);

function Kanji(english, japanese) {
    function k()
        vcTable.Make ("", 1, 2,
                      vcRegion.Create ("", vjLabel.fac, japanese, width <- 0),
                      vcRegion.Create ("", vcLabel.fac, english));
    k;
} /* function Kanji */

function KCmd(cmd, menu, kanji_entry, key, help) {
    if (menu != nil)
        set_car (menu, cons (list (kanji_entry, cmd), car (menu)));
    if (key != nil) {
        if (iscons (key))
            foreach (k; vscript.list_iter (key))
                vcBind.Set (bindings, k, cmd);
        else
            vcBind.Set (bindings, key, cmd);
    } /* if */
    vcCmd.SetHelp (cmd, help);
} /* function K */
                  
function K(vkey, menu, kanji_entry, key, help) {
    cmd := vcWv.Cmd (fac, func (k, e) process (k, e, vkey););
    if (menu != nil) {
        if (kanji_entry == nil)
            kanji_entry = vkey;
        else
            kanji_entry = Kanji (vkey, kanji_entry);
    } /* if */
    KCmd (cmd, menu, kanji_entry, key, help);
} /* function K */

function dictionary(editor, event) {
    reading := vcRegion.Create ("", vjLabel.fac, "");
    notation := vcRegion.Create ("", vjLabel.fac, "");

    reading_str := "";
    notation_str := "";

    function k(english, jap, val) cons (Kanji (english, jap), val);
    part_of_speech := vcRegion.Create ("", vcPicklist.fac,
                       list (list (k ("Noun", "\226\274\216\214", 0x78),
                                   k ("Person's Name", "\220l\226\274", 0x79),
                                   k ("Place's Name", "\222n\226\274", 0x7a)),
               list (k ("Proper Noun", "\214\305\227L\226\274\216\214", 0x7b),
                     k ("Special Noun", "\203T\225\317\226\274\216\214", 0x7c),
                     k ("1 Kanji Character", "\222P\212\277\216\232", 0x70))));

    vcPicklist.SetPick (part_of_speech, 0); /* Noun is default */

    function do_set(l, strp) {
        *strp = Get (editor);
        vjLabel.Set (l, *strp);
    } /* function do_set */
        
    function do_dict(code)
        Dict (editor, reading_str, notation_str,
              vcRegion.Get (part_of_speech, integer),
              code);

    t := vcTable.Make ("", 3, 2,
                       f := vcWv.LabelButtonCreate ("", "Set `Reading'", nil,
                                                    func() do_set (reading,
                                                                &reading_str);,
                                                    nil),
                       reading,
                       vcWv.LabelButtonCreate ("", "Set `Notation'", nil,
                                               func() do_set (notation,
                                                              &notation_str);,
                                               nil),
                       notation,
                       vcRegion.Create ("", vcLabel.fac, "Part of Speech"),
                       part_of_speech);
    t2 := vcTable.Make ("", 1, 3,
                        vcWv.LabelButtonCreate ("", "Add Word", nil,
                                                func() do_dict ("Touroku");,
                                                nil),
                        vcWv.LabelButtonCreate ("", "Delete Word", nil,
                                                func() do_dict ("Sakujyo");,
                                                nil),
                        vcWv.LabelButtonCreate ("", "Dismiss", nil,
                                                func ();,
                                                list (close_window <- TRUE)));
    t = vcTable.Make ("Japanese Word Registration", 2, 1,
                      t, t2);
    vcWin.Popup (vcEvent.Window (event), t, f, vcWin.NoStatus);
} /* function dictionary */

dict_cmd := vcWv.Cmd (fac, dictionary);

function create_pick(k, event) {
    pick := vcRegion.Create ("", vjPick.fac, nil);
    t := vcTable.Make ("Kanji JIS Table", 2, 1,
                       pick,
                       dismiss := vcWv.LabelButtonCreate ("", "Dismiss", nil,
                                               func ();,
                                               list (close_window <- TRUE)));
    vcRegion.Align (dismiss, vgAlign.MiddleCenter);

    vjPick.CallOnPick (pick,
                       func(high, low) {
                           process (k, event, "No_Cnv2");
                           ProcessCode (k, event, high * 256 + low);
                       });
    vcWin.Popup (vcEvent.Window (event), t, pick, vcWin.Bindings);
} /* function create_pick */

pick_cmd := vcWv.Cmd (fac, create_pick);

function get_ascii(editor, event) {
    ascii := Ascii (editor);
    w := vcEvent.Window (event);
    vcWin.ClearStatus (w);
    vcWin.AddStatus (w, ascii + " stored in Cut Buffer");
    vcRegion.StoreCutBuffer (vcEvent.Region (event), ascii);
} /* function get_ascii */

function _ClearSelection(edit_region) {
    vcWin.ClearStatus (vcRegion.Window (edit_region));
} /* function _ClearSelection */

ascii_cmd := vcWv.Cmd (fac, get_ascii);

function list_to_menu(name, l) {
/*
 *    l = reverse_destructive (car (l));
 *    print (l);
 *    vcMenu.Create (name, l);
 */
    reverse_destructive (car (l));
} /* function list_to_menu */

l := list;

/*
 * This table defines the basic user interface for the VJE front end.
 * It is derived from consulation by Tai-Ji, and from the VJE
 * documentation.
 *
 * 
 * The virtual key is the name of the VJE virtual keycode (without the F_)
 * which the menu pcik will generate.  It is used when the entry is a call
 * to function K.  Some of the entries are calls to the function KCmd,
 * which are for commands not directly supported by a single VJE code.
 * They refer to a command object created with the function vcWv.Cmd,
 * above in this file.
 *
 * The Menu is one of the following:  cvt (convert), edit, kbd.
 * If nil is specified, the entry is not added to a menu.
 *
 * The Japanese string is a shift-jis string to put in the menu entry.
 * We use the \NNN octal format, rather than putting in the shift
 * jis directly, which the Viewscript parser cannot yet handle.
 * If nil is specified, no japenese is included in the menu.
 *
 * The Key(s) argument is specified as a Viewscript character constant,
 * which includes all the C character constants, plus function keys,
 * mouse buttons, special keys (like pgup and pgdn), etc.  More than
 * one key can be bound to the same function by specifying the keys
 * as a list.  For example, l(k1, k2, k3...).
 *
 * The help text is displayed when the user highlights a menu entry.
 * For now, it can only be displayed in English.
 *
 * Virtual Key/Command  Menu    Japanese String         Key(s)          Help text
 * ------------------------------------------------
 */
K ("Cnv",               cvt,    "\225\317\212\267",     ' ',            "Convert input buffer to kanji.");
K ("Hira_Cnv",          cvt,    nil,                    '\f5',          "Convert input buffer to karagana");
K ("Kata_Cnv",          cvt,    nil,                    '\f6',          "Convert input buffer to katakana");
K ("Zen_Cnv",           cvt,    nil,                    '\f7',          "Double width ascii & katakana");
K ("Han_Cnv",           cvt,    nil,                    '\f8',          "Single width ascii & katakana");
K ("Code",              cvt,    "\203R\201|\203h",      '\f9',          "Translate Numeric JIS-Code");
KCmd (pick_cmd,         kbd,    "JIS Table...",         '\^\F9',        "Pick Japanese characters from JIS Table");
KCmd (ascii_cmd,        kbd,    "Octal Shift-JIS",   '\^\F10',       "Show Octal Shift-JIS for display.");
K ("No_Cnv",            edit,   nil,                    '\enter',       "Determination.  Accept kana-kanji conversion.");
K ("No_Cnv2",           edit,   nil,                    '\_\enter',    "NFER or transfer the kana-kanji conversion result to result buffer.");
K ("Left",              nil,    nil,                    l('\^B', '\left'), "Move cursor left");
K ("Right",             nil,    nil,                    l('\^F', '\right'), "Move cursor right");
K ("Llim",              kbd,    nil,                    l('\^A','\^\left'),   "Beginning of phrase");
K ("Rlim",              kbd,     nil,    l('\^E','\^\right'),   "End of phrase");
KCmd (dict_cmd,  kbd, "Dictionary...", '\_\F5', "Add/Remove dictionary words");
K ("Space",     edit,    nil,        '\_ ',   "Insert ASCII or JIS space character.");
K ("BS",        nil,     nil,        '\177',  "Backspace");
K ("Del",       nil,     nil,    l('\^D','\delete'),   "Delete character");
K ("Mode",      cvt,    nil,        '\F10', "Toggle input modes");
/* K ("Rom",       mode,     "\203\215\201|\203}\216\232",        '\^\F9', "Romaji input"); */
K ("Cancel",    edit,    nil,        '\e',    "Cancels any previously performed action");
K ("Canall",    edit,    nil,   list ('\^U', '\_\e'),  "Cancels all Kana-Kanji conversion performed");
/*
 * Undo only seems to undo movement to the result buffer.
 * K ("Undo",      edit,    nil,        '\_u',   "Undo");
 */
/* K ("Kana_Lock", kbd,    nil,        nil,     "Flag for Kana-lock key on/off"); */
/*K ("Scrn",      mode,    nil,        '\_\^@', "Screen mode.  Press to reset."); */
/*K ("Hira",      mode,    "\202\320\202\347",        '\^\f5', "Hiragana input");*/
/*K ("Kata",      mode,    "\203J\203^",     '\^\f6', "Katakana input");*/
/*K ("Zen",       mode,    "\221S",        '\^\f7', "Double width input");*/
/*K ("Han",       mode,    "\224\274",     '\^\f8', "Single width input");*/
/*K ("Kigo",      mode,    nil,        '\^\f9', "Continuous code input"); */
/*K ("Eisuu",     kbd,     nil,        '\^\f9', "ASCII QWERTY input");*/
/*
 * Everything you type goes right through to the result buffer.  Boring.
 * K ("Dmode",     mode,    nil,        '\^\f0', "Direct input mode.  NFER or bypass VJE kana-kanji conversion");
 */
K ("Cnv_Bcnv",  nil,    nil,        '\^\f1', "Previous Homonym");
K ("Cnv_Bblk",  edit,   nil,        '\up',   "Display previous set of 9 homonyms");
K ("Cnv_Nblk",  edit,   nil,        '\down', "Display next set of 9 homonyms");
K ("Cnv_Bkoho", nil,    nil,        '\^\_\left', "Display previous homonym");
K ("Cnv_Nkoho", nil,    nil,        '\^\_\right', "Display next homonym");
K ("Cnv_Buns",  kbd, "\225\266\220\337\217k\217\254", '\_\left', "Shorten the current bunsetsu");
K ("Cnv_Bunl",  kbd, "\225\266\220\337\212g\222\243", '\_\right', "Lengthen the current bunsetsu");


/*
 * K ("Touroku",   "Register a phrase with a given pronunciation");
 * K ("Sakujyo",   "Delete a phrase");
 * K ("Mode",      "Not used");
 * K ("Rev_Space", "Not used");
 * K ("Mb_Sel",    "Not used");
 * K ("Lb_Sel",    "Homonym list selected");
 * K ("Menu",      "Brings up menu");
 * K ("Scrn",      "Screen mode.  It is the default mode.");
 * K ("Ctrl",      "Flag to tell if control key pressed/not pressed");
 * K ("Chr",       "printable characters entered");
 * K ("Knj",       "Kanji input");
 * K ("Open",      "Open the main dictionary");
 * K ("Uopen",     "Open the User dictionary");
 * K ("Code_Cnt",  "");
 * K ("Close",     "Close the main dictionary");
 * K ("Uclose",    "Close the user dictionary");
 * K ("Caps_On",   "Cap Lock on");
 * K ("Caps_Off",  "Cap Lock off");
 * K ("Up",        edit,    '\up',   "???");
 * K ("Down",      edit,    '\down', "???");
 * K ("Mode",      cvt, nil, '\f5',   "Toggle modes");
 */

/* code_cmd := ...list ("Jis Code", integer, vcWv.Sticky, 0));... */

edit = list_to_menu ("edit", edit);
/*mode = list_to_menu ("mode", mode);*/
cvt = list_to_menu ("convert", cvt);
kbd = list_to_menu ("kbd", kbd);
l := list (edit, /*mode,*/ cvt, kbd);
main_menu := vcMenu.Create ("kanji", l);

vcFac.SetMenus (fac, main_menu);

/*vcBind.Set (bindings, '\rightdown',
 *            vcWv.Cmd (fac, func(d,e) vcMenuPop.create(main_menu,e);));
 */
