/******************************************************************************/
/* Filename: vsimutil.c - Implementation of Viewsim <-> C Interface utilities */
/******************************************************************************/

/*  Revision History:
 *  ................... 
 *  June 16, 1992 ..... header file created and stuff added by Graham Davies.
 */

/* This file contains useful arrays for manipulating data used
 * by Viewsim <-> C interface software.  All or any part of this
 * file can be complied and linked in with Viewsim applications.
 */

#include <stdio.h>
/* #define PROTO 1 */
#include "viewsim.h"
#include "vsimutil.h"

/*  The following array can be used to map the Viewsim 28 states into
 * the corresponding 4 state enumerated values LOW4, X, HIGHZ4, HIGH4.
 */
vsc_pin_4values vsc_28to4_state[28] =
{  HIGH4,  HIGH4, HIGH4, X4,    X4,    X4,
   HIGH4,  HIGH4, X4,    X4,    X4,
   HIGH4,  X4,    X4,    X4,
   LOW4,   LOW4,  LOW4,
   LOW4,   LOW4,
   LOW4,
   HIGHZ4, X4,    X4,    X4,    X4,    X4,   X4
};

/*  The following array can be used to map the Viewsim 28 states into
 * the corresponding 3 state enumerated values LOW4, X4, HIGH4.
 */
vsc_pin_4values vsc_28to3_state[28] =
{  HIGH4,  HIGH4, HIGH4, X4,    X4,    X4,
   HIGH4,  HIGH4, X4,    X4,    X4,
   HIGH4,  X4,    X4,    X4,
   LOW4,   LOW4,  LOW4,
   LOW4,   LOW4,
   LOW4,
   X4,     X4,    X4,    X4,    X4,    X4,   X4
};

/*  The following array can be used to invert the 4 state or 3 state
 * enumerated type values.
 */
vsc_pin_4values vsc_4state_not[4] =
{  HIGH4,  X4,    X4,    LOW4
};

/*  The following array can be used to display the string representation
 * of a vsc_pin_values type.
 */
char *vsc_28state_display[] =
{  "D1",   "D1W1", "D1C1", "D1C0", "D1W0", "D1D0", "W1",
   "W1C1", "W1C0", "W1W0", "W1D0", "C1",   "C1C0", "C1W0",
   "C1D0", "C0",   "C0W0", "C0D0", "W0",   "W0D0", "D0",
   "Z",    "D1Z",  "W1Z",  "C1Z",  "ZC0",  "ZW0",  "ZD0"
};

/* The following array can be used to display the character representation
 * of a vsc_pin_4values type.
 */
char vsc_4state_display[4] =
{  '0', 'X', 'Z', '1'
};


/* end of file vsimutil.c */
