/*
 *  $Revision: 1.9 $
 *
 *  Implementation for the zsize definition area of the Plot Options Worksheet
 *
 *  Copyright (c) 1991,1993 by Viewlogic Systems, Inc.
 */

package vspoptrc (create_right_col, apply_changes) {

declare zsize_name_reg, zsize_xrange_reg, zsize_yrange_reg,
        zsize_lmarg_reg, zsize_rmarg_reg, zsize_bmarg_reg, zsize_tmarg_reg;

declare zsizex_cmdline, zsizey_cmdline, zmarginl_cmdline, zmarginr_cmdline,
        zmarginb_cmdline, zmargint_cmdline, zsize_name_cmdline;

/* All of these are of the format 'x y'
 */
declare cur_zsizex;
declare cur_zsizey;

/* This is a string
 */
declare cur_zname;
/* Margin is of format 'left right bottom top'
 */
declare cur_zmarginl;
declare cur_zmarginr;
declare cur_zmarginb;
declare cur_zmargint;

/* These are strings which are recognized internal to plotter software
 */
vspZSIZE := "ZSIZE";
vspZNAME := "ZNME";
vspZMARGIN := "ZMRG";

XPRMPT := "X> ";
YPRMPT := " Y> ";

ZSIZE_TITLE := "DEFINE ALTERNATE PAPER SIZE";
ZSIZE_PRMPT := "Range: ";
ZSIZE_NAME := "Name: ";
ZMARG_PRMPT := "Margins: ";
SPACE_PRMPT := "         ";
X1PRMPT := "Left> ";
Y1PRMPT := " Right> ";
X2PRMPT := "Bottom> ";
Y2PRMPT := " Top> ";


/* Called when the 'Apply' button is hit on the plot options worksheet.
 * This is when all changes are actually effected in the plotting software.
 */
function apply_changes () {

        if (cur_zname == nil || cur_zsizex == nil ||
                cur_zsizey == nil || cur_zmarginl == nil ||
                cur_zmarginr == nil || cur_zmarginb == nil ||
                cur_zmargint == nil)
                vlEx.Throw2 (vlEx.FAILURE, "vspzsize", vlErrSev.Error,
                             "Cannot have nil values for zsize information");

        fac_pack.vspSetZname (vcRegion.Get (zsize_name_cmdline, string));

        ret1 := fac_pack.sp_sapopt (string_append(vspZSIZE, " ", 
                vcRegion.Get (zsizex_cmdline, string), " ", 
                vcRegion.Get (zsizey_cmdline, string)));

        ret2 := fac_pack.sp_sapopt (string_append
                (vspZMARGIN, " ", vcRegion.Get (zmarginl_cmdline, string), 
                        " ", vcRegion.Get (zmarginr_cmdline, string),
                        " ", vcRegion.Get (zmarginb_cmdline, string), " ", 
                        vcRegion.Get (zmargint_cmdline, string)));

        if (ret1 && ret2) {
                /* Make sure the paper picklist reflects the new choice */
                update_paper_pklst ();
                TRUE;
        }
        else
                FALSE;
}

/* Called when either the x or y input areas for the range field of the
 * zsize area is changed by the user.
 */
function set_zsize (which, val) {
        /* make sure got a float */
        convert (float, val);
        set_options_modified ();
        if (which == `zsizex)
                cur_zsizex = val;
        else if (which == `zsizey)
                cur_zsizey = val;
}

/* Assures that the zsize and zmargin command lines reflect the current vals.
 */
function update_cmdlines () {

        vcCmdline.Set (zsize_xrange_reg, cur_zsizex);
        vcCmdline.Set (zsize_yrange_reg, cur_zsizey);
        vcCmdline.Set (zsize_lmarg_reg, cur_zmarginl);
        vcCmdline.Set (zsize_rmarg_reg, cur_zmarginr);
        vcCmdline.Set (zsize_bmarg_reg, cur_zmarginb);
        vcCmdline.Set (zsize_tmarg_reg, cur_zmargint);
}

/* Sets all z-values to be their assoc. vals.
 */
function set_rest (name) {

        cur_zsizex = fac_pack.get_named_zsize (name, vspFIRSTVAL);

        cur_zsizey = fac_pack.get_named_zsize (name, vspSECONDVAL);

        cur_zmarginl = fac_pack.get_named_zmargin (name, vspFIRSTVAL);

        cur_zmarginr = fac_pack.get_named_zmargin (name, vspSECONDVAL);

        cur_zmarginb = fac_pack.get_named_zmargin (name, vspTHIRDVAL);

        cur_zmargint = fac_pack.get_named_zmargin (name, vspFOURTHVAL);
        update_cmdlines ();
}

/* Sets all z-values to be zsize defaults.
 */
function def_rest () {

        cur_zsizex = fac_pack.get_def_zsize (vspFIRSTVAL);

        cur_zsizey = fac_pack.get_def_zsize (vspSECONDVAL);

        cur_zmarginl = fac_pack.get_def_zmargin (vspFIRSTVAL);

        cur_zmarginr = fac_pack.get_def_zmargin (vspSECONDVAL);

        cur_zmarginb = fac_pack.get_def_zmargin (vspTHIRDVAL);

        cur_zmargint = fac_pack.get_def_zmargin (vspFOURTHVAL);
        update_cmdlines ();
}

/* Asks user if wish to overwrite the existing user-defined paper or not.
 * If user answers in the affirmative, then the zname is saved and the
 * other values are defaulted to zsize defaults.  Otherwise, the zname
 * field is set to nil, flagged that it will not be updated.
 */
function overwrite_zname (val, event, y_or_n) {
        if (y_or_n == convert (vlYesNoVT, "yes")) {
                save_zname (val);
                set_rest (val);
        }       
        else
                vcCmdline.Set (zsize_name_reg, nil);
}

overwrite_zname_cmd := vcWv.Cmd (fac, overwrite_zname, "Overwrite_zname_cmd",
                                "Overwrite Zname", 
                                vcWv.Param("Paper name already exists, redefine?",
                                vlYesNoVT, vcWv.Constant, "Yes"));

function save_zname (val) {
        set_options_modified ();
        cur_zname = val;
}

/* Called when either the ZSIZE NAME input area is changed by the user.
 */
function set_zname (data, val) {

        if (fac_pack.zname_exists (val))
                vcCmd.Run (overwrite_zname_cmd, val, 
                                vcEvent.Create(0, 0, 32, nil, options_win_id,
                                        zsize_name_reg, nil), nil, TRUE);
        else {
                save_zname (val);
                /* Set the rest of the z-values to be zsize defaults */
                def_rest ();
        }
}

/* Called when the any of the four ZSIZE MARGINS input areas are changed
 * by the user.
 */
function set_zmargin (which, val) {
        /* Make sure got a float type */
        convert(float, val);
        set_options_modified ();
        if (which == `zmarginl)
                cur_zmarginl = val;
        else if (which == `zmarginr)
                cur_zmarginr = val;
        else if (which == `zmarginb)
                cur_zmarginb = val;
        else if (which == `zmargint)
                cur_zmargint = val;
}

/* Used to define new values for the z margin.
 */
function create_quad_cmdlines (title, first_pmpt, second_pmpt, third_pmpt,
        fourth_pmpt, def_func, set_func, sym1, sym2, sym3, sym4) {

        reg := vcTable.Create (string_append(title, "quad_cmdlines"), 1, 2);

        tlst := create_double_cmdline_list (string_append (title, first_pmpt), 
                second_pmpt, fac_pack.get_def_zmargin (vspFIRSTVAL), 
                fac_pack.get_def_zmargin (vspSECONDVAL), set_zmargin, 
                `zmarginl, `zmarginr, FALSE);
        zsize_lmarg_reg = car (tlst);
        zsize_rmarg_reg = cadr (tlst);
        vcTable.Add (reg, caddr (tlst));

        tlst = create_double_cmdline_list (third_pmpt, 
                fourth_pmpt, fac_pack.get_def_zmargin (vspTHIRDVAL), 
                fac_pack.get_def_zmargin (vspFOURTHVAL), set_zmargin, 
                `zmarginb, `zmargint, FALSE);
        zsize_bmarg_reg = car (tlst);
        zsize_tmarg_reg = cadr (tlst);
        vcTable.Add (reg, caddr (tlst));

        reg;
}
                
function create_zsize_workarea () {

        reg := vcTable.Create ("zsize_workarea", 4, 1);
        vcTable.Col (reg, -1, vcTable.Border.Invisible,
                        3, 7, 11, vgAlign.LowerLeft);

        zsize_name_list := create_cmdline (ZSIZE_NAME, 
                                fac_pack.get_def_zname(), set_zname, FALSE,
                                8, 12);
        zsize_name_cmdline = car (zsize_name_list);
        zsize_name_reg = cadr (zsize_name_list);
        vcTable.Add(reg, zsize_name_reg);

        tlst := create_double_cmdline_list (string_append (ZSIZE_PRMPT, 
                XPRMPT), YPRMPT, fac_pack.get_def_zsize (vspFIRSTVAL), 
                fac_pack.get_def_zsize (vspSECONDVAL), set_zsize,
                `zsizex, `zsizey, FALSE);
        zsize_xrange_reg = car (tlst);
        zsize_yrange_reg = cadr (tlst);
        vcTable.Add(reg, caddr (tlst));
        zsizex_cmdline = cadddr (tlst);
        zsizey_cmdline = cadr (cdddr (tlst));


        tlst := create_double_cmdline_list (string_append (ZMARG_PRMPT,
                X1PRMPT), Y1PRMPT, fac_pack.get_def_zmargin (vspFIRSTVAL), 
                fac_pack.get_def_zmargin (vspSECONDVAL), set_zmargin, 
                `zmarginl, `zmarginr, FALSE);
        zsize_lmarg_reg = car (tlst);
        zsize_rmarg_reg = cadr (tlst);
        vcTable.Add (reg, caddr (tlst));
        zmarginl_cmdline = cadddr (tlst);
        zmarginr_cmdline = cadr (cdddr (tlst));
        
        tlst = create_double_cmdline_list (string_append (SPACE_PRMPT,
                X2PRMPT), Y2PRMPT, fac_pack.get_def_zmargin (vspTHIRDVAL), 
                fac_pack.get_def_zmargin (vspFOURTHVAL), set_zmargin, 
                `zmarginb, `zmargint, FALSE);
        zsize_bmarg_reg = car (tlst);
        zsize_tmarg_reg = cadr (tlst);
        vcTable.Add (reg, caddr (tlst));
        zmarginb_cmdline = cadddr (tlst);
        zmargint_cmdline = cadr (cdddr (tlst));

/*
        vcTable.Add(reg, create_quad_cmdlines (ZMARG_PRMPT, X1PRMPT,
                Y1PRMPT, X2PRMPT, Y2PRMPT, fac_pack.get_def_zmargin,
                set_zmargin, `zmarginl, `zmarginr, `zmarginb, `zmargint));
 */

        titled_reg := vcTable.Create ("titled_zsize_workarea", 2, 1);
        vcTable.Add(titled_reg, vcRegion.Create1 (ZSIZE_TITLE, vcLabel.fac, 
                                ZSIZE_TITLE, nil));
        vcTable.Add(titled_reg, reg);
        vcRegion.Create1("zsize_workarea.border", vcBorder.fac, 
                         titled_reg, nil);
}

/* Creates the right column of the Plot Options Worksheet.
 */
function create_right_col () {
        
        cur_zsizex = fac_pack.get_def_zsize (vspFIRSTVAL);
        cur_zsizey = fac_pack.get_def_zsize (vspSECONDVAL);
        cur_zname = fac_pack.get_def_zname ();
        cur_zmarginl = fac_pack.get_def_zmargin (vspFIRSTVAL);
        cur_zmarginr = fac_pack.get_def_zmargin (vspSECONDVAL);
        cur_zmarginb = fac_pack.get_def_zmargin (vspTHIRDVAL);
        cur_zmargint = fac_pack.get_def_zmargin (vspFOURTHVAL);

        create_zsize_workarea ();
}
                
}
