/* ------------------------------------------------------------------------ *\
 * viewtrace viewscript stuff
 *
 * <<<<Prerequisites>>>>
 * <<<<Command Definitions>>>>
\* ------------------------------------------------------------------------ */

/*
 * A couple of functions to help ease typing.
 */
function bindcmd(key, command)  { vcBind.Set(bindings, key, command); }

/* <<<<Prerequisites>>>> */

/* YES NO symbols */
YES := vlYesNo.Yes;
NO := vlYesNo.No;

/* Symbols for default parameter strategies */
STICKY := vcWv.Sticky;
CONSTANT := vcWv.Constant;
DYNAMIC := vcWv.Dynamic;

/* Global variables to control various settings.  Eventually replace with
   preferences */
POPUP_INFO_BOX := YES;
PROMPT_OVERWRITES := YES;
NAV_SELECT := NO;

/* Setup interrupt keys for display cancellation */
vcWv.Interrupts(fac, '\e', '\^C', '\_\^C',
                '\F4', '\F5', '\F6', '\F7', '\F8', '\F9', '\F0',
                '\pgup', '\pgdn', '\home', '\end', '\up', '\down' );

/* ZOOMER */

ZoomHAxisCmd := vcWv.Cmd(fac, ZoomHAxis, "ZoomHAxis", "ZoomHAxis",
                         vcWv.Param("Axis Index", integer, STICKY, 0),
                         vcWv.Param("Lower limit", float, STICKY, 0.0),
                         vcWv.Param("Upper limit", float, STICKY, 1.0));

ZoomVAxisCmd := vcWv.Cmd(fac, ZoomVAxis, "ZoomVAxis", "ZoomVAxis",
                         vcWv.Param("Axis Index", integer, STICKY, 0),
                         vcWv.Param("Lower limit", float, STICKY, 0.0),
                         vcWv.Param("Upper limit", float, STICKY, 1.0));

function
IsClick(area)
{
    (   abs(vgRect.Xmax(area) - vgRect.Xmin(area)) > 5
     || abs(vgRect.Ymax(area) - vgRect.Ymin(area)) > 5);
} /* IsClick */

function
StartZoom(display, event)
{
    area := vgRect.Create(0, 0, 0, 0);
    type := GetZoomArea(display, event, area);

    /* zoom area */
    if (type == 1) {
        vcDrag.Create(event, area, vgColor.White,
                      func(new_area) {
                          if (IsClick(new_area)) {
                              {
                                  HIndex := GetRegionCurrentHAxisIndex(display,
                                                                       event);
                                  left := HCoord2Value(vgRect.Xmin(new_area),
                                                       HIndex);
                                  right := HCoord2Value(vgRect.Xmax(new_area),
                                                        HIndex);
                                  vcCmd.Run(ZoomHAxisCmd, display, event,
                                            list(HIndex, left, right), TRUE);
                              }
                              {
                                  VIndex := GetRegionCurrentVAxisIndex(display,
                                                                       event);
                                  bottom := VCoord2Value(vgRect.Ymin(new_area),
                                                         VIndex);
                                  top := VCoord2Value(vgRect.Ymax(new_area),
                                                      VIndex);
                                  vcCmd.Run(ZoomVAxisCmd, display, event,
                                            list(VIndex, bottom, top), TRUE);
                              }
                              vcCmd.Run(ViewRefreshCmd, display, event, nil,
                                        TRUE);
                          } /* if */
                          else {
                              vcCmd.Run(ViewInCmd, display, event, nil, TRUE);
                          } /* else */
                      }, "Box",
                      vgRect.Xmax(area) - vgRect.Xmin(area),
                      vgRect.Ymax(area) - vgRect.Ymin(area),
                      nil);
    } /* if */
    /* zoom horizontal */
    else if (type == 2) {
        vcDrag.Create(event, area, vgColor.Red,
                      func(xmin, xmax) {
                          HIndex := GetPointHAxisIndex(display, event);
                          if (HIndex == -1)
                              HIndex := GetRegionCurrentHAxisIndex(display,
                                                                   event);
                          left := HCoord2Value(xmin, HIndex);
                          right := HCoord2Value(xmax, HIndex);
                          if (left > right) {
                              tmp := left;
                              left = right;
                              right = tmp;
                          } /* if */
                          MinLeft := HAxisVMin(display, HIndex);
                          if (left < MinLeft)
                              left = MinLeft;
                          MaxRight := HAxisVMax(display, HIndex);
                          if (right > MaxRight)
                              right = MaxRight;
                          vcCmd.Run(ZoomHAxisCmd, display, event,
                                    list(HIndex, left, right), TRUE);
                          vcCmd.Run(ViewRefreshCmd, display, event, nil, TRUE);
                      }, "HBar", vgRect.Ymin(area), vgRect.Ymax(area), nil);
    } /* else if */
    /* zoom vertical */
    else if (type == 3) {
        vcDrag.Create(event, area, vgColor.Red,
                      func(ymin, ymax) {
                          VIndex := GetPointVAxisIndex(display, event);
                          bottom := VCoord2Value(ymin, VIndex);
                          top := VCoord2Value(ymax, VIndex);
                          if (bottom > top) {
                              tmp := bottom;
                              bottom = top;
                              top = tmp;
                          } /* if */
                          MinBottom := VAxisVMin(display, VIndex);
                          if (bottom < MinBottom)
                              bottom = MinBottom;
                          MaxTop := VAxisVMax(display, VIndex);
                          if (top > MaxTop)
                              top = MaxTop;
                          vcCmd.Run(ZoomVAxisCmd, display, event,
                                    list(VIndex, bottom, top), TRUE);
                          vcCmd.Run(ViewRefreshCmd, display, event, nil, TRUE);
                      }, "VBar", vgRect.Xmin(area), vgRect.Xmax(area), nil);
    } /* else if */
} /* StartZoom */

/* EDIT interaction */
function
StartPlaceRegion(display, event, TheCmd)
{
    vcCmd.Run(UnSelectSignalRegionCmd, display, event, nil, TRUE);
    area := vgRect.Create(0, 0, 0, 0);
    type := GetArea(display, event, area);

    Old_x := 0; Old_y := 0;
    New_x := 0; New_y := 0;
    declare x;
    declare time, oldtime;

    gspace := GetMajorGridSpace(display);
    snap := get_snap_mode(display);
    win := vcEvent.Window(event);
    declare CutBufferTime, CutBufferHeight;
    function set_height(x, y)
    {
        CutBufferDims := GetCutBufferDims(display, x, y);
        CutBufferTime = car(CutBufferDims);
        CutBufferHeight = cadr(CutBufferDims);
    }
    declare CutBufferWidth;

    function start(x, y) {
        Old_x = New_x = x;
        Old_y = New_y = y;

        set_height(New_x, New_y);
        CutBufferWidth =   time2pixel(display, CutBufferTime)
                         - time2pixel(display, 0.0);

        if (snap != vtrSnapMode.Off) {
            time = pixel2time(display, x);
            time = round(x / gspace) * gspace;
            x = time2pixel(display, x);
        } /* if */
        else
            time = pixel2time(display, x);

        {
            str := sprintf("T0=%g, T1=%g", time, time + CutBufferTime);
            vcWin.ClearStatus(win);
            vcWin.AddStatus(win, str);
        }
    } /* start */
    function moveit(x, y) {
        Old_x = New_x;
        New_x = x;
        Old_y = New_y;
        New_y = y;
    } /* move */
    function refresh() {
        RefreshWavesRigid(display, event,
                          vgPt.Create(Old_x, Old_y),
                          vgPt.Create(Old_x, Old_y),
                          vgPt.Create(CutBufferWidth, CutBufferHeight), 0);
    } /* refresh */
    function move(x, y) {
        if (snap != vtrSnapMode.Off) {
            time = pixel2time(display, x);
            time = round(time / gspace) * gspace;
            x = time2pixel(display, time);
        } /* if */
        else
            time = pixel2time(display, x);

        if (time != oldtime) {
            str := sprintf("T0=%g, T1=%g", time, time + CutBufferTime);
            vcWin.ClearStatus(win);
            vcWin.AddStatus(win, str);
            oldtime = time;
        } /* if */

        /* if changing regions, we'll just refresh twice */
        if (   GetRegionIndex(display, Old_x, Old_y)
            != GetRegionIndex(display, New_x, New_y)) {
            RefreshWavesRigid(display, event,
                              vgPt.Create(Old_x, Old_y),
                              vgPt.Create(Old_x, Old_y),
                              vgPt.Create(CutBufferWidth, CutBufferHeight), 0);
            set_height(x, y);
            RefreshWavesRigid(display, event,
                              vgPt.Create(New_x, New_y),
                              vgPt.Create(New_x, New_y),
                              vgPt.Create(CutBufferWidth, CutBufferHeight), 0);
        } /* if */
        /* in same region use semi-smart refresh */
        else {
            RefreshWavesRigid(display, event,
                              vgPt.Create(Old_x, Old_y),
                              vgPt.Create(New_x, New_y),
                              vgPt.Create(CutBufferWidth, CutBufferHeight), 1);
        } /* else */
        moveit(x, y);
    } /* move */
    function finish(x, y) {
        move(x, y);
        RefreshWavesRigid(display, event,
                          vgPt.Create(Old_x, Old_y),
                          vgPt.Create(Old_x, Old_y),
                          vgPt.Create(CutBufferWidth, CutBufferHeight), 0);
        TheList := GetRegionPlacement(display, event, New_x, New_y);
        new_event := vcEvent.Duplicate(event);
        vcEvent.SetX(new_event, x);
        vcEvent.SetY(new_event, y);
        vcCmd.Run(TheCmd, display, new_event,
                  list(car(TheList), cadr(TheList), caddr(TheList)), TRUE);
/*
        TheFunc(display, event, car(TheList), cadr(TheList), caddr(TheList));
*/
        vcWin.ClearStatus(win);
    } /* finish */
    function cancel() {
        RefreshWavesRigid(display, event,
                          vgPt.Create(Old_x, Old_y),
                          vgPt.Create(Old_x, Old_y),
                          vgPt.Create(CutBufferWidth, CutBufferHeight), 0);
        vcWin.ClearStatus(win);
    } /* cancel */

    if (type == 0 || type == 1) {
        if (type == 0) {
            Region := vcWin.Region(win);
            area = vcRegion.Area(Region);
        }
        d := vcDrag.Create(event, area,
                           vgColor.White, finish, "VScript",
                           move, refresh, nil);
        vcDrag.SetCancel(d, cancel);

        if (type == 0) {
            x := (vgRect.Xmin(area) + vgRect.Xmax(area)) * 0.5;
            y := (vgRect.Ymin(area) + vgRect.Ymax(area)) * 0.5;
            /* this doesn't work anyway so here it is in a comment
            event = vcEvent.Create(x, y,
                                   vcEvent.Ch(event),
                                   vcEvent.String(event),
                                   vcEvent.Window(event),
                                   vcEvent.Region(event),
                                   vcEvent.Rect(event));*/
            type = 1;
            start(x, y);
        } /* if */
        else if (type == 1)
            start(vcEvent.X(event), vcEvent.Y(event));

        refresh();
    } /* if */
} /* StartPlaceRegion */

function
StartPlaceText(display, event, TheFunc)
{
    area := vgRect.Create(0, 0, 0, 0);
    type := GetArea(display, event, area);

    Origin_x := 100;
    Origin_y := 100;
    Old_x := 100;
    Old_y := 100;
    New_x := 100;
    New_y := 100;

    win := vcEvent.Window(event);

    function start(x, y) {
        Origin_x = Old_x = New_x = x;
        Origin_y = Old_y = New_y = y;
    } /* start */
    function moveit(x, y) {
        Old_x = New_x;
        New_x = x;
        Old_y = New_y;
        New_y = y;
    } /* move */
    function refresh() {
        tmp_x := Old_x - Origin_x;
        tmp_y := Old_y - Origin_y;
        RefreshTextRigid(display, event,
                         vgPt.Create(tmp_x, tmp_y),
                         vgPt.Create(tmp_x, tmp_y), 0);
    } /* refresh */
    function move(x, y) {
        RefreshTextRigid(display, event,
                         vgPt.Create(Old_x - Origin_x, Old_y - Origin_y),
                         vgPt.Create(New_x - Origin_x, New_y - Origin_y),
                         1);
        moveit(x, y);
    } /* move */
    function finish(x, y) {
/*
        move(x, y);
        font_name := gtsize(display);
        string_width := gtwidth(display);
        string_height := gtheight(display, font_name);

        if (x <= vgRect.Xmin(window_area))
            x = vgRect.Xmin(window_area) + (string_width * 0.5);
        if (x >= vgRect.Xmax(window_area))
            x = vgRect.Xmax(window_area) - (string_width * 0.5);
        
        if (y <= vgRect.Ymin(window_area))
            y = vgRect.Ymin(window_area) + (string_height * 0.5);
        if (y >= vgRect.Ymax(window_area))
            y = vgRect.Ymax(window_area) - (string_height * 0.5);
*/
        move(x, y);
        TheFunc(display, event,
                vgPt.Create(Origin_x, Origin_y),
                vgPt.Create(New_x, New_y));
    } /* finish */
    function cancel() {
        /* sim move back to origin */
        RefreshTextRigid(display, event,
                         vgPt.Create(Old_x - Origin_x, Old_y - Origin_y),
                         vgPt.Create(0, 0), 1); 
    } /* cancel */

    if (type == 0 || type == 1) {
        if (type == 0) {
            Region := vcWin.Region(win);
            area = vcRegion.Area(Region);
        } /* if */

        d := vcDrag.Create(event, area,
                           vgColor.White, finish, "VScript",
                           move, refresh, nil);
        vcDrag.SetCancel(d, cancel);

        if (type == 0) {
            x := (vgRect.Xmin(area) + vgRect.Xmax(area)) * 0.5;
            y := (vgRect.Ymin(area) + vgRect.Ymax(area)) * 0.5;
            type = 1;
            start(x, y);
        } /* if */
        else if (type == 1)
            start(vcEvent.X(event), vcEvent.Y(event));

        vcWin.ClearStatus(win);
        vcWin.AddStatus(win, "Place annotation text");
    } /* if */
} /* StartPlaceText */

function
StartTimeLimit(display, event)
{
    gspace := GetMajorGridSpace(display);
    snap := get_snap_mode(display);
    win := vcEvent.Window(event);

    area := vgRect.Create(0, 0, 0, 0);
    type := GetZoomArea(display, event, area);

    if (type == 1 || type == 2) {
        vcDrag.Create(event, area, vgColor.Red,
                      func(xmin, xmax) {
                          if (type == 1)
                              HIndex := GetPointHAxisIndex(display, event);
                          else
                              HIndex := GetRegionCurrentHAxisIndex(display,
                                                                   event);
                          if (HIndex == -1)
                              HIndex := GetRegionCurrentHAxisIndex(display,
                                                                   event);
                          left := HCoord2Value(xmin, HIndex);
                          right := HCoord2Value(xmax, HIndex);
                          vcCmd.Run(EditTimeLimitCmd, display, event,
                                    list(left, right), TRUE);
                          vcCmd.Run(ViewRefreshCmd, display, event, nil, TRUE);
                      }, "HBar", vgRect.Ymin(area), vgRect.Ymax(area),
                      nil);
    } /* if */
} /* StartTimeLimit */

/* WIDTHS */

function
NamesWidth(display, event)
{
    area := vgRect.Create(0, 0, 0, 0);
    if (get_names_area(display, event, area) == 1) {
        all_area := vgRect.Create(0, 0, 0, 0);
        get_all_area(display, event, all_area);
        vgRect.Set(area, vgRect.Xmin(area), vgRect.Ymin(area),
                   vgRect.Xmax(all_area) / 2, vgRect.Ymax(area));

        e := vcEvent.Create(vgRect.Xmin(area), vgRect.Ymin(area), '\0', "",
                            vcEvent.Window(event),
                            vcEvent.Region(event), nil);
        x := vgRect.Xmax(area);
        y := vgRect.Ymin(area) + (vgRect.Ymax(area) - vgRect.Ymin(area))/2;
        vgU.MoveCursor(get_viewport(display), x, y);
        vcDrag.Create(e, area, vgColor.Red,
                      func(xmin, xmax) { 
                          if (xmax > vgRect.Xmax(area))
                              xmax = vgRect.Xmax(area);
                          disp_names_width(display, event, xmax-xmin); },
                      "HBar", vgRect.Ymin(area), vgRect.Ymax(area), nil);
    } /* if */
} /* NamesWidth */

function
ValuesWidth(display, event)
{
    area := vgRect.Create(0, 0, 0, 0);
    if (get_values_area(display, event, area) == 1) {
        e := vcEvent.Create(vgRect.Xmin(area), vgRect.Ymin(area), '\0', "",
                            vcEvent.Window(event),
                            vcEvent.Region(event), nil);
        x := vgRect.Xmin(area);
        y := vgRect.Ymin(area) + (vgRect.Ymax(area) - vgRect.Ymin(area))/2;
        vgU.MoveCursor(get_viewport(display), x, y);
        vcDrag.Create(e, area, vgColor.Red,
                      func(xmin, xmax) { 
                          if (xmax > vgRect.Xmax(area))
                              xmax = vgRect.Xmax(area); 
                          disp_values_width(display, event, xmax-xmin); },
                      "HBar", vgRect.Ymin(area), vgRect.Ymax(area), nil);
    } /* if */
} /* ValuesWidth */

function
VAxisWidth(display, event)
{
    declare e, x, y;

    area := vgRect.Create(0, 0, 0, 0);
    if (get_v_axis_area(display, event, area) == 1) {
        if (get_side(display) == vtrLeftRight.Left) {
            e = vcEvent.Create(vgRect.Xmin(area), vgRect.Ymin(area),
                               '\0', "",
                               vcEvent.Window(event),
                               vcEvent.Region(event), nil);
            x = vgRect.Xmin(area);
            y =   vgRect.Ymin(area)
                + (vgRect.Ymax(area) - vgRect.Ymin(area)) / 2;
        } /* if */
        else {
            e = vcEvent.Create(vgRect.Xmax(area), vgRect.Ymax(area),
                               '\0', "",
                               vcEvent.Window(event),
                               vcEvent.Region(event), nil);
            x = vgRect.Xmax(area);
            y =   vgRect.Ymin(area)
                + (vgRect.Ymax(area) - vgRect.Ymin(area)) / 2;
        } /* else */
        vgU.MoveCursor(get_viewport(display), x, y);
        vcDrag.Create(e, area, vgColor.Red,
                      func(xmin, xmax) { 
                          if (xmax > vgRect.Xmax(area))
                              xmax = vgRect.Xmax(area);
                          disp_v_axis_width(display, event, abs(xmax-xmin)); },
                      "HBar", vgRect.Ymin(area), vgRect.Ymax(area), nil);
    } /* if */
} /* VAxisWidth */

function
HAxisHeight(display, event)
{
    area := vgRect.Create(0, 0, 0, 0);
    if (get_h_axis_area(display, event, area) == 1) {
/*        all_area := vgRect.Create(0, 0, 0, 0);
        get_all_area(display, event, all_area);
        vgRect.Set(area, vgRect.Xmin(area), vgRect.Ymin(area),
                   vgRect.Xmax(all_area) / 2, vgRect.Ymax(area));
*/
        e := vcEvent.Create(vgRect.Xmin(area), vgRect.Ymin(area), '\0', "",
                            vcEvent.Window(event),
                            vcEvent.Region(event), nil);
        x := vgRect.Xmax(area) + (vgRect.Xmax(area) - vgRect.Xmin(area)) / 2;
        y := vgRect.Ymin(area);
        vgU.MoveCursor(get_viewport(display), x, y);
        vcDrag.Create(e, area, vgColor.Red,
                      func(ymin, ymax) { 
                          if (ymax > vgRect.Ymax(area))
                              ymax = vgRect.Ymax(area);
                          disp_h_axis_height(display, event, ymax-ymin); },
                      "VBar", vgRect.Xmin(area), vgRect.Xmax(area), nil);
    } /* if */
} /* HAxisHeight */

function
RegionBoundary(display, event)
{
    function cancel() {
        vcWin.ClearStatus(win);
        bindcmd('\middledown', SetPointWRTMouseCmd);
    }

    win := vcEvent.Window(event);
    area := vgRect.Create(0, 0, 0, 0);
    if((bound_y := get_window_area(display, event, area)) != 0) {
        x := (vgRect.Xmax(area) - vgRect.Xmin(area)) / 2;
/*        y := vgRect.Ymin(area);*/

        e := vcEvent.Create(x, bound_y, '\0', "",
                            vcEvent.Window(event), vcEvent.Region(event),
                            nil);

        vgU.MoveCursor(get_viewport(display), x, bound_y);
        d := vcDrag.Create(e, area, vgColor.Red,
                           func(ymin, ymax)
                           { 
                               disp_move_region_boundary(display, event, 
                                                         ymax);
                               vcWin.ClearStatus(win);
                               /* Get the middle button back to where it
                                  belongs */
                               bindcmd('\middledown', SetPointWRTMouseCmd);
                           },
                           "Vbar", vgRect.Xmin(area), vgRect.Xmax(area), nil);
        vcDrag.SetCancel(d, cancel);
    } /* if */
    else {
        vcWin.ClearStatus(win);
        vcWin.AddStatus(win, "No region boundary present");
        bindcmd('\middledown', SetPointWRTMouseCmd);
    } /* else */
} /* RegionBoundary */
    
function
StartRegionBoundary(display, event)
{
    win := vcEvent.Window(event);
    vcWin.ClearStatus(win);
    vcWin.AddStatus(win, "Select region boundary to move");
    bindcmd('\middledown', RegionBoundaryCmd);    
}

/*!*/
/* MISC */

load("vtmath.vs");

function
ToBeImplemented(d, e)
{
    error("To Be Implemented", "");
} /* ToBeImplemented */

ToBeImplementedCmd := vcWv.Cmd(fac, ToBeImplemented, "_ToBeImplemented");

/*!*/
/* <<<<Command Definitions>>>> */

/* File */
/* RSM - 11/25/91 Changed open and read to not prompt for file name */
FileOpenCmd := vcWv.Cmd(fac, func(d, e) open(d, e, "");, "Mopen");
FileReadCmd := vcWv.Cmd(fac, func(d, e) read(d, e, "");, "Mread");

/* Bob Davis 4/13/92
 * unfortunately the above do not ever allow command line args
 * to be passed to open and read, so we have the following incantations
 * which _do_ allow the command line args, but don't support file picking
 */
FileDoOpenCmd := vcWv.Cmd(fac, open, "Open", "open",
                          vcWv.Param("[args] filename:", vlStringRestOfLineVT,
                          STICKY, ""));
FileDoReadCmd := vcWv.Cmd(fac, read, "Read", "read",
                          vcWv.Param("[args] filename:", vlStringRestOfLineVT,
                          STICKY, ""));

function
FileDoOpenFunc(data, filename, event)
{
    vcCmd.Run(FileDoOpenCmd, data, event, list(filename), TRUE);
} /* FileDoOpenFunc */

FileFreshCmd := vcWv.Cmd(vcWin.fac, fresh, "Fresh", "fresh",
                         vcWv.Param("Stream name:", string, STICKY, ""),
                         vcWv.Param("Stream id:", string, STICKY, ""));
FileUpdateCmd := vcWv.Cmd(fac, func(d, e) update(d, e, 0);, "Update");
FileNUpdateCmd := vcWv.Cmd(fac, update, "Nupdate", "Update Every N Seconds",
                           vcWv.Param("n seconds:", integer, STICKY, ""));
FileCancelUpdateCmd := vcWv.Cmd(fac, func(d, e) update(d, e, -1);, 
                                "CancelUpdate");

function
FWriteFcn(display, event, format, filename, writefunc)
{
    declare fullname, accept_cmd, win;

    function name_and_format(d, format) {
        declare extension;

        if (format == vtrFileFormatWrite.CSDF)
            extension = ".txt";
        else if (format == vtrFileFormatWrite.PWL)
            extension = ".pwl";
        else if (format == vtrFileFormatWrite.GENERIC)
            extension = ".gen";
        else if (format == vtrFileFormatWrite.CMD)
            extension = ".cmd";
        else if (format == vtrFileFormatWrite.BAD)
            extension = ".bad";

        blkname(d) + extension;
    }

    function accept(d, e) {
        declare buf;

        if (writefunc == writeto) {
            vcWin.ClearStatus(win = vcEvent.Window(e));
            if (!null(fullname)) {
                writefunc(d, e, format, fullname);
                buf = sprintf("File \"%s\" was written.", fullname);
                vcWin.AddStatus(win, buf);
            }
            else {
                writefunc(d, e, format, filename);
                buf = sprintf("File \"%s\" was written.", filename);
                vcWin.AddStatus(win, buf);
            }
        }
        else {
            writefunc(d, e, format);
            buf = sprintf("File \"%s\" was written.", 
                          name_and_format(d, format));
            vcWin.AddStatus(win, buf);
        }

        vcMac.Resume();
    } /* accept */
    accept_cmd = vcWv.Cmd(fac, accept);

    function reject() {
        vcWin.ClearStatus(win);
        if (writefunc == write)
            vcWin.AddStatus(win, "File Write command cancelled.");
        else
            vcWin.AddStatus(win, "File Writeto command cancelled.");
        
        vcMac.Resume();
        } /* reject */

    if (PROMPT_OVERWRITES == YES) {
        declare extension;

        vcMac.Suspend();

        win = vcEvent.Window(event);
        if (string_eq(filename, ""))
            filename = blkname(display);

        if (format == vtrFileFormatWrite.CSDF)
            extension = "txt";
        else if (format == vtrFileFormatWrite.PWL)
            extension = "pwl";
        else if (format == vtrFileFormatWrite.GENERIC)
            extension = "gen";
        else if (format == vtrFileFormatWrite.CMD)
            extension = "cmd";
        else if (format == vtrFileFormatWrite.BAD)
            extension = "bad";

        parsed_fname := vscript.vsParse(filename);

        filename = vscript.vsMerge(car(parsed_fname),
                                   cadr(parsed_fname),
                                   caddr(parsed_fname),
                                   extension);

        if (!null(fullname = vscript.vsGetini(filename))) {
            declare buttons, query_t, str, text;
            
            buttons = vcRegion.Create("Buttons", vcButnRow.fac,
                                      list (list ("no", 
                                                  "Don't overwrite",
                                                  reject, 
                                                  list (close_window <- 
                                                        TRUE)),
                                            list ("yes", 
                                                  "Overwrite the file",
                                                  accept_cmd, 
                                                  list (close_window <- 
                                                        TRUE))));
            
            query_t = vcTable.Create ("Overwrite?", 2, 1);
            
            if (!null(fullname))
                str = sprintf ("File %s already exists, overwrite?",
                               fullname);
            else
                str = sprintf ("File %s already exists, overwrite?",
                               filename);
                
            text = vcRegion.Create ("over_text", vcStaticText.fac, str);
            vcTable.Add (query_t, text);
            vcTable.Add (query_t, buttons);
            
            vcWin.Popup (vcEvent.Window (event), query_t,
                         vcButnRow.GetDefault (buttons),
                         vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
        } /* if */
        else
            accept(display, event);
    } /* if */
    else
        accept(display, event);

} /* FWriteFcn */

function 
FileWriteFcn(display, event, format)
{
    FWriteFcn(display, event, format, "", write);
} /* FileWriteFcn */

FileWriteCmd := vcWv.Cmd(fac, FileWriteFcn, "Write", "Write",
                         vcWv.Param("Format", vtrFileFormatWriteVT,
                                    STICKY, vtrFileFormatWrite.BAD));

function 
FileWritetoFcn(display, event, format, filename)
{
    FWriteFcn(display, event, format, filename, writeto);
} /* FileWritetoFcn */

FileWriteToCmd := vcWv.Cmd(fac, FileWritetoFcn, "WriteTo", "WriteTo",
                           vcWv.Param("Format", vtrFileFormatWriteVT,
                                      STICKY, vtrFileFormatWrite.BAD),
                           vcWv.Param("File Name:", string, STICKY, ""));

FileReadStreamCmd := vcWv.Cmd(fac,
                              func(d, e, name) read(d, e,
                                                    "-interactive " + name);,
                              "ReadStream", "ReadStream",
                              vcWv.Param("Stream Name:", string, STICKY, ""));

FileRereadCfgCmd := vcWv.Cmd(fac, 
                             func(d, e) {
                                 RereadCfg(d, e);
                                 refresh(d, e, YES);
                             }, "RereadCfg");



/* Edit */

/** @@@@ **/

/*!*/
/* Selection */

function
StartSelect(display, event)
{
    MakeSelect(display, event, Select, 0);
} /* StartSelect */

function
StartAddSelect(display, event)
{
    MakeSelect(display, event, AddSelect, 0);
} /* StartAddSelect */

function
StartUnSelect(display, event)
{
    MakeSelect(display, event, UnSelect, 0);
} /* StartUnSelect */

function
StartSelectText(display, event)
{
    MakeSelect(display, event, Select, 6);
} /* StartSelectText */

function
StartAddSelectText(display, event)
{
    MakeSelect(display, event, AddSelect, 6);
} /* StartAddSelectText */

function
StartUnSelectText(display, event)
{
    MakeSelect(display, event, UnSelect, 6);
} /* StartUnSelectText */

function
MakeSelect(display, event, select_op, type)
{
    Origin_x := 0; Origin_y := 0;
    Old_x := 0; Old_y := 0;
    New_x := 0; New_y := 0;
    area := vgRect.Create(0, 0, 0, 0);

    /* If the type passed into this function is zero, that means the type
     * should be determined from the location of the event.
     * If the type is non-zero, it will be 6, which is for text(annotations).
     */
    {
        this_type := GetArea(display, event, area);
        if (type == 0)
            type = this_type;
    }

    function move(x, y) {
        Old_x = New_x;
        New_x = x;
        Old_y = New_y;
        New_y = y;
    } /* move */
    function start() {
        Origin_x = Old_x = New_x = vcEvent.X(event);
        Origin_y = Old_y = New_y = vcEvent.Y(event);
    } /* start */

    if (type == 1) {
        if (RegionIsDigital(display, event) == 1)
            digi_waves_select(display, event, select_op);
        else
            vcDrag.Create(event, area, vgColor.White,
                          func(new_area)
                          { select_op(display, event, new_area, type); },
                          "Box",
                          vgRect.Xmax(area) - vgRect.Xmin(area),
                          vgRect.Ymax(area) - vgRect.Ymin(area), nil);
    } /* if */
    else if (type == 2) {
        function refresh_h_axes() {
            vgRect.Set(area, Origin_x, Origin_y, New_x, New_y);
            TheList := Area2HAxesList(display, event, area);
            RefreshHAxesRubber(display, event, TheList);
        } /* refresh_h_axes */
        function move_h_axes(x, y) {
            /* if a change, brute force it */
            if (   GetHAxisIndex(display, event, Old_x, Old_y)
                != GetHAxisIndex(display, event, New_x, New_y)) {
                vgRect.Set(area, Origin_x, Origin_y, Old_x, Old_y);
                TheList := Area2HAxesList(display, event, area);
                RefreshHAxesRubber(display, event, TheList);
                vgRect.Set(area, Origin_x, Origin_y, New_x, New_y);
                TheList := Area2HAxesList(display, event, area);
                RefreshHAxesRubber(display, event, TheList);
            } /* if */
            move(x, y);
        } /* move_h_axes */

        function finish_h_axes(x, y) {
            move_h_axes(x, y);
            refresh_h_axes();
            vgRect.Set(area, Origin_x, Origin_y, New_x, New_y);
            select_op(display, event, area, type);
        } /* finish_h_axis */

        d := vcDrag.Create(event, area, vgColor.Blue,
                           finish_h_axes, "VScript",
                           move_h_axes, refresh_h_axes, nil);

        start();
        refresh_h_axes();
    } /* else if */
    else if (type == 3) {
        function refresh_v_axes() {
            vgRect.Set(area, Origin_x, Origin_y, New_x, New_y);
            TheList := Area2VAxesList(display, event, area);
            RefreshVAxesRubber(display, event, TheList);
        } /* refresh_v_axes */
        function move_v_axes(x, y) {
            /* if a change, brute force it */
            if (   GetVAxisIndex(display, event, Old_x, Old_y)
                != GetVAxisIndex(display, event, New_x, New_y)) {
                vgRect.Set(area, Origin_x, Origin_y, Old_x, Old_y);
                TheList := Area2VAxesList(display, event, area);
                RefreshVAxesRubber(display, event, TheList);
                vgRect.Set(area, Origin_x, Origin_y, New_x, New_y);
                TheList := Area2VAxesList(display, event, area);
                RefreshVAxesRubber(display, event, TheList);
            } /* if */
            move(x, y);
        } /* move_v_axes */

        function finish_v_axes(x, y) {
            move_v_axes(x, y);
            refresh_v_axes();
            vgRect.Set(area, Origin_x, Origin_y, New_x, New_y);
            select_op(display, event, area, type);
        } /* finish_v_axis */

        d := vcDrag.Create(event, area, vgColor.Blue,
                           finish_v_axes, "VScript",
                           move_v_axes, refresh_v_axes, nil);

        start();
        refresh_v_axes();
    } /* else if */
    else if (   type == 4
             || type == 5) {    /* status (names or values) */
        function refresh_status() {
            RefreshStatusRubber(display, event, Origin_y, Old_y, New_y, 0);
        } /* refresh_status */
        function move_status(x, y) {
            RefreshStatusRubber(display, event, Origin_y, Old_y, New_y, 1);
            move(x, y);
        } /* move_status */
        function finish_status(x, y) {
            move_status(x, y);
            RefreshStatusRubber(display, event, Origin_y, Old_y, New_y, 0);
            new_area := vgRect.Create(Origin_x, Origin_y, New_x, New_y);
            select_op(display, event, new_area, type);
        } /* finish_status */
        function cancel_status() {
            RefreshStatusRubber(display, event, Origin_y, Old_y, New_y, 0);
        } /* cancel_status */

        d := vcDrag.Create(event, area,
                           vgColor.White, finish_status, "VScript",
                           move_status, refresh_status, nil);
        vcDrag.SetCancel(d, cancel_status);

        start();
        refresh_status();
    } /* else if */
    else if (type == 6) {
        vcDrag.Create(event, area, vgColor.White,
                      func(new_area)
                      { select_op(display, event, new_area, type); },
                      "Box",
                      vgRect.Xmax(area) - vgRect.Xmin(area),
                      vgRect.Ymax(area) - vgRect.Ymin(area), nil);
    } /* else if */
} /* MakeSelect */

function
digi_waves_select(display, event, select_op)
{
    Origin_x := 0; Origin_y := 0;
    Old_x := 0; Old_y := 0;
    New_x := 0; New_y := 0;
    area := vgRect.Create(0, 0, 0, 0);
    type := GetArea(display, event, area);

    declare time0, time, oldtime;
    gspace := GetMajorGridSpace(display);
    snap := get_snap_mode(display);
    win := vcEvent.Window(event);
    
    function start() {
        Origin_x = Old_x = New_x = vcEvent.X(event);
        Origin_y = Old_y = New_y = vcEvent.Y(event);
    } /* start */
    function move(x, y) {
        if (x >= vgRect.Xmin(area) && x <= vgRect.Xmax(area)) {
            Old_x = New_x;
            New_x = x;
        }
        if (y >= vgRect.Ymin(area) && y <= vgRect.Ymax(area)) {
            Old_y = New_y;
            New_y = y;
        }
    } /* move */
    function refresh_waves() {
        RefreshWavesRubber(display, event,
                           vgPt.Create(Origin_x, Origin_y),
                           vgPt.Create(Old_x, Old_y),
                           vgPt.Create(New_x, New_y), 0);
    } /* refresh_waves */
    function move_waves(x, y) {
        if (snap != vtrSnapMode.Off) {
            time = pixel2time(display, x);
            time = round(time / gspace) * gspace;
            x = time2pixel(display, time);
        } /* if */
        else
            time = pixel2time(display, x);

        if (time != oldtime) {
            str := sprintf("T0=%g, T1=%g", time0, time);
            vcWin.ClearStatus(win);
            vcWin.AddStatus(win, str);
            oldtime = time;
        } /* if */
        RefreshWavesRubber(display, event,
                           vgPt.Create(Origin_x, Origin_y),
                           vgPt.Create(Old_x, Old_y),
                           vgPt.Create(New_x, New_y), 1);
        move(x, y);
    } /* move_waves */
    function finish_waves(x, y) {
        move_waves(x, y);
        RefreshWavesRubber(display, event,
                           vgPt.Create(Origin_x, Origin_y),
                           vgPt.Create(Old_x, Old_y),
                           vgPt.Create(New_x, New_y), 0);
        new_area := vgRect.Create(Origin_x, Origin_y, New_x, New_y);
        select_op(display, event, new_area, type);
/*        vcWin.ClearStatus(win); */
    } /* finish_waves */
    function cancel_waves() {
        RefreshWavesRubber(display, event,
                           vgPt.Create(Origin_x, Origin_y),
                           vgPt.Create(Old_x, Old_y),
                           vgPt.Create(New_x, New_y), 0);
        vcWin.ClearStatus(win);
    } /* cancel_waves */

    if (snap != vtrSnapMode.Off) {
        x := vcEvent.X(event);
        HIndex := GetRegionCurrentHAxisIndex(display, event);
        HAVMax := HAxisVMax(display, HIndex);
        time0 = pixel2time(display, x);
        time0 = round(time0 / gspace) * gspace;
        if (time0 < HAVMax) {
            x = time2pixel(display, time0);
            vcEvent.SetX(event, x);
        } /* if */
        else {
/*            time0 = pixel2time(display, vcEvent.X(event)); */
            time0 = HAVMax;
            vcEvent.SetX(event, time2pixel(display, time0));
        } /* else */
    } /* if */
    else
        time0 = pixel2time(display, vcEvent.X(event));

    {
        str := sprintf("T0=%g, T1=%g", time0, time0);
        vcWin.ClearStatus(win);
        vcWin.AddStatus(win, str);
    }

    if (type == 1) {
        vcCmd.Run(UnSelectSignalRegionCmd, display, event, nil, TRUE);
        d := vcDrag.Create(event, area, vgColor.White,
                           finish_waves, "VScript", move_waves, refresh_waves,
                           nil);
        vcDrag.SetCancel(d, cancel_waves);

        start();
        refresh_waves();
    } /* if */
} /* digi_waves_select */

AddSelectSignalIndexCmd := vcWv.Cmd(fac, AddSelectSignalIndex,
                                    "AddSelectSignalIndex", "",
                                    vcWv.Param("Index:", integer,
                                               STICKY, 0));
vcCmd.EnableMacro(AddSelectSignalIndexCmd, vlOnOff.On);
UnSelectSignalIndexCmd := vcWv.Cmd(fac, UnSelectSignalIndex,
                                   "UnSelectSignalIndex", "",
                                   vcWv.Param("Index:", integer,
                                              STICKY, 0));
UnSelectAllSignalsCmd := vcWv.Cmd(fac, UnSelectAllSignals,
                                  "UnSelectAllSignals");
UnSelectSignalRegionCmd := vcWv.Cmd(fac, UnSelectSignalRegion,
                                    "_UnSelectSignalRegionCmd");

AddSelectHAxisIndexCmd := vcWv.Cmd(fac, AddSelectHAxisIndex,
                                   "AddSelectHAxisIndex", "",
                                   vcWv.Param("Index:", integer,
                                              STICKY, 0));
UnSelectHAxisIndexCmd := vcWv.Cmd(fac, UnSelectHAxisIndex,
                                  "UnSelectHAxisIndex", "",
                                  vcWv.Param("Index:", integer,
                                             STICKY, 0));
UnSelectAllHAxesCmd := vcWv.Cmd(fac, UnSelectAllHAxes,
                                "UnSelectAllHAxes");

AddSelectVAxisIndexCmd := vcWv.Cmd(fac, AddSelectVAxisIndex,
                                   "AddSelectVAxisIndex", "",
                                   vcWv.Param("Index:", integer,
                                              STICKY, 0));
UnSelectVAxisIndexCmd := vcWv.Cmd(fac, UnSelectVAxisIndex,
                                  "UnSelectVAxisIndex", "",
                                  vcWv.Param("Index:", integer,
                                             STICKY, 0));
UnSelectAllVAxesCmd := vcWv.Cmd(fac, UnSelectAllVAxes,
                                "UnSelectAllVAxes");

SelectSignalRegionCmd := vcWv.Cmd(fac, SelectSignalRegion,
                                  "SelectSignalRegion", "SelectSignalRegion",
                                  vcWv.Param("first:", integer, STICKY, 0),
                                  vcWv.Param("last:", integer, STICKY, 0),
                                  vcWv.Param("ltime:", float, STICKY, 0.0),
                                  vcWv.Param("rtime:", float, STICKY, 1.0));

AddSelectTextIndexCmd := vcWv.Cmd(fac, AddSelectTextIndex,
                                  "AddSelectTextIndex", "",
                                  vcWv.Param("Index:", integer,
                                             STICKY, 0));
UnSelectTextIndexCmd := vcWv.Cmd(fac, UnSelectTextIndex,
                                 "UnSelectTextIndex", "",
                                 vcWv.Param("Index:", integer,
                                            STICKY, 0));
UnSelectAllTextCmd := vcWv.Cmd(fac, UnSelectAllText, "UnSelectAllText");

function
Select(display, event, area, type)
{
    if (type == 1) {            /* waves */
        vcCmd.Run(UnSelectAllSignalsCmd, display, event, nil, TRUE);
        if (RegionIsDigital(display, event) == 1) {
            TheList := StatusArea2SignalList(display, event, area);
            ProcessSelectList(display, event, TheList,
                              AddSelectSignalIndexCmd);
            ArgList := GetSelectSignalRegion(display, event, area);
            vcCmd.Run(SelectSignalRegionCmd, display, event,
                      ArgList, TRUE);
        } /* if */
        else {
            TheList := WavesArea2SignalList(display, event, area);
            ProcessSelectList(display, event, TheList,
                              AddSelectSignalIndexCmd);
        } /* else */
    } /* if */
    else if (type == 2) {       /* h axes */
        vcCmd.Run(UnSelectAllHAxesCmd, display, event, nil, TRUE);
        TheList := Area2HAxesList(display, event, area);
        ProcessSelectList(display, event, TheList, AddSelectHAxisIndexCmd);
    } /* else if */
    else if (type == 3) {       /* v axes */
        vcCmd.Run(UnSelectAllVAxesCmd, display, event, nil, TRUE);
        TheList := Area2VAxesList(display, event, area);
        ProcessSelectList(display, event, TheList, AddSelectVAxisIndexCmd);
    } /* else if */
    else if (   type == 4
             || type == 5) {    /* status (names or values) */
        vcCmd.Run(UnSelectAllSignalsCmd, display, event, nil, TRUE);
        TheList := StatusArea2SignalList(display, event, area);
        ProcessSelectList(display, event, TheList, AddSelectSignalIndexCmd);
    } /* else if */
    else if (type == 6) {       /* annotation text */
        vcCmd.Run(UnSelectAllTextCmd, display, event, nil, TRUE);
        TheList := Area2TextList(display, event, area);
        ProcessSelectList(display, event, TheList, AddSelectTextIndexCmd);
    } /* else if */
} /* Select */

function
AddSelect(display, event, area, type)
{
    if (type == 1) {            /* waves */
        if (RegionIsDigital(display, event) == 1) {
            TheList := StatusArea2SignalList(display, event, area);
            ProcessSelectList(display, event, TheList,
                              AddSelectSignalIndexCmd);
            ArgList := GetSelectSignalRegion(display, event, area);
            vcCmd.Run(SelectSignalRegionCmd, display, event,
                      ArgList, TRUE);
        } /* if */
        else {
            TheList := WavesArea2SignalList(display, event, area);
            ProcessSelectList(display, event, TheList,
                              AddSelectSignalIndexCmd);
        } /* else */
    } /* if */
    else if (type == 2) {       /* h axes */
        TheList := Area2HAxesList(display, event, area);
        ProcessSelectList(display, event, TheList, AddSelectHAxisIndexCmd);
    } /* else if */
    else if (type == 3) {       /* v axes */
        TheList := Area2VAxesList(display, event, area);
        ProcessSelectList(display, event, TheList, AddSelectVAxisIndexCmd);
    } /* else if */
    else if (   type == 4
             || type == 5) {    /* status (names or values) */
        TheList := StatusArea2SignalList(display, event, area);
        ProcessSelectList(display, event, TheList, AddSelectSignalIndexCmd);
    } /* else if */
    else if (type == 6) {       /* annotation text */
        TheList := Area2TextList(display, event, area);
        ProcessSelectList(display, event, TheList, AddSelectTextIndexCmd);
    } /* else if */
} /* AddSelect */

function
UnSelect(display, event, area, type)
{
    if (type == 1) {            /* waves */
        if (RegionIsDigital(display, event) == 1) {
            TheList := StatusArea2SignalList(display, event, area);
            ProcessSelectList(display, event, TheList,
                              UnSelectSignalIndexCmd);
        } /* if */
        else {
            TheList := WavesArea2SignalList(display, event, area);
            ProcessSelectList(display, event, TheList,
                              UnSelectSignalIndexCmd);
        } /* else */
    } /* if */
    else if (type == 2) {       /* h axes */
        TheList := Area2HAxesList(display, event, area);
        ProcessSelectList(display, event, TheList, UnSelectHAxisIndexCmd);
    } /* else if */
    else if (type == 3) {       /* v axes */
        TheList := Area2VAxesList(display, event, area);
        ProcessSelectList(display, event, TheList, UnSelectVAxisIndexCmd);
    } /* else if */
    else if (   type == 4
             || type == 5) {    /* status (names or values) */
        TheList := StatusArea2SignalList(display, event, area);
        ProcessSelectList(display, event, TheList, UnSelectSignalIndexCmd);
    } /* else if */
    else if (type == 6) {       /* annotation text */
        TheList := Area2TextList(display, event, area);
        ProcessSelectList(display, event, TheList, UnSelectTextIndexCmd);
    } /* else if */
} /* UnSelect */

function
ProcessSelectList(display, event, TheList, SelectCmd)
{
    declare index;
    while (TheList != nil) {
        index = car(TheList);
        vcCmd.Run(SelectCmd, display, event, list(index), TRUE);
        if (NAV_SELECT == YES)
            if (SelectCmd == AddSelectSignalIndexCmd ||
                SelectCmd == UnSelectSignalIndexCmd)
                SendNavCommand(display, event, SelectCmd, TheList);
        TheList = cdr(TheList);
    } /* while */
    if (NAV_SELECT == YES)
        if (SelectCmd == AddSelectSignalIndexCmd ||
            SelectCmd == UnSelectSignalIndexCmd)
            SendNavCommand(display, event, SelectCmd, nil);
} /* ProcessSelectList */

/** @@@@ **/

SelectDrawSignalsCmd := vcWv.Cmd(fac, SelectDrawSignals,
                                 "SelectDrawSignals", "Select Draw Signals",
                                 vcWv.Param("Signals to select", string,
                                            STICKY, "*"));

function
FromSchematic(display, event, what)
{
    vcWin.WithBusy(vcEvent.Window(event), func() {
        Stuff := GetItcSelect();
        win := vcEvent.Window(event);

        /* maybe should raise exception or something */
        if (string_eq(Stuff, "")) {
            vcWin.ClearStatus(win);
            vcWin.AddStatus(win, "Nothing selected in ViewDraw.");
            nil;
        } /* if */
        else if (string_eq(Stuff, "%NOACK%VDRAW")) {
            vcWin.ClearStatus(win);
            error2("vtrace", vlErrSev.Error, 0, nil, 
                   "ViewDraw is not running.");
            nil;
        } /* if */
        else
            if (what == vtrFromSchWhat.Select)
                vcCmd.Run(SelectDrawSignalsCmd, display, event, list(Stuff), 
                          TRUE);
            else if (what == vtrFromSchWhat.Show)
                vcCmd.Run(ShowDrawSignalsCmd, display, event, list(Stuff), 
                          TRUE);
            else if (what == vtrFromSchWhat.Create)
                vcCmd.Run(CreateDrawSignalsCmd, display, event, list(Stuff), 
                          TRUE);
    }); /* func */
} /* FromSchematic */

FromSchematicCmd := vcWv.Cmd(fac, FromSchematic,
                             "FromSchematic", "FromSchematic",
                             vcWv.Param("Do What", vtrFromSchWhatVT,
                                        STICKY, vtrFromSchWhat.Select));
vcCmd.EnableApply(FromSchematicCmd, vlOnOff.On);
vcCmd.EnableMacro(FromSchematicCmd, vlOnOff.Off);

StartSelectCmd := vcWv.Cmd(fac, StartSelect, "Select");
vcCmd.EnableMacro(StartSelectCmd, vlOnOff.Off);

StartAddSelectCmd := vcWv.Cmd(fac, StartAddSelect, "AddSelect");
vcCmd.EnableMacro(StartAddSelectCmd, vlOnOff.Off);

StartUnSelectCmd := vcWv.Cmd(fac, StartUnSelect, "UnSelect");
vcCmd.EnableMacro(StartUnSelectCmd, vlOnOff.Off);

/*SelectTextWRTMouseCmd := vcWv.Cmd(fac, tSelectClick, "TSelMouse");*/
UnselectTextWRTMouseCmd := vcWv.Cmd(fac, tunselall, "UnselAllText");

StartSelectTextCmd := vcWv.Cmd(fac, StartSelectText, "SelectText");
vcCmd.EnableMacro(StartSelectTextCmd, vlOnOff.Off);

StartAddSelectTextCmd := vcWv.Cmd(fac, StartAddSelectText, "AddSelectText");
vcCmd.EnableMacro(StartAddSelectTextCmd, vlOnOff.Off);

StartUnSelectTextCmd := vcWv.Cmd(fac, StartUnSelectText, "UnSelectText");
vcCmd.EnableMacro(StartUnSelectTextCmd, vlOnOff.Off);

/* selection using strings */

SelectSignalCmd := vcWv.Cmd(fac, SelectSignals,
                            "SelectSignals", "Select Signals",
                            vcWv.Param("Signals to select", 
                                       vlStringRestOfLineVT,
                                       STICKY, "*"));
vcCmd.EnableApply(SelectSignalCmd, vlOnOff.On);

UnSelectSignalCmd := vcWv.Cmd(fac, UnSelectSignals,
                              "UnSelectSignals", "UnSelect Signals",
                              vcWv.Param("Signals to Unselect", 
                                         vlStringRestOfLineVT,
                                         STICKY, "*"));
vcCmd.EnableApply(UnSelectSignalCmd, vlOnOff.On);

UnSelectAllCmd := vcWv.Cmd(fac, UnSelectAll, "UnSelectAll");

SelectTextCmd := vcWv.Cmd(fac, tselect,
                          "TSelect", "Select Text",
                          vcWv.Param("Text to select", vlStringRestOfLineVT,
                                     STICKY, "*"));
vcCmd.EnableApply(SelectTextCmd, vlOnOff.On);

UnSelectTextCmd := vcWv.Cmd(fac, tunselect,
                            "TUnselect", "UnSelect Text",
                            vcWv.Param("Text to unselect", 
                                       vlStringRestOfLineVT,
                                       STICKY, "*"));
vcCmd.EnableApply(UnSelectTextCmd, vlOnOff.On);

/* Editing */
EditCreateSignalCmd := vcWv.Cmd(fac, CreateSignal,
                                "CreateSignal", "Create Signal",
                                vcWv.Param("Name", string, CONSTANT, ""),
                                vcWv.Param("Number of Bits", vtrNBitsVT,
                                           STICKY, 1),
                                vcWv.Param("Radix", vtrRadixVT, STICKY,
                                           vtrRadix.Binary));
vcCmd.EnableApply(EditCreateSignalCmd, vlOnOff.On);

function
InsertTimeFcn(display, event)
{
    RegionList := GetSelectedSignalRegion(display);
    InsertTime(display, event,
               car(RegionList), cadr(RegionList), caddr(RegionList),
               cadddr(RegionList), cadddr(cdr(RegionList)));
} /* InsertTimeFcn */
EditInsertTimeCmd := vcWv.Cmd(fac, InsertTimeFcn,
                              "InsertTime", "Insert Time");

function
ClearFcn(display, event)
{
    RegionList := GetSelectedSignalRegion(display);
    ClearRegion(display, event,
                car(RegionList), cadr(RegionList), caddr(RegionList),
                cadddr(RegionList), cadddr(cdr(RegionList)));
} /* ClearFcn */
EditClearCmd := vcWv.Cmd(fac, ClearFcn,
                         "RClear", "Region Clear");

function
SetValueFcn(display, event, BitValue, BusValue)
{
    RegionList := GetSelectedSignalRegion(display);
    SetValue(display,
             car(RegionList), cadr(RegionList), caddr(RegionList),
             cadddr(RegionList), cadddr(cdr(RegionList)),
             BitValue, BusValue);
} /* SetValueFcn */
EditSetValueCmd := vcWv.Cmd(fac, SetValueFcn, "SetValue", "Set Value",
                            vcWv.Param("Bit Value", vtrBitVT, STICKY, "0"),
                            vcWv.Param("Bus Value", string, STICKY, "0"));
vcCmd.EnableApply(EditSetValueCmd, vlOnOff.On);

function
DeleteFcn(display, event)
{
    RegionList := GetSelectedSignalRegion(display);
    DeleteRegion(display, event,
                 car(RegionList), cadr(RegionList), caddr(RegionList),
                 cadddr(RegionList), cadddr(cdr(RegionList)));
} /* DeleteFcn */
EditDeleteCmd := vcWv.Cmd(fac, DeleteFcn,
                          "RDelete", "Delete");

function
CutFcn(display, event)
{
    RegionList := GetSelectedSignalRegion(display);
    CutRegion(display, event,
              car(RegionList), cadr(RegionList), caddr(RegionList),
              cadddr(RegionList), cadddr(cdr(RegionList)));
} /* CutFcn */
EditCutCmd := vcWv.Cmd(fac, CutFcn,
                       "RCut", "Cut");

function
CopyFcn(display, event)
{
    RegionList := GetSelectedSignalRegion(display);
    CopyRegion(display, event,
               car(RegionList), cadr(RegionList), caddr(RegionList),
               cadddr(RegionList), cadddr(cdr(RegionList)));
} /* CopyFcn */
EditCopyCmd := vcWv.Cmd(fac, CopyFcn,
                        "RCopy", "Copy");

function
ReplaceFcn(display, event, BitValue, BusValue)
{
    RegionList := GetSelectedSignalRegion(display);
    ReplaceRegion(display,
                  car(RegionList), cadr(RegionList),
                  caddr(RegionList), cadddr(RegionList),
                  BitValue, BusValue);
} /* ReplaceFcn */
EditReplaceCmd := vcWv.Cmd(fac, ReplaceFcn,
                           "Replace", "Replace",
                           vcWv.Param("Bit Value", vtrBitVT, STICKY, "0"),
                           vcWv.Param("Bus Value", string, STICKY, "0"));
vcCmd.EnableApply(EditReplaceCmd, vlOnOff.On);

function
RepeatFcn(display, event, count)
{
    vcWin.WithBusy(vcEvent.Window(event), func() {
        RegionList := GetSelectedSignalRegion(display);
        RepeatRegion(display, event,
                     car(RegionList), cadr(RegionList), caddr(RegionList),
                     cadddr(RegionList), cadddr(cdr(RegionList)),
                     count);
    }); /* func */
} /* RepeatFcn */
EditRepeatCmd := vcWv.Cmd(fac, RepeatFcn,
                          "Repeat", "Repeat",
                          vcWv.Param("Count", integer, STICKY, 1));

PasteRegionInternalCmd := vcWv.Cmd(fac,
                                   func(d,e,r,f,l) PasteRegion(d,e,r,f,l);,
                                   "_PasteRegionInternal", 
                                   "_PasteRegionInternal",
                                   vcWv.Param("Region", integer, STICKY, 0),
                                   vcWv.Param("First", integer, STICKY, 0),
                                   vcWv.Param("Ltime", float, STICKY, 0));
EditPasteCmd := vcWv.Cmd(fac,
                         func(d, e) StartPlaceRegion(d, e, 
                                                     PasteRegionInternalCmd);,
                         "RPaste", "Paste");
vcCmd.EnableMacro(EditPasteCmd, vlOnOff.Off);

SubstRegionInternalCmd := vcWv.Cmd(fac,
                                   func(d,e,r,f,l) SubstRegion(d,e,r,f,l);,
                                   "_SubstRegionInternal", 
                                   "_SubstRegionInternal",
                                   vcWv.Param("Region", integer, STICKY, 0),
                                   vcWv.Param("First", integer, STICKY, 0),
                                   vcWv.Param("Ltime", float, STICKY, 0));
EditSubstCmd := vcWv.Cmd(fac,
                         func(d, e) StartPlaceRegion(d, e, 
                                                     SubstRegionInternalCmd);,
                         "RSubstitute", "Substitute");
vcCmd.EnableMacro(EditSubstCmd, vlOnOff.Off);

EditTimeLimitCmd := vcWv.Cmd(fac, TimeLimit, "TLimit", "Time Limit",
                             vcWv.Param("Begin", float, STICKY, 0),
                             vcWv.Param("End", float, STICKY, 100e-9));

EditTimeStepCmd := vcWv.Cmd(fac, SetTimeStep, "TimeStep", "TimeStep",
                            vcWv.Param("Time Step", float,
                                       DYNAMIC, GetTimeStep));

/* View */

ViewZoomCmd := vcWv.Cmd(fac,
                        func(d, e) { StartZoom(d, e);
                                     bindcmd('\middledown',
                                             SetPointWRTMouseCmd); },
                        "Zoom");
vcCmd.EnableMacro(ViewZoomCmd, vlOnOff.Off);

ViewPanCmd := vcWv.Cmd(fac,
                       func(d, e) { pan(d, e, "");
                                    bindcmd('\middledown',
                                            SetPointWRTMouseCmd); },
                       "Pan" );
/*vcCmd.EnableMacro(ViewPanCmd, vlOnOff.Off);*/

ViewFullCmd := vcWv.Cmd(fac, func(d, e) full(d, e, NO);, "Full" );
ViewInCmd := vcWv.Cmd(fac, func(d, e) factor(d, e, 1.25, NO);, "In");
ViewOutCmd := vcWv.Cmd(fac, func(d, e) factor(d, e, 0.80, NO);, "Out");
ViewInFactorCmd := vcWv.Cmd(fac,
                            func(d, e, f) {
                                factor(d, e, f, NO);
                            }, "InFactor", "View In",
                            vcWv.Param("Number:",
                                       vlPositiveFloatVT, STICKY, 1.25));
ViewOutFactorCmd := vcWv.Cmd(fac,
                             func(d, e, f) {
                                 factor(d, e, 1.0/f, NO);
                             }, "OutFactor", "View Out",
                             vcWv.Param("Number:",
                                        vlPositiveFloatVT, STICKY, 1.25));
ViewRefreshCmd := vcWv.Cmd(fac, func(d, e) refresh(d, e, YES);,
                           "Refresh");
ViewScrollCmd := vcWv.Cmd(fac,
                          func(d, e) { scroll(d, e, "");
                                       bindcmd('\middledown',
                                               SetPointWRTMouseCmd); },
                          "Scroll");

{ /* limit scope of this nasty shared variable... */
    declare AxisIndex;

/* this is log10(x) = logE(x)/logE(10) */
/* no longer needed; added as viewscript instrinsic fcn */
/* function log10(f) log(f) * 0.43429448190325176116; */

/* this is exp10(x) = exp(x * ln(10) */
/* no longer needed; added as viewscript instrinsic fcn */
/* function exp10(f) exp(f * 2.30258509299404568402); */

function
GetHAxisVMin(display)
{
    type := HAxisType(display, AxisIndex);
    value := HAxisVMin(display, AxisIndex);
    if (type == vtrDispType.Log && value > 0.0)
        value = exp10(value);
    value;
}

function
GetHAxisVMax(display)
{
    type := HAxisType(display, AxisIndex);
    value := HAxisVMax(display, AxisIndex);
    if (type == vtrDispType.Log && value > 0.0)
        value = exp10(value);
    value;
}

function
ZoomAnHAxis(display, event, vmin, vmax)
{
    type := HAxisType(display, AxisIndex);
    if (   type == vtrDispType.Log
        && (vmin > 0.0 && vmax > 0.0)) {
        vmin = log10(vmin);
        vmax = log10(vmax);
    } /* if */

    vcCmd.Run(ZoomHAxisCmd, display, event,
              list(AxisIndex, vmin, vmax),
              TRUE);
    vcCmd.Run(ViewRefreshCmd, display, event, nil, TRUE);
}

function
GetVAxisVMin(display)
{
    type := VAxisType(display, AxisIndex);
    value := VAxisVMin(display, AxisIndex);
    if (type == vtrDispType.Log && value > 0.0)
        value = exp10(value);
    value;
}

function
GetVAxisVMax(display)
{
    type := VAxisType(display, AxisIndex);
    value := VAxisVMax(display, AxisIndex);
    if (type == vtrDispType.Log && value > 0.0)
        value = exp10(value);
    value;
}

function
ZoomAVAxis(display, event, vmin, vmax)
{
    type := VAxisType(display, AxisIndex);
    if (   type == vtrDispType.Log
        && (vmin > 0.0 && vmax > 0.0)) {
        vmin = log10(vmin);
        vmax = log10(vmax);
    } /* if */

    vcCmd.Run(ZoomVAxisCmd, display, event,
              list(AxisIndex, vmin, vmax),
              TRUE);
    vcCmd.Run(ViewRefreshCmd, display, event, nil, TRUE);
}
} /* ...limit scope of this nasty shared variable */

ZoomAnHAxisCmd := vcWv.Cmd(fac, ZoomAnHAxis, "ZoomAnHAxis", "ZoomAnHAxis",
                           vcWv.Param("From:", float, DYNAMIC, GetHAxisVMin),
                           vcWv.Param("To:", float, DYNAMIC, GetHAxisVMax));
vcCmd.EnableMacro(ZoomAnHAxisCmd, vlOnOff.Off);

ZoomAVAxisCmd := vcWv.Cmd(fac, ZoomAVAxis, "ZoomAVAxis", "ZoomAVAxis",
                          vcWv.Param("From:", float, DYNAMIC, GetVAxisVMin),
                          vcWv.Param("To:", float, DYNAMIC, GetVAxisVMax));
vcCmd.EnableMacro(ZoomAVAxisCmd, vlOnOff.Off);

function
ViewRangeFcn(display, event)
{
    if (   RegionIsDigital(display, event) == 1
        && (NumberOfHAxesSelected(display) == 0)) {
        HAxis := 0;
        AxisIndex = HAxis;
        vcCmd.Run(ZoomAnHAxisCmd, display, event, nil, TRUE);
    } /* if */
    else {
        n := 0;
        {
            HAxis := GetSelectedHAxis(display, -1);
            while (HAxis != -1) {
                AxisIndex = HAxis;
                vcCmd.Run(ZoomAnHAxisCmd, display, event, nil, TRUE);
                ++n;
                HAxis = GetSelectedHAxis(display, HAxis);
            } /* while */
        }
        {
            VAxis := GetSelectedVAxis(display, -1);
            while (VAxis != -1) {
                AxisIndex = VAxis;
                if (!IsAxisDigital(display, VAxis)) {
                    vcCmd.Run(ZoomAVAxisCmd, display, event, nil, TRUE);
                } /* if */
                ++n;
                VAxis = GetSelectedVAxis(display, VAxis);
            } /* while */
        }
        if (n == 0) {
            win := vcEvent.Window(event);
            vcWin.ClearStatus(win);
            vcWin.AddStatus(win, "No Axes Selected");
        } /* if */
    } /* else */
} /* ViewRangeFcn */

ViewRangeCmd := vcWv.Cmd(fac, ViewRangeFcn, "Range", "ViewRange");
vcCmd.EnableMacro(ViewRangeCmd, vlOnOff.Off);

function
DefaultVCFName(d)
{
    convert(string, blkname(d)) + ".vcf";
} /* DefaultVCFName */

function
ViewSaveFcn(display, event, filename)
{
    declare fullname, accept_cmd;

    function accept(d, e) {
        if (!null(fullname))
            SaveView(d, e, fullname);
        else
            SaveView(d, e, filename);
        vcMac.Resume();
    } /* accept */
    accept_cmd = vcWv.Cmd(fac, accept);

    function reject() {
        win := vcEvent.Window(event);
        vcWin.ClearStatus(win);
        vcWin.AddStatus(win, "View Save command cancelled.");
        vcMac.Resume();
    } /* reject */
    
    if (PROMPT_OVERWRITES == YES)
        if (!null(fullname = vscript.vsGetini(filename))) {
                declare buttons, query_t, str, text;
                
                vcMac.Suspend();

                buttons = vcRegion.Create("Buttons", vcButnRow.fac,
                                          list (list ("no", 
                                                      "Don't overwrite",
                                                      reject, 
                                                      list (close_window <- 
                                                            TRUE)),
                                                list ("yes", 
                                                      "Overwrite the file",
                                                      accept_cmd, 
                                                      list (close_window <- 
                                                            TRUE))));
                
                query_t = vcTable.Create ("Overwrite?", 2, 1);

                if (!null(fullname))
                    str = sprintf ("View save file %s already exists, overwrite?",
                                   fullname);
                else
                    str = sprintf ("View save file %s already exists, overwrite?",
                                   filename);                    
                text = vcRegion.Create ("over_text", vcStaticText.fac, str);
                vcTable.Add (query_t, text);
                vcTable.Add (query_t, buttons);
                
                vcWin.Popup (vcEvent.Window (event), query_t,
                             vcButnRow.GetDefault (buttons),
                             vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
            }
        else
            accept(display, event);
    else
        accept(display, event);
    
} /* ViewSaveFcn */

ViewSaveCmd := vcWv.Cmd(fac, ViewSaveFcn, "Save", "View Save",
                        vcWv.Param("File Name:", 
                                   string, DYNAMIC, DefaultVCFName));

function ViewExtensionVerify(display, event, filename)
{
    /* This list should be kept up to date with the list of extensions in
       vtrmain.c, defined right after the vcMain() function. */
    extension_list := list("wfm", "txt", "tr?", "ac?", "sw?", "bad", "gen",
                           "out", "lis", "fam", "mnx", "yld", "stc");

    parsed_fname := vscript.vsParse(filename);
    extension := cadddr(parsed_fname);

    if (member(extension, extension_list) != FALSE)
        error2("vtrace", vlErrSev.Error, 0, nil, 
         "Selected extension matches Viewtrace reserved extension.
                  Please select a different extension.");
} /* ViewExtensionVerify */

function
ViewRestoreFcn(display, event, filename)
{
    declare fullname, buf;

    if (null(fullname = vscript.vsGetini(filename))) {
        win := vcEvent.Window(event);
        vcWin.ClearStatus(win);
        buf = sprintf("View configuration file %s does not exist.", filename);
        vcWin.AddStatus(win, buf);
    }
    else {
        if (!null(fullname))
            RestoreView(display, event, fullname);
        else
            RestoreView(display, event, filename);
    }        
}

ViewRestoreCmd := vcWv.Cmd(fac, ViewRestoreFcn, "Restore", "View Restore",
                           vcWv.Param("File Name:", string,
                                      DYNAMIC, DefaultVCFName));

vcCmd.VerifyParams(ViewSaveCmd, ViewExtensionVerify);
vcCmd.VerifyParams(ViewRestoreCmd, ViewExtensionVerify);

/* This function
 * returns a command which when executed binds the command (cmd)
 * passed in to the middle button. This is used for commands in menus and
 * typed commands which need to wait for a point to be specified before
 * execution can go on.  After that the middle button is bound to that command
 * until another command is executed from the menu which rebinds it.
 */
function
make_button_cmd(cmd)
{
    vcWv.Cmd(fac,
             func(d, e) { vcBind.Set(bindings, '\middledown', cmd); },
             "_" + vcCmd.Name(cmd) + "Button");
} /* make_button_cmd */

/**** josh's thing
function set_mode(c) {
    cc := cmd (func (d, e) set_default_binding (c););
    vcCmd.SetHelp (cc, "Set the current `mode' to " + vcCmd.Name (c));
    vcCmd.SetKey (cc, vcCmd.Key (c));
    cc;
} /* function set_mode */

ViewZoomMenuCmd := make_button_cmd(ViewZoomCmd);
vcCmd.SetKey(ViewZoomMenuCmd, '\F9');
ViewPanMenuCmd := make_button_cmd(ViewPanCmd);
vcCmd.SetKey(ViewPanMenuCmd, '\F6');
ViewScrollMenuCmd := make_button_cmd(ViewScrollCmd);

/*!*/
/* Change */
/* Change-Signal */

ChangeSignalColorCmd := vcWv.Cmd(fac, color, "Color", "Signal Color",
                                 vcWv.Param("Signal Color",
                                            vgColorVT, DYNAMIC, 
                                            get_sig_color));
vcCmd.EnableApply(ChangeSignalColorCmd, vlOnOff.On);

vcFac.Create ("vtsymbol", vtrSymbolVT);
vcPickEnum.RegisterFacility (vtrSymbolVT, vtsymbol.fac);
ChangeSignalSymbolCmd := vcWv.Cmd(fac, func(d,e,s) 
                                  {
                                      ssymbol(d, e, s);
                                  }, 
                                  "Ssymbol", "Choose Symbol",
                                  vcWv.Param ("Symbol", vtrSymbolVT, 
                                              DYNAMIC,
                                              get_sig_symbol));
vcCmd.EnableApply(ChangeSignalSymbolCmd, vlOnOff.On);

ChangeSignalNameCmd := vcWv.Cmd(fac, rename, "Rename", "Signal Name",
                                vcWv.Param("Signal Name", string,
                                           DYNAMIC, get_sig_name));
ChangeSignalRadixCmd := vcWv.Cmd(fac, radix, "Radix", "Signal Radix",
                                 vcWv.Param("Signal Radix", vtrRadixVT,
                                            DYNAMIC, get_sig_radix));
vcCmd.EnableApply(ChangeSignalRadixCmd, vlOnOff.On);

/* Change-Axis */

ChangeAxisTypeCmd := vcWv.Cmd(fac, display_type,
                              "DisplayType", "Axis Display Type",
                              vcWv.Param("Display Type", vtrDispTypeVT,
                                         DYNAMIC, get_display_type));
vcCmd.EnableApply(ChangeAxisTypeCmd, vlOnOff.On);

ChangeAxisNotationCmd := vcWv.Cmd(fac,
                                  func(d, e, n)
                                      notation(d, e, n, vlOnOff.Off);,
                                  "AxisNotation", "Axis Notation",
                                  vcWv.Param("Notation", vtrNotationVT,
                                             DYNAMIC, get_notation));
vcCmd.EnableApply(ChangeAxisNotationCmd, vlOnOff.On);

ChangeAxisLabelCmd := vcWv.Cmd(fac, axislabel, "Label", "Axis Label",
                               vcWv.Param("Axis Label", vlStringRestOfLineVT,
                                          DYNAMIC, get_axis_label));
ChangeAxisGridCmd := vcWv.Cmd(fac, grid, "Grid", "Axis Grid",
                              vcWv.Param("Grid Display", vlOnOffVT,
                                         DYNAMIC, get_grid));
ChangeAxisGridSpaceCmd := vcWv.Cmd(fac, GridSpace, "GridSpace", "GridSpace",
                                   vcWv.Param("Major Grid Space", float,
                                              DYNAMIC, GetMajorGridSpace));

ChangeAxisSideCmd := vcWv.Cmd(fac, disp_side, "Side", "Axis Side",
                              vcWv.Param("Which Side:", vtrLeftRightVT,
                                         DYNAMIC, get_side));
ChangeAxisSymbolsCmd := vcWv.Cmd(fac, toggle_axis_symbols,
                                 "Asymbols", "Axis Symbols",
                                 vcWv.Param("Axis Symbols", vlOnOffVT,
                                            DYNAMIC, get_axis_symbols));

/*!*/
/* Change-Values */

ChangeValuesNotationCmd := vcWv.Cmd(fac, func(d, e, n)
                                    notation(d, e, n, vlOnOff.On);,
                                    "Notation", "Values Notation",
                                    vcWv.Param("Notation", vtrNotationVT,
                                               DYNAMIC, get_notation));
vcCmd.EnableApply(ChangeValuesNotationCmd, vlOnOff.On);

DoChangeValuesNumDigitsCmd := vcWv.Cmd(fac, 
                                       precision,
                                       "_precision", "Values Precision",
                                       vcWv.Param("Number of Significant Digits",
                                                  vtrPrecisionVT, DYNAMIC,
                                                  get_precision, 
                                                  vcScalebar.fac));
vcCmd.EnableApply(DoChangeValuesNumDigitsCmd, vlOnOff.On);

function
maybechangeprecision(display, event)
{
    if (get_values(display) == vlOnOff.Off) {
        vcWin.ClearStatus(win := vcEvent.Window(event));
        vcWin.AddStatus(win, "Values are off");
        FALSE;
    } /* if */
    else
        vcCmd.Run(DoChangeValuesNumDigitsCmd, display, event, nil, TRUE);
}
ChangeValuesNumDigitsCmd := vcWv.Cmd(fac, maybechangeprecision, "Precision");
vcCmd.EnableMacro(ChangeValuesNumDigitsCmd, vlOnOff.Off);

/* Change-Display */

ChangeDisplayInfoCmd := vcWv.Cmd(fac, 
                                 func(d, e, bool) {
                                     /* This handler will allow the user to 
                                        turn the info box off explicitly and
                                        keep it off! */
                                     function turn_off_info(win)
                                         POPUP_INFO_BOX = NO;

                                     info_box(d, e, bool);

                                     if (bool == vlOnOff.Off)
                                         POPUP_INFO_BOX = NO;
                                     else {
                                         top := vcEvent.Window(e);

                                         foreach(iwin; vcWin.Loop())
                                             if(vcWin.Parent(iwin) == top)
                                                 vcWin.DestroyNotify(iwin,
                                                                     turn_off_info);
                                     } /* else */
                                 },
                                 "InfoBox", "Info Box",
                                 vcWv.Param("Display",
                                            vlOnOffVT, DYNAMIC, get_info_box));
ChangeDisplayValuesCmd := vcWv.Cmd(fac, values, "Values", "Values ",
                                   vcWv.Param("Display",
                                              vlOnOffVT, DYNAMIC, get_values));

ChangeDisplayMarkersCmd := vcWv.Cmd(fac, markers, "Markers", "Markers",
                                    vcWv.Param("Display", vlOnOffVT,
                                               DYNAMIC, get_markers));
ChangeDisplaySymbolsCmd := vcWv.Cmd(fac, toggle_wave_symbols,
                                    "Wsymbols", "Symbols in Waves Region",
                                    vcWv.Param("Display", vlOnOffVT,
                                               DYNAMIC, get_wave_symbols));
ChangeDisplayNSymbolsCmd := vcWv.Cmd(fac, n_symbols,
                                     "Nsymbols", "Number of Symbols",
                                     vcWv.Param("Number", vtrNSymbolsVT,
                                                DYNAMIC, get_n_symbols));
ChangeDisplayLinesCmd := vcWv.Cmd(fac, disp_toggle_lines,
                                  "Tlines", "Lines between data points",
                                  vcWv.Param("Display", vlOnOffVT,
                                             DYNAMIC, get_toggle_lines));

/*!*/
/* Change-Display-Widths */

ChangeDisplayWidthsNamesCmd := vcWv.Cmd(fac, NamesWidth, "Wnames");
ChangeDisplayWidthsValuesCmd := vcWv.Cmd(fac, ValuesWidth, "Wvalues");
ChangeDisplayWidthsVAxisCmd := vcWv.Cmd(fac, VAxisWidth, "Wvaxis");
ChangeDisplayWidthsHAxisCmd := vcWv.Cmd(fac, HAxisHeight, "Hhaxis");
RegionBoundaryCmd := vcWv.Cmd(fac, RegionBoundary, "MoveRegionBoundary");
ChangeDisplayRegionBoundaryCmd := vcWv.Cmd(fac, StartRegionBoundary, 
                                           "MoveBoundary");

/* Change-Text */

/* @@@@ */
/*_TextCreateCmd := vcWv.Cmd(fac, tcreate, "_TCreate", "Create Text",
                             vcWv.Param("Text:", vlStringRestOfLineVT,
                                      STICKY, ""));*/

ChangeTextMoveCmd := vcWv.Cmd(fac,
                              func(d, e) {
                                  StartPlaceText(d, e, MoveText);
                                  bindcmd('\middledown',
                                          SetPointWRTMouseCmd); },
                              "TMove");
ChangeTextMoveMenuCmd := make_button_cmd(ChangeTextMoveCmd);
vcCmd.SetKey(ChangeTextMoveMenuCmd, '\^\#\middledown');

function
TextCreate(display, event, str)
    if (tcreate(display, event, str) == 1) {
        tunselect(display, event, "*");
        tselectlast(display);
        StartPlaceText(display, event, MoveText);
        bindcmd('\middledown', SetPointWRTMouseCmd);
    } /* tcreate */
/* TextCreate */
ChangeTextCreateCmd := vcWv.Cmd(fac,
                                func(d, e, s) {
                                    TextCreate(d, e, s);
                                }, "TCreate", "Create Text",
                                vcWv.Param("Text:", vlStringRestOfLineVT,
                                           STICKY, ""));

/*ChangeTextCreateCmd := vcWv.Cmd(fac, tcreate, "TCreate", "Create Text",
                                  vcWv.Param("Text:", vlStringRestOfLineVT,
                                             STICKY, ""));*/
vcCmd.EnableApply(ChangeTextCreateCmd, vlOnOff.On);

ChangeTextDeleteCmd := vcWv.Cmd(fac, tdelete, "TDelete");
ChangeTextStringCmd := vcWv.Cmd(fac, tstring, "TString", "Text String",
                                vcWv.Param("string", vlStringRestOfLineVT,
                                           DYNAMIC, GetTextString));
ChangeTextColorCmd := vcWv.Cmd(fac, tcolor, "TColor", "Text Color",
                               vcWv.Param("color", vgColorVT,
                                          DYNAMIC, gtcolor));
vcCmd.EnableApply(ChangeTextColorCmd, vlOnOff.On);

ChangeTextOriginCmd := vcWv.Cmd(fac, torigin, "TOrigin", "Text Origin",
                                vcWv.Param("origin", vtrTextOriginVT,
                                           DYNAMIC, gtorigin));
vcCmd.EnableApply(ChangeTextOriginCmd, vlOnOff.On);

ChangeTextSizeCmd := vcWv.Cmd(fac, tsize, "TSize", "Text Size",
                              vcWv.Param("size", vtrTextSizeVT,
                                         DYNAMIC, gtsize));
vcCmd.EnableApply(ChangeTextSizeCmd, vlOnOff.On);

/* Change */

ChangeSnapModeCmd := vcWv.Cmd(fac, snap_mode, "SnapMode", "Marker Snap Mode",
                              vcWv.Param("snap mode", vtrSnapModeVT,
                                         DYNAMIC, get_snap_mode));
ChangeAnnotationCmd := vcWv.Cmd(fac, set_values_annotation,
                                "Annotation", "Value Back-annotation",
                                vcWv.Param("annotation",
                                           vlOnOffVT, DYNAMIC,
                                           get_values_annotation));

/*!*/
/* Signal */

DoMoveSelectedSignalsCmd := vcWv.Cmd(fac, 
                                     func(d,e,x,y,direction) {
                                         move_selected_signals(d,x,y,
                                                               direction);
                                     },
                                     "_DoMoveSelectedSignals",
                                     "_DoMoveSelectedsignals",
                                     vcWv.Param("X", integer, 
                                                STICKY, 0),
                                     vcWv.Param("Y", integer,
                                                STICKY, 0),
                                     vcWv.Param("Direction", 
                                                vtrMoveDirectionVT, STICKY,
                                                vtrMoveDirection.Down));

function
movesignal(display, event)
{
    save_y := 0;
    start_y := 0;
    signal_height := SelectedSignalHeight(display, event,
                                          area := vgRect.Create(0, 0, 0, 0));
    if (signal_height == 0)
        return(nil);

    bound_y := get_window_area(display, event, 
                               w_area := vgRect.Create(0, 0, 0, 0));

    vp := get_viewport(display);

    function start() {
        i := GetFirstSelectedSignalDisplayIndex(display);
        if (i != -1) {
            start_y = vgRect.Ymax(area) - (i * signal_height) - 
                (signal_height / 2) - gtheight(display, vtrTextSize.Small);
            vgU.Color(vp, vgColor.White);
            old_rasterop := vgU.Rasterop(vp, vgRasterop.Xor);
            vgU.Line(vp, vgRect.Xmin(area), start_y, vgRect.Xmax(area), 
                     start_y);
            vgU.Rasterop(vp, old_rasterop);
            save_y = start_y;
        } /* if */
    } /* start */

    function refresh() {
        ;
    } /* refresh */

    function finish(x, y) {
        direction := vtrMoveDirection.Down;
        vgU.Color(vp, vgColor.White);
        old_rasterop := vgU.Rasterop(vp, vgRasterop.Xor);
        vgU.Line(vp, vgRect.Xmin(area), start_y,
                 vgRect.Xmax(area), start_y);
        vgU.Rasterop(vp, old_rasterop);
        new_event := vcEvent.Create(vgRect.Xmin(area), vgRect.Ymin(area), 
                                    '\0', "", vcEvent.Window(event), 
                                    vcEvent.Region(event), nil);

        if (y > save_y)
            direction = vtrMoveDirection.Up;
        vcCmd.Run(DoMoveSelectedSignalsCmd, display, new_event, 
                  list(x, y, direction), TRUE);
    } /* finish */

    function move(x, y) {
        if (y >= vgRect.Ymin(area) && y <= vgRect.Ymax(area))
            if (abs(y - start_y) > signal_height) {
                vgU.Color(vp, vgColor.White);
                old_rasterop := vgU.Rasterop(vp, vgRasterop.Xor);
                vgU.Line(vp, vgRect.Xmin(area), start_y,
                         vgRect.Xmax(area), start_y);
                vgU.Rasterop(vp, old_rasterop);
                if (start_y == 0)
                    start_y = y;
                else {
                    if (y - start_y > 0)
                        start_y += signal_height;
                    else
                        start_y -= signal_height;
                } /* else */

                vgU.Color(vp, vgColor.White);
                old_rasterop := vgU.Rasterop(vp, vgRasterop.Xor);
                vgU.Line(vp, vgRect.Xmin(area), start_y,
                         vgRect.Xmax(area), start_y);
                vgU.Rasterop(vp, old_rasterop);
            } /* if */
    } /* move */

    e := vcEvent.Create(vgRect.Xmin(area), vgRect.Ymin(area), '\0', "",
                        vcEvent.Window(event), vcEvent.Region(event), nil);


    r_area := vgRect.Create(vgRect.Xmin(area),
                            vgRect.Xmax(area),
                            vgRect.Ymin(area),
                            bound_y);

    x := (vgRect.Xmax(area) - vgRect.Xmin(area)) / 2;
    vcDrag.Create(e, area, vgColor.White,
                  finish, "Vscript", move, refresh, nil);
    start();
} /* movesignal */

SignalMoveCmd := vcWv.Cmd(fac, movesignal, "Move");
vcCmd.EnableMacro(SignalMoveCmd, vlOnOff.Off);

function
doshow(display, event, name) 
{
    show(display, event, name);
} /* doshow */
DoSignalShowCmd := vcWv.Cmd(fac, doshow, "_DoShow", "doshow",
                            vcWv.Param("Signal Show", vlStringRestOfLineVT,
                                       STICKY, ""));
vcCmd.EnableMacro(DoSignalShowCmd, vlOnOff.On);

function
showsignal(display, event)
{
    declare cols, rows, w, name, temp_str, picklist;

    function notify(data, on_off, index) {
            temp_str = vcRegion.Get(picklist, vscmVT);
    } /* notify */
    
    function cancel_it(new_event) {
        vcWin.Destroy(vcEvent.Window(new_event));
    }
    
    /* Massively hacked to get double clicking to work */
    function accept_it(region) {
        declare str;
        strings := vcRegion.Get(region, vscmVT);
        if (strings == nil) {
            if (temp_str == nil) {
                vcWin.ClearStatus(win := vcEvent.Window(event));
                vcWin.AddStatus(win, "Please select a signal name");
                return(FALSE);
            }
            else { 
                strings = temp_str;                
                str = "";
                foreach(el; vscript.list_iter(strings))
                    str = str + " " + el;
                /* strings now has stuff in it */
            }            
        }
        else {
            str = "";
            foreach(el; vscript.list_iter(strings))
                str = str + " " + el;
            if (temp_str != nil)
                str = "";
            foreach(el; vscript.list_iter(temp_str)) {
                str = str + " " + el;
            }
        }
        vcCmd.Run(DoSignalShowCmd, display, event, list(str), TRUE);
    }
    
    sig_list := get_all_hidden_sig_names(display);
    list_len := length(sig_list);

    if (list_len == 0) {
        vcWin.ClearStatus(win := vcEvent.Window(event));
        vcWin.AddStatus(win, "No signals to show");
        return(FALSE);
    }

    if (list_len < 5) {
        cols = 1;
        rows = list_len;
    }
    else if (list_len < 10) {
        cols = 2;
        rows = (list_len / cols) + 1;
    }
    else {
        cols = 3;
        rows = (list_len / cols) + 1;
    }

    dialog := vcTable.Create("Show Signals", 2, 1);
    
    picklist = vcRegion.Create1("picklist", vcPicklist.fac, nil,
                                list (rows <- 5,
                                      columns <- 3,
                                      exclusive <- FALSE,
                                      enhanced_notify <- TRUE,
                                      range <- TRUE));
/*    printf("Don't forget to re-enable this call in vtrace.vs!\n");*/
    vcPicklist.Notify(picklist, notify);
    
    if (rows <= 12)
        visible := rows;
    else
        visible := 12;
    
    s_picklist := vcPicklist.Scroll("scroll", picklist, 2, visible, visible);
    
    for (sig := sig_list; sig != nil; sig = cdr(sig))
        vcPicklist.Add(s_picklist, car(sig), car(sig));

    buttons := vcRegion.Create("buttons", vcButnRow.fac,
                               list(list("Accept", "Accept",
                                         func() { 
                                             accept_it(picklist);
                                         }, 
                                         list (close_window <- TRUE)),
                                    list("Cancel", "Cancel",
                                         cancel_it, nil)));
    
    vcTable.Add(dialog, s_picklist);
    vcTable.Add(dialog, buttons);
/*    w = vcWin.Popup(vcEvent.Window(event), dialog, dialog,
                     vcWin.NoCmdline +
                     vcWin.NoMenus +
                     vcWin.NoStatus +
                     vcWin.NoBindings +
                     vcWin.NoMacros);   */
    w = vcWin.Create(dialog,
                     vcWin.NoCmdline +
                     vcWin.NoMenus +
                     vcWin.NoStatus +
                     vcWin.NoBindings +
                     vcWin.NoMacros);
} /* showsignal */
SigShowCmd := vcWv.Cmd(fac, showsignal, "_ShowSignal");
vcCmd.EnableMacro(SigShowCmd, vlOnOff.Off);

function
showsiglist(display, event, siglist)
{
    if (string_eq(siglist, "") == TRUE)
        vcCmd.Run(SignalShowCmd, display, event, nil, TRUE);
    else
        vcCmd.Run(DoSignalShowCmd, display, event, list(siglist), TRUE);
}
SignalShowCmd := vcWv.Cmd(fac, showsiglist, "Show", "show",
                          vcWv.Param("Signal to Show", vlStringRestOfLineVT,
                                     STICKY, ""));
vcCmd.EnableMacro(SignalShowCmd, vlOnOff.Off);

SignalShowMenuCmd := vcWv.Cmd(fac,
                              func(d, e) {
                                  showsignal(d, e);
                              }, "_Show");
vcCmd.EnableMacro(SignalShowMenuCmd, vlOnOff.Off);

ShowDrawSignalsCmd := vcWv.Cmd(fac, func(d, e, s) _ShowOnly(d, e, s);,
                               "_ShowOnly", "Show Only",
                               vcWv.Param("signals",
                                          vlStringRestOfLineVT, CONSTANT, ""));

CreateDrawSignalsCmd := vcWv.Cmd(fac,
                                 func(d, e, s) _CreateDrawSignals(d, e, s);,
                                 "_CreateDrawSignals", "Create Draw Signals",
                                 vcWv.Param("signals",
                                            vlStringRestOfLineVT,
                                            CONSTANT, ""));

SignalHideCmd := vcWv.Cmd(fac, hide, "Hide");
SignalDeleteCmd := vcWv.Cmd(fac, delete, "Delete");

PgDnCmd := vcWv.Cmd(fac, pgdn, "PageDown");
DownCmd := vcWv.Cmd(fac, down, "Down");
PgUpCmd := vcWv.Cmd(fac, pgup, "PageUp");
  UpCmd := vcWv.Cmd(fac,   up, "Up");
HomeCmd := vcWv.Cmd(fac, home, "Home");
 EndCmd := vcWv.Cmd(fac,  end, "End");

SignalToAxisCmd := vcWv.Cmd(fac, add_to_axis, "AddToAxis");
SignalFromAxisCmd := vcWv.Cmd(fac, rm_from_axis, "RmFromAxis");
SignalChangeAxisCmd := vcWv.Cmd(fac, change_axis, "ChangeAxis");

SignalLogicCmd := vcWv.Cmd(fac, Logic, "Logic", "Digital Calc",
                           vcWv.Param("Function",
                                      vtrLogicFunctionVT, STICKY,
                                      vtrLogicFunction.and));
vcCmd.EnableApply(SignalLogicCmd, vlOnOff.On);

SignalBusCmd := vcWv.Cmd(fac, Bus, "Bus", "Bus",
                         vcWv.Param("Bus name:",
                                    string, CONSTANT, ""));
SignalExpandCmd := vcWv.Cmd(fac, Expand, "Expand", "Expand");

SignalCopyCmd := vcWv.Cmd(fac, Copy, "Copy", "Copy");
SignalInvertCmd := vcWv.Cmd(fac, Invert, "Invert", "Invert");
SignalTShiftCmd := vcWv.Cmd(fac, Shift, "TShift", "Time Shift",
                            vcWv.Param("Shift by:",
                                       float, CONSTANT, 0.0));
SignalTScaleCmd := vcWv.Cmd(fac, Scale, "TScale", "Time Scale",
                            vcWv.Param("Scale by:",
                                       vlPositiveFloatVT, CONSTANT, 1.0));

function
MergeFcn(display, event, npoints)
{
    vcWin.WithBusy(vcEvent.Window(event), func() {
        merge(display, event, npoints);
    }); /* func */
} /* MergeFcn */
SignalMergeCmd := vcWv.Cmd(fac, MergeFcn, "Merge", "Signal Merge",
                           vcWv.Param("Merge N Points (0=all)",
                                      vtrNPointsVT, STICKY, 0));
                                      /*vcScalebar.fac));*/

function
fft(display, event)
{
    win := vcEvent.Window(event);

    function out_h(string) {
        printf("%s\n", string);
    }

    function err_h(string) {
        printf("%s\n", string);
    }

    function exit_h(mode, code) {
        if (mode == vcSysMode.Exited) {
            if (code == 0) {
                open(display, event, "freq@@@@.gen");
                vcWin.ClearStatus(win);
            } /* if */
            else {
                str := sprintf("FFT failed, exit code %d", code);
                vcWin.ClearStatus(win);
                vcWin.AddStatus(win, str);
            } /* if */
        }
        else {
            str := sprintf("FFT failed, signal %d received", mode);
            vcWin.ClearStatus(win);
            vcWin.AddStatus(win);
        } /* else */
    };

    vscript.vsDelete("time@@@@.gen", vscript.vsFILE);
    vcCmd.Run(FileWriteToCmd, display, event,
              list(vtrFileFormatWrite.GENERIC, "time@@@@"), TRUE);
    vcWin.ClearStatus(win);
    vcSys.Command("fft time@@@@.gen freq@@@@.gen",
                  nil,                  /* environment */
                  out_h, err_h, exit_h,	/* handlers */
                  nil);                 /* host */
} /* fft */

SignalFFTCmd := vcWv.Cmd(fac, fft, "Fft");

/* Axis */

AxisAddCmd := vcWv.Cmd(fac, add_axis, "AddAxis");
AxisSplitCmd := vcWv.Cmd(fac, split_axis, "SplitAxis");
AxisJoinCmd := vcWv.Cmd(fac, join_axes, "JoinAxes");
AxisRemoveCmd := vcWv.Cmd(fac, rm_axis, "RmAxis");

function 
doxvar(display, event, name) {
    xvararg(display, event, name);
}
DoXVarCmd := vcWv.Cmd(fac, doxvar, "_DoXvar", "doxvar",
                      vcWv.Param("X variable", string, STICKY, ""));
vcCmd.EnableMacro(DoXVarCmd, vlOnOff.On);
       
function
xvariable(display, event)
{
    declare cols, rows, w, name;

    function cancel_it(new_event) {
        vcWin.Destroy(vcEvent.Window(new_event));
    }

    function accept_it(new_event) {
        if (name == nil) {
            vcWin.ClearStatus(win := vcEvent.Window(event));
            vcWin.AddStatus(win, "Please select a signal name");
        }
        else {            
            vcCmd.Run(DoXVarCmd, display, event, list(name), TRUE);
            vcWin.Destroy(vcEvent.Window(new_event));
        }
    }

    function assign_entry(data, on_or_off, index) {
        if (on_or_off == vlOnOff.On) {
            name = data;
        }
    }

    sig_list := get_all_sig_names(display);
    list_len := length(sig_list);

    if (list_len < 5) {
        cols = 1;
        rows = list_len;
    }
    else if (list_len < 10) {
        cols = 2;
        rows = (list_len / cols) + 1;
    }
    else {
        cols = 3;
        rows = (list_len / cols) + 1;
    }

    dialog := vcTable.Create("Change X axis variable", 2, 1);
    
    picklist := vcRegion.Create1("picklist", vcPicklist.fac, nil,
                                 list (rows <- rows,
                                       columns <- cols,
                                       enhanced_notify <- TRUE));
    
    if (rows <= 12)
        visible := rows;
    else
        visible := 12;
    s_picklist := vcPicklist.Scroll("scroll", picklist, visible+1, visible, 0);
    vcPicklist.Notify(s_picklist, assign_entry);
    
    for (sig := sig_list; sig != nil; sig = cdr(sig))
        vcPicklist.Add(s_picklist, car(sig), car(sig));

    buttons := vcRegion.Create("buttons", vcButnRow.fac,
                               list(list("Accept", "Accept",
                                         accept_it, nil),
                                    list("Cancel", "Cancel",
                                         cancel_it, nil)));
    
    vcTable.Add(dialog, s_picklist);
    vcTable.Add(dialog, buttons);
    w = vcWin.Popup(vcEvent.Window(event), dialog, dialog,
                     vcWin.NoCmdline +
                     vcWin.NoMenus +
                     vcWin.NoStatus +
                     vcWin.NoBindings +
                     vcWin.NoMacros);   

} /* xvariable */
AxisXVarCmd := vcWv.Cmd(fac, xvariable, "_ChangeXAxis");
vcCmd.EnableMacro(AxisXVarCmd, vlOnOff.Off);
    
function changexvar(display, event, signame)
{
    if (string_eq(signame, "") == TRUE)
        vcCmd.Run(AxisXVarCmd, display, event, nil, TRUE);
    else 
        vcCmd.Run(DoXVarCmd, display, event, list(signame), TRUE);
}
AxisXVariableCmd := vcWv.Cmd(fac, changexvar, "ChangeXAxis", "changexaxis",
                             vcWv.Param("X variable", vlStringRestOfLineVT,
                                        STICKY, ""));
vcCmd.EnableMacro(AxisXVariableCmd, vlOnOff.Off);

AxisXVariableMenuCmd := vcWv.Cmd(fac, 
                                 func(d, e) {
                                     /*vcCmd.Run(AxisXVarCmd, d, e, nil, TRUE);*/
                                     xvariable(d, e);
                                 });  /* no name - not meant to be typed */
vcCmd.EnableMacro(AxisXVariableMenuCmd, vlOnOff.Off);

/*!*/
/* commands used with key bindings only */

SetPointWRTMouseCmd := vcWv.Cmd(fac,
                                func(d, e) {
                                /* comment out the following statement
                                 * if you don't like the info box getting
                                 * turned on for you when you set point.
                                 */
                                    if (POPUP_INFO_BOX == YES)
                                        if (get_info_box(d) == vlOnOff.Off)
                                            vcCmd.Run(ChangeDisplayInfoCmd,
                                                      d, e, list(TRUE), TRUE);
                                    set_point(d, e, "");
                                },
                                "MSPoint");
SetPointCmd := vcWv.Cmd(fac, set_point, "SPoint", "spoint",
                        vcWv.Param("signals",
                                   vlStringRestOfLineVT,
                                   STICKY, ""));

SetMarkWRTMouseCmd := vcWv.Cmd(fac, func(d, e) 
                               {
                                /* comment out the following statement
                                 * if you don't like the info box getting
                                 * turned on for you when you set mark.
                                 */
                                   if (POPUP_INFO_BOX == YES)
                                       if (get_info_box(d) == vlOnOff.Off)
                                           vcCmd.Run(ChangeDisplayInfoCmd,
                                                     d, e, list(TRUE), TRUE);
                                   set_mark(d, e, "");
                               }, 
                               "MSMark");
SetMarkCmd := vcWv.Cmd(fac, set_mark, "SMark", "smark",
                        vcWv.Param("signals",
                                   vlStringRestOfLineVT,
                                   STICKY, ""));

ExchangeCmd := vcWv.Cmd(fac, func(d, e) exchange(d, e);, "Exchange");

/* commands used with command line bindings only */

/*
t_show_cmd := vcWv.Cmd(fac, func(d, e, s) show(d, e, s);,
                       "ShowWRTArgs", "Show Signals",
                       vcWv.Param("signals",
                       vlStringRestOfLineVT, CONSTANT, ""));
*/                       
/*!*/
load("vtracemn.vs");
load("vtracebt.vs");
load("vtraceht.vs");

function
enable_commands(Analog, Digital, Streaming)
{
    /****ANALOG ONLY****/
    vcCmd.Enable(AxisXVariableCmd, Analog);
    vcCmd.Enable(AxisXVariableMenuCmd, Analog);
    vcCmd.Enable(ChangeAxisTypeCmd, Analog);
    vcCmd.Enable(ChangeDisplayNSymbolsCmd, Analog);
    vcCmd.Enable(ChangeDisplaySymbolsCmd, Analog);
    vcCmd.Enable(ChangeDisplayLinesCmd, Analog);
    vcCmd.Enable(SignalMergeCmd, Analog);
    vcCmd.Enable(SignalMathCmd, Analog && !Streaming);
    vcCmd.Enable(SignalFFTCmd, Analog);
    vcCmd.Enable(ZoomVAxisCmd, Analog);
    vcCmd.Enable(ZoomAVAxisCmd, Analog);

    /****DIGITAL ONLY****/
    vcCmd.Enable(EditClearCmd, Digital && !Streaming);
    vcCmd.Enable(EditCopyCmd, Digital && !Streaming);
    vcCmd.Enable(EditCreateSignalCmd, Digital && !Streaming);
    vcCmd.Enable(EditCutCmd, Digital && !Streaming);
    vcCmd.Enable(EditDeleteCmd, Digital && !Streaming);
    vcCmd.Enable(EditInsertTimeCmd, Digital && !Streaming);
    vcCmd.Enable(EditPasteCmd, Digital && !Streaming);
    vcCmd.Enable(EditRepeatCmd, Digital && !Streaming);
    vcCmd.Enable(EditReplaceCmd, Digital && !Streaming);
    vcCmd.Enable(EditSetValueCmd, Digital && !Streaming);
    vcCmd.Enable(EditSubstCmd, Digital && !Streaming);
    vcCmd.Enable(EditTimeLimitCmd, Digital && !Streaming);
    vcCmd.Enable(EditTimeStepCmd, Digital && !Streaming);
    vcCmd.Enable(ChangeSignalRadixCmd, Digital && !Streaming);
    vcCmd.Enable(SignalLogicCmd, Digital && !Streaming);
    vcCmd.Enable(SignalCopyCmd, Digital && !Streaming);
    vcCmd.Enable(SignalInvertCmd, Digital && !Streaming);
    vcCmd.Enable(SignalBusCmd, Digital && !Streaming);
    vcCmd.Enable(SignalExpandCmd, Digital && !Streaming);
    vcCmd.Enable(SignalTShiftCmd, Digital && !Streaming);
    vcCmd.Enable(SignalTScaleCmd, Digital && !Streaming);

    /* EITHER */
    vcCmd.Enable(SignalMoveCmd, vlOnOff.On);
} /* enable_commands */

function
enable_editing_commands(switch)
{
    vcCmd.Enable(EditClearCmd, switch);
    vcCmd.Enable(EditCopyCmd, switch);
    vcCmd.Enable(EditCutCmd, switch);
    vcCmd.Enable(EditDeleteCmd, switch);
    vcCmd.Enable(EditInsertTimeCmd, switch);
    vcCmd.Enable(EditPasteCmd, switch);
    vcCmd.Enable(EditRepeatCmd, switch);
    vcCmd.Enable(EditReplaceCmd, switch);
    vcCmd.Enable(EditSetValueCmd, switch);
    vcCmd.Enable(EditSubstCmd, switch);
} /* enable_editing_commands */

function
disable_for_streaming(switch)
{
    enable_editing_commands(switch);
    vcCmd.Enable(EditTimeLimitCmd, switch);
    vcCmd.Enable(EditTimeStepCmd, switch);
    vcCmd.Enable(EditCreateSignalCmd, switch);
    vcCmd.Enable(SignalLogicCmd, switch);
    vcCmd.Enable(SignalCopyCmd, switch);
    vcCmd.Enable(SignalInvertCmd, switch);
    vcCmd.Enable(SignalBusCmd, switch);
    vcCmd.Enable(SignalExpandCmd, switch);
    vcCmd.Enable(SignalTShiftCmd, switch);
    vcCmd.Enable(SignalTScaleCmd, switch);
} /* disable_for_streaming */

function
disable_for_analog_streaming(switch)
{
    /* There are 2 version of show commands.  One gets invoked with ^S
       and the other one gets invoked by the command line command
       "show". */
    vcCmd.Enable(SignalShowMenuCmd, switch);
    vcCmd.Enable(SignalHideCmd, switch);
    vcCmd.Enable(SignalShowCmd, switch);
    vcCmd.Enable(SignalMathCmd, switch);
} /* disable_for_analog_streaming */

function
set_enable_no_names_selected(on_off)
{
    vcCmd.Enable(SignalMoveCmd, on_off);
    vcCmd.Enable(SignalHideCmd, on_off);
    vcCmd.Enable(SignalDeleteCmd, on_off);
    
    vcCmd.Enable(SignalCopyCmd, on_off);
    vcCmd.Enable(SignalInvertCmd, on_off);
    vcCmd.Enable(SignalExpandCmd, on_off);
    vcCmd.Enable(SignalTShiftCmd, on_off);
    vcCmd.Enable(SignalTScaleCmd, on_off);

    vcCmd.Enable(ChangeSignalColorCmd, on_off);
    vcCmd.Enable(ChangeSignalSymbolCmd, on_off);
    vcCmd.Enable(ChangeSignalNameCmd, on_off);
    vcCmd.Enable(ChangeSignalRadixCmd, on_off);    

    vcCmd.Enable(SignalToAxisCmd, on_off);
    vcCmd.Enable(SignalFromAxisCmd, on_off);
    vcCmd.Enable(SignalChangeAxisCmd, on_off);

    vcCmd.Enable(AxisAddCmd, on_off);
} /* set_enable_no_names_selected */

/* The next function gets called when a V axis is selected, but it
 * is digital.  The commands that are in here have operations that
 * are supported for the digital V axis. */
function
enable_digital_v_axis_selected(on_off)
{
    vcCmd.Enable(ChangeAxisLabelCmd, on_off);
    vcCmd.Enable(ChangeAxisSideCmd, on_off);
    vcCmd.Enable(ChangeAxisSymbolsCmd, on_off);

    vcCmd.Enable(AxisRemoveCmd, on_off);
} /* enable_digital_v_axis_selected */

/* The next function gets called when an H axis is selected. */
function
enable_digital_h_axis_selected(on_off)
{
    vcCmd.Enable(ChangeAxisLabelCmd, on_off);
} /* enable_digital_h_axis_selected */

/* The next function gets called when a V axis is selected, but it 
 * is digital.  The commands that are in here have operations that
 * are undefined for the digital V axis. */
function
disable_digital_v_axis_selected(on_off)
{
    vcCmd.Enable(ChangeAxisTypeCmd, on_off);
    vcCmd.Enable(ChangeAxisNotationCmd, on_off);
    vcCmd.Enable(ChangeAxisGridCmd, on_off);
    vcCmd.Enable(ChangeAxisGridSpaceCmd, on_off);
} /* disable_digital_v_axis_selected */

enable_digital_default := disable_digital_v_axis_selected;
enable_analog_v_axis_selected := enable_digital_v_axis_selected;
enable_analog_default := disable_digital_v_axis_selected;

function
_FocusIn(display, event)
{
    after(display, event);
} /* _FocusIn */

function
after(display, event)
{
    v_index := GetRegionCurrentVAxisIndex(display, event);
    digital_v_axis := IsAxisDigital(display, v_index);

    /* Start with all conditionally enabled commands as disabled. */
    enable_digital_default(vlOnOff.Off);
    enable_digital_v_axis_selected(vlOnOff.Off);

    if (NumberOfSignalsSelected(display) == 0)
        set_enable_no_names_selected(vlOnOff.Off);
    else 
        set_enable_no_names_selected(vlOnOff.On);

    n_h_axes_selected := NumberOfHAxesSelected(display);
    n_v_axes_selected := NumberOfVAxesSelected(display);
    n_axes_selected := n_h_axes_selected + n_v_axes_selected;
    n_hidden_signals := NumberOfHiddenSignals(display);
    RegionNum := car(GetSelectedSignalRegion(display));

    if (digital_v_axis == TRUE) {
        if (n_v_axes_selected > 0) {
            /* These commands can be applied to the digital V axis */
            enable_digital_v_axis_selected(vlOnOff.On);
            
            /* These commands cannot be applied to the digital V axis */
            disable_digital_v_axis_selected(vlOnOff.Off);
        } /* if */
        /* For all of these conditions, we want to enable commands that
         * apply to the H axis, but not the V axis when the V axis is 
         * digital.
         */
        else if (n_v_axes_selected == 0 && n_h_axes_selected == 0) {
            /* These commands can be applied to the H axis in a digital
             * region even if is not selected.  This is because they have
             * no effect on the digital V axis.
             */
            enable_digital_default(vlOnOff.On);
        } /* else if */
        else if (n_v_axes_selected == 0 && n_h_axes_selected > 0) {
            enable_digital_default(vlOnOff.On);
            enable_digital_h_axis_selected(vlOnOff.On);
        } /* else if */
    } /* if */
    else {
        if (n_v_axes_selected > 0 || n_h_axes_selected > 0) {
            enable_analog_v_axis_selected(vlOnOff.On);
            enable_analog_default(vlOnOff.On);
        } /* if */
    } /* else */

    if (n_hidden_signals == 0) {
        vcCmd.Enable(SignalShowCmd, vlOnOff.Off);
        vcCmd.Enable(SignalShowMenuCmd, vlOnOff.Off);
    } /* if */
    else {
        vcCmd.Enable(SignalShowCmd, vlOnOff.On);
        vcCmd.Enable(SignalShowMenuCmd, vlOnOff.On);
    } /* else */

    if (n_axes_selected > 0) {
        if (n_v_axes_selected > 0 && digital_v_axis == TRUE)
            vcCmd.Enable(ViewRangeCmd, vlOnOff.Off);
        else
            vcCmd.Enable(ViewRangeCmd, vlOnOff.On);
    } /* if */
    else
        vcCmd.Enable(ViewRangeCmd, vlOnOff.Off);

    /* If the RegionNum is less than zero, then the signal region is
     * not selected, and the following commands are invalid.
     */
    if (RegionNum < 0)
        enable_editing_commands(FALSE);
    else
        enable_editing_commands(TRUE);

    update_math_signals(display);

    if (event != nil)
        vcWin.UpdateMenus(vcEvent.Window(event));
} /* after */
