/*
 *  $Revision: 1.8.2.1 $ 
 *
 * vcTree facility
 */
function VrtxCreate(args...) {
    name         := car(args); args = cdr(args);
    type         := car(args); args = cdr(args);
    display_info := car(args); args = cdr(args);
    owner        := car(args); args = cdr(args);
    item         := car(args); args = cdr(args);

    VrtxCreate1 (name, type, display_info, owner, item, args);
}

function VrtxAddOwned (owner, owned)
   VrtxAddOwnedBefore (owner, owned, nil);

function VrtxModify (args...)
   VrtxModify1 (car (args), cdr (args));

function EdgeDescriptorCreate (args...)
   EdgeDescriptorCreate1 (args);

function EdgeDescriptorModify (args...)
   EdgeDescriptorModify1 (car (args), cdr (args));

function EdgeModify (args...)
   EdgeModify1 (car (args), cdr (args));

function SelectedVrtx (r)
   vcRegion.Get (r, vscmVT);

/*********************/
/* bindings          */

armed_bindings := vcBind.Create ();

/* selection modality to provide things like drags */
declare old_bindings, old_event, current_op;
function startmode (ev) {
   r := vcEvent.Region (ev);
   old_bindings = vcRegion.Bindings (r);
   vcRegion.SetBindings (r, armed_bindings);
   vcRegion.SetGrab (r, r);
}
function endmode (ev) {
   r := vcEvent.Region (ev);
   vcRegion.SetBindings (r, old_bindings);
   vcRegion.SetGrab (r, nil);
}
function select_proc (op)
   func (t, ev) {
              old_event = ev;
              current_op = op;
              startmode (ev);
              if (!group (t))
                 select (t, op, ev);
            };

select_cmd := vcWv.Cmd (fac, select_proc (vcTree.Select.Replace), "select");
add_cmd := vcWv.Cmd (fac, select_proc (vcTree.Select.Add), "add");
remove_cmd := vcWv.Cmd (fac, select_proc (vcTree.Select.Remove), "remove");

done_cmd := vcWv.Cmd (fac, func (t, ev) {
                   selectfinish (t);
                   endmode (ev);
                   if (group (t))
                      select (t, current_op, ev);
               }, "done");                  

function groupdone_proc (tree, op)
   func (rect) {
      selectrect (tree, rect, op);
      selectfinish (tree);
   };

/* fix the drag so it finishes on left-up */
function fix_drag_bindings (r) {
   b := vcBind.Duplicate (vcRegion.Bindings (r));
   vcBind.Set (b, '\leftup', vcDrag.finish_cmd);
   vcRegion.SetBindings (r, b);
}

/* if we are 'group' and we move a reasonable amount, switch to group sel */
move_cmd := vcWv.Cmd (fac, func (t, ev) {
                   if (group (t) && 
                            !vcWv.ClickTogether  (wv, ev, old_event)) {
                      endmode (ev);
                      r := vcEvent.Region (ev);
                      vp := vcVPCanvas.Viewport (r);
                      d := vcDrag.Create (ev, vcRegion.VisibleArea (r),
                                 groupcolor (t), 
                                 groupdone_proc (t, current_op),
                                 vcDrag.Style.Box, nil, nil, nil);
                      vcDrag.SetViewport (d, vp);
                      fix_drag_bindings (d);
                   }}, "move");
            
                        
                    
selectrect_cmd := vcWv.Cmd (fac, func (t, ev)
               if (group (t)) {
              
                 rect := vcEvent.Rect (ev);
                 vgU.PixelToUserRect (vcVPCanvas.Viewport
                                              (vcEvent.Region (ev)),
                                      rect);                 
                 selectrect (t, rect, vcTree.Select.Replace);},
                 "groupsel");

moveup_cmd    := vcWv.Cmd (fac, moveup,    "moveup");
movedown_cmd  := vcWv.Cmd (fac, movedown,  "movedown");
moveleft_cmd  := vcWv.Cmd (fac, moveleft,  "moveleft");
moveright_cmd := vcWv.Cmd (fac, moveright, "moveright");
ctrlmoveup_cmd    := vcWv.Cmd (fac, ctrlmoveup,    "ctrlmoveup");
ctrlmovedown_cmd  := vcWv.Cmd (fac, ctrlmovedown,  "ctrlmovedown");
ctrlmoveleft_cmd  := vcWv.Cmd (fac, ctrlmoveleft,  "ctrlmoveleft");
ctrlmoveright_cmd := vcWv.Cmd (fac, ctrlmoveright, "ctrlmoveright");
activate_cmd := vcWv.Cmd (fac, activate, "activate");
altactivate_cmd := vcWv.Cmd (fac, altactivate, "altactivate");
space_cmd     := vcWv.Cmd (fac, space,     "space");
kactivate_cmd  := vcWv.Cmd (fac, kactivate,  "kactivate");
kaltact_cmd    := vcWv.Cmd (fac, kaltactivate, "kaltactivate");

vcBind.Set (bindings, '\up',      moveup_cmd);
vcBind.Set (bindings, '\down',    movedown_cmd);
vcBind.Set (bindings, '\left',    moveleft_cmd);
vcBind.Set (bindings, '\right',   moveright_cmd);
vcBind.Set (bindings, '\^\up',    ctrlmoveup_cmd);
vcBind.Set (bindings, '\^\down',  ctrlmovedown_cmd);
vcBind.Set (bindings, '\^\left',  ctrlmoveleft_cmd);
vcBind.Set (bindings, '\^\right', ctrlmoveright_cmd);
vcBind.Set (bindings, ' ',        space_cmd);
vcBind.Set (bindings, '\enter',   kactivate_cmd);
vcBind.Set (bindings, '\^\enter', kaltact_cmd);

vcBind.Set (bindings, '\leftdown', select_cmd);
vcBind.Set (bindings, '\#\leftdown', remove_cmd);
vcBind.Set (bindings, '\^\leftdown', add_cmd);
vcBind.Set (bindings, '\rightdown', altactivate_cmd);
vcBind.Set (bindings, '\middledown', vcGesture.StartCmd);
vcBind.Set (bindings, '\leftdbl', activate_cmd);
vcBind.Set (bindings, 'z', vcVPCanvas.ZoomCmd);

vcBind.Set (armed_bindings, '\leftup', done_cmd);
vcBind.Set (armed_bindings, '\move', move_cmd);
 
vcGesture.Set (gestures, vcGesture.Z,       vcVPCanvas.ZoomToEventCmd);
vcGesture.Set (gestures, vcGesture.Line,    vcVPCanvas.ZoomToEventCmd);
vcGesture.Set (gestures, vcGesture.UpsideDownCheck, vcVPCanvas.FullCmd);
vcGesture.Set (gestures, vcGesture.P,       vcVPCanvas.PanCmd);
vcGesture.Set (gestures, vcGesture.O,       selectrect_cmd);



