/*
 *  $Revision: 1.130.1.41 $  $Date    : 94/06/24 23:36:15 $
 *
 *  GUI definition file for the vdata facility used by ViewData.
 *
 *  Copyright (c) 1994,1995 by Viewlogic Systems, Inc.
 */



/*\vscmuserSection vdata. -- Data browser.
 *
 * DESCRIPTION
 *      The vdata facility implements the GUI for the VDM data browser.  It
 *      does not implement any of the storage and data management functions
 *      itself; it uses the VDM's library API to accomplish those tasks.
 *
 * FACILITY DATA
 *      vdmCntxVT   - VDM session context
 *
 * PREFERENCES
 *      edit_new    - when set, ViewData tries to automatically invoke the
 *                    appropriate editor on the newly create object
 *      select_data - either a vpmLocationVT or vdmObjVT; the facility expands
 *                    the hierarchy to select the given object when its window
 *                    is created for the first time.
 */



/*
 * Register all the auxiliary facilities needed by vdata.
 */
vcWv.RequireFac (wv, "vcBlackLabel");
vcWv.RequireFac (wv, "vdmLocSelect");
vcWv.RequireFac (wv, "vdmClassSelect");

load ("vdata2.vs");



/*
 * These strings are used as drag data, so that they may be easily
 * differentiated.
 */
define_string ("vdmDragObjVT");
define_string ("vdmDragClassVT");



/*
 * ViewData's enumerated types.
 */
define_enum ("vdmWorkOrStorageVT",
             list ("WorkingAreaOnly", "StorageAreaOnly", "Both"));

define_enum ("vdmCheckpointCleanupVT",
             list ("CheckOut", "CheckOutNoLock", "LeaveInStorage"));



/*
 * The following structure definitions are used to define the class browser
 * display.  The main reason for their existance is that the class drawers and
 * the class files nodes are not separate ADT instances.  There's no
 * vdmDrawerVT type to work with.  But, when working the vcBrowse facility, it
 * helps to have a unique type associated with your tree nodes, so that you
 * can search for them easily.  If I were to use consVT, then it wouldn't be
 * unique.
 */
struct vdataDrawerVT
{
    vlStringVT  name;                       /* Drawer name                  */
    vscmVT      definition;                 /* Definition list              */
};



struct vdataClassDrawersVT
{
    vlStringVT  name;                       /* Drawer's name                */
};



struct vdataClassFilesVT
{
    vlStringVT  name;
};



struct vdataClassRootVT
{
    vdataClassDrawersVT  cdrawers;          /* Class drawers                */
    vdataClassFilesVT    cfiles;            /* Class definition files       */
};



/*
 * VDM doesn't have a first-class relationship object, but I need one to
 * attach to the relationsihp node in the tree, so here it is:
 */
struct vdataRelationVT
{
    vlStringVT  name;
    vscmVT      type;
    vscmVT      fanout;
    vdmClassVT  rel_class;
};



/*
 * This structure holds the state of the viewdata editor.  It's called
 * dataviewVT for historical reasons, and the variable name used to receive
 * this structure throughout the module is called "dv".
 */
struct dataviewVT
{
    vdmCntxVT       cntx;                   /* Facility initialization data */
    vpmMapVT        map;                    /* VDM context's search space   */
    vdataClassRootVT croot;                 /* Class root node              */
    vdmWorkOrStorageVT work_or_storage;     /* Use working or storage area? */
    vcRegionVT      region;                 /* Facility's region            */
    vcRegionVT      overlay;                /* Location/Objects & Classes   */
    vcRegionVT      loc_obj_browser;        /* vcBrowse facility instance   */
    vcRegionVT      class_browser;          /* vcBrowse facility instance   */
    vcRegionVT      current_browser;        /* Current browser instance     */
    vcRegionVT      session_log;            /* Session log region           */
    vcRegionVT      table;                  /* vdata facility layout        */
    vlSymTableVT    alias_name_to_obj;      /* Aliases to object ptr table  */
    vlSymTableVT    selected_table;         /* Table of selected nodes      */
    vlIntVT         n_selected_locations;   /* No. of selected locations    */
    vlIntVT         n_selected_lclasses;    /* No. of selected lclasses     */
    vlIntVT         n_selected_objects;     /* No. of selected objects      */
    vlIntVT         n_selected_collections; /* No. of selected collections  */
    vlIntVT         n_selected_compcolls;   /* No. of sel colls w/ comp fn  */
    vlIntVT         n_selected_relationships; /* No. of selected rltionships*/
    vlIntVT         n_selected_classes;     /* No. of selected classes      */
    vlIntVT         n_selected_class_files; /* Is class files node selected?*/
    vlIntVT         n_selected_drawers;     /* No. of selected drawers      */
    vlIntVT         n_selected_maps;        /* No. of selected maps???      */
    vlIntVT         n_selected_relatives;   /* No. of selectec relatives    */
    vcBrowseClassVT location_class;         /* Location node class          */
    vcBrowseClassVT lclass_class;           /* Location class node class    */
    vcBrowseClassVT obj_class;              /* Data object class            */
    vcBrowseClassVT relationship_class;     /* Relationship node class      */
    vcBrowseClassVT version_class;          /* Version node class           */
    vcBrowseClassVT relative_class;         /* Relative node class          */
    vcBrowseClassVT class_files_class;      /* Class files node class       */
    vcBrowseClassVT drawer_class;           /* Drawer node class            */
    vcBrowseClassVT class_class;            /* Class node class             */
    vscmVT          find;                   /* nil or dvFindInfoVT          */
    vlTrueFalseVT   map_modified;           /* Search space modified?       */
    vlTrueFalseVT   class_files_modified;   /* Class files modified?        */
    vlTrueFalseVT   class_drawers_modified; /* Class drawers modified?      */
    vscmVT          current_collection;     /* Current collection object    */
    vscmVT          current_lclass;         /* Current lclass object        */
    vscmVT          call_on_select;         /* List of select handlers      */
};



/*
 * This global variable holds all the initialization handlers that users can
 * register with ViewData.  The handlers will be called each time a new
 * ViewData facility is instantiated.  This is useful for people that might
 * want to register select handlers in their vdataus.vs files.
 *
 * See vdata.CallOnInit(man).
 */
declare InitHandlers;



/*\srcFunction dv_err -- Generic vdata error function.
 * 
 * DESCRIPTION
 *      Automatically applies sprintf to the string arguments.
 */
function dv_err(args...)
    vlEx.Throw2 (vlEx.FAILURE, "ViewData", vlErrSev.Error,
                 apply (sprintf, args));


/*\srcFunction extract_dv -- Extract vdata facility data structure.
 * 
 * DESCRIPTION
 *      Used by all the API functions to extract the dataviewVT structure from
 *      a vdata region argument.
 */
function extract_dv(region)
{
    dv := vcRegion.Data (region, vscmVT);
    if (! istype (dv, dataviewVT))
        dv_err ("Illegal argument type--expected a vdata region, received %s",
                region);
    dv;
}



/*\vscmuserDocpage vdata.GetCntx -- Get the ViewData's VDM context.
 *
 * SYNOPSIS
 *      vdmCntxVT vdata.GetCntx(vdata_region)
 *          vcRegionVT  vdata_region;
 *
 * DESCRIPTION
 *      Returns the ViewData's VDM context.  Each ViewData instantiation has
 *      its own VDM context.  It can be used to take advantage of the host of
 *      the API functions that require a VDM context.
 *
 * SEE ALSO
 *      vdm. (note the dot at the end).
 */
function GetCntx(vdata_region)
    (extract_dv (vdata_region)).cntx;



/*\vscmuserDocpage vdata.GetSelectedLocation -- Iterate through selected locations.
 * 
 * SYNOPSIS
 *      vpmLocationVT vdata.GetSelectedLocation(vdata_region)
 *          vcRegionVT  vdata_region;
 *
 *      foreach (loc; vdata.GetSelectedLocation (vdata_region))
 *          ...
 *
 * DESCRIPTION
 *      Iterates through all the selected locations.  Note that the user is
 *      passed a VPM location object, _not_ its ViewData representation.
 *
 * SEE ALSO
 *      vdata.GetSelectedLocationClass
 */
iterator GetSelectedLocation(region)
{
    dv := extract_dv (region);

    foreach (loc; get_selected_objects (dv, dv.location_class))
        iterate (loc);
}



/*\vscmuserDocpage vdata.GetSelectedLocationClass -- Iterate through selected class.
 * 
 * SYNOPSIS
 *      vdmClassVT vdata.GetSelectedLocationClasses(vdata_region)
 *          vcRegionVT  vdata_region;
 *
 *      foreach (class; vdata.GetSelectedLocationClass (vdata_region))
 *          ...
 *      
 * DESCRIPTION
 *      Iterates through all the selected location classes.  Note that the
 *      user is passed a VDM class object, _not_ its ViewData representation.
 *
 * SEE ALSO
 *      vdata.GetLocationClasses
 */
iterator GetSelectedLocationClass(region)
{
    dv := extract_dv (region);

    foreach (class; get_selected_objects (dv, dv.lclass_class))
        iterate (cadr (class));
}



/*\vscmuserDocpage vdata.GetLocationClasses -- Iterate through location classes.
 * 
 * SYNOPSIS
 *      vdmClassVT vdata.GetLocationClasses(vdata_region, loc)
 *          vcRegionVT      vdata_region;
 *          vpmLocationVT   loc;
 *
 *      foreach (class; vdata.GetLocationClasses (vdata_region, loc))
 *          ...
 *
 * DESCRIPTION
 *      Given a location it iterates through all the vdmClassVT objects
 *      displayed for this location.  Note that the number of class objects
 *      will differ depending on the location's class filter.
 *
 * SEE ALSO
 *      vdata.GetSelectedLocationClass
 */
iterator GetLocationClasses(region, loc)
{
    dv := extract_dv (region);

    foreach (loc_class, type; vcBrowse.ObjLoop (dv.loc_obj_browser, loc))
        iterate (cadr (loc_class));
}



/*\vscmuserDocpage vdata.CallOnSelect -- Register a select handler.
 * 
 * SYNOPSIS
 *      vdata.CallOnSelect(vdata_region, select_handler)
 *          vcRegionVT  vdata_region;
 *          closureVT   select_handler;
 *
 * DESCRIPTION
 *      Registers a user's handler which will be called on every selection
 *      made in the ViewData's location and object browser.  The select
 *      handler must be a function which takes three arguments:
 *
 *      - node data   -- VDM or VPM data structure
 *      - event       -- vcEventVT
 *      - flag        -- TRUE, object was selected; FALSE, object was
 *                       deselected
 *
 *      Registering a ViewData select handler might be necessary if you are
 *      adding new data classes and/or commands and want to enable or disable
 *      them depending on the type of objects being selected.
 *
 *      The handler function will be called as follows:
 *
 *          proc (node_data, event, flag);
 *
 * NOTE
 *      All the select handlers will be called *after* the ViewData's own
 *      select handler finishes execution.
 */
function CallOnSelect(region, select_handler)
{
    dv := extract_dv (region);

    dv.call_on_select = cons (select_handler, dv.call_on_select);
}



/*\vscmuserDocpage vdata.CallOnInit -- Register an initialization handler.
 * 
 * SYNOPSIS
 *      vdata.CallOnInit(handler)
 *          closureVT   handler;
 *
 * DESCRIPTION
 *      Registers a handler to be called when a ViewData facility is
 *      initialized.  Notice that ALL the registered initialization
 *      handlers will be called whenever a new ViewData facility is
 *      instantiated via the vcRegion[.]Create function.
 *
 *      The initialization handler will be called with the ViewData region as
 *      an argument:
 *
 *          handler (vdata_region);
 */
function CallOnInit(init_handler)
    InitHandlers = cons (init_handler, InitHandlers);



/*\vscmuserDocpage vdata.InsertNewObject -- Insert a new object into display.
 * 
 * SYNOPSIS
 *      vdata.InsertNewObject(vdata_region, obj)
 *          vcRegionVT  vdata_region;
 *          vdmObjVT    obj;
 *
 * DESCRIPTION
 *      Use this function if you've created a brand new VDM object and would
 *      like to add it to the display tree.
 */
function InsertNewObject(vdata_region, object)
{
    event := vcEvent.Create (0, 0, '0', "", vcRegion.Window (vdata_region),
                             vdata_region, nil);

    InsertNewObjectAux (extract_dv (vdata_region),
                        vdm.ObjGetLocation (object), vdm.ObjGetClass (object),
                        object, event);
}


/*
 * When looping through attributes on behalf of the user, I'd like to
 * filter out ones that begin with "_vdm", which are used as temp variables
 * by vdm the user really doesn't care about.  This filter takes another
 * iterator as an argument.  It's used with both ClassLoopAttr and
 * ObjLoopAttr are used in places throughout this module.
 */
iterator attr_filter(iter, arg)
{
    foreach (key, val, is_sticky; iter (arg))
       if ((string_length (key) < 4) ||
           (! string_eq (substring (key, 0, 4), "_vdm")))
           iterate (key, val, is_sticky);
}



/*\Docpage is_balanced -- Is the Scheme expression balanced.
 *
 * DESCRIPTION
 *      Determine whether a lisp expression is balanced.  Right now this
 *      is kludgy -- take "(a b c)", transform it to "'(a b c)" and hand
 *      it to Scheme.
 */
function is_balanced(str)
{
    balanced := TRUE;

    catch (vlEx.FAILURE)
        eval_string2 ("'" + str, nil, vscmLanguage.SCM);
    exception
        balanced = FALSE;

    balanced;
}



/*\Docpage write_vpmus -- Write information to the vpmus.scm file.
 * 
 * DESCRIPTION
 *      Common writing function.  It takes a predicate and write functions as
 *      parameters.  The predicate function determines whether the given line
 *      in the vpmus file is to be replaced.  The write function actually
 *      writes the new information.
 */
function write_vpmus(predicate_proc, write_proc)
{
    /*
     * The init file could have either a .scm or a .vs extension.  If can find
     * neither, assume vpmus.scm.
     */
    if ((filename := vsGetini ("vpmus.scm")) == nil)
    {
        filename = vscript.vsAbsPath ("vpmus.scm");
        vlErr.Out2 ("ViewData", vlErrSev.Status,
           sprintf ("No vpmus initialization file was found in your WDIR path.
Will use `%s'.", filename));
    }

    if (fexists (filename))
    {
        declare str, new_str;

        unique_name := vsUnique (cadr (vsParse (vsAbsPath (filename))));
        old_fp      := fopen (filename, "r");
        fp          := fopen (unique_name, "w");
        line_number := 0;

        while ((str = fgetline (old_fp)) != nil)
        {
            ++line_number;

            if (predicate_proc (str))
            {
                while (! is_balanced (str))
                {
                    new_str = fgetline (old_fp);
                    ++line_number;

                    if (new_str == nil)
                    {
                        fclose (fp);
                        vsDelete (unique_name, vsAttr.File);
                        dv_err ("%s line %d: unexpected end of file",
                                filename, line_number);
                    }

                    str += new_str;
                }
            }
            else
                fputline (fp, str);
        }

        fputline (fp, "\n");
        write_proc (fp);
        fclose (fp);
        fclose (old_fp);
        vsRename (unique_name, filename);
    }
    else
    {
        fp := fopen (filename, "w");
        write_proc (fp);
        fclose (fp);
    }
}



/****************************************************************************/
/*                          Core methods.                                   */
/****************************************************************************/


/*\Docpage _Create -- Create method for the vdata facility.
 * 
 * DESCRIPTION
 *      The purpose of this function is really to define all the nodes in the
 *      display tree that the vcBrowse facility will render.  Each node
 *      definition places the node at the same time in the hierarchy.
 */
function _Create(cntx, region)
{
    /*
     * The preferences that control the appearance are obtained from the
     * particular instantiation of the ViewData region.
     */
    class_view        := vcRegion.PrefGet (region, "class_view");
    class_view_filter := vcRegion.PrefGet (region, "class_view_filter");
    std_select        := vcRegion.PrefGet (region, "std_select");


    /*
     * We are going to define schema of information, which the facility
     * "vcBrowse" will be use to present to the user with a tree:
     *
     *  map             search path of directories or equivalent
     *    location      aliased directory in a storage manager
     *      lclass      cons (location, class)
     *        object    vdmObjVT: a real piece of data
     *          rel     a relation of an object
     *  classes         a list 
     *    drawer
     *      class
     *    class file
     *      class
     *
     * This Schema is defined by making vcBrowseClassVT objects for
     * each of the above items.  vcBrowseClassVTs contain methods for
     * getting the name of an object, obtaining miscellaneous
     * textual information about it, and for iterating through
     * the object's children.  The conventino used is to take the
     * name of the vcBrowseClassVT, e.g. "map", and make routines
     * and iterators like "map_name, map_children, map_info", etc.
     * So much of what _Create is is defining lots of these routines.
     * Finally when we create the vcBrowseClassVT we put it in a local
     * variable called, for example, "map_class".  In some cases we
     * need the class elsewhere in this module and we'll later put
     * it into the dataviewVT structure.
     *
     * It would make more sense to do this at the top level of this
     * module, rather than inside _Create, but some of the methods need
     * to refer to the vdmCntxVT, which is passed into _Create.  What
     * would be better is to change vcBrowse to pass through a data object.
     * I guess even in the pure Viewscript world that's sometimes
     * useful; not everything is ideally expressed as a closure.  Sigh.
     */

    /*
     * Define the location node.  Each location node expends into a list of
     * class types for the data objects that can be found in that location.
     * If a class filter was specified in the VPM initialization file, it will
     * expanded to list its component classes.  Note that a class filter might
     * have a class drawer listed as well.  Drawers will be also expanded, to
     * provide a flat list of class type nodes for each location.
     */
    iterator location_info(loc)
    {
        iterate ("Location string: " + vpm.LocationGetString (loc));
        iterate ("Storage manager: " + vpm.LocationGetStorageSpec (loc));
    }


    /*
     * Ah, the beauty of Scheme.  Note the use of the get_class_and_iterate
     * function, which is called recursively to expand all the class drawers
     * encountered while processing the class filter for the location node.
     * Note also that the call to the `iterate' function is made WITHIN the
     * gete_class_and_iterate function.
     *
     * If the class filter hasn't been specified, we loop through all the
     * classes and check if there's at least one data object instance of
     * that class.  If the check is successful, the class is listed.
     */
    iterator location_children(loc, sub_class)
    {
        function iterate_if_there_are_children(class)
        {
            obj_list    := nil;
            op_iterator := vdm.ClassLoopObj;

            if (dv.work_or_storage == vdmWorkOrStorage.WorkingAreaOnly)
                op_iterator = vdm.ClassLoopObjWorking;
            else if (dv.work_or_storage == vdmWorkOrStorage.StorageAreaOnly)
                op_iterator = vdm.ClassLoopObjStorage;

            foreach (obj; op_iterator (class, loc, dv.cntx, nil))
                obj_list = cons (obj, obj_list);

            if (obj_list != nil)
                iterate (list (loc, class, obj_list), lclass_class);
        }


        function compare_classes(c1, c2)
            string_le (vdm.ClassGetName (c1), vdm.ClassGetName (c2));

        class_filter := vpm.LocationGetClassFilter (loc);
        class_list   := nil;

        /*
         * Build the list of classes and store it in the class_list variable.
         * It will be sorted later for display.
         */
        if (class_filter != nil)
        {
            foreach (filter_element; vscript.list_iter (class_filter))
            {
                function get_class_and_iterate(str)
                {
                    class := vdm.CntxFindClass (cntx, str);

                    if (class != nil)
                        class_list = cons (class, class_list);
                    else
                        foreach (elem; 
                                 vscript.list_iter (vpm.ProcessGetDrawerDef
                                                    (str)))
                            get_class_and_iterate (elem);
                }

                get_class_and_iterate (filter_element);
            }
        }
        else
            foreach (class_file, name; vdm.CntxLoopClassFiles (cntx))
                foreach (class, class_name; vdm.ClassFileLoop (class_file))
                    class_list = cons (class, class_list);

        /*
         * Now that we have the class list, we can sort it alphabetically and
         * iterate through it to see if there are any instances of those
         * classes in this location.
         */
        class_list = vscript.Sort (class_list, compare_classes);

        foreach (class; vscript.list_iter (class_list))
            iterate_if_there_are_children (class);
    }


    location_class := 
        vcBrowse.ClassCreate ("Location",
                              children      <- location_children,
                              get_info      <- location_info,
                              get_name      <- vpm.LocationGetAlias,
                              menu          <- LocationMenu,
                              vertex_prefs  <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.Cyan));
                                


    /*
     * Define a location class node.  It differs from the class node used in
     * the class browser in that its children are data objects of this class
     * type found in this location.
     */
    iterator lclass_children(loc_class, sub_class)
    {
        loc      := car (loc_class);
        class    := cadr (loc_class);
        obj_list := caddr (loc_class);

        if (! null (dv.current_lclass))
        {
            obj_list     = nil;
            class        = dv.current_lclass;
            op_iterator := vdm.ClassLoopObj;

            if (dv.work_or_storage == vdmWorkOrStorage.WorkingAreaOnly)
                op_iterator = vdm.ClassLoopObjWorking;
            else if (dv.work_or_storage == vdmWorkOrStorage.StorageAreaOnly)
                op_iterator = vdm.ClassLoopObjStorage;

            foreach (obj; op_iterator (class, loc, dv.cntx, nil))
                obj_list = cons (obj, obj_list);
        }

        function sort_objects(o1, o2)
            string_lei (vdm.ObjGetName (o1), vdm.ObjGetName (o2));

        /*
         * The Sort routine is destructive, so we need to copy the list of
         * objects.  Otherwise, the loc_class node will have its list of
         * objects destructively modified.
         */
        obj_list = vscript.Sort (copy_list (obj_list), sort_objects);

        foreach (obj; vscript.list_iter (obj_list))
        {
            vlSym.Store (dv.alias_name_to_obj,
                         sprintf ("%s:%s", vpm.LocationGetAlias (loc),
                                  vdm.ObjGetName (obj)), obj);

            iterate (obj, obj_class);
        }
    }


    function lclass_name(loc_class)
        vdm.ClassGetName (cadr (loc_class));


    iterator lclass_info(loc_class)
    {
        str       := "";
        class     := cadr (loc_class);
        attr_list := vdm.ClassGetAttrList (class);

        foreach (attr; vscript.list_iter (attr_list))
            str = str + attr + " ";

        iterate ("Attribute List: " + str);

        str = vdm.ClassGetNameTemplate (class);
        if (string_length (str) > 0)
            iterate ("Name Template: " + str);

        iterate ("Physical Template: " + vdm.ClassGetPhysicalTemplate (class));

        iterate ("Type: " + convert (vlStringVT, vdm.ClassGetType (class)));

        foreach (key, val, is_sticky; attr_filter (vdm.ClassLoopAttr, class))
                iterate (sprintf ("Attribute: %s=%s%s", key, val,
                                  is_sticky ? "" : " (temp)"));
    }


    function drag_class(loc_class)
        convert (vdmDragClassVT, vdm.ClassGetName (cadr (loc_class)));


    lclass_class :=
        vcBrowse.ClassCreate ("Class in Location",
                              children      <- lclass_children,
                              get_info      <- lclass_info,
                              get_name      <- lclass_name,
                              menu          <- ClassMenu,
                              drag_data     <- drag_class,
                              vertex_prefs  <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.Green));


    /*
     * Define a true class node for use by the class browser.  Its children
     * will be relations that the given class has with other classes.
     */
    iterator class_info(class)
    {
        if (islist (class))
            class = car (class);

        foreach (option_name, help_string, type, current_value,
                 dflt_value, has_changed;
                 vlPref.Options (vdm.ClassGetPrefs (class)))
            iterate (sprintf ("%s = %s",
                            option_name, convert (vlStringVT, current_value)));

        foreach (key, val, is_sticky; attr_filter (vdm.ClassLoopAttr, class))
            iterate (sprintf ("Attribute: %s=%s%s", key, val,
                              is_sticky ? "" : " (temp)"));
    }


    function class_name(class)
        vdm.ClassGetName (car (class));


    iterator class_children(class, sub_class)
    {
        class = car (class);
        foreach (name, type, fanout, rel_class; vdm.ClassLoopRelation (class))
            iterate (struct vdataRelationVT {name, type, fanout, rel_class},
                     relation_class);
    }


    function class_edit(class, done_notify, vertex)
    {
        event := vcEvent.Create (0, 0, '\0', nil, vcRegion.Window (region),
                                 region, nil);
        view_class_def_aux (dv, event, done_notify);
    }


    function drag_class(class)
        convert (vdmDragClassVT, vdm.ClassGetName (car (class)));


    class_class :=
        vcBrowse.ClassCreate ("Class",
                              children       <- class_children,
                              edit           <- class_edit,
                              get_info       <- class_info,
                              get_name       <- class_name,
                              drag_data      <- drag_class,
                              menu           <- ClassMenu,
                              vertex_prefs   <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.Red));
                                


    /*
     * Define a class relation object.  Relations have no children nodes.  The
     * information panel displays the class' relatives.
     */
    iterator relation_info(r)
    {
        iterate ("Type: " + convert (vlStringVT, r.type));
        iterate ("Fanout: " + convert (vlStringVT, r.fanout));
        classes := "Classes:";

        foreach (class; vdm.ClassLoopRelative (r.rel_class, r.name))
            classes += " " + vdm.ClassGetName (class);

        iterate (classes);
    }

    relation_class :=
        vcBrowse.ClassCreate ("Relation",
                              get_name      <- (func(r) r.name;),
                              get_info      <- relation_info,
                              vertex_prefs  <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.Green));
                                
                                

    /*
     * Define a data object node.  Its children are the object's versions.
     * Its relationships are displayed in the info panel.
     */
    iterator obj_info(obj)
    {
        iterate ("Class: " + vdm.ClassGetName (vdm.ObjGetClass (obj)));
        iterate ("Name: " + vdm.ObjGetName(obj));
        iterate ("Location: " +
                 vpm.LocationGetAlias (vdm.ObjGetLocation(obj)));

        <wk_status, sm_status> := vdm.ObjGetStatus (obj);
        iterate ("Working Status: " + convert (vlStringVT, wk_status));
        iterate ("Storage Status: " + convert (vlStringVT, sm_status));
        foreach (key, val, sticky; attr_filter (vdm.ObjLoopAttr, obj))
            iterate (sprintf ("Attribute: %s=%s%s",
                              key, convert (vlStringVT, val),
                              sticky ? "" : " (temp)"));

        /* If the object is a collection, display the members of the
         * collection in the info panel as well.
         */
        if (vdm.ObjIsCollection (obj))
        {
            iterate ("Collection Members");
            foreach (mem_obj; vdm.CollLoopMember (obj))
                iterate (sprintf (" : %s (%s) %s",
                                  vdm.ObjGetName (mem_obj),
                                  vdm.ClassGetName (vdm.ObjGetClass (mem_obj)),
                                  obj_version (mem_obj)));
        }

        foreach (rel_name, type, fanout, is_sticky, relative;
                 vdm.ObjLoopRelationship (obj))
        {
            iterate (sprintf ("Relationship: %s (%s), %s",
                              rel_name,
                              convert (vlStringVT, type),
                              convert (vlStringVT, fanout)));

            foreach (relative; vdm.ObjLoopRelative (obj, rel_name))
                iterate (sprintf ("%s(%s) %s",
                                  vdm.ObjGetName (relative),
                              vdm.ClassGetName (vdm.ObjGetClass (relative)),
                                  obj_version (relative)));
        }
    }


    iterator obj_children(obj, sub_class)
    {
        declare locking_user, matched_attrs, descriptor, versioned_obj;

        did_iteration := FALSE;

        /*
         * First display any relationships.
         */
        foreach (name, type, fanout, is_sticky, relative;
                 vdm.ObjLoopRelationship (obj))
        {
            iterate (cons (obj, name), relationship_class);
            did_iteration = TRUE;
        }

        /*
         * Second, if the object is a collection, display its members.
         */
        if (vdm.ObjIsCollection (obj))
        {
            /* Collections won't work unless the meta-data is around in the
             * working area, so get it there if it isn't already there. I am
             * not entirely convinced this is the correct solution.
             * Scopus # 16738
             */
            <wk_status, sm_status> := vdm.ObjGetStatus (obj);
            if ((sm_status != vdm.SMStatus.Uncontrolled) &&
                (wk_status == vdm.WorkStatus.Empty))
                shallow_checkout_no_lock (obj);
/*                vdm.ObjCheckoutNoLock (obj);*/

            if (dv.current_collection != nil)
                foreach (member; vdm.CollLoopMember (dv.current_collection))
                {
                    iterate (cons (member, vdm.ObjGetName (member)),
                             relative_class);
                    did_iteration = TRUE;
                }
        }

        /*
         * Third check if there are any versions that need to be displayed.
         */
        if (cadr (vdm.ObjGetStatus (obj)) != vdm.SMStatus.Uncontrolled)
            <log, locks, versions> := vdm.ObjQuery (obj, FALSE, nil);
        else
            versions := nil;

        class           := vdm.ObjGetClass (obj);
        regexp_template := vdm.ClassBuildRegexpTemplate (class, nil);
        regexp          := vscript.re_compile (regexp_template);

        foreach (version; vscript.list_iter (versions))
        {
            locking_user = "unlocked";

            foreach (lock; vscript.list_iter (locks))
                if (string_eq (cdr (lock), version))
                    locking_user = car (lock);

            descriptor = vdm.ObjGetPhysicalName (obj);
            if (vscript.re_exec (regexp, descriptor))
            {
                matched_attrs = vdm.ClassGetMatchedAttributes (class, regexp,
                                                               descriptor);
                versioned_obj = vdm.ObjCreate (vdm.ObjGetName (obj),
                                               version, class,
                                               vdm.ObjGetLocation (obj),
                                               dv.cntx,
                                               matched_attrs);

                iterate (versioned_obj, version_class);
                did_iteration = TRUE;
            }
        }

        /*
         * If there weren't any children nodes of this object, then edit it.
         */
        if (! did_iteration)
            vdm.ObjInvokeEditor (obj, session_print, session_print, nil);
    }

    function session_print(str)
        vcTxShell.PrintString (dv.session_log, str);


    function obj_edit(obj, done_notify, vertex)
    {
        /*
         * Note that the act of editing may change the status of the object,
         * so I need to redisplay it and check the enabling of the relevant
         * commands.
         */
        function done(args...) 
        {
            done_notify ();
            vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, obj, TRUE);
            enable_cmds (dv);
        }

        vdm.ObjInvokeEditor (obj, session_print, session_print, done);
    }


    status_to_sm_bitmap := vlSym.ObjCreate (4);
    status_to_sm_bitmap [vdm.SMStatus.Unlocked]       = "unlocked.xpm";
    status_to_sm_bitmap [vdm.SMStatus.LockedByYou]    = "yourlock.xpm";
    status_to_sm_bitmap [vdm.SMStatus.LockedByOther]  = "locked.xpm";
    status_to_sm_bitmap [vdm.SMStatus.Uncontrolled]   = "uncontrol.xpm";

    status_to_ws_bitmap := vlSym.ObjCreate (5);
    status_to_ws_bitmap [vdm.WorkStatus.New]          = "localnew.xpm";
    status_to_ws_bitmap [vdm.WorkStatus.Modified]     = "localmod.xpm";
    status_to_ws_bitmap [vdm.WorkStatus.Outdated]     = "localout.xpm";
    status_to_ws_bitmap [vdm.WorkStatus.UpToDate]     = "localupt.xpm";
    status_to_ws_bitmap [vdm.WorkStatus.Empty]        = "localemp.xpm";

    /*\Docpage obj_bitmap -- Get the object's bitmap.
     *
     * DESCRIPTION
     *      The vdm.ObjGetStatus function returns two statuses: one for the
     *      storage manager and one for the work status.  In turn, this
     *      function returns a list of two bitmap names.
     */
    function obj_bitmap(obj)
    {
        <w_status, s_status> := vdm.ObjGetStatus (obj);
        list (status_to_ws_bitmap [w_status], status_to_sm_bitmap [s_status]);
    }


    function drag_object(obj)
        convert (vdmDragObjVT, vdm.ObjGetVerboseName (obj));


    obj_class :=
        vcBrowse.ClassCreate ("Object",
                              get_name     <- vdm.ObjGetName,
                              edit         <- obj_edit,
                              get_info     <- obj_info,
                              bitmap       <- obj_bitmap,
                              menu         <- ObjectMenu,
                              children     <- obj_children,
                              drag_data    <- drag_object,
                              vertex_prefs <-
                              list (text_color <- vgColor.Yellow));


    /*
     * Create an object relationship node.  Its children are the object's
     * relatives.
     */
    iterator relationship_info(obj_name)
    {
        obj := car (obj_name);
        rel_name := cdr (obj_name);
        iterate ("Fanout: " +
                 convert (vlStringVT,
                          vdm.ObjGetRelationshipFanout (obj, rel_name)));

        iterate ("Persistent: " +
                 convert (vlStringVT,
                          vdm.ObjGetRelationshipSticky (obj, rel_name)));

        iterate ("Type: " +
                 convert (vlStringVT,
                          vdm.ObjGetRelationshipType (obj, rel_name)));

        foreach (relative; vdm.ObjLoopRelative (obj, rel_name))
            iterate (sprintf ("%s(%s) %s",
                              vdm.ObjGetName (relative),
                              vdm.ClassGetName (vdm.ObjGetClass (relative)),
                              obj_version (relative)));
    }

    iterator relationship_children(obj_name, sub_class)
    {
        obj      := car (obj_name);
        rel_name := cdr (obj_name);

        foreach (relative; vdm.ObjLoopRelative (obj, rel_name))
            iterate (cons (relative, obj_name), relative_class);
    }

    relationship_class :=
        vcBrowse.ClassCreate ("Relationship",
                              children      <- relationship_children,
                              get_info      <- relationship_info,
                              get_name      <- cdr,
                              vertex_prefs  <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.LightRed));
                                


    /*
     * The "relative" class is essentially the "object" class, except
     * that we want to show both the object name and version as the
     * name, we don't want to iterate through children, and we are going
     * to wrap up the vdmObjVT in a cons cell so we get a unique ID
     * for the node.
     */
    function relative_name(relative) 
    {
        obj := car (relative);
        vdm.ObjGetName (obj) + " " + obj_version (obj);
    }


    iterator relative_info(relative)
        foreach (str; obj_info (car (relative)))
            iterate (str);


    function relative_edit(relative, done_notify, vertex)
        vdm.ObjInvokeEditor (car (relative), session_print, session_print,
                             func(args...) done_notify (););


    relative_class :=
        vcBrowse.ClassCreate ("Relative",
                              get_name     <- relative_name,
                              edit         <- relative_edit,
                              get_info     <- relative_info,
                              vertex_prefs <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.Magenta));
                                


    /*
     * The "version" class is essentially the "object" class, except
     * that we don't want to show the versions as the children of
     * the versions, and the name is the version string, not the object
     * name.
     */

    /*
     * This function needs some updating, because it doesn't work, mainly
     * because the obj_children is putting bogus object names.
     */
    function version_edit(obj, done_notify, vertex)
        if (! vdm.ObjIsCollection (obj))
        {
            <wk_status, sm_status> := vdm.ObjGetStatus (obj);
            if ((sm_status != vdm.SMStatus.Uncontrolled) &&
                (wk_status == vdm.WorkStatus.Empty))
                shallow_checkout_no_lock (obj);


            vdm.ObjInvokeEditor (obj, session_print, session_print,
                                 func(args...) done_notify (););
        }
        else
        {
            done_notify ();
            dv_err ("Collection object is not editable.");
        }


    iterator version_children(obj, sub_class)
        if (vdm.ObjIsCollection (obj))
        {
            /* !!! This is disabled until a real solution can be developed
             *     for the "collection which is not local" bug.
             *
             *     Scopus #17094
             */
            /*
            <wk_status, sm_status> := vdm.ObjGetStatus (obj);
            if ((sm_status != vdm.SMStatus.Uncontrolled) &&
                (wk_status == vdm.WorkStatus.Empty))
                shallow_checkout_no_lock (obj);

            foreach (member; vdm.CollLoopMember (obj))
                iterate (cons (member, vdm.ObjGetName (member)),
                         relative_class);
            */
            ;
        }


    iterator version_info(obj)
    {
        iterate ("Class: " + vdm.ClassGetName (vdm.ObjGetClass (obj)));
        iterate ("Name: " + vdm.ObjGetName(obj));
        iterate ("Location: " +
                 vpm.LocationGetAlias (vdm.ObjGetLocation(obj)));

        <wk_status, sm_status> := vdm.ObjGetStatus (obj);
        iterate ("Working Status: " + convert (vlStringVT, wk_status));
        iterate ("Storage Status: " + convert (vlStringVT, sm_status));

        foreach (key, val, sticky; attr_filter (vdm.ObjLoopAttr, obj))
            iterate (sprintf ("Attribute: %s=%s%s",
                              key, convert (vlStringVT, val),
                              sticky ? "" : " (temp)"));
    }



    version_class :=
        vcBrowse.ClassCreate ("Version",
/*                              children     <- version_children, */
                              edit         <- version_edit,
                              get_info     <- version_info,
                              get_name     <- obj_version,
                              menu         <- ObjectMenu,
                              vertex_prefs <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.White));
                                

    /*
     * Define the "Search Space" node.  This is the top-level node in the
     * display tree in the vdata facility.  By default, it remains invisible
     * to the user.
     */
    function map_name(map)
        "Search Space";

    iterator map_children(map, sub_class)
        foreach (location; vpm.MapLoopLocation (map))
            iterate (location, location_class);

    map_class :=
        vcBrowse.ClassCreate ("Map",
                              parse    <- (func(name) dv.map;),
                              get_name <- map_name,
                              children <- map_children);



    /*
     * Define a single drawer node.  Note that at this level, the node
     * information has been defined by the drawers_class to be a list composed
     * of a drawer's name and the drawer's definition.
     */
    function drawer_name(drawer)
        drawer.name;


    function drawer_edit(drawer_info, done_notify, vertex)
    {
        event := vcEvent.Create (0, 0, '\0', nil, vcRegion.Window (region),
                                 region, nil);
        edit_drawer_aux (dv, event, done_notify);
    }


    iterator drawer_children(drawer, sub_class)
        foreach (element; vscript.list_iter (drawer.definition))
        {
            class := vdm.CntxFindClass (dv.cntx, element);

            if (class != nil)
                iterate (list (class), class_class);
            else
                iterate (struct vdataDrawerVT {element,
                                          vpm.ProcessGetDrawerDef (element)},
                         drawer_class);
        }


    drawer_class :=
        vcBrowse.ClassCreate ("Class Drawer",
                              children      <- drawer_children,
                              edit          <- drawer_edit,
                              get_name      <- drawer_name,
                              menu          <- ClassMenu,
                              vertex_prefs  <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.LightGreen));
                                


    /*
     * Define the class drawers node.
     */
    function class_drawers_name(drawers)
        "Class Drawers";


    iterator class_drawers_children(drawers, sub_class)
        foreach (drawer_name, drawer_def; vpm.ProcessLoopDrawers ())
            iterate (struct vdataDrawerVT {drawer_name, drawer_def},
                     drawer_class);


    class_drawers_class := 
        vcBrowse.ClassCreate ("Class Drawers",
                              children      <- class_drawers_children,
                              get_name      <- class_drawers_name,
                              menu          <- ClassMenu,
                              parse         <- (func(name) dv.croot.cdrawers;),
                              vertex_prefs  <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.Cyan));



    /*
     * Define the class file node.  Its children are the class type
     * definitions contained in that class file.
     */
    iterator class_file_info(class_file)
        iterate ("Class file: " + vdm.ClassFileGetFilename (class_file));


    iterator class_file_children(class_file, sub_class)
        foreach (class, class_name; vdm.ClassFileLoop (class_file))
            iterate (list (class), class_class);


    class_file_class := 
        vcBrowse.ClassCreate ("Class File",
                              children      <- class_file_children,
                              get_info      <- class_file_info,
                              get_name      <- vdm.ClassFileGetName,
                              menu          <- ClassMenu,
                              vertex_prefs  <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.LightMagenta));
                                

    /*
     * Define the class files node.  The node expands to list all the class
     * files registered with the ViewData context.  The registration can be
     * done programmatically via an explicit call to the vdm.CntxAddClassFile
     * function, or indirectly through the vmpus file.  The _Initial method
     * looks at the vpmus file.
     */
    function class_files_name(bunches)
        "Class Files";


    iterator class_files_children(class_files, sub_class)
        foreach (class_file, class_filename; vdm.CntxLoopClassFiles (cntx))
            iterate (class_file, class_file_class);    
        

    class_files_class :=
        vcBrowse.ClassCreate ("Class Files",
                              children      <- class_files_children,
                              get_name      <- class_files_name,
                              menu          <- ClassMenu,
                              parse         <- (func(name) nil;),
                              vertex_prefs  <-
                              list (raster_font <- vgFont.small,
                                    text_color  <- vgColor.Cyan));
                                


    /*
     * Define the root node of the class browser.  It will have two children:
     * the class drawers and the class files.
     */
    function class_root_name(root)
        "Class Types";


    iterator class_root_children(croot, sub_class)
    {
        iterate (croot.cdrawers, class_drawers_class);
        iterate (croot.cfiles, class_files_class);
    }


    class_root := vcBrowse.ClassCreate ("Class Types",
                                        parse       <- (func(name) dv.croot;),
                                        get_name    <- class_root_name,
                                        children    <- class_root_children);
    


    cfiles   := struct vdataClassFilesVT {"Class Files"};
    cdrawers := struct vdataClassDrawersVT {"Class Drawers"};
    croot    := struct vdataClassRootVT {cdrawers, cfiles};

    /*
     * Instantiate the vdata facility data structure.
     */
    dv := struct dataviewVT
    {
        cntx,                       /* ViewData's VDM session context       */
        vdm.CntxGetMap (cntx),      /* Search space map                     */
        croot,
        vdmWorkOrStorage.Both,      /* Working or storage area              */
        region,                     /* ViewData's region                    */
        vcRegion.Create ("", vcOverlay.fac, vgAlign.UpperLeft),
        region,                     /* Will be set to location browser      */
        region,                     /* Will be set to class browser         */
        region,                     /* Set to the current of above 2        */
        vcRegion.Create ("", vcTxShell.fac, "",
                         height <- 10,
                         prompt <- "vdata> "),
        vcTable.Create ("", 3, 1),  /* Main layout of the vdata facility    */
        vlSym.StrCreate (11),       /* Table of aliases2objects maps        */
        vlSym.ObjCreate (11),       /* Table of selected nodes              */
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* Selected info                   */
        location_class,             /* Location node                        */
        lclass_class,               /* Location class node                  */
        obj_class,                  /* Object node                          */
        relationship_class,         /* Relationship node                    */
        version_class,              /* Object version node                  */
        relative_class,             /* Relative object node                 */
        class_files_class,          /* Class definition files node          */
        drawer_class,               /* Drawer node                          */
        class_class,                /* Class node                           */
        nil,                        /* Find info                            */
        FALSE, FALSE, FALSE,        /* Modified info                        */
        nil,                        /* Current collection object            */
        nil,                        /* Current lclass object                */
        nil                         /* List of select handlers              */
    };


    /*
     * Register an interpreter for commands typed in the session log.  It's
     * basically the same as the Powerview's default command line editor.
     */
    vcTxShell.CallOnReturn (dv.session_log, func(event, str)
        vcWin.RunCmdline (vcEvent.Window (event), str, event););


    vlSym.Dynamic (dv.alias_name_to_obj, 2, 2);
    vlSym.Dynamic (dv.selected_table, 2, 2);


    /*
     * Register the session log window with the ViewData's VDM context so that
     * all the encapsulation's messages can be redirected to the session's log
     * window.
     */
    vdm.CntxCallOnOutput (dv.cntx, session_print);
    vdm.CntxCallOnError (dv.cntx, session_print);


    /*
     * Instantiate the two vcBrowse regions: one for the data browser and the
     * other for the class browser.  ViewData will be displaying both of them
     * in the same area of the screen using the vcOverlay facility.
     */
    region_size := vgSize.Create (200, 300, 50, 100, 750, 1500);
    vdata_prefs := vcRegion.PrefCopyChanged (region, vcBrowse.fac);

    vdata_prefs  = cons (size         <- region_size,       vdata_prefs);
    vdata_prefs  = cons (drags        <- TRUE,              vdata_prefs);
    vdata_prefs  = cons (hide_root    <- TRUE,              vdata_prefs);
    vdata_prefs  = cons (exclusive    <- FALSE,             vdata_prefs);
    vdata_prefs  = cons (macro_on     <- TRUE,              vdata_prefs);
    vdata_prefs  = cons (std_select   <- std_select,        vdata_prefs);


    dv.loc_obj_browser = vcRegion.Create1 ("loc_obj", vcBrowse.fac,
                                           cons (dv.map, "Map"), vdata_prefs);


    /*
     * Class browser has its tree centered.
     */
    dv.class_browser = vcRegion.Create1 ("class", vcBrowse.fac,
                                         cons (croot, "Class Types"),
                                         vdata_prefs);


    vcRegion.AddChild (dv.overlay, dv.loc_obj_browser);
    vcRegion.AddChild (dv.overlay, dv.class_browser);
    vcOverlay.SetCurrent (dv.overlay, dv.loc_obj_browser);
    dv.current_browser = dv.loc_obj_browser;

    vcRegion.FocusInhibit (dv.region);


    /*\Docpage select_handler -- Handle all selections.
     * 
     * DESCRIPTION
     *      This function serves as the selection handler: all selections for
     *      both the data and the class browser regions will be passing
     *      through here.  Most often, the selected node will be added to or
     *      subtracted from the appropriate selection table.  Almost all of
     *      ViewData commands work only on the selected objects.
     */
    function select_handler(obj, class, event, flag)
    {
        add := vlSym.Size (dv.selected_table);

        if (flag)
            vlSym.Store (dv.selected_table, obj, class);
        else
            vlSym.Remove (dv.selected_table, obj);

        add = vlSym.Size (dv.selected_table) - add;

        if ((class == obj_class) || (class == version_class))
        {
            dv.n_selected_objects = dv.n_selected_objects + add;
            select_collections (obj, add);
        }
        else if (class == relative_class)
        {
            dv.n_selected_relatives = dv.n_selected_relatives + add;
            select_collections (car (obj), add);
        }
        else if (class == class_class)
            dv.n_selected_classes = dv.n_selected_classes + add;
        else if (class == map_class)
            dv.n_selected_maps = dv.n_selected_maps + add;
        else if (class == class_files_class)
            dv.n_selected_class_files = dv.n_selected_class_files + add;
        else if (class == drawer_class)
            dv.n_selected_drawers = dv.n_selected_drawers + add;
        else if (class == location_class)
            dv.n_selected_locations = dv.n_selected_locations + add;
        else if (class == relationship_class)
            dv.n_selected_relationships = dv.n_selected_relationships + add;
        else if (class == lclass_class)
            dv.n_selected_lclasses = dv.n_selected_lclasses + add;

        enable_cmds (dv);

        /*
         * Call all the registered notify functions.  Note I'm not passing to
         * the user function the internal data structures used to hold the
         * semantic data for each node, but a real data object.  (Class node
         * has as its data list of three elements).  This is because I don't
         * want anybody to start relying on the layout of those data
         * structures, so that I'm stuck supporting them forever.
         */
        foreach (proc; vscript.list_iter (dv.call_on_select))
        {
            if (class == lclass_class)
                obj = cadr (obj);

            proc (obj, event, flag);
        }
    }

    function select_collections (obj, add)
        if (is_class_a_collection (vdm.ObjGetClass (obj)))
        {
            dv.n_selected_collections = dv.n_selected_collections + add;
            if (is_collection_composable (obj))
                dv.n_selected_compcolls = dv.n_selected_compcolls + add;

            dv.current_collection = obj;
        }


    /*
     * Register a selection handler for both vcBrowse regions: the data and
     * class browsers.
     */
    vcBrowse.CallOnSelect (dv.loc_obj_browser, select_handler);
    vcBrowse.CallOnSelect (dv.class_browser, select_handler);

    /*
     * SHOULD RE-EXPAND PREVIOUSLY EXPANDED VERTICES!
     */
    vcTable.Set (dv.table, 0, 0, dv.overlay);
    vcTable.Set (dv.table, 1, 0, dv.session_log);
    vcTable.Row (dv.table, 1, vcTable.Border.Adjustable, 5, 5, 5,
                 vgAlign.UpperLeft);

    vcRegion.AddChild (region, dv.table);

    /*
     * Register a function that will be called when the Quit dialog box shows
     * up.  That function will return the text of the confirm message that
     * will be composed by the _Modified method defined below.
     */
    function quit_msg()
        if (! vcRegion.Destroyed (region))
        {
            win := vcRegion.Window (region);

            if ((win != nil) && vcWin.IsAlive (win) && vcWin.IsModified (win))
                vcWin.ConfirmMessage (win);
        }

    vcQuit.AddFunction (quit_msg);


    /*
     * OK, one more thing: process the initialization handlers.  But, note
     * that the handlers need a ViewData region and the binding of the data
     * and the region does not happen until _after_ the _Create method
     * returns.  So, we delay the processing of the initialization handlers
     * until Core returns to its top event loop.
     */
    vcWv.Delay (wv,
                func()
                    foreach (handler; vscript.list_iter (InitHandlers))
                        handler (region););
                        
    dv;
} /* _Create */



/*\Docpage _FocusIn -- What to do when focus is set on us?
 * 
 * DESCRIPTION
 *      Enable/disable commands according to the situation.
 */
function _FocusIn(dv, event)
    enable_cmds (dv);



/*\Docpage _Modified -- Form a prompt message when modifications weren't saved.
 * 
 * DESCRIPTION
 *      If some of the user modifications have not been saved, but the user
 *      quits ViewData, this method will be called to form a prompt message
 *      asking the user to confirm that such and such modifications will be
 *      lost otherwise.
 */
function _Modified(dv, region)
{
    modified := dv.map_modified || dv.class_drawers_modified ||
                dv.class_files_modified;

    if (modified)
    {
        msg := "ViewData has changed some information that should
be saved into the file vpmus, found in the WDIR
search path.  It has changed:\n\n";

        if (dv.map_modified)
            msg += "    The search path location order.\n";

        if (dv.class_files_modified)
            msg += "    The list of files of class definitions.\n";

        if (dv.class_drawers_modified)
            msg += "    The class drawer definitions.\n";

        vcWin.SetConfirmMessage (vcRegion.Window (region), msg);
    }

    modified;
}



/*\Docpage _Initial -- Perform first time initialization.
 * 
 * DESCRIPTION
 *      Load the class definition files specified in the vpm initialization
 *      files.
 */
function _Initial(dv, event)
{
    enable_cmds (dv);

    /*
     * Now check if the facility has been instantiated with a particular
     * object to be selected.  That object will be passed in via the
     * "select_data" preference.
     */
    select_data := vcRegion.PrefGet (dv.region, "select_data");

    /*
     * If there's an initial object to be selected, force a refresh of the
     * vdata region, so that the user is not looking at a black window. 
     */
    if (select_data != nil)
    {
        vcRegion.QueueForRefresh (vcEvent.Region (event));
        vcWv.ProcessPending (wv);
    }

    if (istype (select_data, vdmObjVT))
        SelectObject (dv.region, obj);

    else if (istype (select_data, vpmLocationVT))
        SelectLocation (dv, event, vpm.LocationGetAlias (select_data));

    else if (select_data != nil)
        vlErr.Out2 ("ViewData", vlErrSev.Error,
        sprintf ("select_data preference value not an object or location: %s",
                 select_data));
}



/*\Docpage enable_cmds -- Enable/disable some of the commands.
 * 
 * DESCRIPTION
 *      Most of the commands are enabled based on certain criteria.
 */
function enable_cmds(dv)
{
    vcCmd.Enable (AddAttributeCmd, dv.n_selected_objects > 0);
    vcCmd.Enable (EditAttributesCmd, dv.n_selected_objects > 0);
    vcCmd.Enable (SelectChildrenCmd, vlSym.Size (dv.selected_table) > 0);
    vcCmd.Enable (SelectRelativesCmd,
                  dv.n_selected_collections + dv.n_selected_relationships > 0);
    vcCmd.Enable (AddToCollectionCmd,
                  (dv.n_selected_collections >= 1) &&
                  (dv.n_selected_objects > 1));
    vcCmd.Enable (RemoveFromCollectionCmd,
                  (dv.n_selected_collections >= 1) &&
                  (dv.n_selected_relatives >= 1));
    vcCmd.Enable (ComposeCollectionCmd, (dv.n_selected_compcolls >= 1));
    vcCmd.Enable (AddRelationshipCmd,
                  (dv.n_selected_objects == 1) &&
                  obj_selected_has_spare_relation (dv));
    vcCmd.Enable (RemoveRelationshipCmd, dv.n_selected_relationships > 0);
    vcCmd.Enable (AddToRelationshipCmd,
                  (dv.n_selected_relationships == 1) &&
                  (dv.n_selected_objects > 0));
    vcCmd.Enable (RemoveFromRelationshipCmd,
                  ((dv.n_selected_relationships == 1) &&
                   (dv.n_selected_objects > 0)));
    vcCmd.Enable (FindObjectCmd,
                  dv.current_browser == dv.loc_obj_browser);
    vcCmd.Enable (SelectLocationCmd,
                  dv.current_browser == dv.loc_obj_browser);

    enable_class_cmds (dv);
    enable_object_cmds (dv);
    enable_location_cmds (dv);

    if (! vcRegion.Destroyed (dv.region))
    {
        w := vcRegion.Window (dv.region);
        if ((w != nil) && vcWin.IsAlive (w))
            vcWin.UpdateMenus (w);
    }
}



/*\Docpage enable_class_cmds -- Enable Class Menu commands.
 * 
 * DESCRIPTION
 *      The are enabled only when the class overlay is being displayed.
 *      
 * NOTE
 *      This function is called from the enable_cmds function which takes care
 *      of the menu updates and refreshing.
 */
function enable_class_cmds(dv)
{
    enable              := dv.current_browser == dv.class_browser;
    class_drawers_only  := dv.n_selected_drawers > 0;
    class_drawer_only   := dv.n_selected_drawers == 1;
    class_only          := dv.n_selected_classes > 0;
    class_files_only    := dv.n_selected_class_files == 1;

    vcCmd.Enable (CreateDrawerCmd,      enable);
    vcCmd.Enable (RemoveDrawerCmd,      enable && class_drawers_only);
    vcCmd.Enable (EditDrawerCmd,        enable && class_drawer_only);
    vcCmd.Enable (AddClassFileCmd,      enable && class_files_only);
    vcCmd.Enable (ViewClassDefCmd,      enable && class_only);
    vcCmd.Enable (WriteClassInfoCmd,    enable);

    vcCmd.Enable (LClassUpdateCmd, dv.current_browser == dv.loc_obj_browser &&
                  dv.n_selected_lclasses == 1);
}



/*\Docpage enable_object_cmds -- Enable the Object menu commands.
 * 
 * DESCRIPTION
 *      Enables or disables various Object menu commands.  This happens only
 *      if there's one object selected AND the current browser is the location
 *      browser.  If there's more objects selected, all the Object menu
 *      commands are enabled.  If there are no objects selected, all the
 *      Object menu commands are disabled.
 *
 *      Only the commands that make sense to be disabled are examined.  For
 *      example, the InfoCmd should be enabled independent of the status of
 *      the object.
 *
 *      This function is called from the enable_cmds function.
 *
 * NOTE
 *      The InfoCmd is always enabled when only one object is selected.
 */
function enable_object_cmds(dv)
{
    enable := dv.current_browser == dv.loc_obj_browser;

    /*
     * Some commands are enabled always whenever the location browser is
     * active.
     */
    vcCmd.Enable (NewObjectCmd,     enable);
    vcCmd.Enable (DryRunCmd,        enable);
    vcCmd.Enable (ObjectUpdateCmd,  enable);


    if (enable && (dv.n_selected_objects == 1))
    {
        obj := get_one_obj (dv, vdmObjVT, "enable_object_cmds");

        <wk_status, sm_status> := vdm.ObjGetStatus (obj);

        vcCmd.Enable (EditObjCmd, ((wk_status == vdm.WorkStatus.Empty &&
                                    sm_status == vdm.SMStatus.Uncontrolled) ||
                                   (wk_status != vdm.WorkStatus.Empty)));
        vcCmd.Enable (CheckoutCmd,  (sm_status != vdm.SMStatus.Uncontrolled &&
                                     sm_status != vdm.SMStatus.LockedByYou));
        vcCmd.Enable (CheckoutNoLockCmd,
                      sm_status != vdm.SMStatus.Uncontrolled);
        vcCmd.Enable (CheckinCmd,   (sm_status != vdm.SMStatus.LockedByOther &&
                                     wk_status != vdm.WorkStatus.Empty));
        vcCmd.Enable (CreateCheckpointCmd, vdm.ObjIsCollection (obj));
        vcCmd.Enable (RestoreCheckpointCmd, vdm.ObjIsCollection (obj));
        vcCmd.Enable (LockCmd,      sm_status == vdm.SMStatus.Unlocked);
        vcCmd.Enable (UnlockCmd,    sm_status == vdm.SMStatus.LockedByYou);
        vcCmd.Enable (DeleteCmd,    TRUE);
        vcCmd.Enable (InfoCmd,      TRUE);
        vcCmd.Enable (SetTagCmd,    TRUE);
    }
    else
    {
        enable = enable && (dv.n_selected_objects > 1);

        vcCmd.Enable (EditObjCmd,           enable);
        vcCmd.Enable (CheckoutCmd,          enable);
        vcCmd.Enable (CheckoutNoLockCmd,    enable);
        vcCmd.Enable (CheckinCmd,           enable);
        vcCmd.Enable (CreateCheckpointCmd,  enable);
        vcCmd.Enable (RestoreCheckpointCmd, enable);
        vcCmd.Enable (LockCmd,              enable);
        vcCmd.Enable (UnlockCmd,            enable);
        vcCmd.Enable (DeleteCmd,            enable);
        vcCmd.Enable (InfoCmd,              enable);
        vcCmd.Enable (SetTagCmd,            enable);
    }
}



/*\Docpage enable_location_cmds -- Enable Location menu commands.
 * 
 * DESCRIPTION
 *      Location commands are enabled if the current browser is the location
 *      browser.  Additional selection criteria apply as well.
 *
 *      This function is called from the enable_cmds function.
 */
function enable_location_cmds(dv)
{
    enable := dv.current_browser == dv.loc_obj_browser;

    vcCmd.Enable (LocationCreateCmd,    enable);
    vcCmd.Enable (LocationWriteCmd,     enable);

    vcCmd.Enable (LocationMoveTopCmd,
                  (enable && (dv.n_selected_locations == 1)));
    vcCmd.Enable (LocationMoveUpCmd,    
                  (enable && (dv.n_selected_locations == 1)));
    vcCmd.Enable (LocationMoveDownCmd,  
                  (enable && (dv.n_selected_locations == 1)));
    vcCmd.Enable (LocationRemoveCmd,    
                  (enable && (dv.n_selected_locations > 0)));
    vcCmd.Enable (LocationEditCmd,      
                  (enable && (dv.n_selected_locations == 1)));
    vcCmd.Enable (LocationUpdateCmd,    
                  (enable && (dv.n_selected_locations == 1)));
}



/*\Docpage match_type_or_class -- Match object's type or class.
 * 
 * DESCRIPTION
 *      Auxiliary function to the get_obj functions: matches the object's type
 *      or class with the one that the caller wants.  Returns a boolean.
 */
function match_type_or_class(tc, obj, class)
    ((tc == nil) || (tc == class) || (istype (tc, vlTypespecVT) &&
                                      istype (obj, tc)));



/*\Function obj_version -- Return a prettified version string.
 */
function obj_version(obj)
{
    v := vdm.ObjGetVersion (obj);

    if (string_eq (v, vdm.VersLatest))
        v = "Latest";
    else if (string_eq (v, vdm.VersDefault))
        v = "Default";

    v;
}


/*\Function get_one_obj -- Get one selected object of the given type.
 * 
 * DESCRIPTION
 *      An error will be signaled if there is more than one object selected of
 *      the given type.
 */
function get_one_obj(dv, type_or_class, routine)
{
    counter := 0;
    result  := nil;

    foreach (obj, class; vlSym.Loop (dv.selected_table))
        if (match_type_or_class (type_or_class, obj, class))
        {
            ++counter;
            if (counter == 1)
                result = obj;
        }
                

    if (counter != 1)
    {
        str := "item";

        if (type_or_class != nil)
            str = convert (vlStringVT, type_or_class);

        dv_err ("%s: Exactly one %s should be selected, there are %d.",
                routine, str, counter);
    }

    result;
}



/*\Docpage get_selected_objects -- Iterator through the selected objects.
 *
 * DESCRIPTION
 *      Iterates through selected objects of one kind.
 */
iterator get_selected_objects(dv, type_or_class)
{
    foreach (obj, class; vlSym.Loop (dv.selected_table))
        if (match_type_or_class (type_or_class, obj, class))
            iterate (obj);
}



/*\Docpage get_versioned_objects -- Iterator through selected objects.
 *
 * DESCRIPTION
 *      Iterate through selected vdmObjVT, first setting the versions
 *      appropriately in the event that what the user really selected was a
 *      version.
 */
iterator get_versioned_objects(dv)
{
    objs := nil;

    foreach (obj; get_selected_objects (dv, vdmObjVT))
        objs = cons (obj, objs);

    foreach (relative; get_selected_objects (dv, dv.relative_class))
    {
        obj := car (relative);
        if (!member (obj, objs))
            objs = cons (obj, objs);
    }

    foreach (obj; vscript.list_iter (objs))
        iterate (obj);
}


/*****************************************************************************/
/*          This set of commands deals with the search order (Map)           */
/*****************************************************************************/


/*\Docpage re_generate_maps -- Regenerate the search space map.
 */
function re_generate_maps(dv, loc_set)
{
    vpm.MapClearLocations (dv.map);

    foreach (loc; vscript.iter (loc_set))
        vpm.MapAddLocation (dv.map, loc);

    dv.map_modified = TRUE;
}


function current_loc_name(dv)
    vpm.LocationGetAlias (get_one_obj (dv, vpmLocationVT, "LocationEdit"));

function current_loc_storage_spec(dv)
{
    loc := get_one_obj (dv, vpmLocationVT, "LocationEdit");
    list (vpm.LocationGetStorageSpec (loc), vpm.LocationGetString (loc));
}

function current_loc_class_filter(dv)
    vpm.LocationGetClassFilter (get_one_obj (dv, vpmLocationVT,
                                             "LocationEdit"));


/*\Docpage LocationCreate -- Action routine for the LocationCreateCmd.
 * 
 * DESCRIPTION
 *      The storage information comes in as a list of four elements:
 *
 *          (location_string filename package package_name)
 *
 *      It's composed in the _Get method of the sm_select_fac facility.
 *
 * NOTE
 *      The class_filter, as generated by the _Get method of the vcListEdit
 *      facility, returns a list of lists.  We need to flatten that list,
 *      before passing it on to the vpm.LocationCreate function.
 */
function LocationCreate(dv, event, alias, sm_info, class_filter, save_now)
{
    location := vpm.LocationCreate (alias, cadr (sm_info), caddr (sm_info),
                                    car (sm_info), class_filter);

    dv.map_modified = TRUE;
    vpm.MapAddLocation (dv.map, location);
    vcBrowse.ObjInsert (dv.loc_obj_browser, dv.map, location,
                        dv.location_class);

    if (save_now)
        LocationWrite (dv, event);
}


LocationCreateCmd :=
    vcWv.Cmd (fac, LocationCreate, "location_create", "Create New Location",
              vcWv.Param ("Alias", vlStringVT, vcWv.Sticky, "",
                          "Shorthand name for the location"),
              vcWv.Param ("Storage Manager", vscmVT, vcWv.Sticky, nil,
                          sm_select_fac,
                          "Storage manager specification for this location"),
              vcWv.Param ("Class filter", vscmVT, vcWv.Sticky, nil,
                          vcWv.Secondary, class_filter_fac,
                          "List of class types found in this location"),
              vcWv.Param ("Save it right away?", vlYesNoVT, vcWv.Sticky,
                          vlYesNo.No, vcWv.Secondary,
                          "Is the new location temporary or permanent?"));

vcCmd.SetHelp (LocationCreateCmd,
           "Create a new location describing the physical placement of data.");



/*\Function LocationEdit -- Action routine for the LocationEditCmd.
 * 
 * DESCRIPTION
 *      The user has just entered new information about an existing location.
 *      Regenarates the entire location map.
 *
 *      The storage information comes in as a list of four elements:
 *
 *          (location_string filename package package_name)
 *
 *      It's composed in the _Get method of the sm_select_fac facility.
 */
function LocationEdit(dv, event, alias, sm_info, class_filter, save_now)
{
    location := get_one_obj (dv, vpmLocationVT, "LocationEdit");
    loc_list := nil;

    foreach (loc; vpm.MapLoopLocation (dv.map))
    {
        if (string_eq (vpm.LocationGetAlias (loc),
                       vpm.LocationGetAlias (location)))
            loc = vpm.LocationCreate (alias, cadr (sm_info), caddr (sm_info),
                                      car (sm_info), class_filter);

        loc_list = cons (loc, loc_list);
    }

    re_generate_maps (dv, reverse_destructive (loc_list));

    vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, dv.map, FALSE);

    if (save_now)
        LocationWrite (dv, event);
}


LocationEditCmd :=
    vcWv.Cmd (fac, LocationEdit, "location_edit", "Edit Location",
              vcWv.Param ("Alias", vlStringVT, vcWv.Dynamic, current_loc_name,
                          "Shorthand name for the location"),
              vcWv.Param ("Storage Manager", vscmVT, vcWv.Dynamic,
                          current_loc_storage_spec, sm_select_fac,
                          "Storage manager specification for this location"),
              vcWv.Param ("Class filter", vscmVT, vcWv.Dynamic,
                          current_loc_class_filter, vcWv.Secondary,
                          class_filter_fac,
                          "List of class types found in this location"),
              vcWv.Param ("Save it right away?", vlYesNoVT, vcWv.Sticky,
                          vlYesNo.No, vcWv.Secondary,
                          "Is the new location temporary or permanent?"));

vcCmd.SetHelp (LocationEditCmd,
               "Edit the location specification.");



/*\Docpage LocationMove -- Move a location node one position up or down.
 *
 * DESCRIPTION
 *      Common function used by the action routines of the commands that move
 *      a location node up or down.  The direction parameters indicates which
 *      way.
 */
function LocationMove(dv, direction)
{
    location := get_one_obj (dv, vpmLocationVT, "LocationMove");
    loc_list := nil;
    location_index := nil;

    foreach (loc; vpm.MapLoopLocation (dv.map))
    {
        if (loc == location)
            location_index = length (loc_list);
        loc_list = cons (loc, loc_list);
    }

    new_index  := location_index + direction;
    loc_vector := apply (vector, reverse_destructive (loc_list));

    if ((new_index >= 0) && (new_index < vector_length (loc_vector)))
    {
        loc_vector [location_index] = loc_vector[new_index];
        loc_vector [new_index]      = location;
    }

    re_generate_maps (dv, loc_vector);

    location;
}



/*\Docpage LocationMoveUp -- Action routine for the LocationMoveUpCmd.
 * 
 * DESCRIPTION
 *      Moves the location one node up.
 */
function LocationMoveUp(dv, event)
    vcBrowse.ObjMoveUp (dv.loc_obj_browser, LocationMove (dv, -1));


LocationMoveUpCmd := vcWv.Cmd (fac, LocationMoveUp, "location_move_up");
vcCmd.SetHelp (LocationMoveUpCmd,
               "Move the location one level up in the search order.");



/*\Docpage LocationMoveDown -- Action routine for the LocationMoveDown.
 * 
 * DESCRIPTION
 *      Moves the location one node down.
 */
function LocationMoveDown(dv, event)
    vcBrowse.ObjMoveDown (dv.loc_obj_browser, LocationMove (dv, 1));


LocationMoveDownCmd := vcWv.Cmd (fac, LocationMoveDown, "location_move_down");
vcCmd.SetHelp (LocationMoveDownCmd,
               "Move the location one level down in the search order.");



/*\Docpage LocationMoveTop -- Action routine for the LocationMoveTopCmd.
 * 
 * DESCRIPTION
 *      Moves the location to the front of the search space.
 */
function LocationMoveTop(dv, event)
{
    location := get_one_obj (dv, vpmLocationVT, "LocationMoveTop");
    loc_list := nil;

    foreach (loc; vpm.MapLoopLocation (dv.map))
        if (loc != location)
            loc_list = cons (loc, loc_list);

    re_generate_maps (dv, cons (location, reverse_destructive (loc_list)));

    vcBrowse.ObjMoveFirst (dv.loc_obj_browser, location);
}


LocationMoveTopCmd := vcWv.Cmd (fac, LocationMoveTop,
                                  "location_move_top");
vcCmd.SetHelp (LocationMoveTopCmd,
               "Move the location to the first place in the search order.");



/*\Docpage ConvirmLocationRemove -- Confirm dbox for removing locations.
 *
 * DESCRIPTION
 *      Builds up a picklist listing all the locations that are about to be
 *      removed and adds a string asking the user to confirm this action.
 */
function ConfirmLocationRemove(dv, event)
    confirm_op (dv, "The following location definitions will be removed. Ok?",
                FALSE, vpmLocationVT, vpm.LocationGetAlias);



/*\Docpage LocationRemove -- Action routine for the LocationRemoveCmd.
 * 
 * DESCRIPTION
 *      Removes the location from the search space.
 */
function LocationRemove(dv, event)
{
    kill_list := nil;

    foreach (obj, val; vlSym.Loop (dv.selected_table))
        if (istype (obj, vpmLocationVT))
            kill_list = cons (obj, kill_list);

    foreach (obj; vscript.list_iter (kill_list))
    {
        vpm.MapDeleteLocation (dv.map, obj);
        vcBrowse.ObjRemove (dv.loc_obj_browser, obj);
    }

    dv.map_modified = TRUE;
}


LocationRemoveCmd :=
    vcWv.CmdWithConfirm (fac, LocationRemove, ConfirmLocationRemove,
                         TRUE, "location_remove");
vcCmd.SetHelp (LocationRemoveCmd,
               "Remove the location specification from the search order.");



/*\Docpage LocationUpdate -- Action routine for the LocationUpdateCmd.
 * 
 * DESCRIPTION
 *      Rebuild the location information.
 */
function LocationUpdate(dv, event)
{
    location := get_one_obj (dv, vpmLocationVT, "LocationUpdate");

    vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, location, FALSE);
}


LocationUpdateCmd := vcWv.Cmd (fac, LocationUpdate, "location_update");
vcCmd.SetHelp (LocationUpdateCmd,
               "Update the location information and data object statuses.");



/*\Docpage LocationWrite -- Action routine for the LocationWriteCmd.
 * 
 * DESCRIPTION
 *      Writes only the location search space information back to the vpmus
 *      file.
 */
function LocationWrite(dv, event)
{
    LOC_PREFIX      := "(Process 'AddLocation ";
    LOC_PREFIX_LEN  := string_length (LOC_PREFIX);

    function write_location(f)
        foreach (loc; vpm.MapLoopLocation (dv.map))
            fputline (f,
                      sprintf ("%s%s %s %s %s)\n",
                               LOC_PREFIX,
                               obj2string (vpm.LocationGetAlias (loc)),
                               obj2string (vpm.LocationGetStorageSpec (loc)),
                               obj2string (vpm.LocationGetString (loc)),
                               obj2string (vpm.LocationGetClassFilter (loc))));


    function is_location_entry(str)
        ((string_length (str) > LOC_PREFIX_LEN) &&
         string_eqi (substring (str, 0, LOC_PREFIX_LEN), LOC_PREFIX));

    vcWin.ClearStatus (vcEvent.Window (event));

    if (dv.map_modified)
    {
        write_vpmus (is_location_entry, write_location);

        dv.map_modified = FALSE;

        vcWin.AddStatus (vcEvent.Window (event),
                         "Saved the location search space changes.");

    }
    else
        vcWin.AddStatus (vcEvent.Window (event),
                         "No changes were made to the search order.");
}


LocationWriteCmd := vcWv.Cmd (fac, LocationWrite, "location_write");
vcCmd.SetHelp (LocationWriteCmd,
           "Write the search order changes to the vpmus initialization file.");




/****************************************************************************/
/*                       Location class commands.                           */
/****************************************************************************/


/*\Docpage LClassUpdate -- Action routine for the LClassUpdateCmd.
 * 
 * DESCRIPTION
 *      Recalculates the children of a single location class node.  This has
 *      been one of the first enhancement requests, because an update of an
 *      entire location node is too expansive if there are many data objects.
 *
 * CAUTION
 *      The current_lclass field in the facility data structure must be reset
 *      no matter what, so that any subsequent recalculations don't go to the
 *      disk, but instead use the cache.
 */
function LClassUpdate(dv, event)
{
    lclass := get_one_obj (dv, dv.lclass_class, "LClassUpdate");

    dv.current_lclass = cadr (lclass);

    unwind_protect
        vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, lclass, FALSE);
    cleanup
        dv.current_lclass = nil;
}


LClassUpdateCmd := vcWv.Cmd (fac, LClassUpdate, "lclass_update");
vcCmd.SetHelp (LClassUpdateCmd,
               "Update the list of data objects of this class only.");


/****************************************************************************/
/*                          Display commands.                               */
/****************************************************************************/


/*\Docpage show -- Switch the vcBrowse regions.
 * 
 * DESCRIPTION
 *      Used to switch between the data and class browsers.
 */
function show(dv, browser)
{
    dv.current_browser = browser;
    vcOverlay.SetCurrent (dv.overlay, dv.current_browser);

    enable_cmds (dv);
}



/*\Function NewView -- Action routine for the NewViewCmd.
 * 
 * DESCRIPTION
 *      Peel off a new view of the display tree.
 */
function NewView(dv, event)
{
    function NewViewAux(dv, obj, is_on)
        if (is_on)
            vcBrowse.NewWindow (dv.current_browser, obj);

    ShowSelectedAux (dv, event, NewViewAux);
}


NewViewCmd := vcWv.Cmd (fac, NewView, "new_view");
vcCmd.SetHelp (NewViewCmd,
               "Peel off a separate view of the display tree.");



/*\Docpage ShowLocObj -- Action routine for the ShowLocObjCmd.
 * 
 * DESCRIPTION
 *      Switch to the data browser.
 */
function ShowLocObj(dv, event)
    show (dv, dv.loc_obj_browser);


ShowLocObjCmd := vcWv.Cmd (fac, ShowLocObj, "show_loc_obj");
vcCmd.SetHelp (ShowLocObjCmd, "Switch to the data browser display.");



/*\Docpage ShowClasses -- Action routine for the ShowClassesCmd.
 * 
 * DESCRIPTION
 *      Switch to the class browser.
 */
function ShowClasses(dv, event)
    show (dv, dv.class_browser);    


ShowClassesCmd := vcWv.Cmd (fac, ShowClasses, "show_classes");
vcCmd.SetHelp (ShowClassesCmd, "Switch to the class browser display.");


/*\Docpage ShowSelected -- Action routine for the ShowSelectedCmd.
 * 
 * DESCRIPTION
 *      Show all the selected nodes in the current display tree.  The selected
 *      nodes will displayed in a picklist.  A selection made in the picklist
 *      will select the node in the display tree.  If the object is not in the
 *      current overlay, it will be switched.
 */
function ShowSelected(dv, event)
    ShowSelectedAux (dv, event, nil);



/*\Function ShowSelectedAux -- Auxilliary function for showing selected items.
 * 
 * DESCRIPTION
 *      The user can register his or her own action routine which will be
 *      called when an item is selected.  In that case, the routine will
 *      called as follows:
 *
 *          proc (dv, obj, on_or_off)
 */
function ShowSelectedAux(dv, event, action_proc)
{
    function sort_classes(obj_class1, obj_class2)
    {
        class1 := cdr (obj_class1);
        class2 := cdr (obj_class2);
        string_le (convert (vlStringVT, class1), convert (vlStringVT, class2));
    }


    obj_list := nil;
    foreach (obj, class; vlSym.Loop (dv.selected_table))
        obj_list = cons (cons (obj, class), obj_list);

    obj_list = vscript.Sort (obj_list, sort_classes);


    /*
     * If there are no objects selected, display a warning only.  Otherwise,
     * build our little dialog box.
     */
    if (null (obj_list))
    {
        vlErr.Out2 ("ViewData", vlErrSev.Warning, "Nothing is selected.");
        return (nil);
    }


    classes := vcRegion.Create ("Object classes", vcPicklist.fac, nil,
                                rows        <- vlSym.Size (dv.selected_table),
                                columns     <- 1,
                                exclusive   <- TRUE,
                                will_scroll <- TRUE);

    names := vcRegion.Create ("Object names", vcPicklist.fac, nil,
                              rows        <- vlSym.Size (dv.selected_table),
                              columns     <- 1,
                              exclusive   <- TRUE,
                              will_scroll <- TRUE);


    class := nil;    obj     := nil;
    label := nil;    browser := nil;
    color := nil;
    foreach (obj_class; vscript.list_iter (obj_list))
    {
        obj   = car (obj_class);
        class = cdr (obj_class);

        label = vcRegion.Create ("label", vcBlackLabel.fac,
                                 convert (vlStringVT, class));

        if (vcBrowse.ObjIsPresent (dv.current_browser, obj))
            browser = dv.current_browser;
        else
            browser = ((dv.current_browser == dv.loc_obj_browser) ?
                       dv.class_browser : dv.loc_obj_browser);
                
        color = vcBrowse.ObjGetColor (browser, obj);
        vcRegion.Foreground (label, color, TRUE);

        vcPicklist.Add (classes, label, obj);
        name := vcBrowse.ObjGetName (browser, obj, class);
        vcPicklist.Add (names, name, obj);
    }


    classes := vcRegion.Create ("Classes", vcMargin.fac, classes,
                                margin_size <- 2);
    vcRegion.Background (classes, vgColor.Black, TRUE);

    /*
     * Start with the location overlay, because that's the most often used
     * display tree.
     *
     * NOTE: if the caller has specified his or her own action routine, well,
     *       we've got to honor that.
     */
    function select(obj, is_on)
        if (is_on)
        {
            if (action_proc != nil)
                action_proc (dv, obj, is_on);
            else
            {
                if (! vcBrowse.ObjIsPresent (dv.current_browser, obj))
                    show (dv, (dv.current_browser == dv.loc_obj_browser ?
                               dv.class_browser : dv.loc_obj_browser));

                vcBrowse.ObjShow (dv.current_browser, list (obj));
            }
        }

    /*
     * The same selection mechanism applies to both picklists.
     */
    vcPicklist.Notify (classes, select);
    vcPicklist.Notify (names, select);

    ptable := vcTable.Create ("selection", 1, 2);
    vcTable.Add (ptable, classes);
    vcTable.Add (ptable, names);

    ptable = vcTable.Scroll3 ("Selected Items", ptable, ptable, 1, 1, 1,
                              50, 200, 1000, nil);

    button := vcRegion.Create ("buttons", vcButnRow.fac,
                               list (list ("dismiss", "Dismiss",
                                           vcWin.wclose, nil)));

    msg  := "\002Choose one item at a time.";
    text := vcRegion.Create ("Info", vcStaticText.fac, msg,
                             align          <- vgAlign.MiddleCenter,
                             recessed_text  <- TRUE);

    table := vcTable.Create ("Selected Items", 3, 1);
    vcTable.Add (table, text);
    vcTable.Add (table, ptable);
    vcTable.Add (table, button);

    vcWin.Create (vcEvent.Window (event), table, vcWin.NoBindings);
}


ShowSelectedCmd := vcWv.Cmd (fac, ShowSelected, "disp_selected");
vcCmd.SetHelp (ShowSelectedCmd,
               "Show all the currently selected objects.");



/****************************************************************************/
/*                          Select commands.                                */
/****************************************************************************/


/*\Docpage SelectChildren -- Action routine for the SelectChildrenCmd.
 * 
 * DESCRIPTION
 *      Selects all the children of the currently selected nodes.  It also
 *      expands the selected nodes that haven't been expended.  Works only in
 *      the browser that is currently visible in the overlay.
 */
function SelectChildren(dv, event)
{
    function select_children_aux()
    {
        sel_list := nil;

        foreach (obj, class; vlSym.Loop (dv.selected_table))
            sel_list = cons (obj, sel_list);

        foreach (obj; vscript.list_iter (sel_list))
        {
            /*
             * The selected object may be in the browser that is hidden.
             * This operation only takes affect in the visible browser.
             */
            found_object_in_this_vdm := FALSE;

            catch (vlEx.FAILURE)
            {
                vcBrowse.ObjExpand (dv.current_browser, obj);
                found_object_in_this_vdm = TRUE;
            }
            exception
                found_object_in_this_vdm = FALSE;

            if (found_object_in_this_vdm)
                foreach (obj, class; vcBrowse.ObjLoop (dv.current_browser,
                                                       obj))
                    vcBrowse.ObjSelect (dv.current_browser, obj);
        }
    }

    vcWin.WithBusy (vcEvent.Window (event), select_children_aux);
}


SelectChildrenCmd := vcWv.Cmd (fac, SelectChildren, "select_children");
vcCmd.SetHelp (SelectChildrenCmd,
               "Select all the leaves of the currently selected nodes.");



/*\Function SelectRelatives -- Action routine for the SelectRelativesCmd.
 * 
 * DESCRIPTION
 *      Selects object relatives for all the relationships on the currently
 *      selected data object.  Only the data objects are affected.
 */
function SelectRelatives(dv, event)
{
    function select_relatives_aux()
    {
        sel_list := nil;

        foreach (obj; get_versioned_objects (dv))
            foreach (name, type, fanout, is_sticky, relative;
                     vdm.ObjLoopRelationship (obj))
                 foreach (relative; vdm.ObjLoopRelative (obj, name))
                     sel_list = cons (relative, sel_list);

        foreach (obj_relname; get_selected_objects (dv, dv.relationship_class))
            foreach (relative;
                     vdm.ObjLoopRelative (car (obj_relname),
                                          cdr (obj_relname)))
                sel_list = cons (relative, sel_list);

        foreach (obj; vscript.list_iter (sel_list))
        {
            vcBrowse.ObjExpand (dv.loc_obj_browser, vdm.ObjGetLocation (obj));

            foreach (loc_class, c; vcBrowse.ObjLoop (dv.loc_obj_browser,
                                                     vdm.ObjGetLocation (obj)))
                if (cdr (loc_class) == vdm.ObjGetClass (obj))
                    vcBrowse.ObjExpand (dv.loc_obj_browser, loc_class);
            vcBrowse.ObjSelect (dv.loc_obj_browser, obj);
        }
    }

    vcWin.WithBusy (vcEvent.Window (event), select_relatives_aux);
}


SelectRelativesCmd := vcWv.Cmd (fac, SelectRelatives, "select_relatives");
vcCmd.SetHelp (SelectRelativesCmd,
               "Select relatives of the currently selected data objects.");



/****************************************************************************/
/*                          Object commands.                                */
/****************************************************************************/


/*\Docpage update_collection_members -- Update collection member nodes.
 * 
 * DESCRIPTION
 *      Called from most of the storage management commands.  The object is
 *      assumed to be a collection: its members are updated.
 */
function update_collection_members(dv, obj)
    foreach (member; vdm.CollLoopMember (obj))
        if (vcBrowse.ObjIsPresent (dv.loc_obj_browser, member))
            vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, member, TRUE);



/*\Function update_version_parent -- Update a versioned object's parent.
 * 
 * DESCRIPTION
 *      When an SM operation is performed on a versioned object, update the
 *      status information of the parent node.  This is a temporary fix to the
 *      problem of version object status that will have a better fix in the
 *      next rev.
 */
function update_version_parent(dv, obj)
{
    parent := vcBrowse.ObjGetParent (dv.loc_obj_browser, obj);
    if (istype (parent, vdmObjVT))
    {
        vdm.ObjPurgeStatus (parent);
        vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, parent, TRUE);
    }
}



/*\Docpage perfom_sm_action -- Do the SM operation.
 * 
 * DESCRIPTION
 *      The SM operations are all similar to each other, so this function
 *      is common to them all.  Note the separation of the SM operation with
 *      visual update.
 */
function perform_sm_action(action, dv, args...)
{
    result := nil;

    /*
     * First, peform the action for each selected data object, including the
     * version objects.
     */
    foreach (obj; get_versioned_objects (dv))
    {
        result = apply (action, cons (obj, args));

        if (vdm.ObjIsCollection (obj))
        {
            dv.current_collection = obj;
            update_collection_members (dv, obj);
        }
    }

    /*
     * Second, update the visuals in one swoop, so there's not so much
     * flashing.
     */
    foreach (obj; get_versioned_objects (dv))
    {
        vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, obj, TRUE);

        update_version_parent (dv, obj);
        enable_cmds (dv);
    }

    result;
}



/*\Docpage EditObj -- Action routine for the EditObjCmd.
 * 
 * DESCRIPTION
 *      Invokes the data object's editor.
 */
function EditObj(dv, event)
    /*
     * Check out the obj_edit function which defines an exit function called
     * when the editing is over.
     */
    vcBrowse.ObjEdit (dv.loc_obj_browser, get_one_obj (dv, nil, "EditObj"));


EditObjCmd := vcWv.Cmd (fac, EditObj, "edit");

vcCmd.SetHelp (EditObjCmd, "Edit the selected data object.");



/*\Docpage NewObject -- Action routine for the NewObjectCmd.
 * 
 * DESCRIPTION
 *      Instantiates a new data object and inserts it in the display tree.  If
 *      the vdata.edit_new preference is set to TRUE, it automatically invokes
 *      the editor on the object.
 */
function NewObject(dv, event, object_info, loc)
{
    class := car (object_info);
    obj   := vdm.ObjCreate ("", vdm.VersLatest, class, loc, dv.cntx,
                            cadr (object_info));

    InsertNewObjectAux (dv, event, loc, class, obj);

    if (vcRegion.PrefGet (dv.region, "edit_new"))
        EditObj (dv, event);
}


NewObjectCmd :=
    vcWv.Cmd (fac, NewObject, "new", "New Data",
              vcWv.Param ("Data class", vscmVT, vcWv.Dynamic,
                          GetDefaultContext, ClassSelectFac,
                          "Data's type and attributes",
                          class_filter <- nil),
              vcWv.Param ("Location", vscmVT, vcWv.Dynamic,
                          GetDefaultBrowseData, vdmLocSelect.fac,
                          "Location where the object will be placed"));

vcCmd.SetHelp (NewObjectCmd, "Instantiate a new data object");



/*\Docpage Checkout -- Action routine for the CheckoutCmd.
 * 
 * DESCRIPTION
 *      Checks out all the selected objects.
 *
 * NOTE
 *      Because this function might take a while, the vcWin.WithBusy is used
 *      to give user a visual indication of a pending process.
 */
function Checkout(dv, event)
    verify_versions (dv, event, "check out", vdm.ObjCheckoutLock);


CheckoutCmd := vcWv.Cmd (fac, Checkout, "checkout");
vcCmd.SetHelp (CheckoutCmd,
               "Fetch the selected data from storage and lock it.");



/*\Docpage shallow_checkout_no_lock() -- Checkout an object.
 * 
 * DESCRIPTION
 *      Fetch an object, but if it is a collection, don't get its members.
 */
function shallow_checkout_no_lock(obj)
{
    pdb     := vlPref.Create ("vdm");
    old_val := vlPref.Val (pdb, "shallow_colls");

    unwind_protect
    {
        vlPref.Setval (pdb, "shallow_colls", TRUE);
        vdm.ObjCheckoutNoLock (obj);
    }
    cleanup
        vlPref.Setval (pdb, "shallow_colls", old_val);
}



/*\Docpage CheckoutNoLock -- Action routine for the CheckoutNoLockCmd.
 * 
 * DESCRIPTION
 *      Fetches all the selected objects.
 *
 * NOTE
 *      Because this function might take a while, the vcWin.WithBusy is used
 *      to give user a visual indication of a pending process.
 */
function CheckoutNoLock(dv, event)
    verify_versions (dv, event, "check out no lock", vdm.ObjCheckoutNoLock);


CheckoutNoLockCmd := vcWv.Cmd (fac, CheckoutNoLock, "checkout_no_lock");
vcCmd.SetHelp (CheckoutNoLockCmd,
               "Fetch the selected data from storage, but DO NOT lock it.");



/*\Docpage CreateCheckpoint -- Action routine for the CreateCheckpointCmd.
 * 
 * DESCRIPTION
 *      Checkpoints all the collection members, i.e. makes sure that the
 *      collection object knows about all the versions of its members.
 *
 * NOTE
 *      Because this function might take a while, the vcWin.WithBusy is used
 *      to give user a visual indication of a pending process.
 */
function CreateCheckpoint(dv, event, update, lock_on_modified, cleanup_type,
                          tag, log_message)
{
    declare proc;

    function checkout(obj, action_type)
        vdm.ObjCheckoutLock (obj);

    function checkout_nolock(obj, action_type)
        vdm.ObjCheckoutNoLock (obj);


    function checkpoint_aux()
    {
        /*
         * Unspecified tag needs to be turned into nil.
         */
        if (string_eq (tag, ""))
            tag = nil;

        <result, problem_obj> :=
            perform_sm_action (vdm.ObjCreateCheckpoint, dv, update,
                               lock_on_modified, log_message, tag,
                               nil, proc, current_package ());

        if (result != vdm.CheckpointStatus.Success)
            vlErr.Out2 ("ViewData", vlErrSev.Warning,
               sprintf ("Checkpoint operation stopped at the selected object.
\tThe exit status was: %s.", result));

        SelectObject (dv.region, problem_obj);
    }

    if (string_eq (log_message, ""))
        dv_err ("The log message cannot be an empty string.");

    if (cleanup_type == vdmCheckpointCleanup.CheckOut)
        proc = checkout;
    else if (cleanup_type == vdmCheckpointCleanup.CheckOutNoLock)
        proc = checkout_nolock;

    vcWin.WithBusy (vcEvent.Window (event), func() checkpoint_aux (););
}


CreateCheckpointCmd :=
    vcWv.Cmd (fac, CreateCheckpoint, "checkpoint", "Checkpoint",
              vcWv.Param ("Update", vlTrueFalseVT, vcWv.Sticky,
                          vlTrueFalse.False,
                          "Update members with the Empty status?"),
              vcWv.Param ("Lock on Modified", vlTrueFalseVT, vcWv.Sticky,
                          vlTrueFalse.False,
                          "Attempt to lock and check in modified members?"),
              vcWv.Param ("After a check in", vdmCheckpointCleanupVT,
                          vcWv.Sticky, vdmCheckpointCleanup.LeaveInStorage,
                          "What to do after a member is checked in"),
              vcWv.Param ("Tag", vlStringVT, vcWv.Sticky, "", vcWv.Secondary,
                          "Symbolic tag to identify this checkpoint"),
              vcWv.Param ("Log message", vlStringVT, vcWv.Sticky, "",
                          vtx.fac, height <- 3, width <- 80));

vcCmd.SetHelp (CreateCheckpointCmd,
               "Find out the versions of all the collection members.");



/*\Docpage RestoreCheckpoint -- Action routine for the RestoreCheckpointCmd.
 * 
 * DESCRIPTION
 *      Restore a checkpointed collection, i.e. fetch the exact versions of
 *      the collection members from storage.
 *
 * NOTE
 *      Because this function might take a while, the vcWin.WithBusy is used
 *      to give user a visual indication of a pending process.
 */
function RestoreCheckpoint(dv, event, lock, restore_on_modify, tag)
{
    if (string_eq (tag, ""))
        tag = nil;

    vcWin.WithBusy (vcEvent.Window (event),
                    func()
                    perform_sm_action (vdm.ObjRestoreCheckpoint, dv,
                                       lock, restore_on_modify, tag););
}


RestoreCheckpointCmd :=
    vcWv.Cmd (fac, RestoreCheckpoint, "restore", "Restore Checkpoint",
              vcWv.Param ("Lock", vlTrueFalseVT, vcWv.Sticky,
                          vlTrueFalse.False,
                          "Whether to lock collection members upon restore"),
              vcWv.Param ("Restore on modify", vlTrueFalseVT, vcWv.Sticky,
                          vlTrueFalse.False,
           "Whether to restore members that are modified in the working area"),
              vcWv.Param ("Tag", vscmVT, vcWv.Dynamic, GetDv,
                          vcWv.Secondary, VersionSelectFac,
                          "Restore this tagged version"));

vcCmd.SetHelp (RestoreCheckpointCmd,
               "Restore the versions of all the collection members.");



/*\Docpage Checkin -- Action routine for the CheckinCmd.
 * 
 * DESCRIPTION
 *      Checks in all the selected objects.  If the status of the object is
 *      Uncontrolled, the object is first registered.
 *
 * NOTE
 *      Because this function might take a while, the vcWin.WithBusy is used
 *      to give user a visual indication of a pending process.
 */
function Checkin(dv, event, log_message)
{
    if (string_eq (log_message, ""))
        dv_err ("The log message cannot be an empty string.");

    vcWin.WithBusy (vcEvent.Window (event),
                    func()
                    perform_sm_action (vdm.ObjCheckin, dv, log_message););
}


CheckinCmd :=
    vcWv.Cmd (fac, Checkin, "checkin", "Object Check In",
              vcWv.Param ("Log message", vlStringVT, vcWv.Sticky, "",
                          vtx.fac, height <- 3, width <- 80));
vcCmd.SetHelp (CheckinCmd,
              "Create a new version in storage of the selected data objects.");



/*\Docpage Lock -- Action routine for the LockCmd.
 * 
 * DESCRIPTION
 *      Locks all the selected objects.
 *
 * NOTE
 *      Because this function might take a while, the vcWin.WithBusy is used
 *      to give user a visual indication of a pending process.
 */
function Lock(dv, event)
    verify_versions (dv, event, "lock", vdm.ObjLock);


LockCmd := vcWv.Cmd (fac, Lock, "lock");
vcCmd.SetHelp (LockCmd,
               "Lock the selected objects so that nobody else can modify it.");



/*\Docpage Unlock -- Action routine for the UnlockCmd.
 * 
 * DESCRIPTION
 *      Unlocks all the selected objects.
 *
 * NOTE
 *      Because this function might take a while, the vcWin.WithBusy is used
 *      to give user a visual indication of a pending process.
 */
function Unlock(dv, event)
    vcWin.WithBusy (vcEvent.Window (event),
                    func()
                    perform_sm_action (vdm.ObjUnlock, dv););


UnlockCmd := vcWv.Cmd (fac, Unlock, "unlock");
vcCmd.SetHelp (UnlockCmd,
           "Unlock the selected objects so that somebody else can modify it.");



/*\Docpage confirm_op -- Confirm an operation about to take place.
 * 
 * DESCRIPTION
 *      Builds a picklist of the selected objects and asks the users to
 *      confirm that a given operation can be performed on all of them.
 */
function confirm_op(dv, msg, want_objects, type, name_proc)
{
    result  := FALSE;
    objects := nil;

    if (want_objects)
        foreach (obj; get_versioned_objects (dv))
            objects = cons (vdm.ObjGetName (obj), objects);
    else
        foreach (obj; get_selected_objects (dv, type))
            objects = cons (((name_proc == nil) ? obj.name : name_proc (obj)),
                            objects);

    if (objects != nil)
    {
        object_table := vcTable.Create ("", length (objects), 1);
        vcTable.Col (object_table, 0, vcTable.Border.Invisible, 3, 3, 3,
                     vgAlign.LowerLeft);

        foreach (obj; vscript.list_iter (objects))
            vcTable.Add (object_table, vcRegion.Create ("", vcLabel.fac, obj));

        scroll := vcTable.Scroll ("Scroll", object_table, object_table,
                                  3, 3, 0);

        stat  := vcRegion.Create ("Stat", vcStaticText.fac, msg);
        result = vcTable.Make ("Table", 2, 1, stat, scroll);
    }

    result;
}


/*\Docpage ConfirmDelete -- Build a confirmation table for DeleteCmd.
 * 
 * DESCRIPTION
 *      Builds a picklist of the selected objects that are about to be deleted
 *      together with a message asking the user to confirm it.  It's used by
 *      the DeleteCmd.
 */
function ConfirmDelete(dv, event)
    confirm_op (dv, "The following objects will be deleted.  OK?",
                TRUE, nil, nil);



/*\Docpage Delete -- Action routine for the Object -> Delete command.
 * 
 * DESCRIPTION
 *      Deletes all the selected objects if the user confirms it.
 */
function Delete(dv, event)
{
    kill := nil;

    foreach (obj; get_versioned_objects (dv))
        kill = cons (obj, kill);

    foreach (obj; vscript.list_iter (kill))
    {
        vdm.ObjDelete (obj);
        vcBrowse.ObjRemove (dv.loc_obj_browser, obj);
    }
}


DeleteCmd := vcWv.CmdWithConfirm (fac, Delete, ConfirmDelete, TRUE, "delete",
                                  "Delete Objects");
vcCmd.SetHelp (DeleteCmd,
               "Permanently delete the selected objects from storage.");



/*\Docpage Info -- Action routine for the InfoCmd.
 * 
 * DESCRIPTION
 *      Instantiates the vdmInfo facility on the list of the selected objects.
 */
function Info(dv, event)
{
    tx := vcRegion.Create ("log", vtx.fac, "vdm_info.log",
                           load_file    <- FALSE,
                           scratch      <- TRUE);
    cancelled := FALSE;
    function cancel() cancelled = TRUE;

    buttons := vcRegion.Create ("buttons", vcButnRow.fac,
                                list (list ("cancel", "Cancel", cancel,
                                            nil),
                                      list ("dismiss", "Dismiss",
                                            vcWin.wclose, nil)));

    vcRegion.Enable (vcButnRow.Get (buttons, 1), FALSE, FALSE);
    w := vcWin.Create (vcEvent.Window (event),
                       vcTable.Make ("Object Log", 2, 1, tx, buttons));
    
    foreach (obj; get_versioned_objects (dv))
    {
        vcWin.ClearStatus (w);
        vcWin.AddStatus (w, sprintf ("Getting log information for %s",
                                     vdm.ObjGetName (obj)));

        vdm.ObjQuery (obj, TRUE, func(str) vtx.AppendString (tx, str););

        vtx.AppendString (tx, "\n\n");
        vcWv.ProcessPending (wv);

        if (cancelled)
            break;
    }

    vcRegion.Enable (vcButnRow.Get (buttons, 0), FALSE, TRUE);
    vcRegion.Enable (vcButnRow.Get (buttons, 1), TRUE, TRUE);
}


InfoCmd := vcWv.Cmd (fac, Info, "info");
vcCmd.SetHelp (InfoCmd,
               "Get more information about the selected data objects.");



/****************************************************************************/
/*                          Object extras.                                  */
/****************************************************************************/


/*\Docpage SetTag -- Action routine for the SetTagCmd.
 * 
 * DESCRIPTION
 *      Sets a tag on the selected objects.
 *      
 * NOTE
 *      Because this function might take a while, the vcWin.WithBusy is used
 *      to give user a visual indication of a pending process.
 */
function SetTag(dv, event, tag)
{
    if (string_eq (tag, ""))
        dv_err ("Tag cannot be an empty string.");

    vcWin.WithBusy (vcEvent.Window (event),
                    func()
                        perform_sm_action (vdm.ObjSetTag, dv, tag););
}


SetTagCmd :=
    vcWv.Cmd (fac, SetTag, "settag", "Object Set Tag",
              vcWv.Param ("Tag", vlStringVT, vcWv.Sticky, ""));

vcCmd.SetHelp (SetTagCmd,
              "Set a tag on the selected data objects.");



/*\Docpage ObjectUpdate -- Action routine for the ObjectUpdateCmd.
 *
 * DESCRIPTION
 *      Similar to the Location -> Update and Class -> Update commands, but
 *      works only on the selected objects.  If a particular class has many
 *      data instances, a Class -> Update command might take a while, whereas
 *      this one is quite fast.
 */
function ObjectUpdate(dv, event)
    vcWin.WithBusy (vcEvent.Window (event),
                    func()
                    perform_sm_action (vdm.ObjPurgeStatus, dv););

ObjectUpdateCmd := vcWv.Cmd (fac, ObjectUpdate);

vcCmd.SetHelp (ObjectUpdateCmd,
               "Force recalculation of the object's working & storage status");



/*\Docpage WorkOrStorage -- Action routine for the WorkOrStorageCmd.
 *
 * DESCRIPTION
 *      Where to get the data listing?  Beside storing the user's choice,
 *      check also if a location or a location class node is selected.  If so,
 *      recalculate them automatically to present the data objects that
 *      reflect the user's choice.
 */
function WorkOrStorage(dv, event, choice)
{
    dv.work_or_storage = choice;

    if (dv.n_selected_locations > 0)
        foreach (loc; get_selected_objects (dv, dv.location_class))
            vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, loc, FALSE);

    if (dv.n_selected_lclasses > 0)
        unwind_protect
            foreach (lclass; get_selected_objects (dv, dv.lclass_class))
            {
                dv.current_lclass = cadr (lclass);
                vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, lclass, FALSE);
            }
        cleanup
            dv.current_lclass = nil;
}


WorkOrStorageCmd :=
    vcWv.Cmd (fac, WorkOrStorage, "work_or_storage", "Work Or Storage",
              vcWv.Param ("Loop area", vdmWorkOrStorageVT, vcWv.Sticky,
                          vdmWorkOrStorage.Both,
                          "Which area to use when listing data object?"));

vcCmd.SetHelp (WorkOrStorageCmd,
           "List data object from either the storage, working or both areas?");



/****************************************************************************/
/*                          Attribute commands.                             */
/****************************************************************************/


/*\Docpage ConfirmAddAttribute -- Build a confirmation table for AddAttribute.
 * 
 * DESCRIPTION
 *      Builds a picklist of the selected objects that are about to have a new
 *      attribute added, together with a message asking the user to confirm
 *      it.  It's used by the AddAttributeCmd.
 */
function ConfirmAddAttribute(dv, event)
    confirm_op (dv, "The following objects will have the new attribute added.
 Is it Ok?", TRUE, nil, nil);



/*\Docpage AddAttribute -- Action routine for the AddAttributeCmd.
 * 
 * DESCRIPTION
 *      Adds the specified attribute for each selected object.
 */
function AddAttribute(dv, event, name, attr_val, sticky)
{
    last_obj := nil;

    foreach (obj; get_versioned_objects (dv))
    {
        last_obj = obj;
        vdm.ObjSetAttr (obj, name, attr_val, sticky);
    }

    vcBrowse.ObjSelect (dv.loc_obj_browser, last_obj);
}


AddAttributeCmd :=
    vcWv.CmdWithConfirm (fac, AddAttribute, ConfirmAddAttribute,
                         TRUE, "add_attribute", "Add Attribute",
                         vcWv.Param ("Attribute name", vlStringVT,
                                     vcWv.Sticky, ""),
                         vcWv.Param ("Attribute value", vlStringVT,
                                     vcWv.Sticky, ""),
                         vcWv.Param ("Permanent", vlOnOffVT, vcWv.Sticky,
                                     vlOnOff.Off, vcWv.Secondary,
                                     "Is the attribute permanent?"));
vcCmd.SetHelp (AddAttributeCmd,
               "Add an attribute to the selected data objects.");



/*\Docpage EditAttributes -- Action routine for the EditAttributesCmd.
 * 
 * DESCRIPTION
 *      Uses the vcListEdit facility to allow the user to add/delete/edit
 *      data attributes. 
 */
function EditAttributes(dv, event, only_common)
{
    common_attributes := nil;

    foreach (obj; get_versioned_objects (dv))
    {
        if (common_attributes == nil)
        {
            common_attributes = vlSym.StrCreate (11);
            vlSym.Dynamic (common_attributes, 2.0, 2.0);

            foreach (key, val, sticky; attr_filter (vdm.ObjLoopAttr, obj))
                common_attributes[key] = cons (val, sticky);
        }
        else if (only_common)
        {
            kill_keys := nil;

            foreach (key, val_sticky; vlSym.Loop (common_attributes))
            {
                val := vdm.ObjGetAttr (obj, key);

                if ((val == nil) ||
                    !string_eq (convert (vlStringVT, val),
                                convert (vlStringVT, car (val_sticky))))
                    kill_keys = cons (key, kill_keys);
            }

            foreach (key; vscript.list_iter (kill_keys))
                vlSym.Remove (common_attributes, key);
        }
        else
        {
            foreach (key, val, sticky; attr_filter (vdm.ObjLoopAttr, obj))
                common_attributes [key] = cons (val, sticky);
        }
    }

    le := vcRegion.Create ("atts", vcListEdit.fac, nil,
                           rows <- vlSym.Size (common_attributes),
                           cmd  <- AddAttributeCmd);

    foreach (key, val_sticky; vlSym.Loop (common_attributes))
    {
        vcListEdit.Add (le, key);

        val := car (val_sticky);
        vcListEdit.Add (le, val);
        vcListEdit.Add (le, cdr (val_sticky));
    }


    function install(event)
    {
        declare last_obj, required_attr;

        foreach (obj; get_versioned_objects (dv))
        {
            required_attr = vdm.ClassGetAttrList (vdm.ObjGetClass (obj));

            foreach (key, val_sticky; vlSym.Loop (common_attributes))
                if (! member (key, required_attr))
                    vdm.ObjRemoveAttr (obj, key);

            declare key, val, sticky;

            for (row := 0; row < vcListEdit.NRows (le); ++row)
            {
                key = vcListEdit.Get (le, row, 0);
            
                if (! member (key, required_attr))
                {
                    val    = vcListEdit.Get (le, row, 1);
                    sticky = vcListEdit.Get (le, row, 2);

                    vdm.ObjSetAttr (obj, key, val, sticky);
                }
            }

            last_obj = obj;


            if (last_obj != nil)
                vcBrowse.ObjSelect (dv.loc_obj_browser, last_obj);
        }
    }


    buttons := vcRegion.Create ("Buttons", vcButnRow.fac,
                                list (list ("ok", "OK", install,
                                            list (close_window <- TRUE)),
                                      list ("cancel", "Cancel",
                                            vcWin.wclose, nil)));

    vcWin.Create (vcEvent.Window (event),
                  vcTable.Make ("Edit Attributes", 2, 1, le, buttons));
}


EditAttributesCmd :=
    vcWv.Cmd (fac, EditAttributes, "edit_attributes", "Edit Attribute",
             vcWv.Param ("Only edit attributes common to all selected objects",
                         vlTrueFalseVT, vcWv.Sticky, TRUE));
vcCmd.SetHelp (EditAttributesCmd,
               "Edit existing attributes of the selected data objects.");


/****************************************************************************/
/*                          Collection commands.                            */
/****************************************************************************/


function get_dv()
{
    foreach (w; vcWin.Loop ())
        if ((r := vcRegion.FindFac (vcWin.OwnerRegion (w), fac)) != nil)
            return (vcRegion.GetData (r));

    dv_err ("Cannot find ViewData window.");
}



/*\Docpage is_collection_composable -- Can  this collection be composed?
 */
function is_collection_composable(obj)
    (vdm.ClassGetComposeFunc (vdm.ObjGetClass (obj)) != nil);



/*\Docpage AddToClassFilter -- Action routine for the AddToClassFilterCmd.
 *
 * DESCRIPTION
 *      When a new collection is created and the location where it has been
 *      instantiated has a class filter, the user is asked whether to add the
 *      collection's class type to the filter.  The is the action routine when
 *      the user clicks the Ok button.
 *
 * SEE ALSO
 *      ConfirmAddToClassFilter
 */
function AddToClassFilter(dv, event, class_name, location)
{
    vpm.LocationSetClassFilter (location,
                                cons (class_name,
                                      vpm.LocationGetClassFilter (location)));
    dv.map_modified = TRUE;
}



/*\Docpage ConfirmAddToClassFilter -- Confirm function for AddToClassFilterCmd.
 *
 * DESCRIPTION
 *      Returns a string which prompts a user whether to add the collection's
 *      class type to the class filter for the location where the collection
 *      has been instatiated.
 *
 * SEE ALSO
 *      AddToClassFilter
 */
function ConfirmAddToClassFilter(dv, event, class_name, location)
    "Collection class " + class_name + " is not part of the class filter " +
    "for the " + vpm.LocationGetAlias (location) + " location\n\n" +
    "Do you want to add it to the class filter?";


AddToClassFilterCmd := 
    vcWv.CmdWithConfirm (fac, AddToClassFilter, ConfirmAddToClassFilter, TRUE,
                         "add_to_class_filter", "Add To Class Filter",
                         vcWv.Param ("Collection class", vlStringVT,
                                     vcWv.Sticky, ""),
                         vcWv.Param ("Location", vscmVT, vcWv.Sticky, nil));



/*\Docpage CreateCollection -- Action routine for the CreateCollectionCmd.
 * 
 * DESCRIPTION
 *      Creates a new collection object.
 */
function CreateCollection(dv, event, collection_info, loc)
{
    config_class := car (collection_info);
    collection   := vdm.ObjCreate ("", vdm.VersLatest, config_class,
                                   loc, dv.cntx, cadr (collection_info));


    foreach (obj; get_versioned_objects (dv))
        vdm.CollAddMember (collection, obj, vdm.MetaAction.Now);

    dv.current_collection = collection;

    InsertNewObjectAux (dv, event, loc, config_class, collection);
}



/*\Docpage is_class_a_collection -- Is this a collection class?
 */
function is_class_a_collection(class)
    ((vdm.ClassGetType (class) == vdmClassType.Composite) &&
     (vdm.ClassGetCollectionMembers (class) != nil));



CreateCollectionCmd :=
    vcWv.Cmd (fac, CreateCollection, "create_collection", "Create Collection",
              vcWv.Param ("Collection class", vscmVT, vcWv.Dynamic,
                          GetDefaultContext, ClassSelectFac,
                          "Collection's type and attributes",
                          class_filter <- is_class_a_collection),
              vcWv.Param ("Location", vscmVT, vcWv.Dynamic,
                          GetDefaultBrowseData, vdmLocSelect.fac,
                          "Location where the collection will be placed"));

vcCmd.SetHelp (CreateCollectionCmd, "Create a new collection object.");



/*\Docpage get_one_collection -- Get a single selected collection object.
 */
function get_one_collection(dv, composable_only)
{
    collection := nil;

    if (dv.n_selected_collections > 1)
        dv_err ("More than one collection selected.");

    foreach (obj; get_versioned_objects (dv))
        if (is_class_a_collection (vdm.ObjGetClass (obj)) &&
            (!composable_only || is_collection_composable (obj)))
        {
            if (collection != nil)
                dv_err ("More than one collection selected.");
            else
                collection = obj;
        }

    if (collection == nil)
        dv_err ("No collection objects are selected");


    collection;
}

/*\Docpage ShowSelected -- Action routine for the ShowSelectedCmd.
 * 
 * DESCRIPTION
 *      Show all the selected nodes in the current display tree.  The selected
 *      nodes will displayed in a picklist.  A selection made in the picklist
 *      will select the node in the display tree.  If the object is not in the
 *      current overlay, it will be switched.
 */



/*\Docpage AddToCollection -- Action routine for the AddToCollectionCmd.
 * 
 * DESCRIPTION
 *      Adds the selected objects to the collection.
 */
function AddToCollection(dv, event)
{
    if (dv.n_selected_collections == 1)
    {
        collection := get_one_collection (dv, FALSE);
        AddToCollectionAux (dv, event, collection);
    }
    else if (dv.n_selected_collections > 1)
        vcCmd.Run (AddToCollectionAuxCmd, dv, event, nil, TRUE);            
    else
        dv_err ("No collection objects are selected");
}


function AddToCollectionAux(dv, event, collection)
{
    foreach (obj; get_versioned_objects (dv))
        if ((obj != collection) && !vdm.CollHasMember (collection, obj))
            vdm.CollAddMember (collection, obj, vdm.MetaAction.Now);

    dv.current_collection = collection;

    vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, collection, TRUE);
    vcBrowse.ObjSelect (dv.loc_obj_browser, collection);
    enable_cmds (dv);
}

AddToCollectionCmd := vcWv.Cmd (fac, AddToCollection, "add_to_collection",
    "Add To Collection");
vcCmd.SetHelp (AddToCollectionCmd,
               "Add new entries to an existing collection.");


AddToCollectionAuxCmd :=
    vcWv.Cmd (fac, AddToCollectionAux, "add_to_collection_aux",
              "Add To Collection",
              vcWv.Param ("", vscmVT, vcWv.Dynamic, GetDv,
                          OneCollectionSelectFac));



/*\Docpage RemoveFromCollection -- Action routine for RemoveFromCollectionCmd.
 * 
 * DESCRIPTION
 *      Removes the selected members from the collection.
 */
function RemoveFromCollection(dv, event)
{
    if (dv.n_selected_collections == 1)
    {
        collection := get_one_collection (dv, FALSE);
        RemoveFromCollectionAux (dv, event, collection);
    }
    else if (dv.n_selected_collections > 1)
        vcCmd.Run (RemoveFromCollectionAuxCmd, dv, event, nil, TRUE);
    else
        dv_err ("No collection objects are selected");
}


function RemoveFromCollectionAux(dv, event, collection)
{
    foreach (obj; get_versioned_objects (dv))
        if ((obj != collection) && vdm.CollHasMember (collection, obj))
            vdm.CollRemoveMember (collection, obj, vdm.MetaAction.Now);

    vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, collection, TRUE);
    vcBrowse.ObjSelect (dv.loc_obj_browser, collection);
    enable_cmds (dv);
}


RemoveFromCollectionCmd :=
    vcWv.Cmd (fac, RemoveFromCollection, "remove_from_collection",
              "Remove From Collection");
vcCmd.SetHelp (RemoveFromCollectionCmd, "Remove entries from a collection.");


RemoveFromCollectionAuxCmd :=
    vcWv.Cmd (fac, RemoveFromCollectionAux, "remove_from_collection_aux",
              "Remove From Collection",
              vcWv.Param ("", vscmVT, vcWv.Dynamic, GetDv,
                          OneCollectionSelectFac));



/*\Docpage ComposeCollection -- Action routine for the ComposeCollectionCmd.
 * 
 * DESCRIPTION
 *      Rebuilds the collection object.
 */
function ComposeCollection(dv, event)
{
    if (dv.n_selected_compcolls == 1)
    {
        collection := get_one_collection (dv, TRUE);
        ComposeCollectionAux (dv, event, collection);
    }
    else if (dv.n_selected_compcolls > 1)
        vcCmd.Run (ComposeCollectionAuxCmd, dv, event, nil, TRUE);
    else
        dv_err ("No collection objects are selected.");
}


ComposeCollectionCmd := vcWv.Cmd (fac, ComposeCollection, "compose_collection",
    "Compose Collection");
vcCmd.SetHelp (ComposeCollectionCmd, "Rebuild a collection object.");



/*\Docpage ComposeCollectionAux -- Auxilliary action routine.
 *
 * DESCRIPTION
 *      This is the action routine that actually composes the collection.  It
 *      can be called from two places.  One is from the ComposeCollection
 *      function when there's only one collection selected.  If more than one
 *      collection is selected, the ComposeCollectionAuxCmd is run and the
 *      user is presented with a dialog box asking to be more specific and
 *      choose only one collection to work with.  When the user hits the Ok
 *      button in that dialog box, this function is called.
 */
function ComposeCollectionAux(dv, event, collection)
{
    function aux()
    {
        vdm.CollCompose (collection);

        dv.current_collection = collection;

        vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, collection, TRUE);
        vcBrowse.ObjSelect (dv.loc_obj_browser, collection);
        enable_cmds (dv);
    }

    vcWin.WithBusy (vcEvent.Window (event), aux);
}


ComposeCollectionAuxCmd := vcWv.Cmd (fac, ComposeCollectionAux,
    "compose_collection_aux", "Compose Collection",
    vcWv.Param ("", vscmVT, vcWv.Sticky, nil, OneCollectionSelectFac,
                composable_only <- TRUE));



/*\Docpage DryRun -- Action routine for the DryRunCmd.
 * 
 * DESCRIPTION
 *      Toggles the dry run mode flag on the VDM session context.
 */
function DryRun(dv, event, new_state)
    vdm.CntxSetDryRunMode (dv.cntx, new_state);


DryRunCmd := vcWv.Cmd (fac, DryRun, "dry_run", "Dry Run Mode",
                       vcWv.Param ("Mode", vlOnOffVT, vcWv.Sticky, FALSE));

vcCmd.SetHelp (DryRunCmd,
     "Try operations w/o making permanent changes to the storage area");



/****************************************************************************/
/*                          Relationship commands.                          */
/****************************************************************************/


/*\Docpage object_spare_relations -- Get unused relations.
 *
 * DESCRIPTION
 *      List the relations that an object's class has, that are not already
 *      instantiated on the object.
 */
function object_spare_relations(obj)
{
    class := vdm.ObjGetClass (obj);
    relations := nil;

    foreach (name, type, fanout, class; vdm.ClassLoopRelation (class))
        if (!vdm.ObjHasRelationship (obj, name))
            relations = cons (name, relations);

    relations;
}



/*\Docpage obj_selected_has_spare_relation -- Does it have spare relation?
 * 
 * DESCRIPTION
 *      Used by the select_handler to enable the AddRelationshipCmd.
 */
function obj_selected_has_spare_relation(dv)
{
    obj := get_one_obj (dv, vdmObjVT, "AddRelationship");
    (object_spare_relations (obj) != nil);
}



/*\Docpage selected_relationships -- Iterate over selected relationships.
 */
iterator selected_relationships(dv)
{
    relationships := nil;

    foreach (obj_relname; get_selected_objects (dv, dv.relationship_class))
        relationships = cons (obj_relname, relationships);

    foreach (relationship; vscript.list_iter (relationships))
        iterate (relationship);
}



/*\Docpage get_one_relationship -- Get one relationship member object.
 * 
 * DESCRIPTION
 *      Convenience function for obtaining one selected member of any
 *      relationship.
 */
function get_one_relationship(dv, routine)
    get_one_obj (dv, dv.relationship_class, routine);



/*\Docpage AddRelationship -- Action routine for the AddRelationshipCmd.
 * 
 * DESCRIPTION
 *      Adds the named relationship to the selected object.  Now the
 *      relationship can be populated with members.  A relation of the same
 *      name must exist for the object's class for this operation to be
 *      successful.
 */
function AddRelationship(dv, event, rel_name, sticky)
{
    obj := get_one_obj (dv, vdmObjVT, "AddRelationship");
    vdm.ObjSetRelationship (obj, rel_name, sticky);
    vcBrowse.ObjExpand (dv.loc_obj_browser, obj);
/*
    vcBrowse.ObjInsert (dv.loc_obj_browser, obj, cons (obj, rel_name),
                        dv.relationship_class);
*/
}


AddRelationshipCmd :=
    vcWv.Cmd (fac, AddRelationship, "add_relationship", "Add Relationship",
              vcWv.Param ("Relationship Name", vlStringVT, vcWv.Sticky, "",
                          relation_select_fac),
              vcWv.Param ("Persistent", vlYesNoVT, vcWv.Sticky, vlYesNo.Yes,
                          "Is the relationship persistent?"));
vcCmd.SetHelp (AddRelationshipCmd,
               "Add a new relationship to a data object.");
    


/*\Docpage RemoveRelationship -- Action routine for the RemoveRelationshipCmd.
 * 
 * DESCRIPTION
 *      Remove the selected relationships from the objects to which they are
 *      attached to.
 */
function RemoveRelationship(dv, event)
    foreach (obj_relname; selected_relationships (dv))
    {
        vdm.ObjRemoveRelationship (car (obj_relname), cdr (obj_relname));
        vcBrowse.ObjRemove (dv.loc_obj_browser, obj_relname);
        vcBrowse.ObjSelect (dv.loc_obj_browser, car (obj_relname));
    }


RemoveRelationshipCmd := vcWv.Cmd (fac, RemoveRelationship,
                                   "remove_relationship");
vcCmd.SetHelp (RemoveRelationshipCmd,
               "Remove a relationship from the selected data objects.");



/*\Docpage AddToRelationship -- Action routine for AddToRelationshipCmd.
 * 
 * DESCRIPTION
 *      Add a member to an existing relationship.
 */
function AddToRelationship(dv, event)
{
    obj_relname  := get_one_relationship (dv, "AddToRelationship");
    source       := car (obj_relname);
    source_class := vdm.ObjGetClass (source);
    rel_name     := cdr (obj_relname);
    
    foreach (dest; get_versioned_objects (dv))
        if ((dest != source) && !vdm.ObjHasRelative (source, rel_name, dest))
            vdm.ObjAddRelative (source, rel_name, dest);

    vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, obj_relname, TRUE);
    vcBrowse.ObjSelect (dv.loc_obj_browser, obj_relname);
    enable_cmds (dv);
}


AddToRelationshipCmd := vcWv.Cmd (fac, AddToRelationship,
                                  "add_to_relationship");
vcCmd.SetHelp (AddToRelationshipCmd,
               "Add new data objects to an existing relationship.");



/*\Docpage RemoveFromRelationship -- Action routine for RemoveFromRelationshipCmd.
 *
 * DESCRIPTION
 *      Removing an object from a relationship can be done in 2 ways, by
 *      selecting the relative, or by selecting the relationship and the
 *      object from somewhere else.
 */
function RemoveFromRelationship(dv, event)
{
    if (dv.n_selected_relatives > 0)
    {
        recalc_list := nil;
        relatives := nil;

        foreach (relative; get_selected_objects (dv, dv.relative_class))
            relatives = cons (relative, relatives);

        foreach (relative; vscript.list_iter (relatives))
        {
            dest        := car (relative);
            obj_relname := cdr (relative);
            source      := car (obj_relname);
            rel_name    := cdr (obj_relname);

            vdm.ObjRemoveRelative (source, rel_name, dest);

            if (!memq (source, recalc_list))
            {
                recalc_list = cons (source, recalc_list);
                vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, obj_relname,
                                            FALSE);
                enable_cmds (dv);
            }
        }
    }
    else
    {
        obj_relname  := get_one_relationship (dv, "RemoveFromRelationship");
        source       := car (obj_relname);
        source_class := vdm.ObjGetClass (source);
        rel_name     := cdr (obj_relname);
        failed_classes := nil;

        foreach (dest; get_versioned_objects (dv))
        {
            dest_class := vdm.ObjGetClass (dest);

            if ((dest != source) &&
                vdm.ObjHasRelative (source, rel_name, dest))
                vdm.ObjRemoveRelative (source, rel_name, dest);
        }

        vcBrowse.ObjRecalcChildren (dv.loc_obj_browser, obj_relname, FALSE);
        vcBrowse.ObjSelect (dv.loc_obj_browser, obj_relname);
        enable_cmds (dv);
    }
}


RemoveFromRelationshipCmd := vcWv.Cmd (fac, RemoveFromRelationship,
                                       "remove_from_relationship");
vcCmd.SetHelp (RemoveFromRelationshipCmd,
               "Remove objects from a relationship.");


/****************************************************************************/
/*                          Map/Location commands.                          */
/****************************************************************************/


/*
 * This structure is used to hold the information for the Find interaction
 * window, and related commands.
 */
struct dvFindInfoVT
{
    vcRegionVT      obj_cmdline;    /* Cmdline for object name wildcard     */
    vcRegionVT      class_cmdline;  /* Cmdline for class name wildcard      */
    vcRegionVT      location_status;/* Which location are we looking at?    */
    vcRegionVT      class_status;   /* Which class are we looking at?       */
    vcRegionVT      found_count;    /* How many have we found so far?       */
    vcRegionVT      total_searched; /* Total no. of objects searched        */
    vcRegionVT      progress;       /* Percentage of searches completed     */
    vcRegionVT      entry;          /* Table of all entry-related widgets   */
    vcRegionVT      status;         /* Table of all status-related widgets  */
    vscmVT          find_button;    /* Start the find                       */
    vscmVT          cancel_button;  /* Cancel the operation                 */
    vscmVT          dismiss_button; /* Dismiss the window                   */
    vlTrueFalseVT   cancel;         /* Should we stop the search            */
    vscmVT          region;         /* The entire Find dbox region          */
};



/*\vscmuserDocpage vdata.SelectObject -- Select the specified object.
 *
 * SYNOPSIS
 *      vdata.SelectObject(vdata_region, obj)
 *          vcRegionVT  vdata_region;
 *          vdmObjVT    obj;
 *
 * DESCRIPTION
 *      Selects the specified object in the hierarchy tree.  The tree is
 *      expanded if the nodes have never been expended before.  Depending on
 *      the number of data objects, this might take a few seconds.
 */
function SelectObject(region, obj)
    select_obj_aux (region, obj, vcBrowse.ObjSelect);



/*\vscmuserDocpage vdata.UnSelectObject -- UnSelect the specified object.
 *
 * SYNOPSIS
 *      vdata.UnSelectObject(vdata_region, obj)
 *          vcRegionVT  vdata_region;
 *          vdmObjVT    obj;
 *
 * DESCRIPTION
 *      Unselects the specified object in the hierarchy tree.  The tree is
 *      expanded if the nodes have never been expended before.  Depending on
 *      the number of data objects, this might take a few seconds.
 */
function UnSelectObject(region, obj)
    select_obj_aux (region, obj, vcBrowse.ObjUnSelect);



/*\srcDocpage select_obj_aux -- Perform the selection/unselection.
 * 
 * DESCRIPTION
 *      This is a workhorse function for the SelectObject and UnSelectObject
 *      PI functions.  The action parameter must be either the
 *      vcBrowse.ObjSelect function or the vcBrowse.ObjUnSelect.
 */
function select_obj_aux(region, obj, action)
{
    dv         := extract_dv (region);
    class_name := vdm.ClassGetName (vdm.ObjGetClass (obj));
    loc        := vdm.ObjGetLocation (obj);

    function auxilliary()
    {
        vcBrowse.ObjExpand (dv.loc_obj_browser, loc);

        /*
         * Once the location node is expended, walk through its children,
         * which are the class nodes, and find the one that matches the class
         * of the input parameter object.
         */
        foreach (loc_class, c; vcBrowse.ObjLoop (dv.loc_obj_browser, loc))
            if (string_eq (vdm.ClassGetName (cadr (loc_class)), class_name))
            {
                vcBrowse.ObjExpand (dv.loc_obj_browser, loc_class);

                /*
                 * Note that I cannot select the object that was passed into
                 * this function.  That's because the vcBrowse facility uses
                 * the object's memory footprint in its hash tables.
                 * Therefore, I need to find the exact object that matches the
                 * input parameter logically, i.e. through its name.
                 */
                foreach (obj_node; vscript.list_iter (caddr (loc_class)))
                    if (string_eq (vdm.ObjGetName (obj),
                                   vdm.ObjGetName (obj_node)))
                    {
                        action (dv.loc_obj_browser, obj_node);
                        vcBrowse.ObjShow (dv.loc_obj_browser, list (obj_node));
                    }

                vcRegion.QueueForRefresh (dv.loc_obj_browser);
                break;
            }

    }

    vcWin.WithBusy (vcRegion.Window (region), auxilliary);
}



/*\Docpage display_results -- Diplay Find results.
 * 
 * DESCRIPTION
 *      Expanding objects might take time, depending on the number of objects
 *      in the given location.  For this reason, the Find results are
 *      collected into a picklist from which the user can select which objects
 *      are of real interest.  Only then they are expanded.
 */
function display_results(dv, results)
{
    /*
     * The format of the loc_class_obj parameter is set in the do_find
     * function.
     */
    function navigate(loc_class_obj, flag, index)
        if (flag)
        {
            vcBrowse.ObjExpand (dv.loc_obj_browser, car (loc_class_obj));
            vcBrowse.ObjExpand (dv.loc_obj_browser, cadr (loc_class_obj));
            vcBrowse.ObjSelect (dv.loc_obj_browser, caddr (loc_class_obj));
            vcBrowse.ObjShow (dv.loc_obj_browser, cddr (loc_class_obj));
        }


    pick := vcRegion.Create ("pick", vcPicklist.fac, nil,
                             exclusive   <- FALSE,
                             font        <- vgFont.smallb,
                             columns     <- 1,
                             rows        <- length (results),
                             range       <- TRUE,
                             will_scroll <- TRUE);

    foreach (elem; vscript.list_iter (results))
        vcPicklist.Add (pick,
                        vpm.LocationGetAlias (car (elem)) + " : " +
                        vdm.ObjGetName (caddr (elem)),
                        elem);

    pick = vcPicklist.Scroll ("pick", pick, 3, 15, 0);
    vcPicklist.HighlightNotify (pick, navigate);


    text := "Click on the object(s) that you would like
to see selected in the browser.

Depending on the number of data objects in
the location, the selection process might
take a few seconds.";

    msg := vcRegion.Create ("msg", vcStaticText.fac, text);
    msg  = vcRegion.Create ("msg", vcMargin.fac, msg,
                            margin_size <- 2);
    msg := vcRegion.Create ("msg", vcBorder.fac, msg);


    button := vcRegion.Create ("buttons", vcButnRow.fac,
                               list (list ("dismiss", "Dismiss",
                                           vcWin.wclose, nil)));

    table := vcTable.Make ("Find Results", 3, 1, msg, pick, button);

    vcTable.Row (table, 0, vcTable.Border.Visible, 0, 0, 0,
                 vgAlign.MiddleCenter);

    vcWin.Popup (vcRegion.Window (dv.region), table, pick, vcWin.NoFlags);
}



/*\Docpage do_find -- Action routine for the do_find_cmd.
 *
 * DESCRIPTION
 *      Action routine for the actual finding.  Assumes that the wildcards
 *      have already been set up.  Allows itself to be interrupted by checking
 *      the boolean dv.find.cancel.  It also updateds find.location_status
 *      and find.class_status, which are vcStaticText regions.
 */
function do_find(dv, event, name, class_name)
{
    n_found         := 0;
    refresh_buttons := TRUE;
    all_classes     := string_eq (class_name, "");
    found_objs      := nil;
    window          := vcEvent.Window (event);

    if (dv.find == nil)       /* hand-typed command or from macro */
    {
        setup_find_window (dv, event);
        vcRegion.Set (dv.find.obj_cmdline, name);
        vcRegion.Set (dv.find.class_cmdline, class_name);
        refresh_buttons = FALSE;
    }

    vcRegion.Set (dv.find.found_count, "0");
    vcRegion.Set (dv.find.progress, 0);

    vcRegion.Enable (dv.find.find_button, FALSE, refresh_buttons);
    vcRegion.Enable (dv.find.cancel_button, TRUE, refresh_buttons);
    vcRegion.Enable (dv.find.dismiss_button, FALSE, refresh_buttons);
    vcWin.Busy1 (vcRegion.Window (dv.region), TRUE);
    vcWin.Busy1 (vcEvent.Window (event), TRUE);

    function clean()
    {
        vcWin.Busy1 (vcRegion.Window (dv.region), FALSE);
        vcWin.Busy1 (vcEvent.Window (event), FALSE);
        vcWin.ClearStatus (window);

        if (dv.find != nil)
        {
            vcRegion.Enable (dv.find.find_button, TRUE, TRUE);
            vcRegion.Enable (dv.find.cancel_button, FALSE, TRUE);
            vcRegion.Enable (dv.find.dismiss_button, TRUE, TRUE);
            vcRegion.Set (dv.find.class_status, "");
            vcRegion.Set (dv.find.location_status, "");
            dv.find.cancel = FALSE;
        }
    }

    unwind_protect
    {
        total_searches  := 0;
        searched_so_far := 0;

        vcWin.ClearStatus (window);
        vcWin.AddStatus (window,
                         "Collecting location and class information...");

        /*
         * Count up the total number of class/location pairs we're going
         * to look through
         */
        foreach (loc, c; vcBrowse.ObjLoop (dv.loc_obj_browser, dv.map))
            foreach (loc_class, c; vcBrowse.ObjLoop (dv.loc_obj_browser, loc))
            {
                class := cadr (loc_class);

                if (all_classes ||
                    vlStrMat (class_name, vdm.ClassGetName (class)))
                    ++total_searches;

                vcWv.ProcessPending (wv);
                if (dv.find.cancel || (dv.find == nil))
                {
                    clean ();
                    return (nil);
                }
            }

        vcWin.AddStatus (window, "Done.");


        total_searched := 0;

        foreach (loc, c; vcBrowse.ObjLoop (dv.loc_obj_browser, dv.map))
        {
            vcRegion.Set (dv.find.location_status, vpm.LocationGetAlias (loc));

            foreach (loc_class, c; vcBrowse.ObjLoop (dv.loc_obj_browser, loc))
            {
                class := cadr (loc_class);
                if (all_classes ||
                    vlStrMat (class_name, vdm.ClassGetName (class)))
                {
                    vcRegion.Set (dv.find.class_status,
                                  vdm.ClassGetName (class));
                    foreach (obj, c; vcBrowse.ObjLoop (dv.loc_obj_browser,
                                                       loc_class))
                    {
                        ++total_searched;
                        if (vlStrMat (name, vdm.ObjGetName (obj)))
                        {
                            found_objs = cons (list (loc, loc_class, obj),
                                               found_objs);

                            ++n_found;

                            vcRegion.Set (dv.find.found_count,
                                          convert (vlStringVT, n_found));
                        }
                        
                        vcWv.ProcessPending (wv);
                        if ((dv.find == nil) || dv.find.cancel)
                        {
                            clean ();
                            return (nil);
                        }
                    }

                    ++searched_so_far;
                    vcRegion.Set (dv.find.progress,
                                  100 * searched_so_far / total_searches);
                    vcRegion.Set (dv.find.total_searched, total_searched);
                }
            }
        }
    }
    cleanup
        clean ();

    if (n_found > 0)
        display_results (dv, found_objs);
}


do_find_cmd :=
    vcWv.Cmd (fac, do_find, "do_find", "Find Object",
              vcWv.Param ("Object name", vlStringVT, vcWv.Sticky, ""),
              vcWv.Param ("Class name", vlStringVT, vcWv.Sticky, ""));



/*\Docpage setup_find_window -- Action routine for the FindObjecCmd.
 * 
 * DESCRIPTION
 *      It's actually more than just an action routine, because this function
 *      is also called from the do_find function.  The do_find_cmd can be
 *      invoked manually, and in that case it calls this function to set up
 *      the find window.
 */
function setup_find_window(dv, event)
{
    if (dv.find != nil)
        vcWin.Show (vcRegion.Window (dv.find.region));
    else
    {
        dv.find = struct dvFindInfoVT
        {
            vcRegion.Create ("obj_cmdline", vcCmdline.fac, ""),
            vcRegion.Create ("class_cmdline", vcCmdline.fac, ""),
            vcRegion.Create ("location_status", vcStaticText.fac, "",
                             immediate <- TRUE),
            vcRegion.Create ("class_status", vcStaticText.fac, "",
                             immediate <- TRUE),
            vcRegion.Create ("found_count", vcStaticText.fac, "",
                             immediate <- TRUE),
            vcRegion.Create ("total_searched", vcStaticText.fac, "",
                             immediate <- TRUE),
            vcRegion.Create ("progress", vcScale.fac, 0,
                             min <- 0, max <- 100,
                             read_only <- TRUE),
            vcTable.Create ("entry", 2, 2),
            vcTable.Create ("status", 5, 2),
            nil,                /* Find button      */
            nil,                /* Cancel button    */
            nil,                /* Dismiss button   */
            FALSE,
            nil                 /* The entire region */
        };

        function l(s)
        {
            r := vcRegion.Create ("", vcLabel.fac, s);
            vcRegion.Font (r, vgFont.medium);
            r;
        }

        function bm(r)
            vcRegion.Create ("", vcBorder.fac,
                             vcRegion.Create ("", vcMargin.fac, r),
                             rounded <- FALSE);
                                              

        vcTable.Add (dv.find.entry, l ("Find object matching wildcard: "));
        vcTable.Add (dv.find.entry, dv.find.obj_cmdline);
        vcTable.Add (dv.find.entry, l ("In class matching wildcard: "));
        vcTable.Add (dv.find.entry, dv.find.class_cmdline);

        vcTable.Add (dv.find.status, l ("Searching Location: "));
        vcTable.Add (dv.find.status, dv.find.location_status);
        vcTable.Add (dv.find.status, l ("Searching Class: "));
        vcTable.Add (dv.find.status, dv.find.class_status);
        vcTable.Add (dv.find.status, l ("Found matches: "));
        vcTable.Add (dv.find.status, dv.find.found_count);
        vcTable.Add (dv.find.status, l ("Total searched: "));
        vcTable.Add (dv.find.status, dv.find.total_searched);
        vcTable.Add (dv.find.status, l ("Progress: "));
        vcTable.Add (dv.find.status, dv.find.progress);

        function find_it(event)
            vcCmd.Run (do_find_cmd, dv, event,
                       list (vcRegion.Get (dv.find.obj_cmdline, vlStringVT),
                             vcRegion.Get (dv.find.class_cmdline, vlStringVT)),
                       TRUE);

        function cancel()
            dv.find.cancel = TRUE;
                       
        buttons := vcRegion.Create ("buttons", vcButnRow.fac,
                                    list (list ("find", "Find", find_it, nil),
                                          list ("cancel", "Cancel", cancel,
                                                nil),
                                          list ("dismiss", "Dismiss",
                                                vcWin.wclose, nil)));

        dv.find.find_button    = vcButnRow.Get (buttons, 0);
        dv.find.cancel_button  = vcButnRow.Get (buttons, 1);
        dv.find.dismiss_button = vcButnRow.Get (buttons, 2);

        vcRegion.Enable (dv.find.cancel_button, FALSE, FALSE);

        dv.find.region = vcTable.Make ("Find Object", 3, 1,
                                       bm (dv.find.entry),
                                       bm (dv.find.status),
                                       buttons);

        w := vcWin.Popup (vcRegion.Window (dv.region), dv.find.region,
                          dv.find.obj_cmdline, vcWin.NoFlags);

        vcWin.DestroyNotify (w, func(w) dv.find = nil;);
    }
}


FindObjectCmd := vcWv.Cmd (fac, setup_find_window, "find_object");
vcCmd.SetHelp (FindObjectCmd,
               "Find a data object by name (wildcards are allowed).");



/*\Function SelectLocation -- Action routine for the Find -> Location command.
 * 
 * DESCRIPTION
 *      Loops through all the known locations and compares if the given name
 *      matches any of the aliases.  If so, the location's node is selected in
 *      the display tree.  If not, an appropriate message is displayed in the
 *      status window.
 */
function SelectLocation(dv, event, name)
{
    found := FALSE;

    foreach (location; vpm.MapLoopLocation (dv.map))
        if (string_eq (name, vpm.LocationGetAlias (location)))
        {
            vcBrowse.ObjSelect (dv.loc_obj_browser, location);
            vcBrowse.ObjShow (dv.loc_obj_browser, list (location));
            found = TRUE;
            break;
        }

    if (!found)
    {
        win := vcEvent.Window (event);

        vcWin.ClearStatus (win);
        vcWin.AddStatus (win, sprintf ("Location `%s' not found.", name));
    }
}


SelectLocationCmd :=
    vcWv.Cmd (fac, SelectLocation, "select_location", "Select Location",
              vcWv.Param ("Location name", vlStringVT, vcWv.Sticky, ""));

vcCmd.SetHelp (SelectLocationCmd,
               "Find a location by name.");



/****************************************************************************/
/*                          Class commands.                                 */
/****************************************************************************/


/*\Function AddClassFile -- Action routine for the AddClassFileCmd.
 * 
 * DESCRIPTION
 *      First check if the Class Files node has been selected.  If everything
 *      was successful, set the modify flag.
 */
function AddClassFile(dv, event, class_file)
{
    class_files_node := get_one_obj (dv, dv.class_files_class, "AddClassFile");

    vpm.ProcessAddClassFile (class_file);
    vdm.CntxAddClassFile (dv.cntx, class_file);
    vcBrowse.ObjRecalcChildren (dv.class_browser, class_files_node, FALSE);

    dv.class_files_modified = TRUE;
    vcRegion.QueueForRefresh (dv.class_browser);
}


AddClassFileCmd :=
    vcWv.Cmd (fac, AddClassFile, "add_class_file", "Add Class File",
              vcWv.Param ("Class definition file", vlStringVT, vcWv.Sticky,
                          "", vcFileSelect.fac));
vcCmd.SetHelp (AddClassFileCmd,
               "Add a new file containing class definitions.");



/*\Docpage view_class_def_aux -- Auxiliary function to view a class definition.
 * 
 * DESCRIPTION
 *      It's called from both the ViewClassDef action routine and the
 *      class_edit function used as the class's class editor.  The latter one
 *      requires that the editor notify vcBrowse when it's done, so that's why
 *      we have an end_proc parameter.
 *
 * NOTE
 *      The class node is a list of one element: the class object.
 */
function view_class_def_aux(dv, event, end_proc)
{
    class  := car (get_one_obj (dv, dv.class_class, "ViewClassDef"));
    parent := vcEvent.Window (event);
    vcWv.RequireFac (wv, "vcPref");

    msg := "This is a viewing window only.  To edit a class definition
 you must exit ViewData, edit the class definition file and then reinvoke.";

    warning := vcRegion.Create ("warning", vcLabel.fac, msg);
    vcRegion.Font (warning,
                   vgFont.mediumb);
    warning  = vcRegion.Create ("warning", vcBorder.fac, warning);
    warning  = vcRegion.Create ("warning", vcScroll.fac, warning);
    viewer  := vcRegion.Create ("class viewer", vcPref.fac,
                                list (parent, fac, vdm.ClassGetPrefs (class)));

    buttons := vcRegion.Create ("Buttons", vcButnRow.fac,
                                list (list ("dismiss", "Dismiss",
                                            vcWin.wclose, nil)));

    table   := vcTable.Create ("View Class Definitions", 3, 1);
    vcTable.Add (table, warning);
    vcTable.Add (table, viewer);
    vcTable.Add (table, buttons);

    window  := vcWin.Create (parent, table);

    if (end_proc != nil)
        vcWin.DestroyNotify (window, func(w) end_proc (););
}



/*\Docpage ViewClassDef -- Action routine for the ViewClassDefCmd.
 * 
 * DESCRIPTION
 *      For now, allow the user only to view the class definition.  When we
 *      get a better facility for editing ViewScript functions, we can let
 *      them save the edits into the class file.
 */
function ViewClassDef(dv, event)
    view_class_def_aux (dv, event, nil);


ViewClassDefCmd := vcWv.Cmd (fac, ViewClassDef, "view_class");
vcCmd.SetHelp (ViewClassDefCmd, "Class definition viewer.");



/*\Docpage CreateDrawer -- Action routine for the CreateDrawerCmd.
 * 
 * DESCRIPTION
 *      Inserts the new node into the display tree and invokes the EditDrawer
 *      fucntion to handle the actual drawer definition.
 */
function CreateDrawer(dv, event, name)
{
    if (vdm.CntxFindClass (dv.cntx, name) != nil)
       dv_err ("Drawer name %s is the same as an already existing class name.",
               name);

    if (! null (vpm.ProcessGetDrawerDef (name)))
        dv_err ("Duplicate drawer name: %s", name);

    vcBrowse.ObjInsert (dv.class_browser, dv.croot.cdrawers,
                    struct vdataDrawerVT {name, nil}, dv.drawer_class);
    EditDrawer (dv, event);

    dv.class_drawers_modified = TRUE;
}


CreateDrawerCmd :=
    vcWv.Cmd (fac, CreateDrawer, "create_drawer", "New Drawer",
              vcWv.Param ("Drawer name", vlStringVT, vcWv.Sticky, ""));
vcCmd.SetHelp (CreateDrawerCmd,
               "Create a new class drawer definition.");



function ConfirmRemoveDrawer(dv, event)
    confirm_op (dv, "The following class drawers will be removed.  Ok?",
                FALSE, dv.drawer_class, nil);

function RemoveDrawer(dv, event)
{
    kill_list := nil;

    foreach (obj, val; vlSym.Loop (dv.selected_table))
        if (istype (obj, vdataDrawerVT))
            kill_list = cons (obj, kill_list);

    foreach (drawer; vscript.list_iter (kill_list))
    {
        vpm.ProcessRemoveDrawerDef (drawer.name);
        vcBrowse.ObjRemove (dv.class_browser, drawer);
    }

    dv.class_drawers_modified = TRUE;
}


RemoveDrawerCmd :=
    vcWv.CmdWithConfirm (fac, RemoveDrawer, ConfirmRemoveDrawer, TRUE,
                         "remove_drawer");
vcCmd.SetHelp (RemoveDrawerCmd,
               "Remove a drawer definition.");


/*\Docpage edit_drawer_aux -- Auxialiary function to edit a drawer definition.
 * 
 * DESCRIPTION
 *      It's called from both the EditDrawer action routine and the
 *      drawer_edit function used as the drawer's class editor.  The latter
 *      one requires that the editor notify vcBrowse when it's done, so that's
 *      why we have an end_proc parameter.
 */
function edit_drawer_aux(dv, event, end_proc)
{
    drawer      := get_one_obj (dv, dv.drawer_class, "EditDrawer");
    reg         := vcRegion.Create ("drawer", vcListEdit.fac, nil,
                                    cmd  <- CreateDrawerCmd,
                                    rows <- length (drawer.definition));

    foreach (elem; vscript.list_iter (drawer.definition))
        vcListEdit.Add (reg, elem);


    function install(event)
    {
        def_list := nil;
        for (row := 0; row < vcListEdit.NRows (reg); ++row)
            def_list = cons (vcListEdit.Get (reg, row, 0), def_list);

        /*  
         * Remove the old drawer, if it exists.
         */
        if (vpm.ProcessGetDrawerDef (drawer.name) != nil)
            vpm.ProcessRemoveDrawerDef (drawer.name);

        vpm.ProcessSetDrawerDef (drawer.name, def_list);
        drawer.definition = def_list;

        if (end_proc != nil)
            end_proc ();

        vcBrowse.ObjRecalcChildren (dv.class_browser, drawer, FALSE);
        dv.class_drawers_modified = TRUE;
    }

    function cancel(event)
    {
        vcWin.close_if_modified (vcEvent.Window (event), event);

        if (end_proc != nil)
            end_proc ();
    }

    buttons := vcRegion.Create ("Buttons", vcButnRow.fac,
                                list (list ("install", "Install", install,
                                            list (close_window <- TRUE)),
                                      list ("cancel", "Cancel", cancel, nil)));

    vcWin.Create (vcEvent.Window (event),
                  vcTable.Make ("Edit Drawer", 2, 1, reg, buttons));
}



/*\Docpage EditDrawer -- Action routine for the EditDrawerCmd.
 * 
 * DESCRIPTION
 *      Uses the vcListEdit facility to display the drawer definition.
 */
function EditDrawer(dv, event)
    edit_drawer_aux (dv, event, nil);


EditDrawerCmd := vcWv.Cmd (fac, EditDrawer, "edit_drawer");
vcCmd.SetHelp (EditDrawerCmd,
               "Edit a drawer definition.");



/*\Docpage WriteClassInfo -- Action routine for the WriteClassInfoCmd.
 * 
 * DESCRIPTION
 *      Writes all the changed class information (drawers and class files)
 *      to the vpmus file.
 */
function WriteClassInfo(dv, event)
{
    CLASS_FILE_PREFIX     := "(Process 'AddClassFile ";
    CLASS_FILE_PREFIX_LEN := string_length (CLASS_FILE_PREFIX);

    function write_class_file(f)
        foreach (class_file; vpm.ProcessLoopClassFiles ())
            fputline (f, sprintf ("%s%s)\n",
                                  CLASS_FILE_PREFIX, obj2string (class_file)));


    function is_class_file_entry(str)
        ((string_length (str) > CLASS_FILE_PREFIX_LEN) &&
         string_eqi (substring (str, 0, CLASS_FILE_PREFIX_LEN),
                     CLASS_FILE_PREFIX));


    vcWin.ClearStatus (vcEvent.Window (event));
    if (dv.class_files_modified)
    {
        write_vpmus (is_class_file_entry, write_class_file);

        dv.class_files_modified = FALSE;

        vcWin.AddStatus (vcEvent.Window (event), "Saved class file changes.");
    }
    else
        vcWin.AddStatus (vcEvent.Window (event),
                         "No changes to class file information.");



    CLASS_DRAWER_PREFIX     := "(Process 'AddDrawer ";
    CLASS_DRAWER_PREFIX_LEN := string_length (CLASS_DRAWER_PREFIX);

    function write_class_drawer(f)
        foreach (drawer_name, def; vpm.ProcessLoopDrawers ())
            fputline (f, sprintf ("%s%s %s)\n",
                                  CLASS_DRAWER_PREFIX,
                                  obj2string (drawer_name),
                                  obj2string (def)));


    function is_class_drawer_entry(str)
        ((string_length (str) > CLASS_DRAWER_PREFIX_LEN) &&
         string_eqi (substring (str, 0, CLASS_DRAWER_PREFIX_LEN),
                     CLASS_DRAWER_PREFIX));


    vcWin.ClearStatus (vcEvent.Window (event));
    if (dv.class_drawers_modified)
    {
        write_vpmus (is_class_drawer_entry, write_class_drawer);

        dv.class_drawers_modified = FALSE;

        vcWin.AddStatus (vcEvent.Window (event),
                         "Saved class drawer changes.");
    }
    else
        vcWin.AddStatus (vcEvent.Window (event),
                         "No changes to class drawer information.");
}


WriteClassInfoCmd := vcWv.Cmd (fac, WriteClassInfo, "write_class_info");
vcCmd.SetHelp (WriteClassInfoCmd,
               "Save all the class drawer and class file information.");


/****************************************************************************/
/*                          Preferences.                                    */
/****************************************************************************/

/*
 * ViewData users should be able to specify vcBrowse preferences when
 * instantiating a ViewData region.  They will be passed in to the child
 * facility.
 */
vcFac.CopyPrefs (vcBrowse.fac, fac);


vlPref.Replace (prefs, "class_view", vlTrueFalseVT,
                "Use the class -> location -> object hierarchy",
                FALSE);
vlPref.Replace (prefs, "class_view_filter", vscmVT,
                "Class filter to be used with the class_view preference",
                nil);
vlPref.Replace (prefs, "edit_new", vlTrueFalseVT,
                "When creating new objects, should an editor be invoked?",
                TRUE);
vlPref.Replace (prefs, "select_data", vscmVT,
                "Center ViewData display on this object at start up",
                nil);



/****************************************************************************/
/*                          Key bindings.                                   */
/****************************************************************************/


vcBind.Set (bindings, '\_w', LocationWriteCmd);
vcBind.Set (bindings, '\_W', LocationWriteCmd);



/*
 * Load the menu definitions, which are defined in a separate menu file.
 */
load ("vdatamn.vs");



/*
 * Load the user customization files for this facility, if they exist in the
 * WDIR path.
 */
vcsa.LoadCustomizations ("vdata", current_package ());
