/*
 * $Id: vflLog.vs,v 1.2.1.4 1995/04/28 21:19:01 akoren Exp $
 *
 * Task log facility.
 */

/* this data base should already exist */
p := vlPref.Create ("vflowcolor");
vtx.RegisterHighlight ("vlg", nil, vtxHighlight.CaseSensitive,
            list (list (vtxHighlight.Regexp1, vlPref.Val (p, "log_msg"),
                                 "vfl(ow)?:.*")));

function fix_bind (text) {
    b := vcBind.Duplicate (vcRegion.Bindings (text));
    vcBind.Set (b, '\rightdown', nil);
    vcBind.Set (b, '\_c', nil);
    vcRegion.SetBindings (text, b);
}

function _Create (unused, rgn) {

    log := struct {notetab; text; buffertable; current; rgn};

    vcRegion.FocusInhibit (rgn);
    log.rgn = rgn;
    log.notetab = vcRegion.Create ("", vcNoteTab.fac, nil);
    log.text = vcRegion.Create ("", vtx.fac, "log.vlg", scratch <- TRUE,
                                         highlight_keywords <- TRUE,
                                                 height <- 10);
    log.buffertable = vlSym.ObjCreate (59);
    /* maps object -> { (name . string) *or* window } */

    fix_bind (log.text);
    function select_entry (obj) {
        /* if we got here, we know it's not a window */
        vtx.ReadOnly (log.text, FALSE);
        vtx.Clear (log.text);
        vtx.AppendString (log.text, 
                    cdr (vlSym.Fetch (log.buffertable, obj)));
        vtx.ReadOnly (log.text, TRUE);
        vtx.Update (log.text);

        log.current = obj;
        enable_cmds (log);
    }
    vcNoteTab.CallOnSelect (log.notetab, select_entry);


    t := vcTable.Make ("",2, 1, log.notetab, log.text);
    vcTable.Row (t, 1, vcTable.Border.Invisible, 1, 1, 1, 
                vgAlign.MiddleCenter);
    vcRegion.AddChild (rgn, t);
    log;
}

function AddEntry (r, name, obj) {
    log := vcRegion.GetData (r);
    if (vlSym.Fetch (log.buffertable, obj) == nil) {
        vcNoteTab.AddEntry (log.notetab, name, obj);
        vlSym.Store (log.buffertable, obj, cons (name, ""));
    }
}

function AppendString (r, obj, str) {
    log := vcRegion.GetData (r);
    entry := vlSym.Fetch (log.buffertable, obj);
    if (typeof (entry) == vcRegionVT) {
        vtx.ReadOnly (entry, FALSE);
        vtx.AppendString (entry, str);
        vtx.EndOfBuffer (entry);
        vtx.ReadOnly (entry, TRUE);
        vtx.Update (entry);
    } else {
        oldstr := cdr (entry);
        oldstr = oldstr + str;
        vlSym.Store (log.buffertable, obj, cons (car (entry), oldstr));
        if (obj == log.current) {
            vtx.ReadOnly (log.text, FALSE);
            vtx.AppendString (log.text, str);
            vtx.ReadOnly (log.text, TRUE);
            vtx.EndOfBuffer (log.text);
            vtx.Update (log.text);
        }
    }
}

function AddString (r, obj, str) {
    end := length(str) - 1;
    if (str[end] == '\r')
        str[end] = '\n';
    log := vcRegion.GetData (r);
    entry := vlSym.Fetch (log.buffertable, obj);
    if (typeof (entry) == vcRegionVT) {
        vtx.ReadOnly (entry, FALSE);
        vtx.AppendString (entry, str);
        vtx.EndOfBuffer (entry);
        vtx.ReadOnly (entry, TRUE);
        vtx.Update (entry);
    } else {
        oldstr := cdr (entry);
        AppendString (r, obj, str);
    }
}

function remove (log, ev)
    if (log.current != nil) {
        vcNoteTab.RemoveEntry (log.notetab, log.current);
        vlSym.Remove (log.buffertable, log.current);
        log.current = nil;
    }
function confirm_remove (log, ev) {
    entry := vlSym.Fetch (log.buffertable, log.current);
    sprintf ("Remove log for %s?\n", car (entry));
}      
RemoveCmd := vcWv.CmdWithConfirm (fac, remove, confirm_remove, FALSE,
              "Remove", "Remove Log");

function peel(log, ev)
    if (log.current != nil) {
        obj := log.current;
        entry := vlSym.Fetch (log.buffertable, obj);
        r := vcRegion.Create ("Log: " + car (entry), 
                                vtx.fac, car (entry) + ".vlg", 
                                         highlight_keywords <- TRUE,
                           scratch <- TRUE, height <- 12);
        vtx.AddString (r, cdr (entry));
        vtx.ReadOnly (r, TRUE);
        w := vcWin.Create (r, vcWin.NoMenus);
        remove (log, ev);
        vlSym.Store (log.buffertable, obj, r);

        function restore(w) {
            if (!vcRegion.Destroyed (log.rgn)) {
                vlSym.Remove (log.buffertable, obj);
                AddEntry (log.rgn, car (entry), obj);
                AddString (log.rgn, obj, vtx.GetBuffer (r));
            }
            vcWin.Destroy (w);
        }
        vcWin.CallOnClose (w, restore);
    }
PeelCmd := vcWv.Cmd (fac, peel, "Peel");
        
function clear_entry (log, obj) {
    entry := vlSym.Fetch (log.buffertable, obj);
    if (typeof (entry) == vcRegionVT) {
        vtx.ReadOnly (entry, FALSE);
        vtx.Clear (entry);
        vtx.Update (entry);
        vtx.ReadOnly (entry, TRUE);
    } else {
        vlSym.Store (log.buffertable, obj, cons (car (entry), ""));
        if (obj == log.current) {
            vtx.ReadOnly (log.text, FALSE);
            vtx.Clear (log.text);
            vtx.Update (log.text);
            vtx.ReadOnly (log.text, TRUE);
        }
    }
}
    
function Clear(r, obj)
    clear_entry (vcRegion.GetData (r), obj);

function clear(log, ev)
    clear_entry (log, log.current);
ClearCmd := vcWv.Cmd (fac, clear, "Clear");

function popup(log, ev)
    vcMenuPop.Create (popup_menu, ev);
PopupCmd := vcWv.Cmd (fac, popup, "Menu");

function enable_cmds (log) {
    is_cur := !null(log.current);
    vcCmd.Enable (RemoveCmd, is_cur);
    vcCmd.Enable (ClearCmd, is_cur);
    vcCmd.Enable (PeelCmd, is_cur);
    vcWin.UpdateMenus (vcRegion.Window (log.rgn));
}

function after (log, ev)
    enable_cmds (log);
function _Initial (log, ev)
    enable_cmds (log);

popup_menu := vcWv.MakeMenu ("log_menu",
                           vcWv.Pin (),
                           list ("&Remove", RemoveCmd),
                           list ("&Peel", PeelCmd),
                           list ("&Clear", ClearCmd),
                           list ("&Save", vtx.WritetoCmd)
                           );
main_menu := vcWv.MakeMenu ("log_menu",
                           vcWin.RedSquareEntryNoPlotNoQuit,
                           list ("Log", popup_menu)
                           );

vcBind.Set (bindings, '\^p', PeelCmd);
vcBind.Set (bindings, '\^c', ClearCmd);
vcBind.Set (bindings, '\rightdown', PopupCmd);

vcCmd.SetHelp (PeelCmd, "Peel log for this task into separate window.");
vcCmd.SetHelp (RemoveCmd, "Remove log for this task permanently.");
vcCmd.SetHelp (ClearCmd, "Clear log for this task.");
vcFac.SetMenus  (fac, main_menu);


