    
function cancel_pcb_attrs (data, event)
{
    nil;
}

cancel_cmd := vcWv.Cmd(fac, cancel_pcb_attrs, "Cancel");

/* this is the action function for the accept command
   for the attribute editor. 
   the values are retrieved from the input editors
   and then applied to the objects selected. 
   */
/* requires block (block), region (attr_ed_table)
   object list (obj_list), attribute vector (attr_vector)
   */
function accept_pcb_attrs (data, event)
{
    block := data.block;
    top_table := data.top_table;
    obj_list := data.obj_list;
    attr_vector := data.attr_vector;
    new_attr_list := data.new_attr_list;
    type_string := data.type_string;
    selected_objects := data.selected_objects;
    
    declare deletion_list;

    oats_val := vdraw.ParamGetMode(block, vdraw.vqMode.Oats);
    visibility_val := vdraw.ParamGetValue (block, vdraw.vqValue.NewAttrVis); 

    /* step through all the regions to find the input editor
       regions and then get all the values from each input editor
       found.
       */
    index := -1;
    new_index := -1;
    foreach (child_region; vcRegion.Children (top_table, TRUE)) 
    {
        if (string_eq(vcRegion.Name(child_region), 
                      "attr_input_editor_inst"))
        {
            ++index;
            attr_vector[index][2] = vcRegion.Get(child_region, 
                                                 vlStringVT);
        }
        if (string_eq(vcRegion.Name(child_region), 
                      "attr_input_editor_oat"))
        {
            attr_vector[index][3] = vcRegion.Get(child_region, 
                                                 vlStringVT);
        }
        if (string_eq(vcRegion.Name(child_region), 
                      "attr_new_name"))
        {
            ++new_index;
            new_attr := list_ref(new_attr_list, new_index);
            
            new_attr[0] = vcRegion.Get(child_region, 
                                       vlStringVT);
        }
        if (string_eq(vcRegion.Name(child_region), 
                      "attr_new_inst"))
        {
            new_attr := list_ref(new_attr_list, new_index);
            
            new_attr[2] = vcRegion.Get(child_region, 
                                       vlStringVT);
        }
        if (string_eq(vcRegion.Name(child_region), 
                      "attr_new_oat"))
        {
            new_attr := list_ref(new_attr_list, new_index);
            
            new_attr[3] = vcRegion.Get(child_region, 
                                       vlStringVT);
        }
    }   
    
    declare locX, locY, xIncrement, yIncrement, increment;

    function check_add_attr (obj, segment, curr_attr)
    {
        declare name, curr_attr_handle;

        curr_attr_handle = nil;

        /* check_add_attr first checks for the value "****Various****".
           if this is the value then we needn't add at all. 
           */
        if (!string_eq(curr_attr[2], "****Various****"))
        {
            /* if value is not various then see if the attr already
               exists by getting the handle
               */
            foreach (attr; vdraw.ObjIterAttr(block, obj))
            {
                name = vdraw.AttrGetName(block, attr);
                if (string_eq(name, curr_attr[0]))
                {
                    curr_attr_handle = attr;
                    break;
                }
            }
            
            /* if the attr does not exist and value is ""
               do nothing. otherwise value needs to be added.
               */
            if (!((curr_attr_handle == nil) && 
                  (string_eq(curr_attr[2], ""))))
            {
                /* at this point check if attr exists and is same
                   as new value from editor. if so do nothing 
                   otherwise add_attr
                   */
                if (!((curr_attr_handle != nil) && 
                      (string_eq(curr_attr[2], 
                                 vdraw.AttrGetValue(block, 
                                                    curr_attr_handle)))))
                {
                    /* invoke the check function specified for
                       this attribute to determine validity of
                       the value to be added.  if the value is
                       valid go ahead and add it. */
                    check_func := curr_attr[4] (curr_attr[2], 
                                                curr_attr[5]);
                    if (check_func)   
                    {
                        add_attr(obj, segment, curr_attr, 
                                 curr_attr_handle, FALSE, 2);   
                    }
                }
            }
        }

        
        if (oats_val == vlOnOff.On)
        {
            /* reinitialize handle to get rid of the one for the attr 
               that we had above.
               */
            curr_attr_handle = nil;

            /* check for the value "****Various****".
               if this is the value then we needn't add at all. 
               */
            if (!string_eq(curr_attr[3], "****Various****"))
            {
                /* if value is not various then see if the oat already
                   exists by getting the handle
                   */
                foreach (attr; vdraw.ObjIterOat(block, obj))
                {
                    name = vdraw.AttrGetName(block, attr);
                    if (string_eq(name, curr_attr[0]))
                    {
                        curr_attr_handle = attr;
                        break;
                    }
                }
                
                /* if the attr does not exist and value is ""
                   do nothing. otherwise value needs to be added.
                   */
                if (!((curr_attr_handle == nil) && 
                      (string_eq(curr_attr[3], ""))))
                {
                    /* at this point check if attr exists and is same
                       as new value from editor. if so do nothing 
                       otherwise add_attr
                       */
                    if (!((curr_attr_handle != nil) && 
                          (string_eq(curr_attr[3], 
                                     vdraw.AttrGetValue(block, 
                                                        curr_attr_handle)))))
                    {
                        /* invoke the check function specified for this
                           oat to determine validity of the value to
                           be added.  if the value is valid go ahead and
                           add it. 
                           */
                        check_func := curr_attr[4] (curr_attr[3],
                                                    curr_attr[5]);
                        if (check_func)
                        {
                            add_attr(obj, segment, curr_attr, 
                                     curr_attr_handle, TRUE, 3);
                        } /* end if (check_func) */
                    }
                }
            }
        }
    }
    
    
    function add_attr (obj, segment, curr_attr, 
                       curr_attr_handle, oat, v_index)
    {
        /* add the x and y increments */
        x := locX + xIncrement;
        y := locY + yIncrement;
        declare name, value;
        
        /* you cannot add an attribute if it already
           exists so.. */
        /* if an attribute doesn't already exist then add it */
        if (curr_attr_handle == nil) 
        {
            /* if an attribute's value is NOT an empty
               string or "****Various****", then add it 
               */
            
            if (!(string_eq(curr_attr[v_index], "") || 
                  string_eq(curr_attr[v_index], "****Various****")))
            {
                if (!oat)
                {
                    
                    if (string_eq(type_string, "Net"))
                    {
                        curr_attr_handle = vdraw.NetAddAttr(block, 
                                                            obj, 
                                                            segment,
                                                            curr_attr[0]+
                                                            "="+
                                                            curr_attr[v_index], 
                                                            x, y, 
                                                            visibility_val);
                        
                        
                    }
                    else
                        if (string_eq(curr_attr[0], "#"))
                            vdraw.CompPinSetPinNumber(block, obj, 
                                                      curr_attr[v_index]);
                        else
                        {
                            curr_attr_handle = vdraw.ObjAddAttr(block, obj,
                                                                curr_attr[0]+
                                                                "="+
                                                                curr_attr[v_index], 
                                                                x, y, 
                                                                visibility_val);
                        }
                } /* end if (!oat) */
                else
                {
                    curr_attr_handle = vdraw.ObjAddOat(block, obj,
                                                       curr_attr[0]+
                                                       "="+
                                                       curr_attr[v_index]);
                    vdraw.ObjSetLoc(block, curr_attr_handle, x, y);
/* oat vis */
                    vdraw.ObjSetVis(block, curr_attr_handle, 
                                    visibility_val);
                } /* end else */
                vdraw.ObjSetOrigin(block, curr_attr_handle, 
                                   vgAlign.LowerLeft);
                /* increment indices and displacements */
                yIncrement += increment;
            }
        }
        /*   if an attribute already exists modify it */
        else
        {
            /* if an attribute's value is NOT an empty
               string or ****Various****, then add it 
               */
            if (!(string_eq(curr_attr[v_index], "") || 
                  string_eq(curr_attr[v_index], "****Various****")))
            {
                if (string_eq(curr_attr[0], "#"))
                    vdraw.CompPinSetPinNumber(block, obj, 
                                              curr_attr[v_index]);
                else
                {
                    /* due to a bug in vdraw.ObjSetTextString
                       it throws a failure exception if you
                       try to set an object's string to it's
                       existing value (i.e., no change).
                       therefore, i have to check that the
                       current value is not the same as new
                       value and set the string only if they
                       are different 
                       */

                    attr_string := vdraw.ObjGetTextString(block, 
                                                          curr_attr_handle);
                    if (!(string_eq (attr_string, 
                                     curr_attr[0]+"="+
                                     curr_attr[v_index])))
                    {
                        vdraw.ObjSetTextString(block, 
                                               curr_attr_handle, 
                                               curr_attr[0]+"="+
                                               curr_attr[v_index]);
                    }
                }
            } /* end if string_eq */
            else
                if (string_eq(curr_attr[v_index], ""))
                {
                    /* if the object has an empty
                       value string select the
                       object to be deleted and
                       delete it. 
                       */
/*
                    vdraw.BlockDeselectAll(block);
                    vdraw.BlockSelectObj(block, curr_attr_handle);
                    vdraw.BlockDelSelectedAll(block, FALSE);
                    vdraw.BlockRestoreSelected(block, selected_objects);
                    */

/*************/

                    deletion_list = cons (curr_attr_handle, deletion_list);

/*************/
                }
        } /* end else curr_attr_handle*/ 
    } /* end add_attrs() */ 
    
    
    
    
    
    /* add the attributes to each object in the list of selected
       objects 
       */
    foreach (obj; vscript.list_iter(obj_list)) 
    {
        /* component and net objects have to be handled separately */
        if (string_eq(type_string, "Comp") || 
            string_eq(type_string, "Comp_Pin"))
        {
            /* determine location of the object */
            loc := vdraw.ObjGetLoc(block, obj);
            locX = vdraw.LocGetX(block, loc); 
            locY = vdraw.LocGetY(block, loc);
            
            /* set the incremental displacement of attributes */
            xIncrement = 0;
            yIncrement = 0;
            increment  = 20;
            
            /* go through each attribute in the attribute list and
               add add it to the current object
               */
            foreach(curr_attr; vscript.vect_iter (attr_vector))
            {
                check_add_attr(obj, nil, curr_attr);
            } /*end foreach (curr_attr;...) */
            foreach(curr_attr; vscript.list_iter (new_attr_list))
            {
                check_add_attr(obj, nil, curr_attr);
            } /*end foreach (new_attr_list;...) */
        } /*end if (string_eq(type_string, "Comp")) */
        
        else 
            if (string_eq(type_string, "Net"))
            {
                /* get the first selected segment of the selected net */
                vdraw.BlockSelectObj (block, obj);
                foreach(segment; vdraw.NetIterSelSeg(block, obj))
                {
                    /* determine location of the segment */
                    loc := vdraw.SegGetLoc(block, segment, vdraw.vqSegPt.Low);
                    locX = vdraw.LocGetX(block, loc); 
                    locY = vdraw.LocGetY(block, loc);
                    /* set the incremental displacement of attributes */
                    xIncrement = 0;
                    yIncrement = 0;
                    increment  = 20;
                    
                    /* go through each attribute in the attribute
                       list and add add it to the current object 
                       */
                    foreach(curr_attr; vscript.vect_iter (attr_vector))
                    {
                        check_add_attr(obj, segment, curr_attr);
                    }
                    foreach(curr_attr; vscript.list_iter (new_attr_list))
                    {
                        check_add_attr(obj, segment, curr_attr);
                    } /*end foreach (new_attr_list;...) */
                    /* stop after first net segment.. 
                       find better way to do this */
                    /* sorry.. there isn't one i'm told. */
                    break;
                } /* end foreach */
            } /* end if "Net"*/
    } /* end foreach (obj; vscript.list_iter(obj_list)) */

/****************/

    if (length(deletion_list) != 0)
    {
        vdraw.BlockDeselectAll(block);
        foreach (del_attr; vscript.list_iter (deletion_list))
        {
            vdraw.BlockSelectObj (block, del_attr);
        }
        vdraw.BlockDelSelectedAll(block, FALSE);
    }
/****************/
    vdraw.BlockRestoreSelected(block, selected_objects);
} /* end function accept_pcb_attrs */







accept_cmd := vcWv.Cmd(fac, accept_pcb_attrs, "Accept");






/*
   popup a box to display the help string
   */
function proc_help(help_string)
    func (event)
    {
        help_table := vcTable.Create ("help_table", 2, 1);
        
        help_text := vcRegion.Create ("help_text", 
                                      vcStaticText.fac, help_string,
                                      align <- vgAlign.MiddleLeft
                                      );
        help_text_margin := vcRegion.Create ("help_text_margin", 
                                             vcMargin.fac, help_text);
        vcTable.Add (help_table, help_text_margin);
        
        line := vcRegion.Create ("line", vcPattern.fac, vgLinePattern.Solid);
        vcTable.Add (help_table, line);

        help_button_row := vcRegion.Create ("help_button_row", 
                                            vcButnRow.fac,
                                            list (list ("Close", "Close",
                                                        func(e) nil;,
                                                        list (close_window <- TRUE))));
        vcTable.Add (help_table, help_button_row);
        
        vcWin.Popup(vcEvent.Window(event),
                    help_table, help_button_row,
                    vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);     
    };    
    
function _Create(init_data, region)
{
    data := struct {
        block;
        top_table;
        obj_list;
        attr_vector;
        new_attr_list;
        type_string;
        selected_objects;
        attr_ed_touched
    };

    display := init_data[0];

    type_string := init_data[2];
    attr_notify_table := init_data[3];
    attr_vector := init_data[4];
    selected_objects := init_data[5];

    /* indicates whether this AttrEd region has been touched by user.
       it becomes true in the update_region procedure call.
       */
    attr_ed_touched := FALSE;

    declare new_attr_list;

    block := vdraw.DisplayGetBlock(display);   

    
    oats_val := vdraw.ParamGetMode(block, vdraw.vqMode.Oats);
    




    /* Creates the wonderful button to the right of the search field which
     * lists matching values.
     */
    function ButtonCreate(proc)
    {
        rgnArrow  := vcRegion.Create ("", vcArrow.fac, 3);
        rgnBorder := vcRegion.Create ("conflict_border", vcBorder.fac, rgnArrow);
        fn := func (event) nil;;
        
        rgnButton := vcRegion.Create ("conflict_button", vcButton.fac, proc,
                                      tip <- "Valid Values");

        vcRegion.AddChild (rgnButton, rgnBorder);
        vcButton.HighlightRegion (rgnButton, rgnBorder);

        vcRegion.FocusInhibit (rgnButton);
        vcRegion.SetCursor (rgnButton, vgCursor.Hand);

        rgnButton;

    }

    /* Creates the wonderful button which names the attribute and 
     * provides help. 
     */
    function StaticTextButtonCreate(static_text, attr_help_str)
    {

        rgnStatic := vcRegion.Create ("", vcStaticText.fac, 
                                      "\016\023"+static_text, 
                                      align <- vgAlign.MiddleCenter,
                                      formatted <- TRUE);
        rgnBorder := vcRegion.Create ("", vcBorder.fac, rgnStatic);
        
        rgnButton := vcRegion.Create ("name_button", vcButton.fac, 
                                      proc_help (attr_help_str),
                                      tip <- "Information on this attribute");


        vcRegion.AddChild (rgnButton, rgnBorder);
        vcButton.HighlightRegion (rgnButton, rgnBorder);

        vcRegion.FocusInhibit (rgnButton);
        vcRegion.SetCursor (rgnButton, vgCursor.Hand);

        rgnButton;

    }

    function change_cmdline_val(cmdline, popup_region)
        func (val)
        {
            new_value := string_upper(vcRegion.Get (popup_region, vlStringVT));
            vcCmdline.Set(cmdline, new_value);
        };

    function update_regions (curr_region, key)
        func (string)
        {
            /* operate on only the region that originated the change
               by setting the global top_level_update FALSE and resetting 
               it to TRUE at the end.
               */
            if (top_level_update == TRUE)
            {
                top_level_update = FALSE;

                notification_list := vlSym.Fetch (attr_notify_table, key);
                foreach (region; vscript.list_iter (notification_list))
                {
                        vcCmdline.Set (region, string);
                        vcRegion.Foreground (region, vgColor.Blue, FALSE);
                        vcRegion.Refresh(region, vcRegion.Area (region));
                }
                /* indicate that this AttrEd region has been touched */
                data.attr_ed_touched = TRUE;

                top_level_update = TRUE;
            }

        };

    function register_in_notify_list (name, type, region)
    {
        if (!string_eq(type, "Name"))
        {
            key := name+"_"+type;
            notification_list := vlSym.Fetch (attr_notify_table, key);
            vlSym.Store (attr_notify_table, key, 
                         cons(region, notification_list));
        }
    }

    function add_cmdline_editor (ed_name, name, value, type, read_only, table)
    {
        var_name := vcRegion.Create1 (ed_name, 
                                      vcCmdline.fac, 
                                      value,
                                      list(macro_on <- TRUE,
                                           width <- 16,
                                           case <- vcCmdlineCase.Upper,
                                           read_only <- read_only));
        vcRegion.Foreground (var_name, vgColor.Black, FALSE);
        vcCmdline.CallOnChange (var_name, 
                                update_regions(var_name, name+"_"+type)); 
        register_in_notify_list (name, type, var_name);
        vcTable.Add (table, var_name);                
        return (var_name);
    }
    
/* we now present three functions that add various types of 
   entries in the attribute editor. each has a label for the name, 
   an input editor for adding the atribute value and a button
   for popping up help on THAT attribute. these three regions
   are added to a table which is returned by the function.
*/

/*
   add an entry with a vcPickEnum region as the input editor 
   */
    function add_boolean_entry(table, curr_attr, oats_val, type_string)
    {
        attr_name_str  := curr_attr[0];
        attr_check     := curr_attr[4];
        check_params   := curr_attr[5];
        attr_help_str  := curr_attr[6];
        
        attr_name := StaticTextButtonCreate (attr_name_str, attr_help_str);
        vcTable.Add (table, attr_name);
        
        function add_boolean_editor (ed_name, name, value, type)
        {        
            declare conf_cmdline, conf_arrow, var_name;
            function popup_boolean()
                func (event)
                {
                    attr_table := vcTable.Create ("Valid Values", 2, 1);
                    reset_value := vcRegion.Get (conf_cmdline, vlStringVT);
                    reset_fg := vcRegion.GetForeground (conf_cmdline);
                    
                    boolean_editor := vcRegion.Create ("boolean_editor", 
                                                       vcPickEnum.fac,
                                                       vlTrueFalse.False);   
                    vcPickEnum.CallOnChange(boolean_editor, 
                                            change_cmdline_val(conf_cmdline, 
                                                               boolean_editor));
                    vcTable.Add (attr_table, boolean_editor);
                    
                    list_butn_row := vcRegion.Create("list_butn_row", 
                                                     vcButnRow.fac,
                                                     list (list ("Accept", "Accept",
                                                                 func (event) 
                                                                 {
                                                                     accept_value := string_upper(vcRegion.Get (boolean_editor, vlStringVT));
                                                                     vcCmdline.Set(conf_cmdline, accept_value);
                                                                 },
                                                                 list (close_window <- TRUE)),
                                                           list ("Cancel", "Cancel", 
                                                                 func (event) 
                                                                 {
                                                                     vcCmdline.Set(conf_cmdline, reset_value);
                                                                     vcRegion.Foreground (conf_cmdline, 
                                                                                          reset_fg, FALSE);
                                                                     vcRegion.Refresh(conf_cmdline, 
                                                                                      vcRegion.Area (conf_cmdline));
                                                                 },
                                                                 list (close_window <- TRUE))));
                    vcTable.Add (attr_table, list_butn_row);
                    vcWin.PopupAt(vcEvent.Window(event),
                                  attr_table, 
                                  vcEvent.X (event), 
                                  vcEvent.Y (event),
                                  vcWin.NoMenus + 
                                  vcWin.NoCmdline + 
                                  vcWin.NoStatus); 
                };
            
            conf_cmdline = vcRegion.Create1 (ed_name, 
                                             vcCmdline.fac, 
                                             value,
                                             list(macro_on <- TRUE,
                                                  case <- vcCmdlineCase.Upper,
                                                  width <- 16));
            register_in_notify_list (name, type, conf_cmdline);
            vcCmdline.CallOnChange(conf_cmdline, 
                                   update_regions(conf_cmdline, 
                                                  name+"_"+type));
            conf_arrow = ButtonCreate(popup_boolean());
            var_name   = vcTable.Make ("conflict_table", 1, 2, 
                                       conf_cmdline, conf_arrow);
            
            vcTable.Add (table, var_name);                
            return (var_name);
            
        }
        
        if (!(string_eq(type_string, "Net")))
            add_cmdline_editor ("attr_input_editor_sym", 
                                curr_attr[0],
                                curr_attr[1],
                                "Symbol",
                                TRUE, table);
        
        add_boolean_editor ("attr_input_editor_inst",
                            curr_attr[0],
                            curr_attr[2], 
                            "Instance");
        
        if (oats_val == vlOnOff.On)
            add_boolean_editor ("attr_input_editor_oat", 
                                curr_attr[0],
                                curr_attr[3],
                                "Oat");
    }
    
/*
   add an entry with a vcPicklist region as the input editor 
   */
    function add_picklist_entry(table, curr_attr, oats_val, type_string)
    {
        /* add a check for number of elements in the vector */
        attr_name_str  := curr_attr[0];
        attr_check     := curr_attr[4];
        check_params   := curr_attr[5];
        attr_help_str  := curr_attr[6];
        value_list     := curr_attr[8];
        attr_name := StaticTextButtonCreate (attr_name_str, attr_help_str);
        vcTable.Add (table, attr_name);

        function add_picklist_editor (ed_name, name, value, type, val_list)
        {
            declare conf_cmdline, conf_arrow, var_name;

            function popup_picklist(attr_list)
                func (event)
                {
                    attr_table := vcTable.Create ("Valid Values", 2, 1);
                    attr_picklist := vcRegion.Create ("attr_picklist", 
                                                      vcPicklist.fac, 
                                                      attr_list,
                                                      will_scroll <- TRUE);
                    vcPicklist.SetPick(attr_picklist, 0);
                    vcPicklist.EnsureVisible(attr_picklist, 0, 0);
                    reset_value := vcRegion.Get (conf_cmdline, vlStringVT);
                    reset_fg := vcRegion.GetForeground (conf_cmdline);
                    function attr_select(cmdline)
                        func(entry_data, on_off)
                        {
                            vcCmdline.Set(cmdline, entry_data);
                            nil;
                        };

                    vcPicklist.Notify (attr_picklist, attr_select(conf_cmdline));
                    attr_scroll := vcPicklist.Scroll ("attr_scroll", 
                                                      attr_picklist, 
                                                      1, 4, 0);
                    vcTable.Add (attr_table, attr_scroll);
                    list_butn_row := vcRegion.Create("list_butn_row", 
                                                     vcButnRow.fac,
                                                     list (list ("Accept", "Accept",
                                                                 func (event) 
                                                                 {
                                                                     accept_value := string_upper(vcRegion.Get (attr_picklist, vlStringVT));
                                                                     vcCmdline.Set(conf_cmdline, accept_value);
                                                                 },
                                                                 list (close_window <- TRUE)),
                                                           list ("Cancel", "Cancel", 
                                                                 func (event) 
                                                                 {
                                                                     vcCmdline.Set(conf_cmdline, reset_value);
                                                                     vcRegion.Foreground (conf_cmdline, 
                                                                                          reset_fg, FALSE);
                                                                     vcRegion.Refresh(conf_cmdline, 
                                                                                      vcRegion.Area (conf_cmdline));
                                                                 },
                                                                 list (close_window <- TRUE))));
                    

                    vcTable.Add (attr_table, list_butn_row);
                    vcWin.Popup(vcEvent.Window(event),
                                attr_table, list_butn_row,
                                vcWin.NoMenus + 
                                vcWin.NoCmdline + 
                                vcWin.NoStatus);                    
                };
            conf_cmdline = vcRegion.Create1 (ed_name, 
                                             vcCmdline.fac, 
                                             value,
                                             list(macro_on <- TRUE,
                                                  case <- vcCmdlineCase.Upper,
                                                  width <- 16));
            register_in_notify_list (name, type, conf_cmdline);
            vcCmdline.CallOnChange(conf_cmdline, 
                                   update_regions(conf_cmdline, 
                                                  name+"_"+type));
            conf_arrow = ButtonCreate(popup_picklist(val_list));
            var_name   = vcTable.Make ("conflict_table", 1, 2, 
                                       conf_cmdline, conf_arrow);
            
            vcTable.Add (table, var_name);                
            return (var_name);
        }
        
        if (!(string_eq(type_string, "Net")))
            add_cmdline_editor ("attr_input_editor_sym", 
                                curr_attr[0],
                                curr_attr[1],
                                "Symbol",
                                TRUE, table);

        add_picklist_editor ("attr_input_editor_inst", 
                             curr_attr[0], curr_attr[2], 
                             "Instance", value_list);

        if (oats_val == vlOnOff.On)
            add_picklist_editor ("attr_input_editor_oat", 
                                 curr_attr[0], curr_attr[3], 
                                 "Oat", value_list);
    }

    /*
       add an entry with a vcScalebar region as the input editor 
       */
    function add_scalebar_entry(table, curr_attr, oats_val, type_string)
    {
        attr_name_str  := curr_attr[0];
        attr_check     := curr_attr[4];
        check_params   := curr_attr[5];
        attr_help_str  := curr_attr[6];
        mini           := curr_attr[8];
        maxi           := curr_attr[9];
        num_ticks      := curr_attr[10];

        attr_name := StaticTextButtonCreate (attr_name_str, attr_help_str);
        vcTable.Add (table, attr_name);
        
        function add_scalebar_editor (ed_name, name, value, type)
        {
            declare conf_cmdline, conf_arrow, var_name;
            function popup_scalebar()
                func (event)
                {
                    attr_table := vcTable.Create ("Valid Values", 2, 1);
                    reset_value := vcRegion.Get (conf_cmdline, vlStringVT);
                    reset_fg := vcRegion.GetForeground (conf_cmdline);
                    
                    scalebar_editor := vcRegion.Create ("scalebar_editor", 
                                                        vcScalebar.fac, 
                                                        50,
                                                        min <- mini, max <- maxi, 
                                                        ticks <- num_ticks,
                                                        width <- 100);

                    vcScalebar.CallOnChange(scalebar_editor, 
                                            change_cmdline_val(conf_cmdline, 
                                                               scalebar_editor));
                    vcTable.Add (attr_table, scalebar_editor);
                    
                    list_butn_row := vcRegion.Create("list_butn_row", 
                                                     vcButnRow.fac,
                                                     list (list ("Accept", "Accept",
                                                                 func (event) 
                                                                 {
                                                                     accept_value := vcRegion.Get (scalebar_editor, vlStringVT);
                                                                     vcCmdline.Set(conf_cmdline, accept_value);
                                                                 },
                                                                 list (close_window <- TRUE)),
                                                           list ("Cancel", "Cancel", 
                                                                 func (event) 
                                                                 {
                                                                     vcCmdline.Set(conf_cmdline, reset_value);
                                                                     vcRegion.Foreground (conf_cmdline, 
                                                                                          reset_fg, FALSE);
                                                                     vcRegion.Refresh(conf_cmdline, 
                                                                                      vcRegion.Area (conf_cmdline));
                                                                 },
                                                                 list (close_window <- TRUE))));
                    vcTable.Add (attr_table, list_butn_row);
                    vcWin.Popup(vcEvent.Window(event),
                                attr_table, list_butn_row,
                                vcWin.NoMenus + 
                                vcWin.NoCmdline + 
                                vcWin.NoStatus); 
                };
      
            conf_cmdline = vcRegion.Create1 (ed_name, 
                                             vcCmdline.fac, 
                                             value,
                                             list(macro_on <- TRUE,
                                                  case <- vcCmdlineCase.Upper,
                                                  width <- 16));
            register_in_notify_list (name, type, conf_cmdline);
            vcCmdline.CallOnChange(conf_cmdline, 
                                   update_regions(conf_cmdline, 
                                                  name+"_"+type));
            conf_arrow = ButtonCreate(popup_scalebar());
            var_name   = vcTable.Make ("conflict_table", 1, 2, 
                                       conf_cmdline, conf_arrow);
            
            vcTable.Add (table, var_name);                
            return (var_name);
        }

        if (!(string_eq(type_string, "Net")))
            add_cmdline_editor ("attr_input_editor_sym", 
                                curr_attr[0],
                                curr_attr[1],
                                "Symbol",
                                TRUE, table);

        add_scalebar_editor ("attr_input_editor_inst", 
                             curr_attr[0], 
                             curr_attr[2], 
                             "Instance");
        

        if (oats_val == vlOnOff.On)
            add_scalebar_editor ("attr_input_editor_oat", 
                                 curr_attr[0], 
                                 curr_attr[3],
                                 "Oat");

        }


    /*
       add an entry with a vcCmdline region as the input editor 
       */
    function add_cmdline_entry(table, curr_attr, oats_val, type_string)
    {

        attr_name_str  := curr_attr[0];
        attr_val0      := curr_attr[1];
        attr_val1      := curr_attr[2];
        attr_val2      := curr_attr[3];
        attr_check     := curr_attr[4];
        check_params   := curr_attr[5];
        attr_help_str  := curr_attr[6];

        attr_name := StaticTextButtonCreate (attr_name_str, attr_help_str);

        vcTable.Add (table, attr_name);

        if (!(string_eq(type_string, "Net")))
            add_cmdline_editor ("attr_input_editor_sym", 
                                attr_name_str,
                                attr_val0,
                                "Symbol",
                                TRUE, table);

        cmdline_inst := add_cmdline_editor ("attr_input_editor_inst", 
                                            attr_name_str,
                                            attr_val1, 
                                            "Instance",
                                            FALSE, table);

        if (oats_val == vlOnOff.On)
            add_cmdline_editor ("attr_input_editor_oat", 
                                attr_name_str,
                                attr_val2, 
                                "Oat",
                                FALSE, table);
    }
    

    /*
       add an entry with a vcCmdline region as the input editor 
       */
    function add_new_attr_entry(table, new_attr_list, oats_val, type_string)
    {

        curr_attr := make_vector(8);

        curr_attr[0]    = "";
        curr_attr[1]    = "";
        curr_attr[2]    = "";
        curr_attr[3]    = "";
        curr_attr[4]    = sae_check_no_check;
        curr_attr[5]    = nil;
        curr_attr[6]    = sae_help_no_help;
        curr_attr[7]    = "";

        function mark_changed(region)
            func (string)
            {
                /* indicate that this AttrEd region has been touched */
                data.attr_ed_touched = TRUE;

                vcRegion.Foreground (region, vgColor.Blue, FALSE);
                vcRegion.Refresh(region, vcRegion.Area (region));
            };
        
        
        function add_new_entry_cmdline_editor (ed_name, name, value, 
                                               type, read_only, table)
        {
            var_name := vcRegion.Create1 (ed_name, 
                                          vcCmdline.fac, 
                                          value,
                                          list(macro_on <- TRUE,
                                               case <- vcCmdlineCase.Upper,
                                               width <- 16,
                                               read_only <- read_only));
            vcRegion.Foreground (var_name, vgColor.Black, FALSE);
            vcCmdline.CallOnChange (var_name, 
                                    mark_changed(var_name)); 
            vcTable.Add (table, var_name);                
            return (var_name);
        }
        
        
        
        add_new_entry_cmdline_editor ("attr_new_name", curr_attr[0],
                                      curr_attr[1],
                                      "Name",
                                      FALSE, table);
        
        if (!(string_eq(type_string, "Net")))
            add_new_entry_cmdline_editor ("attr_new_sym", 
                                          curr_attr[0],
                                          curr_attr[1],
                                          "Symbol",
                                          TRUE, table);
        
        cmdline_inst := add_new_entry_cmdline_editor ("attr_new_inst", 
                                                      curr_attr[0],
                                                      curr_attr[2], 
                                                      "Instance",
                                                      FALSE, table);
        
        if (oats_val == vlOnOff.On)
            add_new_entry_cmdline_editor ("attr_new_oat", curr_attr[0],
                                          curr_attr[3], 
                                          "Oat",
                                          FALSE, table);
        new_attr_list = cons(curr_attr, new_attr_list);
    }
    
/*
   go thru the entries in the attribute vector attr_vector and add 
   an entry for each one. 
*/
    
    function make_attr_table(attr_vector)
    { 
        num_cols := 4;
        if (oats_val == vlOnOff.Off)
            --num_cols;
        if (string_eq(type_string, "Net"))
            --num_cols;
        heading_Name := vcRegion.Create ("heading_Name", 
                                         vcStaticText.fac, "Name",
                                         align <- vgAlign.MiddleCenter
                                         );
        heading_Symbol := vcRegion.Create ("heading_Symbol", 
                                         vcStaticText.fac, "Symbol Value",
                                         align <- vgAlign.MiddleCenter
                                         );
        heading_Instance := vcRegion.Create ("heading_Instance", 
                                             vcStaticText.fac, 
                                             "Instance Value",
                                             align <- vgAlign.MiddleCenter
                                             );
        heading_Occurrence := vcRegion.Create ("heading_Occurrence", 
                                               vcStaticText.fac, 
                                               "Occurrence Value",
                                               align <- vgAlign.MiddleCenter
                                               );

        heading_table := vcTable.Create ("heading_table", 1, num_cols);
        table := vcTable.Create ("cpattrs", (vector_length (attr_vector) + 1), 
                                 num_cols);

        vcTable.Add (heading_table, heading_Name);

        if (!(string_eq(type_string, "Net")))
            vcTable.Add (heading_table, heading_Symbol);

        vcTable.Add (heading_table, heading_Instance);

        if (oats_val == vlOnOff.On)
            vcTable.Add (heading_table, heading_Occurrence);


        foreach (curr_attr; vscript.vect_iter (attr_vector))
        {
            if (string_eq(curr_attr[7], "Boolean"))
                add_boolean_entry (table, curr_attr, 
                                   oats_val, type_string);
            else 
                if (string_eq(curr_attr[7], "Scalebar"))
                    add_scalebar_entry (table, curr_attr, 
                                        oats_val, type_string); 
                else 
                    if (string_eq(curr_attr[7], "Picklist"))
                        add_picklist_entry (table, curr_attr, 
                                            oats_val, type_string); 
                    else 
                        add_cmdline_entry (table, curr_attr, 
                                           oats_val, type_string);
        }

        new_attr_list = add_new_attr_entry(table, new_attr_list, 
                                           oats_val, type_string);
        new_attr_list = add_new_attr_entry(table, new_attr_list, 
                                           oats_val, type_string);
        
        vcTable.Col(table, 0, vcTable.Border.Invisible, 
                    3, 3, 3, vgAlign.MiddleRight);
        for (i := 1; i < num_cols; ++i)
            vcTable.Col(table, i, vcTable.Border.Invisible, 
                        3, 3, 3, vgAlign.MiddleLeft);

        vcTable.AlignColumns(list(heading_table, table));
        table_scroll := vcTable.Scroll("cpa_scroll", table, 
                                       table, 1, 6, 0);
        whole_table := vcTable.Make ("whole_table", 2, 1,
                                       heading_table, 
                                       table_scroll);

        return (whole_table);

    } /* end function make_attr_table */


    top_table := make_attr_table(attr_vector);
/*
    button_row := vcRegion.Create ("button_row", vcButnRow.fac,
                                   list (list ("Accept", "Accept",
                                               accept_cmd,
                                               list (close_window <- TRUE)),
                                         list ("Cancel", "Cancel", 
                                               cancel_cmd, 
                                               list (close_window <- TRUE))));

    vcTable.Add (top_table, button_row);
*/
    vcRegion.AddChild (region, top_table);

    data.block = block;
    data.top_table = top_table;
    data.obj_list = init_data[1];
    data.attr_vector = attr_vector;
    data.new_attr_list = new_attr_list;
    data.type_string = type_string;
    data.selected_objects = init_data[5];
    data.attr_ed_touched = attr_ed_touched;
    data;

}












