/***************************************************************************
* $Id: corrpt.vs,v 1.5 1994/11/23 17:43:48 sandy Stab $
*
* corrpt.vs
*
* Viewscript file for correspondence file facility
*
*  Author:
*    Sandy Lucas          -- Oct, 1994
*
*  Modified by:
*  Who     Version     When       What
*  SAL     0.1         10/12/94   Original coding.
*
*  Copyright (c) 1994 by Viewlogic Systems, Inc.
*
***************************************************************************/

set_stdout(TRUE);

/* global variables */
declare this;
declare pickindex, pickdata;
pickdata = nil;
pickindex = -1;

/****************************************************************************/
/*  PickNotify                                                              */
/*                                                                          */
/****************************************************************************/
function PickNotify(data, on_or_off, index)
   if (on_or_off == vlOnOff.On) {
      pickindex = index;
      pickdata = data;
   }

/****************************************************************************/
/*                                                                */
/*                                                                          */
/****************************************************************************/
function EnableCmds(on_or_off)
{
      vcCmd.Enable(RemoveSelectedCmd, on_or_off);
      vcCmd.Enable(ProbeSelectedCmd, on_or_off);
      vcCmd.Enable(SaveFileCmd, on_or_off);
}

/****************************************************************************/
/*  _Destroy                                                                */
/*                                                                          */
/****************************************************************************/
function _Destroy( data )
{
   lvsnav.EnableOpenCFileCmd();
   lvsnav.ClearCorr(data.lvsnav);
}
 
/****************************************************************************/
/*  _Create -- create an instance of this facility                          */
/*                                                                          */
/*  DESCRIPTION                                                             */
/*     'data' is list consisting of the lvsnav facility object, the lvs     */
/*           run directory, the correspondence point file, and a boolean    */
/*           indicating whether the file is opened in append or overwrite   */
/*           mode                                                           */
/*     'reg' is our own Region ID, created by Core.                         */
/*                                                                          */
/*  RETURNS                                                                 */
/*     A value that will be passed by Core to all methods and commands.     */
/****************************************************************************/
function _Create(data, reg)
{
   declare filepath;

   this = struct {
     reg;
     lvsnav;
     rundir;
     corrfile;
     table;
     picklist
   };

   this.reg = reg;
   this.lvsnav = cadddr(data);
   this.rundir = car(data);
   this.corrfile = cadr(data);

   append := caddr(data);

   filelab := vcRegion.Create("file", vcLabel.fac, "Correspondence file:");
   filetext := vcRegion.Create("text", vcStaticText.fac, this.corrfile);

   infotable := vcTable.Create("info", 1, 2);
   vcTable.Add(infotable, filelab);
   vcTable.Add(infotable, filetext);

   /* create a picklist */
   this.picklist = vcRegion.Create("picklist", vcPicklist.fac, nil, 
                                    rows <- 1, 
                                    columns <- 1,
                                    macro_on <- TRUE,
                                    enhanced_notify <- TRUE);
   vcPicklist.Notify(this.picklist, PickNotify);

   /* check if the filename is a path or just a file */
   if ((this.corrfile[0] != '/') && (this.corrfile[0] != '~') &&
       (this.corrfile[0] != '.'))
      filepath = this.rundir + "/" + this.corrfile;
   else filepath = this.corrfile;

   picks := FALSE;

   /* append mode and file exists, read the data first and make a picklist*/
   if (append && fexists(filepath)) {
      fp := fopen(filepath, "r");
      line := fgetline(fp);
      while (line != nil) {  
        line = string_trim(line);
        if (!string_eq(line, "")) {
           l := ParsePickData(line);
           layobj := car(l);
           schobj := cadr(l);
           if (layobj != nil)
              lvsnav.AddLayCorr(this.lvsnav, layobj);
           if (schobj != nil)
              lvsnav.AddSchCorr(this.lvsnav, schobj);

           picks = TRUE;
           vcPicklist.Add(this.picklist, line, line);
        }
        line = fgetline(fp);
      }
   }

   /* if there are no items in the picklist, disable the other commands*/
   if (!picks)
      EnableCmds(vlOnOff.Off);
   else EnableCmds(vlOnOff.On);

   /* select the first item in the picklist to make sure one is selected */
   vcPicklist.SetPick(this.picklist, 0);
   scroll := vcPicklist.Scroll("scroll", this.picklist, 5, 5, 0);

   /* now create a table to hold all of this */
   this.table = vcTable.Create("table", 2, 1);
   vcTable.Add(this.table, infotable);
   vcTable.Add(this.table, scroll);

   vcRegion.AddChild(reg, this.table);
   this;  
   
}

function CorrFileName(this)
   this.corrfile;

function SaveFile(this, event, filename)
{
   declare row, col, index, is_set, visual, data;
   declare filepath;

   /* check if the filename is a path or just a file */
   if ((filename[0] != '/') && (filename[0] != '~') &&
       (filename[0] != '.'))
      filepath = this.rundir + "/" + filename;
   else filepath = filename;
   fp := fopen(filepath, "w");
   if (fp == nil)
      ReportErr1(lvsnav, 3041, filepath);
   else {
      foreach( row, col, index, is_set, visual, data; 
            vcPicklist.Items(this.picklist)) {
         fputline(fp, data);
         fputline(fp, "\n");
      }
      fclose(fp);
   }
}

SaveFileCmd := vcWv.Cmd(fac, SaveFile, "SaveFile", "SaveFile",
   vcWv.Param("File Name", vlStringVT, vcWv.Dynamic, CorrFileName));

function ParsePickData(pickdata)
{
   declare token1, token2;

   if (pickdata == nil)
      ;
   else {
      str := pickdata;
      str = string_token(str, &token1, " ()");
      str = string_token(str, &token2, " ()");
      l := list(substring(token1,1,0), substring(token2,1,0));
   }   
}

function ProbeSelected(this, event)
{
   declare token1, token2;

   if (pickdata == nil)
      ;
   else {
      str := pickdata;
      str = string_token(str, &token1, " ()");
      str = string_token(str, &token2, " ()");
      if (token1[0] == '#' && token2[0] == '#') {
         lvsnav.XprobeLayNet(this.lvsnav, substring(token1,1,0));
         lvsnav.XprobeSchNet(this.lvsnav, substring(token2,1,0), 
            substring(token1,1,0));
      }
      else if (token1[0] == '$' && token2[0] == '$') {
         lvsnav.XprobeLayInst(this.lvsnav, substring(token1,1,0));
         lvsnav.XprobeSchInst(this.lvsnav, substring(token2,1,0), 
            substring(token1,1,0));
      }
      /* error number 3034 must match LVSX_PICKSYNTAXERR in lvsnav.vmt */
      else lvsnav.ReportErr1(this.lvsnav, 3034, pickdata);
  }
}

ProbeSelectedCmd := vcWv.Cmd(fac, ProbeSelected, "ProbeSelected");

function RemoveSelected(this, event)
{
   declare rows,cols;

   if (pickindex != -1) {
      l := ParsePickData(pickdata);
      layobj := car(l);
      schobj := cadr(l);
      if (layobj != nil)
         lvsnav.DelLayCorr(this.lvsnav, layobj);
      if (schobj != nil)
         lvsnav.DelSchCorr(this.lvsnav, schobj);

      vcPicklist.DeleteRows(this.picklist, pickindex, 1);
      vcPicklist.SetPick(this.picklist,0);

      /* see if there are still items in the picklist */
      vcPicklist.Dimensions(this.picklist, &rows, &cols);
      if (rows == 0) {
         EnableCmds(vlOnOff.Off);
         vcRegion.Refresh(this.picklist, vcRegion.Area(this.picklist));
         vcWin.UpdateMenus(vcRegion.Window(this.picklist));
      }    
   } 
}

RemoveSelectedCmd := vcWv.Cmd(fac, RemoveSelected, "RemoveSelected");

function AddtoPicklist(str)
{
   declare rows,cols;

   schadded := TRUE;
   layadded := TRUE;

   if (!string_eq(str, "")) {
      /* parse the string for the 2 object tokens */
      l := ParsePickData(str);
      layobj := car(l);
      schobj := cadr(l);
      /* see if they are already in the list of objects */
      if (layobj != nil) {
         layadded = lvsnav.InLayCorrFile(this.lvsnav, layobj);
      }
      if (schobj != nil) {
         schadded = lvsnav.InSchCorrFile(this.lvsnav, schobj);
      }
      /* if not, add the string to the picklist & the objects to the lists */
      if ((layadded ==0) && (schadded==0)) {
         lvsnav.AddSchCorr(this.lvsnav, schobj);
         lvsnav.AddLayCorr(this.lvsnav, layobj);
      
         vcPicklist.Add(this.picklist, str, str);
         vcPicklist.Dimensions(this.picklist, &rows, &cols);
         vcPicklist.SetPick(this.picklist, rows-1);
         EnableCmds(vlOnOff.On);
         vcRegion.Refresh(this.picklist, vcRegion.Area(this.picklist));
         vcWin.UpdateMenus(vcRegion.Window(this.picklist));
      }
      /* else write an error message */
      else {
         if (layadded == 1)
            lvsnav.ReportErr1(this.lvsnav, 3035, layobj);
         if (schadded == 1)
            lvsnav.ReportErr1(this.lvsnav, 3035, schobj);
      }
   }

}

function AddCorr(this, event)
{
   lvsnav.AddCorrPt(this.lvsnav);
      
}

AddCorrCmd := vcWv.Cmd(fac, AddCorr, "AddCorr");

load("corrptmn.vs");
