/*
 * COUNT.C:     Sample user DRC extension #1
 *
 * This extension adds a rule that counts components on each schematic
 * page, and reports the count as a note on each schematic.
 */

#include "vdrc.h"

/*
 * do_count is called once for each schematic sheet, each time the schematic
 * is changed.  It simply counts the components, and reports an error.
 * 
 * The error is reported with respect to count_error_class, which is created
 * in vdrcCountInit, and passed as "client data" to the test registration
 * routine vdrcAddObjTest.  Whatever is passed into vdrcObjTest will be
 * passed through to the test routine as the first argument.
 */
static void count_components(count_error_class, drc, sch)
    vdrcErrorID count_error_class;
    vdrcID drc;
    SCHEMATICS *sch;
{
    COMPONENTS *comp;
    int count = 0;
    char  *buf;

    for (comp = igschcom (sch); comp != NULL; comp = igcomnxt (comp))
        ++count;

    buf = vlSprintfAlloc ("Schematic %s.%d: %d components",
                          igfullnam (igschown (sch)), igschsht (sch), count);

    vdrcReport (drc, count_error_class, (vlPointerT) sch, vdrcSCH, buf);
    vlStrFree (buf);
} /* static void count_components */

/*
 * This function is called by drcuser.c to initialize the test.  Note
 * that in order to report an error, an error class must be created
 * with vdrccErrorCreate.  This error class is passed as client data
 * through to the test function, count_components.
 */
void vdrcCountInit(drc, argcp, argv)
    vdrcID drc;
    int *argcp;
    char **argv;
{
    vdrcErrorID count_error_class = 
        vdrcErrorCreate (drc, "CompCount", vdrcTestAUTO_ASSIGN, vlErrNOTE,
                        "Number of components on schematic");
    vdrcAddObjTest (drc, vdrcSCH, count_components,
                    (vlPointerT) count_error_class);
} /* void vdrcCountInit */
