/*                       */
/*  lvs.vs               */
/*                       */
/*  Viewscript gui for LVS Navigator*/
/*                       */
/*                       */
set_stdout(TRUE);

declare lvsobj;
declare sbut1win, sbut2win, sbut3win, sbut4win, sbut5win, sbut6win, sbut7win;
declare lbut1win, lbut2win, lbut3win, lbut4win, lbut5win, lbut6win, lbut7win;

/***************************************************************************
*
* function error_type -
* converts the filename to an error type for use in labeling dialog boxes
*
***************************************************************************/
function error_type( filename )
{
   if (string_eq(filename, "netbad.out"))
      return("unmatched nets");
   else if (string_eq(filename, "devbad.out"))
      return("unmatched insts");
   else if (string_eq(filename, "audit.out"))
      return("unmatched attrs");
   else if (string_eq(filename, "termbad.out"))
      return("unmatched terminals");
   else if (string_eq(filename, "mergenet.out"))
      return("merged nets");
   else if (string_eq(filename, "prunenet.out"))
      return("ignored nets");
   else if (string_eq(filename, "prunedev.out"))
      return("ignored devices");
   else return("");
}

/***************************************************************************
*
* function errorFilePath -
* returns the filename associated with an error file region, uses the
* region name as a key to the file name
*
***************************************************************************/
function errorFilePath(regionname)
{
   declare filename;
   rundir := lvsobj.rundir;
   if (string_eq(regionname,"sbut1"))
      filename = rundir + "/schematic/netbad.out";
   else if (string_eq(regionname,"sbut2"))
      filename = rundir + "/schematic/devbad.out";
   else if (string_eq(regionname,"sbut3"))
      filename = rundir + "/schematic/audit.out";
   else if (string_eq(regionname,"sbut4"))
      filename = rundir + "/schematic/termbad.out";
   else if (string_eq(regionname,"sbut5"))
      filename = rundir + "/schematic/mergenet.out";
   else if (string_eq(regionname,"sbut6"))
      filename = rundir + "/schematic/prunenet.out";
   else if (string_eq(regionname,"sbut7"))
      filename = rundir + "/schematic/prunedev.out";
   else if (string_eq(regionname,"lbut1"))
      filename = rundir + "/layout/netbad.out";
   else if (string_eq(regionname,"lbut2"))
      filename = rundir + "/layout/devbad.out";
   else if (string_eq(regionname,"lbut3"))
      filename = rundir + "/layout/audit.out";
   else if (string_eq(regionname,"lbut4"))
      filename = rundir + "/layout/termbad.out";
   else if (string_eq(regionname,"lbut5"))
      filename = rundir + "/layout/mergenet.out";
   else if (string_eq(regionname,"lbut6"))
      filename = rundir + "/layout/prunenet.out";
   else if (string_eq(regionname,"lbut7"))
      filename = rundir + "/layout/prunedev.out";

   filename;
}

/***************************************************************************
*
* function setButnWin -
* stores the window associated with the error file picklist so that resetting
* the button in the main dialog window will close the picklist window also
* uses global variables hardcoded to the region name
*
***************************************************************************/
function setButnWin( win, regname )
   if (string_eq(regname,"sbut1"))
        sbut1win = win;
   else if (string_eq(regname,"sbut2"))
        sbut2win = win;
   else if (string_eq(regname,"sbut3"))
        sbut3win = win;
   else if (string_eq(regname,"sbut4"))
        sbut4win = win;
   else if (string_eq(regname,"sbut5"))
        sbut4win = win;
   else if (string_eq(regname,"sbut6"))
        sbut6win = win;
   else if (string_eq(regname,"sbut7"))
        sbut7win = win;
   else if (string_eq(regname,"lbut1"))
        lbut1win = win;
   else if (string_eq(regname,"lbut2"))
        lbut2win = win;
   else if (string_eq(regname,"lbut3"))
        lbut3win = win;
   else if (string_eq(regname,"lbut4"))
        lbut4win = win;
   else if (string_eq(regname,"lbut5"))
        lbut5win = win;
   else if (string_eq(regname,"lbut6"))
        lbut6win = win;
   else if (string_eq(regname,"lbut7"))
        lbut7win = win;


/***************************************************************************
*
* function getButnWin -
* returns the window associated with the error file picklist region, uses
* global variables hardcoded to the region name
*
***************************************************************************/
function getButnWin( regname )
{
   declare win;
   if (string_eq(regname,"sbut1"))
        win = sbut1win;
   else if (string_eq(regname,"sbut2"))
        win = sbut2win;
   else if (string_eq(regname,"sbut3"))
        win = sbut3win;
   else if (string_eq(regname,"sbut4"))
        win = sbut4win;
   else if (string_eq(regname,"sbut5"))
        win = sbut4win;
   else if (string_eq(regname,"sbut6"))
        win = sbut6win;
   else if (string_eq(regname,"sbut7"))
        win = sbut7win;
   else if (string_eq(regname,"lbut1"))
        win = lbut1win;
   else if (string_eq(regname,"lbut2"))
        win = lbut2win;
   else if (string_eq(regname,"lbut3"))
        win = lbut3win;
   else if (string_eq(regname,"lbut4"))
        win = lbut4win;
   else if (string_eq(regname,"lbut5"))
        win = lbut5win;
   else if (string_eq(regname,"lbut6"))
        win = lbut6win;
   else if (string_eq(regname,"lbut7"))
        win = lbut7win;
   win;
}

function ErrButton(lvsobj, event, regname)
{


   declare child;

   filename := errorFilePath(regname);
   /* parse the filename to extract the type of netlist and type of error*/
   prevtok := nil;
   curtok := nil;
   token := nil;
   foreach(token; string_loop(filename, "/")) {
      prevtok = curtok;
      curtok = token;
   }
   
   error_name := error_type(curtok);

   reg := vcRegion.Create(prevtok + " " + error_name + " (" +curtok + ")", 
              lvsErrF.fac, 
              list(filename, lvsobj.lvsnav, vcEvent.Region(event)));
   butnWin := vcWin.Create(vcEvent.Window(event), reg, vcWin.NoBindings);
   setButnWin(butnWin, regname);

}

ErrButtonCmd := vcWv.Cmd(fac, ErrButton, "ErrButton", "Error Button",
    vcWv.Param("Region Name", vlStringVT, vcWv.Sticky, ""));

/***************************************************************************
*
* function do_button -
* displays the picklist dialog for the particular error file selected from
* the main dialog
*
***************************************************************************/
function do_button(event, butn_status)
{
   if (butn_status) {
      regname := vcRegion.Name(vcEvent.Region(event));
      vcCmd.Run(ErrButtonCmd, lvsobj, event, list(regname), TRUE);
   }
   else {
      butnWin := getButnWin(regname);
      vcWin.Destroy(butnWin);
   }

}

/***************************************************************************
*
* function setenable -
* checks if the error file associated with the button is either nonexistent
* or has a 0 file length and if so disables the button
*
***************************************************************************/
function setenable(butn)
{
   regname := vcRegion.Name(butn);
   filepath := errorFilePath(regname);
   if (!fexists(filepath))
      vcRegion.Enable(butn, vlOnOff.Off, TRUE);
   else {
      infolist := vscript.vsInfo(filepath);
      filesize := cadr(infolist);
      if (filesize == 0)
         vcRegion.Enable(butn, vlOnOff.Off, TRUE);
   }
}

/***************************************************************************
*
* function createErrorTable -
* Creates buttons for each of the error file types, determines whether the
* file is empty to enable it or not and adds the buttons to a table
*
***************************************************************************/
function createErrorTable(lvsobj )
{
   lvserrfiles := vcRegion.PrefGet(lvsobj.reg, "lvserr");

   /* schematic error buttons */
   text1 := vcRegion.Create("Unmatched nets", vcLabel.fac, 
                            "Unmatched nets (netbad.out)");
   text2 := vcRegion.Create("Unmatched insts", vcLabel.fac, 
                            "Unmatched insts (devbad.out)");
   text3 := vcRegion.Create("Unmatched attrs", vcLabel.fac, 
                            "Unmatched attrs (audit.out)");
   text4 := vcRegion.Create("Unmatched terminals", vcLabel.fac, 
                            "Unmatched terminals (termbad.out)");
   text5 := vcRegion.Create("Merged nets", vcLabel.fac, 
                            "Merged nets (mergenet.out)");
   text6 := vcRegion.Create("Ignored nets", vcLabel.fac, 
                            "Ignored nets (prunenet.out)");
   text7 := vcRegion.Create("Ignored devices", vcLabel.fac, 
                            "Ignored devices (prunedev.out)");

   sbut1 := vcRegion.Create("sbut1", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(sbut1);
   sbut2 := vcRegion.Create("sbut2", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(sbut2);
   sbut3 := vcRegion.Create("sbut3", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(sbut3);
   sbut4 := vcRegion.Create("sbut4", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(sbut4);
   sbut5 := vcRegion.Create("sbut5", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(sbut5);
   sbut6 := vcRegion.Create("sbut6", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(sbut6);
   sbut7 := vcRegion.Create("sbut7", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(sbut7);

   lbut1 := vcRegion.Create("lbut1", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(lbut1);
   lbut2 := vcRegion.Create("lbut2", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(lbut2);
   lbut3 := vcRegion.Create("lbut3", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(lbut3);
   lbut4 := vcRegion.Create("lbut4", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(lbut4);
   lbut5 := vcRegion.Create("lbut5", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(lbut5);
   lbut6 := vcRegion.Create("lbut6", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(lbut6);
   lbut7 := vcRegion.Create("lbut7", vcToggleButton.fac, FALSE,
                            action<-do_button, macro_on<-TRUE);

   setenable(lbut7);

   table := vcTable.Create ("table", 7, 3);
   vcRegion.PrefSet(table, list(macro_on <- TRUE));
   if (string_search(lvserrfiles, "netbad")) {
      vcTable.Add (table, text1);
      vcTable.Add (table, lbut1);
      vcTable.Add (table, sbut1);
   }
   if (string_search(lvserrfiles, "devbad")) {
      vcTable.Add (table, text2);
      vcTable.Add (table, lbut2);
      vcTable.Add (table, sbut2);
   }
   if (string_search(lvserrfiles, "audit")) {
      vcTable.Add (table, text3);
      vcTable.Add (table, lbut3);
      vcTable.Add (table, sbut3);
   }
   if (string_search(lvserrfiles, "termbad")) {
      vcTable.Add (table, text4);
      vcTable.Add (table, lbut4);
      vcTable.Add (table, sbut4);
   }
   if (string_search(lvserrfiles, "mergenet")) {
      vcTable.Add (table, text5);
      vcTable.Add (table, lbut5);
      vcTable.Add (table, sbut5);
   }
   if (string_search(lvserrfiles, "prunenet")) {
      vcTable.Add (table, text6);
      vcTable.Add (table, lbut6);
      vcTable.Add (table, sbut6);
   }
   if (string_search(lvserrfiles, "prunedev")) {
      vcTable.Add (table, text7);
      vcTable.Add (table, lbut7);
      vcTable.Add (table, sbut7);
   }
   return(table);
}

/***************************************************************************
*
* function reinitErrorButtons -
* Creates a new error button table and resets the toptable with the new
* error button table - lvsobj is used in reinitialization
*
***************************************************************************/
function reinitErrorButtons(lvsobj)
{
   table := createErrorTable(lvsobj);

   lvsobj.errortab = table;
   vcTable.Set(lvsobj.toptable, 3, 0, lvsobj.errortab);
   
}

/***************************************************************************
*
* function updateRundir - modifies the text in the GUI with a new text
*     string - used during reinitialization
*
***************************************************************************/
function updateRundir(lvsobj, rundir)
{
   lvsobj.rundir = rundir;
   vcStaticText.Clear(lvsobj.rundirtext);
   vcStaticText.Add(lvsobj.rundirtext, rundir);
   vcRegion.Refresh(lvsobj.rundirtext, vcRegion.Area(lvsobj.rundirtext));
}

/***************************************************************************
*
* function _Get
* return the facility object
* (I don't know of any other way to get it )
*
***************************************************************************/
function _Get(lvsobj, reg, desired_type)
{
   return(lvsobj);
}

/****************************************************************************/
/*  _Create -- create an instance of this facility                          */
/*                                                                          */
/*  DESCRIPTION                                                             */
/*     'data' is list consisting of the lvsnav facility object, the lvs     */
/*           run directory, the top design name, and the project path       */
/*     'reg' is our own Region ID, created by Core.                         */
/*                                                                          */
/*  RETURNS                                                                 */
/*     A value that will be passed by Core to all methods and commands.     */
/****************************************************************************/
function _Create(data, reg)
{

   lvsobj = struct {
      reg;
      lvsnav;
      rundir;
      rundirtext;
      projpath;
      design;
      toptable;
      errortab;
      txshellreg
   };

   lvsobj.reg = reg;
   /* get the info from the args passed into the create method */
   lvsobj.projpath = cadddr(data);
   lvsobj.design = caddr(data);
   lvsobj.rundir = cadr(data);
   lvsobj.lvsnav = car(data);

   numlines := vcRegion.PrefGet(lvsobj.reg, "n");

   /* create the text indicating the current environment */
   rundirlab := vcRegion.Create ("text", vcLabel.fac,"LVS run directory");
   lvsobj.rundirtext = vcRegion.Create ("text", vcStaticText.fac, 
                                  lvsobj.rundir);
   projdirlab := vcRegion.Create ("text", vcLabel.fac, 
                                  "Viewlogic project path");
   projdirtext := vcRegion.Create ("text", vcStaticText.fac,
                                   lvsobj.projpath);

   designlab := vcRegion.Create ("text", vcLabel.fac, "Top design");
   designtext := vcRegion.Create ("text", vcStaticText.fac, lvsobj.design);

   /* create the table for these text regions */
   dirtab := vcTable.Create("table", 3, 2);
   vcTable.Add (dirtab, projdirlab);
   vcTable.Add (dirtab, projdirtext);
   vcTable.Add (dirtab, designlab);
   vcTable.Add (dirtab, designtext);
   vcTable.Add (dirtab, rundirlab);
   vcTable.Add (dirtab, lvsobj.rundirtext);

   /* error navigation buttons table */
   lvsobj.errortab = createErrorTable(lvsobj);

   /* this needs some alignment work */
   errlab := vcRegion.Create("label", vcLabel.fac, 
                 "Select type of errors you wish to view");
   blank := vcRegion.Create("blank", vcLabel.fac,
               "                                                      ");
   colhead1 := vcRegion.Create("colhead", vcLabel.fac,
               "from Layout");
   colhead2 := vcRegion.Create("colhead", vcLabel.fac,
               "from Schematic");
   headertable := vcTable.Create("", 1, 3);

   vcTable.Add(headertable, blank);
   vcTable.Add(headertable, colhead1);
   vcTable.Add(headertable, colhead2);

   /* create the region where messages will be written */
   lvsobj.txshellreg = vcRegion.Create("output", vcTxShell.fac, "",
                                     width <- 60,
                                     height <- 10);
   vcTxShell.InhibitInput(lvsobj.txshellreg, FALSE);

   /* create the top table */
   lvsobj.toptable = vcTable.Create ("table", 5, 1);
   vcRegion.PrefSet(lvsobj.toptable, list (macro_on <- TRUE));

   vcTable.Row(lvsobj.toptable, 1, vcTable.Border.Visible, 10, 10, 10, 
      vgAlign.MiddleCenter);
   vcTable.Add(lvsobj.toptable, dirtab);
   vcTable.Add(lvsobj.toptable, errlab);
   vcTable.Add(lvsobj.toptable, headertable);
   vcTable.Add(lvsobj.toptable, lvsobj.errortab);
   vcTable.Add(lvsobj.toptable, lvsobj.txshellreg);

   vcRegion.AddChild (lvsobj.reg, lvsobj.toptable);
   lvsobj;
}

/****************************************************************************
*
* function ReinitErrButn - recreate the error navigation buttons and redisplay
*
****************************************************************************/
function ReinitErrButn(lvsobj, event)
{
   reinitErrorButtons(lvsobj);
   vcRegion.Refresh(lvsobj.toptable, vcRegion.Area(lvsobj.toptable));
}

ReinitErrButnCmd := vcWv.Cmd(fac, ReinitErrButn, "ReinitErrButn");

/****************************************************************************
*
* add preferences to the lvs facility
*
****************************************************************************/
vlPref.Add(prefs, "lvserr", vlStringVT, "Selectable LVS Error Files",
  "devbad termbad audit netbad mergenet prunedev prunenet" ); 
vlPref.Add(prefs, "n", vlIntVT, "Number of error lines to display", 100);

/* create the lvsErrR and corrpt facilities - loads in the viewscript */
vcFac.Create("lvsErrF", vscmVT);
vcFac.Create("corrpt", vscmVT);
