/* -----------------------------------------------------------------------*\
/* Module -- paramtrs.vs
 *
 *      Facility:      Parameters
 *
 *      Environment:   SUN4, Powerview 5.2
 *
 *      Authors:       Mani Alagappan,  Apr 20, 1994
 *                     Jeff Hare
 *
 *      Description:   ViewTime Timing parameters editor facility
 *
 *      Copyright (c) 1994 by Viewlogic Systems, Inc.
 *
\* ---------------------------------------------------------------------- */
/*\Section paramtrs -- ViewTime Timing Parameters Editor Facility
*/

/* ----------------------------------------------------------------------
 * Notes: The latch tracing mode "NEVER" was misnamed.  It should
 *         be labelled "Combinatorial" or something like that since
 *         it actually causes latches to be treated like normal gates
 *         and traces through all levels.  Never obviously implied that
 *         latch tracing is disabled.  Even though Motive uses the
 *         latch tracing mode "disabled" to mean "combinatorial".
 * ----------------------------------------------------------------------
 */



function label(str) vcRegion.Create1(str,vcLabel.fac,str,nil);


CYCLE := "Cycle";
STEP  := "Stepsize";

/*
   Report Options 
*/
ALLMARGINSstr := "All Margins";
VIOLATIONSstr := "Violations Only";
ALL           := "all";
VIO           := "vio";

OCCWARNstr    := "Occ. Warnings";
ZCPMSGstr        := "ZCP Messages";
OCC           := "Occmissing";
ZCP           := "Zcpwarn";

/*
   Trace Options
*/
EXPLMODEstr    := "Explanation Mode";
DETAILMODEstr  := "Detailed Report";
D2QMODEstr     := "Force D -> Q";
SLACKMODEstr   := "Include Slack Info";

EXPL := "Explanation";
D2Q := "Forced2q";
DETAIL := "Detail";
SLACK := "Slack";
SCOUNT := "Scount";

/* 
   General Options
*/
CLKGATstr  := "Clock Gating";
RECONVstr  := "Reconvergence";
SIGPstr    := "Sig(p) Loops";
BIDIIOstr  := "Re-entrant Nets";
BIDIPINstr := "Bidir Pin Tracing";
GSLACKstr  := "Global Slack";

CLK := "Clockgating";
REC := "Reconvergence";
SIG := "Sigp";
BIO := "Bidiio";
BPN := "Bidipin";
GSR := "Gsr";
/*
   Latch & LARIAT
*/
SETUP := "setup ";
HOLD  := "Hold ";
SKEW  := "Skew ";
SMARGIN  := "Smargin ";
HMARGIN  := "Hmargin ";
SKMARGIN  := "Skmargin ";

LATCH := "Latchmode";
LDEPTH := "Ldepth";
ALWAYSstr := "Always Trace";
SETUPstr  := "Setup Trace";
SETHLDstr := "Setup & Hold Trace";
NEVERstr  := "Treat as Combinatorial";
S   := "s";
SH := "sh";
LARIATstr := "LARIAT Mode";
LARIAT := "Lariat";

MISSIDDstr := "Summarize Missing IDDs";
MISSIDD := "missidd";

ALW := "a";
SET:= "s";
STHD := "sh";
NEV := "never";
LAR := "";

/*
   Interconnect Delay
*/
IDDstr := "  Use IDD  ";

/*
   All purpose macros
*/

DISABLE       := "Disable";
ENABLE        := "Enable";

ON  := vlOnOff.On;
OFF := vlOnOff.Off;


/*
   Interconnect options 
*/
NONEstr :=     "        None      ";
XLINEstr :=    " Transmission Line ";
CAPPLACEstr := "Cap. from Placement";
CAPFANstr   := "  Cap. from Fanout ";

NON := 0;
XLN := 1;
PLC := 2;
FAN := 3;
IDD := 4;

NONEcmd  := "None";
XLINEcmd := "Xline";
CAPPcmd  := "Capp";
CAPFcmd  := "Capf";
IDDcmd   := "Idd";

LINEARstr := "Linear";
BILINEARstr := "Bi-Linear";

LIN := 0;
BILIN := 1;

LINEARcmd   := "Linear";
BILINEARcmd := "Bilinear";

RUN := 0;
ADD := 1;

/* Display a msg on the main parameter window */
function StatusMsg(e, msg){
   win := vcEvent.Window (e);
   vcWin.ClearStatus (win);
   vcWin.AddStatus (win,msg);
}

function SubmitCmds(queue,event,cmds, run_or_add ){
   cmds = reverse_destructive (cmds);

   foreach (i;vscript.list_iter(cmds)){
      cmd := car (i) + " " + cadr (i);
      if (run_or_add == RUN)
         vtq.CmdRun (queue, event, cmd);
      else
         vtq.Add (queue, event, cmd);
   }
}

/*=====================================================*/
/*==           R e s t a r t C m d                   ==*/
/*=====================================================*/
RestartCmd := vcWv.Cmd (fac, 
                       func(d,e,yesno){
                          if (yesno == convert (vlYesNoVT, "yes")){
									  Queue := vtime.GetQueue (d.VtimeData);
                             
									  vtq.Add (Queue,d.VtimeEvent,"Restartmtv yes");
                             SubmitCmds (Queue, d.VtimeEvent, d.TimeBaseCmds, ADD);
                             SubmitCmds (Queue, d.VtimeEvent, d.IntconCmd, ADD);
                             SubmitCmds (Queue, d.VtimeEvent, d.IddCmd, ADD);
                          }
								  vcWin.Close (vcEvent.Window (e));
                       },
                        "restart", 
                        "Timebase/Interconnect Parameters Modified",
                        vcWv.param("Restart Analysis to take effect?",
                                   vlYesNoVT, vcWv.Constant, "yes"));
vcCmd.EnableMacro (RestartCmd, vlOnOff.On);

function GetIntconType(str){
   if (string_eqi (str,NONEcmd))
       NON;
   else if (string_eqi (str,XLINEcmd))
      XLN;
   else if (string_eqi (str,CAPPcmd))
      PLC;
   else if (string_eqi (str,CAPFcmd))
      FAN;
   else if (string_eqi (str,IDDcmd))
      IDD;
}
function GetCalcType(str){
   if (string_eqi (str,LINEARcmd))
      LIN;
   else
      BILIN;
}
function UpdateIddMsg(s)
{
   msg := "";
   if ( s.IntconType == NON){
      if ( s.IddType == IDD)
         msg = "Note: Only IDD file is used to calculate NET delays"; 
      else
         msg = "Note: NO Interconnect delay for all NETS";
   }
   else if ( s.IntconType == FAN)
      if ( s.IddType == IDD)
         msg = "Note: IDD file and Cap. from fanout are used to calculate NET delays";          
      else
         msg = "Note: Cap. from fanout method is used to calculate NET delays";          
   
   else if ( s.IntconType == PLC)
      if ( s.IddType == IDD)
         msg = "Note: IDD file and Cap. from placement are used to calculate NET delays";          
      else
         msg = "Note: Cap. from placement method is used to calculate NET delays";          
   
   else if ( s.IntconType == XLN)
      if ( s.IddType == IDD)
         msg = "Note: IDD file and Transmission Line data are used to calculate NET delays";          
      else
         msg = "Note: Transmission Line data is used to calculate NET delays";          
   
   vcRegion.Set (s.IddMsgRgn, msg);
   
}
/*=====================================================*/
/*==          C a l l O n P i c k 1                  ==*/
/*=====================================================*/
function CallOnPick(data, on_or_off) {
   if (isprocedure(cdr(data))) {
      (cdr(data))(car (data),on_or_off);
   }
   else 
      vlEx.Throw2(vlEx.FAILURE,"vtime",vlErrSev.Error,
                  "No action specified for this pick in tracedlg.BuildParamsGui()");
} 

function InterconnectDelay(s,e)
{
   declare IntconTypeRgn;       /*  */
   declare LocFileCmdRgn;       /*  */
   declare LocFileRgn;          /*  */

   declare CalcTypeRgn;         /*  */
   declare CalcRgn;             /*  */
   declare DfltPinCapCmdRgn;    /*  */
   declare DfltPinCapRgn;       /*  */
   declare RouteFactorCmdRgn;   /*  */
   declare RouteFactorRgn;      /*  */
   declare TraceCapCmdRgn;      /*  */
   declare TraceCapRgn;         /*  */
   declare TraceVelCmdRgn;      /*  */
   declare TraceVelRgn;         /*  */

   declare FanoutRgn;           /*  */
   declare PicoCmdRgn;          /*  */
   declare PicoRgn;             /*  */
   declare Seg1OffCmdRgn;       /*  */
   declare Seg1OffRgn;          /*  */
   declare Seg1PfCmdRgn;        /*  */
   declare Seg1PfRgn;           /*  */
   declare BreakptCmdRgn;       /*  */
   declare BreakptRgn;          /*  */
   declare Seg2OffCmdRgn;       /*  */
   declare Seg2OffRgn;          /*  */
   declare Seg2PfCmdRgn;        /*  */
   declare Seg2PfRgn;           /*  */   
   declare RisMinCmdRgn, RisMaxCmdRgn, FallMinCmdRgn, FallMaxCmdRgn;
   declare RisMinRgn, RisMaxRgn, FallMinRgn, FallMaxRgn;
   declare MissiddTypeRgn;       /*  */

   CALC  := 1;
   NCALC := 0;

   ActiveTable := CALC;

   declare Col1, Col2,Row2;

   function UseLocFile(flag)vcRegion.Enable(LocFileRgn, flag, TRUE);
   function UseDfltPinCap (flag) vcRegion.Enable (DfltPinCapRgn , flag, TRUE);
   function UseRouteFactor(flag) vcRegion.Enable (RouteFactorRgn , flag, TRUE);
   function UseTraceCap(flag) vcRegion.Enable (TraceCapRgn , flag, TRUE);
   function UseTraceVel(flag) vcRegion.Enable (TraceVelRgn , flag, TRUE);
   function UseCalc(flag) vcRegion.Enable (CalcRgn, flag, TRUE);
   function UseFanout(flag) vcRegion.Enable (FanoutRgn , flag, TRUE);
   function UsePico(flag) vcRegion.Enable (PicoRgn , flag, TRUE);

   function UseBilinearData(flag){
      vcRegion.Enable (Seg1PfRgn, flag, TRUE);
      vcRegion.Enable (Seg1OffRgn, flag, TRUE);
      vcRegion.Enable (BreakptRgn, flag, TRUE);
      vcRegion.Enable (Seg2PfRgn, flag, TRUE);
      vcRegion.Enable (Seg2OffRgn, flag, TRUE);
   }
   
   function set_none(data,onoff){
      if (onoff == ON){
         s.IntconType = data;
         SetTable ();
         UpdateIntconParameters (NON);
         UseDfltPinCap (OFF);
         UseLocFile (OFF);
         UseRouteFactor(OFF);
         UseTraceCap(OFF);
         UseTraceVel(OFF);
         UseCalc(OFF);
         UseFanout(OFF);
         if (s.IddType == IDD){            
            s.IntconCmd = cons (list ("Interconnect", "None"),
                                s.IntconCmd);
            UseIddCmd ();
         }
         else
            s.IntconCmd = list (list ("Interconnect", "None"));
      }
   }

   function XlineCmd()
   {
      if (s.XlineUpdated == TRUE){
         args := sprintf ("%s %s %s %s %s", s.XlineDfltPinCap,
                          s.XlineRouteFactor,s.XlineTraceCap,
                          s.TraceVel,s.XlineLocFile);
         s.IntconCmd = list (list (XLINEcmd, args));
      }
   }

   function set_xline(data,onoff){
      if (onoff == ON){
         s.IntconType = data;
         SetTable ();
         UpdateIntconParameters (XLN);
         UseDfltPinCap (ON);
         UseLocFile (ON);
         UseRouteFactor(ON);
         UseTraceCap(ON);
         UseTraceVel(ON);
         UseCalc(OFF);
         UseFanout(OFF);
			XlineCmd();
      }
   }

   function CappCmd()
   {
      if (s.CappUpdated == TRUE){
         args := sprintf ("%s %s %s %s", s.CappDfltPinCap,
                          s.CappRouteFactor,s.CappTraceCap,
                          s.CappLocFile);
         s.IntconCmd =  list (list (CAPPcmd, args));
      }
   }

   function set_capplace(data,onoff){
      if (onoff == ON){
         s.IntconType = data;
         SetTable ();
         UpdateIntconParameters (PLC);
         UseDfltPinCap (ON);
         UseLocFile (ON);
         UseRouteFactor(ON);
         UseTraceCap(ON);
         UseTraceVel(OFF);
         UseCalc(OFF);
         UseFanout(OFF);
         CappCmd ();
      }            
   }

   function CapfCmd()
   {
      if (s.CapfUpdated == TRUE){
         args := sprintf ("%s %s %s %s %s %s %s %s", 
                          s.CapfDfltPinCap,s.PicoPerFan,
                          s.CalcType == LIN ? LINEARcmd : BILINEARcmd,
                          s.Seg1Off, s.Seg1Pf,
                          s.Breakpt, s.Seg2Off, s.Seg2Pf);
         s.IntconCmd =  list (list (CAPFcmd, args));
      }
   }

   function set_capfan(data,onoff){
      if (onoff == ON){
         s.IntconType = data;
         SetTable ();
         UpdateIntconParameters (FAN);
         UseDfltPinCap (ON);
         UseLocFile (OFF);
         UseRouteFactor(OFF);
         UseTraceCap(OFF);
         UseTraceVel(OFF);
         UseCalc(ON);
         UseFanout(ON);
			/*print (s.CalcType);*/
         CapfCmd ();
      }      
   }

   function set_linear(data,onoff){
      if (onoff == ON){
         s.CalcType = data;
         UsePico (ON);
         UseBilinearData(OFF);
         CapfCmd ();
      }
   }
   
   function set_bilinear(data,onoff){
      if (onoff == ON){
         s.CalcType = data;
         UsePico (OFF);
         UseBilinearData(ON);
         CapfCmd ();
      }
   }
   
   function UseParamsTable()
      vcTable.Set (Row2,0,0,Col1);
   function UseIddTable()
      vcTable.Set (Row2,0,0,Col2);

   function SetTable (){
      if ( s.IddType == IDD){
         if ( s.IntconType == NON){
            if (ActiveTable == CALC){
               UseIddTable ();
               ActiveTable = NCALC;
            }
         }
         else if (ActiveTable == NCALC){
            UseParamsTable ();
            ActiveTable = CALC;
         }
      }
   }

   function CheckLocFile(value)
   {
      if ( s.IntconType == XLN){
         s.XlineLocFile = value;
         XlineCmd ();
      }
      else if ( s.IntconType == PLC){
         s.CappLocFile = value;
         CappCmd ();
      }
   }
   function CheckDfltPinCap(value){
      if ( s.IntconType == XLN){
         s.XlineDfltPinCap = value;
         XlineCmd ();         
      }
      else if ( s.IntconType == PLC){
         s.CappDfltPinCap = value;
         CappCmd ();
      }
      else if ( s.IntconType == FAN){
         s.CapfDfltPinCap = value;
         CapfCmd ();
      }      
   }

   function CheckRouteFactor(value){
      if ( s.IntconType == XLN){
         s.XlineRouteFactor = value;
         XlineCmd ();
      }
      else if ( s.IntconType == PLC){
         s.CappRouteFactor = value;
         CappCmd ();
      }
   }

   function CheckTraceCap(value){
      if ( s.IntconType == XLN){
         s.XlineTraceCap = value;
         XlineCmd ();
      }
      else if ( s.IntconType == PLC){
         s.CappTraceCap = value;
         CappCmd ();
      }
   }

   function CheckTraceVel(value)
   {
      s.TraceVel = value;
      XlineCmd ();
   }

   function CheckSeg2Pf(value){
      s.Seg2Pf = value;
      CapfCmd ();
   }

   function CheckSeg2Off(value){
      s.Seg2Off = value;
      CapfCmd ();
   }

   function CheckBreakpt(value){
      s.Breakpt = value;
      CapfCmd ();
   }

   function CheckSeg1Pf(value){
      s.Seg1Pf = value;
      CapfCmd ();
   }

   function CheckSeg1Off(value){
      s.Seg1Off = value;
      CapfCmd ();
   }

   function CheckPico(value){
      s.PicoPerFan = value;
      CapfCmd ();
   }


   function UseIddCmd(){
      function GetIddOrNone(){
         if ( s.IntconType == NON){
            if ( s.IddType == IDD)
               "Yes";
         }
         else
            "No";
      }
      if (s.IddUpdated == TRUE){
         args := sprintf ("%s %s %s %s %s %s %s", 
                          "Yes" /*GetIddOrNone()*/,s.Missidd, s.RisMin, s.RisMax,
                          s.FallMin, s.FallMax, s.IddFile);
         s.IntconCmd = list (list (IDDcmd, args));
      }
   }

   function CheckRisMin(value)
   {
      s.RisMin = value;
      UseIddCmd ();
   }
   function CheckRisMax(value)
   {
      s.RisMax = value;
      UseIddCmd ();
   }
   function CheckFallMin(value)
   {
      s.FallMin = value;
      UseIddCmd ();
   }
   function CheckFallMax(value)
   {
      s.FallMax = value;
      UseIddCmd ();
   }
   function set_missidd(data, onoff)
   {
      if (onoff == OFF) s.Missidd = DISABLE;
      else              s.Missidd = ENABLE;      
      UseIddCmd ();
   }



   /*==================================================*\ 
    * This function builds the netire interconnect GUI *
   \*==================================================*/
   function BuildIntconGui(){
      
      /*--------------------------------*\
       *      Interconnect options       *
       \*--------------------------------*/
      NONE := cons (NONEstr, cons (NON, set_none));
      XLINE := cons (XLINEstr, cons (XLN, set_xline));
      CAPPLACE := cons (CAPPLACEstr, cons (PLC, set_capplace));
      CAPFAN := cons (CAPFANstr, cons (FAN, set_capfan));
      
      intcon_types := list (list (NONE, XLINE), list (CAPPLACE, CAPFAN));
      /*intcon_types := list (list (NONE, XLINE,CAPPLACE, CAPFAN));*/
      IntconTypeRgn     = vcRegion.Create("intcon", 
                                          vcPicklist.fac,
                                          intcon_types,
                                          macro_on <- TRUE);
      vcPicklist.Notify (IntconTypeRgn, CallOnPick);
      
      IntconTitleRgn  := vcRegion.Create("", vcLabel.fac, "Additional Calculations");
      vcRegion.Font (IntconTitleRgn, vgFont.mediumb);
      
      LocFileTitleRgn := vcRegion.Create("xy",vcLabel.fac, "XY Placement File: ");
      LocFileCmdRgn    = vcRegion.Create("loc",vcFilePop.fac, "");
      vcCmdline.CallOnChange(LocFileCmdRgn, CheckLocFile); 
      LocFileRgn = vcTable.Make ("",1,2, LocFileTitleRgn,LocFileCmdRgn);
      
      Row1 :=  vcRegion.Create("addl", vcMargin.fac,
                             vcTable.Make("TraceArgEditor", 2, 1,
                                          vcTable.Make ("",1,2,
                                                        IntconTitleRgn,
                                                        IntconTypeRgn),
                                          LocFileRgn));
      
      /*--------------------------------*\
       *  Delay Calculation Parameters  *
       \*--------------------------------*/
      CalcTitleRgn  := vcRegion.Create("", vcLabel.fac, "Calc. Method");
      vcRegion.Font (CalcTitleRgn, vgFont.mediumb);
      
      LINEAR := cons (LINEARstr, cons (LIN, set_linear));
      BILINEAR := cons (BILINEARstr, cons (BILIN, set_bilinear));
      
      calc_types := list (list (LINEAR, BILINEAR));
      CalcTypeRgn     = vcRegion.Create("calc", 
                                        vcPicklist.fac,
                                        calc_types,
                                        macro_on <- TRUE);
      vcPicklist.Notify (CalcTypeRgn, CallOnPick);
      
      DfltPinCapTitleRgn := vcRegion.Create("",vcLabel.fac, "Default Pin Cap (pf): ");
      DfltPinCapCmdRgn    = vcRegion.Create("DfltPinCap", vcCmdline.fac, "",
                                            max_width <- 12, width <- 9,
                                            macro_on <- TRUE);
      DfltPinCapRgn = vcRegion.Create ("", vcMargin.fac,
                                       vcTable.Make ("", 1,2,   
                                                     DfltPinCapTitleRgn,
                                                     DfltPinCapCmdRgn));
      vcCmdline.CallOnChange(DfltPinCapCmdRgn, CheckDfltPinCap); 
      
      RouteFactorTitleRgn := vcRegion.Create("",vcLabel.fac, "Route Factor: ");
      RouteFactorCmdRgn    = vcRegion.Create("RouteFactor", vcCmdline.fac, "",
                                             max_width <- 12, width <- 9,
                                             macro_on <- TRUE);
      RouteFactorRgn = vcRegion.Create ("", vcMargin.fac,
                                        vcTable.Make ("", 1,2,   
                                                      RouteFactorTitleRgn,
                                                      RouteFactorCmdRgn));
      vcCmdline.CallOnChange(RouteFactorCmdRgn, CheckRouteFactor);
      
      TraceCapTitleRgn := vcRegion.Create("",vcLabel.fac, "Trace Capacitance (pf): ");
      TraceCapCmdRgn    = vcRegion.Create("TraceCap", vcCmdline.fac, "",
                                          max_width <- 12, width <- 9,
                                          macro_on <- TRUE);
      TraceCapRgn = vcRegion.Create ("", vcMargin.fac,
                                     vcTable.Make ("", 1,2,   
                                                   TraceCapTitleRgn,
                                                   TraceCapCmdRgn));
      vcCmdline.CallOnChange(TraceCapCmdRgn, CheckTraceCap); 
      
      TraceVelTitleRgn := vcRegion.Create("",vcLabel.fac, "Trace Velocity (ns/ft): ");
      TraceVelCmdRgn    = vcRegion.Create("TraceVel", vcCmdline.fac, "",
                                          max_width <- 12, width <- 9,
                                          macro_on <- TRUE);
      TraceVelRgn = vcRegion.Create ("", vcMargin.fac,
                                     vcTable.Make ("", 1,2,   
                                                   TraceVelTitleRgn,
                                                   TraceVelCmdRgn));
      vcCmdline.CallOnChange(TraceVelCmdRgn, CheckTraceVel); 
      
      Col1 = vcRegion.Create ("", vcBorder.fac, vcTable.Make ("", 7,1,
                                                              CalcRgn = vcTable.Make("",1,2,
                                                                                   CalcTitleRgn,
                                                                                   CalcTypeRgn),
                                                             DfltPinCapRgn, 
                                                             RouteFactorRgn,
                                                             TraceCapRgn,
                                                             TraceVelRgn,
                                                             label (""),
                                                             label ("")));
      /*--------------------------------*\
       * Build IDD table *
      \*--------------------------------*/
      RisMinTitleRgn := vcRegion.Create("",vcLabel.fac, "Rising Min (ns) ");
      RisMinCmdRgn    = vcRegion.Create("RisMin", vcCmdline.fac, "",
                                            max_width <- 12, width <- 9,
                                        macro_on <- TRUE);
      RisMinRgn = vcRegion.Create ("", vcMargin.fac,
                                       vcTable.Make ("", 1,2,   
                                                     RisMinTitleRgn,
                                                     RisMinCmdRgn));
      vcCmdline.CallOnChange(RisMinCmdRgn, CheckRisMin); 

      FallMinTitleRgn := vcRegion.Create("",vcLabel.fac, "Rising Max (ns) ");
      FallMinCmdRgn    = vcRegion.Create("FallMin", vcCmdline.fac, "",
                                            max_width <- 12, width <- 9,
                                         macro_on <- TRUE);
      FallMinRgn = vcRegion.Create ("", vcMargin.fac,
                                       vcTable.Make ("", 1,2,   
                                                     FallMinTitleRgn,
                                                     FallMinCmdRgn));
      vcCmdline.CallOnChange(FallMinCmdRgn, CheckFallMin); 
      
      RisMaxTitleRgn := vcRegion.Create("",vcLabel.fac, "Falling Min (ns) ");
      RisMaxCmdRgn    = vcRegion.Create("RisMax", vcCmdline.fac, "",
                                            max_width <- 12, width <- 9,
                                        macro_on <- TRUE);
      RisMaxRgn = vcRegion.Create ("", vcMargin.fac,
                                       vcTable.Make ("", 1,2,   
                                                     RisMaxTitleRgn,
                                                     RisMaxCmdRgn));
      vcCmdline.CallOnChange(RisMaxCmdRgn, CheckRisMax); 

      FallMaxTitleRgn := vcRegion.Create("",vcLabel.fac, "Falling Max (ns) ");
      FallMaxCmdRgn    = vcRegion.Create("FallMax", vcCmdline.fac, "",
                                            max_width <- 12, width <- 9,
                                         macro_on <- TRUE);
      FallMaxRgn = vcRegion.Create ("", vcMargin.fac,
                                       vcTable.Make ("", 1,2,   
                                                     FallMaxTitleRgn,
                                                     FallMaxCmdRgn));
      vcCmdline.CallOnChange(FallMaxCmdRgn, CheckFallMax); 

      MISSIDD    := cons (MISSIDDstr, cons (LAR, set_missidd));
      missidd_types := list (list (MISSIDD));
      MissiddTypeRgn     = vcRegion.Create("idd_summary",
                                           vcPicklist.fac,
                                           missidd_types, 
                                           exclusive <- FALSE,
                                           macro_on <- TRUE);
      vcPicklist.Notify (MissiddTypeRgn, CallOnPick);

      Col2 = vcRegion.Create ("", vcBorder.fac, vcTable.Make ("", 6,1,
                            tt := vcRegion.Create("",vcLabel.fac, "IDD Only Parameters"),
                                                              MissiddTypeRgn,
                                                              RisMinRgn,
                                                              RisMaxRgn,
                                                              FallMinRgn,
                                                              FallMaxRgn));
      
      vcRegion.Font (tt, vgFont.mediumb);
      /*--------------------------------*\
       *  Linear/Bi-Linear settings     *
       \*--------------------------------*/
      
      PicoTitleRgn := vcRegion.Create("",vcLabel.fac, "Picofards/Fanout: ");
      PicoCmdRgn    = vcRegion.Create("PfPerFanout", vcCmdline.fac, "",
                                      max_width <- 12, width <- 9,
                                      macro_on <- TRUE);
      PicoRgn = vcRegion.Create ("", vcMargin.fac,
                                 vcTable.Make ("", 1,2,   
                                               PicoTitleRgn,
                                               PicoCmdRgn));
      vcCmdline.CallOnChange(PicoCmdRgn, CheckPico); 
      
      Seg1OffTitleRgn := vcRegion.Create("",vcLabel.fac, "Segment 1 Offset: ");
      Seg1OffCmdRgn    = vcRegion.Create("Seg1Off", vcCmdline.fac, "",
                                         max_width <- 12, width <- 9,
                                         macro_on <- TRUE);
      Seg1OffRgn = vcRegion.Create ("", vcMargin.fac,
                                    vcTable.Make ("", 1,2,   
                                                  Seg1OffTitleRgn,
                                                  Seg1OffCmdRgn));
      vcCmdline.CallOnChange(Seg1OffCmdRgn, CheckSeg1Off); 
      
      Seg1PfTitleRgn := vcRegion.Create("",vcLabel.fac, "Segment 1 pf/fanout: ");
      Seg1PfCmdRgn    = vcRegion.Create("Seg1Pf", vcCmdline.fac, "",
                                        max_width <- 12, width <- 9,
                                        macro_on <- TRUE);
      Seg1PfRgn = vcRegion.Create ("", vcMargin.fac,
                                   vcTable.Make ("", 1,2,   
                                                 Seg1PfTitleRgn,
                                                 Seg1PfCmdRgn));
      vcCmdline.CallOnChange(Seg1PfCmdRgn, CheckSeg1Pf); 
      
      BreakptTitleRgn := vcRegion.Create("",vcLabel.fac, "Fanout Break Point: ");
      BreakptCmdRgn    = vcRegion.Create("Breakpt", vcCmdline.fac, "",
                                         max_width <- 12, width <- 9,
                                         macro_on <- TRUE);
      BreakptRgn = vcRegion.Create ("", vcMargin.fac,
                                    vcTable.Make ("", 1,2,   
                                                  BreakptTitleRgn,
                                                  BreakptCmdRgn));
      vcCmdline.CallOnChange(BreakptCmdRgn, CheckBreakpt); 
      
      Seg2OffTitleRgn := vcRegion.Create("",vcLabel.fac, "Segment 2 Offset: ");
      Seg2OffCmdRgn    = vcRegion.Create("Seg2Off", vcCmdline.fac, "",
                                         max_width <- 12, width <- 9,
                                         macro_on <- TRUE);
      Seg2OffRgn = vcRegion.Create ("", vcMargin.fac,
                                    vcTable.Make ("", 1,2,   
                                                  Seg2OffTitleRgn,
                                                  Seg2OffCmdRgn));
      vcCmdline.CallOnChange(Seg2OffCmdRgn, CheckSeg2Off); 
      
      Seg2PfTitleRgn := vcRegion.Create("",vcLabel.fac, "Segment 2 pf/fanout: ");
      Seg2PfCmdRgn    = vcRegion.Create("Seg2Pf", vcCmdline.fac, "",
                                        max_width <- 12, width <- 9,
                                        macro_on <- TRUE);
      Seg2PfRgn = vcRegion.Create ("", vcMargin.fac,
                                   vcTable.Make ("", 1,2,   
                                                 Seg2PfTitleRgn,
                                                 Seg2PfCmdRgn));
      vcCmdline.CallOnChange(Seg2PfCmdRgn, CheckSeg2Pf); 
      
      FanoutRgn = vcRegion.Create ("", vcBorder.fac, vcTable.Make ("", 9,1,
                                                                   PicoRgn,
                                                                   Seg1OffRgn,
                                                                   Seg1PfRgn,
                                                                   BreakptRgn,
                                                                   Seg2OffRgn,
                                                                   Seg2PfRgn));
      

      Row2 = vcRegion.Create ("", vcMargin.fac, vcTable.Make ("", 1, 2,Col1, FanoutRgn));

      /*--------------------------------*\
       *   Define a row of two buttons  *
       \*--------------------------------*/
      function Accept(){
         StatusMsg(e,"");
         UpdateIddMsg (s);
      }
      function Cancel(){
         s.FetchValues = TRUE;
			s.IntconCmd  = nil;
         StatusMsg(e,"Interconnect mode settings are unchanged.");
      }
      
      buttons := vcRegion.Create("Buttons", vcButnRow.fac,
                                 list (list ("AcceptI",
                                             "Accept",
                                             Accept,
                                             list (close_window <- TRUE,
                                                   macro_on     <- TRUE)),
                                       list ("CancelI",
                                             "Cancel",
                                             Cancel,
                                             list (close_window <- TRUE,
                                                   macro_on     <- TRUE))),
                                 macro_on <- TRUE);
      
      SubTitleRgn  := vcRegion.Create("", vcLabel.fac, "Delay Calculation Parameters");
      vcRegion.Font (SubTitleRgn, vgFont.mediumb);
      
      L := vcRegion.Create ("Interconnect Delay Calculations", 
                            vcMargin.fac, vcTable.Make ("", 4, 1, 
                                                        Row1,
                                                        SubTitleRgn,
                                                        Row2,
                                                        buttons));
      vcTable.Row (L, -1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);
      
      L;
   }
   /*==================================================*\ 
    * Fetch all current values from vtime data object  *
   \*==================================================*/
   function FetchIntconParameters()
   {
      declare value, number;

      /* XLINE */
      s.XlineLocFile = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_xline_file_loc,
                                       vtime.MtvValue.VALUE);
      s.XlineDfltPinCap = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_xline_def_pin_cap,
                                          vtime.MtvValue.VALUE);
      s.XlineRouteFactor = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_xline_route_factor,
                                           vtime.MtvValue.VALUE);
      s.XlineTraceCap = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_xline_trc_cap,
                                   vtime.MtvValue.VALUE);
      s.TraceVel = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_xline_trc_vel,
                                   vtime.MtvValue.VALUE);
      
      /* CAPACITIVE FROM PLACEMENT */
      s.CappLocFile = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_capp_file_loc,
                                      vtime.MtvValue.VALUE);
      s.CappDfltPinCap = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.
                                         MV_iam_capp_def_pin_cap,
                                         vtime.MtvValue.VALUE);
      s.CappRouteFactor = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.
                                          MV_iam_capp_route_factor,
                                          vtime.MtvValue.VALUE);
      s.CappTraceCap = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_capp_trc_cap,
                                       vtime.MtvValue.VALUE);

      /* CAPACITIVE FROM FANOUT */      
      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_capf_cap_func,
                                   vtime.MtvValue.VALUE);
      s.CalcType = GetCalcType (value);

      s.CapfDfltPinCap = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.
                                         MV_iam_capf_def_pin_cap,
                                         vtime.MtvValue.VALUE);
      s.PicoPerFan = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_capf_lin_slope,
                                     vtime.MtvValue.VALUE);
      s.Seg1Off = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_capf_bi_seg1_offset,
                                  vtime.MtvValue.VALUE);
      s.Seg1Pf = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_capf_bi_seg1_slope,
                                 vtime.MtvValue.VALUE);
      s.Breakpt = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_capf_bi_break_point,
                                  vtime.MtvValue.VALUE);
      s.Seg2Off = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_capf_bi_seg2_offset,
                                  vtime.MtvValue.VALUE);
      s.Seg2Pf = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_capf_bi_seg2_slope,
                                 vtime.MtvValue.VALUE);

      if ( s.IddType == IDD){
         s.RisMin = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_default_lhmin,
                                    vtime.MtvValue.VALUE);
         s.RisMax = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_default_lhmax,
                                    vtime.MtvValue.VALUE);
         s.FallMin = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_default_hlmin,
                                    vtime.MtvValue.VALUE);
         s.FallMax = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_default_hlmax,
                                    vtime.MtvValue.VALUE);
         s.Missidd = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_summary,
                                     vtime.MtvValue.VALUE);
      }
   }

   /*==================================================*\ 
    * Update the necessary interconnect data info.     *
   \*==================================================*/
   function UpdateIntconParameters(mode)
   {
      if ( mode == NON){
         if ( s.IddType == IDD){
            vcCmdline.Set (RisMinCmdRgn,s.RisMin);
            vcCmdline.Set (RisMaxCmdRgn,s.RisMax);
            vcCmdline.Set (FallMinCmdRgn,s.FallMin);
            vcCmdline.Set (FallMaxCmdRgn,s.FallMax);
            if (string_eqi (s.Missidd,ENABLE))
               vcPicklist.SetPick(MissiddTypeRgn,0);
            s.IddUpdated = TRUE;
         }
         ;
      }
      else if ( mode == XLN){
         
         vcCmdline.Set (LocFileCmdRgn,s.XlineLocFile);
         vcCmdline.Set (DfltPinCapCmdRgn,s.XlineDfltPinCap);
         vcCmdline.Set (RouteFactorCmdRgn,s.XlineRouteFactor);
         vcCmdline.Set (TraceCapCmdRgn,s.XlineTraceCap);
         vcCmdline.Set (TraceVelCmdRgn,s.TraceVel);
         s.XlineUpdated = TRUE;
      }
      else if ( mode == PLC){
         vcCmdline.Set (LocFileCmdRgn,s.CappLocFile);
         vcCmdline.Set (DfltPinCapCmdRgn,s.CappDfltPinCap);
         vcCmdline.Set (RouteFactorCmdRgn,s.CappRouteFactor);
         vcCmdline.Set (TraceCapCmdRgn,s.CappTraceCap);
         s.CappUpdated = TRUE;         
      }
      else if ( mode == FAN){
         if ( s.CalcType == LIN)
            vcPicklist.SetPick (CalcTypeRgn,0);
         else if ( s.CalcType == BILIN)
            vcPicklist.SetPick (CalcTypeRgn,1);

         vcCmdline.Set (DfltPinCapCmdRgn,s.CapfDfltPinCap);
         vcCmdline.Set (PicoCmdRgn,s.PicoPerFan);
         vcCmdline.Set (Seg1OffCmdRgn,s.Seg1Off);
         vcCmdline.Set (Seg1PfCmdRgn,s.Seg1Pf);
         vcCmdline.Set (BreakptCmdRgn,s.Breakpt);
         vcCmdline.Set (Seg2OffCmdRgn,s.Seg2Off);
         vcCmdline.Set (Seg2PfCmdRgn,s.Seg2Pf);
         s.CapfUpdated = TRUE;
      }
   }

   /*==================================================*\ 
    * Update the necessary defaults, and set dflt pick *
   \*==================================================*/
   function UpdateIntconGui()
   {
      declare value, number;

      if (s.FetchValues == TRUE){
         FetchIntconParameters ();
		}

      s.XlineUpdated = FALSE;
      s.CappUpdated  = FALSE;
      s.CapfUpdated  = FALSE;
      /*s.IddUpdated   = FALSE;*/

      if ( s.IntconType == NON || s.IntconType == IDD)
         vcPicklist.SetPick (IntconTypeRgn,NON);
      else if ( s.IntconType == XLN)
         vcPicklist.SetPick (IntconTypeRgn,XLN);
      else if ( s.IntconType == PLC)
         vcPicklist.SetPick (IntconTypeRgn,PLC);
      else if ( s.IntconType == FAN)
         vcPicklist.SetPick (IntconTypeRgn,FAN);


      if (s.FetchValues == TRUE){
         s.FetchValues = FALSE;
         s.IntconCmd  = nil;
      }
   }

   /*==================================================*\ 
    * The action for interconnect analysis begins here *
   \*==================================================*/
   vcWin.Busy (vcEvent.Window (e), TRUE);
   StatusMsg (e, "Building Interconnect Delay Window");
   layout := BuildIntconGui ();
   StatusMsg (e, "Querying interconnect values");
   UpdateIntconGui ();
   StatusMsg (e, "Querying interconnect values...done");
   w := vcWin.Create (layout);
   vcWin.Busy (vcEvent.Window (e), FALSE);
   vcWin.Forward (vcEvent.Window (e), w);
   
   w;

} /* InterconnectDelay */

/*
   Define a command for idd parameters. 
   This way the GUI is upward compatible.
   */
IntconParams := vcWv.Cmd (fac, InterconnectDelay,"_IntconParams", 
                       "Interconnect Delay Calculation Parameter Setup");

/*===========================================================================*/
/*==                       _ C r e a t e                                   ==*/
/*===========================================================================*/
/*\Function _Create  -- Parameters facility data creation function
  
  DESCRIPTION
  RETURNS
       None.

  EXAMPLE
  CAUTION
  SEE ALSO
       
*/
function _Create(Defaults, TopRegion)
{

   /*--------------------------------------------------*\
    *  The following region declarations are global to
    *  all the functions defined in _Create()
   \*--------------------------------------------------*/
   declare ReportTypeRgn;           /*  */
   declare OccZcpTypeRgn;           /*  */
   declare SetupMarginCmdRgn;      /*  */
   declare HoldMarginCmdRgn;       /*  */
   declare SkewMarginCmdRgn;       /*  */

   declare TraceTypeRgn;        /*  */
   declare SlackCountCmdRgn;    /*  */
   declare SlackCountRgn;

   declare CycleTimeCmdRgn;     /*  */
   declare TimeStepCmdRgn;      /*  */

   declare GeneralTypeRgn;      /*  */
   declare LatchTypeRgn;        /*  */
   declare LatchDepthCmdRgn;    /*  */
   declare LatchDepthRgn;    /*  */
   declare LariatTypeRgn;       /*  */

   declare IddTypeRgn;          /* */
   declare IddFileCmdRgn;       /*  */
   declare IddFileRgn;          /*  */
   declare IddMsgRgn;           /*  */
   declare XConRgn;             /*  */

   /* Report modes */
   function set_reporttype(data,onoff)
      if (onoff == ON){
         settings.ReportType = data;
         AddCommand (settings, "Reporttype",settings.ReportType);
      }

   function set_occwarn(data, onoff)
   {
      if (onoff == OFF) settings.OccWarn = DISABLE;
      else              settings.OccWarn = ENABLE;
      AddCommand (settings, OCC, settings.OccWarn);
   }
   function set_zcpmsg (data, onoff)
   {
      if (onoff == OFF) settings.ZcpMsg = DISABLE;
      else              settings.ZcpMsg = ENABLE;
      AddCommand (settings, ZCP, settings.ZcpMsg);
   }
   
   function CheckSetupMargin(value){
      settings.SetupMargin = value;
      AddCommand (settings, SMARGIN, value);
   }
   function CheckHoldMargin(value){
      settings.HoldMargin  = value;
      AddCommand (settings, HMARGIN, value);
   }
   function CheckSkewMargin(value) {
      settings.SkewMargin  = value;
      AddCommand (settings, SKMARGIN, value);
   }

   /* Trace modes */
   function set_explmode(data, onoff)
   {
      if (onoff == OFF) settings.ExplMode = DISABLE;
      else              settings.ExplMode = ENABLE;
      AddCommand (settings, data, settings.ExplMode);
   }
   function set_detailmode(data, onoff)
   {
      if (onoff == OFF) settings.DetailMode = DISABLE;
      else              settings.DetailMode = ENABLE;
      AddCommand (settings, data, settings.DetailMode);
   }
   function set_d2qmode(data, onoff)
   {
      if (onoff == OFF) settings.D2qMode = DISABLE;
      else              settings.D2qMode = ENABLE;
      AddCommand (settings, D2Q, settings.D2qMode);
   }
   function set_slackmode(data, onoff)
   {
      if (onoff == OFF) settings.SlackMode = DISABLE;
      else              settings.SlackMode = ENABLE;
      UseSlackCount (onoff);
      AddCommand (settings, SLACK, settings.SlackMode);      
   }
   function UseSlackCount(flag) vcRegion.Enable(SlackCountRgn, flag, TRUE);
   function CheckSlackCount(value){
      settings.SlackCount = value;
      AddCommand (settings, SCOUNT,value);
   }

   /* time base */
   function CheckCycleTime(value)
   {
      settings.CycleTime = value;
      AddTimebaseCmd (settings, CYCLE, settings.CycleTime);      
   }
   function CheckTimeStep(value) 
   {
      settings.TimeStep = value;
      AddTimebaseCmd (settings, STEP, settings.TimeStep);      
   }

   /* General modes */
   function set_clkgat(data, onoff)
   {
      if (onoff == OFF) settings.Clkgating = DISABLE;
      else              settings.Clkgating = ENABLE;
      AddCommand (settings, CLK, settings.Clkgating);      
   }
   function set_reconv(data, onoff)
   {
      if (onoff == OFF) settings.Reconvergence = DISABLE;
      else              settings.Reconvergence = ENABLE;
      AddCommand (settings, REC, settings.Reconvergence);
   }
   function set_sigp(data, onoff)
   {
      if (onoff == OFF) settings.Sigp = DISABLE;
      else              settings.Sigp = ENABLE;
      AddCommand (settings, SIG, settings.Sigp);
   }
   function set_bidiio(data, onoff)
   {
      if (onoff == OFF) settings.Bidiio = DISABLE;
      else              settings.Bidiio = ENABLE;
      AddCommand (settings, BIO, settings.Bidiio);
   }
   function set_bidipin(data, onoff)
   {
      if (onoff == OFF) settings.Bidipin = DISABLE;
      else              settings.Bidipin = ENABLE;
      AddCommand (settings, BPN, settings.Bidipin);
   }

   function set_gsr(data, onoff)
   {
      if (onoff == OFF) settings.Gsr = DISABLE;
      else              settings.Gsr = ENABLE;
      AddCommand (settings, GSR, settings.Gsr);
   }
   

   /* latch modes */
   function set_latchmode(data,onoff){
      function gettype(value){
         if (string_eqi (value,NEV))
            "Never";
         else if (string_eqi (value,SET))
            "Setup";
         else if (string_eqi (value,STHD))
            "Setup_Hold";
         else
            "Always";
      }

      if (onoff == ON){
         settings.LatchType = data;
         if (string_eqi (data,NEV) == TRUE)
            UseLatchDepth (OFF);
         else
            UseLatchDepth (ON);
         AddCommand (settings, LATCH, gettype(settings.LatchType));
      }
   }

   function UseLatchDepth(flag){
      vcRegion.Enable(LatchDepthRgn, flag, TRUE);
   }
   function CheckLatchDepth(value){
      settings.LatchDepth = value;
      AddCommand (settings, LDEPTH, value);
   }

   function set_lariat(data, onoff)
   {
      if (onoff == OFF) settings.Lariat = DISABLE;
      else              settings.Lariat = ENABLE;      
      AddCommand (settings, LARIAT, settings.Lariat);
   }

   /* Interconnect analysis */
   function UseIddCmd(){
      function GetIddOrNone(){
         if ( settings.IddType == IDD)
            return ("Yes");
         else
            return ("No");
      }
      args := sprintf ("%s %s %s %s %s %s %s", 
                       GetIddOrNone(),settings.Missidd, settings.RisMin, 
                       settings.RisMax, settings.FallMin, settings.FallMax, 
                       settings.IddFile);
      settings.IddCmd = list (list (IDDcmd, args));
   }

   function set_idd(data, onoff)
   {
      if (onoff == OFF) settings.IddType = NON;
      else              settings.IddType = data;
      UseIddFile (onoff);
      UpdateIddMsg (settings);
      UseIddCmd ();

   }
   function UseIddFile(flag)
   {
      vcRegion.Enable(IddFileRgn, flag, TRUE);
   }
   function CheckIddFile(value){
      settings.IddFile = value;
      UseIddCmd ();
   }



   /*====================================================*/
   /*==       BuildParamsGui ==*/
   /*====================================================*/
   /*\Function BuildParamsGui  -- Establish the Dialog box
     
     DESCRIPTION
          Builds the verification results GUI.
     RETURNS
     
   */
   function BuildParamsGui(s)      
   {  
      /*---------------------------------*\
        * Verification Report settings
      \*---------------------------------*/

      ALLMARGINS := cons (ALLMARGINSstr, cons (ALL, set_reporttype));
      VIOLATIONS := cons (VIOLATIONSstr, cons (VIO, set_reporttype));
      
      report_types := list (list (ALLMARGINS, VIOLATIONS));
      ReportTypeRgn     = vcRegion.Create("rtype", 
                                          vcPicklist.fac,
                                          report_types,
                                          macro_on <- TRUE);
      vcPicklist.Notify (ReportTypeRgn, CallOnPick);

      OCCWARN   := cons (OCCWARNstr, cons (OCC, set_occwarn));
      ZCPMSG    := cons (ZCPMSGstr,  cons (ZCP, set_zcpmsg));

      occ_zcp_types := list (list (OCCWARN, ZCPMSG));
      OccZcpTypeRgn     = vcRegion.Create("report", 
                                          vcPicklist.fac,
                                          occ_zcp_types, 
                                          exclusive <- FALSE,
                                          macro_on <- TRUE);
      vcPicklist.Notify (OccZcpTypeRgn, CallOnPick);

      SetupMarginTitleRgn := vcRegion.Create("setup",vcLabel.fac, "Setup Margin(ns) :");
      HoldMarginTitleRgn := vcRegion.Create("Hold", vcLabel.fac, "Hold Margin (ns) :");
      SkewMarginTitleRgn := vcRegion.Create("Skew",vcLabel.fac, "Skew Margin (ns) :");
      
      SetupMarginCmdRgn    = vcRegion.Create("SetupMargin Cmd", vcCmdline.fac, "",
                                             max_width <- 12, width <- 9,
                                             macro_on <- TRUE);
      HoldMarginCmdRgn    = vcRegion.Create("HoldMargin Cmd", vcCmdline.fac, "",
                                            max_width <- 12, width <- 9,
                                            macro_on <- TRUE);
      SkewMarginCmdRgn    = vcRegion.Create("SkewMargin Cmd", vcCmdline.fac, "",
                                            max_width <- 12, width <- 9,
                                            macro_on <- TRUE);
      
      vcCmdline.CallOnChange(SetupMarginCmdRgn, CheckSetupMargin); 
      vcCmdline.CallOnChange(HoldMarginCmdRgn, CheckHoldMargin);
      vcCmdline.CallOnChange(SkewMarginCmdRgn, CheckSkewMargin);

      su_t := vcTable.Make ("",1,2, SetupMarginTitleRgn,SetupMarginCmdRgn);
      vcTable.Col (su_t, 1, vcTable.Border.Invisible, 2,2,2,vgAlign.MiddleCenter);
      hd_t := vcTable.Make ("",1,2, HoldMarginTitleRgn,HoldMarginCmdRgn);
      vcTable.Col (hd_t, 1, vcTable.Border.Invisible, 2,2,2,vgAlign.MiddleCenter);
      sk_t := vcTable.Make ("",1,2, SkewMarginTitleRgn,SkewMarginCmdRgn);
      vcTable.Col (sk_t, 1, vcTable.Border.Invisible, 2,2,2,vgAlign.MiddleCenter);
      
      ReportTitleRgn  := vcRegion.Create("", vcLabel.fac, "Verification Report");
      vcRegion.Font (ReportTitleRgn, vgFont.mediumb);
      ReportRgn :=  vcRegion.Create("rpt", vcMargin.fac,
                                     tt := vcTable.Make("TraceArgEditor", 9,1,
                                                        ReportTitleRgn,
                                                        ReportTypeRgn,
                                                        OccZcpTypeRgn,
                                                        su_t, hd_t, sk_t));
      vcTable.Row (tt, 1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);

      /*-------------------------------*\
        *     Trace Analysis Report    *
      \*-------------------------------*/
      
      EXPLMODE     := cons (EXPLMODEstr,   cons (EXPL, set_explmode));
      DETAILMODE   := cons (DETAILMODEstr, cons (DETAIL, set_detailmode));
      D2QMODE      := cons (D2QMODEstr,    cons (D2Q, set_d2qmode));
      SLACKMODE    := cons (SLACKMODEstr,  cons (SLACK, set_slackmode));

      trace_types := list (list (EXPLMODE, DETAILMODE), list (D2QMODE, SLACKMODE));
      TraceTypeRgn     = vcRegion.Create("trace",vcPicklist.fac,
                                         trace_types, exclusive <- FALSE,
                                         macro_on <- TRUE);
      vcPicklist.Notify (TraceTypeRgn, CallOnPick);

      SlackCountTitleRgn := vcRegion.Create("slack",vcLabel.fac, "Slack Count : ");
      SlackCountCmdRgn    = vcRegion.Create("SlackCount Cmd", vcCmdline.fac, "5",
                                            max_width <- 3, width <- 3,
                                            macro_on <- TRUE);
      vcCmdline.CallOnChange(SlackCountCmdRgn, CheckSlackCount); 
      SlackCountRgn = vcTable.Make ("",1,2, SlackCountTitleRgn,SlackCountCmdRgn);
      vcTable.Col (SlackCountRgn, 1, vcTable.Border.Invisible, 2,2,2,vgAlign.MiddleCenter);
      UseSlackCount (OFF);

      TraceTitleRgn  := vcRegion.Create("", vcLabel.fac, "Trace Analysis Report");
      vcRegion.Font (TraceTitleRgn, vgFont.mediumb);
      TraceRgn :=  vcRegion.Create("trc", vcMargin.fac,
                                     tt := vcTable.Make("r", 3,1,
                                                        TraceTitleRgn,
                                                        TraceTypeRgn,
                                                        SlackCountRgn));
      vcTable.Row (tt, 1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);      

      /*-------------------------------*\
        *        Time Base             *
      \*-------------------------------*/
      CycleTimeTitleRgn := vcRegion.Create("cycle",vcLabel.fac, "Cycle Time (ns): ");
      TimeStepTitleRgn := vcRegion.Create("step", vcLabel.fac, "Time Step (ns): ");
      
      CycleTimeCmdRgn    = vcRegion.Create("CycleTime Cmd", vcCmdline.fac, "",
                                           max_width <- 12, width <- 9,
                                           macro_on <- TRUE);
      TimeStepCmdRgn    = vcRegion.Create("TimeStep Cmd", vcCmdline.fac, "",
                                          max_width <- 12, width <- 9,
                                          macro_on <- TRUE);
      
      vcCmdline.CallOnChange(CycleTimeCmdRgn, CheckCycleTime); 
      vcCmdline.CallOnChange(TimeStepCmdRgn, CheckTimeStep);
      
      BaseTitleRgn  := vcRegion.Create("", vcLabel.fac, "Time Base");
      vcRegion.Font (BaseTitleRgn, vgFont.mediumb);
      TimebaseRgn :=  vcRegion.Create("base", vcMargin.fac,
                                     tt := vcTable.Make("TraceArgEditor", 3,1,
                                                        BaseTitleRgn,
                                                        cy := vcTable.Make ("",1,2, CycleTimeTitleRgn,
                                                                            CycleTimeCmdRgn),
                                                        ts := vcTable.Make ("",1,2, TimeStepTitleRgn,
                                                                            TimeStepCmdRgn)));
      vcTable.Col (cy, 1, vcTable.Border.Invisible, 5,7,10,vgAlign.MiddleCenter);
      vcTable.Col (ts, 1, vcTable.Border.Invisible, 5,7,10,vgAlign.MiddleCenter);

      /*-------------------------------*\
        *           General            *
      \*-------------------------------*/
      CLKGAT   := cons (CLKGATstr, cons (CLK, set_clkgat));
      RECONV   := cons (RECONVstr, cons (REC, set_reconv));
      SIGP   := cons (SIGPstr, cons (SIG, set_sigp));
      BIDIIO   := cons (BIDIIOstr, cons (BIO, set_bidiio));
      BIDIPIN   := cons (BIDIPINstr, cons (BPN, set_bidipin));
      GSLACK    := cons (GSLACKstr, cons (GSR, set_gsr));

      general_types := list (list (CLKGAT), list (RECONV), list (SIGP), 
                             list (BIDIIO), list (BIDIPIN), list(GSLACK));
      GeneralTypeRgn     = vcRegion.Create("general",vcPicklist.fac,
                                         general_types, exclusive <- FALSE,
                                           macro_on <- TRUE);
      vcPicklist.Notify (GeneralTypeRgn, CallOnPick);

      GeneralTitleRgn  := vcRegion.Create("", vcLabel.fac, "General");
      vcRegion.Font (GeneralTitleRgn, vgFont.mediumb);
      GeneralRgn :=  vcRegion.Create("trc", vcMargin.fac,
                                     tt := vcTable.Make("", 2,1,
                                                        GeneralTitleRgn,
                                                        GeneralTypeRgn));
      vcTable.Row (tt, 1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);

      /*-------------------------------*\
        *           latch              *
      \*-------------------------------*/
      ALWAYS    := cons (ALWAYSstr, cons (ALW, set_latchmode));
      SETUP    := cons (SETUPstr, cons (SET, set_latchmode));
      SETHLD    := cons (SETHLDstr, cons (STHD, set_latchmode));
      NEVER    := cons (NEVERstr, cons (NEV, set_latchmode));

      latch_types := list (list (ALWAYS), list (SETUP), 
                           list (SETHLD),list (NEVER));
      LatchTypeRgn     = vcRegion.Create("latch",vcPicklist.fac,
                                         latch_types,
                                         macro_on <- TRUE);
      vcPicklist.Notify (LatchTypeRgn, CallOnPick);
      
      LARIAT    := cons (LARIATstr, cons (LAR, set_lariat));
      lariat_types := list (list (LARIAT));
      LariatTypeRgn     = vcRegion.Create("lariat",vcPicklist.fac,
                                          lariat_types, exclusive <- FALSE,
                                          macro_on <- TRUE);
      vcPicklist.Notify (LariatTypeRgn, CallOnPick);

      LatchDepthTitleRgn := vcRegion.Create("slack",vcLabel.fac, "Latch Depth : ");
      LatchDepthCmdRgn    = vcRegion.Create("LatchDepth Cmd", 
                                            vcCmdline.fac, "5",
                                            max_width <- 3, width <- 3,
                                            macro_on <- TRUE);
      vcCmdline.CallOnChange(LatchDepthCmdRgn, CheckLatchDepth); 
      LatchDepthRgn = vcTable.Make ("",1,2, LatchDepthTitleRgn,LatchDepthCmdRgn);
      vcTable.Col (LatchDepthRgn, 1, vcTable.Border.Invisible, 2,2,2,vgAlign.MiddleCenter);
      UseLatchDepth (OFF);

      LatchTitleRgn  := vcRegion.Create("", vcLabel.fac, "Latch");
      vcRegion.Font (LatchTitleRgn, vgFont.mediumb);
      LatchRgn :=  vcRegion.Create("trc", vcMargin.fac,
                                     tt := vcTable.Make("", 4,1,
                                                        LatchTitleRgn,
                                                        LatchTypeRgn,
                                                        LatchDepthRgn,
                                                        LariatTypeRgn));
      vcTable.Row (tt, 1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);

      /*-------------------------------*\
        *         Interconnect         *
      \*-------------------------------*/
      IDDPK    := cons (IDDstr, cons (IDD, set_idd));
      idd_types := list (list (IDDPK));
      IddTypeRgn     = vcRegion.Create("idd",vcPicklist.fac,
                                       idd_types, exclusive <- FALSE,
                                       macro_on <- TRUE);
      vcPicklist.Notify (IddTypeRgn, CallOnPick);
      
      IddFileTitleRgn := vcRegion.Create("slack",vcLabel.fac, "Filename: ");
      IddFileCmdRgn    = vcRegion.Create("IddFile Cmd", vcFilePop.fac, "");
      vcCmdline.CallOnChange(IddFileCmdRgn, CheckIddFile); 
      IddFileRgn = vcTable.Make ("",1,2, IddFileTitleRgn,IddFileCmdRgn);

      s.IddMsgRgn = vcRegion.Create("", vcStaticText.fac, "Msg area");
      vcRegion.Align (s.IddMsgRgn,vgAlign.MiddleLeft);
      

      XConRgn  = vcWv.LabelButtonCreate ("Addl. Calculations", "Calculations",
                                        nil, IntconParams,list (macro_on <- TRUE));
      vcRegion.Font (XConRgn, vgFont.mediumb);

      IddTitleRgn  := vcRegion.Create("", vcLabel.fac, "Interconnect Delay Analysis");
      vcRegion.Font (IddTitleRgn, vgFont.mediumb);
      IddRgn :=  vcRegion.Create("idd", vcMargin.fac,
                                     tt := vcTable.Make("", 3,1,
                                                        IddTitleRgn,
                                                        vcTable.Make("",1,2,
                                                                     IddTypeRgn,
                                                                      IddFileRgn),
                                                        vcTable.Make ("",1,2,
                                                                      XConRgn,
                                                                      s.IddMsgRgn)));
                                                        
      vcTable.Row (tt, 1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);
      UseIddFile (OFF);
      
      
      /*--------------------------------*\
       *   Define a row of two buttons  *
       \*--------------------------------*/
      function Accept()
      {         
			Queue := vtime.GetQueue (s.VtimeData);
			SubmitCmds (Queue,s.VtimeEvent,s.CmdList, ADD);
         
         if (s.TimeBaseCmds != nil || s.IntconCmd != nil ||
             s.IddCmd != nil){
				design_loaded := vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.
															MV_design_loaded,
															vtime.MtvValue.VALUE);
            
				if (string_eqi (design_loaded, "FALSE")){
					SubmitCmds (Queue, s.VtimeEvent, s.TimeBaseCmds, ADD);
					SubmitCmds (Queue, s.VtimeEvent, s.IntconCmd, ADD);
					SubmitCmds (Queue, s.VtimeEvent, s.IddCmd, ADD);
               
					vcWin.Close (vcRegion.Window (TopRegion));
				}
				else{
					e := vcEvent.Create (0, 0, ' ', "", vcRegion.Window (TopRegion),
												TopRegion, nil);      
					vcCmd.Run (RestartCmd, s, e, nil, TRUE);
				}
         }
			else
				vcWin.Close (vcRegion.Window (TopRegion));
      }
      function Cancel() FALSE;
      
      buttons := vcRegion.Create("Buttons", vcButnRow.fac,
                                 list (list ("Accept",
                                             "Accept",
                                             Accept,
                                             list (close_window <- FALSE,
                                                   macro_on     <- TRUE)),
                                       list ("Cancel",
                                             "Cancel",
                                             Cancel,
                                             list (close_window <- TRUE,
                                                   macro_on     <- TRUE))));
      /*-------------------------------*\
       *       Build final table       *
      \*-------------------------------*/
      C1 := vcRegion.Create("", vcMargin.fac, vcTable.Make ("",2,1,ReportRgn, TraceRgn));
      vcTable.Row (C1, -1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);

      C2 := vcRegion.Create("", vcMargin.fac, vcTable.Make ("",1,2,GeneralRgn, LatchRgn));
      vcTable.Col (C2, -1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);
      L3 := vcRegion.Create("", vcMargin.fac, vcTable.Make ("",2,1,
                                                            TimebaseRgn, 
                                                            C2));
                                                            /*LatchDepthRgn));*/
      vcTable.Row (L3, -1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);

      L4 := vcRegion.Create("", vcMargin.fac, vcTable.Make ("",1,2, C1, L3));
      vcTable.Col (L4, -1, vcTable.Border.Visible, 1,3,5,vgAlign.MiddleCenter);

      L := vcRegion.Create("", vcMargin.fac, vcTable.Make ("",3,1, L4, IddRgn,
                                                           buttons));
      vcTable.Row (L, -1, vcTable.Border.Visible, 5,5,5,vgAlign.MiddleCenter);

      /*
         vcPicklist.SetPickByString (ReportTypeRgn, ALLMARGINSstr);
         vcPicklist.SetPickByString (OccZcpTypeRgn, OCCWARNstr);
         vcPicklist.SetPickByString (LatchTypeRgn, SETUPstr);
         */

      L;      
   } /* BuildParamsGui */


   /*====================================================*/
   /*==       U p d a t e T r a c e G u i              ==*/
   /*====================================================*/
   /*\Function UpdateParamsGui  -- Refresh using settings
     
     DESCRIPTION
     
   */
   function UpdateParamsGui(s) 
   {
      declare value, number;

      /* report options */
      StatusMsg (s.VtimeEvent, "Querying report options");
      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_report_margin_report,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ALL) == TRUE)
         vcPicklist.SetPick (ReportTypeRgn, 0);
      else
         vcPicklist.SetPick (ReportTypeRgn, 1);
         
      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_msg_occ_missing,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (OccZcpTypeRgn, 0);

      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_msg_zcp_warn,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (OccZcpTypeRgn, 1);

      number = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_report_margin_setup,
                               vtime.MtvValue.VALUE);
      vcCmdline.Set (SetupMarginCmdRgn,number);
      number = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_report_margin_hold,
                               vtime.MtvValue.VALUE);
      vcCmdline.Set (HoldMarginCmdRgn,number);
      number = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_report_margin_skew,
                               vtime.MtvValue.VALUE);
      vcCmdline.Set (SkewMarginCmdRgn,number);

      /* trace options */
      StatusMsg (s.VtimeEvent, "Querying trace options");
      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_trace_expl_mode,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (TraceTypeRgn,0);

      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_trace_detailed_elec,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (TraceTypeRgn,1);

      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_trace_force_d2q,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (TraceTypeRgn,2);

      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_trace_slack,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (TraceTypeRgn,3);


      /* time base */
      StatusMsg (s.VtimeEvent, "Querying timebase options ");
      number = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_timebase_cycle_time,
                               vtime.MtvValue.VALUE);
      vcCmdline.Set (CycleTimeCmdRgn,number);

      number = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_timebase_resolution,
                               vtime.MtvValue.VALUE);
      vcCmdline.Set (TimeStepCmdRgn,number);

      /* general */
      StatusMsg (s.VtimeEvent, "Querying general options");
      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_analysis_clock_gating,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (GeneralTypeRgn,0);

      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_analysis_reconvergence,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (GeneralTypeRgn,1);
      
      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_analysis_sigp,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (GeneralTypeRgn,2);

      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_analysis_bi_io_trc,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (GeneralTypeRgn,3);

      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_analysis_bi_pin_trc,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (GeneralTypeRgn,4);
      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_analysis_gslack,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (GeneralTypeRgn,5);

      /* latch */
      StatusMsg (s.VtimeEvent, "Querying latch options");
      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_analysis_latch_mode,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,DISABLE))
         vcPicklist.SetPick (LatchTypeRgn,3);
      else if (string_eqi (value,S))
         vcPicklist.SetPick (LatchTypeRgn,1);
      else if (string_eqi (value,SH))
         vcPicklist.SetPick (LatchTypeRgn,2);
      else if (string_eqi (value,ALL))
         vcPicklist.SetPick (LatchTypeRgn,0);

      number = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_analysis_latch_depth,
                               vtime.MtvValue.VALUE);
      vcCmdline.Set (LatchDepthCmdRgn,number);

      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_analysis_lariat,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value,ENABLE))
         vcPicklist.SetPick (LariatTypeRgn,0);

      /* idd mode */
      StatusMsg (s.VtimeEvent, "Querying IDD options");
      s.RisMin = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_default_lhmin,                                 vtime.MtvValue.VALUE);
      s.RisMax = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_default_lhmax,                                 vtime.MtvValue.VALUE);
     s.FallMin = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_default_hlmin,
                                  vtime.MtvValue.VALUE);
     s.FallMax = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_default_hlmax,
                                  vtime.MtvValue.VALUE);
      s.Missidd = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam_idd_summary,
                                  vtime.MtvValue.VALUE);      
      s.IddUpdated = TRUE;


      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_iam,
                              vtime.MtvValue.VALUE);
      s.IntconType = GetIntconType (value);

      /* Look for idd file and then set the pick */
      value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.MV_file_idd,
                              vtime.MtvValue.VALUE);
      if (string_eqi (value, "") == FALSE)
         vcPicklist.SetPick (IddTypeRgn,0);
      else{
         value = vtime.GetMtvVar(s.VtimeData, vtime.MtvVar.VT_file_idd,
                                 vtime.MtvValue.VALUE);
         if (value == nil)
            value = "";
      }
      vcCmdline.Set (IddFileCmdRgn,value);
      
      s.IntconCmd = nil;
      s.IddCmd = nil;
      UpdateIddMsg (s);

      s;
   } /* UpdateParamsGui */
   

   /*----------------------------------------------------*\
    *               Start processing data now            *
   \*--------------------------------------------------- */
   if (Defaults == nil)
      return (FALSE);
   
   settings := Str2Data(Defaults);
   StatusMsg (settings.VtimeEvent, "Creating Parameters Dialog Box");
   DialogRgn := BuildParamsGui(settings);
   StatusMsg (settings.VtimeEvent, "Querying current settings");
   UpdateParamsGui(settings);
   StatusMsg (settings.VtimeEvent, "Querying current settings...done");

   vcRegion.AddChild(TopRegion,DialogRgn);

   settings.RecordCmds = TRUE;
   settings.CmdList = nil;
   settings.TimeBaseCmds = nil;
   settings;  /* Data is a structure */
}


/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function Data2Str  -- Convert a settings structure to a string
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function Data2Str(s,r)
{
   declare str;

   str;
} /* Data2Str */



/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function Str2Data  -- Convert a string to a settings structure
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function Str2Data(defaults)
{
   s := GetDefaultSettings();
   s.VtimeData  = car (defaults);
   s.VtimeEvent = cadr (defaults);
   s;
} /* Str2Data */

/*===========================================================================*/
/*==                      _ G e t                                          ==*/
/*===========================================================================*/
/*\Function _Get  -- The _Get data method for this facility
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function _Get(d,r,type) 
{      

   nil;
   
} /* _Get */

/*===========================================================================*/
/*==                             AddCommand                                ==*/
/*===========================================================================*/
function AddCommand(d, cmd, args)
{
   if (d.RecordCmds == FALSE)
      return (TRUE);

   l := list(cmd, args);

   if (d.CmdList != nil){
      t := assq (cmd, d.CmdList);
      if (t != FALSE)
         list_set (t,1,args);
      else
         d.CmdList = cons (l, d.CmdList);
   }
   else
      d.CmdList = cons (l, d.CmdList);

}

/*===========================================================================*/
/*==                        AddTimebaseCmd                                ==*/
/*===========================================================================*/
function AddTimebaseCmd(d, cmd, args)
{
   if (d.RecordCmds == FALSE)
      return (TRUE);

   l := list(cmd, args);

   if (d.TimeBaseCmds != nil){
      t := assq (cmd, d.TimeBaseCmds);
      if (t != FALSE)
         list_set (t,1,args);
      else
         d.TimeBaseCmds = cons (l, d.TimeBaseCmds);
   }
   else
      d.TimeBaseCmds = cons (l, d.TimeBaseCmds);
	
	/*print (d.TimeBaseCmds);*/
}

/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function GetDefaultSettings  -- 
  
  DESCRIPTION
       Get the default settings of the facility dialog box. 
       
  RETURNS
      The  default settings are returned in a viewscript structure.
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function GetDefaultSettings() {
   struct {

      VtimeData  = nil;
      VtimeEvent = nil;

      /* report data */
      ReportType = ALL;
      OccWarn    = DISABLE;
      ZcpMsg     = DISABLE;
      SetupMargin = 0.011;
      HoldMargin = 0.011;
      SkewMargin = 0.011;

      /* trace data */
      ExplMode   = OFF;
      DetailMode = OFF;
      D2qMode    = OFF;
      SlackMode  = OFF;
      SlackCount = 5;

      /* Time Base */
      CycleTime  = 20.0;
      TimeStep   = 0.001;

      /* General */
      Clkgating  = DISABLE;
      Reconvergence = DISABLE;
      Sigp       = DISABLE;
      Bidiio     = DISABLE;
      Bidipin    = DISABLE;
      Gsr        = DISABLE;

      /* latch */
      LatchType  = "";      
      LatchDepth = 1;
      Lariat     = OFF;

      /* idd */
      IddType    = "";
      IddFile    = "";
      
      FetchValues   = TRUE;
      XlineUpdated = FALSE;
      CapfUpdated  = FALSE;
      CappUpdated  = FALSE;
      IddUpdated   = FALSE;

      IntconType = NON;

      XlineLocFile    = "";
      XlineDfltPinCap = 0;
      XlineRouteFactor= 1;
      XlineTraceCap   = "";
      TraceVel   = "";

      CappLocFile    = "";
      CappDfltPinCap = 0;
      CappRouteFactor= 1;
      CappTraceCap   = "";
      
      CalcType   = "";
      CapfDfltPinCap = 0;
      PicoPerFan = 0;
      Seg1Off    = 0;
      Seg1Pf     = 0;
      Seg2Off    = 0;
      Seg2Pf     = 0;
      Breakpt    = 1;

      RecordCmds = FALSE;
      CmdList    = nil;
      TimeBaseCmds = nil;
      IntconCmd  = nil;
      IddCmd     = nil;

      RisMin = 0;
      RisMax = 0;
      FallMin = 0;
      FallMax = 0;
      Missidd = DISABLE;
      IddMsgRgn = nil;

      dummy      = OFF
   } settings;

} /* GetDefaultSettings */
