/* ---------------------------------------------------------------------- *\
 * Module -- path.vs
 *
 *      Facility:      Path Display
 *
 *      Environment:   SUN4, Powerview 5.2
 *
 *      Author:        Jeff Hare,  Jan 31, 1994
 *
 *      Description:   path facility vs file
 *       
 *
 *      Copyright (c) 1994 by Viewlogic Systems, Inc.
 *
\* ---------------------------------------------------------------------- */
/*\Section Path -- Path Trace Display Facility
*/

normal_edge  := nil;
select_edge  := nil;


/*===========================================================================*/
/*==                      _ C r e a t e                                    ==*/
/*===========================================================================*/
/*\Docpage _Create -- Path display facility's data creation method.

DESCRIPTION

*/
function _Create (vtdata, region)
{
   vlPref.Setval(vge.prefs, "set_bindings", FALSE);
  
   
   normal_edge = vcTree.EdgeDescriptorCreate(color <- vgColor.LightBlue,
                                             thickness <- 3);
   
   select_edge = vcTree.EdgeDescriptorCreate(color <- vgColor.White,
                                             thickness <- 4);

   tree := vcRegion.Create ("My Path Display Tree Widget",
                            vcTree.fac,
                            nil,
                            compass <- vgCompass.West,
                            level_space <- 150,
                            sibling_space <- 200,
                            group <- TRUE,
                            deselect_on_miss <- FALSE,
                            interactive <- TRUE,
                            exclusive <- FALSE,
                            scrollbars	<- TRUE,
                            anchor_root <- FALSE,
                            scalable  <- TRUE,
                            squish <- FALSE,
                            follow_focus <- FALSE,
                            default_edge_descriptor <- normal_edge,
                            inhibit_select_box <- FALSE
                           );


   pdata := struct {     /*----------------------------------------------*/
      tree = tree;       /* vcTree data object                           */
      Current_Edge = nil;/* Currently selected edge.                     */ 
      vtdata = vtdata    /* list(vtime_data, vtime_event)                */
   };                    /*----------------------------------------------*/


   /*===========================================================================*/
   /*==               C a l l O n E d g e S e l e c t                         ==*/
   /*===========================================================================*/
   function CallOnEdgeSelect (edge, pdata) {
      
      vcTree.DeselectAll(region);
      
      if (edge == pdata.Current_Edge) {
         vcTree.EdgeModify (pdata.Current_Edge, descriptor <- normal_edge);
         pdata.Current_Edge = nil;
      }
      else {
         if (pdata.Current_Edge != nil) 
            vcTree.EdgeModify (pdata.Current_Edge, descriptor <- normal_edge);
         
         vcTree.EdgeModify (edge, descriptor <- select_edge, curved <- vlTrueFalse.False);
         pdata.Current_Edge = edge;
      }
      
      i := vcTree.EdgeGetItem(edge);
      
      window := vcRegion.Window(region);
      vcWin.ClearStatus(window);
      if (i != nil)
         vcWin.AddStatus(window,sprintf("Net:  %s    Delay:  %s",
                                        vtime.GetNet(i),
                                        vtime.GetIdd(i)));
      else
         vcWin.AddStatus(window,"No data for this net.");
      
      vcRegion.Refresh (tree, vcRegion.Area (tree)); 
   }

   
   b := vcBind.Duplicate (vcTree.bindings);  /* get a copy of the binding table */
   vcBind.Set (b, '\rightdown', nil );       /* cancel this binding             */
   vcRegion.SetBindings (tree, b);           /* override tree's binding table   */

   c := vcBind.Duplicate (path.bindings);        /* get my own binding table */
   vcBind.Set (c, '\rightdown', PopupMenuCmd );  /* cancel this binding      */
   vcRegion.SetBindings (region, c);             /* use this for my region   */

   vcRegion.AddChild(region, tree);

   RegisterVrtxSelect   (region, pdata);
   RegisterVrtxActivate (region, pdata);

   vcTree.CallOnEdgeSelect (tree, func(edge) CallOnEdgeSelect(edge,pdata););

   pdata;
} /* function _Create */


/*===========================================================================*/
/*==                D o P o p u p M e n u F u n c                          ==*/
/*===========================================================================*/
function DoPopupMenuFunc (pdata, event) {
   
   if (pdata.Current_Edge != nil) {
      vcMenuPop.Create(vcWv.MakeMenu("Popup Title",
                                     list("Info...",    NetInfoCmd),
                                     list("Constant 1", Set1Cmd),
                                     list("Constant 0", Set0Cmd),
                                     list("Constant X", SetXCmd)),
                       event);
      
   }
   else {
      vcMenuPop.Create(vcWv.MakeMenu("Popup Title",
                                     list("Component Info",CompInfoCmd),
                                     list("Model Info",ModelInfoCmd),
                                     list("Generate MCP...",McpCmd),
                                     list("Unselect All",UnselectAllCmd),
                                     list("Visit",DoVisitCmd)),
                       event);
   }
}


/*===========================================================================*/
/*==               C a l l M c p D i a l o g                               ==*/
/*===========================================================================*/
function CallMcpDialog(d,e) {
   
   vtdata := d.vtdata;
   rgn    := vcEvent.Region(e);
   window := vcRegion.Window(rgn);
   vlst   := Roots(d,e);
   len    := length(vlst);
   
   if ( len < 1) {
      vcWin.ClearStatus(vcEvent.Window(e));
      vcWin.AddStatus(window,"ERROR: Nothing selected..");
      vtime.AlertBox(car(vtdata),cdr(vtdata),
                     car(vtime.msg_McpNothingSelected),
                     vlErrSev.Error,
                     cdr(vtime.msg_McpNothingSelected));
   }      
   else {
      rgn := vcRegion.Create("(Path Assertion) MCP Generator", 
                             vtimemcp.fac,
                             cons(vlst,vtdata));
      
      vcWin.Popup (vcEvent.Window (e), rgn, rgn,vcWin.NoMenus);
   }
}
                       
/*===========================================================================*/
/*==           F e t c h C u r r e n t N e t D a t a                       ==*/
/*===========================================================================*/
function FetchCurrentNetData(pdata) {
   
   if (pdata.Current_Edge != nil) 
      netdata := vcTree.EdgeGetItem(pdata.Current_Edge);
   else
      netdata := nil;
} /* FetchCurrentNetData */

/*===========================================================================*/
/*==           F e t c h C u r r e n t C o m p D a t a                     ==*/
/*===========================================================================*/
function FetchCurrentCompData(rgn) {
   if (length(vrtx := SelectedVrtxList(rgn)) == 1) 
      compdata := vcTree.VrtxGetItem(car(vrtx));
   else
      compdata := nil;
}



/*===========================================================================*/
/*==               N e t I n f o F u n c                                   ==*/
/*===========================================================================*/
function NetInfoFunc(pdata,e) {
   vtdata := pdata.vtdata;
   if ((net := FetchCurrentNetData(pdata)) != nil) 
      vtq.Add (vtime.GetQueue (car(vtdata)),cdr(vtdata),                  
               sprintf ("NetInfo %s",vtime.GetNet(net)));
}

/*===========================================================================*/
/*==                    S e t 1                                            ==*/
/*===========================================================================*/
function Set1(pdata,e) {
   vtdata := pdata.vtdata;
   if ((net := FetchCurrentNetData(pdata)) != nil) 
      vtq.Add (vtime.GetQueue (car(vtdata)),cdr(vtdata),                  
               sprintf ("H %s",vtime.GetNet(net)));
}

/*===========================================================================*/
/*==                   S e t 0                                             ==*/
/*===========================================================================*/
function Set0(pdata,e) {
   vtdata := pdata.vtdata;
   if ((net := FetchCurrentNetData(pdata)) != nil) 
      vtq.Add (vtime.GetQueue (car(vtdata)),cdr(vtdata),                  
               sprintf ("L %s",vtime.GetNet(net)));
}

/*===========================================================================*/
/*==                   S e t X                                             ==*/
/*===========================================================================*/
function SetX(pdata,e) {
   vtdata := pdata.vtdata;
   if ((net := FetchCurrentNetData(pdata)) != nil) 
      vtq.Add (vtime.GetQueue (car(vtdata)),cdr(vtdata),                  
               sprintf ("X %s",vtime.GetNet(net)));
}

/*===========================================================================*/
/*==                C o m p I n f o F u n c                                ==*/
/*===========================================================================*/
function CompInfoFunc(pdata,e) {
   
   declare vrtx;
   declare vlst;
   declare c;
   
   vtdata := pdata.vtdata;
   Queue := vtime.GetQueue(car(vtdata));
   Data  := car(vtdata);
   Event := cdr(vtdata);
   count := 1;

   infofile  := vtime.GetMtvVar(Data, vtime.MtvVar.MV_file_tmp_info, vtime.MtvValue.VALUE);
   if (infofile != nil) 
      if (length(vlst=SelectedVrtxList(vcEvent.Region(e))) > 0) {
         foreach (vrtx; vscript.list_iter(vlst)) 
            if (vrtx != nil && (c = vcTree.VrtxGetItem(vrtx)) != nil) {
               if (count == 1)
                  vtq.CmdRun (Queue, Event,
                              sprintf("Mtv info %s loc %s",infofile, vtime.GetPart(c)));
               else
                  vtq.CmdRun (Queue, Event,
                              sprintf("Mtv info +%s loc %s",infofile, vtime.GetPart(c)));
               count = count + 1;
            }
         vtq.CmdRun (Queue, Event, "_autoload " + infofile);
         vtq.CmdRun (Queue, Event, "current Info");
      }
      else {
         vcWin.ClearStatus(vcEvent.Window(e));
         vcWin.AddStatus(window := vcRegion.Window(vcEvent.Region(e)),"No Components Selected..");
         vtime.AlertBox(Data,Event,
                        car(vtime.msg_InfoNothingSelected),
                        vlErrSev.Error,
                        cdr(vtime.msg_InfoNothingSelected));
      }
}

/*===========================================================================*/
/*==               M o d e l I n f o F u n c                               ==*/
/*===========================================================================*/
function ModelInfoFunc(pdata,e) {
   declare vrtx;
   declare vlst;
   declare c;
   
 vtdata:= pdata.vtdata;
   Queue := vtime.GetQueue(car(vtdata));
   Data  := car(vtdata);
   Event := cdr(vtdata);
   count := 1;
   
   infofile  := vtime.GetMtvVar(Data, vtime.MtvVar.MV_file_tmp_info, vtime.MtvValue.VALUE);
   if (infofile != nil) 
      if (length(vlst=SelectedVrtxList(vcEvent.Region(e))) > 0) {
         foreach (vrtx; vscript.list_iter(vlst)) 
            if (vrtx != nil && (c = vcTree.VrtxGetItem(vrtx)) != nil) {
               if (count == 1)
                  vtq.CmdRun (Queue, Event,
                              sprintf("Mtv info %s mod %s",infofile, vtime.GetModel(c)));
               else
                  vtq.CmdRun (Queue, Event,
                              sprintf("Mtv info +%s mod %s",infofile, vtime.GetModel(c)));
               count = count + 1;
            }
         vtq.CmdRun (Queue, Event, "_autoload " + infofile);
         vtq.CmdRun (Queue, Event, "current Info");
      }
      else {
         vcWin.ClearStatus(vcEvent.Window(e));
         vcWin.AddStatus(window := vcRegion.Window(vcEvent.Region(e)),"No Components Selected..");
         vtime.AlertBox(Data,Event,
                        car(vtime.msg_InfoNothingSelected),
                        vlErrSev.Error,
                        cdr(vtime.msg_InfoNothingSelected));
      }
}

/*===========================================================================*/
/*==                      U n s e l e c t A l l                            ==*/
/*===========================================================================*/
/*\Docpage UnselectAll -- 

DESCRIPTION

*/
function UnselectAll(pdata,event)
{
   region := vcEvent.Region(event);
   vcTree.DeselectAll(region);
   vcRegion.Refresh (region, vcRegion.VisibleArea (region));
} /* UnselectAll */



/*===========================================================================*/
/*==                      C o m p N o d e                                  ==*/
/*===========================================================================*/
/*\Docpage CompNode -- Create an unconstrained component in the path display

DESCRIPTION

*/
function CompNode(pregion,  /* Tree in which this node belongs  */
                  parent,   /* Parent component                 */
                  NetName,  /* Name of net connecting to parent */
                  Cname,    /* Component Instance Identifier    */
                  Model,    /* Model Name                       */
                  Ipin,     /* Input Pin Name                   */
                  Opin,     /* Output Pin Name (to parent)      */
                  Itime,    /* Input arrival time annotation    */
                  Otime,    /* Output time annotation           */
                  Edge,     /* Ipin to Opin edge transition     */
                  Border,   /* Attribute to use for the border  */
                  Data)     /* Structure to attach to this comp */

{
   pdata := vcRegion.GetData(pregion);
   tree := pdata.tree;
   node := vcTree.VrtxCreate(Cname,
                             vcTree.VrtxType.Text,
                             "\n "+Model+" \n",
                             parent,
                             nil,
                             text_size    <- 20,
                             text_color   <- vgColor.Yellow,
                             stroke_font  <- vgStk.SansSerif,
                             border       <- Border,
									  border_thickness <- 3,
                             border_color <- vgColor.LightMagenta);

   vcTree.VrtxSetItem(node,Data);

   declare t;
   if (Border == vcTree.Border.Circle) {
      
      t = vge.TextCreateXY (20,-20,Itime,0,vgAlign.LowerRight,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Cyan);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerLeft));

      t = vge.TextCreateXY (-20,-20,Otime,0,vgAlign.LowerLeft,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Cyan);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerRight));

      t = vge.TextCreateXY (-5,5,Ipin,0,vgAlign.LowerRight,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Red);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleLeft));

      t = vge.TextCreateXY (5,5,Opin,0,vgAlign.LowerLeft,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Red);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleRight));

      t = vge.TextCreateXY (0,10,Cname,0,vgAlign.LowerCenter,vge.vgeScore.None,25,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.White);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.UpperCenter));

      t = vge.TextCreateXY (0,-20,Edge,0,vgAlign.UpperCenter,vge.vgeScore.None,25,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Green);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerCenter));

   } /* if */

   if (Border == vcTree.Border.Box) {
      t = vge.TextCreateXY (20,-20,Itime,0,vgAlign.LowerRight,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Cyan);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerLeft));

      t = vge.TextCreateXY (-20,-20,Otime,0,vgAlign.LowerLeft,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Cyan);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerRight));

      t = vge.TextCreateXY (-5,5,Ipin,0,vgAlign.LowerRight,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Red);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleLeft));

      t = vge.TextCreateXY (5,5,Opin,0,vgAlign.LowerLeft,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Red);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleRight));

      t = vge.TextCreateXY (0,10,Cname,0,vgAlign.LowerCenter,vge.vgeScore.None,25,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.White);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.UpperCenter));

      t = vge.TextCreateXY (0,-20,Edge,0,vgAlign.UpperCenter,vge.vgeScore.None,25,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Green);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerCenter));

   } /* if */

   if (Border == vcTree.Border.Diamond) {
      t = vge.TextCreateXY (20,-20,Itime,0,vgAlign.LowerCenter,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Cyan);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerLeft));

      t = vge.TextCreateXY (-20,-20,Otime,0,vgAlign.LowerCenter,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Cyan);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerRight));

      t = vge.TextCreateXY (-5,5,Ipin,0,vgAlign.LowerRight,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Red);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleLeft));

      t = vge.TextCreateXY (5,5,Opin,0,vgAlign.LowerLeft,vge.vgeScore.None,20,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Red);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleRight));

      t = vge.TextCreateXY (0,10,Cname,0,vgAlign.LowerCenter,vge.vgeScore.None,25,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.White);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.UpperCenter));

      t = vge.TextCreateXY (0,-20,Edge,0,vgAlign.UpperCenter,vge.vgeScore.None,25,vgStk.SansSerif);
      vge.ObjectSetColor(t, vgColor.Green);
      vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerCenter));

   } /* if */

   node;

} /* function Comp */




/*===========================================================================*/
/*==                      R o o t N o d e                                  ==*/
/*===========================================================================*/
/*\Docpage RootNode -- Create an root node for a path display 

DESCRIPTION

*/
function RootNode(pregion,    /* Tree in which this node belongs  */
                  Cname,      /* Component Instance Identifier    */
                  Model,      /* Model Name                       */
                  Cpin,       /* Clock Pin Name                   */
                  Dpin,       /* Data Pin Name                    */
                  Ctime,      /* Input arrival time for Clock     */
                  Dtime,      /* Input arrival time for Data      */
                  Constraint, /* Constraint annotation            */
                  Trace,      /* Trace type and violation string  */
                  Margin,     /* Margin Failure/Success string    */
                  Data)       /* Structure to attach to this comp */
{
   pdata := vcRegion.GetData(pregion);
   tree := pdata.tree;
   node := vcTree.VrtxCreate(Cname,
                             vcTree.VrtxType.Text,
                             "\n "+Model+" \n",
                             nil,   
                             nil,
                             text_size    <- 20,
                             text_color   <- vgColor.Yellow,
                             stroke_font  <- vgStk.SansSerif,
                             border       <- vcTree.Border.Box,
									  border_thickness <- 3,
                             border_color <- vgColor.Red);

   vcTree.VrtxSetItem(node,Data);
   declare t;
      
   t = vge.TextCreateXY (-10, 0,Ctime,0,vgAlign.UpperRight,vge.vgeScore.None,20,vgStk.SansSerif);
   vge.ObjectSetColor(t, vgColor.Cyan);
   vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerLeft));
   
   t = vge.TextCreateXY (-10,0,Dtime,0,vgAlign.LowerRight,vge.vgeScore.None,20,vgStk.SansSerif);
   vge.ObjectSetColor(t, vgColor.Cyan);
   vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.UpperLeft));
   
   t = vge.TextCreateXY (-5,-5,Cpin,0,vgAlign.UpperRight,vge.vgeScore.None,20,vgStk.SansSerif);
   vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleLeft));
   
   t = vge.TextCreateXY (-5,5,Dpin,0,vgAlign.LowerRight,vge.vgeScore.None,20,vgStk.SansSerif);
   vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleLeft));
   
   t = vge.TextCreateXY (0,20,Cname,0,vgAlign.LowerCenter,vge.vgeScore.None,25,vgStk.SansSerif);
   vge.ObjectSetColor(t, vgColor.White);
   vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.UpperCenter));
   
   t = vge.TextCreateXY (0,-25,Constraint,0,vgAlign.UpperCenter,vge.vgeScore.None,25,vgStk.SansSerif);
   vge.ObjectSetColor(t, vgColor.Green);
   vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.LowerCenter));
   
   t = vge.TextCreateXY (-150,30,Trace,0,vgAlign.MiddleRight,vge.vgeScore.None,30,vgStk.SansSerif);
   vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleLeft));
   
   t = vge.TextCreateXY (-150,-40,Margin,0,vgAlign.MiddleRight,vge.vgeScore.None,30,vgStk.SansSerif);
   vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleLeft));
   
  
   node;
   
} /* function RootNode */





/*===========================================================================*/
/*==                      S t u b N o d e                                  ==*/
/*===========================================================================*/
/*\Docpage StubNode -- Add a labelled stub attached to specified node

DESCRIPTION

*/
function StubNode(pregion, parent, label)     /* Label this chain with a stub */
{
   pdata := vcRegion.GetData(pregion);
   tree := pdata.tree;
   node := vcTree.VrtxCreate(label,
                             vcTree.VrtxType.None,
                             nil,
                             parent,
                             nil,
                             width        <- 20,
                             height       <- 20,
                             border       <- vcTree.Border.Circle,
                             border_fill  <- vgFill.Grey04);
 
   vcTree.VrtxSetItem(node,nil);
   t := vge.TextCreateXY (-10,0,label,0,vgAlign.MiddleRight,vge.vgeScore.None,15,vgStk.SansSerif);
   vge.ObjectSetColor(t, vgColor.Cyan);
   vcTree.AddDecoration(tree,vcTree.DecorationCreateVrtx(t,node,vgAlign.MiddleLeft));
   node;
} /* function StubNode */



/*===========================================================================*/
/*==                      D r a w P a t h                                  ==*/
/*===========================================================================*/
/*\Docpage DrawPath -- Misc. Display Cleanup Routines

DESCRIPTION

*/
function DrawPath(pregion,root)     /* Tree to clean up for display  */
{
   pdata := vcRegion.GetData(pregion);
   tree := pdata.tree;
   foreach (a, b, e; vcTree.EdgesFrom(tree, root)) 
      vcTree.EdgeModify(e, style <- vcTree.EdgeStyle.Bent);
      
} /* function DrawPath */


/*===========================================================================*/
/*==                      S e l e c t e d V r t x L i s t                  ==*/
/*===========================================================================*/
/*\Docpage SelectedVrtxList --

DESCRIPTION

*/
function SelectedVrtxList(region)
{
      
   vertices := nil;
   declare i;
   declare c;
   selected := 0;

   pdata := vcRegion.GetData(region);
   tree := pdata.tree;
   
   if (tree != nil) {
      foreach (v; vcTree.Vertices(region)) {
         if (vcTree.IsVrtxSelected(tree,v))
            if ((i = vcTree.VrtxGetItem(v)) != nil)
               if (string_eqi(i.Key, "INST")) {
                  vertices = cons(v,vertices);
                  selected = selected + 1;
               }
      }
   
      window := vcRegion.Window(region);
      vcWin.ClearStatus(window);
      vcWin.AddStatus(window,sprintf("Selected Components: %d\n",selected));
   }
   vertices;
}




/*===========================================================================*/
/*==                      R e g i s t e r V r t x S e l e c t              ==*/
/*===========================================================================*/
/*\Docpage RegisterVrtxSelect -- Call when vertex is clicked

DESCRIPTION

*/
function RegisterVrtxSelect (region, pdata)
{

   tree := pdata.tree;

   function vertex_selection (vertex, flag) {
      declare		this;
      declare		i;
      declare     part;
      declare     pin;
      declare     parent;
      declare     window;

      if (!vcRegion.Destroyed(tree)) {
      
         this          = vcRegion.GetData (region);
         if ((i = vcTree.VrtxGetItem (vertex)) != nil) {
            if (flag) 
               vcTree.VrtxModify(vertex, border_fill <- vgFill.Grey04);
            else 
               vcTree.VrtxModify(vertex, border_fill <- vgFill.Hollow);

            if (pdata.Current_Edge != nil) {
               vcTree.EdgeModify (pdata.Current_Edge, descriptor <- normal_edge);
               pdata.Current_Edge = nil;
            }

            window = vcRegion.Window(region);
            vcWin.ClearStatus(window);
            if (i != nil) 
               vcWin.AddStatus(window,sprintf("Model:  %s   Comp:  %s   In:  %s  (%s)   Out:  %s  (%s)",
                                              vtime.GetModel(i),
                                              vtime.GetPart(i),
                                              vtime.GetIpin(i),
                                              vtime.GetItime(i),
                                              vtime.GetOpin(i),
                                              vtime.GetOtime(i)));
            
            else
               vcWin.AddStatus(window,"No information available for this selection");
         }
         else {
            vcTree.DeselectVrtx(tree,vertex);
            window = vcRegion.Window(region);
            vcWin.ClearStatus(window);
         }
      }
   }
      
   vcTree.CallOnSelect (tree, vertex_selection);
}


/*===========================================================================*/
/*==            R e g i s t e r V r t x A c t i v a t e                    ==*/
/*===========================================================================*/
/*\Docpage RegisterVrtxActivate - Call when vertex is Double clicked

DESCRIPTION

*/
function RegisterVrtxActivate (region, pdata)
{

   tree := pdata.tree;

   function vertex_activation (event, vertex) {
      declare		this;
      declare		design_item;
      declare		selected_node;
      
      
      this    = vcRegion.GetData (region);
      selected_node = vcTree.VrtxGetItem (vertex);
      /*design_item   = selected_node.design_item;*/
      /* Now Do something */
   }
   
   vcTree.CallOnVrtxActivate (tree, vertex_activation);
}



/*===========================================================================*/
/*==               D u m p T r e e                             Debug       ==*/
/*===========================================================================*/
function DumpTree(d,e)
{
   rgn := vcEvent.Region(e);
   vcTree.Print(rgn);

}
/*
PrintWholeTree := vcWv.Cmd (fac, 
                      DumpTree,
                      "PrintTree", "PrintTree");
*/


/*===========================================================================*/
/*==              R o o t s                                                ==*/
/*===========================================================================*/
function Roots(d,e)
{

   rgn := vcEvent.Region(e);
   window := vcRegion.Window(rgn);
   vertices := nil;
   declare i;
   declare c;
   selected := 0;
   deselected := 0;

   pdata    := vcRegion.GetData(rgn);
   tree    := pdata.tree;
   CurVrtx := vcTree.GetFocusVrtx(rgn);

   if (CurVrtx != nil) {
      Root := vcTree.VrtxTop(CurVrtx);
      V1   := vcTree.VrtxFirstOwned(Root);
      EmptyFlag := vlTrueFalse.True;
      
      if (V1 != nil) 
         for (v := vcTree.VrtxLevFirst(V1); 
              v != nil;  
              v = vcTree.VrtxLevNext(v)) {
            
            if (vertices != nil) 
               EmptyFlag = vlTrueFalse.False;

            foreach (vrtx; vcTree.VrtxSubtreePreorder (v)) {
               if (vcTree.IsVrtxSelected(rgn,vrtx)) {
                  if ((i = vcTree.VrtxGetItem(vrtx)) != nil) {
                     if (EmptyFlag && (i.Ptype != 2) && string_eqi(i.Key, "INST")) {
                        vertices = cons(vrtx,vertices);
                        selected = selected + 1;
                     }
                     else {
                        deselected = deselected + 1;
                        vcTree.DeselectVrtx(tree,vrtx);
                     }
                  }
               }
            }
         }
   }

   vcWin.ClearStatus(window);
   vcWin.AddStatus(window,sprintf("   %d Component(s) Selected.  %d Component(s) Deselected\n",
                                  selected,deselected));
   
   reverse_destructive(vertices);

}


/*===========================================================================*/
/*==              D o V i s i t                                            ==*/
/*===========================================================================*/
function DoVisit(d,e)
{
   rgn    := vcEvent.Region(e);
   window := vcRegion.Window(rgn);
   vlst   := SelectedVrtxList(rgn);
   len    := length(vlst);
   vtdata := d.vtdata;

   if ( len > 1) {
      vcWin.ClearStatus(vcEvent.Window(e));
      vcWin.AddStatus(window,"ERROR: Cannot Navigate with more than one component selected");
   }
   else if ( len < 1) {
      vcWin.ClearStatus(vcEvent.Window(e));
      vcWin.AddStatus(window,"ERROR: Nothing selected.  Select a component first.");
   }      
   else {
      part := car(vlst);
      partdata := vcTree.VrtxGetItem(part);
      cmd  := sprintf ("%s object %s", vtime.GetBlockName (car(vtdata)), vtime.GetPart(partdata));
      vcCmd.Run (TrackCmd, d, e, list (cmd), TRUE);
   }
      
}

TrackCmd := vcWv.Cmd (vcAlert.fac, 
                       func(d,e,location_string) {
                          if (vcAlert.track("vtime", 
                                            convert (vlIntVT,vlErrSev.Error), 
                                            convert (vlIntVT, 0), 
                                            location_string,"Navigation") != nil) {
                             vipc.ErrGetStr();
                             w := vcRegion.Window(vcEvent.Region(e));
                             vcWin.ClearStatus(w);
                             vcWin.AddStatus(w,"Navigator (vnav) is not running in this Framework");
                          }
                       },
                       "", "",
                       vcWv.Param("location", vlStringRestOfLineVT, vcWv.Constant,""));


McpCmd          := vcWv.Cmd (fac, CallMcpDialog);
NetInfoCmd      := vcWv.Cmd (fac, NetInfoFunc);
Set1Cmd         := vcWv.Cmd (fac, Set1);
Set0Cmd         := vcWv.Cmd (fac, Set0);
SetXCmd         := vcWv.Cmd (fac, SetX);
CompInfoCmd     := vcWv.Cmd (fac, CompInfoFunc);
ModelInfoCmd    := vcWv.Cmd (fac, ModelInfoFunc);
PopupMenuCmd    := vcWv.Cmd (fac, DoPopupMenuFunc);
DoVisitCmd      := vcWv.Cmd (fac, DoVisit);
UnselectAllCmd  := vcWv.Cmd (fac, UnselectAll);
   
   

