/* -----------------------------------------------------------------------*\
 * Module -- tracedlg.vs
 *
 *      Facility:      tracedlg.fac
 *
 *      Environment:   SUN4, Powerview 5.2
 *
 *      Authors:       Mani Alagappan,  Dec 1, 1993
 *                     Jeff Hare
 *
 *      Description:   ViewTime Trace Command Argument Editor Facility
 *       
 *
 *      Copyright (c) 1993 by Viewlogic Systems, Inc.
 *
\* ---------------------------------------------------------------------- */
/*\Section tracedlg -- ViewTime Trace Command Argument Editor Facility
*/

function label(str) vcRegion.Create1(str,vcLabel.fac,str,nil);

/* -------------------------------------------------- *\
 *  The following definitions map the actual Motive
 *  Trace command name to a symbolic name.  These
 *  strings are returned by the _Get method
\* -------------------------------------------------- */
TF  := "tf";   /* Source to Destination Trace     */
TL  := "tl";   /* Depth Limited Trace All         */
TO  := "to";   /* Trace All with Probe            */
TP  := "tp";   /* Primary Output Trace All (net)  */
TT  := "tt";   /* Trace All                       */
TW  := "tw";   /* Worst Case Trace                */
TZ  := "tz";   /* Primary Output Worst Case Trace */
TPI := "tpi";  /* Trace Primary Input             */
TPO := "tpo";  /* Trace Primary Output            */

/* -------------------------------------------------- *\
 *  The following definitions map Trace types to the
 *  button strings in the trace picklist. 
\* -------------------------------------------------- */
TFstr  := "Source to Destination";
TLstr  := "Depth Limited Trace All";
TOstr  := "Trace All with Probe";
TPstr  := "Primary Output Trace All";
TTstr  := "Trace All";
TWstr  := "Worst Case Trace";
TZstr  := "Primary Output Worst Case";
TPIstr := "Trace Primary Input";
TPOstr := "Trace Primary Output";

/* -------------------------------------------------- *\
 *  The following definitions map timing constraints
 *  and edges to constraint & edge strings.  These
 *  strings are used to form the string returned by
 *  the _Get method.
\* -------------------------------------------------- */
ALL := "All";
SET := "Setup";
HLD := "Hold";
PW  := "Pulse";     /* Pulsewidth = hold check?*/
LH  := "Rising";
HL  := "Falling";
LOW := "Low";       /* Pulsewidth low = Falling edge check? */
HI  := "High";      /* Pulsewidth hi  = Rising edge check? */

/* -------------------------------------------------- *\
 *  The following definitions map the Constraint and
 *  Edge types to the button strings in the picklist.
\* -------------------------------------------------- */
ALLstr         := "All Constraints";
SETUPstr       := "Setup Time";
HOLDstr        := "Hold Time";
PULSEstr       := "Pulsewidth";
RISINGstr      := "Rising Edge";
FALLINGstr     := "Falling Edge";
PULSELOWstr    := "Pulse Low";
PULSEHIGHstr   := "Pulse High";

/* -------------------------------------------------- *\
 * The following options are used for various types of
 * trace reports and modes.
\* -------------------------------------------------- */
SLK    := "slack";
SLKON  := "slack_on";
SLKOFF := "slack_off";

DET    := "detail";
DETON  := "detail_on";
DETOFF := "detail_off";

EXP    := "expl";
EXPON  := "expl_on";
EXPOFF := "expl_off";

SLKstr := "Slack Display";
DETstr := "Detailed Electrical Mode";
EXPstr := "Explanation Mode";


/*===========================================================================*/
/*==                       _ C r e a t e                                   ==*/
/*===========================================================================*/
/*\Function _Create  -- pathtrc facility data creation routine
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function _Create(Defaults, TopRegion)
{
   /*--------------------------------------------------*\
    *  The following region declarations are global to
    *  all the functions defined in _Create()
   \*--------------------------------------------------*/
   declare TrcRgn;        /* Main Trace Type Table     */
   declare TrcDepthRgn;   /* Trace Depth Scalebar      */
   declare ConEdgeRgn;    /* Const & Edges Table       */
   declare ConAllTypeRgn; /* Constraints "All" pklst   */
   declare ConTypeRgn;    /* Constraints Picklist      */
   declare EdgeTypeRgn;   /* Edge Type Picklist        */
   declare LevelTypeRgn;  /* Logic Level Picklist      */
   declare SrcRgn;        /* Source Comp/Pin Table     */
   declare DstRgn;        /* Dest Comp/Pin Table       */
   declare NetRgn;        /* Net Name Table            */
   declare SrcCmdRgn;     /* Source Comp/Pin Cmdline   */
   declare DstCmdRgn;     /* Dest Comp/Pin Cmdline     */
   declare NetCmdRgn;     /* Net Name Cmdline          */
   declare OptionRgn;     /* Other trace options       */
   declare EdgeLevelRgn;


   casemode := vcRegion.PrefGet(TopRegion,"case");

   if (Defaults == nil) {
      settings := GetDefaultSettings();
   }
   else {
      settings := Str2Data(Defaults);
   }
   

   /*====================================================*/
   /*==       B u i l d T r a c e G u i                ==*/
   /*====================================================*/
   /*\Function BuildTraceGui  -- Establish the Dialog box
     
     DESCRIPTION
     
     RETURNS
     
   */
   function BuildTraceGui()
   {  
      
      /*------------------------------------------*\
       *     Set up the Trace Type Region 
       *     ----------------------------
       * These symbols are used to generate the
       * Trace Picklist. The car(x) is the label
       * text. The cdr(x) is a cons cell that gets
       * passed to the picklist notify function.
       * The picklist notify function uses the 
       * car(y) for the trace type return value
       * string and the cdr(y) for a function to
       * call to set up this trace mode.
      \*------------------------------------------*/
      TRCWC      := cons(TWstr,  cons(TW,set_tw));
      TRCALL     := cons(TTstr,  cons(TT,set_tt));
      TRCSD      := cons(TFstr,  cons(TF,set_tf));
      TRCDPTH    := cons(TLstr,  cons(TL,set_tl));
      TRCPOUTALL := cons(TPstr,  cons(TP,set_tp));
      TRCPOUTWC  := cons(TZstr,  cons(TZ,set_tz));
      TRCALLPRB  := cons(TOstr,  cons(TO,set_to));
      TRCPRII    := cons(TPIstr, cons(TPI,set_tpi));
      TRCPRIO    := cons(TPOstr, cons(TPO,set_tpo));
      

      analysis_types := list(list( TRCWC,   TRCPOUTWC ),
                             list( TRCALL,  TRCALLPRB ),
                             list( TRCSD,   TRCPOUTALL ),
                             list( TRCPRII, TRCPRIO   ),
                             list( TRCDPTH ));
      
      TrcTypeTitleRgn := vcRegion.Create("Analysis Type", vcLabel.fac, "Analysis Type");
      TrcTypeRgn      := vcRegion.Create("Analysis Types Picklist", 
                                         vcPicklist.fac,
                                         analysis_types);
         
      vcPicklist.Notify( TrcTypeRgn, CallOnPick);
         
      TrcRgn   =  vcRegion.Create("TDBorder", vcBorder.fac, 
                                  tt := vcTable.Make("Trace Modes", 1, 2, 
                                                     TrcTypeTitleRgn, TrcTypeRgn));
      vcTable.Col (tt, -1, vcTable.Border.Visible, 5,5,5,vgAlign.MiddleLeft);
         
      /*----------------------------------------*\
             Set up the Trace Depth Region
      \*----------------------------------------*/
      TrcDepthTitleRgn := vcRegion.Create("Trace Depth Title", vcLabel.fac, "Trace Depth");
         
      TrcDepthScaleRgn := vcRegion.Create("Trace Depth Scalebar", 
                                          vcScalebar.fac, 5,
                                          increment <- 1,
                                          min <- 1,
                                          max <- 50,
                                          ticks <- 5,
                                          clip <- FALSE);
      vcScalebar.CallOnChange(TrcDepthScaleRgn, set_depth);   
      TrcDepthRgn = vcRegion.Create("TDBorder", vcBorder.fac, 
                                    tt := vcTable.Make("Trace Depth", 1, 2, 
                                                       TrcDepthTitleRgn, TrcDepthScaleRgn));
      vcTable.Col (tt, -1, vcTable.Border.Visible, 5,5,5,vgAlign.MiddleLeft);
         

      /*-------------------------------------------*\
       * Set up the Slack Region
       *-------------------------------------------*/
      SLACK       := cons(SLKstr,  cons(SLK,set_options));
      DETAIL      := cons(DETstr,  cons(DET,set_options));
      EXPLANATION := cons(EXPstr,  cons(EXP, set_options));
      
      option_types := list(list( SLACK, EXPLANATION, DETAIL));
                           
      OptionTypeTitleRgn  := vcRegion.Create("Option Title", vcLabel.fac, "Options");
      OptionTypeRgn       := vcRegion.Create("Option Picklist", 
                                             vcPicklist.fac,
                                             option_types,
                                             exclusive <- FALSE);
      
      vcPicklist.Notify( OptionTypeRgn, CallOnPick1);
      
      OptionRgn      =  vcTable.Make("Options", 1, 2, 
                                     OptionTypeTitleRgn, OptionTypeRgn);

      /*-------------------------------------------*\
       *   Set up the Trace Constraints Region 
       *   -----------------------------------
       * These symbols are used to generate the
       * Constraints Picklist. The car(x) is the 
       * label. The cdr(x) is a cons cell that gets
       * passed to the picklist notify function.
       * The picklist notify function uses the 
       * car(y) for the constraint type return
       * value string and the cdr(y) for a function
       * call to set up this trace mode.
      \*--------------------------------------------*/
      ALLCON    := cons(ALLstr,    cons(ALL,set_all));

      SETUP     := cons(SETUPstr,  cons(SET,set_setup));
      HOLD      := cons(HOLDstr,   cons(HLD,set_hold));
      PULSE     := cons(PULSEstr,  cons(PW, set_pulse));
      
      conall_types     := list (list(ALLCON));
      constraint_types := list(list(SETUP, HOLD, PULSE ));

      ConTypeTitleRgn  := vcRegion.Create("Con Title", vcLabel.fac, "Constraints");
      ConAllTypeRgn     = vcRegion.Create("Conall", 
                                          vcPicklist.fac,
                                          conall_types,
                                          exclusive <- FALSE);
      ConTypeRgn        = vcRegion.Create("Constraints Picklist", 
                                          vcPicklist.fac,
                                          constraint_types);
      vcPicklist.SetPick (ConTypeRgn, 0);
      vcPicklist.Notify( ConAllTypeRgn, CallOnPick1);
      vcPicklist.Notify( ConTypeRgn, CallOnPick);
         
      ConRgn      :=  vcTable.Make("Constraints", 1, 3, 
                                   ConTypeTitleRgn,ConAllTypeRgn, ConTypeRgn);

      
      /*---------------------------------------------*\
       *    Set up the Constraint Edge Region   
       *   -----------------------------------
       * These symbols are used to generate the
       * Constrained Edge Picklist. The car(x) is
       * the label. The cdr(x) is a cons cell is
       * passed to the picklist notify function.
       * The picklist notify function uses the 
       * car(y) for the constraint edge return
       * value string and the cdr(y) for a function
       * call to set up this edge/level mode.
      \*---------------------------------------------*/
      RISING      := cons(RISINGstr,  cons(LH,set_rising));
      FALLING     := cons(FALLINGstr, cons(HL,set_falling));
      
      PULSELOW    := cons(PULSELOWstr,  cons(LOW,set_low));
      PULSEHIGH   := cons(PULSEHIGHstr, cons(HI, set_high));
      
      edge_types  := list(list( RISING, FALLING ));
      level_types := list(list( PULSELOW, PULSEHIGH ));
      
      EdgeTypeTitleRgn := vcRegion.Create("Edge Title", vcLabel.fac, "Edge:");
      
      EdgeTypeRgn  = vcRegion.Create("Edge Picklist", 
                                     vcPicklist.fac,
                                     edge_types);
      LevelTypeRgn = vcRegion.Create("Level Picklist", 
                                     vcPicklist.fac,
                                        level_types);
      
      vcPicklist.Notify( EdgeTypeRgn,  CallOnPick);
      vcPicklist.Notify( LevelTypeRgn, CallOnPick);
      
      EdgeLevelRgn    = vcTable.Make("Constraints", 1, 3, EdgeTypeTitleRgn, 
                                     EdgeTypeRgn, LevelTypeRgn);
      ConEdgeRgn  =  vcRegion.Create("TDBorder", vcBorder.fac, 
                                     tt := vcTable.Make("Cons_and_Edges", 2, 1, 
                                                        ConRgn, EdgeLevelRgn));
      vcTable.Col (tt, -1, vcTable.Border.Visible, 5,5,5,vgAlign.MiddleLeft);
         
      /*----------------------------------------*\
           Set up the Src/Dst/Net names Region   
      \*----------------------------------------*/
      SrcTitleRgn := vcRegion.Create("Source",      vcLabel.fac, "Source: (Part Pin):");
      DstTitleRgn := vcRegion.Create("Destination", vcLabel.fac, "Destination: (Part Pin):");
      NetTitleRgn := vcRegion.Create("Net",         vcLabel.fac, "Net Name:");

      SrcCmdRgn    = vcRegion.Create("Src Cmd", vcCmdline.fac, "", case <- casemode);
      DstCmdRgn    = vcRegion.Create("Dst Cmd", vcCmdline.fac, "", case <- casemode);
      NetCmdRgn    = vcRegion.Create("Net Cmd", vcCmdline.fac, "", case <- casemode);
      
      vcCmdline.CallOnChange(SrcCmdRgn, CheckSrc); 
      vcCmdline.CallOnChange(DstCmdRgn, CheckDst);
      vcCmdline.CallOnChange(NetCmdRgn, CheckNet);

/*
      vcCmdline.CallOnReturn(SrcCmdRgn, func(e,str) CheckSrc(str);); 
      vcCmdline.CallOnReturn(DstCmdRgn, func(e,str) CheckDst(str);); 
      vcCmdline.CallOnReturn(NetCmdRgn, func(e,str) CheckNet(str););
*/
      SrcRgn       = vcTable.Make("Src", 1, 2,
                                  SrcTitleRgn, SrcCmdRgn);
      DstRgn       = vcTable.Make("Dst", 1, 2,
                                  DstTitleRgn, DstCmdRgn);
      NetRgn       = vcTable.Make("Net", 1, 2,
                                  NetTitleRgn, NetCmdRgn);

      NamesRgn    := vcRegion.Create("TDBorder", vcBorder.fac, 
                                     tt := vcTable.Make("Names", 3, 1, 
                                                        SrcRgn,DstRgn,NetRgn));
      vcTable.Col (tt, -1, vcTable.Border.Visible, 5,5,5,vgAlign.MiddleLeft);
      
         

      /*----------------------------------------*\
            Layout the Trace Analysis Window
      \*----------------------------------------*/
      LayoutRgn :=  vcRegion.Create("OkQuit", vcMargin.fac,
                                    tt := vcTable.Make("TraceArgEditor", 5, 1,
                                                       TrcRgn, 
                                                       TrcDepthRgn, 
                                                       OptionRgn,
                                                       ConEdgeRgn, 
                                                       NamesRgn));
      /*vcTable.AlignColumns (tt);*/
      vcTable.Col (tt, -1, vcTable.Border.Visible, 5,5,5,vgAlign.MiddleLeft);
      
      
      LayoutRgn;
   } /* BuildTraceGui */
   
   
   
   /*=====================================================*/
   /*==          C a l l O n P i c k                    ==*/
   /*=====================================================*/
   /*\Function CallOnPick  -- Control dialog presentation
     
     DESCRIPTION
     
     RETURNS
     
   */
   function CallOnPick(data, on_or_off) {
      if (on_or_off == vlOnOff.On) {
         if (isprocedure(cdr(data))) {
            (cdr(data))();
         }
         else {
            vlEx.Throw2(vlEx.FAILURE,"vtime",vlErrSev.Error,
                        "No action specified for this pick in tracedlg.BuildTraceGui()");
         }
      }
   } /* CallOnPick */
   
   /*=====================================================*/
   /*==          C a l l O n P i c k 1                  ==*/
   /*=====================================================*/
   /*\Function CallOnPick1  -- Control dialog presentation with args
     
     DESCRIPTION
          This is similar to the CallOnPick() except the notify
          function iscalled with arguments.
     RETURNS
     
   */
   function CallOnPick1(data, on_or_off) {
      if (isprocedure(cdr(data))) {
         (cdr(data))(car (data),on_or_off);
      }
      else 
         vlEx.Throw2(vlEx.FAILURE,"vtime",vlErrSev.Error,
                     "No action specified for this pick in tracedlg.BuildTraceGui()");
   } /* CallOnPick1 */


   /*=====================================================*/
   /*==  r e s e t _ a n d _ d i s a b l e _ a l l c o n==*/
   /*=====================================================*/
   function reset_and_disable_allcon(){
      reset_all_con_pick ();
      UsesAllCons(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ t w                       ==*/
   /*=====================================================*/
   function set_tw() {
      settings.TrcType = TW;
      UsesDepth(FALSE);
      UsesAllCons(TRUE);
      UsesCons(TRUE);
      UsesSrc(TRUE);
      UsesDst(FALSE);
      UsesNet(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ t t                       ==*/
   /*=====================================================*/
   function set_tt() {
      settings.TrcType = TT;
      UsesDepth(FALSE);

      if (string_eqi (settings.ConType, ALL))
         reset_and_disable_allcon ();

      UsesCons(FALSE);
      UsesSrc(TRUE);
      UsesDst(FALSE);
      UsesNet(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ t f                       ==*/
   /*=====================================================*/
   function set_tf() {
      settings.TrcType = TF;

      if (string_eqi (settings.ConType, ALL))
         reset_and_disable_allcon ();
      else
         UsesCons(TRUE);

      UsesDepth(FALSE);
      UsesSrc(TRUE);
      UsesDst(TRUE);
      UsesNet(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ t l                       ==*/
   /*=====================================================*/
   function set_tl() {
      settings.TrcType = TL;
      UsesDepth(TRUE);

      if (string_eqi (settings.ConType, ALL))
         reset_and_disable_allcon ();

      UsesCons(FALSE);
      UsesSrc(TRUE);
      UsesDst(FALSE);
      UsesNet(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ t p                       ==*/
   /*=====================================================*/
   function set_tp() {
      settings.TrcType = TP;
      UsesDepth(FALSE);

      if (string_eqi (settings.ConType, ALL))
         reset_and_disable_allcon ();

      UsesCons(FALSE);
      UsesSrc(FALSE);
      UsesDst(FALSE);
      UsesNet(TRUE);
   }
   /*=====================================================*/
   /*==               s e t _ t z                       ==*/
   /*=====================================================*/
   function set_tz() {
      settings.TrcType = TZ;
      UsesDepth(FALSE);

      if (string_eqi (settings.ConType, ALL))
         reset_and_disable_allcon ();
      else
         UsesCons(TRUE);

      UsesSrc(TRUE);
      UsesDst(FALSE);
      UsesNet(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ t o                       ==*/
   /*=====================================================*/
   function set_to() {
      settings.TrcType = TO;
      UsesDepth(FALSE);

      if (string_eqi (settings.ConType, ALL))
         reset_and_disable_allcon ();

      UsesCons(FALSE);
      UsesSrc(TRUE);
      UsesDst(TRUE);
      UsesNet(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ t p i                     ==*/
   /*=====================================================*/
   function set_tpi() {
      settings.TrcType = TPI;
      UsesDepth(FALSE);

      if (string_eqi (settings.ConType, ALL))
         reset_and_disable_allcon ();
      else
         UsesCons(TRUE);

      UsesSrc(TRUE);
      UsesDst(FALSE);
      UsesNet(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ t p o                     ==*/
   /*=====================================================*/
   function set_tpo() {
      settings.TrcType = TPO;
      UsesDepth(FALSE);

      if (string_eqi (settings.ConType, ALL))
         reset_and_disable_allcon ();
      else
         UsesCons(TRUE);

      UsesSrc(TRUE);
      UsesDst(FALSE);
      UsesNet(FALSE);
   }
   /*=====================================================*/
   /*==            s e t _ c o n _ t y p e              ==*/
   /*=====================================================*/
   function Set_Con_Type(){
      declare str;
      vcPicklist.Dimensions (ConTypeRgn,&nrows, &ncols);
      for(i:=0;i <nrows;++i)
         for (j:=0;j<ncols;++j){
            if (vcPicklist.IsSet (ConTypeRgn,i+j)){
               pick_data := vcPicklist.Get (ConTypeRgn, i, j);
               pick_str := car (pick_data);
               if (string_eqi (pick_str,SETUPstr)){
                  UsesEdge (TRUE);
                  str = SET;
               }
               else if (string_eqi (pick_str,HOLDstr)){
                  UsesEdge (TRUE);
                  str = HLD;
               }
               else if (string_eqi (pick_str,PULSEstr)){
                  UsesLevel (TRUE);
                  str = PW;
               }
               break;
            }
         }
      if (str == nil)
         str = "";
      /*if (str == nil){
         vcPicklist.SetByString(ConTypeRgn,SETUPstr);
         UsesEdge (TRUE);
         str = SET;         
      }*/
      str;
   }

   /*=====================================================*/
   /*==        r e s e t _ a l l _ c o n _ p i c k      ==*/
   /*=====================================================*/
   function reset_all_con_pick()
      if (string_eqi (settings.ConType, ALL))
         vcPicklist.SetByString (ConAllTypeRgn,ALLstr);
   /*=====================================================*/
   /*==               s e t _ a l l                     ==*/
   /*=====================================================*/
   function set_all(data,onoff) {
      if (onoff == vlOnOff.On){
         /*UsesCons (FALSE);*/
         vcRegion.Enable (ConTypeRgn,vlOnOff.Off, TRUE);
         settings.ConType = ALL;
         UsesEdgeLevel(FALSE);
      }
      else{
         /*UsesCons (TRUE);*/
         vcRegion.Enable (ConTypeRgn,vlOnOff.On, TRUE);
         settings.ConType = Set_Con_Type();
         UsesEdgeLevel (TRUE);
      }
   }
   /*=====================================================*/
   /*==               s e t _ s e t u p                 ==*/
   /*=====================================================*/
   function set_setup() {
      reset_all_con_pick ();
      settings.ConType = SET;
      /*UsesCons(FALSE);*/
      UsesEdge(TRUE);
      UsesLevel(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ d e p t h                 ==*/
   /*=====================================================*/
   function set_depth(val) {
      settings.TrcDepth = val;
      vcScalebar.Set(TrcDepthRgn,settings.TrcDepth);
      
   }
   /*=====================================================*/
   /*==               s e t _ h o l d                   ==*/
   /*=====================================================*/
   function set_hold() {
      reset_all_con_pick ();
      settings.ConType = HLD;
      UsesEdge(TRUE);
      UsesLevel(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ p u l s e                 ==*/
   /*=====================================================*/
   function set_pulse() {
      reset_all_con_pick ();
      settings.ConType = PW;
      UsesEdge(FALSE);
      UsesLevel(TRUE);
   }
   /*=====================================================*/
   /*==               s e t _ r i s i n g               ==*/
   /*=====================================================*/
   function set_rising() {
      settings.EdgeType = LH;
      UsesEdge(TRUE);
      UsesLevel(FALSE);
   }
   /*=====================================================*/
   /*==               s e t _ f a l l i n g             ==*/
   /*=====================================================*/
   function set_falling() {
      settings.EdgeType = HL;
      UsesEdge(TRUE);
      UsesLevel(FALSE);
   } 
  /*=====================================================*/
   /*==               s e t _ h i g h                   ==*/
   /*=====================================================*/
   function set_high() {                  /* !!! CHECK THESE !!! */
      settings.LevelType = HI;            /* how is a pulsewidth check done? */
      UsesEdge(FALSE);
      UsesLevel(TRUE);
   }
   /*=====================================================*/
   /*==               s e t _ l o w                     ==*/
   /*=====================================================*/
   function set_low() {
      settings.LevelType = LOW;
      UsesEdge(FALSE);
      UsesLevel(TRUE);
   }
   /*=====================================================*/
   /*==               s e t _ s r c                     ==*/
   /*=====================================================*/
   function set_src(str) {
      vcCmdline.Set(SrcCmdRgn,str);
   }
   /*=====================================================*/
   /*==               s e t _ d s t                     ==*/
   /*=====================================================*/
   function set_dst(str) {
      vcCmdline.Set(DstCmdRgn,str);
   }
   /*=====================================================*/
   /*==               s e t _ n e t                     ==*/
   /*=====================================================*/
   function set_net(str) {
      vcCmdline.Set(NetCmdRgn,str);
   }
   /*=====================================================*/
   /*==            s e t _ o p t i o n s                ==*/
   /*=====================================================*/
   function set_options(data,onoff){
      if (onoff == vlOnOff.On){
         if (string_eqi (data,SLK))
            settings.SlackType = SLKON;
         else if (string_eqi (data,DET))
            settings.DetailMode = DETON;
         else if (string_eqi (data,EXP))
            settings.ExplMode = EXPON;
      }
      else{
         if (string_eqi (data,SLK))
            settings.SlackType = SLKOFF;
         else if (string_eqi (data,DET))
            settings.DetailMode = DETOFF;
         else if (string_eqi (data,EXP))
            settings.ExplMode= EXPOFF;
      }
   }
   /*=====================================================*/
   /*==               U s e s C o n s                   ==*/
   /*=====================================================*/
   function UsesCons(yesno) {

      if (yesno == vlTrueFalse.True)
         vcRegion.Enable(ConEdgeRgn, vlOnOff.On, TRUE);  
      else
         vcRegion.Enable(ConEdgeRgn, vlOnOff.Off, TRUE);
   }

   /*=====================================================*/
   /*==               U s e s C o n s                   ==*/
   /*=====================================================*/
   function UsesAllCons(yesno) {
      if (yesno == vlTrueFalse.True){
         vcRegion.Enable(ConAllTypeRgn, vlOnOff.On, TRUE);  
      }
      else{
         vcRegion.Enable(ConAllTypeRgn, vlOnOff.Off, TRUE);
      }
   }

   /*=====================================================*/
   /*==               U s e s D e p t h                 ==*/
   /*=====================================================*/
   function UsesDepth(yesno) {
      if (yesno == vlTrueFalse.True)
         vcRegion.Enable(TrcDepthRgn, vlOnOff.On, TRUE);  
      else
         vcRegion.Enable(TrcDepthRgn, vlOnOff.Off, TRUE);
   }

   function UsesEdgeLevel(yesno){
      if (yesno == vlTrueFalse.True)
         vcRegion.Enable(EdgeLevelRgn, vlOnOff.On, TRUE);  
      else
         vcRegion.Enable(EdgeLevelRgn, vlOnOff.Off, TRUE);
   }

   /*=====================================================*/
   /*==               U s e s E d g e                   ==*/
   /*=====================================================*/
   function UsesEdge(yesno) {
      if (yesno == vlTrueFalse.True)
         vcRegion.Enable(EdgeTypeRgn, vlOnOff.On, TRUE);  
      else
         vcRegion.Enable(EdgeTypeRgn, vlOnOff.Off, TRUE);
   }

   /*=====================================================*/
   /*==               U s e s L e v e l                 ==*/
   /*=====================================================*/
   function UsesLevel(yesno) {
      if (yesno == vlTrueFalse.True)
         vcRegion.Enable(LevelTypeRgn, vlOnOff.On, TRUE);  
      else
         vcRegion.Enable(LevelTypeRgn, vlOnOff.Off, TRUE);
   }

   /*=====================================================*/
   /*==               U s e s S r c                     ==*/
   /*=====================================================*/
   function UsesSrc(yesno) {
      if (yesno == vlTrueFalse.True)
         vcRegion.Enable(SrcRgn, vlOnOff.On, TRUE);  
      else
         vcRegion.Enable(SrcRgn, vlOnOff.Off, TRUE);

   }

   /*=====================================================*/
   /*==               U s e s D s t                     ==*/
   /*=====================================================*/
   function UsesDst(yesno) {
      if (yesno == vlTrueFalse.True)
         vcRegion.Enable(DstRgn, vlOnOff.On, TRUE);  
      else
         vcRegion.Enable(DstRgn, vlOnOff.Off, TRUE);
      
   }

   /*=====================================================*/
   /*==               U s e s N e t                     ==*/
   /*=====================================================*/
   function UsesNet(yesno) {
      if (yesno == vlTrueFalse.True)
         vcRegion.Enable(NetRgn, vlOnOff.On, TRUE);  
      else
         vcRegion.Enable(NetRgn, vlOnOff.Off, TRUE);
   }


   /*====================================================*/
   /*==       U p d a t e T r a c e G u i              ==*/
   /*====================================================*/
   /*\Function UpdateTraceGui  -- Refresh using settings
     
     DESCRIPTION
     
   */
   function UpdateTraceGui(s) {
      
      str := "";

      if (isstring(s.SrcComp)) set_src(s.SrcComp);

      if (isstring(s.DstComp)) set_dst(s.DstComp);

      if (s.NetName != nil)  set_net(s.NetName);

      if (s.TrcDepth != nil) set_depth(s.TrcDepth);

      if (string_eqi(s.EdgeType,LH)) {
         set_rising();
         vcPicklist.SetByString(EdgeTypeRgn,RISINGstr);
      }         
      if (string_eqi(s.EdgeType,HL)) {
         set_falling();
         vcPicklist.SetByString(EdgeTypeRgn,FALLINGstr);
      }         
      if (string_eqi(s.LevelType,HI)) {
         set_high();
         vcPicklist.SetByString(LevelTypeRgn,PULSEHIGHstr);
      }         
      if (string_eqi(s.LevelType,LOW)) {
         set_low();
         vcPicklist.SetByString(LevelTypeRgn,PULSELOWstr);
      }         
      if (string_eqi(s.ConType, ALL)) {
         set_all(ALL,vlOnOff.On);
         vcPicklist.SetByString(ConAllTypeRgn,ALLstr);
      }         
      else if (string_eqi(s.ConType, SET)) {
         set_setup();
         vcPicklist.SetByString(ConTypeRgn,SETUPstr);
      }         
      else if (string_eqi(s.ConType, HLD)) {
         set_hold();
         vcPicklist.SetByString(ConTypeRgn,HOLDstr);
      }         
      else if (string_eqi(s.ConType, PW)) {
         set_pulse();
         vcPicklist.SetByString(ConTypeRgn,PULSEstr);
      }          
     if (string_eqi(s.TrcType, TF)) {
         if (s.TrcType)  set_tf();
         vcPicklist.SetByString(TrcRgn,TFstr);
      }
      else if (string_eqi(s.TrcType, TL)) {
         if (s.TrcType)  set_tl();
         vcPicklist.SetByString(TrcRgn,TLstr);
      }
      else if (string_eqi(s.TrcType, TO)) {
         if (s.TrcType)  set_to();
         vcPicklist.SetByString(TrcRgn,TOstr);
      }
      else if (string_eqi(s.TrcType, TP)) {
         if (s.TrcType)  set_tp();
         vcPicklist.SetByString(TrcRgn,TPstr);
      }
      else if (string_eqi(s.TrcType, TT)) {
         if (s.TrcType)  set_tt();
         vcPicklist.SetByString(TrcRgn,TTstr);
      }
      else if (string_eqi(s.TrcType, TW)) {
         if (s.TrcType)  set_tw();
         vcPicklist.SetByString(TrcRgn,TWstr);
      }
      else if (string_eqi(s.TrcType, TZ)) {
         if (s.TrcType)  set_tz();
         vcPicklist.SetByString(TrcRgn,TZstr);
      }
      else if (string_eqi(s.TrcType, TPI)) {
         if (s.TrcType)  set_tpo();
         vcPicklist.SetByString(TrcRgn,TPIstr);
      }
      else if (string_eqi(s.TrcType, TPO)) {
         if (s.TrcType)  set_tpo();
         vcPicklist.SetByString(TrcRgn,TPOstr);
      }
      else {
         print ("Unrecognized trace type"+s.TrcType);
      }

      if (string_eqi(s.SlackType,SLKON)) 
         vcPicklist.SetByString (OptionRgn,SLKstr);

      if (string_eqi(s.DetailMode,DETON)) 
         vcPicklist.SetByString (OptionRgn,DETstr);

      if (string_eqi(s.ExplMode,EXPON)) 
         vcPicklist.SetByString (OptionRgn,EXPstr);
      

   } /* UpdateTraceGui */
   

   function CheckSrc(cname)
   {
      declare token;
      maybePin := nil;
      i := 1;

      foreach(token; string_loop(cname," ,")) {
         if (i==1) {
            settings.SrcComp = token;
         }
         if (i==2) {
            if (string_eqi("pin",token)) 
               maybePin = token;
            else
               settings.SrcComp = settings.SrcComp + " " + token;
         }
         if (i==3) {
            if (maybePin != nil) {
               settings.SrcComp = settings.SrcComp + " " + token;
            }
            else
               printf ("ignoring extra characters...\n");
         }
         if (i>3)
            printf ("ignoring extra characters...\n");

         i = i + 1;
      }

      /*-----------------------------------------------
         Handles the one case where only two arguments
         are present and the pin's name is "pin"...
      */
      if (i==3 && maybePin != nil) {
         settings.SrcComp = settings.SrcComp + " " + maybePin;
      }

   }

   function CheckDst(cname)
   {
      declare token;
      maybePin := nil;
      i := 1;
      foreach(token; string_loop(cname," ,")) {
         if (i==1) {
            settings.DstComp = token;
         }
         if (i==2) {
            if (string_eqi("pin",token)) 
               maybePin = token;
            else
               settings.DstComp = settings.DstComp + " " + token;
         }
         if (i==3) {
            if (maybePin != nil) {
               settings.DstComp = settings.DstComp + " " + token;
            }
            else
               printf ("ignoring extra characters...\n");
         }
         i = i + 1;
      }

      /*-----------------------------------------------
         Handles the one case where only two arguments
         are present and the pin's name is "pin"...
      */
      if (i==3 && maybePin != nil) {
         settings.DstComp = settings.DstComp + " " + maybePin;
      }

   }

   function CheckNet(cname)
   {
      declare token;

      i := 1;
      foreach(token; string_loop(cname," ,")) {
         if (i==1) {
            settings.NetName = token;
         }
         if (i>1) {
               printf ("ignoring extra characters...\n");
         }
         i = i + 1;
      }
   }
   
   TraceRgn := BuildTraceGui();
   UpdateTraceGui(settings);
   vcRegion.AddChild(TopRegion,TraceRgn);

   settings;  /* Data is a structure */
}


/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function Data2Str  -- Convert a settings structure to a string
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function Data2Str(s,r)
{
   declare str;

   function ResetOptions(s){
      s.SlackType  = "";
      s.DetailMode = "";
      s.ExplMode   = "";
   }

   if (string_eqi(s.TrcType, TF)) {
      str = sprintf("%s %s %s %s %s %s %s %s", s.TrcType,
                    s.SrcComp,
                    s.DstComp,
                    s.ConType,
                    (string_eqi(s.ConType,PW)?s.LevelType:s.EdgeType),
                    s.SlackType,
                    s.DetailMode,
                    s.ExplMode);
      vcWin.AddStatus (vcRegion.Window(r),"Note: Path display NOT available for this trace");
      ResetOptions(s);
   }
   else if (string_eqi(s.TrcType, TL)) {
      str = sprintf("%s %s %s %s %s %s", s.TrcType,
                    s.SrcComp,
                    s.TrcDepth,
                    s.SlackType,
                    s.DetailMode,
                    s.ExplMode);
      vcWin.AddStatus (vcRegion.Window(r),"Note: Path display NOT available for this trace");
      ResetOptions(s);
   }
   else if (string_eqi(s.TrcType, TO)) {
      str = sprintf("%s %s %s %s %s %s", s.TrcType,
                    s.SrcComp,
                    s.DstComp,
                    s.SlackType,
                    s.DetailMode,
                    s.ExplMode);
      vcWin.AddStatus (vcRegion.Window(r),"Note: Path display NOT available for this trace");
      ResetOptions(s);
   }
   else if (string_eqi(s.TrcType, TP)) {
      str = sprintf("%s %s %s %s %s", s.TrcType,
                    s.NetName,
                    s.SlackType,
                    s.DetailMode,
                    s.ExplMode);
      vcWin.AddStatus (vcRegion.Window(r),"Note: Path display NOT available for this trace");
      ResetOptions(s);
   }
   else if (string_eqi(s.TrcType, TT)) {
      str = sprintf("%s %s %s %s %s", s.TrcType,
                    s.SrcComp,
                    s.SlackType,
                    s.DetailMode,
                    s.ExplMode);
      vcWin.AddStatus (vcRegion.Window(r),"Note: Path display NOT available for this trace");
      ResetOptions(s);
   }
   else if (string_eqi(s.TrcType, TW)) {
      if (string_eqi (s.ConType,ALL))
         str = sprintf("%s %s %s %s %s %s", 
                       s.TrcType,
                       s.SrcComp,
                       s.ConType, 
                       s.SlackType,
                       s.DetailMode,
                       s.ExplMode);
      else
         str = sprintf("%s %s %s %s %s %s %s", 
                       s.TrcType,
                       s.SrcComp,
                       s.ConType, 
                       (string_eqi(s.ConType,PW)?s.LevelType:s.EdgeType),
                       s.SlackType,
                       s.DetailMode,
                       s.ExplMode);
      vcWin.AddStatus (vcRegion.Window(r),"Note: Path display available for this trace");
      ResetOptions(s);      
   }
   else if (string_eqi(s.TrcType, TZ)) {
      str = sprintf("%s %s %s %s %s %s %s", s.TrcType,
                    s.SrcComp,
                    s.ConType, (string_eqi(s.ConType,PW)?s.LevelType:s.EdgeType),
                    s.SlackType,
                    s.DetailMode,
                    s.ExplMode);
      vcWin.AddStatus (vcRegion.Window(r),"Note: Path display NOT available for this trace");
      ResetOptions(s);
   }
   else if (string_eqi(s.TrcType, TPI)) {
      str = sprintf("%s %s %s %s %s", s.TrcType,
                    s.SrcComp,
                    s.SlackType,
                    s.DetailMode,
                    s.ExplMode);
      vcWin.AddStatus (vcRegion.Window(r),"Note: Path display NOT available for this trace");
      ResetOptions(s);
   }
   else if (string_eqi(s.TrcType, TPO)) {
      str = sprintf("%s %s %s %s %s", s.TrcType,
                    s.SrcComp,
                    s.SlackType,
                    s.DetailMode,
                    s.ExplMode);
      vcWin.AddStatus (vcRegion.Window(r),"Note: Path display NOT available for this trace");
      ResetOptions(s);
   }
   else {
      vcWin.AddStatus (vcRegion.Window(r),
                       sprintf ("Error:Unrecognized trace type %s",s.TrcType));
   }
   str;
} /* Data2Str */



/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function Str2Data  -- Convert a string to a settings structure
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function Str2Data(str)
{

   function SetDefOptions(argv,start, end)
   {
      for (i=start;i < end; ++i){
         if (string_eqi (argv[i],SLKON)){
            s.SlackType = SLKON;
         }
         else if (string_eqi (argv[i],DETON))
            s.DetailMode = DETON;
         else if (string_eqi (argv[i],EXPON))
            s.ExplMode = EXPON;             
      }
   }

   /*-----------------------------------------------------*\
      tp               net
      [tpi | tpo | tt] comp pin
      tl               comp pin depth
      [tz  | tw]       comp pin constraint edge
      to               comp pin comp pin
      tf               comp pin comp pin constraint edge
   \*-----------------------------------------------------*/
   cmdlist := nil;
   i := 0;

   foreach(token; string_loop(str," ")) {
      cmdlist = cons(token,cmdlist);
      i = i+1;
   }

   cmdlist = reverse_destructive(cmdlist);

   s := GetDefaultSettings();
   if (i > 0) if (string_eqi(cmdlist[0], TF)) {
      s.TrcType = cmdlist[0];
      if (i > 1) s.SrcComp = cmdlist[1];
      if (i > 2) s.SrcComp = s.SrcComp + " " + cmdlist[2];
      if (i > 3) s.DstComp = cmdlist[3];
      if (i > 4) s.DstComp = s.DstComp + " " + cmdlist[4];
      if (i > 5) s.ConType = cmdlist[5];
      if (i > 6) SetDefOptions (cmdlist,6,i);
      if (string_eqi(s.ConType,PW))
         if (i > 6)
            s.LevelType    = cmdlist[6];
         else
            s.LevelType    = LOW;
      else
         if (i > 6) 
            s.EdgeType= cmdlist[6];
         else
            s.EdgeType=LH;
   }
   else if (string_eqi(cmdlist[0], TL)) {
      s.TrcType = cmdlist[0];
      if (i > 1) s.SrcComp = cmdlist[1];
      if (i > 2) s.SrcComp = s.SrcComp + " " + cmdlist[2];
      if (i > 3) SetDefOptions (cmdlist,3,i);
      catch (vlEx.FAILURE) 
         if (i > 3) s.TrcDepth= convert(vlIntVT,cmdlist[3]);
      exception
         s.TrcDepth= 5;
   }
   else if (string_eqi(cmdlist[0], TO)) {
      s.TrcType = cmdlist[0];
      if (i > 1) s.SrcComp = cmdlist[1];
      if (i > 2) s.SrcComp = s.SrcComp + " " + cmdlist[2];
      if (i > 3) s.DstComp = cmdlist[3];
      if (i > 4) s.DstComp  = s.DstComp + " " + cmdlist[4];
      if (i > 5) SetDefOptions (cmdlist,5,i);
   }
   else if (string_eqi(cmdlist[0], TP)) {
      s.TrcType = cmdlist[0];
      if (i > 1) s.NetName = cmdlist[1];
      if (i > 2) SetDefOptions (cmdlist,6,2);
   }
   else if (string_eqi(cmdlist[0], TT)) {
      s.TrcType = cmdlist[0];
      if (i > 1) s.SrcComp = cmdlist[1];
      if (i > 2) s.SrcComp = s.SrcComp + " " + cmdlist[2];
      if (i > 3) SetDefOptions (cmdlist,3,i);
   }
   else if (string_eqi(cmdlist[0], TW)) {
      s.TrcType = cmdlist[0];
      if (i > 1) s.SrcComp = cmdlist[1];
      if (i > 2) s.SrcComp = s.SrcComp + " " + cmdlist[2];
      if (i > 3) s.ConType = cmdlist[3];      
      if (string_eqi (s.ConType, ALL)){         
         if (i > 4) SetDefOptions (cmdlist,4,i);
      }
      else{
         if (string_eqi(s.ConType,PW))
            if (i > 4)
               s.LevelType    = cmdlist[4];
            else
               s.LevelType    = LOW;
         else
            if (i > 4)
               s.EdgeType= cmdlist[4];
            else
               s.EdgeType=LH;
         if (i > 5) SetDefOptions (cmdlist,5,i);
      }
   }
   else if (string_eqi(cmdlist[0], TZ)) {
      s.TrcType = cmdlist[0];
      if (i > 1) s.SrcComp = cmdlist[1];
      if (i > 2) s.SrcComp = s.SrcComp + " " + cmdlist[2];
      if (i > 3) s.ConType = cmdlist[3];
      if (string_eqi(s.ConType,PW))
         if (i > 4)
            s.LevelType    = cmdlist[4];
      else
         if (i > 4) 
            s.EdgeType= cmdlist[4];
      if (i > 5) SetDefOptions (cmdlist,5,i);
   }
   else if (string_eqi(cmdlist[0], TPI)) {
      s.TrcType = cmdlist[0];
      if (i > 1) s.SrcComp = cmdlist[1];
      if (i > 2) s.SrcComp = s.SrcComp + " " + cmdlist[2];
      if (i > 3) SetDefOptions (cmdlist,3,i);
   }
   else if (string_eqi(cmdlist[0], TPO)) {
      s.TrcType = cmdlist[0];
      if (i > 1) s.SrcComp = cmdlist[1];
      if (i > 2) s.SrcComp = s.SrcComp + " " + cmdlist[2];
      if (i > 3) SetDefOptions (cmdlist,3,i);
   }
   else {
      print ("Unrecognized trace type"+cmdlist[0]);
   }

   s;
   
} /* Str2Data */

/*===========================================================================*/
/*==                      _ G e t                                          ==*/
/*===========================================================================*/
/*\Function _Get  -- The _Get data method for this facility
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function _Get(d,r,type) {      

	foo := Data2Str(d,r);   
} /* _Get */


/*===========================================================================*/
/*==                                                                       ==*/
/*===========================================================================*/
/*\Function GetDefaultSettings  -- 
  
  DESCRIPTION
       
  RETURNS
       
  EXAMPLE
       
  CAUTION
       
  SEE ALSO
       
*/
function GetDefaultSettings() {
   tt := TW;
   sc := "";
   sp := "";
   dc := "";
   dp := "";
   nn := "";
   td := 5;
   lt := LOW;
   et := LH;
   ct := SET;

   struct {
      TrcType   = tt;
      SrcComp   = sc;
      DstComp   = dc;
      NetName   = nn;
      TrcDepth  = td;
      ConType   = ct;
      EdgeType  = et;
      LevelType = lt;
      SlackType = SLKOFF;
      DetailMode = DETOFF;
      ExplMode   = EXPOFF
   } settings;
} /* GetDefaultSettings */


vlPref.Replace (prefs, "case", vcCmdlineCaseVT,
            "Fold character case to Upper, Lower or Either (mixed)",
            vcCmdlineCase.Lower);


