/*
 * Include/Exclude facility.
 *
 */
function _Create(data, region) {

    this := struct {
        lInclude = nil;
        rgnInclude;
        lExclude = nil;
        rgnExclude
    };

    declare include_list, exclude_list;
    foreach(elem; vscript.list_iter(car(data))) {
        this.lInclude = cons(elem, this.lInclude);
        include_list  = cons(cons(elem, elem), include_list);
    }
    foreach(elem; vscript.list_iter(cdr(data))) {
        this.lExclude = cons(elem, this.lExclude);
        exclude_list  = cons(cons(elem, elem), exclude_list);
    }
    will_scroll := vcRegion.PrefGet (region, "scroll");
    function build_picklist(title, items) {
        picklist := vcRegion.Create(title, vcPicklist.fac, items,
                                    columns <- 1,
                                    will_scroll <- will_scroll,
                                    enhanced_notify <- TRUE,
                                    exclusive <- FALSE);
        if (will_scroll)
            scroll := 
                vcPicklist.Scroll ("scroll", picklist,
                                   vcRegion.PrefGet (region, "scroll_min"),
                                   vcRegion.PrefGet (region, "scroll_ideal"),
                                   vcRegion.PrefGet (region, "scroll_max"));
        else
            scroll := picklist;
    }
    this.rgnInclude = build_picklist("Include", include_list);
    this.rgnExclude = build_picklist("Exclude", exclude_list);
    rgnInclude := vcTable.Make("Include", 2, 1, 
                               vcRegion.Create("Include", vcStaticText.fac, 
                                               "Include",
                                               formatted <- TRUE, 
                                               recessed_text <- TRUE),
                               this.rgnInclude);
    rgnExclude := vcTable.Make("Include", 2, 1, 
                               vcRegion.Create("Exclude", vcStaticText.fac, 
                                               "Exclude",
                                               formatted <- TRUE, 
                                               recessed_text <- TRUE),
                               this.rgnExclude);
    function remove_from_list(l, d) {
        newl := nil; 
        foreach(e; vscript.list_iter(l)) {
            if (e != d) 
                newl = cons(e, newl);
        }
        newl;
    }
    function move_selected(include_to_exclude) {
        del_list := nil;
        plFrom := (include_to_exclude) ? this.rgnInclude : this.rgnExclude;
        plTo := (include_to_exclude) ? this.rgnExclude : this.rgnInclude;
        foreach (r, c, i, is_set, v, d; vcPicklist.Items(plFrom)) 
            if (is_set) {
                e := vcPicklist.Get(plFrom, i, 0);
                vcPicklist.Add(plTo, car(e), cdr(e));
                del_list = cons(i, del_list);
                if (include_to_exclude) {
                    this.lExclude = cons(cdr(e), this.lExclude);
                    this.lInclude = remove_from_list(this.lInclude, cdr(e));
                }
                else {
                    this.lInclude = cons(cdr(e), this.lInclude);
                    this.lExclude = remove_from_list(this.lExclude, cdr(e));
                }
            }
        foreach (i; vscript.list_iter(del_list)) 
            vcPicklist.DeleteRows(plFrom, i, 1);
    }
        
    btnLeft := vcWv.IconButtonCreate("left", "arrow-l", nil, func (e) {
        move_selected(FALSE); }, nil);

    btnRight := vcWv.IconButtonCreate("right", "arrow-r", nil, func (e) {
        move_selected(TRUE); }, nil);
    rgnButtons := vcTable.Make("Buttons", 4, 1, 
                               vcRubber.Create("", 0, 0),
                               btnLeft, btnRight,
                               vcRubber.Create("", 0, 0));
    rgnTable := vcTable.Make("table", 1, 3,
                             rgnInclude,
                             rgnButtons,
                             rgnExclude);
    vcRegion.AddChild (region, rgnTable);
    this;
} /* function _Create */

function _Get(data, region, type) 
    vcRegion.GetData(region);

function GetIncludeList(this) 
    this.lInclude;

function GetExcludeList(this) 
    this.lExclude;


vlPref.Replace (prefs, "scroll", vlTrueFalseVT,
                "Show a scrollbar for the picklist", TRUE);
vlPref.Replace (prefs, "scroll_min", vlUnsignedIntVT,
                "Minimum number of lines for scrolled region", 4);
vlPref.Replace (prefs, "scroll_ideal", vlUnsignedIntVT,
                "Initial number of lines for scrolled region", 6);
vlPref.Replace (prefs, "scroll_max", vlUnsignedIntVT,
                "Maximum number of lines for scrolled region", 0);

/* Stop window from closing if you click too quickly */ 
vcBind.Set (bindings, '\leftdbl', vcWv.Cmd (fac, func (d, e);, ""));
