/************** command line completion stuff  - thanks, Adrian! *********/

/* compares s1 and s2 and returns the first index where they don't match
 * starts the search at 'start', which promises that the strings
 * are already known to be equal up to that point
 */
function equal_to (s1, s2, start) {
    declare i;
    for (i = start; 
         i < length (s1) && i < length (s2) && s1[i] == s2[i];
         ++i);
    i;
}
function complete(str, list) {
    declare possibilities;
    l := length (str);
    /* collect all the ones that match */
    foreach (elem; vscript.iter (list)) {
        /* Note that this version works with assoc lists */
        e := car(elem);
        if (l <=  length (e) && equal (str, substring (e, 0, l)))
             possibilities = cons (e, possibilities);
    }
 
    /* shrink back the best_match to whatever everyone can match */
    declare ret;
    if (possibilities != nil) {
        first_match := car (possibilities);
        possibilities = cdr (possibilities);
        best_match := length (first_match);
        while (possibilities != nil && best_match > l) {
            elem := car (possibilities);
            possibilities = cdr (possibilities);
            match := equal_to (elem, first_match, l);
            if (match < best_match)
                best_match = match;
        }
        ret = substring (first_match, 0, best_match);
        if (equal (ret, str))
            ret = nil;
    } 
    ret;
}

/*
 * Pulldown picklist facility...not documented yet.
 * 
 * Data passed to the Create routine will be passed back to
 * the initial and structure preferences if those preferences
 * are functions.
 * 
 */
function _Create(userdata, region) 
{
    initial := vcRegion.PrefGet (region, "initial");
    if (isprocedure(initial))
        value := initial(userdata);
    else
        value := initial;
    
    type := typeof (value);
    edit_fac := vcRegion.PrefGet (region, "edit_fac");
    edit_prefs := vcRegion.PrefGet (region, "edit_prefs");

    width := vcRegion.PrefGet (region, "width");
    do_scroll := vcRegion.PrefGet (region, "scroll");

    arrow := vcRegion.Create ("arrow", vcArrow.fac, Orient.down,
                             triangle <- TRUE);
    label_fac := vcRegion.PrefGet (region, "label_fac");
    label_prefs := vcRegion.PrefGet (region, "label_prefs");
    if (label_fac == vcScalebar.fac)
        label_prefs = cons (width <- width, label_prefs);
    label := vcRegion.Create1 ("str", label_fac, value, label_prefs);

    /* Command line completion for picklists */
    if ((label_fac == vcCmdline.fac) &&
        (edit_fac == vcPicklist.fac)) {
        vcCmdline.CallOnComplete(label, func(str) {
            structure := vcRegion.PrefGet (region, "structure");
            if (isprocedure(structure))
                picklist_structure := structure(userdata);
            else
                picklist_structure := structure;
            newstr := complete(str, picklist_structure);
            if (!null(newstr)) vcCmdline.Set(label, newstr);
            else  
                popup_picklist(vcEvent.Create (0, 0, '\0', nil, 
                                               vcRegion.Window (label),
                                               label, nil));
        });
    } 
    value_label := cons (value, label);

    function popup_picklist(e) 
    {
        declare edit_region;
        
        set_car (value_label, vcRegion.Get (cdr (value_label), type));
        if (edit_fac == vcPicklist.fac) {
            structure := vcRegion.PrefGet (region, "structure");
            if (isprocedure(structure))
                picklist_structure := structure(userdata);
            else
                picklist_structure := structure;
            edit_region = vcRegion.Create ("picklist", vcPicklist.fac,
                                           picklist_structure);
            foreach (row, col, index, is_set, visual, data;
                     vcPicklist.Items (edit_region))
            {
                if (string_eq(car (value_label), data)) { 
                    vcPicklist.SetPick (edit_region, index);
                    break;
                } /* if */
            } /* foreach */
        } /* if */
        else
            edit_region = vcRegion.Create1 ("edit_region", edit_fac,
                                            car (value_label), edit_prefs);
        function reset_value()
            _Set (value_label, vcRegion.Get (edit_region, vscmVT));

        buttons := vcRegion.Create ("OkOrCancel:butnRow", vcButnRow.fac,
                                list (list ("Accept", "Accept", reset_value,
                                            list (close_window <- TRUE)),
                                      list ("Cancel", "Cancel", func();,
                                            list (close_window <- TRUE))));

        if (do_scroll && (edit_fac == vcPicklist.fac))
            scroll := vcPicklist.Scroll ("", edit_region,
                                         vcRegion.PrefGet (region,
                                                           "scroll_min"),
                                         vcRegion.PrefGet (region,
                                                           "scroll_ideal"),
                                         vcRegion.PrefGet (region,
                                                           "scroll_max"));
        else
            scroll := edit_region;

        win := vcRegion.Window (region);
        table := vcTable.Make (vcRegion.Name (region), 2, 1, scroll, buttons);
        margin := vcRegion.Create (name, vcMargin.fac, table);
        popup := vcWin.Popup (win, margin, edit_region,
                              vcWin.NoStatus + vcWin.NoBindings);
        vcWin.Forward (win, popup);
    } /* function popup_picklist */

    vcRegion.Font (label, vcRegion.PrefGet (region, "font"));
    label_and_arrow := vcTable.Make ("la", 1, 2, label, arrow);
    name := vcRegion.Name (region);
    border := vcRegion.Create (name, vcBorder.fac, label_and_arrow);
    button := vcRegion.Create (name, vcButton.fac,
                               func(ev) popup_picklist (ev););
    vcRegion.FocusInhibit (button);
    vcRegion.AddChild (button, border);
    vcButton.HighlightRegion (button, border);
    vcRegion.AddChild (region, button);
    value_label;
} /* function _Create */

function _Get(value_label, region, type) {
    vcRegion.Get (cdr (value_label), type);
} /* function _Get */

function _Set(value_label, new_val) {
    set_car (value_label, new_val);
    vcRegion.Set (cdr (value_label), new_val);
} /* function _Set */

vlPref.Replace (prefs, "initial", vscmVT,
                "Initial label string, or function to return string", "");
vlPref.Replace (prefs, "structure", vscmVT,
                "Picklist structure (see vcPicklist. man page)", nil);
vlPref.Replace (prefs, "scroll", vlTrueFalseVT,
                "Show a scrollbar for the picklist", TRUE);
vlPref.Replace (prefs, "scroll_min", vlUnsignedIntVT,
                "Minimum number of lines for scrolled region", 4);
vlPref.Replace (prefs, "scroll_ideal", vlUnsignedIntVT,
                "Initial number of lines for scrolled region", 6);
vlPref.Replace (prefs, "scroll_max", vlUnsignedIntVT,
                "Maximum number of lines for scrolled region", 0);
vlPref.Replace (prefs, "font", vgFontVT,
                "Font of text line", vgFont.small);
vlPref.Replace (prefs, "edit_fac", vcFacilityVT,
                "Facility used to edit data in popup", vcPicklist.fac);
vlPref.Replace (prefs, "edit_prefs", vscmVT,
                "Preferences used to edit data in popup", nil);
vlPref.Replace (prefs, "label_fac", vcFacilityVT,
                "Facility used to show data in line", vcStaticText.fac);
vlPref.Replace (prefs, "label_prefs", vscmVT,
                "Preferences for facility used to show data in line",
                nil);
vlPref.Replace (prefs, "width", vlUnsignedIntVT,
                "width of status line in characters, 0 to set dynamically",
                0);

