/*
 * Name:      vdaddini.vs
 * Facility:  Viewdraw
 * 
 * Description: 
 *    Viewscript file for Viewdraw facilitates dynamic search order
 *    changes.
 *
 * Environment:
 *     SunOS 4.1 & CORE 5.0
 *     
 * Author:  
 *   Eric Fontana,  8-Nov-94 09:40:55 (vdaddini.vs)
 *
 * Modified by: 
 * Who            Version   When      What
 *
 * EJF            5.3.1    8-Nov-94   Original coding
 * 
 * (c) Copyright 1991, 1992 by Viewlogic Systems, Inc.
 */

vcWv.RequireFac (wv, "vdle");

function GetLibOrder (d) {
    i := 0;

    lib_list := nil;
    foreach(type, alias, path; IniIterLibInfo ()) {
        if (++i > 1) {
            if (!null (alias))
                str := sprintf("[%s] %s (%s)", type, path, alias);
            else
                str := sprintf("[%s] %s", type, path);

            lib_list = cons (str, lib_list);
        }
    }
    
    reverse_destructive (lib_list);
}

function order_libs(d, e, worklist) {

    function convert_type(type) {
        declare str;
        
        if (type == vdraw.vqLibType.Writable)
            str = "w";
        else if (type == vdraw.vqLibType.ReadOnly)
            str = "r";
        else if (type == vdraw.vqLibType.MegaFile)
            str = "rm";
        else if (type == vdraw.vqLibType.Primary)
            str = "pw";
    }

    foreach(item; vscript.list_iter (worklist)) {

        type := car (item);
        todo := cdr (item);

        if (string_eq (type, "Move")) {

            catch (vlEx.FAILURE)
                vdraw.IniMoveLib (car (todo), cadr (todo));

            exception
                vlEx.Throw2 (vlEx.FAILURE, "vdraw", vlErrSev.Error,
             sprintf("Unable to move library from position %d to position %d",
                     car (todo), cadr (todo)));

        } else if (string_eq (type, "Add")) {

            index := car (todo);
            info := cadr (todo);
            libtype := list_ref (info, 0);
            path := list_ref (info, 1);
            alias := list_ref (info, 2);

            catch (vlEx.FAILURE)
                vdraw.IniAddLib (path, alias, libtype, index);

            exception
                vlEx.Throw2 (vlEx.FAILURE, "vdraw", vlErrSev.Error,
             sprintf("Unable to add library at position %d \n`[%s] %s %s'",
                     index, convert_type (libtype), path,
                     string_eq (alias, "") ? "" : sprintf("(%s)", alias)));

        } else if (string_eq (type, "Delete")) {

            if (!vdraw.IniDeleteLib (car (todo)))

                vlEx.Throw2 (vlEx.FAILURE, "vdraw", vlErrSev.Error,
                     sprintf("Unable to remove library %s at position %d", 
                             car (todo), index));
        }
    }

    IniSave ();

    if (!null (fopen_window)) {

        picklist := build_pathpicklist (getprimarylib ());

        function set_line(str, on_off) {

            if (on_off == vlOnOff.On) {
                vcCmdline.Set (fopen_cmdline, str);
                fopen_name = str;
            }
        }
        
        fopen_lastlib = getprimarylib ();
        build_and_set_files (fopen_window, fopen_table, 0, fopen_datatype,
                             fopen_lastlib, "*", set_line);
        
        vcPicklist.Notify (picklist,
                           func(lib, on_off) {
                               if (on_off == vlOnOff.On) {
                                   fopen_lastlib = lib;
                                   build_and_set_files (fopen_window,
                                                        fopen_table, 0,
                                                        fopen_datatype,
                                                        fopen_lastlib, "*",
                                                        set_line);
                               }
                           });
        
        fopen_scrollp = vcPicklist.Scroll ("", picklist, 4, 6, 8);
        vcTable.Set (fopen_table, 3, 0, fopen_scrollp);
    }
}

lib_order_cmd := vcWv.Cmd (vcWin.fac, order_libs, "orderlibs",
                           "Change library search order",
                           vcWv.Param ("Search Order", vscmVT, vcWv.Dynamic,
                                       GetLibOrder, vdle.fac));
