/* 
 * $RCSfile: vdbChecklist.vs,v $ $Revision: 1.1 $
 *
 * The vdbChecklist facility -- methods + commands
 *
 * vdbChecklist implements a scrolled list of toggle buttons
 */


struct vdbChecklistVT {
    vcRegionVT rgnTable
};


/**********************************************************************/
/* _Create -- create an instance of this facility
 *
 * DESCRIPTION
 *      `lCheckedOptions' is a list of options to start checked (set);
 *      the preference `data' contains the list of all options.
 *
 *      `rgnMyRegion' is our very own Region ID, created by Core.
 *
 * RETURNS
 *      A value that will be passed by Core to all our methods and commands.
 */
function _Create(lCheckedOptions, rgnThis)
{
    lOptions := vcRegion.PrefGet (rgnThis, "data");
    rgnTable := vcTable.Create ("checklist table", length (lOptions), 1);
    foreach (l; vscript.list_iter (lOptions))
        vcTable.Add (rgnTable,
                     vcRegion.Create (l, vcToggleButton.fac,
                                      (assoc (l, lCheckedOptions) != FALSE),
                                      label <- l));
    rgnScroll :=
        vcTable.Scroll3 ("checklist scroll", rgnTable, rgnTable,
                         1, 7, 0,
                         0, 0, 0,
                         list (vertical <- vcScroll.Presence.Always,
                               horizontal <- vcScroll.Presence.Never));

    vcRegion.AddChild (rgnThis, rgnScroll);

    /*
     * Facility data object is return value.
     */
    this := struct vdbChecklistVT {rgnTable};

} /* _Create */


function _Get(this, rgnThis, desired_type)
{
    declare nrows, ncols, lSet;
    vcTable.Dimensions (this.rgnTable, &nrows, &ncols);

    /* for each set toggle button in the table, put an element
       in our alist that looks like `("name" . TRUE)' */
    for (i := 0; i < nrows; ++i)
        if (vcRegion.Get (rgn := vcTable.Get (this.rgnTable, i, 0), boolVT))
            lSet = cons (cons (vcRegion.Name (rgn), TRUE), lSet);

    lSet;

} /* _Get */


/*
 * Preferences.
 */
vlPref.Replace (prefs, "data", vscmVT,
    "ViewScript list of options present in the checklist", nil);
