/*
 * $RCSfile: vdbVdraw.vs,v $ $Revision: 1.22 $
 *
 * Stuff used to communicate to viewdraw -- common to `vdb2' and
 * `vdbResults'.
 */

cntxSrc := vitc.SrcCntxCreate (vitcTOOLID_VDB, nil);
cntxVdraw := GenericCntxCreate (tid <- vitc.TidVDRAW,
                                name <- sVdrawNameField);
cntxAnyVdb := GenericCntxCreate (tid <- vitcTOOLID_VDB);

function RegisterOne(this, sThisMsgType, cntxSrc, cntxMatch, fnAction)
{
    fnWrapper := func(sMsgType, cntxSender, cntxMatch, sData)
    {
        DEBUG ("vdb: received %s: \"%s\"\n", sMsgType, sData);
        fnAction (this, sMsgType, cntxSender, cntxMatch, sData);
    };

    /*DEBUG("Registering to receive mtype: \"%s\"\nsrc_cntx:   %s\nmatch_cntx: %s\n", sThisMsgType, cntxSrc, cntxMatch);*/
    this.lItcRegistrations =
        cons (vitc.Register (sThisMsgType, cntxSrc, cntxMatch, fnWrapper),
              this.lItcRegistrations);
}

function ItcUnregisterAll(this)
    foreach (e; vscript.list_iter(this.lItcRegistrations))
        vitc.Unregister (e);

function RegisterFromVdraw(this, sThisMsgType, fn)
    RegisterOne (this, sThisMsgType, cntxSrc, cntxVdraw, fn);

/*
 * ReceiveSelectedComponents -- called when ITC message SEL_COMP
 * received from ViewDraw.  Message contains filename to load.
 */
function HandleSelComp(this, sMsgType, cntxSender, cntxMatch, sFilename)
{
    lGroups := load (sFilename);
    vscript.vsDelete (sFilename, vsAttr.File); /* don't need any more */

    if (null (lGroups))
        vdbThrow (1012, "There are no non-excluded components to load");

    /* OK, display the groups! */
    DisplayGroups (this, lGroups);

}

function HandleGetReady(this, sMsgType, cntxSender, cntxMatch, sData)
{
    /* seems like a good idea to reread cfg file here (VDB1 did it) */
    vdbCfgReadConfigFile ();
    sMsg := vdbCfgReadyMsg ();
    SendToVdraw (sMsgREADY, sMsg);

}

function _ResetConfig()
{
    sMsg := vdbCfgReadyMsg ();
    SendToVdraw (sMsgREADY, sMsg);

}

function HandleBackAnnoDone(this, sMsgType, cntxSender, cntxMatch, sData)
{
    vcWin.Busy (vcRegion.Window (this.rgnThis), FALSE);
    /* HACK_ALERT: should clear window now, or likewise indicate
     * our state of completion */

}


/* --------------------------------------------------------------
 * reportMultiProcess
 *   Bring up a dialog box when more than one Viewdraw or
 *   ViewDatabook is running.  (The broadcast ITC scheme that 
 *   we're using works only when single processes are running.
 *   Its OK to have two Viewdraw running, you just can't use
 *   ViewDatabook at the same time.)
 */
function reportMultiProcess(prog1, prog2, severity)
{
    msg := "There are multiple " + prog1 + " programs running.\n";

    if (severity != vlErrSev.Error)
        msg = msg + "\nThis is not an error.  However:\n\n";

    msg = msg + prog2 + " can only communicate with one at a time.\n" +
        "Do not try to use ViewDatabook until you have exited all\n" +
        "but one of the " + prog1 + " processes.";

    vdbAlert(nil, MainWindowId, msg, severity, nil);
} /* reportMultiProcess */


declare lVdrawPidList;

/*
 * DESCRIPTION
 *      ViewDraw started; add its PID to the list of active ViewDraw
 *      processes.  List should always be of length zero or one, or else
 *      something's wrong and we'll issue a warning.
 */
function HandleToolStarted(this, sMsgType, cntxSender, cntxMatch, sData)
{
    pid := vipc.CntxGetPid (cntxSender);
    if (memv (lVdrawPidList, pid) == FALSE)
        lVdrawPidList = cons (pid, lVdrawPidList);

    if (length (lVdrawPidList) > 1)
        reportMultiProcess ("Viewdraw", "ViewDatabook", vlErrSev.Warning);

}


/*
 * DESCRIPTION
 *      ViewDraw stopped; delete its PID from the list of active ViewDraw
 *      processes.
 */
function HandleToolStopped(this, sMsgType, cntxSender, cntxMatch, sData)
    lVdrawPidList = vscript.delete_list_element (lVdrawPidList,
                                                 vipc.CntxGetPid (cntxSender));


function HandlePing(this, sMsgType, cntxSender, cntxMatch, sData)
    SendToVdraw (sMsgTOOL_STARTED, "");


function ItcRegisterAll(this)
{
    RegisterFromVdraw (this, sMsgSEL_COMP, HandleSelComp);
    RegisterFromVdraw (this, sMsgGET_READY, HandleGetReady);
    RegisterFromVdraw (this, sMsgBACK_ANNO_DONE, HandleBackAnnoDone);
    RegisterFromVdraw (this, sMsgTOOL_STARTED, HandleToolStarted);
    RegisterFromVdraw (this, sMsgTOOL_STOPPED, HandleToolStopped);
    RegisterFromVdraw (this, sMsgPING, HandlePing);

    /*DEBUG ("vdb: Sending ToolStarted: %s\n", cntxSrc);*/
    vitc.ToolStartNotify (cntxSrc);

}

function SendToVdraw(sMsgType, sData)
{
    /*DEBUG ("vdb: Sending msg %s: \"%s\"\n", sMsgType, sData);*/
    if (vitc.Send (sMsgType, cntxSrc, cntxVdraw,
                   sData, vipc.FlgMULTI) != vitc.Success)
    {
        msg := sprintf("Error sending message to ViewDraw.  Message was \"%s\"", sData);
        vdbAlert (nil, nil, msg, vlErrSev.Error, nil);
    }

} /* SendToVdraw */


/*
 * DESCRIPTION
 *      Method to cause load of values from ViewDraw.
 */
function _Load()
    SendToVdraw (sMsgREQ_SEL_COMP, "");


/*
 * DESCRIPTION
 *      Method to add component into ViewDraw.
 *
 *      `lAttrs' is a list of the form:
 *
 *          ((attr_name value visibility oat_flag) ...)
 *
 *      Views that should not be annotated as attributes will be trimmed
 *      from this list.
 */
function _Add(sSymbol, lAttrs)
{
/*
    if (null (lAttrs))
        vdbThrow ("Can't add -- component has no attributes");
*/

    /* don't think this can happen */
    if (null (sSymbol))
        vdbThrow (1006, "Can't add component. Select unique %s first.", 
                  EncapGetVtype());

    /* Handle link syntax */
    if (sSymbol[0] == '@')
    {
        <sLocation, sClass, sObj> := vdb_cb.ParseLink (sSymbol);
        if (null(sObj))
            vdbThrow (1015, "Invalid link syntax: %s", sSymbol);
        sSymbol = sObj;
    }

/*
    if (vdbCfg.bAddWithLibName)
        sSymbol = ...
*/

    sMsg := sAddCompCmdName + " "
        + sSymbol + " "
            + convert (vlStringVT, lAttrs);

    SendToVdraw (sMsgADD_COMP, sMsg);

} /* _Add */


/*
 * DESCRIPTION
 *      Method to annotate existing components in ViewDraw.
 *
 *      `lComps' is the list of component UIDs to annotate.
 */
function _Annotate(lCompsToAnnotate, lAttrs)
{
    if (null (lCompsToAnnotate))
        vdbThrow (1016, "No components to annotate");

/* Can't happen:
    if (null (lAttrs))
        vdbThrow ("No views to annotate"); 
*/

    sMsg := "\nvdraw.FinderStoreAttrs("
        + convert(vlStringVT, lCompsToAnnotate) + ","
            + convert(vlStringVT, lAttrs) + ");";

    SendToVdraw (sMsgBACK_ANNO, sMsg);

} /* _Annotate */


/*
 * DESCRIPTION
 *      Method to track components in ViewDraw.
 *
 *      `lComps' is the list of components (UIDs) to select and maybe zoom.
 */
function _Track(lComps)
{
    if (vdbCfg.eTracking != Tracking.No)
    {
        SendToVdraw (sMsgTRACK_COMPS,
                     convert (vlStringVT, lComps) + ";");
        if (vdbCfg.eTracking == Tracking.Zoom)
            SendToVdraw (sMsgCMD, "zsel 2");
    }

}
