/*
 * $RCSfile: vdbopt.vs,v $ $Revision: 1.27 $
 *
 * ViewDatabook II -- processing user configuration options.
 *
 * Copyright (c) 1995 by Viewlogic Systems, Inc.
 */

/*
 * Enum used to determine highlighting style in ViewDraw.
 */
define_enum ("TrackingVT",
             list ("Yes", "No", "Zoom"));

/*
 * Enum used for testing values.  I expect more in the future.
 */
define_enum ("vdbTestValuesVT",
             list ("Yes", "No"));


/*
 * Struct which describes user configuration options.
 */
vdbCfg := struct
{
    /*
     * Fields that are commented with a "VD:" have to be known on the
     * ViewDraw end.  They get passed to ViewDraw in the READY message.  See
     * also vdbCfgReadyMsg(), and struct vdbCfg in "vdr_vdb.vs".
     */

    /* stuff used only by VDB */
    symCategoryToDisplayedAttrs; /* maps category -> list of displayed attrs */
    symExcludedAttrs;           /* maps attr name -> TRUE if excluded */
    symVtypeToAttr;             /* maps vtype -> attr name */
    symAttrToCatVtype;          /* maps attr name -> (category . vtype) */
    symAttrNameToVis;           /* maps attr name -> visibility */
    symVtypeToWildcarding;      /* maps vtype -> enum specifying whether to
                                   add leading/trailing wildcard '*' */
    alRequiredViews;            /* alist of required views / values */
    eDefaultVis;                /* if not found in hash, use this value */
    bAddWithLibName;            /* use alias:symbol when adding? */
    lExcludeAttrsWhenAnnotating;/* list of attrs not annotated */
    lIncludeAttrsWhenAnnotating;/* list of attrs not shown but annotated */
    eTestValues;                /* check values before annotating */
    bDisplayTips;               /* display button tips? */
    bTextColorEnabled;          /* display text in different colors? */
    eTextColorCommonView;       /* color for common views */
    eTextColorLoadedView;       /* color for loaded views */
    sUndefValue;                /* undefined value */
    bAnnotateUndef;             /* annotate undefined value? */
    sLibNameAttr;               /* name of LibName attribute, nil if unset */

    /* hooks for extensions */
    lCallOnAdd;
    lCallOnLoad;
    lCallOnAnnotate;

    /* library organization */
    eOrg = vdbOrganized.ByLibrary; /* ByLibrary or ByNamedView */
    sOrgViewName;               /* Vtype by which categories are organized */

    /* oats stuff & stuff used by VD */
    eTracking;                /* Yes/No/Zoom */
    lExcludeSymsWhenLoading;  /* VD: list of symbols excluded when selecting */
    lExcludeAttrsWhenLoading; /* VD: list of attrs excluded when loading */
    bMaintainAttrVis;         /* VD: maintain attr vis from placeholder? */
    bSymAttrsGetLoaded;       /* VD: look for sym attrs when loading? */
    bOatsEnabled;             /* VD: GUI shows oats buttons? */
    bOatsGetLoaded;           /* VD: look for oats when loading? */
    bOatsOnByDefault          /* oats buttons on by default? */
};


/*
 * These are in the same order as vdraw.vqAttrVisibilityVT so that we can
 * convert back and forth if needed.  This type is needed because we use it
 * inside & outside vdraw.
 */
/*
define_enum ("AttrVisVT",
             list ("Invisible","Visible","Name","Value"));
*/


function vdbCfgReset()
{
    vdbCfg.sLibNameAttr = nil;
    vdbCfg.sUndefValue = nil;
    vdbCfg.bAnnotateUndef = FALSE;
    vdbCfg.eTracking = Tracking.Yes;
    vdbCfg.lExcludeSymsWhenLoading     = nil;
    vdbCfg.lExcludeAttrsWhenLoading    = nil;
    vdbCfg.lExcludeAttrsWhenAnnotating = nil;
    vdbCfg.lIncludeAttrsWhenAnnotating = nil;
    vdbCfg.alRequiredViews             = nil;
    vdbCfg.symCategoryToDisplayedAttrs = vlSym.StriCreate (37);
    vlSym.Dynamic (vdbCfg.symCategoryToDisplayedAttrs, 2.0, 2.0);
    vdbCfg.symExcludedAttrs      = vlSym.StrCreate (11);
    vlSym.Dynamic (vdbCfg.symExcludedAttrs, 2.0, 2.0);
    vdbCfg.symVtypeToAttr        = vlSym.StrCreate (11);
    vlSym.Dynamic (vdbCfg.symVtypeToAttr, 2.0, 2.0);
    vdbCfg.symAttrToCatVtype     = vlSym.StrCreate (11);
    vlSym.Dynamic (vdbCfg.symAttrToCatVtype, 2.0, 2.0);
    vdbCfg.symAttrNameToVis      = vlSym.StriCreate (37);
    vlSym.Dynamic (vdbCfg.symAttrNameToVis, 2.0, 2.0);
    vdbCfg.symVtypeToWildcarding  = vlSym.StriCreate (11);
    vlSym.Dynamic (vdbCfg.symVtypeToWildcarding, 2.0, 2.0);
    /*
     * Default is to add leading & trailing wildcard to SYMBOL vtype --
     * schematic knows about only "00.1", and sometimes "00.2", but vllm
     * knows either "@vl74ls:Symbol:00.1" or "00" depending on how the
     * view was added.
     */
    vlSym.Store (vdbCfg.symVtypeToWildcarding, EncapGetVtype(),
                 Wildcarding.Leading + Wildcarding.Trailing);

    vdbCfg.eDefaultVis           = "Visible";
    vdbCfg.bAddWithLibName       = FALSE;
    vdbCfg.bDisplayTips          = FALSE;
    vdbCfg.eTestValues           = vdbTestValues.Yes;

    vdbCfg.bTextColorEnabled     = TRUE;
    vdbCfg.eTextColorCommonView  = vgAllColor.Blue;
    vdbCfg.eTextColorLoadedView  = vgAllColor.Red;

    vdbCfg.bMaintainAttrVis      = FALSE;
    vdbCfg.bSymAttrsGetLoaded    = FALSE;
    vdbCfg.bOatsEnabled          = FALSE;
    vdbCfg.bOatsGetLoaded        = FALSE;
    vdbCfg.bOatsOnByDefault      = FALSE;

    vdbCfg.lCallOnAdd = nil;
    vdbCfg.lCallOnLoad = nil;
    vdbCfg.lCallOnAnnotate = nil;

    /* library organization */
    vdbCfg.eOrg = vdbOrganized.ByLibrary;
    vdbCfg.sOrgViewName = nil;

}


/*
 * DESCRIPTION
 *      Used only to send READY message to VDRAW.  String returned is
 *      ViewScript expression to evaluate in the VDRAW process.
 */
function vdbCfgReadyMsg()
{
    function MsgAdd(sMsg, symbol, value)
        sMsg + symbol + " = " + convert (vlStringVT, value) + ";\n";

    sMsg := "";
    sMsg = MsgAdd (sMsg, "vdbCfg.lExcludeSymsWhenLoading",
                   vdbCfg.lExcludeSymsWhenLoading);
    sMsg = MsgAdd (sMsg, "vdbCfg.lExcludeAttrsWhenLoading",
                   vdbCfg.lExcludeAttrsWhenLoading);
    sMsg = MsgAdd (sMsg, "vdbCfg.bSymAttrsGetLoaded", vdbCfg.bSymAttrsGetLoaded);
    sMsg = MsgAdd (sMsg, "vdbCfg.bOatsEnabled", vdbCfg.bOatsEnabled);
    sMsg = MsgAdd (sMsg, "vdbCfg.bOatsGetLoaded", vdbCfg.bOatsGetLoaded);
    sMsg = MsgAdd (sMsg, "vdbCfg.bMaintainAttrVis", vdbCfg.bMaintainAttrVis);

}


/*\BeginDebugVS
function print_hash(sName, sym)
{
    printf ("%-30s  %s\n", sName + " (hash)",
            (vlSym.Size (sym) == 0) ? "*empty*" : "");
    foreach (sSymbol, sValue; vlSym.Loop (sym))
        printf ("%30s: %s\n", sSymbol, sValue);
}
\EndDebugVS*/

/*\BeginDebugVS
function vdbCfgPrint()
{
    printf ("\n--- ViewDatabook II Configuration ---\n\n");
    print_hash ("symCategoryToDisplayedAttrs", vdbCfg.symCategoryToDisplayedAttrs);
    print_hash ("symExcludedAttrs", vdbCfg.symExcludedAttrs);
    print_hash ("symVtypeToAttr", vdbCfg.symVtypeToAttr);
    print_hash ("symAttrToCatVtype", vdbCfg.symAttrToCatVtype);
    print_hash ("symAttrNameToVis", vdbCfg.symAttrNameToVis);
    print_hash ("symVtypeToWildcarding", vdbCfg.symVtypeToWildcarding);
    printf ("%-30s  %s\n", "alRequiredViews", vdbCfg.alRequiredViews);
    printf ("%-30s  %s\n", "eDefaultVis", vdbCfg.eDefaultVis);
    printf ("%-30s  %s\n", "bAddWithLibName", vdbCfg.bAddWithLibName);
    printf ("%-30s  %s\n", "eOrg", vdbCfg.eOrg);
    printf ("%-30s  %s\n", "sOrgViewName", vdbCfg.sOrgViewName);
    printf ("%-30s  %s\n", "lExcludeSymsWhenLoading", vdbCfg.lExcludeSymsWhenLoading);
    printf ("%-30s  %s\n", "lExcludeAttrsWhenLoading", vdbCfg.lExcludeAttrsWhenLoading);
    printf ("%-30s  %s\n", "lExcludeAttrsWhenAnnotating", vdbCfg.lExcludeAttrsWhenAnnotating);
    printf ("%-30s  %s\n", "lIncludeAttrsWhenAnnotating", vdbCfg.lIncludeAttrsWhenAnnotating);
    printf ("%-30s  %s\n", "bMaintainAttrVis", vdbCfg.bMaintainAttrVis);
    printf ("%-30s  %s\n", "bSymAttrsGetLoaded", vdbCfg.bSymAttrsGetLoaded);
    printf ("%-30s  %s\n", "bOatsEnabled", vdbCfg.bOatsEnabled);
    printf ("%-30s  %s\n", "bOatsGetLoaded", vdbCfg.bOatsGetLoaded);
    printf ("%-30s  %s\n", "bOatsOnByDefault", vdbCfg.bOatsOnByDefault);
    printf ("%-30s  %s\n", "bTextColorEnabled", vdbCfg.bTextColorEnabled);
    printf ("%-30s  %s\n", "eTracking", vdbCfg.eTracking);
    printf ("%-30s  %s\n", "sUndefValue", vdbCfg.sUndefValue);
    printf ("%-30s  %s\n", "bAnnotateUndef", vdbCfg.bAnnotateUndef);
    printf ("%-30s  %s\n", "sLibNameAttr", vdbCfg.sLibNameAttr);

    printf ("\n-------------------------------------\n\n");

}
\EndDebugVS*/


function vdbCfgSetAttrVisibility(vis, l)
{
    if (null (l))
        vdbCfg.eDefaultVis = vis;
    else
        foreach (e; vscript.list_iter (l))
            vlSym.Store (vdbCfg.symAttrNameToVis, e, vis);

}


function vdbCfgGetAttrVisibility(sAttrName)
{
    ret := vlSym.Fetch (vdbCfg.symAttrNameToVis, sAttrName);
    if (null (ret))
        ret = vdbCfg.eDefaultVis;

    ret;

}


/*
 * DESCRIPTION
 *      This is the main option processing function that gets called from
 *      within "vdbum.scm".
 */
function SetOption(option, args...)
{
    if (option == `ExcludeSymsWhenLoading)
        vdbCfg.lExcludeSymsWhenLoading = car (args);
    else if (option == `ExcludeAttrsWhenLoading)
        vdbCfg.lExcludeAttrsWhenLoading = car (args);
    else if (option == `ExcludeAttrsWhenAnnotating)
        vdbCfg.lExcludeAttrsWhenAnnotating = car (args);
    else if (option == `IncludeAttrsWhenAnnotating)
        vdbCfg.lIncludeAttrsWhenAnnotating = car (args);
    else if (option == `OrganizeByAttr)
    {
        vdbCfg.eOrg = vdbOrganized.ByNamedView;
        vdbCfg.sOrgViewName = car (args);
    }
    else if (option == `CategoryDisplayAttrsDefault)
        vlSym.Store (vdbCfg.symCategoryToDisplayedAttrs, nil, car (args));
    else if (option == `CategoryDisplayAttrs)
        vlSym.Store (vdbCfg.symCategoryToDisplayedAttrs,
                     car (args), cadr (args));
    else if (option == `ExcludeAttrsFromDisplay)
        foreach (e; vscript.list_iter (car (args)))
            vlSym.Store (vdbCfg.symExcludedAttrs, e, TRUE);
    else if (option == `RequiredAttrs)
    {
        /* Required view specification may be either TRUE or a value.
         * Modify list in place, replacing TRUE with "*".
         */
        vdbCfg.alRequiredViews = car (args);
        foreach (e; vscript.list_iter (vdbCfg.alRequiredViews))
            if (cdr (e) == TRUE)
                set_cdr (e, "*");
    }
    else if (option == `MapAttrs)
        /*
         * Difficult thing here.  There could be a many-to-one relationship
         * vtypes -> attr name.  E.g.
         *
         *     '(("C_VALUE" "VALUE" "Capacitors")
         *       ("R_VALUE" "VALUE" "Resistors")))
         *
         * The way we'll do it is hash attr name to an alist of
         * (category . vtype) pairs.  The category will be nil if it wasn't
         * specified.  Now the only thing hard about converting attr name to
         * vtype is deducing the category.
         */
        foreach (e; vscript.list_iter (car (args)))
        {
            if (length (e) == 2 || length (e) == 3)
            {
                sVtype    := car (e);
                sAttr     := cadr (e);
                sCategory := caddr (e); /* will be nil if length == 2 */
                /* store directly in Vtype -> Attr hash */
                vlSym.Store (vdbCfg.symVtypeToAttr, sVtype, sAttr);
                /* cons into Attr -> Vtype hash */
                old := vlSym.Fetch (vdbCfg.symAttrToCatVtype, sAttr);
                vlSym.Store (vdbCfg.symAttrToCatVtype, sAttr,
                             cons (cons (sCategory, sVtype), old));
            }
            else
                vdbAlert (nil, nil, sprintf ("Invalid specification for option 'MapAttrs -- `%s'", car (args)), vlErrSev.Error, nil);
        }
    else if (option == `AttrVisVisible)
        vdbCfgSetAttrVisibility ("Visible", car (args));
    else if (option == `AttrVisInvisible)
        vdbCfgSetAttrVisibility ("Invisible", car (args));
    else if (option == `AttrVisValue)
        vdbCfgSetAttrVisibility ("Value", car (args));
    else if (option == `AttrVisName)
        vdbCfgSetAttrVisibility ("Name", car (args));
    else if (option == `MaintainAttrVis)
        vdbCfg.bMaintainAttrVis = car (args);
    else if (option == `TestValues)
        vdbCfg.eTestValues = convert (vdbTestValuesVT, car (args));
    else if (option == `SymAttrsGetLoaded)
        vdbCfg.bSymAttrsGetLoaded = car (args);
    else if (option == `OatsEnabled)
    {
        vdbCfg.bOatsEnabled = car (args);
        vdbCfg.bOatsGetLoaded = car (args);
        vdbCfg.bOatsOnByDefault = car (args);
    }
    else if (option == `OatsGetLoaded)
        vdbCfg.bOatsGetLoaded = car (args);
    else if (option == `OatsOnByDefault)
        vdbCfg.bOatsOnByDefault = car (args);
    else if (option == `AddWithLibName)
        vdbCfg.bAddWithLibName = car (args);
    else if (option == `CallOnAdd)
    {
        if (isprocedure (car (args)))
            vdbCfg.lCallOnAdd = cons (car (args), vdbCfg.lCallOnAdd);
        else
            vdbAlert (nil, nil, sprintf ("Expected ViewScript procedure for option 'CallOnAdd, received `%s'", car (args)), vlErrSev.Error, nil);
    }
    else if (option == `CallOnLoad)
    {
        if (isprocedure (car (args)))
            vdbCfg.lCallOnLoad = cons (car (args), vdbCfg.lCallOnLoad);
        else
            vdbAlert (nil, nil, sprintf ("Expected ViewScript procedure for option 'CallOnLoad, received `%s'", car (args)), vlErrSev.Error, nil);
    }
    else if (option == `CallOnAnnotate)
    {
        if (isprocedure (car (args)))
            vdbCfg.lCallOnAnnotate = cons (car (args), vdbCfg.lCallOnAnnotate);
        else
            vdbAlert (nil, nil, sprintf ("Expected ViewScript procedure for option 'CallOnAnnotate, received `%s'", car (args)), vlErrSev.Error, nil);
    }
    else if (option == `TextColorsEnabled)
        vdbCfg.bTextColorEnabled = car (args);
    else if (option == `CommonValueColor)
        vdbCfg.eTextColorCommonView = car (args);
    else if (option == `LoadedValueColor)
        vdbCfg.eTextColorLoadedView = car (args);
    else if (option == `VtypeAddTrailingWildcard)
        foreach (sVtype; vscript.list_iter (car (args)))
        {
            e := vlSym.Fetch (vdbCfg.symVtypeToWildcarding, sVtype);
            if (null (e))
                e = Wildcarding.Trailing;
            else
                e = e | Wildcarding.Trailing;
            vlSym.Store (vdbCfg.symVtypeToWildcarding, sVtype, e);
        }
    else if (option == `VtypeAddLeadingWildcard)
        foreach (sVtype; vscript.list_iter (car (args)))
        {
            e := vlSym.Fetch (vdbCfg.symVtypeToWildcarding, sVtype);
            if (null (e))
                e = Wildcarding.Leading;
            else
                e = e | Wildcarding.Leading;
            vlSym.Store (vdbCfg.symVtypeToWildcarding, sVtype, e);
        }
    else if (option == `ComponentTracking)
        vdbCfg.eTracking = convert (TrackingVT, car (args));
    else if (option == `UndefinedValue)
        vdbCfg.sUndefValue = car (args);
    else if (option == `AnnotateUndefined)
        vdbCfg.bAnnotateUndef = car (args);
    else if (option == `LibNameAttr)
        vdbCfg.sLibNameAttr = car (args);
    else
        vdbAlert (nil, nil,
                  sprintf ("Unrecognized configuration option -- %s %s",
                           option, args),
                  vlErrSev.Warning, nil);

} /* SetOption */


declare sConfigFile;

function vdbCfgGetFilename()
{
    if (null (sConfigFile))
        sConfigFile = vscript.vsGetini ("vdbum.scm");
    sConfigFile;
}

function vdbCfgReadConfigFile ()
{
    vdbCfgReset ();
    vcsa.LoadCustomizations ("vdb", current_package ());
/*\BeginDebugVS
    vdbCfgPrint ();
\EndDebugVS*/

} /* vdbCfgReadConfigFile */
