;;;;
;;;; $RCSfile: vdbum.scm,v $ $Revision: 1.20 $
;;;;
;;;; Configuration file for ViewDatabook II
;;;;


;;; LIBRARY DATA ORGANIZATION ------------------------------------------------


;;
;; NAME
;;      OrganizeByAttr -- library data organization
;;
;; DESCRIPTION
;;      If set, it is the name of the attribute by which the library data is
;;      organized.  All components with the same attribute value are grouped
;;      together for the purposes of searching and back-annotation.  If set,
;;      you must also set the CategoryDisplayAttrsDefault option.
;;
;;      If unset, library data is organized by library.
;;
;; SEE ALSO
;;      Chapter 3, User's Guide
;;
;; DEFAULT VALUE
;;      unset
;;
;; EXAMPLE
;(SetOption 'OrganizeByAttr "CATEGORY")


;;
;; NAME
;;      MapAttrs -- map attribute names
;;
;; DESCRIPTION
;;      Map attribute names as they appear in the databases (e.g. PAR files)
;;      attribute names as you want them to appear in ViewDatabook II and
;;      in ViewDraw.
;;
;; DEFAULT VALUE
;;      none
;;
;; EXAMPLE
;(SetOption 'MapAttrs
;            ;; database_attr  attr
;            '(("TOLERANCE"    "TOL")))


;;; ATTRIBUTE PROCESSING -----------------------------------------------------


;;
;; NAME
;;      ExcludeSymsWhenLoading -- exclude named symbols from processing
;;
;; DESCRIPTION
;;      When loading an entire ViewDraw sheet, ignore these symbols.
;;
;; DEFAULT VALUE
;;      nil
;;
;; EXAMPLE
;(SetOption 'ExcludeSymsWhenLoading '("gnd"))


;;
;; NAME
;;      ExcludeAttrsWhenLoading -- exclude attributes from processing
;;
;; DESCRIPTION
;;      Don't load these attributes.
;;
;; DEFAULT VALUE
;;      nil
;;
;; EXAMPLE
;(SetOption 'ExcludeAttrsWhenLoading '("REFDES"))


;;
;; NAME
;;      ExcludeAttrsWhenAnnotating -- don't annotate these attributes
;;
;; DESCRIPTION
;;      Don't annotate the given attributes (even if those attributes are
;;      used as search criteria).
;;
;; DEFAULT VALUE
;;      nil
;;
;; EXAMPLE
(SetOption 'ExcludeAttrsWhenAnnotating '("SYMBOL"))


;;
;; NAME
;;      AttrVis* -- attribute visibility
;;
;; SYNOPSIS
;;      (SetOption 'AttrVisVisible   attr_list)
;;      (SetOption 'AttrVisInvisible attr_list)
;;      (SetOption 'AttrVisValue     attr_list)
;;      (SetOption 'AttrVisName      attr_list)
;;
;;          attr_list -- consVT; list of attributes to be given the
;;                       specified visibility.  Use nil to set the default
;;                       visibility
;;
;; DESCRIPTION
;;      These options together determine the visibility of added attributes
;;      (those with no placeholder on the symbol or instance).  See the
;;      MaintainAttrVis option below for what happens to existing
;;      attributes.
;;
;; DEFAULT VALUE
;;      all attributes are visible
;;
;; EXAMPLE 1
;;      ;; After this sequence, all attrs will be visible, except those named
;;      (SetOption 'AttrVisInvisible '("PART_NUMBER" "MFG"))
;;      (SetOption 'AttrVisValue     '("PCB_LIB"))
;;      (SetOption 'AttrVisName      '("VENDOR"))
;;
;; EXAMPLE 2
;;      ;; After this sequence, all attrs will be invisible, except those named
;;      (SetOption 'AttrVisInvisible nil)         ; nil means all attrs
;;      (SetOption 'AttrVisVisible   '("PART_NUMBER"))
;;
;; EXAMPLE 3
;(SetOption 'AttrVisInvisible '("CATEGORY"))


;;
;; NAME
;;      MaintainAttrVis -- maintain or override existing attribute visibility?
;;
;; DESCRIPTION
;;      If true (#t), ViewDatabook will maintain the visibility of existing
;;      attributes.  By default (#f), ViewDatabook will use the settings of
;;      the AttrVis* options to determine the visibility of existing
;;      attributes.
;;
;;      In any case, the visibility of new attributes (those with no
;;      placeholder on the symbol or instance) will be determined by the
;;      settings of the AttrVis* options.
;;
;;      The location of attributes is always taken from the placeholder, if
;;      present.
;;
;; DEFAULT VALUE
;;      #f
;;
;; EXAMPLE
;(SetOption 'MaintainAttrVis #t)


;;
;; NAME
;;      SymAttrsGetLoaded -- enable loading of symbol attributes
;;
;; DESCRIPTION
;;      Causes ViewDatabook II to look at both component and symbol
;;      attributes when loading from a ViewDraw schematic.  (Annotated
;;      attributes will only be added to the component, however.)
;;
;; DEFAULT VALUE
;;      #f
;;
;; EXAMPLE
;(SetOption 'SymAttrsGetLoaded #t)


;;; OCCURRENCE ATTRIBUTES ----------------------------------------------------


;;
;; NAME
;;      OatsEnabled -- enables the use of oats
;;
;; DESCRIPTION
;;      Display buttons which allow the user to mark individual attributes
;;      as occurrenceAttributes.  Also modifies the default value of the
;;      'OatsGetLoaded and 'OatsOnByDefault options noted below.
;;
;; DEFAULT VALUE
;;      #f
;;
;; EXAMPLE
;(SetOption 'OatsEnabled #t)


;;
;; NAME
;;      OatsGetLoaded -- enable loading of oats
;;
;; DESCRIPTION
;;      Causes ViewDatabook II to look for occurrence attributes when
;;      loading from a ViewDraw schematic.
;;
;; DEFAULT VALUE
;;      value of 'OatsEnabled option
;;
;; EXAMPLE
;(SetOption 'OatsGetLoaded #f)


;;
;; NAME
;;      OatsOnByDefault -- startup state of oats buttons
;;
;; DESCRIPTION
;;      Determines the starting state of the oats buttons, whether they are
;;      on or off.
;;
;; DEFAULT VALUE
;;      value of 'OatsEnabled option
;;
;; EXAMPLE
;(SetOption 'OatsOnByDefault #f)


;;; SEARCH FIELD CUSTOMIZATION -----------------------------------------------


;; OVERVIEW
;;      The algorithm for determining what attributes show up in the vdb
;;      window as search fields is as follows (in pseudo-code):
;;
;;          IF ( display_list := CategoryDisplayAttrs[category] )
;;          THEN use display_list
;;          ELSE
;;              IF (data organized by library)
;;              THEN display_list := all attributes in library minus
;;                                   ExcludeAttrsFromDisplay[category]
;;              ELSE display_list := CategoryDisplayAttrsDefault
;;
;;          IF ( null( display_list ) )
;;          THEN error
;;


;;
;; NAME
;;      ExcludeAttrsFromDisplay -- easiest search form customization
;;
;; DESCRIPTION
;;      Specifies a list of attrs to exclude when building the search form.
;;
;; DEFAULT VALUE
;;      nil
;;
;; EXAMPLE
;(SetOption 'ExcludeAttrsFromDisplay
;           '("B_PKG" "CATEGORY" "INSERTION_CODE" "REVISION"))


;;
;; NAME
;;      CategoryDisplayAttrs -- search form customization
;;
;; DESCRIPTION
;;      CategoryDisplayAttrs specifies the list of attributes to be
;;      displayed in the search form for a particular category (library) of
;;      components.  If not set, the search form displays all attributes
;;      present in the category.
;;
;;      Use 'CategoryDisplayAttrsDefault to specify the list of attributes 
;;      displayed if there is no list specified for the particular category,
;;      or if the user is searching all components.
;;
;; EXAMPLE
;(SetOption 'CategoryDisplayAttrsDefault
;           '("LOGICAL_NAME" "TECHNOLOGY" "PKG_TYPE"
;             "MFGR" "SYMBOL" "PART_NUMBER"))
;(SetOption 'CategoryDisplayAttrs "Capacitors"
;           '("PART_NUMBER" "LOGICAL_NAME" "VALUE" "VOLTAGE" "TOL"
;             "PKG_TYPE" "MFGR"))


;;
;; NAME
;;      RequiredAttrs -- define required attributes
;;
;; SYNOPSIS
;;      (SetOption 'RequiredAttrs attr_alist)
;;
;;          attr_alist -- consVT; association list of attribute name/value
;;                        pairs.  A value of #t means ensure that an
;;                        attribute of the given name exists.  A string
;;                        value indicates a search string such as one would
;;                        enter in a search field (i.e. wildcards and range
;;                        specifications are allowed)
;;
;; DESCRIPTION
;;      Specifies the minimum set of criteria a component is required to
;;      meet in order to be browsed by ViewDatabook.  Required attrs will
;;      always be displayed in the search form.
;;
;; DEFAULT VALUE
;;      nil
;;
;; EXAMPLE 1
;;      To limit browsing to only those components which have SYMBOL and
;;      PKG_TYPE attributes:
;;
;;          (SetOption 'RequiredAttrs '(("SYMBOL" . #t)
;;                                      ("PKG_TYPE" . #t)))
;;
;;      Note that by default, SYMBOL attributes are added automatically by
;;      ViewDatabook.
;;
;; EXAMPLE 2
;;      To limit browsing to only those components which have a SYMBOL and a
;;      "soic" PKG_TYPE, use this (note the wildcard):
;;
;;          (SetOption 'RequiredAttrs '(("SYMBOL" . #t)
;;                                      ("PKG_TYPE" . "soic*")))
;;
(SetOption 'RequiredAttrs '(("SYMBOL" . #t)))


;;; MISCELLANEOUS ------------------------------------------------------------


;;
;; NAME
;;      TextColorsEnabled -- enable colored text in search fields
;;
;; DESCRIPTION
;;      'TextColorsEnabled enables or disables the use of colors to indicate
;;      the source of the value in a search field.
;;
;; DEFAULT VALUE
;;      #t
;;
;; EXAMPLE
;(SetOption 'TextColorsEnabled #f)


;;
;; NAME
;;      CommonValueColor  -- set text color of "common values"
;;      LoadedValueColor  -- set text color of values loaded from ViewDraw
;;
;; DESCRIPTION
;;      'CommonValueColor determines the color of "common values",
;;      i.e. attribute values that are in common between all components
;;      matching the search criteria.
;;
;;      'LoadedValueColor determines the color of values loaded from
;;      ViewDraw.
;;
;; DEFAULT VALUES
;;      vgAllColor|Blue
;;      vgAllColor|Red
;;
;; EXAMPLE
;(SetOption 'CommonValueColor  vgAllColor|Green)
;(SetOption 'LoadedValueColor  vgAllColor|Magenta)


;;
;; NAME
;;      ComponentTracking -- highlighting in ViewDraw
;;
;; DESCRIPTION
;;      Set 'ComponentTracking to "Yes" to have the relevant components
;;      highlighted on the schematic as each group is processed.  Set it to
;;      "Zoom" to cause ViewDraw to zoom in on the selected components.  Set
;;      it to "No" to disable this feature.
;;
;; DEFAULT VALUE
;;      "Yes"
;;
;; EXAMPLE
;(SetOption 'ComponentTracking "Zoom")


;;
;; NAME
;;      UndefinedValue -- specify the value placeholder
;;      AnnotateUndefined -- should undefined value be annotated?
;;
;; DESCRIPTION
;;      PAR files require a value in every column.  Values which are equal
;;      to the specified UndefinedValue may optionally be excluded from
;;      annotation, using AnnotateUndefined.
;;
;; DEFAULT VALUE
;;      no default value for UndefinedValue
;;      #f
;;
;; EXAMPLE
;(SetOption 'UndefinedValue "----")
;(SetOption 'AnnotateUndefined #t)


;;
;; NAME
;;      TestValues -- verify attributes have values before annotating
;;
;; DEFAULT VALUE
;;      "Yes"
;;
;; EXAMPLE
;(SetOption 'TestValues "No")


;;
;; NAME
;;      LibNameAttr -- name of library is specified as an attribute
;;
;; DESCRIPTION
;;      The 'LibNameAttr option specifies the name of an attribute.  The
;;      value of this attribute, if present on a component, indicates the
;;      name of a VLLM library in which to look for the part information.
;;      Any file extensions will be removed before looking for the library.
;;
;;      This replaces the functionality of the "parfile_attribute" option in
;;      the VCS ViewDatabook "partslib.cfg" file.
;;
;; DEFAULT VALUE
;;      none
;;
;; EXAMPLE
;;      In the "vllm.scm" file is the following, which defines a library named
;;      "nand" consisting of a database file "nand.par".
;;
;;          (Process 'AddLibrary "74ls" #t
;;                   '(("74ls" "74ls.par" "libs" "PAR"))
;;                   '())
;;
;;      In the "vdbum.scm" file, there is the option:
;;
;;          (SetOption 'LibNameAttr "PARFILE")
;;
;;      For a given component 74ls04, there is an attribute:
;;
;;          PARFILE=74ls.par
;;
;;      Then, when this component is selected for processing by ViewDatabook
;;      II, it looks in the "74ls" library for the part information.
;;
;; EXAMPLE
;(SetOption 'LibNameAttr "PARFILE")
