/* ---------------------------------------------------------------------- *\
 * Module -- vtimefo.vs
 *
 *      Facility:      Viewtime
 *      Environment:   sun4
 *      Author:        Mani Alagappan - May 1992
 *      Description:   Contains all code related to setting up of timing block
 *                     and case and eventually opens the facility.
 *       
 *
 *      Copyright (c) 1992 by Viewlogic Systems, Inc.
 */
/* ---------------------------------------------------------------------- *\
 Modification History
$Id: vdinifo.vs,v 1.4 93/11/09 10:04:27 mania Exp Locker: mania $
$Log:	vdinifo.vs,v $
Revision 1.4  93/11/09  10:04:27  mania
Tried enable/disabling library picklist regions.

Revision 1.3  93/11/08  14:56:57  mania
Added command to show/hide library picklist region.

Revision 1.2  93/10/28  17:43:22  mania
Library aliasing added.

Revision 1.1  93/08/02  15:47:58  mania
Initial revision

Revision 1.4  93/05/20  11:14:31  mania
Added documentation.

Revision 1.3  93/05/17  09:17:10  mania
Added facility fields and rearranged args order.
Added callbacks for dir select notification. Added
check for null directory. Default pick is zero.

Revision 1.2  93/05/14  11:40:33  mania
Expanded file select facility to take args list for
dir search, file notify and cmdline return.

Revision 1.1  93/05/12  17:09:55  mania
Initial revision

\* ---------------------------------------------------------------------- */	
function label(str) vcRegion.Create1(str,vcLabel.fac,str,nil);

function entrybox(prompt, cmdline)
{
    text_entry := vcTable.Create ("", 1, 2);
    vcTable.Add (text_entry, label (prompt));
    vcTable.Add (text_entry, cmdline);

    text_entry;
} /* function entrybox */

function cmdline(default, onreturn)
{
    cmd_line := vcRegion.Create1 ("", vcCmdline.fac, default, 
                                  list (cons (`width, 30)));
    vcCmdline.CallOnReturn (cmd_line, onreturn);

    cmd_line;
} /* function cmdline */

/*
 * Read the Viewdraw search order from the Viewdraw.ini file
 */
function read_viewdraw_search_order(cntx) {
    declare cmd, dir_type, path, alias;
    declare vdrawini;

    fname := vscript.vsGetini ("viewdraw.ini");
    if (fname == nil)
       return (nil);

    vdrawini = fopen (fname, "r");
    path_list  := nil;
    alias_list := nil;    
    for (s := fgetline (vdrawini); s != nil; s = fgetline (vdrawini)) {

        /*
         * If it's a directory line, "DIR [r] /pathname/...."
         * parse it and add it to the path list.
         */
        if (((s = string_token (s, &cmd, " \t")) != nil) &&
            (string_eqi (cmd, "dir")) &&
            ((s = string_token (s, &dir_type, " \t")) != nil) &&
            ((s = string_token (s, &path, " \t\n")) != nil)){
            alias = nil;
            string_token (s, &alias, " \t\n");
            path = vscript.vsAbsPath (path);

            if (string_eqi (dir_type, "[p]") == TRUE ||
                string_eqi (dir_type, "[pw]") == TRUE){
                cntx.PrimaryDir = path;
            }


            
            if (alias == nil){
                alias = path;
                path_list = cons (list (path), path_list);
                alias_list = cons (alias, alias_list);
            }

            else{
                index := 0;
                found := FALSE;
                foreach(i;vscript.list_iter (alias_list)){
                    if (string_eqi (i,alias) == TRUE){
                        found = TRUE;
                        break;                        
                    }
                    ++index;
                }

                if (found){
                    path_entry := list_ref (path_list, index);
                    /*if (ispair (path_entry) == FALSE)
                        path_entry = list(path_entry);*/
                    newpaths := append (path_entry, list(path));
                    list_set (path_list, index, newpaths);
                }

                else{
                    path_list = cons (list (path), path_list);
                    alias_list = cons (alias, alias_list);
                }
            }
        }
    } /* for */

    fclose (vdrawini);
    pl := reverse_destructive (path_list);    
    al := reverse_destructive (alias_list);    
    list (al, pl);
} /* function read_viewdraw_search_order */

Files  := nil;
NFiles := 0;

function AddFile2List(fspec)
{
    list := vscript.vsParse (fspec);
    fname := caddr (list);
    Files = cons (fname, Files);
    ++NFiles;
} /* function AddFile2List */

function BuildFileList(pathlist,dir,name,ext,delim, primarydir)
{
   /*printf ("path:%s, dir: %s, name: %s ext: %s, delim: %s, pdir=%s\n", 
            pathlist,dir,name,ext,delim, primarydir);*/

    Done  := FALSE;
    Files  = nil;
    NFiles = 0;

    foreach (path;vscript.list_iter (pathlist)){
        if (path[string_length(path)-1] != delim[0])
            path  = path + delim;

        if (vscript.vsInfo (path) != nil){
            if (string_eqi (dir, "tim") == TRUE && Done != TRUE){
                Done = TRUE;
                projdir := vsGetdef ();
                path = projdir + delim + delim + dir;
                path = vscript.vsConvPath (path);
                fspec := vscript.vsMerge ("", path, name, ""); 
                vscript.vsLoop (fspec, vscript.vsDIR, AddFile2List);
            }
            else{
                path   = path + dir;
                fspec := vscript.vsMerge ("", path, name, ext);
                vscript.vsLoop (fspec, vscript.vsFILE, AddFile2List);
            } 
            
            if (Files != nil){
                list := reverse_destructive (Files);
                Files = vscript.Sort (list, string_lt);
            } 
        } 
    }
    Files;
} /* function BuildFileList */

/*\Section vtimefo - File Select for viewdraw.ini
 *
 * DESCRIPTION
 *      This facility returns a table containing three regions.
 *      The last region contains a picklist of absolute pathnames 
 *      of all libraries found in viewdraw.ini file. The middle 
 *      region contains a user supplied picklist of dirs to search
 *      under each library like sch, sym, wir etc. The first region
 *      contains a picklist of all files found under 'directory'
 *      selected in the picklist from the middle.
 *
 *      Facility takes list of four arguments. The first argument is
 *      a Viewscript list of lists of directories to select under each 
 *      library. Second argument is a file selection fileter. An empty 
 *      string, for example, selects all files under the 'sch' directory. 
 *      Third arg is a notify function when a selection is made in the file 
 *      picklist. The fourth argument is a notify function for library 
 *      selection. Both these functions receive two args, the name of the 
 *      selection and a vlOnOff flag.
 *
 *      function file_notify(file, onoff){
 *      ...
 *      } 
 *  
 *      function lib_notify(library, onoff){
 *      ...
 *      } 
 *      dirs := list (list (cons ("Schematics","sch"),
 *                          cons ("Symbols", "sym")));
 *      reg := vcRegion.Create ("", vtimefo.fac, 
 *                              list(dirs, "", file_notify, lib_notify));
 *
 * SYNOPSIS
 *   vcRegion.Create ("", vtimefo.fac, facility_arguments_list);
*/
function _Create(args, region)
{
    Context := struct { Table;
                        Filelist;
                        Select;
                        FileNotify;
                        DirNotify;
                        SearchPick;
                        Filter;
                        Dir;
                        Library;
                        ScrollLiblist;
                        PrimaryDir;
                        Design
                        };

    function build_files(newlib, onoff)
    {
        if (onoff == vlOnOff.On){
            w := vcRegion.Window (region);
            if (w != nil)
                vcWin.Busy (w, TRUE);

            if (newlib == nil)
               newlib = list (".");
            Context.Library = newlib;
            vcCmdline.Set (Context.Select, "");
            filelist := BuildFileList (newlib, Context.Dir,"*",
                                       Context.Filter, "/",
                                       Context.PrimaryDir);
            /*printf ("Total files: %s\n", NFiles);*/
            r := NFiles / 4;
            if (r < 1)
                r = 1;

            Context.Filelist = vcRegion.Create ("files", vcPicklist.fac, 
                                                nil, 
                                                rows <- r /*10*/, 
                                                columns <- 4,
                                                /*will_scroll <- TRUE,*/
                                                macro_on <- TRUE);

            count   := 0;
				pick    := 0;
            foreach (i; vscript.list_iter (filelist)){
					if (string_eqi (i,Context.Design) == TRUE)
						pick = count;
					vcPicklist.Add (Context.Filelist, i, i);
					++count;
             }
            
            Context.FileNotify ("nil", vlOnOff.On);
            vcPicklist.Notify (Context.Filelist, 
                               func (file, onoff){
                                   if (onoff == vlOnOff.On)
                                       vcCmdline.Set (Context.Select,file);
                                   Context.FileNotify (file, onoff);
                               });
            vcTable.Set (Context.Table, 0, 0, 
                         vcPicklist.Scroll ("", Context.Filelist, 4, 6, 0));

            vcPicklist.SetPick (Context.Filelist, pick);
            if (w != nil)
					vcWin.Busy (w, FALSE);
        } /* if */
    } /* function build_files */
    

    /* walk the arglist */
    Context.SearchPick = vcRegion.Create ("cur_lib", vcPicklist.fac, 
                                          car (args), 
                                          rows <- 2, columns <- 3,
                                          macro_on <- TRUE);
    Context.Filter     = cadr (args);
    Context.FileNotify = caddr (args);
    Context.DirNotify  = cadddr (args);
    Context.Dir        = nil;
    Context.Filelist   = nil;
    Context.PrimaryDir = "";
    Context.ScrollLiblist    = nil;

    listargs := cdddr (args);
    Context.Design     = car (cdr (listargs));


    vcPicklist.Notify (Context.SearchPick, 
                       func (dir, onoff)
                       {
                           if (onoff == vlOnOff.On){
                               if (Context.Dir == nil)
                                   Context.Dir = dir;
                               else{
                                   Context.Dir = dir;
                                   Context.DirNotify (dir, onoff);
                                   build_files (Context.Library, onoff);
                               }
                           } /* if */
                       });    
    vcPicklist.SetPick (Context.SearchPick, 1);

    Context.Table   = vcTable.Create ("", 4, 1);
    vcTable.Row (Context.Table, -1, vcTable.Border.Visible, 5, 8, 12,
                 vgAlign.MiddleCenter);

    Context.Select = cmdline ("", 
                              func (e, str)
                                  Context.FileNotify (str, vlOnOff.On););
    vcTable.Set (Context.Table, 1, 0, 
                 entrybox ("Design Name: ", Context.Select));
    vcTable.Set (Context.Table, 2, 0, Context.SearchPick);

    libs := read_viewdraw_search_order (Context);
    liblist := vcRegion.Create1 ("libraries", vcPicklist.fac, nil,
                                 list (cons (`exclusive, TRUE),
                                       /*will_scroll <- TRUE,*/
                                       cons (`rows, 5),
                                       cons (`columns, 1)));
    al := car (libs);
    pl := cadr (libs);
    foreach (i; vscript.list_iter (al)){
        vcPicklist.Add (liblist, i, car (pl));
        pl = cdr (pl);
    }
    Context.ScrollLiblist = vcPicklist.Scroll ("scroll_libs",liblist, 4, 6, 8);
    vcTable.Set (Context.Table, 3, 0, Context.ScrollLiblist);
    vcPicklist.Notify (liblist, build_files);
    vcPicklist.SetPick (liblist, 0);

    vcRegion.AddChild (region, Context.Table);

    Context;
} /* function _Create */

function SetSrchPick (d, pick)
   vcPicklist.SetPick (d.SearchPick, pick);

function HideLibs(d){
   vcRegion.Enable (vcTable.Get (d.Table, 3, 0), FALSE, TRUE);
   /*print (vcTable.Get (d.Table,3,0));*/
   vcRegion.Enable (d.ScrollLiblist,FALSE, TRUE);
}
hide_libraries_cmd := vcWv.Cmd(fac,func (d,e) HideLibs (d);,"_hidelibs");

function ShowLibs(d)
{
   vcRegion.Enable (vcTable.Get (d.Table, 3, 0), TRUE, FALSE);
   /*print (vcTable.Get (d.Table,3,0));*/
   vcRegion.Enable (d.ScrollLiblist,TRUE, TRUE);
}
show_libraries_cmd := vcWv.Cmd(fac,func (d,e) ShowLibs (d);,"_showlibs");

def_pick_cmd := vcWv.Cmd(fac,func (d,e)vcPicklist.SetPick (d.SearchPick, 0);,"_defpick");
function PrimaryDir(d) d.PrimaryDir;
    
function GetBlockName(d) vcCmdline.Get (d.Select);

