vipcMsgVDRAW_ROR      := 10011;   /* Read Or Readreplace  */
vipcMsgVDRAW_SIM      := 10012;   /* Save If Modified     */
vipcMsgVDRAW_OOF      := 10013;   /* Open Or Front        */

/* This function returns a window id for a viewdraw window.  If name is
   nil, then any viewdraw window is returned.  If name is not nil, a
   check is done to make sure the design name matches.  If no window
   is found with that name, then a window is opened (of the correct
   type) with if open_win is TRUE. */
function get_vdraw_win(name, type, open_win) 
{
    the_window := nil;
    foreach (w; vcWin.Loop ()) {
        if ((the_window == nil) && (vcWin.Parent (w) == nil)) {
            r := vcWin.OwnerRegion (w);
            if (vcRegion.Fac (r) == vdraw.fac) {
                if (name != nil) {
                    if (string_eqi (name, vdraw.getdesignname(r)))
                        the_window = w;
                }
                else
                    the_window = w;
            }
        } /* if */
    } /* foreach */

    if ((the_window != nil) || (open_win == vlYesNo.No) || (name == nil))
        the_window;
    else if (string_eq(type, "sch")) 
        the_window = schematic (name);
    else
        the_window = symbol (name);
}

/* This routine opens a schematic window if none are already open.  If a 
   schematic window is already open, then issue the "readreplace" command
   to reread the schematics and all instantiated symbols from disk.
   You might send this command if your tool just modified a schematic and you
   want to show it to the user. */
function read_readreplace(mid, src_cntx, mtch_cntx, data_len, data_type, data)
{
    window := get_vdraw_win(data, "sch", vlYesNo.Yes);

    /* If get_vdraw_win had to open the schematic, then we don't need
       to do anything else.  If the window already existed, then we
       need to issue a readreplace command. The check to see whether
       get_vdraw_win had to open the schematic is vcWin.EverRefreshed. */
    if ((window != nil) && vcWin.EverRefreshed (window)) {
        event := vcEvent.Create (0, 0, '0', nil, window,
                                 vcWin.OwnerRegion (window), nil);
        vcWin.RunCmdline(window, "readreplace " + data, event);
    }
    /* Reply to message */
    vipc.Send(vipcMsgVDRAW_ROR, mtch_cntx, src_cntx, " ", vipc.FlgDIRECT);
}

/* This routine saves a schematic if that schematic window is open, and
   the data has been modified.  You might send this command if your tool
   is about to read a WIR file and you want to catch any changes the user
   may have just made to the schematic. */
function save_if_mod(mid, src_cntx, mtch_cntx, data_len, data_type, data)
{
    window := get_vdraw_win(data, "sch", vlYesNo.No);
    if (window != nil) {
        event := vcEvent.Create (0, 0, '0', nil, window,
                                 vcWin.OwnerRegion (window), nil);
        display := wingetdisplay(window);
        if (modified(display, event, vqDataType.Schematic) == 1)       
            vcWin.RunCmdline(window, "write", event);
    }
    /* Reply to message */
    vipc.Send(vipcMsgVDRAW_SIM, mtch_cntx, src_cntx, " ", vipc.FlgDIRECT);
}

/* This routine opens a schematic window on the specified name; if there is
   already a window open, it brings it to the front.  You might send this
   command if you want the user to start editing a schematic. */
function open_or_front(mid, src_cntx, mtch_cntx, data_len, data_type, data)
{
    window := get_vdraw_win(data, "sch", vlYesNo.Yes);
    vcWin.Show (window);
    /* Reply to message */
    vipc.Send(vipcMsgVDRAW_OOF, mtch_cntx, src_cntx, " ", vipc.FlgDIRECT);
}

function send_pong(mid, src_cntx, mtch_cntx, data_len, data_type, data)
{
    vipc.Send(vipc.MsgPONG, mtch_cntx, src_cntx, " ", vipc.FlgDIRECT);
}


/* Create the ipc context, and register the handler routines */
VdrContext := vipc.Init (vipc.TidVDRAW);
AnyContext := vipc.CntxCreate(vipc.GidANY, vipc.UidANY, vipc.FidDFLT,
                              vipc.TidANY, vipc.HnmANY, vipc.PidANY,
                              vipc.CidANY);
vipc.Register(vipc.MsgPING, VdrContext, AnyContext, send_pong,
              nil, vipc.FlgNULL);
vipc.Register(vipcMsgVDRAW_ROR, VdrContext, AnyContext, read_readreplace,
              nil, vipc.FlgNULL);
vipc.Register(vipcMsgVDRAW_SIM, VdrContext, AnyContext, save_if_mod,
              nil, vipc.FlgNULL);
vipc.Register(vipcMsgVDRAW_OOF, VdrContext, AnyContext, open_or_front,
              nil, vipc.FlgNULL);
