/* This file contains the following major sections:
   - definitions of preferences for all tools
   - definitions of search method for all files 
   - definitions of tools and their inputs, outputs and preferences
   - definitions of tool chains for all targets
*/


/*** PREFERENCES ***/


/* Global preferences */
vlPref.Add (prefs, "ignore_timestamps", vlYesNoVT,
   "Force tool invocation by ignoring timestamps", vlYesNo.No);
vlPref.Add (prefs, "crossprobe", vfpgaXprobeVT,
   "Allow cross-probing to and from ViewDraw", vfpgaXprobe.None);
vlPref.Add (prefs, "warnalerts", vlYesNoVT,
   "Pop alerts before overwriting files", vlYesNo.Yes);
vlPref.Add (prefs, "menumask", vfpgaMenuMaskVT,
   "Mask for vendor menus (no immediate effect)",
   vfpgaMenuMask.Actel + vfpgaMenuMask.Altera + vfpgaMenuMask.PLD +
   vfpgaMenuMask.Xilinx);

/* Less important global prefs */
vlPref.Add (prefs, "topname", vlStringVT, "Name of top level design", "");
vlPref.Add (prefs, "toptype", vlStringVT,
   "Type of top level design: sch,vhdl,abel", "");
vlPref.Add (prefs, "typeatts", vlStringVT, "Attributes which determine type",
   "VHDL:jed=JEDEC:pla=Partition:ABEL:TDF:altera=AlteraChip:+LPM_TYPE=LPM");
vlPref.Add (prefs, "levels", vlStringVT, "Symbol levels to stop descent",
   "unit=Actel:hard=Actel:xilinx=Xilinx:mxilinx=Xilinx:hmxilinx=Xilinx:dio:vhdl");
vlPref.Add (prefs, "maxloaderr", vlIntVT, "Errors allowed during load", 5);
vlPref.Add (prefs, "testmode", vlIntVT,
   "Flags for test mode (no immediate effect)", 0);
vlPref.Add (prefs, "graphics", vlYesNoVT,
   "Display windows (no immediate effect)", vlYesNo.Yes);


/* Parse ini file and command line */
path := vscript.vsSrchdirs ("vfpga.ini", vscript.vsFILE);
args := vcWv.args;
vlPref.SetShorthand ("vfpga");
vlPref.Parse (path, &args);
g_menumask := vlPref.Val (prefs, "menumask");
cb_prefSetTestmode(vlPref.Val (prefs, "testmode"));


/* Device aliases for device selector */
cb_cfgAddDeviceAlias ("MACH110_GEN", "MACH110A");
cb_cfgAddDeviceAlias ("MACH120_GEN", "MACH120A");
cb_cfgAddDeviceAlias ("MACH130_GEN", "MACH130A");
cb_cfgAddDeviceAlias ("MACH210_GEN", "MACH210A");
cb_cfgAddDeviceAlias ("MACH215_GEN", "MACH215A");
cb_cfgAddDeviceAlias ("MACH220_GEN", "MACH220A");
cb_cfgAddDeviceAlias ("MACH230_GEN", "MACH230A");
cb_cfgAddDeviceAlias ("P5016_GEN",   "P5016A");
cb_cfgAddDeviceAlias ("P5024_GEN",   "P5024A");
cb_cfgAddDeviceAlias ("P5032_GEN",   "P5032A");
cb_cfgAddDeviceAlias ("P5016C_GEN",  "P5016AC");
cb_cfgAddDeviceAlias ("P5024C_GEN",  "P5024AC");
cb_cfgAddDeviceAlias ("P5032C_GEN",  "P5032AC");
cb_cfgAddDeviceAlias ("P5016_GEN",   "P5016S");
cb_cfgAddDeviceAlias ("P5024_GEN",   "P5024S");
cb_cfgAddDeviceAlias ("P5032_GEN",   "P5032S");
cb_cfgAddDeviceAlias ("P5016C_GEN",  "P5016SC");
cb_cfgAddDeviceAlias ("P5024C_GEN",  "P5024SC");
cb_cfgAddDeviceAlias ("P5032C_GEN",  "P5032SC");


/* Global of mappings from technology name to synthesis library name */
g_vdes_alias_hash := vlSym.StrCreate (11);

/* Design preferences */
function prefDefoneatt (name, tspec, help, initval, prefdb, startup) {
   if ((vcWv.UI(wv) != vcUi.Motif) && (vcWv.UI(wv) != vcUi.OpenLook))
      vlPref.Add (prefdb, name, tspec, help, initval);
   else vlPref.Replace (prefdb, name, tspec, help, initval);
   if (startup) cb_prefDefatt (name, help); }


function pldDefallatts (prefdb, startup) {
   function defatt (name, tspec, help, initval)
      prefDefoneatt (name, tspec, help, initval, prefdb, startup);

   defatt ("fit_run", vlYesNoVT, "FIT: run fitter", vlYesNo.Yes);
   defatt ("fit_pin", vfpgaFitPinVT, "FIT: Pin preassignment", 
      vfpgaFitPin.Attempt);
   defatt ("fit_args", vlStringVT, "FIT: additional arguments", "");
   defatt ("j2v_delay", vfpgaJ2VDelayVT, "J2VHDL: delay type", 
      vfpgaJ2VDelay.Typ);
   defatt ("j2v_utf", vlYesNoVT, "J2VHDL: user timing file", vlYesNo.No);
   defatt ("part_auto", vlStringVT, "AUTOPART: auto device", "P22V10");
   defatt ("part_strat", vlStringVT, "AUTOPART: strategy", "newauto");
   defatt ("part_args", vlStringVT, "AUTOPART: additional arguments","");

   defatt ("jed_out", vlStringVT, "FUSEASM: JEDEC file name", "");
   defatt ("jed_chksum", vfpgaJedChksumVT, "FUSEASM: checksum format",
      vfpgaJedChksum.Full);
   defatt ("jed_docfmt", vfpgaJedDocfmtVT, "FUSEASM: document format",
   vfpgaJedDocfmt.Brief);
   defatt ("jed_plcc", vlYesNoVT, "FUSEASM: PLCC pinout in report", vlYesNo.No);
   defatt ("jed_blow", vlYesNoVT, "FUSEASM: blow unused fuses", vlYesNo.No);
   defatt ("jed_turbo", vlYesNoVT, "FUSEASM: blow turbo fuse", vlYesNo.No);
   defatt ("jed_miser", vlYesNoVT, "FUSEASM: blow miser fuse", vlYesNo.No);
   defatt ("jed_lock", vlYesNoVT, "FUSEASM: blow security fuse",vlYesNo.No);
   defatt ("jed_xval", vfpgaJedXvalVT, "FUSEASM: default X value",
      vfpgaJedXval.Zero);
   defatt ("jed_fmt", vlStringVT, "FUSEASM: JEDEC format", "default");
   defatt ("jed_ues", vlStringVT, "FUSEASM: user signature word", "");
   defatt ("sel_man", vlStringVT, "DEVSEL: manufacturers", "");
   defatt ("sel_temp", vlStringVT, "DEVSEL: temperature grades", "");
   defatt ("sel_tech", vlStringVT, "DEVSEL: fab technologies", "");
   defatt ("sel_pkg", vlStringVT, "DEVSEL: package styles", "");
   defatt ("sel_jam", vfpgaSelJamVT, "DEVSEL: register preloadability",
      vfpgaSelJam.Any);
   defatt ("sel_era", vfpgaSelEraVT, "DEVSEL: erasability", vfpgaSelEra.Any);
   defatt ("sel_pins", vlStringVT, "DEVSEL: number of pins", "");
   defatt ("sel_spd", vlStringVT, "DEVSEL: speed", "");
   defatt ("sel_set", vlStringVT, "DEVSEL: setup time", "");
   defatt ("sel_hold", vlStringVT, "DEVSEL: hold time", "");
   defatt ("sel_cost", vlStringVT, "DEVSEL: cost", "");
   defatt ("sel_pow", vlStringVT, "DEVSEL: power usage", "");
   defatt ("sel_u1", vlStringVT, "DEVSEL: user field 1", "");
   defatt ("sel_sort", vlStringVT, "DEVSEL: sort order", ""); }


function prefDefallatts (prefdb, startup) {
   function defatt (name, tspec, help, initval)
      prefDefoneatt (name, tspec, help, initval, prefdb, startup);

   defatt ("device", vlStringVT, "PLD device name", "");
   defatt ("part", vlStringVT, "PLD exact part name", "");
   defatt ("part_out", vlStringVT, "AUTOPART: output name", "");
   defatt ("technology", vlStringVT, "FPGA technology", "");
   defatt ("fpga_die", vlStringVT, "FPGA target die", "ANY");
   defatt ("fpga_package", vlStringVT, "FPGA target package", "ANY");
   defatt ("fpga_speed", vlStringVT, "FPGA target speed grade", "ANY");
   defatt ("sym_type", vfpgaSymTypeVT, "SYMBOL: library symbol name",
      vfpgaSymType.Automatic);
   defatt ("sym_pinno", vlYesNoVT, "SYMBOL: use pin numbers", vlYesNo.No);
   defatt ("sym_libsym", vlStringVT, "SYMBOL: library symbol name", "");
   defatt ("sym_pinspace", vlPositiveIntVT, "SYMBOL: pin spacing", 10);
   defatt ("sym_pinlen", vlPositiveIntVT, "SYMBOL: pin length", 20);
   defatt ("sym_biside", vfpgaSideVT, "SYMBOL: side for bidir pins",
      vfpgaSide.Right);
   defatt ("a2p_retain", vlYesNoVT, "AHDL2PLA: retain redundant", vlYesNo.No);
   defatt ("a2p_tmv", vlYesNoVT, "AHDL2PLA: translate top level test vectors",
      vlYesNo.No);
   defatt ("a2p_defs", vlStringVT, "AHDL2PLA: definitions", "");
   defatt ("a2p_args", vlStringVT, "AHDL2PLA: additional arguments", "");
   defatt ("po_strat", vlStringVT, "PLAOPT: strategy", "default");
   defatt ("po_exact", vlYesNoVT, "PLAOPT: exact optimization", vlYesNo.No);
   defatt ("vhdl_err", vlStringVT, "VHDL: error level", "1");
   defatt ("vdes_opt", vfpgaVdesOptVT, "VHDLDES: optimization goal", 
           vfpgaVdesOpt.Area);
   defatt ("vdes_max", vlStringVT, "VHDLDES: maximum delay", "");   /* MAA */
   defatt ("vdes_cpu_effort", vfpgaLowMedHighVT, "CPU effect", 
           vfpgaLowMedHigh.Medium);
   defatt ("vdes_flatten", vfpgaNoneLowMedHighVT, "Equation collapsing",
           vfpgaNoneLowMedHigh.None);
   defatt ("vdes_redundancy_removal", vlYesNoVT, "Redundancy removal", vlYesNo.No);
   defatt ("vdes_iopads", vlYesNoVT, "Insert I/O pads", vlYesNo.No);
   defatt ("vdes_clb", vlYesNoVT, "Pack into CLBs", vlYesNo.Yes);
   defatt ("vdes_file", vlStringVT, "VHDLDES: constraint file", "");
   defatt ("vdes_vsm", vlYesNoVT, "VHDLDES: create VSM file", vlYesNo.No);
   defatt ("vdes_vgen", vlYesNoVT, "VHDLDES: run viewgen", vlYesNo.No);
   defatt ("vdes_args", vlStringVT, "VHDLDES: additional arguments", "");

   if ((g_menumask & vfpgaMenuMask.Actel) == vfpgaMenuMask.Actel) 
      actDefallatts (prefdb, startup);
   if ((g_menumask & vfpgaMenuMask.Altera) == vfpgaMenuMask.Altera) 
      altDefallatts (prefdb, startup);
   if ((g_menumask & vfpgaMenuMask.PLD) == vfpgaMenuMask.PLD) 
      pldDefallatts (prefdb, startup);
   if ((g_menumask & vfpgaMenuMask.Xilinx) == vfpgaMenuMask.Xilinx) 
      xilDefallatts (prefdb, startup); }


/* Functions to set up attributes */
function prefDefAttrs() {
   prefDefallatts (prefs, TRUE);
   cb_prefChangeatt ("device", "arch"); }


/*** SEARCH METHODS ***/

/* cb_cfgAddFile("filename", list (list ("dir", "base", "ext"), ...));
 *    where dir can be %sch, %sym, %wir, %behv, %dot, subdir;
 *      and base can be either %base or a preference name;
 *      and ext can be %sheet or file extension;
 */
cb_cfgAddFile("sch",    list (list ("%sch",  "%base",    "1")));
cb_cfgAddFile("sym",    list (list ("%sym",  "%base",    "1")));
cb_cfgAddFile("timsym", list (list ("%sym",  "jed_out",  "1")));
cb_cfgAddFile("wir",    list (list ("%wir",  "%base",    "1")));
/* Is this right MAA */
cb_cfgAddFile("synth",  list (list ("synth","%base",    ""))); 
cb_cfgAddFile("prtsch", list (list ("%sch",  "part_out", "1")));
cb_cfgAddFile("prtwir", list (list ("%wir",  "part_out", "1")));
cb_cfgAddFile("vhdl",   list (list ("%dot",  "%base",    "vhd"),
                              list ("%behv", "%base",    "vhd"))); 
cb_cfgAddFile("timvhd", list (list ("%dot",  "jed_out",  "vhd")));
cb_cfgAddFile("vli",    list (list ("%dot",  "%base",    "vli"),
                              list ("%behv", "%base",    "vli"))); 
cb_cfgAddFile("timvli", list (list ("%dot",  "jed_out",  "vli")));
cb_cfgAddFile("vsm",    list (list ("%dot",  "%base",    "vsm")));
cb_cfgAddFile("prtvsm", list (list ("%dot",  "part_out", "vsm")));
cb_cfgAddFile("cmd",    list (list ("%dot",  "%base",    "cmd")));
cb_cfgAddFile("abel",   list (list ("%dot",  "%base",    "abl")));
cb_cfgAddFile("bl0",    list (list ("%dot",  "%base",    "bl0")));
cb_cfgAddFile("tt2",    list (list ("%dot",  "%base",    "tt2")));
cb_cfgAddFile("tt3",    list (list ("%dot",  "%base",    "tt3")));
cb_cfgAddFile("timtt3", list (list ("%dot",  "jed_out",  "tt3")));
cb_cfgAddFile("prtin",  list (list ("%dot",  "%base",    "prt")));
cb_cfgAddFile("prtout", list (list ("%dot",  "%base",    "pr1")));
cb_cfgAddFile("chip",   list (list ("%dot",  "%base",    "chp")));
cb_cfgAddFile("sel",    list (list ("%dot",  "%base",    "sel")));
cb_cfgAddFile("ord",    list (list ("%dot",  "%base",    "ord")));
cb_cfgAddFile("timord", list (list ("%dot",  "jed_out",  "ord")));
cb_cfgAddFile("jedec",  list (list ("%dot",  "%base",    "jed"),
                              list ("%sch",  "%base",    "jed")));
cb_cfgAddFile("timjed", list (list ("%dot",  "jed_out",  "jed")));

/*** TOOL DEFINITIONS ***/

/* cb_cfgAddTool (name, inputlist, outputlist, preflist) where elements in
   input and output list are defined with cfgAddFile, above, and elements in
   preflist are defined in prefDefallatts, above */

cb_cfgAddTool("p2a", list ("tt2"), list ("abel"), nil);
cb_cfgAddTool("p2v", list ("tt2"), list ("vhdl"), list ("a2p_tmv"));
cb_cfgAddTool("p2v_dio", list ("tt2"), list ("wir"), list ("a2p_tmv"));
cb_cfgAddTool("p2v_diosch", list ("tt2"), list ("sch"), nil);
cb_cfgAddTool("p2v_vli", list ("tt2"), list ("vli"), list ("a2p_tmv"));
cb_cfgAddTool("p2v_sym", list ("tt2"), list ("sym"),
              list ("sym_type", "sym_pinno", "sym_libsym", "sym_pinspace",
                    "sym_pinlen", "sym_biside"));
cb_cfgAddTool("p2v_pcb", list ("tt3"), list ("sym", "ord"),
              list ("sym_type", "sym_pinno", "sym_libsym", "sym_pinspace",
                    "sym_pinlen", "sym_biside"));
cb_cfgAddTool("p2v_pin", list ("ord"), list ("sym"),
              list ("sym_type", "sym_pinno", "sym_libsym", "sym_pinspace",
                    "sym_pinlen", "sym_biside"));
cb_cfgAddTool("timp2v", list ("timtt3"), list ("timsym", "timord"),
              list ("sym_type", "sym_pinno", "sym_libsym", "sym_pinspace",
                    "sym_pinlen", "sym_biside"));
cb_cfgAddTool("sch2prt", list ("prtsch"), list ("prtin"), nil);
cb_cfgAddTool("prt2sch", list ("prtout", "tt2"),list ("prtsch"), 
              list ("part_out"));
cb_cfgAddTool("vdes1_vhdl", list ("vhdl"), list ("synth"),
              list ("vdes_opt", "vdes_max", "vdes_cpu_effort",
                    "vdes_flatten", "vdes_redundancy_removal",
                    "vdes_file","vdes_args", "technology"));
cb_cfgAddTool("vdes2_vhdl", list ("vhdl"), list ("synth"),
              list ("vdes_opt", "vdes_max", "vdes_cpu_effort",
                    "vdes_flatten", "vdes_redundancy_removal",
                    "vdes_file", "vdes_args", "technology"));
cb_cfgAddTool("vdes1_dio", list ("vhdl"), list ("synth"),
              list ("vdes_opt", "vdes_max", "vdes_cpu_effort",
                    "vdes_flatten", "vdes_redundancy_removal",
                    "vdes_file","vdes_args"));
cb_cfgAddTool("vdes2_dio", list ("vhdl"), list ("synth"),
              list ("vdes_opt", "vdes_max", "vdes_cpu_effort",
                    "vdes_flatten", "vdes_redundancy_removal",
                    "vdes_file","vdes_args"));   
cb_cfgAddTool("vdes_viewsim", list ("wir"), list ("vsm"),
              list ("vdes_vsm"));
cb_cfgAddTool("vdes_viewgen", list ("wir"), list ("sch"),
              list ("vdes_vgen"));
cb_cfgAddTool("vhdl_ana", list ("vhdl"), list ("vli", "vsm"),
              list ("vhdl_err"));
cb_cfgAddTool("viewgen", list ("wir"), list ("sch"), nil);
cb_cfgAddTool("vg_sym", list ("wir"), list ("sym"),
              list ("sym_pinspace", "sym_pinlen"));
cb_cfgAddTool("prtcheck", list ("prtsch"), list ("prtwir"), nil);
cb_cfgAddTool("vsm", list ("wir"), list ("vsm"), nil);
cb_cfgAddTool("prtvsm", list ("prtwir"), list ("prtvsm"), nil);
cb_cfgAddTool("vhdl2sym", list ("vhdl"), list ("sym"),
              list ("sym_pinspace", "sym_pinlen"));
cb_cfgAddTool("ahdl2pla", list ("abel"), list ("bl0"),
              list ("a2p_retain", "a2p_defs", "a2p_args"));
cb_cfgAddTool("ahdl2prt", list ("abel"), list ("bl0", "prtin"),
              list ("a2p_retain", "a2p_defs", "a2p_args"));
cb_cfgAddTool("plaopt", list ("bl0"), list ("tt2"),
              list ("po_strat", "po_exact"));
cb_cfgAddTool("mergett", list ("bl0"), list ("tt2"), nil);
cb_cfgAddTool("autopart", list ("tt2", "prtin"), list ("prtout"),
              list ("part_out", "part_auto", "part_strat", "part_args"));
cb_cfgAddTool("fit", list ("tt2"), list ("tt3"),
              list ("device", "fit_run", "fit_pin", "fit_args"));
cb_cfgAddTool("fuseasm", list ("tt3"), list ("jedec"),
              list ("device", "jed_chksum", "jed_docfmt", "jed_plcc",
                    "jed_blow", "jed_turbo", "jed_miser", "jed_lock",
                    "jed_xval", "jed_fmt", "jed_ues"));
cb_cfgAddTool("timfuse", list ("tt3"), list ("timjed"),
              list ("device", "jed_chksum", "jed_docfmt", "jed_plcc",
                    "jed_blow", "jed_turbo", "jed_miser", "jed_lock",
                    "jed_xval", "jed_fmt", "jed_ues", "jed_out"));
cb_cfgAddTool("j2vhdl", list ("jedec"), list ("vhdl", "vli"),
              list ("device", "part", "j2v_delay", "j2v_utf"));
cb_cfgAddTool("timj2v", list ("timjed"), list ("timvhd", "timvli"),
              list ("device", "part", "j2v_delay", "j2v_utf", "jed_out"));
cb_cfgAddTool("devsel", list ("tt2"), list ("chip", "sel"),
              list ("device", "sel_man", "sel_temp", "sel_tech", "sel_pkg",
                    "sel_jam", "sel_era", "sel_pins", "sel_spd", "sel_set",
                    "sel_hold", "sel_cost", "sel_pow", "sel_u1", "sel_sort"));
cb_cfgAddTool("jed2ahdl", list ("jedec"), list ("abel"), list ("device"));
cb_cfgAddTool("wir2abl", list ("wir"), list ("abel"), nil);

/* The following "pseudo-tools" are defined:
 * loadvsm: sends ITC message to viewsim to load new vsm file
 * loadprtvsm: sends ITC message to viewsim to load partition vsm file
 * savesrc: sends ITC message to viewdraw to save schematic if modified
 * saveprt: sends ITC message to viewdraw to save partitioning schematic
 * drawprt: sends ITC message to viewdraw to redisplay partitioning schematic
 * drawsch: sends ITC message to viewdraw to redisplay selected schematic
 * loadprt: reloads partitioning schematic
 */
cb_cfgAddTool("loadvsm", nil, nil, nil);
cb_cfgAddTool("loadprtvsm", nil, nil, nil);
cb_cfgAddTool("savesrc", nil, nil, nil);
cb_cfgAddTool("saveprt", nil, nil, nil);
cb_cfgAddTool("drawprt", nil, nil, nil);
cb_cfgAddTool("drawsch", nil, nil, nil);
cb_cfgAddTool("loadprt", nil, nil, nil);

/*** TOOL CHAIN DEFINITIONS ***/



/*
 * If the top level is a schematic, it is run last.
 * Using "%target" in chain tool list means that all sub-elements will be
 * updated to that target, then remaining tools will be run.
 */

/* From arbitrary schematic */
cb_cfgAddChain("symbol",  "sch", list("savesrc", "vg_sym"));
cb_cfgAddChain("funcsim", "sch", list("savesrc", "%funcsim", "vsm",
                                      "loadvsm"));
cb_cfgAddChain("fpganet", "sch", list("savesrc", "%fpganet"));
cb_cfgAddChain("devsel",  "sch", list("savesrc", "%dio", "wir2abl",
                                      "ahdl2pla", "plaopt", "devsel"));
cb_cfgAddChain("partsch", "sch", list("savesrc", "%dio", "wir2abl",
                                      "ahdl2prt", "plaopt", "autopart", 
                                      "prt2sch", "prtcheck", "loadprt", 
                                      "drawprt"));
cb_cfgAddChain("jedec",   "sch", list("savesrc", "%dio", "wir2abl",
                                      "ahdl2pla", "plaopt", "fit", 
                                      "fuseasm"));
cb_cfgAddChain("timing",  "sch", list("savesrc", "%dio", "wir2abl",
                                      "ahdl2pla", "plaopt", "fit", "fuseasm",
                                      "p2v_pcb", "j2vhdl", "loadvsm")); 
cb_cfgAddChain("jedsch",  "sch", list("%jedec"));
cb_cfgAddChain("timsch",  "sch", list("%timing", "prt2sch", "prtvsm", 
                                      "loadprt", "drawprt", "loadvsm")); 

/* From DIO */
cb_cfgAddChain("symbol",  "dio", list("savesrc","vg_sym"));
cb_cfgAddChain("funcsim", "dio", list("vsm", "loadvsm"));
cb_cfgAddChain("fpganet", "dio", list("wir2abl", "ahdl2pla", "mergett", 
                                      "p2v", "vdes1_vhdl","vdes2_vhdl", "vdes_viewsim",
                                      "vdes_viewgen"));
cb_cfgAddChain("partsch", "dio", list("wir2abl", "ahdl2prt", "plaopt", 
                                      "saveprt", "sch2prt", "autopart", 
                                      "prt2sch", "prtcheck", "loadprt", 
                                      "drawprt"));
cb_cfgAddChain("devsel",  "dio", list("wir2abl", "ahdl2pla", "plaopt", 
                                      "devsel"));
cb_cfgAddChain("jedec",   "dio", list("wir2abl", "ahdl2pla", "plaopt", 
                                      "fit", "fuseasm"));
cb_cfgAddChain("timing",  "dio", list("wir2abl", "ahdl2pla", "plaopt", 
                                      "fit", "fuseasm", "p2v_pcb", "j2vhdl", 
                                      "loadvsm"));

/* From VHDL */
cb_cfgAddChain("symbol",  "vhdl", list("vhdl2sym")); 
cb_cfgAddChain("funcsim", "vhdl", list("vhdl_ana", "loadvsm"));
cb_cfgAddChain("fpganet", "vhdl", list("vdes1_vhdl", "vdes2_vhdl", "vdes_viewsim",
                                       "vdes_viewgen"));
cb_cfgAddChain("partsch", "vhdl", list("vdes1_dio", "vdes2_dio", "wir2abl", "ahdl2prt", 
                                       "plaopt", "saveprt", "sch2prt", 
                                       "autopart", "prt2sch", "prtcheck", 
                                       "loadprt", "drawprt"));
cb_cfgAddChain("devsel",  "vhdl", list("vdes1_dio", "vdes2_dio", "wir2abl", "ahdl2pla", 
                                       "plaopt", "devsel"));
cb_cfgAddChain("jedec",   "vhdl", list("vdes1_dio", "vdes2_dio", "wir2abl", "ahdl2pla", 
                                       "plaopt", "fit", "fuseasm"));
cb_cfgAddChain("timing",  "vhdl", list("vdes1_dio", "vdes2_dio", "wir2abl", "ahdl2pla", 
                                       "plaopt", "fit", "timfuse", "timp2v",
                                       "timj2v", "loadvsm"));
cb_cfgAddChain("dio",     "vhdl", list("vdes1_dio", "vdes2_dio"));
cb_cfgAddChain("diosch",  "vhdl", list("vdes1_dio", "vdes2_dio", "viewgen", "drawsch"));

/* From LPM (like from VHDL) */
cb_cfgAddChain("funcsim", "lpm", list("vhdl_ana", "loadvsm"));
cb_cfgAddChain("fpganet", "lpm", list("vdes1_vhdl", "vdes2_vhdl", "vdes_viewsim",
                                      "vdes_viewgen"));
cb_cfgAddChain("partsch", "lpm", list("vdes1_dio", "vdes2_dio", "wir2abl", "ahdl2prt", 
                                      "plaopt", "saveprt",
                                      "sch2prt", "autopart", 
                                      "prt2sch", "prtcheck", "loadprt", 
                                      "drawprt"));
cb_cfgAddChain("devsel",  "lpm", list("vdes1_dio", "vdes2_dio", "wir2abl", "ahdl2pla", 
                                      "plaopt", "devsel"));
cb_cfgAddChain("jedec",   "lpm", list("vdes1_dio", "vdes2_dio", "wir2abl", "ahdl2pla", 
                                      "plaopt", "fit", "fuseasm"));
cb_cfgAddChain("timing",  "lpm", list("vdes1_dio", "vdes2_dio", "wir2abl", "ahdl2pla", 
                                      "plaopt", "fit", "timfuse", "timp2v",
                                      "timj2v", "loadvsm"));
cb_cfgAddChain("dio",     "lpm", list("vdes1_dio", "vdes2_dio"));
cb_cfgAddChain("diosch",  "lpm", list("vdes1_dio", "vdes2_dio", "viewgen", "drawsch"));

/* From ABEL */
cb_cfgAddChain("symbol",  "abel", list("ahdl2pla", "mergett", "p2v_sym"));
cb_cfgAddChain("funcsim", "abel", list("ahdl2pla", "mergett", "p2v_vli", 
                                       "loadvsm"));
cb_cfgAddChain("fpganet", "abel", list("ahdl2pla", "mergett", "p2v", 
                                       "vdes1_vhdl", "vdes2_vhdl", "vdes_viewsim", 
                                       "vdes_viewgen"));
cb_cfgAddChain("partsch", "abel", list("ahdl2prt", "plaopt", "saveprt", 
                                       "sch2prt", "autopart", "prt2sch", 
                                       "prtcheck", "loadprt", "drawprt"));
cb_cfgAddChain("devsel",  "abel", list("ahdl2pla", "plaopt", "devsel"));
cb_cfgAddChain("jedec",   "abel", list("ahdl2pla", "plaopt", "fit", 
                                       "fuseasm"));
cb_cfgAddChain("timing",  "abel", list("ahdl2pla", "plaopt", "fit", 
                                       "fuseasm", "p2v_pcb", "j2vhdl", 
                                       "loadvsm"));
cb_cfgAddChain("dio",     "abel", list("ahdl2pla", "mergett", "p2v_dio"));
cb_cfgAddChain("diosch",  "abel", list("ahdl2pla", "mergett", "p2v_diosch",
                                       "drawsch"));

/* From PLA file */
cb_cfgAddChain("symbol",  "partition", list("p2v_sym"));
cb_cfgAddChain("funcsim", "partition", list("p2v_vli", "loadvsm"));
cb_cfgAddChain("fpganet", "partition", list("p2v","vdes1_vhdl", "vdes2_vhdl",
                                            "vdes_viewsim", "vdes_viewgen"));
cb_cfgAddChain("partsch", "partition", list("p2a", "ahdl2prt", "plaopt", 
                                            "autopart","prt2sch","prtcheck",
                                            "loadprt","drawprt"));
cb_cfgAddChain("devsel",  "partition", list("devsel"));
cb_cfgAddChain("jedec",   "partition", list("fit", "fuseasm"));
cb_cfgAddChain("timing",  "partition", list("fit", "fuseasm", "p2v_pcb",
                                            "j2vhdl"));
cb_cfgAddChain("dio",     "partition", list("p2v_dio"));
cb_cfgAddChain("diosch",  "partition", list("p2v_diosch","drawsch"));

/* From JEDEC */
cb_cfgAddChain("symbol",  "jedec", list("jed2ahdl", "ahdl2pla", "mergett",
                                        "p2v_sym"));
cb_cfgAddChain("funcsim", "jedec", list("jed2ahdl", "ahdl2pla", "mergett",
                                        "p2v_vli", "loadvsm"));
cb_cfgAddChain("fpganet", "jedec", list("jed2ahdl", "ahdl2pla", "mergett", 
                                        "p2v", "vdes1_vhdl", "vdes2_vhdl", "vdes_viewsim",
                                        "vdes_viewgen"));
cb_cfgAddChain("partsch", "jedec", list("jed2ahdl", "ahdl2prt", "plaopt", 
                                        "autopart", "prt2sch", "prtcheck", 
                                        "loadprt", "drawprt"));
cb_cfgAddChain("devsel",  "jedec", list("jed2ahdl", "ahdl2pla", "plaopt", 
                                        "devsel"));
cb_cfgAddChain("jedec",   "jedec", list(""));
cb_cfgAddChain("timing",  "jedec", list("j2vhdl", "p2v_pin", "loadvsm"));
cb_cfgAddChain("dio",     "jedec", list("jed2ahdl", "ahdl2pla", "mergett",
                                        "p2v_dio"));
cb_cfgAddChain("diosch",  "jedec", list("jed2ahdl", "ahdl2pla", "mergett",
                                        "p2v_diosch", "drawsch"));


