/* This file contains the following major sections:
   - status window functions
   - functions to grey and ungrey menus
   - ITC function for VSM load
   - functions for utility menu
*/


/*** STATUS WINDOW FUNCTIONS ***/


/* Because of the 5.1.1 core bug which makes setpick (-1) do nothing, this
   function must delete and rebuild the picklist instead.  The global variable
   g_ispartsel is used to make this rebuilding happen only when needed.  That
   global is also passed in requests to the loader to distinguish between
   source and partition elements with the same name */
g_ispartsel := FALSE;
function statSelect (name, update, ispart) {
   if (ispart) row := 1; else row := 0;

   /* Redisplay the other picklist if needed */
   if (g_ispartsel != ispart) {
      if (row == 1) statMakepick (cb_loadSrcEls (), FALSE);
      else statMakepick (cb_loadPrtEls (), TRUE); }
   g_ispartsel = ispart;

   /* Update the value in the picklist */
   if (update) {
      r := vcTable.Get (vcTable.Get (gr_stat, 1, 0), row, 1);
      if (vcRegion.Fac (r) == vcPicklist.fac)
         vcPicklist.SetByString (r, name); }

   /* Update the selected element and type static text fields */
   g_selel = name;
   r_text := vcTable.Get (vcTable.Get (gr_stat, 0, 0), 0, 3);
   vcStaticText.Clear (r_text); vcStaticText.Add (r_text, name);
   r_text = vcTable.Get (vcTable.Get (gr_stat, 0, 0), 0, 5);
   vcStaticText.Clear (r_text);
   type := cb_loadGettype (name, g_ispartsel);
   vcStaticText.Add (r_text, type);
   menuSettype (type); 

   /* Send scomp message to ViewDraw if type is not schematic,
      if it is not the top-level source file, and only if we're
      here because of the notify routine. */
   if ((!string_eqi(substring (type, 0, 3), "sch")) &&
       (!cb_loadIstop (name)) && !update)
       cb_itcScomp (name);

}


/* Compute size of list and arrange list structure; display picklist */
function statMakepick (l, ispart) {

   pref_name := car (l); l_orig := cdr (l);

   /* Compute number of rows and columns */
   n_orig := length (l_orig) + 1;
   n_rows := ((n_orig - 1) / 10) + 1;
   if (n_rows < 2) n_cols := n_orig;
   else n_cols :=  10;

   /* Sort list; then prefix name */
   if (n_orig == 1) l_sort := list (pref_name);
   else l_sort := reverse (cons (pref_name, vscript.Sort (l_orig, string_lt)));

   /* Build list structure with correct number of rows and columns*/
   declare l_out, l_int, i, j, k, s;
   for (l_out=nil, i=0, k=0; i<n_rows; ++i) {
      for (l_int=nil, j=0; j<n_cols; ++j, ++k, l_sort=cdr(l_sort)) {
         if (null (car (l_sort)))
            break;
         s = car (l_sort);
         l_int = cons (cons (s, s), l_int); }
      if (!null(l_int)) l_out = cons (l_int, l_out); }

   /* Create the picklist and display it */
   r_pick := vcRegion.Create (ispart ? "r_prtpick":"r_srcpick", vcPicklist.fac,
      l_out, rows <- n_rows, columns <- n_cols, macro_on <- TRUE);
   if (ispart) row := 1; else row := 0;
   vcPicklist.Notify (r_pick, func (d, onoff)
      if (onoff) statSelect (d, FALSE, ispart););
   vcTable.Set (vcTable.Get (gr_stat, 1, 0), row, 1, r_pick); }


function statBlank () {
   vcTable.Set (vcTable.Get (gr_stat, 1,0), 0,1, vcSteel.Create ("", 0, 0));
   vcTable.Set (vcTable.Get (gr_stat, 1,0), 1,1, vcSteel.Create ("", 0, 0));
   vcTable.Set (vcTable.Get (gr_stat, 1,0), 1,0, vcSteel.Create  ("", 0, 0));
   vcStaticText.Clear (vcTable.Get (vcTable.Get (gr_stat, 0,0), 0,1));
   vcStaticText.Clear (vcTable.Get (vcTable.Get (gr_stat, 0,0), 0,3));
   vcStaticText.Clear (vcTable.Get (vcTable.Get (gr_stat, 0,0), 0,5));
   vcRegion.Refresh (gr_stat, vcRegion.VisibleArea (gr_stat)); }


function statDisplay () {

   l_src := cb_loadSrcEls ();
   if (!null (l_src)) {
       /* Display picklist region for source elements and top level name */
       r_src := statMakepick (l_src, FALSE);
       top_name := car (l_src);
       statSelect (top_name, TRUE, FALSE);
       r_text := vcTable.Get (vcTable.Get (gr_stat, 0, 0), 0, 1);
       vcStaticText.Clear (r_text); vcStaticText.Add (r_text, top_name);

       /* If used, display picklist region for partition elements, 
          else blank it */
       l_part := cb_loadPrtEls ();
       if (null (l_part)) {
           vcTable.Set(vcTable.Get(gr_stat, 1,0), 1,1, vcSteel.Create("",0,0));
           vcTable.Set(vcTable.Get(gr_stat, 1,0), 1,0, vcSteel.Create("",0,0));
       }
       else {
           statMakepick (l_part, TRUE);
           vcTable.Set (vcTable.Get (gr_stat, 1,0), 1,0,
                        vcRegion.Create ("", vcLabel.fac, 
                                         "Partition elements:")); 
       }
       vcRegion.Refresh (gr_stat, vcRegion.VisibleArea (gr_stat)); 
   }
   else
       statBlank ();
}


function statCreate () {

   declare r_upper;
   function add2 (title) {
      vcTable.Add (r_upper, vcRegion.Create ("", vcLabel.fac, title));
      vcTable.Add (r_upper, vcRegion.Create ("", vcStaticText.fac, "",
         width <- 1)); }

   /* Create three groups with label, static text */
   r_upper = vcTable.Create ("r_upper", 1, 6);
   add2 ("Top level name:    ");
   add2 ("Selected element: ");
   add2 ("Element type: ");

   /* Create two by three array, with rubber to the right */
   r_lower := vcTable.Create ("r_lower", 2, 3);
   vcTable.Set (r_lower, 0, 0, vcRegion.Create ("", vcLabel.fac,
      "Source elements:"));
   vcTable.Set (r_lower, 0, 1, vcSteel.Create ("", 0, 0));
   vcTable.Set (r_lower, 1, 0, vcSteel.Create ("", 0, 0));
   vcTable.Set (r_lower, 1, 1, vcSteel.Create ("", 0, 0));
   vcTable.Set (r_lower, 0, 2, vcRubber.Create ("", 0, 0));
   vcTable.Set (r_lower, 1, 2, vcRubber.Create ("", 0, 0));
   vcTable.Col (r_lower, -1, vcTable.Border.Invisible, 20, 20, 20,
      vgAlign.MiddleLeft);

   /* Attach regions together and return top level */
   gr_stat = vcTable.Create ("gr_stat", 2, 1);
   vcTable.Add (gr_stat, r_upper);
   vcTable.Add (gr_stat, r_lower);
   gr_stat; }


function statDoload (name, type, ispart) {
   function doload () {
      retcode := cb_loadDoload (name, type, ispart);
      if (!null (gr_log)) {
         menuFileload (retcode);
         statDisplay (); } }
   if (!null (gr_log))
      vcWin.WithBusy (vcRegion.Window (gr_log), doload);
   else doload(); }


/*** MENU FUNCTIONS ***/


function menuEnableList (onoff, msg, cmdlist) {
   for (l:=cmdlist; !null(l); l=cdr(l))
      vcCmd.Enable (car (l), onoff);
   if (onoff == vlOnOff.Off)
      for (l=cmdlist; !null(l); l=cdr(l)) {
         vcCmd.SetDisabledMsg (car (l), msg); } }


g_isvenvalid := vlSym.StrCreate (7);
function menuEnableVenlist (name, onoff, msg, cmdlist) {
   if (vlSym.Fetch (g_isvenvalid, name) == vlOnOff.On)
      menuEnableList (onoff, msg, cmdlist); }


g_validtypelist := nil;
function menuValidTypes (name, typelist, cmdlist)
   g_validtypelist = cons (list (name, typelist, cmdlist), g_validtypelist);


function menuFileload (onoff) {
   declare name, suptlist, cmdlist, found, supported, msg, isvalid, m, n, o;
   typelist := cb_loadTypelist ();
   s := "Use the file menu to load a file first";

   menuEnableList (onoff, s, list (
      c_file_edit, c_cre_sym, c_cre_func, c_cre_wir,
      c_opt_ahdl2pla, c_opt_plaopt, c_opt_vhdl, c_opt_synth));

   /* Loop through the valid type triples supplied by vendors */
   for (m=g_validtypelist; !null(m); m=cdr(m)) {
      name = caar (m); suptlist = cadar (m); cmdlist = caddar (m);

      /* Perform one operation on cmdlist.  If load failed or suptlist is nil,
         grey it.  Loop through types in design.  If one is not supported,
         grey cmdlist.  If all were supported, ungrey cmdlist. */
      if (null (suptlist)) {
         isvalid = vlOnOff.Off; msg = name + " environment not available"; }
      else if (onoff == vlOnOff.Off) { isvalid = vlOnOff.Off; msg = s; }
      else {
         for (supported=TRUE, n=typelist; !null(n); n=cdr(n)) {
            for (found=FALSE, o=suptlist; !null(o); o=cdr(o))
               if (string_eq (car (n), car (o))) { found = TRUE; break; }
            if (!found) { supported = FALSE; break; } }
          if (supported) { msg = ""; isvalid = vlOnOff.On; }
          else {
             isvalid = vlOnOff.Off;
             msg = "Type " + car (n) + " not supported for " + name; } }
      vlSym.Store (g_isvenvalid, name, isvalid);
      menuEnableList (isvalid, msg, cmdlist); }

   if (!null (gr_stat))
      vcWin.UpdateMenus (vcRegion.Window (gr_stat)); }


function menuSettype (type) {
   function setonoff (type, onlist, offlist) {
      menuEnableList (vlOnOff.On, "", onlist);
      menuEnableList (vlOnOff.Off, "Not permitted on " + type + " file",
         offlist); }

   /* Special dynamic processing for ABEL and VHDL languages */
   if (string_eq (type, "ABEL")) setonoff ("ABEL",
      list (c_opt_ahdl2pla, c_opt_plaopt),
      list (c_opt_vhdl));
   else if (string_eq (type, "PLA")) setonoff ("PLA",
      list (c_opt_ahdl2pla, c_opt_plaopt),
      list (c_opt_vhdl));
   else if (string_eq (type, "JEDEC")) setonoff ("JEDEC",
      list (c_opt_ahdl2pla, c_opt_plaopt),
      list (c_opt_vhdl));
   else if (string_eq (type, "VHDL")) setonoff ("VHDL",
      list (c_opt_vhdl),
      list (c_opt_ahdl2pla, c_opt_plaopt));
   else if (string_eq (type, "LPM")) setonoff ("LPM",
      nil,
      list (c_opt_vhdl, c_opt_ahdl2pla, c_opt_plaopt));


   /* Special processing for vendor routines */
   actSettype (type); altSettype (type); xilSettype (type);

   vcWin.UpdateMenus (vcRegion.Window (gr_stat)); }


/* Used to create submenus dynamically */
function menuSubcreate (title, maskval, mainmenu, name, sublist) {
   if (g_menumask == maskval) {
      for (; !null(sublist); sublist=cdr(sublist))
         vcMenu.AppendEntry (mainmenu, caar (sublist), cadar (sublist)); }
   else if ((g_menumask & maskval) == maskval) {
      submenu := vcMenu.Create ("m_" + title + "_" + name, nil);
      for (; !null(sublist); sublist=cdr(sublist))
         vcMenu.AppendEntry (submenu, caar (sublist), cadar (sublist));
      vcMenu.AppendEntry (mainmenu, name, submenu); } }


/* Function to tell if pldonly mode is set: note this is called from C! */
function menuIsPLDonly () { (g_menumask == vfpgaMenuMask.PLD); }

/* Related functions, not called from C */
function menuIsFPGAonly () { (g_menumask == vfpgaMenuMask.FPGA); }

function menuHasFPGA () {
   ((g_menumask & vfpgaMenuMask.FPGA) == vfpgaMenuMask.FPGA); }

function menuHasPLD () {
   ((g_menumask & vfpgaMenuMask.PLD) == vfpgaMenuMask.PLD); }


/*** UTILITY MENU COMMANDS AND FUNCTIONS ***/


c_util_diosch := vcWv.Cmd (fac,
   func (d, e) cb_makeDochain ("diosch", g_selel, nil);,
   "CreateLogSch", "Create logical schematic");


/*** TUTORIAL FUNCTIONS ***/


function menuCopyTutFiles (alias_list, copy_list, delete_list) {

    declare path;

    function copyfiles () {
        found_all := TRUE;
        for (l:=copy_list; !null (l); l=cdr(l)) {
            dest := vscript.vsConvPath (car (l));
            if (!null (src := vscript.vsConvPath (path + dest))) 
                vscript.vsCopy (src, dest); 
            else
                found_all = FALSE;
        } 
        if (found_all)
            cb_mainErr (0, "Status", nil, "Tutorial files copied");
        else
            cb_mainErr (1207, "Error", nil, "Failed to find tutorial files");
    }

    function setup () {
        /* Check to make sure alias exists */
        found_all := TRUE;
        for (; !null (alias_list); alias_list = cdr (alias_list))
            if (!cb_loadIfalias (car (alias_list), TRUE))
                found_all = FALSE;

        /* Check to make sure tutorial directory exists */
        path = vscript.vsSrchdirs (vscript.vsConvPath ("../tutor/viewfpga/"),
                                   vscript.vsDIR);
        if (null (path))
            cb_mainErr (1206, "Error", nil,
                        "Failed to find installed tutorial directory");
        else if (found_all) {
            /* Create list of files to delete */
            for (actual_del:=nil, l:=delete_list; !null (l); l=cdr(l)) {
                f := vscript.vsConvPath (car (l));
                if (fexists (f)) actual_del = cons (f, actual_del); }
            for (l=copy_list; !null (l); l=cdr(l)) {
                f := vscript.vsConvPath (car (l));
                if (fexists (f)) actual_del = cons (f, actual_del); }

            /* Confirm deletion, then continue setup */
            n := length (actual_del);
            if (n == 0) copyfiles();
            else vfpgaConfirmFiles (actual_del, "CopyTutConfirm", copyfiles); 
        }
    }
    vcWin.WithBusy (vcRegion.Window (gr_log), setup); 
}

c_util_diotut := vcWv.Cmd (fac,
   func (d, e) menuCopyTutFiles (list ("dio"),
      list ("binbcd.abl", "allbjack.abl", "accum.abl", "reg5.abl",
         "sym/reg5.1", "sym/accum.1", "sch/muxpart.1"),
      list ("binbcd.dmc", "binbcd.tmv", "binbcd.bl0", "binbcd.xrf",
         "binbcd.tt2", "binbcd.tt3", "binbcd.jed", "allbjack.dmc",
         "allbjack.tmv", "allbjack.bl0", "allbjack.xrf", "allbjack.tt2",
         "allbjack.pr0", "allbjack.prt", "pldtut1.tt2", "pldtut2.tt2",
         "pldtut3.tt2", "sym/pldtut1.1", "sym/pldtut2.1", "sym/pldtut3.1",
         "accum.bl0", "accum.xrf", "accum.tt2", "accum.dmc", "accum.vhd", 
         "sch/accum.1", "sym/accum.pin", "wir/accum.1", "reg5.bl0", 
         "reg5.xrf", "reg5.tt2", "reg5.dmc", "reg5.vhd", "sch/reg5.1", 
         "sym/reg5.pin", "wir/reg5.1"));,
   "CopyDIOTut", "Copy files for online PLD tutorial");


/*** DEVICE HELP ROUTINES ***/


g_helplist := nil;
g_help_win := nil;
g_help_win_is_hidden := FALSE;
function f_util_pldpins (d, e) {
   declare t_top;

   function helpshow (s) {
      slist := CallShowhelp(s);
      t_temp := vcTable.Get (t_top, 0, 1);
      for (i:=0; !null(slist); slist=cdr(slist),i=i+1) {
         r_temp := vcTable.Get (t_temp, i, 0);
         vcStaticText.Clear (r_temp);
         vcStaticText.Add (r_temp, car (slist)); }
      for (; i<60; i=i+1) vcStaticText.Clear (vcTable.Get (t_temp, i, 0)); }

   function helpick (wildcard) {
      r_helpick := vcPicklist.Scroll ("helpscroll",
         vcRegion.Create1 ("helpick", vcPicklist.fac,
            CallWildcard (wildcard, g_helplist, 1), list (macro_on <- TRUE)),
         15, 15, 15);
      vcPicklist.Notify (r_helpick, func (d, o) {
         if (o == vlOnOff.On) { 
             vcCmdline.set (t_typein, d); helpshow (d); } });
      r_helpick; }

   function helpreturn (e, data) {
      if ((lindex (data, '*') != FALSE) || (lindex (data, '?') != FALSE)) {
         vcTable.Set (vcTable.Get (t_top, 0, 0), 2, 0, helpick (data)); }
      else helpshow (data); }

   /* Function to create a window */

   win := vcRegion.Window (gr_log);
   vcWin.Busy (win, TRUE);
   vcCmd.Enable (c_util_pldpins, vlOnOff.Off);
   vcWin.UpdateMenus (win);

   if (null (g_helplist)) g_helplist = CallReadhelp ();

   t_top    = vcTable.Create ("help_most", 1, 2);
   t_left   := vcTable.Create ("help_left", 3, 1);
   t_right  := vcTable.Create ("help_right", 60, 1);
   t_typein := vcRegion.Create1 ("helptype", vcCmdline.fac, "",
      list (macro_on <- TRUE));
   vcCmdline.CallOnReturn (t_typein, helpreturn);
   vcTable.Add (t_left, vcRegion.Create1 ("helptitle", vcStaticText.fac,
      "Select Device:", nil));
   vcTable.Add (t_left, t_typein);
   vcTable.Add (t_left, helpick ("*"));

   for (i:=0; i<60; i=i+1) {
      r_temp := vcRegion.Create1("", vcStaticText.fac, "", list (width <- 60));
      vcRegion.Font (r_temp, vgFont.smallf);
      vcTable.Add (t_right, r_temp); }

   vcTable.Add (t_top, t_left);
   t_new := vcTable.Scroll ("fds", t_right, t_right, 24, 24, 24);
   vcTable.Add (t_top, vcRegion.Create("help_scroll", vcScroll.fac, t_new, 
                                       horizontal <- 1, vertical <- 1));
   vcTable.Col (t_top, 1, vcTable.Border.Visible, 1,1,1, vgAlign.MiddleCenter); 
   t_all := vcTable.Create ("Device Help", 2, 1);
   vcTable.Add (t_all, t_top);
   vcTable.Add (t_all, vcRegion.Create ("", vcButnRow.fac, list (
      list ("Dismiss", "Dismiss", func () {
         vcCmd.Enable (c_util_pldpins, vlOnOff.On);
         vcWin.UpdateMenus (win);
         g_help_win_is_hidden = TRUE;
         vcWin.Hide (g_help_win); },
         list (close_window <- FALSE)),
      list ("Help", "Help",
         func () vcAlert.SpawnHelp (win,  "vfpga", "PLDPins");,
         list (close_window <- FALSE))), macro_on <- TRUE));
   g_help_win = vcWin.Create (win, t_all, 
                              vcWin.NoMenus+vcWin.NoCmdline+vcWin.NoStatus);
   vcWin.CallOnClose (g_help_win, 
                      func(w) { vcCmd.Enable(c_util_pldpins, vlOnOff.On);
                                vcWin.UpdateMenus (win);
                                g_help_win_is_hidden = TRUE;
                                vcWin.Hide(w);});
/*
   vcWin.DestroyNotify(hw, func(w) { vcCmd.Enable(c_util_pldpins, vlOnOff.On);
                                     vcWin.UpdateMenus (win);}); */
   vcWin.Busy (win, FALSE); }


c_util_pldpins := vcWv.Cmd (fac, 
                            func (d,e) {
                                if ((g_help_win_is_hidden == FALSE) || 
                                    (g_help_win == nil))
                                    f_util_pldpins (d, e);
                                else {
                                    g_help_win_is_hidden = FALSE;
                                    vcWin.Show (g_help_win); } },
                            "ShowPLDPin", "Show PLD pinouts");


/*** PROGRAMMER ROUTINES ***/

function f_util_burndev (d, e) {
   win := vcRegion.Window (gr_log);
   vcWin.Busy (win, TRUE);
   vcCmd.Enable (c_util_burndev, vlOnOff.Off);
   vcWin.UpdateMenus (win);

   t_top := vcTable.Create ("Device programmer", 2, 1);
   if ((vcWv.UI(wv) == vcUi.Motif) || (vcWv.UI(wv) == vcUi.OpenLook))
       cmdstr := "tip unisite";
   else
       cmdstr := "abelprog";
   vcTable.Add (t_top, vcRegion.Create1 ("", vcShell.fac, cmdstr, nil));
   vcTable.Add (t_top, vcRegion.Create ("", vcButnRow.fac, list (
      list ("Dismiss", "Dismiss", func () {
         vcCmd.Enable (c_util_burndev, vlOnOff.On);
         vcWin.UpdateMenus (win); },
         list (close_window <- TRUE)),
      list ("Help", "Help",
         func () vcAlert.SpawnHelp (win,  "vfpga", "ProgWindow");,
         list (close_window <- FALSE)))));

   pw := vcWin.Create (t_top, 
                       vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
   vcWin.DestroyNotify(pw, func(w) { vcCmd.Enable(c_util_burndev, vlOnOff.On);
                                     vcWin.UpdateMenus (win);});
   vcWin.Busy (win, FALSE); }


c_util_burndev := vcWv.Cmd (fac, f_util_burndev, "ProgWindow",
                            "Programmer window");


/*** CLEANUP ROUTINES ***/


function p_delint (name, help, default) {
   vcWv.Param (name, vlYesNoVT, vcWv.Sticky, default, "Delete " + help); }


function f_util_delint (d, e, name, extlist) {

   declare files;
   function oneloop (ext) {
      path := vscript.vsMerge ("", "", name, ext);
      foreach (file; vscript.TraverseDir (path, vscript.vsFILE)) {
         flist := vscript.vsParse (file);
         fname := vscript.vsMerge ("", "", caddr (flist), cadddr (flist));
         files = cons (fname, files); } }

   files = nil;
   if (!string_eq(name, ""))
       foreach (token; string_loop (extlist, " ()\"")) oneloop (token);
   n := length (files);
   if (n == 0) cb_mainErr (1204, "Error", nil,
      "No intermediate files specified");
   else vfpgaConfirmFiles (files, "DelIntFilesConfirm", func(); ); }


function fl_delint () { list (
   list (cons ("ABL", "abl"), cons ("BAK", "bak"), cons ("BL0", "bl0")),
   list (cons ("CHP", "chp"), cons ("CMD", "cmd"), cons ("DMC", "dmc")),
   list (cons ("FIT", "fit"), cons ("JED", "jed"), cons ("LIS", "lis")),
   list (cons ("ORD", "ord"), cons ("PIN", "pin"), cons ("PR0", "pr0")), 
   list (cons ("PR1", "pr1"), cons ("PRT", "prt"), cons ("SEL", "sel")), 
   list (cons ("TIM", "tim"), cons ("TMV", "tmv"), cons ("TT2", "tt2")),
   list (cons ("TT3", "tt3"), cons ("VNT", "vnt"), cons ("XRF", "xrf"))); }


function fl_delfpga () { list (
   list (cons ("LIS", "lis"), (cons ("VLI", "vli")))); }


function cg_util_delint (name, title, extlist, initval) {
   vcWv.Cmd (fac, f_util_delint, name, title,
      vcWv.Param ("Filter", vlStringVT, vcWv.Sticky, "",
         "Enter the filename or wildcard for designs"),
      vcWv.Param ("Extensions", vlStringVT, vcWv.Sticky, initval,
         vfpgaVertPick.fac, "Select the extensions to be deleted",
         funclist <- extlist, exclusive <- FALSE)); }


c_util_delint := cg_util_delint ("DelIntFiles",
   "Delete PLD intermediate files", "fl_delint",
   "bak bl0 chp dmc fit lis ord pin pr0 prt sel tim tmv tt2 tt3 vnt xrf");


c_util_fpgaint := cg_util_delint ("DelIntFilesFPGA",
   "Delete FPGA intermediate files", "fl_delfpga",
   "lis vli");


if ((vcWv.UI(wv) != vcUi.Motif) && (vcWv.UI(wv) != vcUi.OpenLook)) {
    vcCmd.Enable (vcWin.system_cmd, vlOnOff.Off);
    vcCmd.Enable (c_util_burndev, vlOnOff.Off);
}
