/**
  *
  *    VFRAME.VS
  *
  *    Facility:  Viewframe
  *
  *    Functional Description: 
  *       Vframe viewscript functions, commands
  *
  *    Environment:
  *        SUNOS 4.1.1
  *
  *    Author:
  *        Bill Holbrook -- September 1991
  *
  *    Modified by:
  *    Who            Version   When     What
  *    SAL            5.2       07/07/93 Changed create_picklist_scroll to call
  *                                      vcPicklist.Dimension
  *    BH             5.0       2/26/92  Added enable_cfilog_command,        
  *    BH             5.0       2/23/92  Added editenv, edittef commands     
  *    SAL            5.0       1/14/92  Added enable_proj_commands, 
  *                                      enable_lib_commands
  *    SAL            5.0       12/16/91 Added call to disable plotgo command 
  *    BH             5.0       3/27/92  Added enable_activetool_command  
  *
  *    Copyright (c) 1991 by Viewlogic Systems, Inc.
  *
  **/

declare main_menu;

dm_ref_count := 0;

vcQuit.CallOnQuit(cockpit_quit);

function disable_changeproj_commands(data)
{
  ++dm_ref_count;
  if (dm_ref_count == 1) {
     vcCmd.Enable(project, FALSE);
     vcCmd.Enable(pdelete, FALSE);
     vcCmd.Enable(premove, FALSE);
     vcCmd.Enable(pmove, FALSE);
     vcCmd.Enable(pcreate, FALSE);
     vcCmd.Enable(pdcreate, FALSE);
     vcCmd.Enable(ppcreate, FALSE);
     vcCmd.Enable(psdraw, FALSE);
     vcCmd.Enable(psplace, FALSE);
     vcCmd.Enable(psearch, FALSE);
     disableprojselect();
  }
}

function enable_changeproj_commands(data)
{
  --dm_ref_count;
  if (dm_ref_count == 0) {
     vcCmd.Enable(project, TRUE);
     vcCmd.Enable(pdelete, TRUE);
     vcCmd.Enable(premove, TRUE);
     vcCmd.Enable(pmove, TRUE);
     vcCmd.Enable(pcreate, TRUE);
     vcCmd.Enable(pdcreate, TRUE);
     vcCmd.Enable(ppcreate, TRUE);
     vcCmd.Enable(psdraw, TRUE);
     vcCmd.Enable(psplace, TRUE);
     vcCmd.Enable(psearch, TRUE);
     enableprojselect();
  }
}

/***************************/
/* enable project commands */
/***************************/

function enable_proj_commands(on_off) 
{
   vcCmd.Enable(psdraw, on_off);  
   vcCmd.Enable(psplace, on_off);
   vcCmd.Enable(psearch, on_off);  
   vcCmd.Enable(premove, on_off);
   vcCmd.Enable(phierarchy, on_off);
   vcCmd.Enable(drawlib, on_off);
   vcCmd.Enable(placelib, on_off);
   vcCmd.Enable(pdelete, on_off);
   vcCmd.Enable(pdirsch, on_off);
   vcCmd.Enable(pdirsym, on_off);
   vcCmd.Enable(pdirboa, on_off);
   vcCmd.Enable(pdirsha, on_off);
   vcCmd.Enable(pdiroth, on_off);
   vcCmd.Enable(parchive, on_off);
   vcCmd.Enable(pcopy, on_off);
   vcCmd.Enable(pmove, on_off);
   vcCmd.Enable(pltsch, on_off);
   vcCmd.Enable(pltsym, on_off);
   vcCmd.Enable(pltboa, on_off);
   vcCmd.Enable(pltoat, on_off);
   vcCmd.Enable(schdelete, on_off);
   vcCmd.Enable(symdelete, on_off);

}

/****************************************/
/*    enable project select command     */
/****************************************/

function enable_projsel_command(on_off) 
{
   vcCmd.Enable(project, on_off);  
}


/****************************************/
/* enable tool commands */
/****************************************/

/*function enable_tool_commands(on_off) 
{
   vcCmd.Enable(helpcurtool, on_off);  
   vcCmd.Enable(desccurtool, on_off);  
   vcCmd.Enable(toolcopy, on_off);  
   vcCmd.Enable(toolmove, on_off);  
   vcCmd.Enable(tooldel, on_off);  
}
*/


/****************************************/
/* enable sel active tool command */
/****************************************/

function enable_activetool_command(on_off) 
{
   vcCmd.Enable(selactivetool, on_off);  
}

/****************************************/
/* enable tool start command */
/****************************************/

function enable_toolstart_command(on_off) 
{
   vcCmd.Enable(startcurtool, on_off);  
}

/****************************************/
/* enable tool stop command */
/****************************************/

function enable_toolstop_command(on_off) 
{
   vcCmd.Enable(stopcurtool, on_off);  
}

/****************************************/
/* enable stopalltools command */
/****************************************/

function enable_stopalltools_command(on_off) 
{
   vcCmd.Enable(stopalltools, on_off);  
}

/****************************************/
/* enable library select command */
/****************************************/

function enable_libsel_command(on_off) 
{
   vcCmd.Enable(library, on_off);  
}

/****************************************/
/* enable library commands */
/****************************************/

function enable_lib_commands(on_off) 
{
   vcCmd.Enable(ldirsch, on_off);
   vcCmd.Enable(ldirsym, on_off);
   vcCmd.Enable(ldirboa, on_off);
   vcCmd.Enable(ldirsha, on_off);
   vcCmd.Enable(lcopy, on_off);
}

/****************************************/
/* disable the 'plotgo' command in viewframe */
/****************************************/

vcCmd.Enable (vcWin.plotgo_cmd, vlOnOff.Off);

/****************************************/
/*    enable cfilog ON/OFF  command     */
/****************************************/

function enable_cfilogon_command(on_off) 
{
   vcCmd.Enable(startcfilog, on_off); 
}

function enable_cfilogoff_command(on_off) 
{
   vcCmd.Enable(stopcfilog, on_off); 
}

/****************************************/
/* Viewscript Functions                 */
/****************************************/
/* some handy functions */
function label(str) vcRegion.Create1("",vcLabel.fac,str,nil);
function rubber(x,y) vcRubber.Create("",x,y);
function cmdline(name,default) vcRegion.Create1(name,vcCmdline.fac,default,
                                           list(cons(`width, 20)));
function icon(str) vcRegion.Create1(str,vcIcon.fac,str,nil);

function label_cmd(str,default)
{
  label_cmd_t := vcTable.Create(str,1,2);
  vcTable.Add(label_cmd_t, label(str));
  cmd := cmdline(str,default);
  vcTable.Add(label_cmd_t, cmd);
  return(label_cmd_t);
}

ptype := nil;
pathcmd := nil;
/* build a dynamic picklist of project views */
function build_view_picklist()
{
   count := 0;
   picklist := vcRegion.Create1("", vcPicklist.fac, nil,
                                list (cons (`exclusive, TRUE),
                                      cons (`rows, 5),
                                      cons (`columns, 1)));
   /* for each view of a project add a project type to picklist */
   foreach (str; projview()) {
      vcPicklist.Add(picklist, str, str);
      if (string_eq(str,"Viewdraw"))
         vcPicklist.SetPick(picklist, count);
      count = count + 1;
   }
 
   return(picklist);
}

/* create a scroll around a picklist */
function create_scroll_picklist(picklist,min,max)
{
   declare rows, cols;
   
   vcPicklist.Dimensions(picklist, &rows, &cols);
   if (rows > max)
      rows = max;

   if (rows < min)
      rows = min;
  
   if (rows < max)
      max = rows;

   vcPicklist.Scroll("", picklist, min, rows, max);

}

/* given a picklist pick, set the ptype to the value */
function set_ptype(val, on_off)
{

   if (on_off)
      ptype = val;
     
}

/* create a dialog for project create function */
function Pcreate(data, event)
{
   declare createprojw;

   pcreate_dialog := vcTable.Create("Create Project",5,1);

   /* entry field for project pathname */
   patht := vcTable.Create("Project_path", 1, 2);
   vcTable.Add(patht, label("Project Pathname"));
   vcTable.Add(patht, vcRegion.Create("", vcFilePop.fac, "",
                                      file_columns<-0,
                                      read_now<-TRUE));
   pathcmd = vcTable.Get(patht,0,1);

   vcCmdline.CallOnReturn(pathcmd, 
             func(event, str) {
                path := vcRegion.Get(pathcmd, string);
                vcCmd.Run(_pcreate_cmd, data, event, list(path, ptype),TRUE);
                ptype = nil;
                vcWin.Close(createprojw);
             });

   /* picklist for existing project types */
   ptype = "Viewdraw";
   viewpicklist := build_view_picklist();
   vcPicklist.Notify(viewpicklist, set_ptype);
   scrollpicklist := create_scroll_picklist(viewpicklist,2,5);

   /* buttons for dialog */
   buttons := vcRegion.Create("buttons", vcButnRow.fac,
              list (list ("Accept", "Accept",
                     func() {
                        path := vcRegion.Get(pathcmd, string);
                        vcCmd.Run(_pcreate_cmd, data, event, 
                                  list(path, ptype),TRUE);
                        ptype = nil;
                     },list(close_window <- TRUE)),
                   (list ("Cancel", "Cancel",
                     func() {
                        vcWin.Close(createprojw);
                     },nil))));
           
   vcTable.Add(pcreate_dialog,patht);
   vcTable.Add(pcreate_dialog,label("Select Project Type"));
   vcTable.Add(pcreate_dialog,scrollpicklist);
   vcTable.Add(pcreate_dialog,buttons);

   vcRegion.SetFocus(pathcmd, pathcmd);

   /* create the window for the dialog */
   createprojw = vcWin.Create(pcreate_dialog,
                      (vcWin.NoCmdline + vcWin.NoMenus + vcWin.NoBindings));

}

pcreate := vcWv.Cmd(fac,Pcreate, "pcreate","Create Project");
vcCmd.EnableMacro(pcreate,FALSE);
vcCmd.SetHelp(pcreate,"Create a project directory");

_pcreate_cmd := vcWv.Cmd(fac,Prjcreate,"_pcreate","Project Create",
                         vcWv.Param("Project Path", string, vcWv.Sticky,""),
                         vcWv.Param("Project Type", string, vcWv.Sticky,""));


/****************************************/
/* Process Commands */
/****************************************/

stopcurtool := vcWv.Cmd(fac,StopCurTool,"stopcurtool");
vcCmd.SetHelp(stopcurtool,"Stops the selected tool");

starttoolname := vcWv.Cmd(fac,StartToolName,"starttoolname", 
         "Find and start a tool.",
         vcWv.Param("Tool Name ", string, vcWv.Sticky,""));
vcCmd.SetHelp(starttoolname,"Finds and starts a tool");

starttoolcmdline := vcWv.Cmd(fac,StartToolCmdline,"starttoolcmdline", 
         "Find and start a tool with a given args.",
         vcWv.Param("Tool Name ", string, vcWv.Sticky,""),
         vcWv.Param("Command Args ", string, vcWv.Sticky,""));
vcCmd.SetHelp(starttoolcmdline,"Finds and starts a tool with given args");

starttool := vcWv.Cmd(fac,StartTool,"starttool");
vcCmd.SetHelp(starttool,"Selects and and starts a tool");

selectstarttool := vcWv.Cmd(fac,SelectStartTool,"selectstarttool");
vcCmd.SetHelp(selectstarttool,"Selects and and starts a tool");

startcurtool := vcWv.Cmd(fac,StartCurTool,"startcurtool");
vcCmd.SetHelp(startcurtool,"Starts the selected tool");

stopalltools := vcWv.Cmd(fac,StopAllTools,"stopalltools");
vcCmd.SetHelp(stopalltools,"Stops all started tools");

selactivetool := vcWv.Cmd(fac,SelActiveTool,"selactivetool");
vcCmd.SetHelp(selactivetool,"Select an active tool");

helpcurtool := vcWv.Cmd(fac,HelpCurTool,"helpcurtool");
vcCmd.SetHelp(helpcurtool,"Displays the help information for selected tool");

whichtef := vcWv.Cmd(fac,WhichTef,"whichtef");
vcCmd.SetHelp(whichtef,"Displays name of selected tool's .tef file");

desccurtool := vcWv.Cmd(fac,DescCurTool,"desccurtool");
vcCmd.SetHelp(desccurtool,"Displays the description for the selected tool");

clearcurlog := vcWv.Cmd(fac,ClearCurLog,"clearcurlog");
vcCmd.SetHelp(clearcurlog,"Clears the currently displayed tool log window");

findtool := vcWv.Cmd(fac,FindTool, "findtool","Find tool in toolboxes",
         vcWv.Param("Tool Name ", string, vcWv.Sticky,""));
vcCmd.SetHelp(findtool,"Finds tne name tool within the configured toolboxes.");

/****************************************/
/* Project Commands */
/****************************************/

psdraw := vcWv.Cmd(fac,SrchDraw,"psdraw");
vcCmd.SetHelp(psdraw,"Modify viewdraw search order");

psplace := vcWv.Cmd(fac,SrchPlace,"psplace");
vcCmd.SetHelp(psplace,"Modify viewplace search order");

psearch := vcWv.Cmd(fac,ChgSrch,"psearch");
vcCmd.SetHelp(psearch,"Modify current project search order for current view");

pdcreate := vcWv.Cmd(fac,PrjcreDraw, "pdcreate","Create VIEWDRAW Project",
         vcWv.Param("Project pathname ", string, vcWv.Sticky,""));
vcCmd.SetHelp(pdcreate,"Create a viewdraw project directory");

ppcreate := vcWv.Cmd(fac,PrjcrePlace, "ppcreate","Create VIEWPLACE Project",
         vcWv.Param("Project pathname ", string, vcWv.Sticky,""));
vcCmd.SetHelp(ppcreate,"Create a viewplace project directory");

premove := vcWv.Cmd(fac,PrjRemove, "premove");
vcCmd.SetHelp(premove,"Remove current project from project list");

phierarchy := vcWv.Cmd(fac,PrjHier, "phierarchy","Show Project Hierarchy");
vcCmd.SetHelp(phierarchy,"Display the design hierarchy of current project");

drawlib := vcWv.Cmd(fac,PLibDraw, "drawlib");
vcCmd.SetHelp(drawlib,"Display the current project's viewdraw libraries");

placelib := vcWv.Cmd(fac,PLibPlace, "placelib");
vcCmd.SetHelp(placelib,"Display the current project's viewplace libraries");

projlib := vcWv.Cmd(fac,PLib, "projlib");
vcCmd.SetHelp(projlib,"Display the current project's libraries for the current view");

pdelete := vcWv.Cmd(fac,PrjDelete, "pdelete");
vcCmd.SetHelp(pdelete,"Delete files and directories of current project");

pdirsch := vcWv.Cmd(fac,PrjDirsch, "pdirsch");
vcCmd.SetHelp(pdirsch,"Display contents of current project's schematic directory");

pdirsym := vcWv.Cmd(fac,PrjDirsym, "pdirsym");
vcCmd.SetHelp(pdirsym,"Display contents of current project's symbol directory");

pdirboa := vcWv.Cmd(fac,PrjDirboa, "pdirboa");
vcCmd.SetHelp(pdirboa,"Display contents of current project's board directory");

pdirsha := vcWv.Cmd(fac,PrjDirsha, "pdirsha");
vcCmd.SetHelp(pdirsha,"Display contents of current project's shape directory");

pdiroth := vcWv.Cmd(fac,PrjDiroth, "pdiroth");
vcCmd.SetHelp(pdiroth,"Display contents of current project's top directory");


parchive := vcWv.Cmd(fac,PrjArch, "parchive", "Project Archive");
vcCmd.SetHelp(parchive,"Archive files related to current project's design hierarchy");

pcopy := vcWv.Cmd(fac,PrjCopy, "pcopy", "Project Copy",
        vcWv.Param("Directory Name to Copy to ", string, vcWv.Sticky,"",
         vcFilePop.fac, file_columns<-0));
vcCmd.SetHelp(pcopy,"Copy the current project directory to a new directory");

pmove := vcWv.Cmd(fac,PrjMove, "pmove", "Project Move",
        vcWv.Param("Project Pathname to Move to ", string, vcWv.Sticky,"",
         vcFilePop.fac, file_columns<-0));
vcCmd.SetHelp(pmove,"Move the current project to a new directory");

prestore := vcWv.Cmd(fac,PrjRestore, "prestore", "Project Restore",
        vcWv.Param("Directory to Restore from ",string,vcWv.Sticky,"",
         vcFilePop.fac, file_columns<-0),
        vcWv.Param("Directory to Restore to ",string,vcWv.Sticky,"",
         vcFilePop.fac, file_columns<-0));
vcCmd.SetHelp(prestore,"Restore a copied or archived project");

pltsch := vcWv.Cmd(fac,PrjPlotSch, "pltsch","Plot Project Schematics");
vcCmd.SetHelp(pltsch,"Plot all schematics of a design in current project");

pltoat := vcWv.Cmd(fac,PrjPlotOat, "pltoat","Plot Project Schematics/OAT ");
vcCmd.SetHelp(pltoat,"Plot all schematics of a design in current project with occurrence attributes");

pltsym := vcWv.Cmd(fac,PrjPlotSym, "pltsym","Plot Project Symbols");
vcCmd.SetHelp(pltsym,"Plot all symbols of a design in current project");

pltboa := vcWv.Cmd(fac,PrjPlotBoa, "pltboa","Plot Project Board");
vcCmd.SetHelp(pltboa,"Plot board and all shapes of a design in current project");

schdelete := vcWv.Cmd(fac,SchDel, "schdelete", "Delete Schematic");
vcCmd.SetHelp(schdelete,"Delete selected schematic sheets from current project");

symdelete := vcWv.Cmd(fac,SymDel, "symdelete", "Delete Symbol");
vcCmd.SetHelp(symdelete,"Delete selected symbol sheets from current project");

project := vcWv.Cmd(fac, Project, "project");
vcCmd.SetHelp(project,"Set the current project");

selprojname := vcWv.Cmd(fac,SelProjName,"selprojname", 
         "Select a project by name.",
         vcWv.Param("Project Name ", string, vcWv.Sticky,""),
         vcWv.Param("View Name ", string, vcWv.Sticky,""));
vcCmd.SetHelp(selprojname,"Find a project by name");

selprojpath := vcWv.Cmd(fac,SelProjPath,"selprojpath", 
         "Select a project by path.",
         vcWv.Param("Project Path ", string, vcWv.Sticky,""),
         vcWv.Param("View Name ", string, vcWv.Sticky,""));
vcCmd.SetHelp(selprojname,"Find a project by path");


/****************************************/
/* Library Commands */
/****************************************/

lcopy := vcWv.Cmd(fac,LibCopy, "lcopy", "Library Copy",
        vcWv.Param("Directory Name to Copy to ", string, vcWv.Sticky,"",
         vcFilePop.fac, file_columns<-0));
vcCmd.SetHelp(lcopy,"Copy the current library directory to a new directory");

library := vcWv.Cmd(fac, Library, "library");
vcCmd.SetHelp(library,"Set the current library");

ldirsch := vcWv.Cmd(fac,LibDirsch, "ldirsch");
vcCmd.SetHelp(ldirsch,"Display contents of current library's schematic directory");

ldirsym := vcWv.Cmd(fac,LibDirsym, "ldirsym");
vcCmd.SetHelp(ldirsym,"Display contents of current library's symbol directory");

ldirboa := vcWv.Cmd(fac,LibDirboa, "ldirboa");
vcCmd.SetHelp(ldirboa,"Display contents of current library's board directory");
ldirsha := vcWv.Cmd(fac,LibDirsha, "ldirsha");
vcCmd.SetHelp(ldirsha,"Display contents of current library's shape directory");
/****************************************/
/*  config commands  */
/****************************************/

startcfilog := vcWv.Cmd(fac,StartCfiLog, "startcfilog");
vcCmd.SetHelp(startcfilog,"Start CFI logging");

stopcfilog := vcWv.Cmd(fac,StopCfiLog, "stopcfilog");
vcCmd.SetHelp(stopcfilog,"Stop CFI logging");

initreg := vcWv.Cmd(fac, InitReg, "initreg");
vcCmd.SetHelp(initreg,"Reinitialize Viewframe");

savecfg := vcWv.Cmd(fac, SaveCfg, "savecfg");
vcCmd.SetHelp(savecfg,"Save current configuration");

editenv := vcWv.Cmd(fac, EditEnv, "editenv");
vcCmd.SetHelp(editenv,"Set environment variable");

edittef := vcWv.Cmd(fac, EditTef, "edittef");
vcCmd.SetHelp(edittef,"Edit selected tool's tef file");

/****************************************/
/*  misc commands  */
/****************************************/

exit_cmd := vcWv.Cmd(fac,func(r,e) vcWin.Close(vcEvent.Window(e));,"exit");

/********************************************/
/* browser commands */
/********************************************/
browseproj := vcWv.Cmd(fac, BrowseProj, "browseproj");

function enable_browseproj_command(on_off) 
{
   vcCmd.Enable(browseproj, on_off);  

}

/****************************************/
/* Load menus */
/****************************************/

load("vframemn.vs");

/****************************************/
/* Load bindings */
/****************************************/

load("vframebt.vs");

/****************************************/
/* Load storagemanager functions */
/****************************************/
if (IsSMVersion()==1)
   load("vframesm.vs");


