------------------------------------------------------------------------
-- This package is an extension of std_logic_1164 package to provide 
-- simulation support for synthesis subprograms.

-- Last Modified:  11/July/94
------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package pack1164 is

------------------------------------------------------------------------
-- Type Definitions                                                   --
------------------------------------------------------------------------
   type   std_ulogic_2d  is array (integer range<>,
                                   integer range<>) of std_ulogic;
   type   std_logic_2d   is array (integer range<>,
                                   integer range<>) of std_logic;
------------------------------------------------------------------------
-- Conversion Functions                                               --
------------------------------------------------------------------------
  function boo2std  (b: boolean)                return std_ulogic;
  function std2boo  (v: std_ulogic)             return boolean;
  function int2v1d  (i: integer)                return std_ulogic_vector;
  function int2v1d  (i: integer)                return std_logic_vector;
  function v1d2int  (v: std_ulogic_vector)      return integer;
  function v1d2int  (v: std_logic_vector)       return integer;

------------------------------------------------------------------------
-- Arithmetic Functions                                               --
------------------------------------------------------------------------
  function addum    (v1, v2:std_ulogic_vector)       return std_ulogic_vector;
  function addum    (v1, v2:std_logic_vector)        return std_logic_vector;
  function add2c    (v1, v2:std_ulogic_vector)       return std_ulogic_vector;
  function add2c    (v1, v2:std_logic_vector)        return std_logic_vector;
  function comp2c   (v:std_ulogic_vector)            return std_ulogic_vector;
  function comp2c   (v:std_logic_vector)             return std_logic_vector;
  function extendum (v:std_ulogic_vector; i:integer) return std_ulogic_vector;
  function extendum (v:std_logic_vector; i:integer)  return std_logic_vector;
  function extend2c (v:std_ulogic_vector; i:integer) return std_ulogic_vector;
  function extend2c (v:std_logic_vector; i:integer)  return std_logic_vector;
  function shiftlum (v:std_ulogic_vector; i:integer) return std_ulogic_vector;
  function shiftlum (v:std_logic_vector; i:integer)  return std_logic_vector;
  function shiftl2c (v:std_ulogic_vector; i:integer) return std_ulogic_vector;
  function shiftl2c (v:std_logic_vector; i:integer)  return std_logic_vector;
  function shiftrum (v:std_ulogic_vector; i:integer) return std_ulogic_vector;
  function shiftrum (v:std_logic_vector; i:integer)  return std_logic_vector;
  function shiftr2c (v:std_ulogic_vector; i:integer) return std_ulogic_vector;
  function shiftr2c (v:std_logic_vector; i:integer)  return std_logic_vector;
  function subum    (v1, v2:std_ulogic_vector)       return std_ulogic_vector;
  function subum    (v1, v2:std_logic_vector)        return std_logic_vector;
  function sub2c    (v1, v2:std_ulogic_vector)       return std_ulogic_vector;
  function sub2c    (v1, v2:std_logic_vector)        return std_logic_vector;

------------------------------------------------------------------------
-- Overload Functions
------------------------------------------------------------------------
  function "+" (v1, v2:std_ulogic_vector)  return std_ulogic_vector;
  function "+" (v1, v2:std_logic_vector)   return std_logic_vector;
  function "-" (v1, v2:std_ulogic_vector)  return std_ulogic_vector;
  function "-" (v1, v2:std_logic_vector)   return std_logic_vector;

------------------------------------------------------------------------
-- Memory Functions                                                   --
------------------------------------------------------------------------
   PROCEDURE dff (CONSTANT data     :  IN    std_ulogic;
                  SIGNAL   clk      :  IN    std_ulogic;   -- + edge triggered
                  SIGNAL   q        :  INOUT std_ulogic);

   PROCEDURE dffc (CONSTANT data    :  IN    std_ulogic;
                   SIGNAL   clear   :  IN    std_ulogic;   -- active high
                   SIGNAL   clk     :  IN    std_ulogic;   -- + edge triggered
                   SIGNAL   q       :  INOUT std_ulogic);

   PROCEDURE dffp (CONSTANT data    :  IN    std_ulogic;
                   SIGNAL   preset  :  IN    std_ulogic;   -- active high
                   SIGNAL   clk     :  IN    std_ulogic;   -- + edge triggered
                   SIGNAL   q       :  INOUT std_ulogic);

   PROCEDURE dffpc (CONSTANT data   :  IN    std_ulogic;
                    SIGNAL   preset :  IN    std_ulogic;   -- active high
                    SIGNAL   clear  :  IN    std_ulogic;   -- active high
                    SIGNAL   clk    :  IN    std_ulogic;   -- + edge triggered
                    SIGNAL   q      :  INOUT std_ulogic);

   PROCEDURE dff_v (CONSTANT data   :  IN    std_ulogic_vector;
                    SIGNAL   clk    :  IN    std_ulogic;   -- + edge triggered
                    SIGNAL   q      :  INOUT std_ulogic_vector);

   PROCEDURE dff_v (CONSTANT data   :  IN    std_logic_vector;
                    SIGNAL   clk    :  IN    std_ulogic;   -- + edge triggered
                    SIGNAL   q      :  INOUT std_logic_vector);

   PROCEDURE dffc_v (CONSTANT data  :  IN    std_ulogic_vector;
                     SIGNAL   clear :  IN    std_ulogic;   -- active high
                     SIGNAL   clk   :  IN    std_ulogic;   -- + edge triggered
                     SIGNAL   q     :  INOUT std_ulogic_vector);

   PROCEDURE dffc_v (CONSTANT data  :  IN    std_logic_vector;
                     SIGNAL   clear :  IN    std_ulogic;   -- active high
                     SIGNAL   clk   :  IN    std_ulogic;   -- + edge triggered
                     SIGNAL   q     :  INOUT std_logic_vector);

   PROCEDURE dffp_v (CONSTANT data  :  IN    std_ulogic_vector;
                     SIGNAL   preset:  IN    std_ulogic;   -- active high
                     SIGNAL   clk   :  IN    std_ulogic;   -- + edge triggered
                     SIGNAL   q     :  INOUT std_ulogic_vector);

   PROCEDURE dffp_v (CONSTANT data  :  IN    std_logic_vector;
                     SIGNAL   preset:  IN    std_ulogic;   -- active high
                     SIGNAL   clk   :  IN    std_ulogic;   -- + edge triggered
                     SIGNAL   q     :  INOUT std_logic_vector);

   PROCEDURE dffpc_v(CONSTANT data  :  IN    std_ulogic_vector;
                     SIGNAL   preset:  IN    std_ulogic;   -- active high
                     SIGNAL   clear :  IN    std_ulogic;   -- active high
                     SIGNAL   clk   :  IN    std_ulogic;   -- + edge triggered
                     SIGNAL   q     :  INOUT std_ulogic_vector);

   PROCEDURE dffpc_v(CONSTANT data  :  IN    std_logic_vector;
                     SIGNAL   preset:  IN    std_ulogic;   -- active high
                     SIGNAL   clear :  IN    std_ulogic;   -- active high
                     SIGNAL   clk   :  IN    std_ulogic;   -- + edge triggered
                     SIGNAL   q     :  INOUT std_logic_vector);

   PROCEDURE dlatch (CONSTANT data  :  IN    std_ulogic;
                     SIGNAL   gate  :  IN    std_ulogic;   -- active high
                     SIGNAL   q     :  INOUT std_ulogic);

   PROCEDURE dlatchc (CONSTANT data :  IN    std_ulogic;
                      SIGNAL   clear:  IN    std_ulogic;   -- active high
                      SIGNAL   gate :  IN    std_ulogic;   -- active high
                      SIGNAL   q    :  INOUT std_ulogic);

   PROCEDURE dlatchp(CONSTANT data  :  IN    std_ulogic;
                     SIGNAL   preset:  IN    std_ulogic;   -- active high
                     SIGNAL   gate  :  IN    std_ulogic;   -- active high
                     SIGNAL   q     :  INOUT std_ulogic);

   PROCEDURE dlatchpc(CONSTANT data :  IN    std_ulogic;
                      SIGNAL  preset:  IN    std_ulogic;   -- active high
                      SIGNAL  clear :  IN    std_ulogic;   -- active high
                      SIGNAL  gate  :  IN    std_ulogic;   -- active high
                      SIGNAL  q     :  INOUT std_ulogic);

   PROCEDURE dlatch_v (CONSTANT data:  IN    std_ulogic_vector;
                       SIGNAL   gate:  IN    std_ulogic;   -- active high
                       SIGNAL   q   :  INOUT std_ulogic_vector);

   PROCEDURE dlatch_v (CONSTANT data:  IN    std_logic_vector;
                       SIGNAL   gate:  IN    std_ulogic;   -- active high
                       SIGNAL   q   :  INOUT std_logic_vector);

   PROCEDURE dlatchc_v(CONSTANT data:  IN    std_ulogic_vector;
                       SIGNAL  clear:  IN    std_ulogic;   -- active high
                       SIGNAL  gate :  IN    std_ulogic;   -- active high
                       SIGNAL  q    :  INOUT std_ulogic_vector);

   PROCEDURE dlatchc_v(CONSTANT data:  IN    std_logic_vector;
                       SIGNAL  clear:  IN    std_ulogic;   -- active high
                       SIGNAL  gate :  IN    std_ulogic;   -- active high
                       SIGNAL  q    :  INOUT std_logic_vector);

   PROCEDURE dlatchpc_v(CONSTANT data  :  IN    std_ulogic_vector;
                        SIGNAL   preset:  IN    std_ulogic;  -- active high
                        SIGNAL   clear :  IN    std_ulogic;  -- active high
                        SIGNAL   gate  :  IN    std_ulogic;  -- active high
                        SIGNAL   q     :  INOUT std_ulogic_vector);

   PROCEDURE dlatchpc_v(CONSTANT data  :  IN    std_logic_vector;
                        SIGNAL   preset:  IN    std_ulogic;  -- active high
                        SIGNAL   clear :  IN    std_ulogic;  -- active high
                        SIGNAL   gate  :  IN    std_ulogic;  -- active high
                        SIGNAL   q     :  INOUT std_logic_vector);

   PROCEDURE srlatch (SIGNAL   set     :  IN    std_ulogic;  -- active high
                      SIGNAL   reset   :  IN    std_ulogic;  -- active high
                      SIGNAL   q       :  INOUT std_ulogic);

   PROCEDURE jkff (SIGNAL   j          :  IN    std_ulogic;
                   SIGNAL   k          :  IN    std_ulogic;
                   SIGNAL   clk        :  IN    std_ulogic;  -- + edge triggered
                   SIGNAL   q          :  INOUT std_ulogic);

   PROCEDURE jkffc (SIGNAL   j         :  IN    std_ulogic;
                    SIGNAL   k         :  IN    std_ulogic;
                    SIGNAL   clear     :  IN    std_ulogic;  -- active high
                    SIGNAL   clk       :  IN    std_ulogic;  -- + edge triggered
                    SIGNAL   q         :  INOUT std_ulogic);

   PROCEDURE jkffp (SIGNAL   j         :  IN    std_ulogic;
                    SIGNAL   k         :  IN    std_ulogic;
                    SIGNAL   preset    :  IN    std_ulogic;  -- active high
                    SIGNAL   clk       :  IN    std_ulogic;  -- + edge triggered
                    SIGNAL   q         :  INOUT std_ulogic);

   PROCEDURE jkffpc (SIGNAL   j        :  IN    std_ulogic;
                     SIGNAL   k        :  IN    std_ulogic;
                     SIGNAL   preset   :  IN    std_ulogic;  -- active high
                     SIGNAL   clear    :  IN    std_ulogic;  -- active high
                     SIGNAL   clk      :  IN    std_ulogic;  -- + edge triggered
                     SIGNAL   q        :  INOUT std_ulogic);

   PROCEDURE pla_table(I          : in  std_ulogic_vector;
                       signal O   : out std_ulogic_vector;
                       tbl        : in  std_ulogic_2d);

   PROCEDURE pla_table(I          : in  std_logic_vector;
                       signal O   : out std_logic_vector;
                       tbl        : in  std_logic_2d);

end pack1164;

------------------------------------------------------------------------
-- Package Body for pack1164                                          --
------------------------------------------------------------------------

package body pack1164 is

------------------------------------------------------------------------
   function max(i1, i2: integer)  return integer is
   begin
      if i1 < i2 then
         return i2;
      else
         return i1;
      end if;
   end;

------------------------------------------------------------------------
   function min(i1, i2: integer)  return integer is
   begin
      if i1 > i2 then
         return i2;
      else
         return i1;
      end if;
   end;

------------------------------------------------------------------------
   function bitX(v: std_ulogic)  return boolean is
   begin
      case v is
         when 'X'    => return true;
         when others => return false;
      end case;
   end;

------------------------------------------------------------------------
   function bitZ(v: std_ulogic) return boolean is
   begin
      case v is
         when 'Z'    => return true;
         when others => return false;
      end case;
   end;

------------------------------------------------------------------------
   function bitunknown(v: std_ulogic)    return boolean is
   begin
      case v is
         when 'U' | 'W' | '-' | 'X' | 'Z' => return true;  -- MMM.july.7.94
         when others    => return false;
      end case;
   end;

------------------------------------------------------------------------
   function vecunknown(v:std_ulogic_vector) return boolean is
   begin
      for b in v'range loop
         if bitunknown (v (b)) then
            return true;
         end if;
      end loop;
      return false;
   end;

------------------------------------------------------------------------
   function vecunknown(v:std_logic_vector) return boolean is
   begin
      for b in v'range loop
         if bitunknown (v (b)) then
            return true;
         end if;
      end loop;
      return false;
   end;

------------------------------------------------------------------------
   function pchanging(signal s: std_ulogic)    return boolean is
   begin
      return s'event;
   end;

------------------------------------------------------------------------
   function pfalling(signal s: std_ulogic)    return boolean is
   begin
      return s'event and s = '0';
   end;

------------------------------------------------------------------------
   function prising(signal s: std_ulogic)    return boolean is
   begin
      return s'event and s = '1';
   end;

------------------------------------------------------------------------
   function boo2std(b: boolean) return std_ulogic is
   begin
      case b is
         when false => return '0';
         when true  => return '1';
      end case;
   end;

------------------------------------------------------------------------
   function std2boo(v: std_ulogic)   return boolean is
   begin
      case v is
         when '1'    => return true;
         when 'H'    => return true;
         when others => return false;
      end case;
   end;

------------------------------------------------------------------------
   function int2v1d(i: integer)  return std_ulogic_vector is
      variable temp:   integer := i;
      variable result: std_ulogic_vector (31 downto 0) := 
           ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
            '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');
   begin
      for b in 0 to 31 loop
         if temp rem 2 = 1 then
            result (b) := '1';
         end if;
         temp := temp / 2;
      end loop;
      return result;
   end;

------------------------------------------------------------------------
   function int2v1d(i: integer)  return std_logic_vector is
      variable temp:   integer := i;
      variable result: std_logic_vector (31 downto 0) :=
           ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0',
            '0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');
   begin
      for b in 0 to 31 loop
         if temp rem 2 = 1 then
            result (b) := '1';
         end if;
         temp := temp / 2;
      end loop;
      return result;
   end;

------------------------------------------------------------------------
-- FIX THIS --

   function v1d2int(v: std_ulogic_vector) return integer is
      variable result: integer := 0;
      variable low: integer := min(v'right,v'left);
      variable high: integer := max(v'right,v'left);
      variable temp: std_ulogic_vector(low to high) := v;
      variable addition: integer := 1;
      variable numOfelem: integer := high - low + 1;
      variable neg: boolean := FALSE;
   begin
      if numOfelem >= 32 then
         numOfelem := 31;
         if temp(high - 31) = '1' then  -- convert to complement 1.
            for b in low to high loop
                temp (b) := not temp(b);
            end loop;
            neg := TRUE;
         end if;                   
     end if;        


     for b in temp'reverse_range loop
         if numOfelem > 0 then
            if temp (b) = '1' then
               result := result + addition;
            end if;
            if numOfelem > 1 then
               addition := addition * 2;
            end if;
            numOfelem := numOfelem - 1;
         end if;
     end loop;
     if neg then
       return (-1 * result - 1);
     end if;
      return result;
  end;

------------------------------------------------------------------------
-- FIX THIS --

   function v1d2int(v: std_logic_vector) return integer is
      variable result: integer := 0;
      variable low: integer := min(v'right,v'left);
      variable high: integer := max(v'right,v'left);
      variable temp: std_logic_vector(low to high) := v;
      variable addition: integer := 1;
      variable numOfelem: integer := high - low + 1;
      variable neg: boolean := FALSE;
   begin
      if numOfelem >= 32 then
         numOfelem := 31;
         if temp(high - 31) = '1' then  -- convert to complement 1.
            for b in low to high loop
                temp (b) := not temp(b);
            end loop;
            neg := TRUE;
         end if;
     end if;


     for b in temp'reverse_range loop
         if numOfelem > 0 then
            if temp (b) = '1' then
               result := result + addition;
            end if;
            if numOfelem > 1 then
               addition := addition * 2;
            end if;
            numOfelem := numOfelem - 1;
         end if;
     end loop;
     if neg then
       return (-1 * result - 1);
     end if;
      return result;
  end;

------------------------------------------------------------------------
  function addum(v1, v2 : std_ulogic_vector) return std_ulogic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_ulogic_vector (1 to i) is v1 ;
      alias rv: std_ulogic_vector (1 to j) is v2 ;
      variable bit1: std_ulogic := lv (i) ;
      variable bit2: std_ulogic := rv (j) ;
      variable carry:  std_ulogic:= '0';
      variable result: std_ulogic_vector (0 to max (v1'length, v2'length));
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := (bit1 and bit2) or (carry and (bit1 or bit2));
         if i = 1 then
            bit1 := '0';
         else
            i := i - 1 ;
            bit1 := lv (i);
         end if;
         if j = 1 then
            bit2 := '0';
         else
            j := j - 1 ;
            bit2 := rv (j);
         end if;
      end loop;
      return result;
   end;
------------------------------------------------------------------------
  function addum(v1, v2 : std_logic_vector) return std_logic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_logic_vector (1 to i) is v1 ;
      alias rv: std_logic_vector (1 to j) is v2 ;
      variable bit1: std_logic := lv (i) ;
      variable bit2: std_logic := rv (j) ;
      variable carry:  std_logic:= '0';
      variable result: std_logic_vector (0 to max (v1'length, v2'length));
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := (bit1 and bit2) or (carry and (bit1 or bit2));
         if i = 1 then
            bit1 := '0';
         else
            i := i - 1 ;
            bit1 := lv (i);
         end if;
         if j = 1 then
            bit2 := '0';
         else
            j := j - 1 ;
            bit2 := rv (j);
         end if;
      end loop;
      return result;
   end;

------------------------------------------------------------------------ 
   function add2c(v1, v2: std_ulogic_vector) return std_ulogic_vector is
      variable i: integer := v1'length ; 
      variable j: integer := v2'length ; 
      alias lv: std_ulogic_vector (1 to i) is v1 ; 
      alias rv: std_ulogic_vector (1 to j) is v2 ; 
      variable bit1: std_ulogic := lv (i) ;
      variable bit2: std_ulogic := rv (j) ; 
      variable carry:  std_ulogic := '0'; 
      variable result: std_ulogic_vector (0 to max (v1'length, v2'length)); 
   begin
      for k in (result'length - 1) downto 1 loop
         result (k) := bit1 xor bit2 xor carry; 
         carry      := (bit1 and bit2) or (carry and (bit1 or bit2)); 
         if i /= 1 then 
            i := i - 1 ; 
            bit1 := lv (i); 
         end if; 
         if j /= 1 then 
            j := j - 1 ;
            bit2 := rv (j); 
         end if; 
      end loop; 
      if bitX(bit1) or bitX(bit2) or bitZ(bit1) or bitZ(bit2) then
         result(0) := 'X';
      else
         result(0) := bit1 xor bit2 xor carry;
      end if;
      return result;
   end;

------------------------------------------------------------------------
   function add2c(v1, v2: std_logic_vector) return std_logic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_logic_vector (1 to i) is v1 ;
      alias rv: std_logic_vector (1 to j) is v2 ;
      variable bit1: std_logic := lv (i) ;
      variable bit2: std_logic := rv (j) ;
      variable carry:  std_logic := '0';
      variable result: std_logic_vector (0 to max (v1'length, v2'length));
   begin
      for k in (result'length - 1) downto 1 loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := (bit1 and bit2) or (carry and (bit1 or bit2));
         if i /= 1 then
            i := i - 1 ;
            bit1 := lv (i);
         end if;
         if j /= 1 then
            j := j - 1 ;
            bit2 := rv (j);
         end if;
      end loop;
      if bitX(bit1) or bitX(bit2) or bitZ(bit1) or bitZ(bit2) then
         result(0) := 'X';
      else
         result(0) := bit1 xor bit2 xor carry;
      end if;
      return result;
   end;

------------------------------------------------------------------------
   function comp2c(v: std_ulogic_vector) return std_ulogic_vector is
      variable i: integer := v'length ;
      alias av: std_ulogic_vector (1 to i) is v ;
      variable carry:  std_ulogic := '1';
      variable result: std_ulogic_vector (0 to v'length - 1);
   begin
      for j in result'reverse_range loop
         result (j) := carry xor not av (i);
         carry      := carry and not av (i);
         i := i - 1 ;
      end loop;
      return result;
   end;

------------------------------------------------------------------------
   function comp2c(v: std_logic_vector) return std_logic_vector is
      variable i: integer := v'length ;
      alias av: std_logic_vector (1 to i) is v ;
      variable carry:  std_logic := '1';
      variable result: std_logic_vector (0 to v'length - 1);
   begin
      for j in result'reverse_range loop
         result (j) := carry xor not av (i);
         carry      := carry and not av (i);
         i := i - 1 ;
      end loop;
      return result;
   end;

------------------------------------------------------------------------
   function extendum(v: std_ulogic_vector; i: integer) return std_ulogic_vector is
      variable result: std_ulogic_vector (0 to i - 1);
   begin
      assert i >= v'length report "extendum: invalid extension length"
                           severity FAILURE;
      for j in 0 to i - v'length - 1 loop
         result (j) := '0';
      end loop;
      result (i - v'length to i - 1) := v;
      return result;
   end;

------------------------------------------------------------------------
   function extendum(v: std_logic_vector; i: integer) return std_logic_vector is
      variable result: std_logic_vector (0 to i - 1);
   begin
      assert i >= v'length report "extendum: invalid extension length"
                           severity FAILURE;
      for j in 0 to i - v'length - 1 loop
         result (j) := '0';
      end loop;
      result (i - v'length to i - 1) := v;
      return result;
   end;

------------------------------------------------------------------------
   function extend2c(v: std_ulogic_vector; i: integer) return std_ulogic_vector is
      variable result: std_ulogic_vector (0 to i - 1);
   begin
      assert i >= v'length report "extend2c: invalid extension length"
                           severity FAILURE;
      for j in 0 to i - v'length - 1 loop
         result (j) := v (v'left);
      end loop;
      result (i - v'length to i - 1) := v;
      return result;
   end;

------------------------------------------------------------------------
   function extend2c(v: std_logic_vector; i: integer) return std_logic_vector is
      variable result: std_logic_vector (0 to i - 1);
   begin
      assert i >= v'length report "extend2c: invalid extension length"
                           severity FAILURE;
      for j in 0 to i - v'length - 1 loop
         result (j) := v (v'left);
      end loop;
      result (i - v'length to i - 1) := v;
      return result;
   end;

------------------------------------------------------------------------
   function shiftlum(v: std_ulogic_vector; i: integer) return std_ulogic_vector is
      variable result: std_ulogic_vector (0 to v'length + i - 1);
   begin
      result (0 to v'length - 1) := v;
      for j in v'length to v'length + i - 1 loop
         result (j) := '0';
      end loop;
      return result (i to v'length + i - 1);
   end;

------------------------------------------------------------------------
   function shiftlum(v: std_logic_vector; i: integer) return std_logic_vector is
      variable result: std_logic_vector (0 to v'length + i - 1);
   begin
      result (0 to v'length - 1) := v;
      for j in v'length to v'length + i - 1 loop
         result (j) := '0';
      end loop;
      return result (i to v'length + i - 1);
   end;

------------------------------------------------------------------------
   function shiftl2c(v: std_ulogic_vector; i: integer) return std_ulogic_vector is
      variable result: std_ulogic_vector (0 to v'length + i - 1);
   begin
      result (0 to v'length - 1) := v;
      for j in v'length to v'length + i - 1 loop
         result (j) := '0';
      end loop;
      return result (i to v'length + i - 1);
   end;

------------------------------------------------------------------------
   function shiftl2c(v: std_logic_vector; i: integer) return std_logic_vector is
      variable result: std_logic_vector (0 to v'length + i - 1);
   begin
      result (0 to v'length - 1) := v;
      for j in v'length to v'length + i - 1 loop
         result (j) := '0';
      end loop;
      return result (i to v'length + i - 1);
   end;

------------------------------------------------------------------------
   function shiftrum(v: std_ulogic_vector; i: integer) return std_ulogic_vector is
      variable result: std_ulogic_vector (0 to v'length + i - 1);
   begin
      for j in 0 to i - 1 loop
         result (j) := '0';
      end loop;
      result (i to v'length + i - 1) := v;
      return result (0 to v'length - 1);
   end;

------------------------------------------------------------------------
   function shiftrum(v: std_logic_vector; i: integer) return std_logic_vector is
      variable result: std_logic_vector (0 to v'length + i - 1);
   begin
      for j in 0 to i - 1 loop
         result (j) := '0';
      end loop;
      result (i to v'length + i - 1) := v;
      return result (0 to v'length - 1);
   end;

------------------------------------------------------------------------
   function shiftr2c(v: std_ulogic_vector; i: integer) return std_ulogic_vector is
      variable result: std_ulogic_vector (0 to v'length + i - 1);
   begin
      for j in 0 to i - 1 loop
         result (j) := v (v'left);
      end loop;
      result (i to v'length + i - 1) := v;
      return result (0 to v'length - 1);
   end;

------------------------------------------------------------------------
   function shiftr2c(v: std_logic_vector; i: integer) return std_logic_vector is
      variable result: std_logic_vector (0 to v'length + i - 1);
   begin
      for j in 0 to i - 1 loop
         result (j) := v (v'left);
      end loop;
      result (i to v'length + i - 1) := v;
      return result (0 to v'length - 1);
   end;

------------------------------------------------------------------------
   function subum(v1, v2: std_ulogic_vector) return std_ulogic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_ulogic_vector (1 to i) is v1 ;
      alias rv: std_ulogic_vector (1 to j) is v2 ;
      variable bit1: std_ulogic := lv (i) ;
      variable bit2: std_ulogic := rv (j) ;
      variable carry:  std_ulogic := '0';
      variable result: std_ulogic_vector (0 to max (v1'length, v2'length));
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := ((not bit1) and (bit2 or  carry)) or
                            (bit1  and (bit2 and carry));
         if i = 1 then
            bit1 := '0';
         else
            i := i - 1 ;
            bit1 := lv (i) ;
         end if;
         if j = 1 then
            bit2 := '0' ;
         else
            j := j - 1 ;
            bit2 := rv (j) ;
         end if;
      end loop;
      return result;
   end;

------------------------------------------------------------------------
   function subum(v1, v2: std_logic_vector) return std_logic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_logic_vector (1 to i) is v1 ;
      alias rv: std_logic_vector (1 to j) is v2 ;
      variable bit1: std_logic := lv (i) ;
      variable bit2: std_logic := rv (j) ;
      variable carry:  std_logic := '0';
      variable result: std_logic_vector (0 to max (v1'length, v2'length));
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := ((not bit1) and (bit2 or  carry)) or
                            (bit1  and (bit2 and carry));
         if i = 1 then
            bit1 := '0';
         else
            i := i - 1 ;
            bit1 := lv (i) ;
         end if;
         if j = 1 then
            bit2 := '0' ;
         else
            j := j - 1 ;
            bit2 := rv (j) ;
         end if;
      end loop;
      return result;
   end;

-----------------------------------------------------------------------
   function sub2c (v1, v2: std_ulogic_vector) return std_ulogic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_ulogic_vector (1 to i) is v1 ;
      alias rv: std_ulogic_vector (1 to j) is v2 ;
      variable bit1: std_ulogic := lv (i) ;
      variable bit2: std_ulogic := rv (j) ;
      variable carry:  std_ulogic := '0';
      variable result: std_ulogic_vector (0 to max (v1'length, v2'length));
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := ((not bit1) and (bit2 or  carry)) or
                            (bit1  and (bit2 and carry));
         if i /= 1 then
            i := i - 1 ;
            bit1 := lv (i);
         end if;
         if j /= 1 then
            j := j - 1 ;
            bit2 := rv (j);
         end if;
      end loop;
      return result;
   end;

-----------------------------------------------------------------------
   function sub2c (v1, v2: std_logic_vector) return std_logic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_logic_vector (1 to i) is v1 ;
      alias rv: std_logic_vector (1 to j) is v2 ;
      variable bit1: std_logic := lv (i) ;
      variable bit2: std_logic := rv (j) ;
      variable carry:  std_logic := '0';
      variable result: std_logic_vector (0 to max (v1'length, v2'length));
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := ((not bit1) and (bit2 or  carry)) or
                            (bit1  and (bit2 and carry));
         if i /= 1 then
            i := i - 1 ;
            bit1 := lv (i);
         end if;
         if j /= 1 then
            j := j - 1 ;
            bit2 := rv (j);
         end if;
      end loop;
      return result;
   end;

------------------------------------------------------------------------
  function "+"(v1, v2 : std_ulogic_vector) return std_ulogic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_ulogic_vector (1 to i) is v1 ;
      alias rv: std_ulogic_vector (1 to j) is v2 ;
      variable bit1: std_ulogic := lv (i) ;
      variable bit2: std_ulogic := rv (j) ;
      variable carry:  std_ulogic:= '0';
      variable result: std_ulogic_vector (0 to max(v1'length, v2'length) - 1);
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := (bit1 and bit2) or (carry and (bit1 or bit2));
         if i = 1 then
            bit1 := '0';
         else
            i := i - 1 ;
            bit1 := lv (i);
         end if;
         if j = 1 then
            bit2 := '0';
         else
            j := j - 1 ;
            bit2 := rv (j);
         end if;
      end loop;
      return result;
   end;

------------------------------------------------------------------------
  function "+"(v1, v2 : std_logic_vector) return std_logic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_logic_vector (1 to i) is v1 ;
      alias rv: std_logic_vector (1 to j) is v2 ;
      variable bit1: std_logic := lv (i) ;
      variable bit2: std_logic := rv (j) ;
      variable carry:  std_logic:= '0';
      variable result: std_logic_vector (0 to max(v1'length, v2'length) - 1);
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := (bit1 and bit2) or (carry and (bit1 or bit2));
         if i = 1 then
            bit1 := '0';
         else
            i := i - 1 ;
            bit1 := lv (i);
         end if;
         if j = 1 then
            bit2 := '0';
         else
            j := j - 1 ;
            bit2 := rv (j);
         end if;
      end loop;
      return result;
   end;

------------------------------------------------------------------------
   function "-"(v1, v2: std_ulogic_vector) return std_ulogic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_ulogic_vector (1 to i) is v1 ;
      alias rv: std_ulogic_vector (1 to j) is v2 ;
      variable bit1: std_ulogic := lv (i) ;
      variable bit2: std_ulogic := rv (j) ;
      variable carry:  std_ulogic := '0';
      variable result: std_ulogic_vector (0 to max (v1'length, v2'length) - 1);
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := ((not bit1) and (bit2 or  carry)) or
                            (bit1  and (bit2 and carry));
         if i = 1 then
            bit1 := '0';
         else
            i := i - 1 ;
            bit1 := lv (i) ;
         end if;
         if j = 1 then
            bit2 := '0' ;
         else
            j := j - 1 ;
            bit2 := rv (j) ;
         end if;
      end loop;
      return result;
   end;
------------------------------------------------------------------------
   function "-"(v1, v2: std_logic_vector) return std_logic_vector is
      variable i: integer := v1'length ;
      variable j: integer := v2'length ;
      alias lv: std_logic_vector (1 to i) is v1 ;
      alias rv: std_logic_vector (1 to j) is v2 ;
      variable bit1: std_logic := lv (i) ;
      variable bit2: std_logic := rv (j) ;
      variable carry:  std_logic := '0';
      variable result: std_logic_vector (0 to max (v1'length, v2'length) - 1);
   begin
      for k in result'reverse_range loop
         result (k) := bit1 xor bit2 xor carry;
         carry      := ((not bit1) and (bit2 or  carry)) or
                            (bit1  and (bit2 and carry));
         if i = 1 then
            bit1 := '0';
         else
            i := i - 1 ;
            bit1 := lv (i) ;
         end if;
         if j = 1 then
            bit2 := '0' ;
         else
            j := j - 1 ;
            bit2 := rv (j) ;
         end if;
      end loop;
      return result;
   end;

-----------------------------------------------------------------------
-- D type flip flop
--    positive edge triggered
--    no preset or clear
--
   PROCEDURE dff (
      CONSTANT data     :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      IF bitz(clk) THEN
         q <= (data and data);
      ELSIF (bitunknown(clk) and not(q = data)) THEN
         q <= 'X';
      ELSIF prising(clk) THEN
         q <= (data and data);
            -- NOTE: The AND in the above lines handles the case where
            --       data is a Z.  We want Q to be assigned to X.
      END IF;
   END dff;


------------------------------------------------------------------------
-- D type flip flop
--    positive edge triggered
--    async. clear (active high)
--
   PROCEDURE dffc (
      CONSTANT data     :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      ASSERT not( prising(clk) and pchanging(clear))
         REPORT "On DFFC: Clk and Clear both changing - Clk action suppressed";

      IF clear = '1' THEN
         q <= '0';
      ELSIF (bitunknown(clear) and not(q = '0')) THEN
         q <= 'X';
      ELSIF (bitunknown(clk) and not(q = data)) THEN
         q <= 'X';
      ELSIF bitz(clk) THEN
         q <= (data and data);
      ELSIF (prising(clk) and not(pchanging(clear))) THEN
         q <= (data and data);
            -- NOTE: The AND in the above lines handles the case where
            --       data is a Z.  We want Q to be assigned to X.
      END IF;
   END dffc;

------------------------------------------------------------------------
-- D type flip flop
--    positive edge triggered
--    async. preset (active high)
--
   PROCEDURE dffp (
      CONSTANT data     :  IN    std_ulogic;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN
      ASSERT not( prising(clk) and pchanging(preset))
         REPORT "On DFFP: Clk and Preset both changing - Clk action suppressed";

      IF preset = '1' THEN
         q <= '1';
      ELSIF (bitunknown(preset) and not(q = '1')) THEN
         q <= 'X';
      ELSIF (bitunknown(clk) and not(q = data)) THEN
         q <= 'X';
      ELSIF bitz(clk) THEN
         q <= (data and data);
      ELSIF (prising(clk) and not(pchanging(preset))) THEN
         q <= (data and data);
            -- NOTE: The AND in the above lines handles the case where
            --       data is a Z.  We want Q to be assigned to X.
      END IF;
   END dffp;

------------------------------------------------------------------------
-- D type flip flop
--    positive edge triggered
--    async. clear and preset (active high)
--
   PROCEDURE dffpc (
      CONSTANT data     :  IN    std_ulogic;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN
     ASSERT not( prising(clk) and pchanging(preset))
        REPORT "On DFFPC: Clk and Preset both changing - Clk action suppressed";

     ASSERT not( prising(clk) and pchanging(clear))
        REPORT "On DFFPC: Clk and Clear both changing - Clk action suppressed";

--   ASSERT not( pfalling(clear) and pfalling(preset))
--      REPORT "On DFFPC: Preset and clear both released at same time";

      IF clear /= '0' AND preset /= '0' THEN
         q <= 'X';
      ELSIF preset = '1' AND clear = '0' THEN
         q <= '1';
      ELSIF clear = '1' AND preset = '0' THEN
         q <= '0';
--    ELSIF (pfalling(clear) and pfalling(preset)) THEN
--       q <= 'X';
      ELSIF (bitunknown(preset) and not(q = '1')) THEN
         q <= 'X';
      ELSIF (bitunknown(clear) and not(q = '0')) THEN
         q <= 'X';
      ELSIF (bitunknown(clk) and not(q = data)) THEN
         q <= 'X';
      ELSIF bitz(clk) THEN
         q <= (data and data);
      ELSIF (prising(clk) and not(pchanging(clear) or pchanging(preset))) THEN
         q <= (data and data);
            -- NOTE: The AND in the above lines handles the case where
            --       data is a Z.  We want Q to be assigned to X.
      END IF;
   END dffpc;

------------------------------------------------------------------------
-- D type flip flop vectored register
--    register is width of passed data vector
--    positive edge triggered
--    no clear or preset
--
   PROCEDURE dff_v (
      CONSTANT data     :  IN    std_ulogic_vector;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic_vector)
   IS

   BEGIN
      IF bitz(clk) THEN
         q <= (data and data);
      ELSIF (bitx(clk) and not(q = data)) THEN
         q <= extend2c("X",data'length);
      ELSIF (prising(clk)) THEN
         q <= (data and data);
      END IF;
   END dff_v;

------------------------------------------------------------------------
-- D type flip flop vectored register
--    register is width of passed data vector
--    positive edge triggered
--    no clear or preset
--
   PROCEDURE dff_v (
      CONSTANT data     :  IN    std_logic_vector;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_logic_vector)
   IS

   BEGIN
      IF bitz(clk) THEN
         q <= (data and data);
      ELSIF (bitx(clk) and not(q = data)) THEN
         q <= extend2c("X",data'length);
      ELSIF (prising(clk)) THEN
         q <= (data and data);
      END IF;
   END dff_v;

------------------------------------------------------------------------
-- D type flip flop vectored register
--    register is width of passed data vector
--    positive edge triggered
--    async. clear (active high)
--
   PROCEDURE dffc_v (
      CONSTANT data     :  IN    std_ulogic_vector;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic_vector)
   IS

   BEGIN

     ASSERT not( prising(clk) and pchanging(clear))
       REPORT "On DFFC_V: Clk and Clear both changing - Clk action suppressed";


     IF clear = '1' THEN
       q <= data and (not data);
     ELSIF ((bitunknown(clear) and
            not(q = (data and (not data)))) or
            (bitx(clk) and not(q = data))) THEN
       q <= extend2c("X",data'length);
     ELSIF bitz(clk) THEN
       q <= (data and data);
     ELSIF (prising(clk) and not(pchanging(clear))) THEN
       q <= (data and data);
     END IF;
   END dffc_v;

------------------------------------------------------------------------
-- D type flip flop vectored register
--    register is width of passed data vector
--    positive edge triggered
--    async. clear (active high)
--
   PROCEDURE dffc_v (
      CONSTANT data     :  IN    std_logic_vector;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_logic_vector)
   IS

   BEGIN

     ASSERT not( prising(clk) and pchanging(clear))
       REPORT "On DFFC_V: Clk and Clear both changing - Clk action suppressed";


     IF clear = '1' THEN
       q <= data and (not data);
     ELSIF ((bitunknown(clear) and
            not(q =(data and (not data)))) or
            (bitx(clk) and not(q = data))) THEN
       q <= extend2c("X",data'length);
     ELSIF bitz(clk) THEN
       q <= (data and data);
     ELSIF (prising(clk) and not(pchanging(clear))) THEN
       q <= (data and data);
     END IF;
   END dffc_v;

------------------------------------------------------------------------
-- D type flip flop vectored register
--    register is width of passed data vector
--    positive edge triggered
--    async. preset (active high)
--
   PROCEDURE dffp_v (
      CONSTANT data     :  IN    std_ulogic_vector;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic_vector)
   IS

   BEGIN

     ASSERT not( prising(clk) and pchanging(preset))
       REPORT "On DFFP_V: Clk and Preset both changing - Clk action suppressed";

     IF preset = '1' THEN
       q <= data or (not data);
     ELSIF ((bitunknown(preset) and
          not(q = (data and (not data)))) or
             (bitx(clk) and not(q = data)))  THEN
       q <= extend2c("X",data'length);
     ELSIF bitz(clk) THEN
       q <= (data and data);
     ELSIF (prising(clk) and not(pchanging(preset))) THEN
       q <= (data and data);
     END IF;
   END dffp_v;

------------------------------------------------------------------------
-- D type flip flop vectored register
--    register is width of passed data vector
--    positive edge triggered
--    async. preset (active high)
--
   PROCEDURE dffp_v (
      CONSTANT data     :  IN    std_logic_vector;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_logic_vector)
   IS

   BEGIN

     ASSERT not( prising(clk) and pchanging(preset))
       REPORT "On DFFP_V: Clk and Preset both changing - Clk action suppressed";

     IF preset = '1' THEN
       q <= data or (not data);
     ELSIF ((bitunknown(preset) and
          not(q = (data and (not data)))) or
             (bitx(clk) and not(q = data)))  THEN
       q <= extend2c("X",data'length);
     ELSIF bitz(clk) THEN
       q <= (data and data);
     ELSIF (prising(clk) and not(pchanging(preset))) THEN
       q <= (data and data);
     END IF;
   END dffp_v;

------------------------------------------------------------------------
-- D type flip flop vectored register
--    register is width of passed data vector
--    positive edge triggered
--    async. clear and preset (active high)
--
   PROCEDURE dffpc_v (
      CONSTANT data     :  IN    std_ulogic_vector;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic_vector)
   IS

   BEGIN
     ASSERT not( prising(clk) and pchanging(preset))
       REPORT "On DFFPC_V: Clk and Preset both changing - Clk action suppressed"
;

     ASSERT not( prising(clk) and pchanging(clear))
       REPORT "On DFFPC_V: Clk and Clear both changing - Clk action suppressed";

--      ASSERT not( pfalling(clear) and pfalling(preset))
--         REPORT "On DFFPC: Preset and clear both released at same time";

     IF preset /= '0' AND clear /= '0' THEN
        q <= extend2c("X", data'length);
     ELSIF preset = '1' AND clear = '0' THEN
        q <= data or (not data);
     ELSIF clear = '1' AND preset = '0' THEN
        q <= data and (not data);
     ELSIF ((bitunknown(preset) and 
          not(q = (data or not data))) or
          (bitunknown(clear) and not(q = (data and 
          (not data)) )) or (bitx(clk) and not(q = data)) or
          (pfalling(clear) and pfalling(preset))) THEN
               q <= extend2c("X",data'length);
     ELSIF bitz(clk) THEN
        q <= (data and data);
     ELSIF (prising(clk) and not(pchanging(clear) or pchanging(preset))) THEN
        q <= (data and data);
     END IF;
   END dffpc_v;

------------------------------------------------------------------------
-- D type flip flop vectored register
--    register is width of passed data vector
--    positive edge triggered
--    async. clear and preset (active high)
--
   PROCEDURE dffpc_v (
      CONSTANT data     :  IN    std_logic_vector;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_logic_vector)
   IS

   BEGIN
     ASSERT not( prising(clk) and pchanging(preset))
       REPORT "On DFFPC_V: Clk and Preset both changing - Clk action suppressed"
;

     ASSERT not( prising(clk) and pchanging(clear))
       REPORT "On DFFPC_V: Clk and Clear both changing - Clk action suppressed";

--      ASSERT not( pfalling(clear) and pfalling(preset))
--         REPORT "On DFFPC: Preset and clear both released at same time";

     IF preset /= '0' AND clear /= '0' THEN
        q <= extend2c("X", data'length);
     ELSIF preset = '1' AND clear = '0' THEN
        q <= data or (not data);
     ELSIF clear = '1' AND preset = '0' THEN
        q <= data and not data;
     ELSIF ((bitunknown(preset) and not(q = (data or (not data)))) or
          (bitunknown(clear) and not(q = (data and (not data)))) or 
          (bitx(clk) and not(q = data)) or
          (pfalling(clear) and pfalling(preset))) THEN
               q <= extend2c("X",data'length);
     ELSIF bitz(clk) THEN
        q <= (data and data);
     ELSIF (prising(clk) and not(pchanging(clear) or pchanging(preset))) THEN
        q <= (data and data);
     END IF;
   END dffpc_v;

------------------------------------------------------------------------
-- DLATCH
--    positive transparent
--    async. clear (active high)
--
   PROCEDURE dlatch (
      CONSTANT data     :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      IF (bitunknown(gate) and not(q = data)) THEN
         q <= 'X';
      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (gate = '1' ) THEN
         q <= (data and data);
            -- NOTE: The AND in the above lines handles the case where
            --       data is a Z.  We want Q to be assigned to X.
      END IF;
   END dlatch;

------------------------------------------------------------------------
-- DLATCH
--    positive transparent
--    async. clear (active high)
--
   PROCEDURE dlatchc (
      CONSTANT data     :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      IF clear = '1' THEN
         q <= '0';
      ELSIF (bitunknown(clear) and not(q = '0')) THEN
         q <= 'X';
      ELSIF (bitunknown(gate) and not(q = data)) THEN
         q <= 'X';
      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (bitunknown(data) and (gate = '1')) THEN
         q <= 'X';
      ELSIF (gate = '1' ) THEN
         q <= (data and data);
            -- NOTE: The AND in the above lines handles the case where
            --       data is a Z.  We want Q to be assigned to X.
      END IF;
   END dlatchc;

------------------------------------------------------------------------
-- DLATCH
--    positive transparent
--    async. preset (active high)
--
   PROCEDURE dlatchp (
      CONSTANT data     :  IN    std_ulogic;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      IF preset = '1' THEN
         q <= '1';
      ELSIF (bitunknown(preset) and not(q = '1')) THEN
         q <= 'X';
      ELSIF (bitunknown(gate) and not(q = data)) THEN
         q <= 'X';
      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (bitunknown(data) and (gate = '1')) THEN
         q <= 'X';
      ELSIF (gate = '1' ) THEN
         q <= (data and data);
            -- NOTE: The AND in the above lines handles the case where
            --       data is a Z.  We want Q to be assigned to X.
      END IF;
   END dlatchp;
------------------------------------------------------------------------

-- DLATCH
--    positive transparent
--    async. clear and preset (active high)
--
   PROCEDURE dlatchpc (
      CONSTANT data     :  IN    std_ulogic;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      IF preset /= '0' and clear /= '0' THEN
         q <= 'X';
      ELSIF preset = '1' THEN
         q <= '1';
      ELSIF clear = '1' THEN
         q <= '0';
--      ELSIF (pfalling(clear) and pfalling(preset) and (gate /= '0')) THEN
--         q <= 'X';
      ELSIF (bitunknown(preset) and not(q = '1')) THEN
         q <= 'X';
      ELSIF (bitunknown(clear) and not(q = '0')) THEN
         q <= 'X';
      ELSIF (bitunknown(gate) and not(q = data)) THEN
         q <= 'X';
      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (bitunknown(data) and (gate = '1')) THEN
         q <= 'X';
      ELSIF (gate = '1' ) THEN
         q <= (data and data);
            -- NOTE: The AND in the above lines handles the case where
            --       data is a Z.  We want Q to be assigned to X.
      END IF;
   END dlatchpc;
------------------------------------------------------------------------
-- D type latch vectored register
--    register is width of passed data vector
--    positive transparent
--
   PROCEDURE dlatch_v (
      CONSTANT data     :  IN    std_ulogic_vector;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic_vector)
   IS

   BEGIN

      IF (bitx(gate) and not(q = data)) THEN
               q <= extend2c("X",data'length);

      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (gate = '1') THEN
         q <= (data and data);
      END IF;
   END dlatch_v;

------------------------------------------------------------------------
-- D type latch vectored register
--    register is width of passed data vector
--    positive transparent
--
   PROCEDURE dlatch_v (
      CONSTANT data     :  IN    std_logic_vector;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_logic_vector)
   IS

   BEGIN

      IF (bitx(gate) and not(q = data)) THEN
               q <= extend2c("X",data'length);

      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (gate = '1') THEN
         q <= (data and data);
      END IF;
   END dlatch_v;

------------------------------------------------------------------------
-- D type latch vectored register
--    register is width of passed data vector
--    positive transparent
--    async. clear (active high)
--
   PROCEDURE dlatchc_v (
      CONSTANT data     :  IN    std_ulogic_vector;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic_vector)
   IS

   BEGIN

     IF clear = '1' THEN
       q <= data and not data;
     ELSIF ((bitunknown(clear) and not(q = (data and not data))) or
           (vecunknown(data) and (gate = '1')) or
           (bitx(gate) and not(q = data)) ) THEN
       q <= extend2c("X",data'length);
     ELSIF bitz(gate) THEN
       q <= (data and data);
     ELSIF (gate = '1') THEN
       q <= (data and data);
     END IF;
   END dlatchc_v;

------------------------------------------------------------------------
-- D type latch vectored register
--    register is width of passed data vector
--    positive transparent
--    async. clear (active high)
--
   PROCEDURE dlatchc_v (
      CONSTANT data     :  IN    std_logic_vector;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_logic_vector)
   IS

   BEGIN

     IF clear = '1' THEN
       q <= data and not data;
     ELSIF ((bitunknown(clear) and not(q = (data and not data))) or
           (vecunknown(data) and (gate = '1')) or
           (bitx(gate) and not(q = data)) ) THEN
       q <= extend2c("X",data'length);
     ELSIF bitz(gate) THEN
       q <= (data and data);
     ELSIF (gate = '1') THEN
       q <= (data and data);
     END IF;
   END dlatchc_v;

------------------------------------------------------------------------
-- D type latch vectored register
--    register is width of passed data vector
--    positive transparent
--    async. preset (active high)
--
   PROCEDURE dlatchp_v (
      CONSTANT data     :  IN    std_ulogic_vector;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic_vector)
   IS

   BEGIN

      IF preset = '1' THEN
        q <= data or (not data);
      ELSIF ((bitunknown(preset) and not(q = (data or not data) )) or
             (vecunknown(data) and (gate = '1')) or
             (bitx(gate) and not(q = data)) ) THEN
                q <= extend2c("X",data'length);

      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (gate = '1') THEN
         q <= (data and data);
      END IF;
   END dlatchp_v;

------------------------------------------------------------------------
-- D type latch vectored register
--    register is width of passed data vector
--    positive transparent
--    async. preset (active high)
--
   PROCEDURE dlatchp_v (
      CONSTANT data     :  IN    std_logic_vector;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_logic_vector)
   IS

   BEGIN

      IF preset = '1' THEN
        q <= data or (not data);
      ELSIF ((bitunknown(preset) and not(q = (data or not data) )) or
             (vecunknown(data) and (gate = '1')) or
             (bitx(gate) and not(q = data)) ) THEN
                q <= extend2c("X",data'length);

      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (gate = '1') THEN
         q <= (data and data);
      END IF;
   END dlatchp_v;

------------------------------------------------------------------------
-- D type latch vectored register
--    register is width of passed data vector
--    positive transparent
--    async. clear and preset (active high)
--
   PROCEDURE dlatchpc_v (
      CONSTANT data     :  IN    std_ulogic_vector;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic_vector)
   IS

   BEGIN

      IF clear /= '0' and preset /= '0' THEN
         q <= extend2c("X", data'length);
      ELSIF preset = '1' THEN
         q <= data or (not data);
      ELSIF clear = '1' THEN
         q <= data and (not data);
      ELSIF ((bitunknown(preset) and not(q = (data or (not data) ))) or
             (bitunknown(clear) and not(q = (data and (not data)))) or
             (vecunknown(data) and (gate = '1')) or
             (bitx(gate) and not(q = data)) or
             (pfalling(clear) and pfalling(preset))) THEN
                q <= extend2c("X",data'length);

      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (gate = '1') THEN
         q <= (data and data);
      END IF;
   END dlatchpc_v;

------------------------------------------------------------------------
-- D type latch vectored register
--    register is width of passed data vector
--    positive transparent
--    async. clear and preset (active high)
--
   PROCEDURE dlatchpc_v (
      CONSTANT data     :  IN    std_logic_vector;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   gate     :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_logic_vector)
   IS

   BEGIN

      IF clear /= '0' and preset /= '0' THEN
         q <= extend2c("X", data'length);
      ELSIF preset = '1' THEN
         q <= data or (not data);
      ELSIF clear = '1' THEN
         q <= data and (not data);
      ELSIF ((bitunknown(preset) and not(q = (data or (not data) ))) or
             (bitunknown(clear) and not(q = (data and (not data)))) or
             (vecunknown(data) and (gate = '1')) or
             (bitx(gate) and not(q = data)) or
             (pfalling(clear) and pfalling(preset))) THEN
                q <= extend2c("X",data'length);

      ELSIF bitz(gate) THEN
         q <= (data and data);
      ELSIF (gate = '1') THEN
         q <= (data and data);
      END IF;
   END dlatchpc_v;


------------------------------------------------------------------------
-- SR LATCH
--    positive transparent
--    async. clear and preset (active high)
--
   PROCEDURE srlatch (
      SIGNAL   set      :  IN    std_ulogic;
      SIGNAL   reset    :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      ASSERT not( pfalling(reset) and pfalling(set))
         REPORT "On SRLATCH: Set and reset both released at same time";

      IF set = '1' THEN
         q <= '1';
      ELSIF reset = '1' THEN
         q <= '0';
      ELSIF (pfalling(reset) and pfalling(set)) THEN
         q <= 'X';
      ELSIF (bitunknown(reset) and bitunknown(set)) THEN
         q <= 'X';
      ELSIF (bitunknown(set) and not(q = '1')) THEN
         q <= 'X';
      ELSIF (bitunknown(reset) and not(q = '0')) THEN
         q <= 'X';
      END IF;
   END srlatch;

------------------------------------------------------------------------
-- JK flip flop
--    positive edge triggered
--    no clear or preset
--
   PROCEDURE jkff (
      SIGNAL   j        :  IN    std_ulogic;
      SIGNAL   k        :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      IF (bitunknown(clk)) THEN
         q <= 'X';
      ELSIF prising (clk) THEN

         IF (j = '1' AND k = '1') THEN
            q <= NOT q;
         ELSIF (j = '1' AND k = '0') THEN
            q <= '1';
         ELSIF (j = '0' AND k = '1') THEN
            q <= '0';
         ELSIF (bitunknown(j) OR bitunknown(k)) THEN
            q <= 'X';
         END IF;
      END IF;
   END jkff;

------------------------------------------------------------------------
-- JK flip flop
--    positive edge triggered
--    async. clear (active high)
--
   PROCEDURE jkffc (
      SIGNAL   j        :  IN    std_ulogic;
      SIGNAL   k        :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      ASSERT not( prising(clk) and pchanging(clear))
         REPORT "On JKFFC: Clk and Clear both changing - Clk action suppressed";

      IF clear = '1' THEN
         q <= '0';
      ELSIF bitunknown(clk) THEN
         q <= 'X';
      ELSIF (bitunknown(clear) and not(q = '0')) THEN
         q <= 'X';
      ELSIF (prising(clk) and not(pchanging(clear))) THEN
         IF (j = '1' AND k = '1') THEN
            q <= NOT q;
         ELSIF (j = '1' AND k = '0') THEN
            q <= '1';
         ELSIF (j = '0' AND k = '1') THEN
            q <= '0';
         ELSIF (bitunknown(j)  OR bitunknown(k)) THEN
            q <= 'X';
         END IF;
      END IF;
   END jkffc;

------------------------------------------------------------------------
-- JK flip flop
--    positive edge triggered
--    async. preset (active high)
--
   PROCEDURE jkffp (
      SIGNAL   j        :  IN    std_ulogic;
      SIGNAL   k        :  IN    std_ulogic;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

     ASSERT not( prising(clk) and pchanging(preset))
       REPORT "On JKFFP_V: Clk and Preset both changing - Clk action suppressed";

      IF preset = '1' THEN
         q <= '1';
      ELSIF bitunknown(clk) THEN
         q <= 'X';
      ELSIF (bitunknown(preset) and not(q = '1')) THEN
         q <= 'X';
      ELSIF (prising(clk) and not(pchanging(preset))) THEN
         IF (j = '1' AND k = '1') THEN
            q <= NOT q;
         ELSIF (j = '1' AND k = '0') THEN
            q <= '1';
         ELSIF (j = '0' AND k = '1') THEN
            q <= '0';
         ELSIF (bitunknown(j)  OR bitunknown(k)) THEN
            q <= 'X';
         END IF;
      END IF;
   END jkffp;

------------------------------------------------------------------------
-- JK flip flop
--    positive edge triggered
--    async. clear and preset (active high)
--
   PROCEDURE jkffpc (
      SIGNAL   j        :  IN    std_ulogic;
      SIGNAL   k        :  IN    std_ulogic;
      SIGNAL   preset   :  IN    std_ulogic;
      SIGNAL   clear    :  IN    std_ulogic;
      SIGNAL   clk      :  IN    std_ulogic;
      SIGNAL   q        :  INOUT std_ulogic)
   IS

   BEGIN

      ASSERT not( prising(clk) and pchanging(preset))
         REPORT "On JKFFPC: Clk and Preset both changing - Clk action suppressed";
      ASSERT not( prising(clk) and pchanging(clear))
         REPORT "On JKFFPC: Clk and Clear both changing - Clk action suppressed";

--    ASSERT not( pfalling(preset) and pfalling(clear))
--       REPORT "On JKFFPC: Preset and Clear both released at same time";

      IF preset /= '0' AND clear /= '0' THEN
         q <= 'X';
      ELSIF preset = '1' THEN
         q <= '1';
      ELSIF clear = '1' THEN
         q <= '0';
      ELSIF bitunknown(clk) THEN
         q <= 'X';
      ELSIF (bitunknown(preset) and not(q = '1')) THEN
         q <= 'X';
      ELSIF (bitunknown(clear) and not(q = '0')) THEN
         q <= 'X';
      ELSIF (prising(clk) and not(pchanging(clear) or pchanging(preset))) THEN
         IF (j = '1' AND k = '1') THEN
            q <= NOT q;
         ELSIF (j = '1' AND k = '0') THEN
            q <= '1';
         ELSIF (j = '0' AND k = '1') THEN
            q <= '0';
         ELSIF (bitunknown(j)  OR bitunknown(k)) THEN
            q <= 'X';
         END IF;
      END IF;
   END jkffpc;

------------------------------------------------------------------------
--    for VdesII pla synthesis.  Remember to run filter on vhdl file
--    to transform constants:  "1ZX0"&"001X"
--                      into:  "1ZX0001X"
--
    procedure pla_table(         I: in  std_ulogic_vector;
                          signal O: out std_ulogic_vector;
                               tbl: in  std_ulogic_2d) is
        variable term     : std_ulogic;
        variable idx, inc : integer;
    begin
--
-- check bad tbl
--
        if ( (I'length+O'length) /= tbl'length(2) ) then
                assert false report "bad pla_table" severity failure;
                return;
        end if;
--
-- get direction for 2nd index
--
        if ( tbl'left(2) < tbl'right(2) ) then
            inc := 1;
        else
            inc := -1;
        end if;
--
-- init O to 0's
--
        l0: for j  in O'range loop
            O(j) <= '0';
        end loop;
        l1: for k in tbl'range(1) loop  -- For the number of rows
--
-- get and plane
--
            term := '1';
            idx := tbl'left(2);
            l2: for j in I'range loop
                if (tbl(k,idx) = '1') then
                    term := term and I(j);
                elsif (tbl(k,idx) = '0') then
                    term := term and not(I(j));
--              else
--                  >>>*** input don't care ***<<<
--                  HANDLED CORRECTLY BY SKIPPING HERE!!!
                end if;
                idx := idx + inc;
            end loop;
--
-- get or plane
--
            if (term = '1') then -- If the AND is true: output.
                if ( inc = 1 ) then
                    idx := tbl'left(2) + I'length;
                else
                    idx := tbl'left(2) - I'length;
                end if;
                l3: for j  in O'range loop
                    if ('1'= tbl(k, idx)) then
                        O(j) <= '1';
--                  elsif ( DONT_CARE(tbl(k, idx)) ) then
--                      >>>*** output don't care ***<<<
--                      NOT HANDLED AND TREATED AS ZEROS
                    end if;
                    idx := idx + inc;
                end loop;
            end if;
        end loop;
    end pla_table;
------------------------------------------------------------------------
--    for VdesII pla synthesis.  Remember to run filter on vhdl file
--    to transform constants:  "1ZX0"&"001X"
--                      into:  "1ZX0001X"
--
    procedure pla_table(         I: in  std_logic_vector;
                          signal O: out std_logic_vector;
                               tbl: in  std_logic_2d) is
        variable term     : std_ulogic;
        variable idx, inc : integer;
    begin
--
-- check bad tbl
--
        if ( (I'length+O'length) /= tbl'length(2) ) then
                assert false report "bad pla_table" severity failure;
                return;
        end if;
--
-- get direction for 2nd index
--
        if ( tbl'left(2) < tbl'right(2) ) then
            inc := 1;
        else
            inc := -1;
        end if;
--
-- init O to 0's
--
        l0: for j  in O'range loop
            O(j) <= '0';
        end loop;
        l1: for k in tbl'range(1) loop  -- For the number of rows
--
-- get and plane
--
            term := '1';
            idx := tbl'left(2);
            l2: for j in I'range loop
                if (tbl(k,idx) = '1') then
                    term := term and I(j);
                elsif (tbl(k,idx) = '0') then
                    term := term and not(I(j));
--              else
--                  >>>*** input don't care ***<<<
--                  HANDLED CORRECTLY BY SKIPPING HERE!!!
                end if;
                idx := idx + inc;
            end loop;
--
-- get or plane
--
            if (term = '1') then -- If the AND is true: output.
                if ( inc = 1 ) then
                    idx := tbl'left(2) + I'length;
                else
                    idx := tbl'left(2) - I'length;
                end if;
                l3: for j  in O'range loop
                    if ('1'= tbl(k, idx)) then
                        O(j) <= '1';
--                  elsif ( DONT_CARE(tbl(k, idx)) ) then
--                      >>>*** output don't care ***<<<
--                      NOT HANDLED AND TREATED AS ZEROS
                    end if;
                    idx := idx + inc;
                end loop;
            end if;
        end loop;
    end pla_table;

end pack1164;
------------------------------------------------------------------------
------------------------------------------------------------------------



