
--
--                    Proprietary and Confidential
--                  Copyright 1994 Viewlogic Systems Inc.
-- 
--  This is an unpublished work and the information contained herein 
--  is proprietary and confidential to Viewlogic Systems Inc.    Any 
--  reproduction of this software, in any form, is prohibited except 
--  as otherwise specifically agreed to in writing. This unpublished
--  work is protected by trade secrets law and federal copyright law
--  and all rights thereunder are reserved.
--
--  Revision History:
-- Revision 1.4  1994/09/15             jrs
--      Jim Chiu added dff with enable models:
--              viewtech_fd[1-3]le
--      Jay Southard made some modifications on viewtech_fd1le to
--              implement some workarounds:
--              SCOPUS bug #16283, bug#16290.
--  $Log: viewtech-lib.vhd,v $
-- Revision 1.3  1994/06/09  13:05:23  doug
--   Scopus #: 0 - Correction to LD3. Added more cells. May be complete now.
--
-- Revision 1.2  94/05/27  08:59:57  doug
--   Scopus #: 0 - Added complete list of ViewTech cells. Some still to be modeled.
-- 
-- Revision 1.1  94/05/27  08:34:20  doug
--   Scopus #: 0 - Initial module creation.
-- 
-- Update: 26-May-94 Corrected NOR4 model.
-- Update: 25-May-94 Changed latch models to use latch procedures.
--
-- NOTE: Not all ViewTech cells have been modeled!
--    Cell_name     	Model_present
--    VIEWTECH_ADD_AB 	JS (VS maps to EO and AN2)
--    VIEWTECH_ADD_ABC	DP (VS reduces to gates)
--    VIEWTECH_AND2     DP
--    VIEWTECH_AND3     DP
--    VIEWTECH_AND4     DP (maps to two ND2 and a NR2)
--    VIEWTECH_BUF  	JS  
--    VIEWTECH_FD1   	JS 
--    VIEWTECH_FD1LE    JC/JS
--    VIEWTECH_FD2    	DP
--    VIEWTECH_FD2LE    JC
--    VIEWTECH_FD3      DP
--    VIEWTECH_FD3LE    JC/JS
--    VIEWTECH_FD4    	DP
--    VIEWTECH_FD4LE    JC
--    VIEWTECH_LD1	DP
--    VIEWTECH_LD2	DP (VS maps to LD1 and inverter!)
--    VIEWTECH_LD3	DP 
--    VIEWTECH_LD4      DP (VS maps to LD3 and inverter!)
--    VIEWTECH_MUX2     -- currently not used by ViewArchitect
--    VIEWTECH_NAND2  	JS
--    VIEWTECH_NAND3  	JS
--    VIEWTECH_NAND4  	JS
--    VIEWTECH_NAND5  	JS
--    VIEWTECH_NAND8    -- currently not used by ViewArchitect
--    VIEWTECH_NOR2   	DP
--    VIEWTECH_NOR3   	JS
--    VIEWTECH_NOR4   	JS
--    VIEWTECH_NOR5   	JS
--    VIEWTECH_NOR8     -- currently not used by ViewArchitect
--    VIEWTECH_NOT    	JS
--    VIEWTECH_OR2   	DP
--    VIEWTECH_OR3      DP
--    VIEWTECH_OR4      DP (VS maps to 2 NORs and a NAND)
--    VIEWTECH_TBUF   	JS
--    VIEWTECH_XNOR2    DP
--    VIEWTECH_XNOR3    DP (VS maps to mpx, xor and inv)
--    VIEWTECH_XOR2   	JS

-- ------------------------------------------------------------------------
entity viewtech_tbuf is
        port (
            A: in VLBIT;
            E: in VLBIT;
            Z: out VLBIT );
end VIEWTECH_TBUF;

architecture behv of viewtech_tbuf is
begin
  z <= a when (e = '1') else
       'Z';
end behv;

-- ------------------------------------------------------------------------
entity viewtech_not is
        port (
            A: in VLBIT;
            Z: out VLBIT );
end viewtech_not;
architecture behv of VIEWTECH_NOT is
    begin
        Z <= NOT A;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_buf is
        port (
            A: in VLBIT;
            Z: out VLBIT );
end viewtech_buf;
architecture behv of VIEWTECH_buf is
    begin
        Z <= A;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_nor2 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            Z: out VLBIT );
end viewtech_nor2;
architecture behv of viewtech_nor2 is
    begin
        Z <= NOT (A OR B);
end behv;

-- ------------------------------------------------------------------------
entity viewtech_nand2 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            Z: out VLBIT );
end viewtech_nand2;
architecture behv of VIEWTECH_NAND2 is
    begin
        Z <= NOT (A AND B);
end behv;

-- ------------------------------------------------------------------------
entity viewtech_nor3 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            Z: out VLBIT );
end viewtech_nor3;
architecture behv of VIEWTECH_NOR3 is
begin
        Z <= NOT (A OR B OR C);
end behv;

-- ------------------------------------------------------------------------
entity viewtech_nand3 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            Z: out VLBIT );
end viewtech_nand3;
architecture behv of VIEWTECH_NAND3 is
begin
        Z <= NOT (A AND B AND C);
end behv;

-- ------------------------------------------------------------------------
entity viewtech_nand4 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            D: in VLBIT;
            Z: out VLBIT );
end viewtech_nand4;
architecture behv of viewtech_nand4 is
begin
        Z <= NOT ((A AND B) AND (C AND D));
end behv;

-- ------------------------------------------------------------------------
entity viewtech_nor4 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            D: in VLBIT;
            Z: out VLBIT );
end viewtech_nor4;
architecture behv of viewtech_nor4 is
begin
        Z <= NOT (A OR B OR C OR D);
end behv;

-- ------------------------------------------------------------------------
entity viewtech_nand5 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            D: in VLBIT;
            E: in VLBIT;
            Z: out VLBIT );
end viewtech_nand5;
architecture behv of VIEWTECH_NAND5 is
begin
        Z <= NOT ((A AND B) AND (C AND D AND E));
end behv;

-- ------------------------------------------------------------------------
entity viewtech_nor5 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            D: in VLBIT;
            E: in VLBIT;
            Z: out VLBIT );
end viewtech_nor5;
architecture behv of VIEWTECH_NOR5 is
begin
        Z <= NOT ((A OR B) OR (C OR D OR E));
end behv;

-- ------------------------------------------------------------------------
entity viewtech_fd1 is
        port (
            D: in VLBIT;
            CP: in VLBIT;
            Q: out VLBIT;
            QN: out VLBIT );
end viewtech_fd1;
architecture behv of viewtech_fd1 is
  signal next_q: vlbit;
begin
  q <= next_q;
  qn <= not next_q;
process
  begin
  wait until (cp = '1');
  next_q <= D;
  end process;
end behv;   

-- ------------------------------------------------------------------------
entity viewtech_fd2 is
        port (
            D: in VLBIT;
            CP: in VLBIT;
            CD: in VLBIT;
            Q: out VLBIT;
            QN: out VLBIT );
end VIEWTECH_FD2;

architecture behv of viewtech_fd2 is
  signal next_q: vlbit;
begin
  q <= next_q;
  qn <= not next_q;
process
  begin
  wait until ((cd = '0') or (cp'event and (cp = '1')));
  if (cd = '0') then
     next_q <= '0';
  else
     next_q <= D;
  end if;
end process;
end behv;   

-- ------------------------------------------------------------------------
entity viewtech_fd4 is
        port (
            D: in VLBIT;
            CP: in VLBIT;
            SD: in VLBIT;
            Q: out VLBIT;
            QN: out VLBIT );
end VIEWTECH_FD4;

architecture behv of viewtech_fd4 is
  signal next_q: vlbit;
begin
  q <= next_q;
  qn <= not next_q;
process
  begin
  wait until ((sd = '0') or (cp'event and (cp = '1')));
  if (sd = '0') then
     next_q <= '1';
  else
     next_q <= D;
  end if;
end process;
end behv;   

-- ------------------------------------------------------------------------
entity viewtech_add_ab is
        port (
            A: in VLBIT;
            B: in VLBIT;
            S: out VLBIT;
            COUT: out VLBIT );
end viewtech_add_ab;
architecture behv of viewtech_add_ab is
begin
  S <= A XOR B;
  COUT <= A AND B;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_add_abc is
        port (
            C: in VLBIT;
            A: in VLBIT;
            B: in VLBIT;
            S: out VLBIT;
            COUT: out VLBIT );
end viewtech_add_abc;
architecture behv of viewtech_add_abc is
signal tmp : vlbit;
begin
  tmp <= A XOR B;
  S <= c XOR tmp;
  COUT <= (A AND B) or (tmp and c);
end behv;

-- ------------------------------------------------------------------------
entity viewtech_xor2 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            Z: out VLBIT );
end viewtech_xor2;
architecture behv of viewtech_xor2 is
begin
  Z <= A XOR B;
end behv;

-- ------------------------------------------------------------------------
-- Modified to use latch procedure. [doug:19940525.0858]
entity viewtech_ld1 is
        port (
            D: in VLBIT;
            G: in VLBIT;
            Q: out VLBIT;
            QN: out VLBIT );
end VIEWTECH_LD1;
architecture behv of viewtech_ld1 is
signal state : vlbit;
begin
  dlatch(d, g, state);
  q <= state;
  qn <= not state;
end behv;

-- ------------------------------------------------------------------------
-- Why does this not map to an LD2 cell with ViewSynthesis?
-- Modified to use latch procedure. [doug:19940525.0858]
entity viewtech_ld2 is
        port (
            D: in VLBIT;
            GN: in VLBIT;
            Q: out VLBIT;
            QN: out VLBIT );
end VIEWTECH_LD2;
architecture behv of viewtech_ld2 is
signal state,gate : vlbit;
begin
  dlatch(d, gate, state);
  gate <= not gn;
  q <= state;
  qn <= not state;
end behv;

-- ------------------------------------------------------------------------
-- Modified to use latch procedure. [doug:19940525.0858]
entity viewtech_ld3 is
        port (
            D: in VLBIT;
            G: in VLBIT;
            CD: in VLBIT;
            Q: out VLBIT;
            QN: out VLBIT );
end VIEWTECH_LD3;
architecture behv of viewtech_ld3 is
  signal state, clear : vlbit;
begin
  dlatchc(d, clear, g, state);
  clear <= not cd;
  q <= state;
  qn <= not state;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_ld4 is
        port (
            D: in VLBIT;
            GN: in VLBIT;
            CD: in VLBIT;
            Q: out VLBIT;
            QN: out VLBIT );
end VIEWTECH_LD4;
architecture behv of viewtech_ld4 is
  signal state, clear, gate : vlbit;
begin
  dlatchc(d, clear, gate, state);
  clear <= not cd;
  gate <= not gn;
  q <= state;
  qn <= not state;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_or2 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            Z: out VLBIT );
end viewtech_or2;
architecture behv of viewtech_or2 is
    begin
        Z <= A OR B;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_or3 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            Z: out VLBIT );
end viewtech_or3;
architecture behv of viewtech_or3 is
    begin
        Z <= A OR B OR C;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_or4 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            D: in VLBIT;
            Z: out VLBIT );
end viewtech_or4;
architecture behv of viewtech_or4 is
    begin
        Z <= A OR B OR C OR D;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_and2 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            Z: out VLBIT );
end viewtech_and2;
architecture behv of VIEWTECH_AND2 is
    begin
        Z <= (A AND B);
end behv;

-- ------------------------------------------------------------------------
entity viewtech_and3 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            Z: out VLBIT );
end viewtech_and3;
architecture behv of VIEWTECH_AND3 is
    begin
        Z <= A AND B AND C;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_and4 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            D: in VLBIT;
            Z: out VLBIT );
end viewtech_and4;
architecture behv of VIEWTECH_AND4 is
    begin
        Z <= ((A AND B) AND (C AND D));
end behv;

-- ------------------------------------------------------------------------
entity viewtech_fd3 is
        port (
            D: in VLBIT;
            CP: in VLBIT;
            CD: in VLBIT;
            SD: in VLBIT;
            Q: out VLBIT;
            QN: out VLBIT );
end VIEWTECH_FD3;
architecture behv of viewtech_FD3 is
  signal state, clear, preset : vlbit;
begin
  dffpc(d, preset, clear, cp, state);
  clear <= not cd;
  preset <= not sd;
  q <= state;
  qn <= not state;
end behv;

-- ------------------------------------------------------------------------
entity viewtech_xnor2 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            Z: out VLBIT );
end viewtech_xnor2;
architecture behv of viewtech_xnor2 is
begin
  Z <= not (A XOR B);
end behv;

-- ------------------------------------------------------------------------
entity viewtech_xnor3 is
        port (
            A: in VLBIT;
            B: in VLBIT;
            C: in VLBIT;
            Z: out VLBIT );
end viewtech_xnor3;
architecture behv of viewtech_xnor3 is
begin
  Z <= not (A XOR B XOR C);
end behv;

-- ------------------------------------------------------------------------
-- Load-enable flops : 4 variations 
-- General truth table:
--         data  load-enable  clock  set  reset   q   qbar
--          x        x          x     1      0     1    0
--          x        x          x     0      1     0    1
--          x        x          x     1      1     0    0
--      due to Xilinx limitations, SR is not useful
--          0        1          ^     0      0     0    1
--          1        1          ^     0      0     1    0
--          x        0          ^     0      0     q    q'
--
entity viewtech_fd1le is
  port (
    D, CP, LE: in  VLBIT;
    Q, QN              : out VLBIT);
end viewtech_fd1le;

architecture behv of viewtech_fd1le is
  signal tmp: VLBIT;
begin
  process
  begin
  wait until (cp'event and cp='1');
      if (le='1') then
        tmp <= d;
      end if;
  end process;

  q <= tmp;
  qn <= not tmp;
end behv;


entity viewtech_fd2le is
  port (
    D, CP, CD, LE: in  VLBIT;
    Q, QN              : out VLBIT);
end viewtech_fd2le;

architecture behv of viewtech_fd2le is
  signal tmp: VLBIT;
begin
  process(cd, cp)
  begin
    if (cd='1') then   
      tmp <= '0';
    elsif (cp'event and cp='1') then 
      if (le='1') then
        tmp <= d;
      end if;
    end if;
  end process;

  q <= tmp;
  qn <= not tmp;
end behv;


entity viewtech_fd3le is
  port (
    D, CP, CD, SD, LE: in  VLBIT;
    Q, QN              : out VLBIT);
end viewtech_fd3le;

architecture behv of viewtech_fd3le is
  signal tmp: VLBIT;
begin
  process
  begin
  wait until ((cp'event and cp='1') OR (sd='1') OR (cd='1'));
    if (sd ='1') then 
      tmp <= '1';
    elsif (cd='1') then   
      tmp <= '0';
    else
      if (le='1') then
        tmp <= d;
      end if;
    end if;
  end process;

  q <= tmp;
  qn <= not tmp;
end behv;


entity viewtech_fd4le is
  port (
    D, CP, SD, LE: in  VLBIT;
    Q, QN              : out VLBIT);
end viewtech_fd4le;

architecture behv of viewtech_fd4le is
  signal tmp: VLBIT;
begin
  q <= tmp;
  qn <= not tmp;
  process(sd, cp)
  begin
    if (sd='1') then 
      tmp <= '1';
    elsif (cp'event and cp='1') then 
      if (le='1') then
        tmp <= d;
      end if;
    end if;
  end process;
end behv;

