;;;;
;;;; $RCSfile: vllm.tmp,v $ $Revision: 1.6 $
;;;;
;;;; DESCRIPTION
;;;;      Viewlogic Library Management (VLLM) initialization file.
;;;;
;;;; SEE ALSO
;;;;      Chapter 2, ViewLibrarian User's Guide or
;;;;      Chapter 2, ViewDatabook II User's Guide.
;;;;
;;;; Copyright (c) 1994,1995 by Viewlogic Systems, Inc.
;;;;


;;;
;;; NAME
;;;      DefineAttr -- define an attribute
;;;
;;; SYNOPSIS
;;;      (Process 'DefineAttr attr_name vllmBase|Float [ units_string ] )
;;;
;;;      (Process 'DefineAttr attr_name vllmBase|String )
;;;
;;;          attr_name    -- vlStringVT; attribute name
;;;          units_string -- optional vlStringVT; if present, allows the
;;;                          attribute value to be expressed with trailing
;;;                          units, e.g. "5watts" as opposed to "5".  If
;;;                          present, other units are not accepted.  If
;;;                          omitted, any units will be accepted.
;;;
;;; DESCRIPTION
;;;      Define the type and units characteristics of an attribute.
;;;
;;; EXAMPLE
;;;      (Process 'DefineAttr "C_VALUE" #<vllmBaseVT Float> "Farads")
;;;


;;;
;;; NAME
;;;      AddLibrary -- define a library
;;;
;;; SYNOPSIS
;;;      (Process 'AddLibrary library_name allow_eclipsing
;;;               database_list
;;;               global_view_list)
;;;
;;;          library_name     -- vlStringVT; name of the library
;;;          allow_eclipsing  -- boolVT; flag indicating whether views
;;;                              within this library can be eclipsed.  #t
;;;                              means that eclipsing is allowed, #f means
;;;                              eclipsing is not allowed, and ViewLibrary
;;;                              will alert you to any attempts to redefine
;;;                              a view
;;;          database_list    -- consVT; list of lists, each element has the
;;;                              form `(db_name file_name location class)'.
;;;                              `Location' and `class' are VDM concepts.
;;;                              `location' is defined with 'AddLocation in
;;;                              the "vpm.scm" file.  VLLM has two built-in
;;;                              classes, "ViewDB" and "PAR".
;;;          global_view_list -- consVT; list of strings, each representing
;;;                              a view to be defined for all components in
;;;                              the library.  Each string has the form
;;;                              `" value"'.  If nil, no global views will
;;;                              be added.
;;;
;;; DESCRIPTION
;;;      Define a library.
;;;
;;; EXAMPLE
;;;      (Process 'AddLibrary "Capacitors" #t
;;;               '(("Capacitors" "Capacitors.lib" "libs" "ViewDB"))
;;;               '())
;;;


;;;
;;; NAME
;;;      LibExcludeComp -- exclude given components from a library
;;;
;;; SYNOPSIS
;;;      (Process 'LibExcludeComp library_name comp_list)
;;;
;;;          library_name -- vlStringVT; name of the library
;;;          comp_list    -- consVT; list of strings, each element being the
;;;                          name of a component to exclude
;;;
;;; DESCRIPTION
;;;      Exclude a list of components from a library.  By default, all
;;;      components which appear in the library are included.
;;;
;;; EXAMPLE
;;;      (Process 'LibExcludeComp "Capacitors" '("10-24051-78" "10-24051-85"))
;;;


;;;
;;; NAME
;;;      DbExcludeAttr -- exclude given attributes from a database
;;;
;;; SYNOPSIS
;;;      (Process 'DbExcludeAttr library_name db_name attr_list)
;;;
;;;          library_name -- vlStringVT; name of the library
;;;          db_name      -- vlStringVT; name of the database
;;;          attr_list    -- consVT; list of strings, each element being the
;;;                          name of an attribute to exclude
;;;
;;; DESCRIPTION
;;;      Exclude the given attributes from a database of a library.  By
;;;      default, all attributes are included.
;;;
;;; EXAMPLE
;;;      (Process 'DbExcludeAttr "Capacitors" "Capacitors" '("DEVICE_LABEL"))
;;;


;;;
;;; NAME
;;;      DbSetDefaultMagnitude -- set default magnitude for numeric attribute
;;;
;;; SYNOPSIS
;;;      (Process 'DbSetDefaultMagnitude library_name db_name attr_name magnitude)
;;;
;;;          library_name -- vlStringVT; name of the library
;;;          db_name      -- vlStringVT; name of the database
;;;          attr_name    -- vlStringVT; name of the attribute
;;;          magnitude    -- vllmMagVT; default magnitude.  Valid values are:
;;;                          { vllmMag|Atto, vllmMag|Femto, vllmMag|Pico,
;;;                          vllmMag|Nano, vllmMag|Micro, vllmMag|Milli,
;;;                          vllmMag|None, vllmMag|Kilo, vllmMag|Mega,
;;;                          vllmMag|Giga, vllmMag|Tera }
;;;
;;; DESCRIPTION
;;;      Define the default magnitude of all numeric attributes of the given
;;;      attribute in the given database.  If an attribute value does not
;;;      have a magnitude specifier, then it will be assumed to be expressed
;;;      in the default magnitude.  For example, with a default magnitude of
;;;      vllmMag|Pico, "200" and "200p" will represent the same number,
;;;      200e-12.
;;;
;;;      By default, all numbers without a magnitude specifier are assumed
;;;      to have unit magnitude (vllmMag|None, e0).
;;;
;;; EXAMPLE
;;;      (Process 'DbSetDefaultMagnitude "Capacitors" "Capacitors" "C_VALUE"
;;;               vllmMag|Pico)
;;;
