/**  
 *
 *    Name: VPLACE.VS
 *
 *    Facility: VPLACE
 * 
 *    Functional Description: 
 *       Viewscript file for Viewplace
 *
 *    Environment:
 *        SunOS 4.1 & CORE 5.0
 *     
 *    Author:  
 *      
 *
 *    Modified by: 
 *    Who            Version   When      What
 * 
 *    Copyright (c) 1991 by Viewlogic Systems, Inc.
 *
 *    Some useful Viewscript enumerated type symbols
 *    ----------------------------------------------
 *    vgLinePattern.Solid   vlYesNo.No
 *    vgLinePattern.Dash    vlYesNo.Yes
 *    vgLinePattern.Center  vlOnOff.Off
 *    vgLinePattern.Phantom vlOnOff.On
 *
 *    vgFill.Hollow         vgColor.Black
 *    vgFill.Solid          vgColor.Blue
 *    vgFill.Diagdn1        vgColor.Green
 *    vgFill.Diagup2        vgColor.Cyan
 *    vgFill.Grey08         vgColor.Red
 *    vgFill.Diagdn2        vgColor.Magenta
 *    vgFill.Diagup1        vgColor.Brown
 *    vgFill.Horiz          vgColor.LightGrey
 *    vgFill.Vert           vgColor.DarkGrey
 *    vgFill.Grid2          vgColor.LightBlue
 *    vgFill.Grid1          vgColor.LightGreen
 *    vgFill.X2             vgColor.LightCyan
 *    vgFill.X1             vgColor.LightRed
 *    vgFill.Grey50         vgColor.LightMagenta
 *    vgFill.Grey92         vgColor.Yellow
 *    vgFill.Grey04         vgColor.White
 *
 *    pMode.Dtype           pValue.Grid
 *    pMode.Border          pValue.HistSize
 *    pMode.Grid            pValue.AutoLog
 *    pMode.Header          pValue.TextSize
 *    pMode.CompText        pValue.TextOrigin
 *    pMode.Bell            pValue.Selection
 *    pMode.Detail          pValue.CompOrient
 *    pMode.Pad             pValue.PadSize
 *    pMode.RefDes          pValue.EqIC
 *    pMode.Histogram       pValue.DRCSpace
 *    pMode.Vectors         pValue.TextThres
 *    pMode.Undo            pValue.Offset
 *    pMode.NetLength
 *    pMode.Thermal
 *    pMode.Reliability
 *    pMode.OtherScale
 *    pMode.CalcScale
 *    pMode.RatChain 
 *
 *    pObject.Line
 *    pObject.Box
 *    pObject.Text
 *    pObject.Circle
 *    pObject.Arc
 *    pObject.Net
 *    pObject.Attribute
 *    pObject.Component
 *    pObject.Label
 *    pObject.Pin
 *
 **/

/* Some handy shortcuts for specify different types of regions */
function label(str) vcRegion.Create1(str,vcLabel.fac,str,nil);
function rubber(x,y) vcRubber.Create("Rubber",x,y);
function cmdline(default) vcRegion.Create1("CommandLine",vcCmdline.fac,default,
                                           list (cons (`width, 30)));
function picklist(l) vcRegion.Create1("Picklist", vcPicklist.fac, l, nil);

/*set_stdout(TRUE);*/

/* Symbols for default parameter strategies */
STICKY   := vcWv.Sticky;
CONSTANT := vcWv.Constant;
DYNAMIC  := vcWv.Dynamic;

/* ON OFF symbols */
ON  := vlOnOff.On;
OFF := vlOnOff.Off;

/* Should a point be added when creating lines */
NOPOINT  := vlYesNo.No;
ADDPOINT := vlYesNo.Yes;

/* Symbols for snapping point to grid or not */
NOSNAP := vlYesNo.No;
SNAP   := vlYesNo.Yes;

/* Global symbol used by initmove to determine where the move operation
   was initiated from */
from_where := pMoveFrom.Move;

/* YES NO symbols */
YES := vlYesNo.Yes;
NO  := vlYesNo.No;

/* Indicates drawing mode (middle button bound) */
mode_flag := pDrawMode.NoDraw;

/* Attribute filter file name */
FILTER_FILE := "attr.flt";

/* Create binding table that can be used instead of the standard table */
bt := vcBind.Create();

/* this function clears the binding table "bt" */
function clear_binding_table() vcBind.Clear(bt);

open_cmd := vcWv.Cmd(fac, func(d,e) open_block(d);, "Open");

/* Create a command that does nothing */
noop_cmd := vcWv.Cmd(fac,
                     func(d,e);,
                     "Noop");

function unbind_display(display, event)
{
    if(vcBind.Get(bindings, '\middledown') == nil)
        {
        vcWin.SetCursor(vcEvent.Window(event), vgCursor.Arrow);
        mode_flag = pDrawMode.NoDraw;
        }
    else
        if(mode_flag == pDrawMode.Draw)
            vcWin.SetCursor(vcEvent.Window(event), vgCursor.Cross);

    unbind(display);
    }
    
/* This function binds the middle button to a command */
function bind_middle_button(cmd)
{
    vcGesture.Set(gestures, vcGesture.Default, cmd);
    vcBind.Set(bindings,'\middledown',cmd);
    }

/***
 * This function returns a command which when executed binds the command (cmd)
 * passed in to the middle button. This is used for commands in menus and
 * typed commands which need to wait for a point to be specified before 
 * execution can go on.  After that the middle button is bound to that command 
 * until another command is executed from the menu which rebinds it.
 ***/
function make_button_cmd(cmd, name)
{
    button_cmd := vcWv.Cmd(fac,
                           func(d,e)
                           {
                           mode_flag = pDrawMode.Draw;
                           vcWin.SetCursor(vcEvent.Window(e), vgCursor.Cross);
                           bind_middle_button(cmd);
                           },
                           name);

    return(button_cmd);
    }

/* This function returns another function that gets the value of the 
   specified mode */
function get_mode(mode) { func(mode) { getmode(mode); }; }

/* This function returns another function that gets the value of the 
   specified value */
function get_value(value) { func(display) { getvalue(value); }; }

/* This function returns another function that gets the visibility of the 
   specified object type */
function get_obj_vis(obj) { func(display) {getobjtypevis(obj);};}

/* This function returns another function that gets the color of the 
   specified object type */
function get_obj_color(obj) { func(display) {getcolor(obj);}; }

/* This function returns another function that gets the linestyle of the 
   specified object type */
function get_obj_lstyle(obj) { func(display) {getlstyle(obj);}; }

/* This function returns another function that gets the fillstyle of the 
   specified object type */
function get_obj_fstyle(obj) { func(display) {getfstyle(obj);}; }

/* These command are for dragging a box in a window. */
drag_box_nosnap_cmd := vcWv.Cmd(fac,
                                func(d,e)
                                {
                                contbox(d,e,NOSNAP);
                                },
                                "DragBoxNosnap");
vcCmd.EnableMacro(drag_box_nosnap_cmd, OFF);

drag_box_snap_cmd := vcWv.Cmd(fac,
                              func(d,e)
                              {
                              contbox(d,e,SNAP);
                              },
                              "DragBoxSnap");
vcCmd.EnableMacro(drag_box_snap_cmd, OFF);

cancel_drag_box_cmd := vcWv.Cmd(fac,
                                func(d,e) 
                                {
                                canbox(d,e);
                                unbind_display(d,e);
                                vrefresh(d);
                                },
                                "CancelDragBox");

end_drag_box_cmd := vcWv.Cmd(fac,
                             func(d,e)
                             {
                             finbox(d,e);
                             unbind_display(d,e);
                             },
                             "EndDragBox");

/* this function does a pan while dragging */
function pan_drag(display,event)
{
    pan(display,event);
    }
pan_drag_cmd := vcWv.Cmd(fac,
                         pan_drag,
                         "DragPan");

function in_drag(display,event)
{
    view(display,event,.6);
    contmove(display,event);
    }
in_drag_cmd := vcWv.Cmd(fac,
                        in_drag,"DragIn");


function out_drag(display,event)
{
    view(display,event,1.4);
    contmove(display,event);
    }
out_drag_cmd := vcWv.Cmd(fac,
                         out_drag,"DragOut");


function refresh_drag(display,event)
{
    vrefresh(display);
    contmove(display,event);
    }
refresh_drag_cmd := vcWv.Cmd(fac,
                             refresh_drag,"DragRefresh");

function full_drag(display,event)
{
    full(display,event);
    contmove(display,event);
    }
full_drag_cmd := vcWv.Cmd(fac,
                          full_drag,"DragFull");

function rotate_drag(display, event)
{
    rotate(display, get_center_point(display, event));
    contmove(display, event);
    vrefresh(display);
    }
rotate_drag_cmd := vcWv.Cmd(fac, rotate_drag, "DragRotate");

/***
 * This function returns an event which contains the center point of the 
 * selected objects MBB.  This function is used by numerous routines, such 
 * as move, to begin an operation from a menu entry or keyboard command
 * when there is not initial point specified.  This function also takes and
 * offset which is added to the center point.  
 * Commands that use this are: move, copy, rotate & scale.
 ***/
function get_center_point(display,event)
{
    declare e;
    
    /* if something selected then calculate center point */
    if (numsel(display) != 0)
        {
        /* get MBB of selected objects */
        mbb := mbbselected(display);
        
        /* calculate center point */
        x := vgRect.Xmin(mbb) + ((vgRect.Xmax(mbb) - vgRect.Xmin(mbb))/2);
        y := vgRect.Ymin(mbb) + ((vgRect.Ymax(mbb) - vgRect.Ymin(mbb))/2);
        
        /* convert to pixel coordinates */
        vgU.UserToPixel(getviewport(display),&x,&y);
        
        /* create new event */
        e = vcEvent.Create(x,y,'\0',"",vcEvent.Window(event),
                           vcEvent.Region(event),nil);
        }
    else 
        e = event;   /* nothing selected so return old event */
    
    return(e);
    }

/*****
 * IMPORTANT INFORMATION:
 *
 * Operations that require a coordinate in order to begin execution need two
 * commands built for them.  The first command is the do_xx_cmd.  This command
 * is bound to a button and/or a key and it uses the position of the cursor
 * when the button or key is pressed as the first coordinate. The second 
 * command is the xx_cmd.  This command is bound to a menu and/or a typed 
 * keyword.  When executed this command  binds the middle button to execute 
 * the do_xx_cmd.
 ******/
/* These commands unselect objects */
do_unselect_cmd := vcWv.Cmd(fac,
                            func(d,e)
                            {
                            disp_dxdy(d,e);
                            unselect(d,e);
                            enable_cmds(d,e);
                            },
                            "DoUnselect");
unselect_cmd := make_button_cmd(do_unselect_cmd,"Unselect");

/* This function makes commands for the select operation */
function make_select(add_select)
{
    select_bt := vcBind.Create();
    
    /* This function ends the select operation and deselects */
    function end_select(display,event,add_select)
        {
        finbox(display,event);
        disp_dxdy(display, event);
        unbind_display(display,event);
        select(display,event,add_select);
        enable_cmds(display,event);
        }

    if(add_select == ON)
        {
        end_select_cmd := vcWv.Cmd(fac,
                                   func(d,e) 
                                   end_select(d,e,ON);,
                                   "SelectExecuteAdd");
        name := "SelectAdd";
        }
    else
        {
        end_select_cmd := vcWv.Cmd(fac,
                                   func(d,e)
                                   end_select(d,e,OFF);,
                                   "SelectExecute");
        name := "Select";
        }

    /* bind the move key to continue dragging box */
    vcBind.Set(select_bt,'\move',drag_box_nosnap_cmd);
    vcBind.Set(select_bt,'\#\move',drag_box_nosnap_cmd);
    
    /* bind the leftup and ctrl-leftup to the end select command */
    vcBind.Set(select_bt,'\leftup',end_select_cmd);
    vcBind.Set(select_bt,'\^\leftup',end_select_cmd);
            
    /* Initialize for box drag without snapping to grid */
    function start_select(display,event)
        {
        disp_dxdy(display,event);
        initbox(display,event,vgColor.White,vgLinePattern.Solid,NOSNAP);
        bind(display,select_bt);
        }

    return(vcWv.Cmd(fac,
                    start_select,
                    name));
    }

/* This function makes commands for the zoom operation */
/*-*/
function make_zoom()
{
    function end_zoom(display,event)
        {
        finbox(display,event);
        unbind_display(display,event);
        zoom(display,event);
        }
    end_zoom_cmd := vcWv.Cmd(fac,
                             end_zoom,
                             "EndZoom");

    function cancel_zoom(display,event)
        {
        canbox(display,event);
        unbind_display(display,event);
        vrefresh(display);
        }
    cancel_zoom_cmd := vcWv.Cmd(fac,
                                cancel_zoom,
                                "CancelZoom");

    vcCmd.SetHelp(end_zoom_cmd, "End zoom operation");
    vcCmd.SetHelp(cancel_zoom_cmd, "Cancel zoom operation");

    function start_zoom(display,event)
        {
        /* Initialize for box dragging */
        initbox(display,event,vgColor.White,vgLinePattern.Solid,NOSNAP); 
        
        /* bind keys for dragging box */
        clear_binding_table();
        vcBind.Set(bt,'\move',drag_box_nosnap_cmd);
        vcBind.Set(bt,'\middledown',end_zoom_cmd);
        vcBind.Set(bt,'\leftdown',end_zoom_cmd);
        vcBind.Set(bt,'\rightdown',cancel_zoom_cmd);
        
        /* binds the binding table */
        bind(display,bt);
        }
    /* set middle button to start zoom command */
    return(vcWv.Cmd(fac,
                    start_zoom,
                    "StartZoom"));
    }
do_zoom_cmd := make_zoom();
zoom_cmd := make_button_cmd(do_zoom_cmd,"Zoom");

zregion_cmd := vcWv.Cmd (fac, func(d, e, w, x, y, z)
                         zregion (d, e, w, x, y, z);,
                         "ZoomRegion",
                         "Zoom Coordinates",
                         vcWv.Param ("LLX", pBlockWidthVT,  STICKY, 1,
                                     "Lower-Left X coordinate"),
                         vcWv.Param ("LLY", pBlockHeightVT, STICKY, 1,
                                     "Lower-Left Y coordinate"),
                         vcWv.Param ("URX", pBlockWidthVT,  STICKY, 100,
                                     "Upper-Right X coordinate"),
                         vcWv.Param ("URY", pBlockHeightVT, STICKY, 100,
                                     "Upper-Right Y coordinate"));

vcCmd.EnableApply (zregion_cmd, ON);

function zoom_to_area(display, event)
{
    toggle_busy(display);
    r := vcEvent.Rect(event);
    vp := getviewport(display);

    vgU.PixelToUserRect(vp, r);

    vcCmd.Run (zregion_cmd, display, event, list (vgRect.Xmin (r),
                                                  vgRect.Ymin (r),
                                                  vgRect.Xmax (r),
                                                  vgRect.Ymax (r)), TRUE);    
    toggle_busy(display);
    }
zoom_to_area_cmd := vcWv.Cmd (fac, zoom_to_area, "ZoomToArea");


/* This function makes commands for the move operation */
function make_move()
{
    /* this function ends the move operation */
    function end_move(display,event)
        {
        vcCmd.EnableMacro(cont_move_cmd, ON);
        contmove(display,event);
        vcCmd.EnableMacro(cont_move_cmd, OFF);
        if (finmove(display,event) == 1)
            {
            unbind_display(display,event);
            restoresel(display); /* this restores selected after text create */
            }
        }
    end_move_cmd := vcWv.Cmd(fac,
                             end_move,
                             "EndMove");
    
    /* this function cancels the move operation */
    function cancel_move(display,event)
        {
        canmove(display,event);
        restoresel(display);  /* this restores selected after text create */
        unbind_display(display,event);
        }
    cancel_move_cmd := vcWv.Cmd(fac,
                                cancel_move,
                                "CancelMove");
    
    /* toggle detail command */
    tdetail_cmd := vcWv.Cmd(fac,
                            tdetail,
                            "ToggleDetail");  
    
    vcCmd.SetHelp(end_move_cmd, "End move operation");
    vcCmd.SetHelp(cancel_move_cmd, "Cancel move operation");
    vcCmd.SetHelp(tdetail_cmd, "Toggle detail for the move operation");
    
    /* continue move command */
    cont_move_cmd := vcWv.Cmd(fac,
                              func(d,e)
                              {
                              contmove(d,e);
                              auto_scroll(d,e);
                              disp_dxdy(d,e);
                              },
                              "ContMove");
    vcCmd.EnableMacro(cont_move_cmd, OFF);

    
    /* This function starts the move operation */
    function start_move(display,event)
        {
        if (initmove(display,event,from_where) == 1)
            {
            from_where = pMoveFrom.Move;  /* set it back for move operation */
            clear_binding_table();
            
            /* Move binding keys */
            vcBind.Set(bt,'\move',cont_move_cmd);
            vcBind.Set(bt,'\middledown',end_move_cmd);
            vcBind.Set(bt,'\leftdown',tdetail_cmd);
            vcBind.Set(bt,'\rightdown',cancel_move_cmd);
            vcBind.Set(bt,'\^S',pan_drag_cmd); 
            vcBind.Set(bt,'\F6',pan_drag_cmd);
            vcBind.Set(bt, '\F5',refresh_drag_cmd);
            vcBind.Set(bt, '\F7',in_drag_cmd);
            vcBind.Set(bt, '\F8',out_drag_cmd);
            vcBind.Set(bt, '\F9',rotate_drag_cmd);
            vcBind.Set(bt, '\F0',full_drag_cmd);
            bind(display,bt);  /* bind the table */
            disp_dxdy(display,event);
            }
        }

    return(vcWv.Cmd(fac,
                    start_move,
                    "StartMove"));
    }
do_move_cmd := make_move();

/***
 * This function does a move operation using the center point of the 
 * MBB of the selected objects as the initial point.
 ***/
function move_center(display,event)
{
    bind_middle_button(do_move_cmd);
    vcCmd.Run(do_move_cmd,display,get_center_point(display,event),nil,FALSE);
    }
move_cmd := vcWv.Cmd(fac,
                     move_center,
                     "Move");
vcCmd.EnableMacro(move_cmd, OFF);

/* This function is called to move an object after creation */
function move_created_object(display,event)
{
    from_where = pMoveFrom.Create;
    move_created_cmd := vcCmd.Run(do_move_cmd,display,
                                  event,nil,FALSE);
    }

/* This function makes commands for the stretch operation */
function make_stretch()
{
    /* command to end the stretch operation */
    end_stretch_cmd := vcWv.Cmd(fac,
                                func(d,e)
                                {
                                finstrch(d,e);
                                unbind_display(d,e);
                                vrefresh(d);
                                },
                                "EndStretch");
    
    /* command to cancel the stretch operation */
    cancel_stretch_cmd := vcWv.Cmd(fac,
                                   func(d,e)
                                   {
                                   canstrch(d,e);
                                   unbind_display(d,e);
                                   },
                                   "CancelStretch");
    vcCmd.SetHelp(end_stretch_cmd, "End the stretch command");
    vcCmd.SetHelp(cancel_stretch_cmd, "Cancel the stretch command");

    /*  command to continue stretch operation */
    cont_stretch_cmd := vcWv.Cmd(fac,
                                 func(d,e)
                                 {
                                 contstrch(d,e);
                                 },
                                 "ContinueStretch");

    /* This function starts the stretch operation, the first call
       of contstrch looks for a point to stretch */
    function start_stretch(display,event)
        if (contstrch(display,event) == 1)
            {
            clear_binding_table();
            vcBind.Set(bt,'\move',cont_stretch_cmd);
            vcBind.Set(bt,'\middledown',end_stretch_cmd);
            vcBind.Set(bt,'\leftdown',end_stretch_cmd);
            vcBind.Set(bt,'\rightdown',cancel_stretch_cmd);
            vcBind.Set(bt,'\^S',pan_drag_cmd); 
            vcBind.Set(bt,'\F6',pan_drag_cmd);
            vcBind.Set(bt, '\F5',refresh_drag_cmd);
            vcBind.Set(bt, '\F7',in_drag_cmd);
            vcBind.Set(bt, '\F8',out_drag_cmd);
            vcBind.Set(bt, '\F9',rotate_drag_cmd);
            vcBind.Set(bt, '\F0',full_drag_cmd);
            bind(display,bt);
            }
    
    /* This function inits the stretch operation */
    function init_stretch(display,event)
        /* Initialize stretch operation */
        if (initstrch(display) == 1)
            {
            start_stretch(display,event);
            bind_middle_button(stretch_cmd);
            }

    return(stretch_cmd := vcWv.Cmd(fac,
                                   init_stretch,
                                   "InitStretch"));
    }

do_stretch_cmd := make_stretch();
stretch_cmd := make_button_cmd(do_stretch_cmd, "Stretch");

/* This function and command does the copy operation followed by a move */
function copy_objects(display,event)
    if (copy(display,event) == 1)
        {
        from_where = pMoveFrom.Copy;
        vcCmd.Run(do_move_cmd,display,event,nil,TRUE);
        }

do_copy_cmd := vcWv.Cmd(fac,
                        copy_objects,
                        "DoCopy");

/***
 * This function does a copy operation using the center point of the 
 * MBB of the selected objects as the initial point.
 ***/
function copy_center(display,event)
{
    bind_middle_button(do_copy_cmd);
    vcCmd.Run(do_copy_cmd,display,get_center_point(display,event),nil,TRUE);
    }
copy_cmd := vcWv.Cmd(fac,
                     copy_center,
                     "Copy");


/***
 * IMPORTANT INFORMATION:
 *
 * The operations for creating text, attributes, labels and components need 
 * only one command that can be bound to a menu entry and a mouse button.  
 * In either case the user is prompted for a text string first and then thrown 
 * into a move operation.
 ***/
/* This function create an attribute */
function create_attr(display,event,str)
    if (attr(display,event,str) == 1)
        {
        enable_cmds(display,event);
        move_created_object(display,event);
        bind_middle_button(attr_cmd);
        }

attr_cmd := vcWv.Cmd(fac,
                     create_attr,
                     "Attr",
                     "Enter Attribute",
                     vcWv.Param("Attribute String",string,
                                STICKY,""));

vcCmd.EnableApply (attr_cmd, ON);

/* This function creates a label with specified string */
function create_label_str(display,event,str)
{
    if (newlabel(display,event,str) == 1)
        {
        enable_cmds(display,event);
        move_created_object(display,event);
        }
    }
create_label_str_cmd := vcWv.Cmd(fac,
                                 create_label_str,
                                 "CreateLabelString",
                                 "Enter Label",
                                 vcWv.Param("Label String",
                                            string,STICKY,""));
vcCmd.EnableApply (create_label_str_cmd, ON);


/*** 
  This function determines which label command to call depending on whether
  or not the internal label string buffer is empty. If the buffer is empty
  then we want to execute the above command so that the user will be prompted
  for a label string.  Otherwise we want to simply create a label and force
  to use the string from the buffer.
  ***/
function create_label(display,event)
{
    /* This function creates a label from internal label string buffer */
    function label_nostr(display,event)
        {
        if (newlabel(display,event,"") == 1)
            {
            enable_cmds(display,event);
            move_created_object(display,event);
            }
        }
    
    /* if the label string buffer is empty then prompt user for label */
    if (emptystring(display) == 1)
        vcCmd.Run(create_label_str_cmd,display,event,nil,TRUE);
    else 
        label_nostr(display,event);

    bind_middle_button(label_cmd);
    }
label_cmd := vcWv.Cmd(fac,
                      create_label,
                      "Label");

/* This function creates a text object */
function create_text(display,event,str)
    if (text(display,event,str) == 1)
{
    enable_cmds(display,event);
    move_created_object(display,event);
    bind_middle_button(text_cmd);
    }
text_cmd := vcWv.Cmd(fac,
                     create_text,
                     "Text",
                     "Enter Text",
                     vcWv.Param("Text String",
                                vlStringRestOfLine1VT,DYNAMIC,getstring));

vcCmd.EnableApply (text_cmd, ON);


/* This function creates a component object */
function create_comp(display,event,str)
    if (comp(display,event,str) == 1)
{
    enable_cmds(display,event);
    move_created_object(display,event);
    bind_middle_button(comp_cmd);
    }
comp_cmd := vcWv.Cmd(fac,
                     create_comp,
                     "DoComp",
                     "Enter Symbol Name",
                     vcWv.Param("Symbol Name",string,STICKY,"",
                                "Annotation symbol to be instantiated"));

vcCmd.EnableApply (comp_cmd, ON);

/* This function creates a pin object */
function create_pin(display,event)
{
    if (pin(display,event) == 1)
        {
        enable_cmds(display,event);
        move_created_object(display,event);
        }
    }
do_pin_cmd := vcWv.Cmd(fac,
                       create_pin,
                       "DoPin");
pin_cmd := make_button_cmd(do_pin_cmd,"Pin");


/* This function makes commands for the box object creation operation */
function make_box()
{
    function end_box(display,event)
        {
        finbox(display,event); /* finish dragging */
        unbind_display(display,event);
        box(display); /* create the actual box objects */
        enable_cmds(display,event);
        vrefresh(display);
        }
    end_box_cmd := vcWv.Cmd(fac,
                            end_box,
                            "EndBox");
    vcCmd.SetHelp(end_box_cmd, "End drawing a box");

    function start_box(display,event)
        {
        initbox(display,event,getcolor(pObject.Box),
                getlstyle(pObject.Box),SNAP);
        clear_binding_table();
        vcBind.Set(bt,'\move',drag_box_nosnap_cmd);
        vcBind.Set(bt,'\middledown',end_box_cmd);
        vcBind.Set(bt,'\leftdown',end_box_cmd);
        vcBind.Set(bt,'\rightdown',cancel_drag_box_cmd);
        vcBind.Set(bt,'\^S',pan_drag_cmd); 
        vcBind.Set(bt,'\F6',pan_drag_cmd);
        vcBind.Set(bt, '\F5',refresh_drag_cmd);
        vcBind.Set(bt, '\F7',in_drag_cmd);
        vcBind.Set(bt, '\F8',out_drag_cmd);
        vcBind.Set(bt, '\F9',rotate_drag_cmd);
        vcBind.Set(bt, '\F0',full_drag_cmd);
        bind(display,bt);
        }
    return(vcWv.Cmd(fac,
                    start_box,
                    "StartBox"));
    }
/*-*/
do_box_cmd := make_box();
box_cmd := make_button_cmd(do_box_cmd,"Box");


/* This function makes commands for the circle operation */
function make_circle()
{
    /* This command ends the circle operation */
    end_circle_cmd := vcWv.Cmd(fac,
                               func(d,e)
                               {
                               contcirc(d,e);
                               if (fincirc(d,e) == 1)
                                   {
                                   unbind_display(d,e);
                                   enable_cmds(d,e);
                                   }
                               },
                               "EndCircle"); 

    /* This command cancels the circle operation */
    cancel_circle_cmd := vcWv.Cmd(fac,
                                  func(d,e)
                                  {
                                  cancirc(d,e);
                                  unbind_display(d,e);
                                  },
                                  "CancelCircle"); 
    vcCmd.SetHelp(end_circle_cmd, "End drawing a circle");
    vcCmd.SetHelp(cancel_circle_cmd, "Cancel drawing a circle");

    /* This command drags the circle */
    drag_circle_cmd := vcWv.Cmd(fac,
                                contcirc,
                                "DragCircle");
    vcCmd.EnableMacro(drag_circle_cmd, OFF);

    function start_circle(display,event)
        {
        initcirc(display,event);
        clear_binding_table();
        vcBind.Set(bt,'\move',drag_circle_cmd);
        vcBind.Set(bt,'\middledown',end_circle_cmd);
        vcBind.Set(bt,'\leftdown',end_circle_cmd);
        vcBind.Set(bt,'\rightdown',cancel_circle_cmd);
        vcBind.Set(bt,'\^S',pan_drag_cmd); 
        vcBind.Set(bt,'\F6',pan_drag_cmd);
        vcBind.Set(bt, '\F5',refresh_drag_cmd);
        vcBind.Set(bt, '\F7',in_drag_cmd);
        vcBind.Set(bt, '\F8',out_drag_cmd);
        vcBind.Set(bt, '\F9',rotate_drag_cmd);
        vcBind.Set(bt, '\F0',full_drag_cmd);
        bind(display,bt);
        }
    return(vcWv.Cmd(fac,
                    start_circle,
                    "StartCircle"));
    }
do_circle_cmd := make_circle();
circle_cmd := make_button_cmd(do_circle_cmd,"Circle");

/* These command are used for both line and arc creation */
drag_line_cmd := vcWv.Cmd(fac,func(d,e)
                          contline(d,e,NOPOINT);,
                          "DragLine");
vcCmd.EnableMacro(drag_line_cmd, OFF);

cancel_line_cmd := vcWv.Cmd(fac,
                            func(d,e)
                            {
                            unbind_display(d,e);
                            while(canline(d,e) != 1);
                            },
                            "CancelLine");

/* This function makes commands for the line operation */
function make_line()
{
    end_line_cmd := vcWv.Cmd(fac,
                             func(d,e)
                             {unbind_display(d,e);finline(d,e);
                              enable_cmds(d,e);},
                             "EndLine");

    undo_point_cmd := vcWv.Cmd(fac,
                               func(d,e)
                               {if(canline(d,e) == 1) 
                                    unbind_display(d,e);},
                               "UndoPoint");

    add_point_cmd := vcWv.Cmd(fac,func(d,e)
                              contline(d,e,ADDPOINT);,
                              "AddPoint");
    vcCmd.SetHelp(end_line_cmd, "End drawing a line");
    vcCmd.SetHelp(undo_point_cmd, "Undo the last point on the line");
    vcCmd.SetHelp(add_point_cmd, "Add a point to the line");

    function start_line(display,event)
        {
        initline(display,event,
                 getcolor(pObject.Line),getlstyle(pObject.Line)); 
        clear_binding_table();
        vcBind.Set(bt,'\move',drag_line_cmd);
        vcBind.Set(bt,'\middledown',add_point_cmd);
        vcBind.Set(bt,'\leftdown',undo_point_cmd);
        vcBind.Set(bt,'\rightdown',end_line_cmd);
        vcBind.Set(bt,'\^S',pan_drag_cmd); 
        vcBind.Set(bt,'\F6',pan_drag_cmd);
        vcBind.Set(bt, '\F5',refresh_drag_cmd);
        vcBind.Set(bt, '\F7',in_drag_cmd);
        vcBind.Set(bt, '\F8',out_drag_cmd);
        vcBind.Set(bt, '\F9',rotate_drag_cmd);
        vcBind.Set(bt, '\F0',full_drag_cmd);
        bind(display,bt);
        }
    return(vcWv.Cmd(fac,
                    start_line,
                    "StartLine"));
    }
do_line_cmd := make_line();
line_cmd := make_button_cmd(do_line_cmd,"Line");

/* This function makes commands for the arc creation operation */
function make_arc()
{
    end_arc_cmd := vcWv.Cmd(fac,
                            func(d,e)
                            {
                            contarc(d,e);
                            finarc(d,e);
                            unbind_display(d,e);
                            enable_cmds(d,e);
                            },
                            "EndArc");

    cancel_arc_cmd := vcWv.Cmd(fac,
                               func(d,e)
                               {
                               canarc(d,e);
                               unbind_display(d,e);
                               },
                               "CancelArc");

    drag_arc_cmd := vcWv.Cmd(fac,
                             contarc,
                             "DragArc");
    vcCmd.EnableMacro(drag_arc_cmd, OFF);

    /* This function ends the arc line creation */
    function end_arc_line(display,event)
        {
        contline(display,event,ADDPOINT); /* add second point */
        initarc(display,event);    /* init arc */
        clear_binding_table();
        vcBind.Set(bt,'\move',drag_arc_cmd);
        vcBind.Set(bt,'\middledown',end_arc_cmd);
        vcBind.Set(bt,'\leftdown',end_arc_cmd);
        vcBind.Set(bt,'\rightdown',cancel_arc_cmd);
        vcBind.Set(bt,'\^S',pan_drag_cmd); 
        vcBind.Set(bt,'\F6',pan_drag_cmd);
        vcBind.Set(bt, '\F5',refresh_drag_cmd);
        vcBind.Set(bt, '\F7',in_drag_cmd);
        vcBind.Set(bt, '\F8',out_drag_cmd);
        vcBind.Set(bt, '\F9',rotate_drag_cmd);
        vcBind.Set(bt, '\F0',full_drag_cmd);
        bind(display,bt);
        }
    end_arc_line_cmd := vcWv.Cmd(fac,
                                 end_arc_line,
                                 "EndArcLine");
    vcCmd.SetHelp(end_arc_cmd, "End drawing an arc");
    vcCmd.SetHelp(cancel_arc_cmd, "Cancel drawing an arc");
    vcCmd.SetHelp(end_arc_line_cmd, "End the arc line");

    function start_arc(display,event)
        {
        /* add first point of arc */
        initline(display,event,getcolor(pObject.Arc),getlstyle(pObject.Arc));
        clear_binding_table();
        vcBind.Set(bt,'\move',drag_line_cmd);
        vcBind.Set(bt,'\middledown',end_arc_line_cmd);
        vcBind.Set(bt,'\leftdown',end_arc_line_cmd);
        vcBind.Set(bt,'\rightdown',cancel_line_cmd);
        bind(display,bt);
        }
    return(vcWv.Cmd(fac,
                    start_arc,
                    "StartArc"));
    }
do_arc_cmd := make_arc();
arc_cmd := make_button_cmd(do_arc_cmd,"Arc");

/* This function makes commands for the net creation operation */
function make_net()
{
    /* This functions adds a pin connection to a net */
    function add_pin(display,event)
        {
        if (addpinnet(display,event) == 0)
            {
            finnet(display,event);
            unbind_display(display,event);
            }
        }
    add_pin_cmd := vcWv.Cmd(fac,
                            add_pin,
                            "AddPin");

    /* This command ends the adding of pins to a net */
    end_net_cmd := vcWv.Cmd(fac,
                            func(d,e)
                            {finnet(d,e);unbind_display(d,e);},
                            "EndNet");
    vcCmd.SetHelp(add_pin_cmd, "Add a pin to the net");
    vcCmd.SetHelp(end_net_cmd, "End drawing the net");

    /* This function starts the net operation */
    function start_net(display,event,net_name)
        {
        if (initnet(display,event,net_name) == 1)
            {
            clear_binding_table();
            vcBind.Set(bt,'\middledown',add_pin_cmd);
            vcBind.Set(bt,'\leftdown',add_pin_cmd);
            vcBind.Set(bt,'\rightdown',end_net_cmd);
            vcBind.Set(bt,'\^S',pan_drag_cmd); 
            vcBind.Set(bt,'\F6',pan_drag_cmd);
            vcBind.Set(bt, '\F5',refresh_drag_cmd);
            vcBind.Set(bt, '\F7',in_drag_cmd);
            vcBind.Set(bt, '\F8',out_drag_cmd);
            vcBind.Set(bt, '\F9',rotate_drag_cmd);
            vcBind.Set(bt, '\F0',full_drag_cmd);
            bind(display,bt);
            bind_middle_button(net_cmd);  /* middle button is bound here */
            }
        }
    return(vcWv.Cmd(fac,
                    start_net,
                    "Net",
                    "Enter Net",
                    vcWv.Param("Net Name",pNameVT,STICKY,"")));
    }
net_cmd := make_net();

vcCmd.EnableApply (net_cmd, ON);


/** Rotate selected objects about their center point **/
rotate_cmd := vcWv.Cmd(fac,
                       func(d,e)rotate(d,get_center_point(d,e));,
                       "Rotate");

/** Rotate board outline about it's center point **/
rotate_ol_cmd := vcWv.Cmd(fac,
                       func(d,e)rot_outline(d,get_center_point(d,e));,
                       "Rotate_outline");


/** Scale selected objects about their center point **/
scale_cmd := vcWv.Cmd(fac,
                      func(d,e,s)scale(d,get_center_point(d,e),s);,
                      "Scale",
                      "Enter Scale",
                      vcWv.Param("Scale Factor",vlPositiveFloatVT,STICKY,1.0,
                                 "Factor by which to scale the selected object(s)"));

vcCmd.EnableApply (scale_cmd, ON);

/*reflect_cmd := vcWv.Cmd(fac,reflect,"Reflect");

/* This is the cmd to create a default symbol */
crdsym_cmd := vcWv.Cmd(fac,
                       func(d,e,w,h,l,r,t,b,s) 
                       crdsym(d,w,h,l,r,t,b,s);,
                       "Crdsym",
                       "Enter Defaults",
                       vcWv.Param("Width",vlPositiveIntVT,STICKY,300,
                                  "Default symbol width"),
                       vcWv.Param("Height",vlPositiveIntVT,STICKY,1000,
                                  "Default symbol height"),
                       vcWv.Param("Left Pins",vlUnsignedIntVT,STICKY,0,
                                  "Number of pins to be placed on the left edge"),
                       vcWv.Param("Right Pins",vlUnsignedIntVT,STICKY,0,
                                  "Number of pins to be placed in the right edge"),
                       vcWv.Param("Top Pins",vlUnsignedIntVT,STICKY,0,
                                  "Number of pins to be placed along the top edge"),
                       vcWv.Param("Bottom Pins",vlUnsignedIntVT,STICKY,0,
                                  "Number of pins to be plaved along the bottom edge"),
                       vcWv.Param("Pin Spacing",vlPositiveIntVT,STICKY,100,
                                  "Spacing between the pins in mils"));

/* The swap component command */
swap_cmd := vcWv.Cmd(fac,
                     swap,
                     "Swap");

/* REFARY function and command */
function refary(display,event,refdes,xspacing,yspacing,orient)
{
    function end_refary(display,event)
        {
        finbox(display,event);
        rarray(display,event,refdes,xspacing,yspacing,orient);
        unbind_display(display,event);
        vrefresh(display);
        }
    end_refary_cmd := vcWv.Cmd(fac, end_refary, "EndRefary");
    
    function start_box(display,event)
        {
        clear_binding_table();
        box_prompt(display);
        vcBind.Set(bt,'\move',drag_box_nosnap_cmd);
        vcBind.Set(bt,'\middledown',end_refary_cmd);
        vcBind.Set(bt,'\leftdown',end_refary_cmd);
        vcBind.Set(bt,'\rightdown',cancel_drag_box_cmd);
        initbox(display,event,vgColor.White,vgLinePattern.Solid,NOSNAP);
        bind(display, bt);
        }
    start_box_cmd := vcWv.Cmd(fac, start_box, "Refary-StartBox");

    vcBind.Set(bindings,'\middledown',start_box_cmd);
    }
refary_cmd := vcWv.Cmd(fac,
                       refary,
                       "Refary",
                       "Enter Placement Data",
                       vcWv.Param("RefDes Name",pNameVT,STICKY,"*",
                                  "Refdes to be placed in an array"),
                       vcWv.Param("X Spacing",vlUnsignedIntVT,STICKY,1,
                                  "X spacing between components (mils)"),
                       vcWv.Param("Y Spacing",vlUnsignedIntVT,STICKY,1,
                                  "Y spacing between components (mils)"),
                       vcWv.Param("Orientation",pOrientVT,STICKY,1,
                                  "Orientation of the components (0-3)"));
vcCmd.EnableApply (refary_cmd, ON);

/* TYPARY command */
function typary(display,event,comptype,xspacing,yspacing,orient)
{
    function end_typary(display,event)
        {
        finbox(display,event);
        tarray(display,event,comptype,xspacing,yspacing,orient);
        unbind_display(display,event);
        }
    end_typary_cmd := vcWv.Cmd(fac, end_typary, "EndTypary");
    
    function start_box(display,event)
        {
        clear_binding_table();
        box_prompt(display);
        vcBind.Set(bt,'\move',drag_box_nosnap_cmd);
        vcBind.Set(bt,'\middledown',end_typary_cmd);
        vcBind.Set(bt,'\leftdown',end_typary_cmd);
        vcBind.Set(bt,'\rightdown',cancel_drag_box_cmd);
        initbox(display,event,vgColor.White,vgLinePattern.Solid,NOSNAP);
        bind(display, bt);
        }

    vcBind.Set(bindings,'\middledown',vcWv.Cmd(fac,
                                               start_box,
                                               "Typary-StartBox"));
    }
typary_cmd := vcWv.Cmd(fac,typary,
                       "Typary",
                       "Enter Placement Data",
                       vcWv.Param("Component Type",pNameVT,STICKY,"*",
                                  "Comptype to be placed in an array"),
                       vcWv.Param("X Spacing",vlUnsignedIntVT,STICKY,1,
                                  "X spacing between components (mils)"),
                       vcWv.Param("Y Spacing",vlUnsignedIntVT,STICKY,1,
                                  "Y spacing between components (mils)"),
                       vcWv.Param("Orientation",pOrientVT,STICKY,1,
                                  "Orientation of the components (0-3)"));
vcCmd.EnableApply (typary_cmd, ON);

/* NETARY command */
function netary(display,event,netname,xspacing,yspacing,orient)
{
    function end_netary(display,event)
        {
        finbox(display,event);
        narray(display,event,netname,xspacing,yspacing,orient);
        unbind_display(display,event);
        }
    end_netary_cmd := vcWv.Cmd(fac, end_netary, "EndNetary");
    
    function start_box(display,event)
        {
        clear_binding_table();
        box_prompt(display);
        vcBind.Set(bt,'\move',drag_box_nosnap_cmd);
        vcBind.Set(bt,'\middledown',end_netary_cmd);
        vcBind.Set(bt,'\leftdown',end_netary_cmd);
        vcBind.Set(bt,'\rightdown',cancel_drag_box_cmd);
        initbox(display,event,vgColor.White,vgLinePattern.Solid,NOSNAP);
        bind(display, bt);
        }

    vcBind.Set(bindings,'\middledown',vcWv.Cmd(fac,
                                               start_box,
                                               "Netary-StartBox"));
    }
netary_cmd := vcWv.Cmd(fac,netary,
                       "Netary",
                       "Enter Placement Data",
                       vcWv.Param("Net Name",pNameVT,STICKY,"*",
                                  "Components connected to this net name will be placed in an array"),
                       vcWv.Param("X Spacing",vlUnsignedIntVT,STICKY,1,
                                  "X spacing between components (mils)"),
                       vcWv.Param("Y Spacing",vlUnsignedIntVT,STICKY,1,
                                  "Y spacing between components (mils)"),
                       vcWv.Param("Orientation",pOrientVT,STICKY,1,
                                  "Orientation of the components (0-3)"));
vcCmd.EnableApply (netary_cmd, ON);

/* This is the array command */
array_cmd := vcWv.Cmd(fac,
                      func(d,e,r,c,r_s,c_s)
                      {
                      toggle_busy(d);
                      array(d,e,r,c,r_s,c_s);
                      toggle_busy(d);
                      },
                      "Array",
                      "Enter Array Specs",
                      vcWv.Param("Rows",vlPositiveIntVT,STICKY,1,
                                 "Number of rows to array object"),
                      vcWv.Param("Columns",vlPositiveIntVT,STICKY,1,
                                 "Number of columns to array object"),
                      vcWv.Param("Row Spacing",vlUnsignedIntVT,STICKY,1,
                                 "Spacing in mils between rows"),
                      vcWv.Param("Column Spacing",vlUnsignedIntVT,STICKY,1,
                                 "Spacing in mils between columns"));
vcCmd.EnableApply (array_cmd, ON);

/* XYPLC command */
xyplc_cmd := vcWv.Cmd(fac,
                      xyplc,
                      "Xyplc",
                      "Enter Placement Data",
                      vcWv.Param("RefDes Name",pNameVT,STICKY,"",
                                 "Refdes to be placed at a specific X-Y location"),
                      vcWv.Param("X Coordinate",integer,STICKY,1,
                                 "X coordinate to place the component"),
                      vcWv.Param("Y Coordinate",integer,STICKY,1,
                                 "Y coordinate to place the component"));
vcCmd.EnableApply (xyplc_cmd, ON);

/*-*/
/* SYMORG command */
function make_symorg()
{
    change_sym_origin_cmd := vcWv.Cmd(fac,
                                      func(d,e)
                                      {
                                      symorg(d,e);
                                      vrefresh(d);
                                      unbind_display(d,e);
                                      },
                                      "ChangeSymOrg");
    
    exit_change_sym_origin_cmd := vcWv.Cmd(fac,
                                           func(d,e)
                                           unbind_display(d,e);,
                                           "ExitChangeSymOrg");
    
    return(do_symorg_cmd := vcWv.Cmd(fac,
                    func(d,e)
                    {
                    clear_binding_table();
                    vcBind.Set(bt,'\rightdown',exit_change_sym_origin_cmd);
                    vcBind.Set(bt,'\e',exit_change_sym_origin_cmd);
                    vcBind.Set(bt,'\middledown',change_sym_origin_cmd);
                    vcBind.Set(bt,'\leftdown',change_sym_origin_cmd);
                    bind(d,bt);
                    },
                    "SymOrg"));
    vcCmd.EnableMacro(do_symorg_cmd, OFF);
    }
symorg_cmd := make_symorg();

/* PLCALL command */
function place_comp(display,event)
{
    if (plcall(display,event) == 1)
        move_created_object(display,event);
    bind_middle_button(plcall_cmd);
    }
plcall_cmd := vcWv.Cmd(fac,
                       place_comp,
                       "Plcall");

/* VIEW commands */
full_cmd := vcWv.Cmd(fac,full,"Full");

in_cmd := vcWv.Cmd(fac,
                   func(d,e) view(d,e,.6);,
                   "In");

out_cmd := vcWv.Cmd(fac,
                    func(d,e) view(d,e,1.4);,
                    "Out");

refresh_cmd := vcWv.Cmd(fac,
                        func(d,e) vrefresh(d);,
                        "Refresh");

do_scroll_cmd := vcWv.Cmd(fac,func(d,e)
                          {
                          disp_dxdy(d,e);
                          scroll(d,e);
                          vrefresh(d);
                          }
                          ,"DoScroll");
scroll_cmd := make_button_cmd(do_scroll_cmd,"Scroll");

zselect_cmd := vcWv.Cmd(fac,zselect,"Zselect");

tthres_cmd := vcWv.Cmd(fac,
                       func(d,e,t)
                       {
                       setvalue(pValue.TextThres,t);
                       vrefresh(d);
                       },
                       "Thres",
                       "Enter Threshold",
                       vcWv.Param("Text Threshold",vlPositiveIntVT,
                                  DYNAMIC,get_value(pValue.TextThres),
                                  "Text sizes less than this value will not be displayed"));
vcCmd.EnableApply (tthres_cmd, ON);

do_pan_cmd := vcWv.Cmd(fac,func(d,e)
                       {
                       disp_dxdy(d,e);
                       pan(d,e);
                       },
                       "DoPan");
pan_cmd := make_button_cmd(do_pan_cmd,"Pan");

/* VPBRD command */
function start_board(d,e,name)
{
    declare window;
    
    function create_buttons()
        {
        buttons := vcTable.Create ("Buttons", 1, 1);

/*        
        vcTable.Add (buttons, vcLabel.ButtonCreate ("Accept",
                                                    "Accept",
                                                    func() 
                                                    vcWin.Destroy (window);,
                                                    nil));
                                                    */

        vcTable.Add (buttons, vcLabel.ButtonCreate ("Cancel",
                                                    "Cancel",
                                                    func() 
                                                    vcWin.Destroy (window);,
                                                    nil));
        return buttons;
        
        } /* create_buttons */
    
    function create_outline_picklist(oid)
        {
        function grab_outline(str, on_off, index)
            {
            out_name := caddr(vscript.vsParse(str));
            vpbrd(d,e,name,out_name);
            vcWin.Destroy(window);
            }

        outline_table := vcTable.Create("Board Outlines", 7, 1);

        pick_region := create_picklist(d, e, "Picklist", 6, 4);

        vcPicklist.Notify(pick_region, grab_outline);

        /* we will get the possible outlines from this oid only (for now) */
        add_outline_names(d, e, pick_region, oid);

        scroll_region := vcPicklist.Scroll("PickScroll", pick_region, 4, 6, 20);

        buttons := create_buttons();

        vcTable.Add(outline_table, label(""));
        /*        vcTable.Add(outline_table, label("Board Outlines")); */
        vcTable.Add(outline_table, scroll_region);
        vcTable.Add(outline_table, label("The board "+convert(string,name)+
                                         " has no outline. Please pick"));
        vcTable.Add(outline_table, label("a board outline from this list."));
        vcTable.Add(outline_table, buttons);
        vcTable.Add(outline_table, label(""));

        window = vcWin.Create(outline_table, vcWin.NoCmdline +
                               vcWin.NoMenus + vcWin.NoStatus +
                               vcWin.NoBindings);
        }


    if(string_eqi(convert(string, name), "*") || 
       string_eqi(convert(string, name), "?"))
        vpbrd(d,e,name,"");
    else
        if((oid := board_stat(name)) != -1)
            if(oid == 0)
                vpbrd(d,e,name,"");
            else
                {
                if(plot_stat() == 0)
                    create_outline_picklist(oid);
                else
                    plot_warning();
                }
        else
            error("No Viewplace database found", name);
    }
vpbrd_cmd := vcWv.Cmd(fac, start_board, "Vpbrd", 
                      "Enter Board Name", 
                      vcWv.Param("Board Name",pBlockNameVT,
                                 DYNAMIC, getblkname,
                                 "Board name to open in a new window"));
vcCmd.EnableApply (vpbrd_cmd, ON);

/* VPSYM command */
vpsym_cmd := vcWv.Cmd(fac,
                      func(d,e,n)
                      vpsym(d,e,n);,
                      "Vpsym",
                      "Enter Symbol Name",
                      vcWv.Param("Symbol Name",pBlockNameVT,
                                 DYNAMIC,getblkname,
                                 "Symbol name to open in a new window"));
vcCmd.EnableApply (vpsym_cmd, ON);

create_cmd := vcWv.Cmd(fac, func(d,e) call_pcb(d,e,0);, "CreateBoard");
update_cmd := vcWv.Cmd(fac, func(d,e) call_pcb(d,e,1);, "UpdateBoard");

/* GRID Spacing command */
gspacing_cmd := vcWv.Cmd(fac,gspacing,"Gspacing",
                         "Enter Grid",
                         vcWv.Param("Grid Spacing",vlUnsignedIntVT,
                                    DYNAMIC,ggspacing,
                                    "Set the grid spacing to this number"));
vcCmd.EnableApply (gspacing_cmd, ON);

/* These commands are all the old mode commands */
ron_cmd := vcWv.Cmd(fac,func(d,e)rdisp(d,e,pOnOffSelect.On);,"Ron");
roff_cmd := vcWv.Cmd(fac,func(d,e)rdisp(d,e,pOnOffSelect.Off);,"Roff");
rsel_cmd := vcWv.Cmd(fac,func(d,e)rdisp(d,e,pOnOffSelect.Select);,"Rsel");
don_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Detail,ON);},"Don");
doff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Detail,OFF);},"Doff");
cton_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.CompText,ON);
                                   vrefresh(d);},"Cton");
ctoff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.CompText,OFF);
                                    vrefresh(d);},"Ctoff");
bon_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Border,ON);
                                  vrefresh(d);},"Bon");
boff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Border,OFF);
                                   vrefresh(d);},"Boff");
ton_cmd := vcWv.Cmd(fac,func(d,e){setobjtypevis(pObject.Text,ON);
                                  vrefresh(d);},"Ton");
toff_cmd := vcWv.Cmd(fac,func(d,e){setobjtypevis(pObject.Text,OFF);
                                   vrefresh(d);},"Toff");
lon_cmd := vcWv.Cmd(fac,func(d,e){setobjtypevis(pObject.Label,ON);
                                  vrefresh(d);},"Lon");
loff_cmd := vcWv.Cmd(fac,func(d,e){setobjtypevis(pObject.Label,OFF);
                                   vrefresh(d);},"Loff");
aon_cmd := vcWv.Cmd(fac,func(d,e){setobjtypevis(pObject.Attribute,ON);
                                  vrefresh(d);},"Aon");
aoff_cmd := vcWv.Cmd(fac,func(d,e){setobjtypevis(pObject.Attribute,OFF);
                                   vrefresh(d);},"Aoff");
hon_cmd := vcWv.Cmd(fac,func(d,e){setmode(pObject.Header,ON);
                                  vrefresh(d);},"Hon");
hoff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pObject.Header,OFF);
                                  vrefresh(d);},"Hoff");
botbrd_cmd := vcWv.Cmd(fac,func(d,e){brdside(d,e,pBoardSide.Bottom);
                                    vrefresh(d);},"Botbrd");
topbrd_cmd := vcWv.Cmd(fac,func(d,e){brdside(d,e,pBoardSide.Top);
                                    vrefresh(d);},"Topbrd");
togbrd_cmd := vcWv.Cmd(fac,func(d,e){brdside(d,e,pBoardSide.Toggle);
                                    vrefresh(d);},"Togbrd");
allbrd_cmd := vcWv.Cmd(fac,func(d,e){brdside(d,e,pBoardSide.All);
                                    vrefresh(d);},"Allbrd");
gon_cmd := vcWv.Cmd(fac,func(display,event){gdisp(display,event,ON);},
                    "Gon");
goff_cmd := vcWv.Cmd(fac,func(display,event){gdisp(display,event,OFF);},
                     "Goff");
fvon_cmd := vcWv.Cmd(fac,func(d,e){fvdisp(d,e,pOnOffSelect.On);},"Fvon");
fvoff_cmd := vcWv.Cmd(fac,func(d,e){fvdisp(d,e,pOnOffSelect.Off);},"Fvoff");
fvsel_cmd := vcWv.Cmd(fac,func(d,e){fvdisp(d,e,pOnOffSelect.Select);},"Fvsel");
hson_cmd := vcWv.Cmd(fac,func(display,event){hsdisp(display,event,ON);},
                     "Hson");
hsoff_cmd := vcWv.Cmd(fac,func(display,event){hsdisp(display,event,OFF);},
                      "Hsoff");
dvton_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Dtype,ON);vrefresh(d);},
                      "Dvton");
dvtoff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Dtype,OFF);vrefresh(d);},
                       "Dvtoff");
rfon_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.RefDes,ON);vrefresh(d);},
                     "Rfon");
rfoff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.RefDes,OFF);vrefresh(d);},
                      "Rfoff");
pon_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Pad,ON);vrefresh(d);},
                    "Pon");
poff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Pad,OFF);vrefresh(d);},
                     "Poff");
thermon_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Reliability,OFF);
                                      setmode(pMode.OtherScale,OFF);
                                      setmode(pMode.Thermal,ON);
                                      vrefresh(d);},"Thermon");
thermoff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Thermal,OFF);
                                       vrefresh(d);},"Thermoff");
relon_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Reliability,ON);
                                    setmode(pMode.OtherScale,OFF);
                                    setmode(pMode.Thermal,OFF);
                                    vrefresh(d);},"Relon");
reloff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Reliability,OFF);
                                     vrefresh(d);},"Reloff");
otheron_cmd := vcWv.Cmd(fac,func(display,event){otset(display,event,ON);},
                        "Otheron");
otheroff_cmd := vcWv.Cmd(fac,func(display,event){otset(display,event,OFF);},
                         "Otheroff");
scalcon_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.CalcScale,ON);},
                        "Scalcon");
scalcoff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.CalcScale,OFF);},
                         "Scalcoff");
ratchain_cmd := vcWv.Cmd(fac,func(d,e){rcdisp(d,e,ON);},
                         "Ratchain");
ratspan_cmd := vcWv.Cmd(fac,func(d,e){rcdisp(d,e,OFF);},
                        "Ratspan");

auton_cmd := vcWv.Cmd(fac,
                      func(d,e,name)
                      aset(d,e,ON,name);,"Auton",
                      "Enter Net",
                      vcWv.Param("Net Name",pNameVT,CONSTANT,""));
vcCmd.EnableApply (auton_cmd, ON);

autoff_cmd := vcWv.Cmd(fac,
                       func(d,e,name)
                       aset(d,e,OFF,name);,"Autoff",
                       "Enter Net",
                       vcWv.Param("Net Name",pNameVT,CONSTANT,""));
vcCmd.EnableApply (autoff_cmd, ON);

uon_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Undo,ON);},"Uon");
uoff_cmd := vcWv.Cmd(fac,func(d,e){setmode(pMode.Undo,OFF);},"Uoff");
nlon_cmd := vcWv.Cmd(fac,func(d,e){nldisp(d,e,ON);},"Nlon");
nloff_cmd := vcWv.Cmd(fac,func(d,e){nldisp(d,e,OFF);},"Nloff");

/* TEXT ORIGIN command */
origin_cmd := vcWv.Cmd(fac,torigin,"Origin",
                       "Enter Origin",
                       vcWv.Param("Text Origin",pOrigin1VT,
                                  DYNAMIC,getorigin));
vcCmd.EnableApply (origin_cmd, ON);

o1_cmd := vcWv.Cmd(fac, func(d,e) torigin(d,e,0);,"O1");
o2_cmd := vcWv.Cmd(fac, func(d,e) torigin(d,e,1);,"O2");
o3_cmd := vcWv.Cmd(fac, func(d,e) torigin(d,e,2);,"O3");
o4_cmd := vcWv.Cmd(fac, func(d,e) torigin(d,e,3);,"O4");
o5_cmd := vcWv.Cmd(fac, func(d,e) torigin(d,e,4);,"O5");
o6_cmd := vcWv.Cmd(fac, func(d,e) torigin(d,e,5);,"O6");
o7_cmd := vcWv.Cmd(fac, func(d,e) torigin(d,e,6);,"O7");
o8_cmd := vcWv.Cmd(fac, func(d,e) torigin(d,e,7);,"O8");
o9_cmd := vcWv.Cmd(fac, func(d,e) torigin(d,e,8);,"O9");

/* TEXT SIZE command */
size_cmd := vcWv.Cmd(fac,tsize,"Size",
                     "Enter Size",
                     vcWv.Param("Text Size",vlPositiveIntVT,DYNAMIC,getsize,
                                "Size of annotation text"));
vcCmd.EnableApply (size_cmd, ON);

/* TEXT STRING command */
string_cmd := vcWv.Cmd(fac,tstring,"String",
                       "Enter String",
                       vcWv.Param("Text String",
                                  vlStringRestOfLine1VT,DYNAMIC,getstring,
                                  "New text string value"));
vcCmd.EnableApply (string_cmd, ON);

/* Attribute VISIBILITY command */
attrvis_cmd := vcWv.Cmd(fac,avis,"Attrvis",
                        "Enter Attribute Parameters",
                        vcWv.Param("Attribute Name",string,STICKY,"",
                                   "Name of attribute to change visibility"),
                        vcWv.Param("Visibility",pAttrVisibilityVT,
                                   STICKY,"Invisible",
                                   "Visibility Value"));
vcCmd.EnableApply (attrvis_cmd, ON);

/* These are the old attribute visibility commands */
avis_cmd := vcWv.Cmd(fac,func(d,e,n){avis(d,e,n,"Visible");},"Avis",
                     "Enter Name",
                     vcWv.Param("Attribute Name",string,STICKY,"",
                                "Name of attribute to make visible"));
vcCmd.EnableApply (avis_cmd, ON);

anvis_cmd := vcWv.Cmd(fac,func(d,e,n){avis(d,e,n,"Name");},"Anvis",
                      "Enter Name",
                      vcWv.Param("Attribute Name",string,STICKY,"",
                                 "Name of attribute to make name invisible"));
vcCmd.EnableApply (anvis_cmd, ON);

avvis_cmd := vcWv.Cmd(fac,func(d,e,n){avis(d,e,n,"Value");},"Avvis",
                      "Enter Name",
                      vcWv.Param("Attribute Name",string,STICKY,"",
                                 "Name of attribute to make value visible"));
ainvis_cmd := vcWv.Cmd(fac,func(d,e,n){avis(d,e,n,"Invisible");},"Ainvis",
                       "Enter Name",
                       vcWv.Param("Attribute Name",string,STICKY,"",
                                  "Name of attribute to make invisible"));

/* BRDSIDE command */
brdside_cmd := vcWv.Cmd(fac,brdside,"Brdside",
                        "Change Board Side",
                        vcWv.Param("Board Side",pBoardSideVT,STICKY,"Top",
                                   "Change view to new board side"));

/* COLOR command */
color_cmd := vcWv.Cmd(fac,color,"Color",
                      "Change Color",
                      vcWv.Param("Color",vgColorVT,DYNAMIC,getobjcolor,
                                 "Change the color of selected objects"));

/* FONT command */
font_cmd := vcWv.Cmd(fac,font,"Font",
                     "Change Font",
                     vcWv.Param("Font", vgStkVT, STICKY, "Fixed",
                                "Change the font of the selected text"));

/* FSTYLE command */
fstyle_cmd := vcWv.Cmd(fac,fstyle,"Fstyle",
                       "Change Fill Style",
                       vcWv.Param("Fill",vgFillVT,DYNAMIC,getobjfstyle,
                                  "Change the fill style of the selected object(s)"));

/* LSTYLE command */
lstyle_cmd := vcWv.Cmd(fac,lstyle,"Lstyle",
                       "Change Line Style",
                       vcWv.Param("Line Pattern",vgLinePatternVT,
                                  DYNAMIC,getobjlstyle,
                                  "Change the line style"));

/* JSTYLE command */
jstyle_cmd := vcWv.Cmd(fac,func(d,e,join)
                       {
                       jstyle(d,e,join);
                       vrefresh(d);
                       },"Jstyle",
                       "Change Join Style",
                       vcWv.Param("Join",vgLineJoinVT,STICKY,"Miter",
                                  "Change the join style"));
    
    /* CSTYLE command */
    cstyle_cmd := vcWv.Cmd(fac,func(d,e,cap)
                       {
                       cstyle(d,e,cap);
                       vrefresh(d);
                       },"Cstyle",
                       "Change Cap Style",
                       vcWv.Param("Cap",vgLineCapVT,STICKY,"Butt",
                                  "Change the cap style"));

/* THICK command */
thick_cmd := vcWv.Cmd(fac,func(d,e,t)
                      {
                      thick(d,e,t);
                      vrefresh(d);
                      },"Thick",
                      "Change Line Thickness",
                      vcWv.Param("Thickness",vlUnsignedIntVT,
                                 STICKY,0,
                                 "Change the line thickness"));
    
vcCmd.EnableApply (font_cmd, ON);
vcCmd.EnableApply (size_cmd, ON);
vcCmd.EnableApply (color_cmd, ON);
vcCmd.EnableApply (fstyle_cmd, ON);
vcCmd.EnableApply (lstyle_cmd, ON);
vcCmd.EnableApply (jstyle_cmd, ON);
vcCmd.EnableApply (cstyle_cmd, ON);
vcCmd.EnableApply (thick_cmd, ON);

swpside_cmd := vcWv.Cmd(fac,swpside,"Swpside");

function delete_net()
{
    function end_selrat(display, event)
        {
        unbind_display(display,event);
        }
    end_selrat_cmd := vcWv.Cmd(fac, end_selrat, "End-Remove-Rat");

    function delrat(display, event)
        select_rat(display, event, 0);
    delrat_cmd := vcWv.Cmd(fac, delrat, "Delete-Rat");

    function selrat(display, event)
        select_rat(display, event, 1);
    selrat_cmd := vcWv.Cmd(fac, selrat, "Select-Rat");

    function init_selrat(display, event)
        {
        clear_binding_table();
        vcBind.Set(bt, '\leftdown', selrat_cmd);
        vcBind.Set(bt, '\middledown', delrat_cmd);
        vcBind.Set(bt, '\rightdown', end_selrat_cmd);
        bind(display, bt);
        vcWin.ClearStatus(getwindow(display));
        vcWin.AddStatus(getwindow(display),
                        "Specify net segments to delete");
        }
    init_selrat_cmd := vcWv.Cmd(fac, init_selrat, "Init-Selrat");

    return(vcWv.Cmd(fac, init_selrat, ""));
    }
do_delnet_cmd := delete_net();

function run_delnet(display, event)
{
    vcCmd.Run(do_delnet_cmd, display, event, nil, TRUE);
    }
delnet_cmd := vcWv.Cmd(fac, run_delnet, "Delete-Net");

delete_cmd := vcWv.Cmd(fac,func(d,e)
                       {
                       if(numsel(d) == 1)
                           {
                           if(getobjtype(d) == pObject.Net && board(d) == 1)
                               vcCmd.Run(delnet_cmd, d, e, nil, TRUE);
                           else
                               delete(d,e);
                           }
                       else
                           delete(d,e);
                       vrefresh(d);
                       },
                       "Delete");

undo_cmd := vcWv.Cmd(fac,undo,"Undo");

snet_cmd := vcWv.Cmd(fac,snet,"Snet",
                     "Select Net",
                     vcWv.Param("Net Name",string,STICKY,"*",
                                "Name of the net to be selected"));

satt_cmd := vcWv.Cmd(fac,satt,"Satt",
                     "Select Attribute",
                     vcWv.Param("Attribute Name",string,STICKY,"*",
                                "Name of the attribute to be selected"));

stype_cmd := vcWv.Cmd(fac,stype,"Stype",
                      "Select Device Type",
                      vcWv.Param("Device Type",string,STICKY,"*",
                                 "Type of device to be selected"));

sref_cmd := vcWv.Cmd(fac,sref,"Sref",
                     "Select Refdes",
                     vcWv.Param("Reference Designation",string,
                                STICKY,"*",
                                "Refdes to be selected"));

sdist_cmd := vcWv.Cmd(fac,sdistance,"Sdistance",
                      "Select Distance",
                      vcWv.Param("Selection Distance",vlPositiveIntVT,
                                 DYNAMIC,get_value(pValue.Selection),
                                 "Selection distance (pixels)"));

dbrd_cmd := vcWv.Cmd(fac,
                     func(d,e,bname)
                     {
                     toggle_busy(d);
                     dbrd(d,e,bname);
                     toggle_busy(d);
                     },"Dirbrd",
                     "Directory of Boards",
                     vcWv.Param("Board Name",string,STICKY,"*",
                                "Name of board(s) to be listed"));

dsym_cmd := vcWv.Cmd(fac,
                     func(d,e,sname)
                     {
                     toggle_busy(d);
                     dsym(d,e,sname);
                     toggle_busy(d);
                     },"Dirsym",
                     "Directory of Symbols",
                     vcWv.Param("Symbol Name",string,STICKY,"*",
                                "Name of symbol(s) to be listed"));

dir_cmd := vcWv.Cmd(fac,
                    func(d,e,name)
                    {
                    toggle_busy(d);
                    dir(d,e,name);
                    toggle_busy(d);
                    },"Dir",
                    "Directory of Memory",
                    vcWv.Param("Name",string,STICKY,"*",
                               "Names of file(s) in memory to be listed"));

odetail_cmd := vcWv.Cmd(fac,
                        func(d,e)
                        {
                        toggle_busy(d);
                        odetail(d,e);
                        toggle_busy(d);
                        },"Odetail");

ocount_cmd := vcWv.Cmd(fac,
                       func(d,e)
                       {
                       toggle_busy(d);
                       ocount(d,e);
                       toggle_busy(d);
                       }
                       ,"Ocount");

rlist_cmd := vcWv.Cmd(fac,rlist,"Rlist");

nlist_cmd := vcWv.Cmd(fac,
                      func(d,e,name,length,sort_flag)
                      {
                      toggle_busy(d);
                      nlist(d,e,name,length,sort_flag);
                      toggle_busy(d);
                      }
                      ,"Nlist",
                      "Info Nets",
                      vcWv.Param("Net Name",string,STICKY,"*",
                                 "Names of nets to be listed"),
                      vcWv.Param("Minimum length",float,STICKY,0.0,
                                 "Smallest length of net to be included in listing"),
                      vcWv.Param("Sort by name",vlYesNoVT,STICKY,NO,
                                 "Select YES for alphabetized list"));

status_cmd := vcWv.Cmd(fac,status,"Status");

function get_log_name(display)
{
    /* had to convert getblkname to a string because it is pBlockNameVT */
    if(getblktype(display) == pBlockType.Board)
        return(convert(string, getblkname(display))+".brd");
    else
        return(convert(string, getblkname(display))+".1");
    }

/* this function reads a block but checks to see if it is modified */
function read_block(display,event,block_name)
{
    prompt := "Block "+convert(string,
                               getblkname(display))+" has changed; Are you sure?";
    
    function are_you_sure(d, e, y_n)
        if (y_n == YES) 
            read(d,e,block_name);
    
    prompt_read_cmd := vcWv.Cmd(fac, are_you_sure, "PromptRead",
                                "Are you sure?",
                                vcWv.Param(prompt, vlYesNoVT, CONSTANT, YES));
    
    if (blkmodified(display) == 1)
        vcCmd.Run(prompt_read_cmd,display,event,nil,TRUE);
    else 
        read(display, event, block_name);
    }
read_cmd := vcWv.Cmd(fac,read_block,"Read",
                     "File Read",
                     vcWv.Param("Block Name",pBlockNameVT,
                                DYNAMIC,getblkname,
                                "Name of block to be read into this window"));

/* This callback determines whether the file for the block exists or not
   and then invokes the appropriate command. */
function maybe_writeto(display,event,name)
{
    fname := getblkfname(display,name);
    prompt := "File "+fname+" exists; Are you sure?";
    
    prompt_cmd := vcWv.Cmd(fac,func(d,e,y_n)
                           {
                           if (y_n == YES)
                               writeto(d,e,name);
                           }
                           ,"Prompt",
                           "Are you sure?",
                           vcWv.Param(prompt,vlYesNoVT,CONSTANT,YES));
    if (fexists(fname) == TRUE)
        vcCmd.Run(prompt_cmd,display,event,nil,TRUE);
    else writeto(display,event,name);
    }
writeto_cmd := vcWv.Cmd(fac,maybe_writeto,"WriteTo",
                        "File WriteTo",
                        vcWv.Param("Name",pBlockNameVT,DYNAMIC,getblkname,
                                   "Name of file to write this block to"));

savelog_cmd := vcWv.Cmd(fac,savelog,"SaveLog");

write_cmd := vcWv.Cmd(fac,write,"Write");

restore_cmd := vcWv.Cmd(fac,restore,"Restore");

check_cmd := vcWv.Cmd(fac,check,"Check");

rdatt_cmd := vcWv.Cmd(fac,rdatt,"Rdatt");

/* Board commands */
cbrd_cmd := vcWv.Cmd(fac,cbrd,"Cbrd",
                     "Change Board Outline",
                     vcWv.Param("Board Outline Name",pBlockNameVT,STICKY,"",
                                "Name of outline to replace current outline"));

bsize_cmd := vcWv.Cmd(fac,bsize,"Bsize",
                      "Change Block Size",
                      vcWv.Param("Block Width",pBlockWidthVT,
                                 DYNAMIC,getblkwidth,
                                 "New block width"),
                      vcWv.Param("Block Height",pBlockHeightVT,
                                 DYNAMIC,getblkheight,
                                 "New block height"));

btype_cmd := vcWv.Cmd(fac,btype,"Btype",
                      "Change Block Type",
                      vcWv.Param("Block Type",pBlockTypeVT,
                                 DYNAMIC,getblktype,
                                 "Change the block type to..."));

bsymbol_cmd := vcWv.Cmd(fac,func(d,e)btype(d,e,"Symbol");,"Bsymbol");
bannotate_cmd := vcWv.Cmd(fac,func(d,e)btype(d,e,"Annotate");,"Bannotate");
bboard_cmd := vcWv.Cmd(fac,func(d,e)btype(d,e,"Board");,"Bboard");

/* Place components */
do_rman_cmd := vcWv.Cmd(fac,
                        func(d,e,r)
                        {
                        rman(d,e,r);
                        unbind_display(d,e);
                        },
                        "DoRman",
                        "Place By Refdes",
                        vcWv.Param("RefDes Name",pNameVT,STICKY,"",
                                   "Name of refdes to place"));
vcCmd.EnableMacro(do_rman_cmd, OFF);
rman_cmd := make_button_cmd(do_rman_cmd,"Rman");

/* Remove ratsnest */
rrmv_cmd := vcWv.Cmd(fac,rrmv,"Rrmv",
                     "Remove Ratsnest",
                     vcWv.Param("Net Name",pNameVT,STICKY,"",
                                "Name of net to remove"));
    
/* Dialog box commands */
cattr_cmd := vcWv.Cmd(fac,
                      func(d,e) cattr(d,e,FILTER_FILE);,
                      "Cattr");
vcCmd.EnableMacro(cattr_cmd, OFF);

cmodes_cmd := vcWv.Cmd(fac,cmodes,"Cmodes");
vcCmd.EnableMacro(cmodes_cmd, OFF);

cvalues_cmd := vcWv.Cmd(fac,cvalues,"Cvalues");

runplc_cmd := vcWv.Cmd(fac,runplc,"Runplc");

/* Create command for hightlighting components */
hcomp_cmd := vcWv.Cmd(fac,hcomp,"Hcomp",
                      nil,
                      vcWv.Param("RefDes",string,STICKY,""),
                      vcWv.Param("Color",vgColorVT,STICKY,vgColor.Red),
                      vcWv.Param("Fill",vgFillVT,STICKY,vgFill.Diagdn1),
                      vcWv.Param("Line",vgLinePatternVT,STICKY,
                                 vgLinePattern.Solid),
                      vcWv.Param("Board Name",pBlockNameVT,DYNAMIC,getblkname));
/* Create command for display net values */
dnvalue_cmd := vcWv.Cmd(fac,dnvalue,"Dnvalue",
                        nil,
                        vcWv.Param("Net Name",string,STICKY,""),
                        vcWv.Param("Value",string,STICKY,""),     
                        vcWv.Param("Color",vgColorVT,STICKY,vgColor.Red),
                        vcWv.Param("Board Name",pBlockNameVT,DYNAMIC,getblkname));

/* Command to select pins */
spin_cmd := vcWv.Cmd(fac, spin, "Spin",
                     nil,
                     vcWv.Param("Pin Name",string,STICKY,""));

/* This is to assist plotting */
toggle_cmd := vcWv.Cmd(fac, func(d,e)ptoggle();, "Ptoggle");

/*-*/
load ("vplacebt.vs");
    
/*-*/
load ("vplacemn.vs");

load ("vplaceht.vs");

/***
 * This function extract the name portion from a full file specification
 ***/
function extract_name(bname)
{
    fname := convert(string,bname);
    /* find the last slash character, if none then start at first character */
    start := rindex(fname,'/');
    if(start == nil) 
        start = 0;
    else 
        start = start+1;
    
    /* find last dot character, if none then end at last character */
    end := rindex(fname,'.');
    if(end == nil) 
        end = string_length(fname);
    
    /* return the extracted substring for the name */
    return(convert(pBlockNameVT,substring(fname,start,end)));
    }

/***
 * This function creates a component when a file is dropped in a vplace 
 * window.
 ***/
function _Drop(display,event)
{
    if (comp(display,event,extract_name(vcEvent.String(event))) == 1)
        {
        enable_cmds(display,event);
        move_created_object(display,event);
        bind_middle_button(comp_cmd);
        }
    }

/* This function disables all commands could be disabled */
function disable_cmds()
{
    vcCmd.Enable(comp_cmd,OFF);
    vcCmd.Enable(pin_cmd,OFF);
    vcCmd.Enable(net_cmd,OFF);
    vcCmd.Enable(label_cmd,OFF);
    vcCmd.Enable(attr_cmd,OFF);
    
    vcCmd.Enable(move_cmd,OFF);
    vcCmd.Enable(copy_cmd,OFF);
    vcCmd.Enable(rotate_cmd,OFF);
    vcCmd.Enable(rotate_ol_cmd,OFF);
    vcCmd.Enable(swpside_cmd,OFF);
    vcCmd.Enable(refary_cmd,OFF);
    vcCmd.Enable(typary_cmd,OFF);
    vcCmd.Enable(netary_cmd,OFF);
    vcCmd.Enable(xyplc_cmd,OFF);
    vcCmd.Enable(scale_cmd,OFF);
    vcCmd.Enable(rrmv_cmd,OFF);
    vcCmd.Enable(rman_cmd,OFF);
    vcCmd.Enable(plcall_cmd,OFF);
    vcCmd.Enable(symorg_cmd,OFF);
    vcCmd.Enable(stretch_cmd,OFF);
    vcCmd.Enable(delete_cmd,OFF);
    vcCmd.Enable(array_cmd,OFF);
    vcCmd.Enable(swap_cmd,OFF);
    vcCmd.Enable(crdsym_cmd,OFF);
    vcCmd.Enable(color_cmd,OFF);
    vcCmd.Enable(fstyle_cmd,OFF);
    vcCmd.Enable(lstyle_cmd,OFF);
    vcCmd.Enable(jstyle_cmd,OFF);
    vcCmd.Enable(cstyle_cmd,OFF);
    vcCmd.Enable(font_cmd,OFF);
    vcCmd.Enable(thick_cmd,OFF);
    
    vcCmd.Enable(size_cmd,OFF);
    vcCmd.Enable(string_cmd,OFF);
    vcCmd.Enable(origin_cmd,OFF);
    vcCmd.Enable(font_cmd,OFF);
    
    vcCmd.Enable(zselect_cmd,OFF);
    
    vcCmd.Enable(cbrd_cmd,OFF);
    vcCmd.Enable(brdside_cmd,OFF);
    vcCmd.Enable(bsize_cmd,OFF);
    vcCmd.Enable(btype_cmd,OFF);

    vcCmd.Enable(check_cmd,OFF);
    }


/* This function enables commands based on what data is selected */
function enable_cmds(display,event)
{
    disable_cmds(); /* disable everything first */
    
    num_selected := numsel(display);
    board_type := board(display);
    
    if (num_selected == 0)
        {
        vcCmd.Enable(attr_cmd,ON);
        }
    else if (num_selected >= 1)
        {
        if (num_selected == 1)
            {
            if (editstrch(display) == 1)
                vcCmd.Enable(stretch_cmd,ON);
            }
        
        vcCmd.Enable(zselect_cmd,ON);
        vcCmd.Enable(color_cmd,ON);
/*        vcCmd.Enable(fstyle_cmd,ON); */
        vcCmd.Enable(lstyle_cmd,ON); 
        vcCmd.Enable(cstyle_cmd,ON); 
        vcCmd.Enable(jstyle_cmd,ON); 
        vcCmd.Enable(thick_cmd,ON); 
        
        if (eselected(display,"move") == 1)
            vcCmd.Enable(move_cmd,ON); 
        if (eselected(display,"delete") == 1)
            vcCmd.Enable(delete_cmd,ON); 
        if (eselected(display,"copy") == 1)
            vcCmd.Enable(copy_cmd,ON); 
        if (eselected(display,"rotate") == 1)
            vcCmd.Enable(rotate_cmd,ON); 
        if (eselected(display,"scale") == 1)
            vcCmd.Enable(scale_cmd,ON); 
        if (eselected(display,"array") == 1)
            vcCmd.Enable(array_cmd,ON);
        
        if (tselected(display) == 1)
            {
            vcCmd.Enable(size_cmd,ON);
            vcCmd.Enable(string_cmd,ON);
            vcCmd.Enable(origin_cmd,ON);
            vcCmd.Enable(font_cmd,ON);
            }
        else
            vcCmd.Enable(fstyle_cmd,ON);
        
        /* must a symbol and ok to label */
        if ((board_type != 1) && (oklabel(display) == 1))
            vcCmd.Enable(label_cmd,ON); 
        
        if (okattr(display) == 1)
            vcCmd.Enable(attr_cmd,ON); 
        
        /* need two components selected for swap command */
        if (twocomps(display) == 1)
            vcCmd.Enable(swap_cmd,ON);
        }
    
    /* if board block */
    if (board_type == 1)
        {
        vcCmd.Enable(check_cmd,ON);
        vcCmd.Enable(refary_cmd,ON);
        vcCmd.Enable(typary_cmd,ON);
        vcCmd.Enable(netary_cmd,ON);
        vcCmd.Enable(xyplc_cmd,ON);
        vcCmd.Enable(comp_cmd,ON);
        vcCmd.Enable(net_cmd,ON);
        vcCmd.Enable(refary_cmd,ON);
        vcCmd.Enable(typary_cmd,ON);
        vcCmd.Enable(netary_cmd,ON);
        vcCmd.Enable(cbrd_cmd,ON);
        vcCmd.Enable(brdside_cmd,ON);
        vcCmd.Enable(rman_cmd,ON);
        vcCmd.Enable(rrmv_cmd,ON);
        vcCmd.Enable(plcall_cmd,ON);
        vcCmd.Enable(rotate_ol_cmd,ON); 
        vcCmd.Enable(swpside_cmd,ON);
        }
    else
        {
        vcCmd.Enable(pin_cmd,ON);
        vcCmd.Enable(crdsym_cmd,ON);
        vcCmd.Enable(symorg_cmd,ON);
        vcCmd.Enable(bsize_cmd,ON);
        vcCmd.Enable(btype_cmd,ON);
        }

    vcWin.UpdateMenus(vcEvent.Window(event));
    }

/* these commands bring up the appropriate pop-up menu */
brd_popup_cmd := vcWv.Cmd(fac,
                          func(d,e)vcMenuPop.create(board_popup_menu,e);,
                          "Menu-Board");  
vcCmd.EnableMacro(brd_popup_cmd, OFF);

sym_popup_cmd := vcWv.Cmd(fac,
                          func(d,e)vcMenuPop.create(symbol_popup_menu,e);,
                          "Menu-Symbol");
vcCmd.EnableMacro(sym_popup_cmd, OFF);

/* This function scrolls the view if the cursor is close to the edge */
function auto_scroll(display,event)
{
    x := vcEvent.X(event);
    y := vcEvent.Y(event);
    vp := getviewport(display);
    size := hssize();
    
    r := vgU.Region(vp);

    if ((x - vgRect.Xmin(r)) < (10 + (2 * size))
        || (vgRect.Xmax(r) - x) < (10 + (2 * size))
        || (y - vgRect.Ymin(r)) < (10 + (2 * size))
        || (vgRect.Ymax(r) - y) < (10 + (2 * size)))
        {
        vgU.PixelToUser(vp,&x,&y);
        scroll(display,event);
        vgU.MoveCursor(vp,x,y);
        vrefresh(display);        
        }
    }

/*
function before(display,event)
{
    ;
    }
*/

function after(display,event)
{
    enable_cmds(display,event);
    }

/* This function is called whenever the cursor enters a window */
function _FocusIn(display,event)
{
    /* enable appropriate commands for current display */
    enable_cmds(display,event);
    
    /* bind appropriate pop-up menu based on type of block */
    if (board(display) == 1)
        {
        vcBind.Set(bindings,'\rightdown',brd_popup_cmd);
        vcBind.Set(bindings,'\^\rightdown',brd_popup_cmd);
        }
    else
        {
        vcBind.Set(bindings,'\rightdown',sym_popup_cmd);
        vcBind.Set(bindings,'\^\rightdown',sym_popup_cmd);
        }

    if(mode_flag == pDrawMode.Draw)
        vcWin.SetCursor(getwindow(display), vgCursor.Cross);
    else
        vcWin.SetCursor(getwindow(display), vgCursor.Arrow);
    }

function _FocusOut(display,event)
{
    vcWin.SetCursor(vcEvent.Window(event), vgCursor.Arrow);
    }


function box_prompt(display)
{
    vcWin.ClearStatus(window := getwindow(display));
    vcWin.AddStatus(window, "Specify placement box");
    }
