/*  vsimList.vs -- a simpler version of list browser
 *
 *  $Log: vsimList.vs,v $
Revision 1.13.2.2  1995/03/03  21:59:35  dahan
Add fcs_vsim531 label

Revision 1.13  1994/09/05  15:17:06  sridhar
General code clenup

Revision 1.11  1994/08/05  00:21:02  sridhar
bug fix; bug id# 15178, 15151, 15287

Revision 1.9  1994/07/28  20:29:53  sridhar
bug fix: bug_id: 15223

Revision 1.8  1994/07/21  14:52:13  sridhar
a host of changes:
added region caching for spreadsheet.
added "DeselectAll()" function
added some optimizations to QueryAndAddRecords()
modified the prefs to be more "user-friendly"

Revision 1.7  94/07/14  11:03:29  sridhar
added CORE53 improvements for vcPicklist: vcPicklist.CallOnSelectDone()
added the "std_select" preference for vcPicklist

Revision 1.6  94/07/05  16:52:04  sridhar
Fixed a scale_update bug...

Revision 1.5  94/07/01  11:52:15  sridhar
*** empty log message ***

Revision 1.4  94/06/17  16:39:22  sridhar
Fixed a filter-refresh bug

Revision 1.3  94/06/14  15:44:32  sridhar
Hides the "scale"/"Prev"/"Next" buttons in spreadsheet, if the
number of items displayed is < frame_size

Revision 1.2  94/06/14  11:08:58  sridhar
Added a picklist_action_func() which calls a vsimNav API func for updating
the cutbuffer with the current  spreadsheet selection

Revision 1.1  94/06/13  16:30:49  sridhar
Initial revision

 *  Revision 1.5  94/06/02  11:30:26  sridhar
 *  Design browser has a picklist for large designs
 *
 *  Revision 1.4  94/05/26  10:22:23  sridhar
 *  'C' query and update function.
 *
 *  Revision 1.3  94/05/16  12:00:59  sridhar
 *  Has 'C' caching of query and update records
 *
 *  Revision 1.2  94/04/22  14:26:38  sridhar
 *  added filteration function -- vscript.re_exec
 *
 *  Revision 1.1  94/04/20  09:15:01  sridhar
 *  Initial revision
 */



function construct_vector_label_structs (num_fields, label_structs)
{
    vec_label_structs := make_vector (num_fields);
    for (i := 0;  i < num_fields;  ++i)
    {
	label_struct := label_structs[i];
	vec := [label_struct.label,
		label_struct.fac,
		label_struct.prefs,
		label_struct.notify_func,
		label_struct.value_pos,
		label_struct.visible];
	vec_label_structs[i] = vec;
    }

    vec_label_structs;
}


function update_regexp_column (this, field_name)
{
    label_structs	:= this.label_structs;
    num_label_structs	:= this.num_label_structs;

    for (i := 0;  i < num_label_structs;  ++i)
    {
	label_struct := label_structs [i];
	if (string_eq (field_name, label_struct.label))
	{
	    this.regexp_column	= i;
	    this.field_name	= field_name;
	    i = num_label_structs;
	}
    }
}


/*  reorders the given tables' contents using the old_new_assoc_array
 */
function swap_table_contents (table, num_cols, region_array,
				old_new_assoc_array, new_label_structs)
{
    /*  copy table to temporary array
     */
    for (i := 0;  i < num_cols;  ++i)
	region_array[i] = vcTable.Get (table, 0, i);

    /*  reset the table
     */
    for (i = 0;  i < num_cols;  ++i)
	vcTable.Set (table, 0, i, vcRubber.Create ("x", 0, 0));

    /*  reassign the regions to the correct slots
     */
    reg := nil;
    for (i = 0;  i < num_cols;  ++i) {
	reg = region_array [i];
	vcTable.Set (table, 0, old_new_assoc_array[i], reg);

	/*
	if (new_label_structs [old_new_assoc_array[i]].visible) 
	    vcRegion.SetVisible (reg, vcRegion.Area (reg));
	else
	    vcRegion.SetInvisible (reg);
	*/
    }
}


/*  ReorderFormat -- of the vsimlist. ie., the column ordering/visibility, etc
 */
function ReorderFormat (region, order_list)
{
    this := vcRegion.GetData (region);

    label_table := this.label_table;

    label_structs	:= this.label_structs;
    num_label_structs	:= this.num_label_structs;
    new_label_structs	:= make_vector (num_label_structs);
    region_array	:= make_vector (num_label_structs);
    old_new_assoc_array	:= make_vector (num_label_structs);
    name		:= nil;

    /*  associate the old column contents to new columns
     */
    last_one := num_label_structs;
    label_struct := nil;

    for (i := 0;  i < num_label_structs;  ++i)
    {
	name = label_structs[i].label;

	foreach (item; vscript.iter (order_list))
	    if (string_eq (name, item[0]))
	    {
		if (! item[1])
		{
		    old_new_assoc_array [i] = --last_one;
		    new_label_structs [last_one] = label_structs[i];
		    label_struct = new_label_structs [last_one];
		    label_struct.visible = item[1];
		}
		else
		{
		    old_new_assoc_array [i] = item[2];
		    new_label_structs [item[2]] = label_structs[i];
		    label_struct = new_label_structs [item[2]];
		    label_struct.visible = item[1];
		}
	    }
    }

    /*  change header labels
     */
    swap_table_contents (label_table, num_label_structs, region_array,
			 old_new_assoc_array, new_label_structs);

    /*  change the column ordering in the picklist
     *
     *  foreach picklist entry
     */
    tables := get_picklist_entries (this.picklist);
    foreach (table;  vscript.iter (tables))
    {
	swap_table_contents (table, num_label_structs, region_array,
			     old_new_assoc_array, new_label_structs);
	vcTable.Col (table, vcTable.All, vcTable.Border.Invisible, 3, 3, 3,
		 vgAlign.MiddleCenter);
    }

    vcTable.AlignColumns (cons (label_table, tables));

    /*  change the column ordering of the tables in region_cache 
     *
     *  for each table in "region_cache"
     */
    tables := this.region_cache;
    foreach (table;  vscript.iter (tables))
    {
	swap_table_contents (table, num_label_structs, region_array,
			     old_new_assoc_array, new_label_structs);
	vcTable.Col (table, vcTable.All, vcTable.Border.Invisible, 3, 3, 3,
		 vgAlign.MiddleCenter);
    }

    this.label_structs = new_label_structs;
    update_regexp_column (this, this.field_name);

    this.vec_label_structs =
    construct_vector_label_structs (num_label_structs, this.label_structs);

    nil;
}


/*  returns the formatting of the spreadsheet
 *  ie., Column-1: name, isvisible, Column-2: name, isvisible, etc.
 */
function GetFormat (region)
{
    this		:= vcRegion.GetData (region);
    out_list		:= nil;
    label_struct	:= nil;
    label_structs	:= this.label_structs;
    num_label_structs	:= this.num_label_structs;

    /*  for each label_struct corr. to the order of the columns.....
     */
    for (i := 0, j := 0;  i < num_label_structs;  ++i)
    {
	label_struct = label_structs[i];
	j = (label_struct.visible) ? (i) : -1;
	out_list = cons (list (label_struct.label, label_struct.visible, j),
			 out_list);
    }
    reverse_destructive (out_list);
}


/*  returns a list of table regions corr. to each entry in the picklist
 */
function get_picklist_entries (picklist)
{
    declare	rows, cols;

    vcPicklist.Dimensions (picklist, &rows, &cols);

    tab_list := nil;
    for (--rows;  rows >= 0;  --rows)
	tab_list = cons (car (vcPicklist.Get (picklist, rows, 0)), tab_list);
    tab_list;
}


/*  returns a list of table regions corr to the selected entries in the picklist
 */
function get_selected_entries (picklist)
{
    declare	rows, cols;

    vcPicklist.Dimensions (picklist, &rows, &cols);

    tab_list := nil;
    for (--rows;  rows >= 0;  --rows)
	if (vcPicklist.IsSet (picklist, rows))
	    tab_list = cons (car (vcPicklist.Get (picklist, rows, 0)),tab_list);
    tab_list;
}


/*  get_records_in_list -- return the information stored in each table 
 *  in tab_list. the tab_list corr. to the picklist rows displayed
 */
function get_records_in_list (region, tab_list)
{
    declare	labels;
    declare	table_record;
    declare	label_struct;
    declare	col;
    declare	reg;


    this		:= vcRegion.GetData (region);

    /*  for each table in tab_list
     *      for each entry in label_struct
     *          get table entry;
     *          get the value of the table entry;
     *          append (label_struct entry . value) to table_record
     *      end_for
     *      append table_record to out_list
     *  end_for
     *  return reverse of out_list
     */
    out_list	:= nil;
    for ( ;  tab_list != nil;  tab_list = cdr (tab_list)) {
	table_record = nil;
	col = 0;

	for (col = 0;  col < this.num_label_structs;  ++col) {
	    label_struct = vector_ref (this.label_structs, col);
	    reg = vcTable.Get (car (tab_list), 0, col);
	    table_record = cons (
			    list (label_struct.label, vcRegion.Get(reg,vscmVT)),
			    table_record);
	}

	out_list = cons (reverse (table_record), out_list);
    }

    reverse (out_list);
}


/*  deletes all records in the picklist 
 */
function remove_picklist_entries (region)
{
    this := vcRegion.GetData (region);
    vcPicklist.DeselectAll (this.picklist);
}


function SetRegExp (region, field_name, regexp_string)
{
    this := vcRegion.GetData (region);
    n    := FALSE;

    if (! isstring (regexp_string))
	regexp_string = this.regexp_string;

    if (string_eqi (this.regexp_string, regexp_string))
	return (nil);

    /*  update the "match field column number" if field_name
     *  is different
     */
    if (! string_eqi (this.field_name, field_name)) 
	update_regexp_column (this, field_name);

    /*  update and recompile regular expression
     */
    regexp_comp := CompileRegexp (regexp_string);

    /*  if proper regular expression, then process the rest
     */
    if (regexp_comp != nil)
    {
	this.regexp_string	= regexp_string;
	this.regexp_compiled	= regexp_comp;

	/*  and get the matching tables from primary table_list
	 */
	query_info := this.query_info;
	remove_picklist_entries (region);
	n = QueryAndAddRecords(region,query_info.scope,query_info.type_name,TRUE);
    }
    n;
}


function call_notify_func (list_reg, rec_table, event)
{
    arg         := vcRegion.PrefGet (list_reg, "arg");
    notify_func := vcRegion.PrefGet (list_reg, "notify_func");
    record      := get_records_in_list (list_reg, list (rec_table));
    if (notify_func != nil)
	(notify_func) (list_reg, car (record), arg, event);
}


function create_label_struct ()
    struct {
	label;
	fac;
	prefs;
	notify_func;
	value_pos;
	visible
    };


/*  create a new scale with "min <- 1" and "max <- num_frames"
 */
function create_and_add_scale (region, directory_table, query_info)
{
    scale := vcRegion.Create ("Scale", vcScale.fac, 1, 
				min   <- 1,
				max   <- query_info.num_frames,
				ticks <- query_info.num_frames);
    vcScale.CallOnChange (scale, reg_scale (region));
    vcTable.Set (directory_table, 0, 0, scale);
    scale;
}


function update_directory_panel (region, frame_num)
{
    this := vcRegion.GetData (region);

    /*  update scale
     */

    vcScale.Set (this.scale, frame_num);
    /*
    vcRegion.Refresh (this.directory_table,vcRegion.Area(this.directory_table));
    */
}


function show_current_frame (region)
{
    this       := vcRegion.GetData (region);
    query_info := this.query_info;

    this.region_cache =
    ShowNthFrame (query_info.scope, query_info.type_name,this.vec_label_structs,
			list (this.label_table, this.picklist, region),
			query_info.name_list, query_info.max_items,
			query_info.frame_size, query_info.current_frame,
			this.region_cache);
    /*
    vcRegion.Refresh (region, vcRegion.Area (region));
    */
}


function QueryAndAddRecords (region, scope, type_name, force)
{
    this := vcRegion.GetData (region);
    arg  := vcRegion.PrefGet (region, "arg");


    /*  if the vsimList region is showing signals of the same scope
     *  then don't bother re-querying them
     */
    if ((! force)  &&
	isstring (this.query_info.scope)  &&
	string_eq (this.query_info.scope, scope))
    {
	if (this.query_info.max_items > 0)
	    return (TRUE);
	return (FALSE);
    }

    num_fields        := this.num_label_structs;
    vec_label_structs := this.vec_label_structs;

    name_list := QueryMatchingItems (scope, type_name, vec_label_structs,
			num_fields, this.label_table, this.picklist,
			this.regexp_compiled, this.regexp_column);

    query_info := this.query_info;
    query_info.name_list = name_list;
    max_len := (name_list == nil) ? 0 : vector_length (name_list);
    query_info.max_items = max_len;

    for (i := 0;  i < max_len;  ++i)
	if (name_list[i] == nil)
	{
	    query_info.max_items = i;
	    i = max_len;
	}

    query_info.current_frame = 1;

    if (max_len > 0)
    {
	num_frames  := query_info.max_items / query_info.frame_size;
	rnum_frames := round (num_frames);

	if (rnum_frames > num_frames)
	    num_frames = rnum_frames;
	else if (rnum_frames < num_frames)
	{
	    num_frames = rnum_frames;
	    ++num_frames;
	}
	query_info.num_frames = num_frames;
    }
    else
	query_info.num_frames = 100;

    query_info.scope = scope;
    query_info.type_name = type_name;

    this.region_cache =
    ShowNthFrame (scope, type_name, vec_label_structs,
			list (this.label_table, this.picklist, region),
			name_list, query_info.max_items,
			query_info.frame_size, query_info.current_frame,
			this.region_cache);

    main_table := this.main_table;

    if (max_len > query_info.frame_size)
    {
	/*  show the directory panel and
	 */
	vcTable.Set (main_table, 0, 0, this.directory_table);

	/*  update the "scale"'s value to reflect the current frame position
	 */
	this.scale = create_and_add_scale (region, this.directory_table,
					    query_info);
	update_directory_panel (region, query_info.current_frame);
    }
    else
	/*  hide the directory panel
	 */
	vcTable.Set (main_table, 0, 0, this.rubber);

    /*
    vcRegion.Refresh (region, vcRegion.Area (region));
    */
    vcTable.Row (main_table, vcTable.All, vcTable.Border.Invisible, 3, 3, 3,
		 vgAlign.MiddleCenter);
    /*
		 */

    (max_len <= 0) ? FALSE : TRUE;
}


function QueryValueAndUpdateRecords (region, scope, type_name, update_label)
{
    this		:= vcRegion.GetData (region);
    label_structs	:= this.label_structs;

    name_pos		:= -1;
    value_pos		:= -1;
    num_fields		:= this.num_label_structs;

    for (col_num := 0;  col_num < num_fields;  ++col_num)
    {
	label_struct := label_structs [col_num];

	if (string_eq (update_label, label_struct.label))
	    value_pos	= col_num;
	else if (string_eq (type_name, label_struct.label))
	    name_pos	= col_num;
    }

    tab_list := get_picklist_entries (this.picklist);
    if ((name_pos >= 0)  &&  (value_pos >= 0))
	QueryValueAndUpdateRecords_C_func (scope, type_name,
					name_pos, value_pos, tab_list);

    /*
    vcRegion.Refresh (region, vcRegion.Area (region));
    */
}


function DelAllRecords (region)
    remove_picklist_entries (region);


/*  return the selected records from picklist
 *  return-value: list (list (label, value)...)
 */
function GetSelectedRecords (region)
{
    this := vcRegion.GetData (region);
    get_records_in_list (region, get_selected_entries (this.picklist));
}


/*  returns all the records in the picklist
 *  return-value: list (list (label, value)...)
 */
function GetAllRecords (region)
{
    this := vcRegion.GetData (region);
    get_records_in_list (region, get_picklist_entries (this.picklist));
}


vlPref.Replace (prefs, "format", vscmVT, "~~list (list (label, facility..)..)",
	    list (list ("Name",  vcStaticText.fac, nil),
		  list ("Type",  vcStaticText.fac, nil),
		  list ("Value", vcCmdline.fac, 
		list (width <- 12, max_width <- 128, font_id <- vgFont.tiny)))); 
vlPref.Replace (prefs, "header_color", vgColorVT, "header label color",
	    vgColor.Black);
vlPref.Replace (prefs, "header_font",  vgFontVT,  "header label font",
	    vgFont.smallb);
vlPref.Replace (prefs, "field_font",   vgFontVT,  "field label font",
	    vgFont.tiny);
vlPref.Replace (prefs, "arg", vscmVT, "~~action function argument", nil);
vlPref.Replace (prefs, "notify_func", vscmVT, "~~action function", nil);


function reg_scale (vsimlist)
{
    function scale_action_func (value)
    {
	this := vcRegion.GetData (vsimlist);
	query_info := this.query_info;

	frame_num := value;
	if (frame_num < 1)
	    frame_num = 1;

	if (frame_num != query_info.current_frame)
	{
	    /*  show new frame
	     */
	    query_info.current_frame = frame_num;

	    remove_picklist_entries (vsimlist);
	    show_current_frame (vsimlist);
	}
    }

    scale_action_func;
}


function reg_prev (vsimlist)
{
    function prev_action_func (data, event)
    {
	this := vcRegion.GetData (vsimlist);
	query_info := this.query_info;

	/*  get current frame number
	 */
	if (query_info.current_frame > 1)
	    /*  decrement frame number
	     */

	    /*  update directory controls
	     */
	    update_directory_panel (vsimlist, query_info.current_frame-1);
    }

    vcWv.Cmd (fac, prev_action_func);
}


function reg_next (vsimlist)
{
    function next_action_func (data, event)
    {
	this := vcRegion.GetData (vsimlist);
	query_info := this.query_info;

	if (query_info.current_frame < query_info.num_frames)
	    /*  increment frame number
	     */

	    /*  update directory controls
	     */
	    update_directory_panel (vsimlist, query_info.current_frame+1);
    }

    vcWv.Cmd (fac, next_action_func);
}


function GetType (region)
{
    this := vcRegion.GetData (region);
    this.field_name;
}


function register_picklist_action_func (region, picklist, arg)
{
    /*  pre-CORE53 versions only
     *
    function picklist_action_func (entry_data, on_or_off)
	 *  call vsimNav.StoreSSheetSelectionInCutBuffer()
	 *  this will inquire all the current selections and add them
	 *  to the cut-buffer
	 * 
	vsimNav.StoreSSheetSelectionInCutBuffer (arg);

    vcPicklist.Notify (picklist, picklist_action_func);
     */

    function picklist_action_func ()
	/*  call vsimNav.StoreSSheetSelectionInCutBuffer()
	 *  this will inquire all the current selections and add them
	 *  to the cut-buffer
	 */
	vsimNav.StoreSSheetSelectionInCutBuffer (arg);

    vcPicklist.CallOnSelectDone (picklist, picklist_action_func);
}


/*  unselects all the selected entries in the vsimList region
 */
function DeselectAll (region)
{
    this := vcRegion.GetData (region);
    vcPicklist.DeselectAll (this.picklist);
}


/*  creates a list of vsimList entries which hold the diffrent fields
 *  the structure of the region_cache is a list of tables
 *  each table contains "num_label_structs" columns
 *  each field in the table contains a region specified in label_structs[i]
 */
function create_region_cache (label_structs, num_label_structs, frame_size,
			      field_font)
{
    region_cache := nil;

    /*  create "frame_size" entries
     */
    for (i := 0;  i < frame_size;  ++i)
    {
	/*  create a table with "num_label_structs" columns
	 */
	table := vcTable.Create ("t", 1, num_label_structs);

	/*  for each field in "label_structs", create its region  
	 */
	foreach (label_struct;  vscript.iter (label_structs))
	{
	    field_region := vcRegion.Create1 ("fr", label_struct.fac,
						"", label_struct.prefs);

	    vcRegion.Font (field_region, field_font);
	    vcTable.Add (table, field_region);
	}

	region_cache = cons (table, region_cache);
    }

    region_cache;
}


function GetMaxItems (region)
{
    this := vcRegion.GetData (region);
    this.query_info.max_items;
}


function _Create (init_data, region)
{
    this := struct {
	region_cache;
	label_structs;
	num_label_structs;
	vec_label_structs;
	field_name;
	regexp_column;
	regexp_string;
	regexp_compiled;
	query_info;
	rubber;
	directory_table;
	scale;
	prev_button;
	next_button;
	picklist;
	main_table;
	label_table;
	order_list
    };

    this.query_info = struct {
	scope;
	type_name;
	name_list;
	max_items;
	frame_size;
	num_frames;
	current_frame
    };

    this.query_info.max_items     = 0;
    this.query_info.frame_size    = 60;
    this.query_info.current_frame = 1;

    main_table	:= vcTable.Create ("spreadsheet", 3, 1);
    format_list	:= vcRegion.PrefGet (region, "format");
    header_font	:= vcRegion.PrefGet (region, "header_font");
    header_color:= vcRegion.PrefGet (region, "header_color");
    field_font  := vcRegion.PrefGet (region, "field_font");

    /*  create the directory traversing scale-bar, buttons
     */
    directory_table := vcTable.Create ("dt", 1, 3);
    scale := vcRegion.Create ("s", vcScale.fac, convert (vlPositiveIntVT, 1),
			      increment <- 1, min <- 1, max <- 100);
    vcScale.CallOnChange (scale, reg_scale (region));
    vcTable.Add (directory_table, scale);


    prev_button := vcWv.LabelButtonCreate ("pb", "PgUp", nil,
					   reg_prev (region), nil);
    vcTable.Add (directory_table, prev_button);

    next_button := vcWv.LabelButtonCreate ("pb", "PgDown", nil,
					   reg_next (region), nil);
    vcTable.Add (directory_table, next_button);
    vcTable.Add (main_table, directory_table);

    /*  create the label region set for the headers
     *  associate the header format with the internal facil, prefs, labels
     */
    i := 0;
    num_label_structs	:= length (format_list);
    label_table		:= vcTable.Create ("label table", 1, num_label_structs);
    label_structs	:= make_vector (num_label_structs);

    foreach (label_info;  vscript.iter (format_list))
    {
	label_struct := create_label_struct ();
	vector_set (label_structs, i, label_struct);

	label_struct.label	= car (label_info);
	label_struct.visible	= TRUE;
	label_struct.fac	= car (cdr (label_info));
	label_struct.prefs	= car (cdr (cdr (label_info)));
	if (iscons (cdr (cdr (cdr (label_info)))))
	    label_struct.notify_func = car (cdr (cdr (cdr (label_info))));
	else
	    label_struct.notify_func = nil;

	label_struct.value_pos	= -1;
	++i;
    }

    /*  construct the "vector version of label_structs" --
     *  infact, the "label_struct" itself should not be used; its redundant
     */
    vec_label_structs := 
    construct_vector_label_structs (num_label_structs, label_structs);

    /*  create the header (Name, Type, Value) for the spreadsheet
     */
    for (i = 0;  i < num_label_structs;  ++i)
    {
	label_struct := vector_ref (label_structs, i);
	lab := vcRegion.Create (label_struct.label, vcLabel.fac,
			       label_struct.label);
	vcRegion.Font  (lab, header_font);
	vcRegion.Color (lab, header_color);

	vcTable.Add (label_table, lab);
    }
    vcTable.Col (label_table, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.LowerLeft);

    vcTable.Add (main_table, label_table);

    /*  create a Picklist region for displaying the records 
     */
    picklist	:= vcRegion.Create ("pick list", vcPicklist.fac, nil,
				    std_select  <- FALSE,  /* CORE53 & later */
				    rows 	<- 100,
				    columns 	<- 1,
				    macro_on 	<- TRUE,
				    will_scroll <- TRUE,
				    range	<- TRUE,
				    exclusive 	<- FALSE);

    arg := vcRegion.PrefGet (region, "arg");
    register_picklist_action_func (region, picklist, arg);

    picklist_s	:= vcPicklist.Scroll ("pickscroll", picklist, 2, 8, 0);
    vcTable.Add (main_table, picklist_s);

    vcRegion.AddChild (region, main_table);

    field_name			:= label_structs[0].label;
    this.field_name		= field_name;
    this.regexp_column		= 0;
    this.regexp_string		= "*";
    this.regexp_compiled	= CompileRegexp ("*");
    this.main_table		= main_table;
    this.directory_table	= directory_table;
    this.scale			= scale;
    this.rubber			= vcSteel.Create ("ub", 0, 0);
    this.prev_button		= prev_button;
    this.next_button		= next_button;
    this.label_table		= label_table;
    this.picklist		= picklist;
    this.label_structs		= label_structs;
    this.num_label_structs	= num_label_structs;
    this.vec_label_structs	= vec_label_structs;

    /*  create a cache of regions which will be reused in ShowNthFrame
     */
    this.region_cache = create_region_cache (label_structs, num_label_structs,
					     this.query_info.frame_size,
					     field_font);

    this;
}


function _Size (data, size_struct, region, wv_id)
    vgSize.Set (size_struct, 240, 50, 230, 50, 20000, 20000);
