/*  vsimPref.vs -- vsimPref facility definition
 *
 *  $Log: vsimPref.vs,v $
Revision 1.5.2.2  1995/03/03  21:59:37  dahan
Add fcs_vsim531 label

Revision 1.5  1994/08/25  21:15:02  dahan
merged version of 521(cosim+) with 53 baseline

Revision 1.4  1994/08/05  00:23:20  sridhar
bug fix; bug id# 15178, 15151, 15287

Revision 1.3  1994/07/11  15:50:01  sridhar
Changed the layout. Added "Install" button

Revision 1.2  94/07/06  17:05:30  sridhar
added labels to the preference sheet    

Revision 1.1  94/06/13  16:31:12  sridhar
Initial revision

Revision 1.1  94/04/22  14:28:34  sridhar
Initial revision

 */
 

function deep_copy(x) {
    if (x == nil)
        nil;
    else if (istype (x, consVT))
        cons (deep_copy (car (x)), deep_copy (cdr (x)));
    else if (istype (x, vectorVT)) {
        l := vector_length (x);
        v := make_vector (l);
        for (i := 0; i < l; ++i)
            v[i] = deep_copy (x[i]);
        v;
    } /* else if */
    else
        x;
} /* function deep_copy */



function get_col_selection (sel_table)
    if (string_eq (vcRegion.Get (vcTable.Get (sel_table, 0, 1), vscmVT), " "))
	-1;
    else
	string2number (vcRegion.Get (vcTable.Get (sel_table, 0, 1), vscmVT));


function create_col_selection_mechanism (num_visible, selection, mtab, max_cols,
					region)
{
    declare	i;
    declare	str;
    declare	table;
    declare	label;
    declare	menu_list;
    declare	popup_menu;
    declare	pulldown_menu;


    function gen_update_col_sel (num)
    {

	function update_col_sel (d, e)
	{
	    declare	lab;
	    declare	sel;
	    declare	i;

	    this := vcRegion.GetData (region);

	    if (this.change_func != nil)
		(this.change_func)  ("value");

	    for (i = 0;  i < max_cols;  ++i) {
		sel = get_col_selection (vcTable.Get (mtab, 1, i));

		/*  swap labels if they clash,,,
		 */
		if (sel == num) {
		    lab = vcTable.Get (table, 0, 1);
		    vcTable.Set (table, 0, 1,
			vcRegion.Create ("l", vcLabel.fac, number2string(num)));
		    vcTable.Set (vcTable.Get (mtab, 1, i), 0, 1, lab);
		    vcRegion.Refresh (mtab, vcRegion.Area (mtab));
		    return (nil);
		}
	    }
	}
	vcWv.Cmd (fac, update_col_sel);
    }

    str   = istype (selection, string) ? (selection) : number2string(selection);
    if (! istype (selection , string))
	if (selection < 0)
	    str = " ";

    table = vcTable.Create ("cs table", 1, 2);

    if (! string_eq (str, " ")) {
	for (i = num_visible-1;  i >= 0;  --i)
	    menu_list = cons (list (number2string (i), gen_update_col_sel(i)),
			menu_list);
	popup_menu	= vcMenu.Create ("Popup", menu_list);
	pulldown_menu	= vcWv.PullDownMenu ("Pulldown", "Column:", popup_menu);
	vcTable.Set (table, 0, 0, pulldown_menu);
    } else {
	vcTable.Set (table, 0, 0, vcRubber.Create ("r", 0, 0));
    }

    label = vcRegion.Create ("l", vcLabel.fac, str);
    vcTable.Set (table, 0, 1, label);

    vcTable.Col (table, vcTable.All, vcTable.Border.Invisible,
		 1, 1, 1, vgAlign.MiddleCenter);
    vcTable.Row (table, vcTable.All, vcTable.Border.Invisible,
		 1, 1, 1, vgAlign.MiddleCenter);

    table;
}


function change_tables (entry_data, on_or_off)
    if (equal (on_or_off, vlOnOff.On))
    {
	col_tab := vcTable.Get (entry_data[0], 0, 1);
	vcTable.Set (col_tab, 1, 0, entry_data[1]);
    }

function create_toggle_button (this,item_num, col_num, subitem, max_cols, table,
				region)
{

    function change_menu_state (event, status)
    {
	declare		i;
	declare		sel;
	declare		mysel;
	declare		sel_table;

	if (status) {
	    /*  recreate all the menus for col_selection
	     *  with an additional column;
	     */
	    this.item_num_columns[item_num] =
		this.item_num_columns[item_num] + 1;
	    for (i = 0;  i < max_cols;  ++i) {
		sel = get_col_selection (vcTable.Get (table, 1, i));

		if (i == col_num)
		    sel = this.item_num_columns[item_num]-1;

		sel_table = create_col_selection_mechanism (
				this.item_num_columns[item_num], sel,
				table, max_cols, region);
		vcTable.Set (table, 1, i, sel_table);
	    }
	} else {
	    /*  recreate all the menus for col_selection
	     *  with one less menu selection
	     */
	    this.item_num_columns[item_num] =
		this.item_num_columns[item_num] - 1;
	    mysel = get_col_selection (vcTable.Get (table, 1, col_num));

	    for (i = 0;  i < max_cols;  ++i) {
		sel = get_col_selection (vcTable.Get (table, 1, i));
		if (sel >= mysel)
		    sel = sel - 1;
		if (i == col_num)
		    sel = -1;
		sel_table = create_col_selection_mechanism (
			    this.item_num_columns[item_num], sel,
			    table, max_cols, region);
		vcTable.Set (table, 1, i, sel_table);
	    }
	}
    }

    toggle_button := vcRegion.Create ("", vcToggleButton.fac, subitem[1],
				label	<- subitem[0],
				action	<- change_menu_state);
}



function CallOnChange (region, change_func)
{
    this := vcRegion.GetData (region);
    this.change_func = change_func;
}


function _Get (this, region, data_type)
{

    out_list := deep_copy (this.data);
    picklist := this.top_picklist;

    table	:= nil;
    item_num	:= 0;
    i		:= 0;

    foreach (item;  vscript.iter (out_list))
    {
	table = car (cdr (cdr (vcPicklist.Get (picklist, item_num, 0))));

	i = 0;
	foreach (subitem;  vscript.iter (cdr (item))) {
	    if (vcRegion.Get (vcTable.Get (table, 0, i), vscmVT)) {
		subitem [1] = TRUE;
		subitem [2] = get_col_selection (vcTable.Get (table, 1, i));
	    } else {
		subitem [1] = FALSE;
		subitem [2] = -1;
	    }
	    ++i;
	}
	++item_num;
    }

    convert (data_type, out_list);
}


function _Set (this, in_item_list)
{
    item_list := deep_copy (in_item_list);
    top_picklist := this.top_picklist;
    row := 0;
    col := 0;

    /*  foreach "Spreadsheet"
     */
    foreach (item;  vscript.iter (item_list))
    {
	table := cdr (vcPicklist.Get (top_picklist, row, col));
	table = table[1];

	/*  update the column ordering in the table corr. to the sheet
	 */
	col_num := 0;
	foreach (subitem;  vscript.iter (cdr (item)))
	{

	    /*  change the column number of each field
	     */
	    col_sel_table := vcTable.Get (table, 1, col_num);

	    selection := subitem[2];

	    str := istype (selection, string) ? (selection) :
		   number2string (selection);
	    if (! istype (selection , string))
		if (selection < 0)
		    str = " ";

	    label := vcRegion.Create ("l", vcLabel.fac, str);
	    vcTable.Set (col_sel_table, 0, 1, label);

	    ++col_num;
	}

	++row;
    }
}



function _Create (init_data, region)
{
    declare	i;

    this := struct {
	main_table;
	top_picklist;
	change_func;
	data;
	table;
	item_num_columns
    };

    item_list   := deep_copy (init_data);
    this.data	= item_list;

    main_table		:= vcTable.Create ("Pref Sheet", 1, 1);
    vcTable.Row (main_table, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleCenter);
    vcTable.Col (main_table, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleCenter);

    sub_main_table	:= vcTable.Create ("Pref Sheet", 1, 2);
    vcTable.Row (sub_main_table, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleCenter);
    vcTable.Col (sub_main_table, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleCenter);


    /*  create the Top picklist
     */
    top_picklist	:= vcRegion.Create ("Top Pick", vcPicklist.fac, nil,
					rows		<- length (item_list), 
					columns		<- 1,
					macro_on	<- TRUE,
					exclusive	<- TRUE);

    this.item_num_columns = make_vector (length (item_list));
    this.table		  = make_vector (length (item_list));

    item_num		:= 0;
    col_num		:= 0;
    num_visible		:= 0;
    table		:= nil;
    toggle_button	:= nil;
    col_sel_table	:= nil;

    foreach (item;  vscript.iter (item_list)) {
	
	/*  create a table for each item
	 */
	table = vcTable.Create (car (item) + " table", 2, length(item)-1);
	vcTable.Row (table, vcTable.All, vcTable.Border.Invisible, 10, 10, 10,
		     vgAlign.MiddleCenter);
	vcTable.Col (table, vcTable.All, vcTable.Border.Invisible, 10, 10, 10,
		     vgAlign.MiddleCenter);

	/*  get the number of _visible_ subitems
	 */
	num_visible = 0;
	foreach (subitem;  vscript.iter (cdr (item)))
	    if (equal (subitem[1], TRUE))
		++num_visible;
	this.item_num_columns [item_num] = num_visible;

	/*  create a bunch of toggle buttons for the 0th row of "table"
	 *  associate them with a column selection mechanism in 1st row.
	 */
	col_num = 0;
	foreach (subitem;  vscript.iter (cdr (item)))
	{
	    col_sel_table = create_col_selection_mechanism (num_visible,
						subitem[2], table,
						length(item)-1, region);
	    vcTable.Set (table, 1, col_num, col_sel_table);

	    /*
	    tog_button := create_toggle_button (this, item_num, col_num,subitem,
				  length(item)-1, table, region);
				  */
	    tog_button := vcRegion.Create ("", vcLabel.fac, subitem[0]);
	    vcTable.Set (table, 0, col_num, tog_button);

	    ++col_num;
	}

	/*  add a picklist item to the top_picklist
	 *  a selection in top_picklist will update the pref sheet to its
	 *  corr. table
	 */
	this.table[item_num] = table;
	vcPicklist.Add (top_picklist, car (item), list (sub_main_table, table));
	++item_num;
    }

    outer_tab := vcTable.Create ("picklist table", 2, 1);
    ss_lab := vcRegion.Create ("ss", vcLabel.fac, "Spreadsheets: ");
    vcRegion.Font (ss_lab, vgFont.mediumb);
    vcTable.Add (outer_tab, ss_lab);

    vcTable.Add (outer_tab, top_picklist);
    b := vcRegion.Create ("c", vcBorder.fac, outer_tab, rounded <- FALSE);
    vcTable.Add (sub_main_table, b);

    col_table := vcTable.Create ("col_table", 2, 1);
    vcTable.Row (col_table, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleLeft);
    vcTable.Col (col_table, vcTable.All, vcTable.Border.Invisible, 1, 1, 1,
		 vgAlign.MiddleLeft);

    col_lab := vcRegion.Create ("co", vcLabel.fac, "Column Ordering: ");
    vcRegion.Font (col_lab, vgFont.mediumb);
    vcTable.Add (col_table, col_lab);
    
    b = vcRegion.Create ("d", vcBorder.fac, col_table, rounded <- FALSE);
    vcTable.Add (sub_main_table, b);

    vcTable.Add (main_table, sub_main_table);

    /*  associate children to region
     */
    vcRegion.AddChild (region, main_table);


    this.main_table = main_table;
    this.top_picklist = top_picklist;

    vcPicklist.Notify  (top_picklist, change_tables);
    vcPicklist.SetPick (top_picklist, 0);

    this;
}

