/*
 *  $Revision: 1.12.4.1 $
 *
 *  Implements the 'Output' picklist and associated command line
 *
 *  Copyright (c) 1991,1993 by Viewlogic Systems, Inc.
 */

package vspport (create_out_list, update_output_cmdline,
                setup_valid_ports, close_port_win, get_cmdline_val) {

declare cport_win_id;
declare valid_ports;
declare out_list;
declare vspCportReg;
declare output_cmdline;
declare cur_output;
declare cur_com_port;
declare save_output;     /* Just in case port is selected & user cancels out of
                                popup- want to restore old output */

/* Valid com. ports for each machine that we support.  Ports may be added
 * to existing lists.  However, if a machine is added, some code must be
 * altered as well.
 */
sun_ports := list ("/dev/ttya", "/dev/ttyb");
dec_ports := list ("/dev/tty00", "/dev/tty01");
hp9k_ports := list ("/dev/tty00", "/dev/tty01");
ibm_ports := list ("/dev/lp0", "/dev/rp0");
pc_ports := list ("prn", "lpt1", "lpt2");

/* These numbers must correspond with the order in which they are setup
 * in the output picklist.
 */
vspPRT_FILE := 0;
vspPRT_SPOOL := 1;
vspPRT_PORT := 2;

vspSTR_FILE := "FILE";
vspSTR_PORT := "PORT";
vspSTR_SPOOL := "SPOOL";

/* This is called when the user hits the apply button in the main
 * plotting worksheet.  It is required just in case the values have
 * changed and the user neglected to enter a <cr>.
 */
function get_cmdline_val ()
    get_output_cmdline_val (nil, vcRegion.Get (output_cmdline, vlStringVT));


/* This function ties the "Output" picklist (ie File, Spool, Port) to the
 * command line.  Whenever some different entry is selected, the command
 * line is updated with the last value with which that particular item was
 * associated.  This value could be either a default or something that the
 * user set at one point during the session.  Then the actual value itself
 * is updated in the `C` world.
 */
function set_output (which, on_off) {

        if (on_off == vlOnOff.Off)
                return (nil);

        save_output = cur_output;
        cur_output = get_out_index (which);

        if (string_eqi (which, vspSTR_FILE)) {

                /* Do nothing if already set to file */
                if (string_eqi(fac_pack.get_def_output_type(), vspSTR_FILE))
                        return (nil); 

                /* update the output type to 'file' */
                fac_pack.sp_sapopt (string_append (vspSTR_PORT, " ", 
                        vspSTR_FILE));

                /* Get the default file name */
                file_name := fac_pack.get_cur_file ();

                /* stuff command line editor w/ cur_file */
                vcCmdline.Set (output_cmdline, file_name);
        
                /* Update the software with the default file name */
                fac_pack.sp_sapopt (string_append (vspSTR_FILE, " ", 
                        file_name));
        }
        else if (string_eqi (which, vspSTR_SPOOL)) {

                /* Do nothing if already set to Spool */
                if (string_eqi(fac_pack.get_def_output_type(),"Spool"))
                        return (nil); 

                /* stuff command line editor w/ cur_spool */
                new_val := fac_pack.get_cur_spool (valid_ports);
                vcCmdline.Set (output_cmdline, new_val);
                fac_pack.sp_sapopt (string_append (vspSTR_PORT, " ", new_val));
        }
        else {
                /* popup a menu of com. ports for this machine */
                popup_cport_menu ();
        }
}

outputs := list (0, `vertical, "OUTPUT",
                list (cons(vspSTR_FILE, vspSTR_FILE),
                        cons(vspSTR_SPOOL, vspSTR_SPOOL),
                        cons(string_append(vspSTR_PORT, "..."), vspSTR_PORT)),
                set_output);

/* Stuff the output command line with the new com. port and set the com.
 * port in the software.
 */
function set_com_port (val, on_off)
    if (on_off == vlOnOff.Off)
        nil;
    else
        cur_com_port = val;


/* Initial list for port popup picklist- contains everything except for
 * final array.
 */
port_popup := list (0, `vertical, "COM. PORTS", 
                        set_com_port);

/* Returns a space-separated string of com. ports which are valid for
 * the particular machine on which we are running.  The default is Sun.
 */
function setup_valid_ports () {

        if (fac_pack.is_dec ())
                valid_ports = dec_ports;

        else if (fac_pack.is_ibm ())
                valid_ports = ibm_ports;

        else if (fac_pack.is_pc ())
                valid_ports = pc_ports;

        else if (fac_pack.is_hp9k ())
                valid_ports = hp9k_ports;

        else
                valid_ports = sun_ports;
}

/* Returns the output index/type depending on the current string
 * which represents the file name, printer name, or port name.
 */
function get_output_type (str)
    if (string_eqi (str, vspSTR_FILE))
        vspPRT_FILE;
    else if (fac_pack.vsp_gq_spl_dev (str))
        vspPRT_SPOOL;
    else
        vspPRT_PORT;


/* Returns the index into the output selection picklist, depending 
 * on the choice.  This is useful to highlight the proper entry upon
 * invocation of the software.
 */
function get_out_index (choice)
    if (string_eqi (choice, vspSTR_FILE))
        vspPRT_FILE;
    else if (string_eqi (choice, vspSTR_SPOOL))
        vspPRT_SPOOL;
    else
        vspPRT_PORT;


/* Returns the index into the port selection picklist, depending 
 * on the choice.  This is useful to highlight the proper entry upon
 * invocation of the software.
 */
function get_port_index (choice) {

        for (i := 0, port := valid_ports; port != nil; port = cdr(port), ++i) {
                if (string_eqi (choice, car(port)))
                        return (i);
        }
        nil;
}

/* Converts an index into the comport picklist to a string

function conv_index2cport_name (index) {

        for (i := 0, port:=valid_ports; (i < index) && (port != nil); 
                port = cdr(port), ++i);

        if (port)
                car(port);

        nil;
}
 */

/* Called when the user wishes to accept the current port choice which could
 * be either the default or anyother selection.
 */
function accept_port () {

        /* Make sure the command line reflects the choice */
        vcCmdline.Set (output_cmdline, cur_com_port);

        /* Set it in the software */
        fac_pack.sp_sapopt (string_append(vspSTR_PORT, " ", cur_com_port));

        /* popdown the window */
        cport_win_id = window_close (cport_win_id);

}

/* Called when user wishes to get rid of the port popup without choosing anything
 */
function cancel_port () {
        /* popdown the window */
        cport_win_id = window_close (cport_win_id);

        /* revert to old output */
        vcPicklist.SetPick (out_list, save_output);
}

function port_popup_killed (wid)
    if (wid == cport_win_id)
        cport_win_id = nil;


/* Pops up a picklist of all the com. ports which are possible on that 
 * particular machine.  When an entry is selected, the menu is popped down.
 */
function popup_cport_menu () {

        declare tcons;

        /* Create the type of list that the picklist facility expects */
        for (tlst := valid_ports; tlst != nil; tlst = cdr(tlst)) {
                tcons = cons(car(tlst), car(tlst));
                if (tlst == valid_ports)
                        cport_pklst := cons(tcons, nil);
                else
                        cport_pklst = cons(tcons, cport_pklst);
        }

        cport_pklst = reverse (cport_pklst);

        vspCportReg = create_pklst (func () {
                cur_com_port = fac_pack.get_cur_port(valid_ports);
                get_port_index(cur_com_port);
                },
                cadr(port_popup), caddr(port_popup), 
                cport_pklst, cadddr(port_popup), FALSE);

        ttab := vcTable.Create ("port.tab", 2, 1);
        vcTable.Add (ttab, vspCportReg);
        ttab2 := vcTable.Create ("port_app_can.tab", 1, 2);
        add_but (ttab2, vspACCEPT, 0, 0, accept_port);
        add_but (ttab2, vspCANCEL, 0, 1, cancel_port);
        vcTable.Add (ttab, ttab2);

        /* Create a window to display the com. port picklist */
        cport_win_id = vcWin.Popup (plot_win_id, ttab, out_list, 7);

        /* Register window close routine incase of op. sys. close */
        vcWin.DestroyNotify (cport_win_id, port_popup_killed);

        /* Forward all events to the current popup */
        vcWin.Forward (plot_win_id, cport_win_id);
}

/* Called when the 'Output' command line has been edited by the user and
 * a carriage return has been hit.
 */
function get_output_cmdline_val (data, str) {

        ret := FALSE;

        which := get_nth_val_from_lst (cadddr(outputs), cur_output);

        /* Set output value according to both the str. and menu selection */
        if (string_eqi (which, vspSTR_FILE))
            ret = fac_pack.sp_sapopt (string_append (vspSTR_FILE, 
                                                " ", str));

        else if (string_eqi (which, vspSTR_SPOOL)) {
            if (!fac_pack.vsp_gq_spl_dev (str)) {
                vcCmdline.Set (output_cmdline,  
                               fac_pack.get_def_output_name ());
                vlEx.Throw2 (vlEx.FAILURE, "vspport", vlErrSev.Error,
                             sprintf ("Not a spoolable device: %s", str));
                }
            else
                ret = fac_pack.sp_sapopt (string_append (vspSTR_PORT,
                                " ", str));
        }
        else if (fexists (str))
            ret = fac_pack.sp_sapopt (string_append (vspSTR_PORT, 
                                            " ", str));
        else {
            vcCmdline.Set (output_cmdline, 
                    fac_pack.get_def_output_name ());
            vlEx.Throw2 (vlEx.FAILURE, "vspport", vlErrSev.Error,
                         sprintf ("Bad output device: %s", str));
        }

        if (ret)
                /* Make sure that the string is echoed back to command line */
                vcCmdline.Set (output_cmdline, str);
        else
                /* reset to last value */
                vcCmdline.Set (output_cmdline, 
                               fac_pack.get_def_output_name ());
}

/*  Only updates the output command line if "File" is selected in the
 *  output menu.  This is called to keep the output command line and
 *  the "OUTDIR" command line in sync.  It appends the plot file name
 *  to 'new_dir'.
 */
function update_output_cmdline (new_dir) {

        which := get_nth_val_from_lst(cadddr(outputs), cur_output);

        /* Update command line only if mode is "File" */
        if (string_eqi (which, "File")) {
                leaf := fac_pack.vspGetLeafName(
                        fac_pack.get_def_output_name());
                path := fac_pack.vspFormPath (new_dir, leaf);

                /* Make sure that the string is echoed back to command line */
                vcCmdline.Set (output_cmdline, path);
        }
}

/* Create a command line to type in possible plot outputs
 */
function create_output_cmdline () {

        output_cmdline_list := create_cmdline("Output> ", 
                        fac_pack.get_def_output_name(), 
                        get_output_cmdline_val, FALSE, 12, 15);
        output_cmdline = car (output_cmdline_list);
        cadr (output_cmdline_list);
}

/* Create a picklist for the possible Outputs. */
function create_out_list () {

        cur_output = get_output_type(fac_pack.get_def_output_type ());

        out_list = create_pklst (func ()
                           get_output_type(fac_pack.get_def_output_type ());,
                        cadr(outputs), caddr(outputs), cadddr(outputs),
                        cadr(cdddr(outputs)), FALSE);

        ttab := vcTable.Create ("outlist.tab", 2, 1);
        vcTable.Add (ttab, out_list);
        vcTable.Add (ttab, create_output_cmdline ());

        vcRegion.Create1("outlist.border", vcBorder.fac, ttab, nil);
}

/* Called when the plot worksheet has been closed.  If the port popup is
 * showing, then get rid of it.
 */
function close_port_win ()
    if (cport_win_id != nil)
        vcWin.Close (cport_win_id);


} /* vspport package */

