
/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   vsyn_itc.vs       viewsynthesis 5.0           06 January 1995           */
/*                     Copyrite January 1995, Viewlogic Systems, Inc.        */
/*                     Phil Labee                                            */
/*                                                                           */
/*   This module allows viewsynthesis to set up and respond to ITC messages  */
/*   for communicating with viewFSM, core, hac, and other executables.  It   */
/*   contains these functions:                                               */
/*                                                                           */
/*       put_err             throws exceptions if ITC cannot be initialized  */
/*       get_strng           allows the icons in the GUI to respond to HAC   */
/*       VfsmSetTech_func    allows viewFSM to set technology in viewsynth   */
/*       VfsmSetAnal_func    allows viewFSM to perform VHDL analysis         */
/*       execVfsmCmd_func    allows viewFSM to send synth/wire/report cmds.  */
/*       VsynStarted_func    viewsynth announces that it's now running       */
/*       VsynGUIStop_func    viewsynthe announces that is has stopped        */
/*       VsynGUIPong_func    standard ITC pong message                       */
/*       VsynDestroy_func    viewsynth has been terminated                   */
/*                                                                           */
/*---------------------------------------------------------------------------*/ 

 
 
declare vsynCNTX,                /* ViewSynth vsynmain.vs ITC source context */
        vfsmCNTX;                /* ViewFSM vsynmain.vs ITC (match) context  */
        vfsmIsRunning := FALSE;  /* Do we believe ViewFSM is now running?    */
 
 

/*---------------------------------------------------------------------------*/
/*   PUT_ERR will throw an exeption.  It is here to document the possible    */
/*   exceptions, in anticipation of handling exceptions systematically for   */
/*   the whole tool.                                                         */
/*---------------------------------------------------------------------------*/ 

function put_err(ii)
    {
    FAL := vlEx.FAILURE;    app := "ViewSyn";  wrn := vlErrSev.Warning;
 
    if (ii== 1) vlExThrow2(FAL,app,wrn,"Failed to create ITC source context");
    if (ii== 2) vlExThrow2(FAL,app,wrn,"Failed to create viewFSM ITC context");
    if (ii== 3) vlExThrow2(FAL,app,wrn,"Failed to create vsyn ITC context");
    if (ii== 4) vlExThrow2(FAL,app,wrn,"Cannot determine ITC user id");
    if (ii== 5) vlExThrow2(FAL,app,wrn,"Cannot determine ITC group id");
    if (ii== 6) vlExThrow2(FAL,app,wrn,"Cannot determine ITC frame id");
    if (ii== 7) vlExThrow2(FAL,app,wrn,"Cannot determine ITC host id");
    if (ii== 8) vlExThrow2(FAL,app,wrn,"Cannot determine ITC process id");
    vlExThrow2(vlEx.INTERNAL, app, wrn,"Product bug !");
 
    }                                                /* end function PUT_ERR */
 




/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   GET_STRNG  function is registered to be called upon receipt of an ITC   */
/*   message from HAC.  Hac compiles, optimizes, and maps the design, and    */
/*   transmits messages back to the viewsynth GUI to follow its progress.    */
/*---------------------------------------------------------------------------*/ 

function get_strng (mytype, scntx, mcntx, data) {

declare token,icon, opt_failed;

   data := string_token( data, &token, " " );
   entity := string_copy( token ); 

   data := string_token( data, &token, " " );
   operation := string_copy( token ); 

   data := string_token( data, &token, " " );
   flag := convert( integer, token ); 
   
   if (string_eq( operation, "optimizing" ) == TRUE) {
      icon := string_copy( "compwork" );
      update_1_icon( entity, icon, flag );
   }
   else if (string_eq( operation, "mapping" ) == TRUE) {
      icon := string_copy( "optwork" );
      update_1_icon( entity, icon, flag );
   }
   else if (string_eq( operation, "optimized" ) == TRUE) {
      icon := string_copy( "opt" );
      update_1_icon( entity, icon, flag );
   }
   else if (string_eq( operation, "mapped" ) == TRUE) {
      icon := string_copy( "syn" );
      update_1_icon( entity, icon, flag );
   }
   else {
      if (amgdebug == 1) printf( "\n\n**ERROR: operation = %s, flag = %s, entity = %s\n", operation, flag, entity );
      /* An error occurred */
      sflag := show_hierctl( flag ); 
      if ((sflag == 1) || (sflag == 2)) {
          /* dissolved, so use dissolved icon */
          sflag = show_icon_type( flag );
          if (sflag == 32)
             newicon := "compdis";
          else if (sflag == 64)
             newicon := "opt";
      }
      else { 
          /* not dissolved, so use normal icon */
          sflag = show_icon_type( flag );
          if (amgdebug == 1) printf( "'show_icon_type(%d)' = %d\n", flag, sflag );
          if (sflag == 128)
             newicon := "vhdlsrc";
          else if (sflag == 64)
             newicon := "vhdlcomp";
          else if (sflag == 32)
             newicon := "opt";
          else
             newicon := "syn";
      }
      if (amgdebug == 1)  printf( "newicon = '%s'\n", newicon );
      update_1_icon( entity, newicon, flag );
      if (amgdebug == 1) printf( "operation = %s, flag = %s, entity = %s\n", operation, flag, entity );
   }
   if (flag == -1) {
       tmpstr := sprintf("%s entity '%s'...\n", operation, entity );
       vcWin.ClearStatus( vdes2_win );
       vcWin.AddStatus(vdes2_win, tmpstr );
   }
}



/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   VfsmSetTech_func  function will set technology per request by viewFSM.  */
/*                                                                           */
/*---------------------------------------------------------------------------*/

function VfsmSetTech_func (msg_type, src_cntx, mtch_cntx, tech)
    {
    if (vfsmIsRunning)
       {
       pre_tech();
       vcShell.TypeIn (vdes_shell,"tech " + tech, TRUE);
       check_packfile( func() { post_tech( tech ); });
       }
    }

/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   VfsmSetAnal_func  function will preform VHDL analysis per viewFSM.      */
/*                                                                           */
/*---------------------------------------------------------------------------*/

function VfsmSetAnal_func (msg_type, src_cntx, mtch_cntx, data)
    {
    current_design = data;
    vcShell.TypeIn(vdes_shell, "vhdl " + data, TRUE);
    enable_syn();
    vcCmd.Enable (constraint_cmd, vlOnOff.On );
    vcCmd.Enable (wire_cmd,       vlOnOff.On);
    vcCmd.Enable (report_cmd,     vlOnOff.On);
    vcWin.UpdateMenus (vdes2_win);
    }


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   execVfsmCmd_func  function will process synthesis, wire, and report     */
/*   commands issues from viewFSM via ITC calls.                             */
/*                                                                           */
/*---------------------------------------------------------------------------*/

function execVfsmCmd_func (msg_type, src_cntx, mtch_cntx, data) 
    {
    if (vfsmIsRunning) vcShell.TypeIn(vdes_shell, data, TRUE);
    }


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   VsynStarted_func  function executes when the tool starts and ITC is up. */
/*                                                                           */
/*---------------------------------------------------------------------------*/ 

function VsynStarted_func (dtype, src, match, data)
    {
    vitc.Send ("Pong", vsynCNTX, vfsmCNTX, "pong", vipc.FlgDIRECT);
    vfsmIsRunning = TRUE; 
    }


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   VsynGUIStop_func  function executes when the tool is stopped by anyone. */
/*                                                                           */
/*---------------------------------------------------------------------------*/

function VsynGUIStop_func (dtype, src, match, data) 
    {
    vfsmIsRunning = FALSE;
    }


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   VsynGUIPong_func  function is a standard ITC pong-message function.     */
/*                                                                           */
/*---------------------------------------------------------------------------*/

function VsynGUIPong_func (dtype, src, match, data)
    {
    vfsmIsRunning = TRUE;
    }







/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   VsynDestroy_func  function is a standard ITC tool stop function.        */
/*                                                                           */
/*---------------------------------------------------------------------------*/

function VsynDestroy_func ()
    {
    if (vfsmIsRunning)
       {
       vitc.Send ("ToolStop", vsynCNTX, vfsmCNTX, " ", vipc.FlgDIRECT);
       vitc.SrcCntxDestroy (vsynCNTX);
       }
    }
 


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   SETUP_ITC  function is called from <vsynmain.vs> to perform ITC startup.*/
/*                                                                           */
/*---------------------------------------------------------------------------*/

function setup_itc() 
    {
    vsynid      := "Viewsynth GUI";
    vfsmid      := "ViewFSM";
    hackid      := "Viewsynth HAC";
    msgid_reply :=  "hac_reply";

    /* msgid_req   = "hac_cmd"; (01/06/95 commented out; not used. plabee)  */

    vsynCNTX  = vitc.SrcCntxCreate(vsynid, nil);

    if (vsynCNTX == vipc.FAILURE)   put_err(1);
 
    if ( (user   := vipc.CntxGetUid(vsynCNTX)) == vipc.FAILURE)  put_err(4);
    if ( (ugroup := vipc.CntxGetGid(vsynCNTX)) == vipc.FAILURE)  put_err(5);
    if ( (uframe := vipc.CntxGetFid(vsynCNTX)) == vipc.FAILURE)  put_err(6);
    if ( (uhost  := vipc.CntxGetHnm(vsynCNTX)) == vipc.FAILURE)  put_err(7);
    if ( (upid   := vipc.CntxGetPid(vsynCNTX)) == vipc.FAILURE)  put_err(8);
 
    vfsmCNTX  = vitc.CntxCreate(ugroup, user, vipc.FidANY, vfsmid, nil, uhost,
                                vipc.PidANY, vipc.CidANY);
    if (vfsmCNTX == vipc.FAILURE) put_err(2);

    hackCNTX  := vitc.CntxCreate(ugroup, user, vipc.FidANY, hackid, nil, uhost,
                                 vipc.PidANY, vipc.CidANY);
    if (vfsmCNTX == vipc.FAILURE) put_err(3);

    vitc.Register( "vfsmToVsynthTech", vsynCNTX, vfsmCNTX, VfsmSetTech_func);
    vitc.Register( "vfsmToVsynthAnal", vsynCNTX, vfsmCNTX, VfsmSetAnal_func);
    vitc.Register( "vfsmToVsynth",     vsynCNTX, vfsmCNTX, execVfsmCmd_func);
    vitc.Register( "ToolStarted",      vsynCNTX, vfsmCNTX, VsynStarted_func);
    vitc.Register( "ToolStop",         vsynCNTX, vfsmCNTX, VsynGUIStop_func);
    vitc.Register( "Pong",             vsynCNTX, vfsmCNTX, VsynGUIPong_func);
    vitc.Register( msgid_reply,        vsynCNTX, hackCNTX, get_strng);
    vitc.ToolStartNotify(vsynCNTX);
    }


vcQuit.CallOnQuit( VsynDestroy_func );
