/* -----------------------------------------------------------------------*\
/* Module -- vtimemcp.vs
 *
 *      Facility:      vtimemcp.fac
 *
 *      Environment:   SUN4, Powerview 5.2
 *
 *      Authors:       Jeff Hare    June 8, 1994
 *
 *      Description:   ViewTime MCP generator
 *       
 *
 *      Copyright (c) 1993 by Viewlogic Systems, Inc.
 *
\* ---------------------------------------------------------------------- */
/*\Section vtimemcp -- ViewTime MCP Generator
*/

function label(str) vcRegion.Create1(str,vcLabel.fac,str,nil);

vcWv.MakeEnum("McpVT",list("Constant","Cycle"));
vcWv.MakeEnum("EdgeVT",list("Rising","Falling"));

/*===========================================================================*/
/*==                  M a k e E v e n t                                    ==*/
/*===========================================================================*/
function MakeEvent(r)
{
   w := vcRegion.Window (r);
   e := vcEvent.Create (0, 0, ' ', "", w, r, nil);
   e;
}
 

/*===========================================================================*/
/*==                       _ C r e a t e                                   ==*/
/*===========================================================================*/
/*\Function _Create  -- Create a mcp definition editor facility
  
  DESCRIPTION
       
  RETURNS


  EXAMPLE
       
  CAUTION

  SEE ALSO
       
*/
function _Create(InitData, TopRegion)
{
   Data   := car(InitData);
   vtdata := cdr(InitData);


   Data = reverse_destructive(Data);
   /*--------------------------------------------------*\
    *  The following region declarations are global to
    *  all the functions defined in _Create()
   \*--------------------------------------------------*/
   declare SetupHoldRgn;    /* Setup/Hold Cycles Table      */
   declare HoldCycleRgn;    /* Hold Cycles Scalebar region  */
   declare SetupCycleRgn;   /* Setup Cycles Scalebar region */
   declare VtxData;
   

   /*====================================================*/
   /*==             C r e a t e S e t t i n g s         =*/
   /*====================================================*/
   function CreateSettings(Data) {
      nd := "";
      cpart := FindTargetPart(Data);
      cpin  := FindTargetPin(Data);
      declare part;
      declare ipin;
      declare opin;
      declare v;
      declare node;

      foreach (v; vscript.list_iter(Data)) {
         node  = vcTree.VrtxGetItem(v);
         part  = vtime.GetPart(node);
         ipin  = vtime.GetIpin(node);
         opin  = vtime.GetOpin(node);

         if (string_eq(cpart,part) && string_eq(cpin,ipin))
            nd = sprintf("%s\tTO %s %s\n",nd,part,ipin);
         else
            nd = sprintf("%s\tTHRU %s %s\n",nd,part,opin);
      }

      struct {
         vlst         = Data;
         McpType      = Mcp.Constant;
         SetupCycles  = 1;
         HoldCycles   = 0;
         UseDataEdge  = vlTrueFalse.True;
         DataEdge     = Edge.Rising;
         UseClockEdge = vlTrueFalse.True;
         ClockEdge    = Edge.Rising;
         UseClockPin  = vlTrueFalse.True;
         ClockPin     = FindClockPin(Data);
         Nodes        = nd
      };
   }
   
   
   /*====================================================*/
   /*==          F i n d C l o c k P i n                =*/
   /*====================================================*/
   function FindClockPin(Data) {
      ClkPin := "Unknown";
      if (islist(Data)) {
         node := vcTree.VrtxGetItem(car(Data));
         c := node.Constraint;
         ClkPin = car(c.ClkPin);
      }
      
      ClkPin;
   }

   /*====================================================*/
   /*==          F i n d T a r g e t P a r t            =*/
   /*====================================================*/
   function FindTargetPart(Data) {
      P := "Unknown";
      if (islist(Data)) {
         node := vcTree.VrtxGetItem(car(Data));
         c := node.Constraint;
         P = car(c.ConPart);
      }
      P;
   }

   /*====================================================*/
   /*==          F i n d T a r g e t P i n              =*/
   /*====================================================*/
   function FindTargetPin(Data) {
      P := "Unknown";
      if (islist(Data)) {
         node := vcTree.VrtxGetItem(car(Data));
         c := node.Constraint;
         P = car(c.ConPin);
      }
      P;
   }

   /*====================================================*/
   /*==             B u i l d M c p G u i               =*/
   /*====================================================*/
   function BuildMcpGui() {  
      
      declare t;
      
      
      function SetMcpType(d,onoff) {
         if (onoff == vlOnOff.On) 
            if (car(d) == Mcp.Constant) {
               vcRegion.Enable(caddr(d),vlOnOff.Off,vlTrueFalse.True);
               S.McpType = Mcp.Constant;
            } 
            else {
               S.McpType = Mcp.Cycle;
               vcRegion.Enable(caddr(d),vlOnOff.On,vlTrueFalse.True);
            }
         RedoMcp();
      } /* function SetMcpType */


      function UseDataEdge(d,onoff) {
         vcRegion.Enable(d,onoff,vlTrueFalse.True);
         if (onoff == vlOnOff.On) 
            S.UseDataEdge = vlTrueFalse.True;
         else
            S.UseDataEdge = vlTrueFalse.False;
         RedoMcp();
      } /* function SetMcpType */


      function UseClkEdge(d,onoff) {
         vcRegion.Enable(d,onoff,vlTrueFalse.True);
         if (onoff == vlOnOff.On) 
            S.UseClockEdge = vlTrueFalse.True;
         else
            S.UseClockEdge = vlTrueFalse.False;
         RedoMcp();
      } /* function UseClkEdge */



      function UseClkPin(d,onoff) {
         vcRegion.Enable(d,onoff,vlTrueFalse.True);
         if (onoff == vlOnOff.On) 
            S.UseClockPin = vlTrueFalse.True;
         else
            S.UseClockPin = vlTrueFalse.False;
         RedoMcp();
      } /* function UseClkPin */



      function SetClkPin(s) {
         S.ClockPin = s;
         RedoMcp();
      } /* function SetClkPin */
      
      
      function SetClkEdge(d,onoff) {
         if (onoff == vlOnOff.On) {
            if (car(d) == Edge.Rising) 
               S.ClockEdge = Edge.Rising;
            else 
               S.ClockEdge = Edge.Falling;
            RedoMcp();
         }
      } /* function SetClkEdge */
      
      
      function SetDataEdge(d,onoff) {
         if (onoff == vlOnOff.On) {
            if (car(d) == Edge.Rising) 
               S.DataEdge = Edge.Rising;
            else 
               S.DataEdge = Edge.Falling;
            RedoMcp();
         }   
      } /* function SetDataEdge */
         

      function SetHoldCycles(v) {
         val := string2number(v);
         if (val != S.HoldCycles) {
            S.HoldCycles = val;
            RedoMcp();
         }
      } /* function SetHoldCycles */


      function SetSetupCycles(v) {
         val := string2number(v);
         if (val != S.SetupCycles) {
            S.SetupCycles = val;
            RedoMcp();
         }
      } /* function SetSetupCycles */


      function RedoMcp() {

         if (S.UseDataEdge)
            dataedge := S.DataEdge == Edge.Rising ? "RISE" : "FALL";
         else
            dataedge := "";
         
         if (S.UseClockEdge)
            clockedge := S.ClockEdge == Edge.Rising ? "CLK_EDGE R" : "CLK_EDGE F";
         else
            clockedge := "";
         
         if (S.UseClockPin && length(S.ClockPin) > 0)
            clockpin := "CLK_PIN " + S.ClockPin;
         else
            clockpin := "";
         
         if (S.McpType == Mcp.Cycle) {

            setup := sprintf("SETUP %s",S.SetupCycles);
            hold  := sprintf("HOLD %s",S.HoldCycles);

            /* MCP RISE SETUP 1 HOLD 0 CLK_PIN CK CLK_EDGE R */
            mcp := sprintf("MCP %s %s %s %s %s\n{\n%s}\n",
                           dataedge,
                           setup,
                           hold,
                           clockpin,
                           clockedge,
                           S.Nodes);
            
            vtx.ClearSelection(VtxData);
            vtx.BeginningOfBuffer(VtxData);
            vtx.AnchorToggle(VtxData);
            vtx.EndOfBuffer(VtxData);
            vtx.Delete(VtxData);
            vtx.InsertString(VtxData,mcp);
            vtx.Update(VtxData);
         }
         else {
            /* MCP RISE CONST CLK_PIN CK CLK_EDGE R */
            mcp := sprintf("MCP %s CONST %s %s\n{\n%s}\n",
                           dataedge,
                           clockpin,
                           clockedge,
                           S.Nodes);

            vtx.ClearSelection(VtxData);
            vtx.BeginningOfBuffer(VtxData);
            vtx.AnchorToggle(VtxData);
            vtx.EndOfBuffer(VtxData);
            vtx.Delete(VtxData);
            vtx.InsertString(VtxData,mcp);
            vtx.Update(VtxData);
         }
      }
         
         function Cancel()  TRUE;
         function Accept() {
            fname := vtime.GetFileInHierarchy (car(vtdata), "mcptemp.mcp");
            vtx.Writeto(VtxData,fname);
            vtq.Add (vtime.GetQueue (car(vtdata)),cdr(vtdata),                  
                     sprintf ("mtv mcp add %s",fname));
         }


      /*------------------------------------------*/
      /*     Set up the MCP Editor Region         */
      /*------------------------------------------*/
      Title := vcRegion.Create("TDBorder", 
                               vcBorder.fac,
                               t = vcRegion.Create("McpTitle",
                                                    vcLabel.fac, 
                                                    "MCP Path Assertion Edit Window"));
      vcRegion.Font(t, vgFont.mediumi);

         
      McpEditRgn  := vcRegion.Create("MCP_Editor_vtx",
                                     vtx.fac,
                                     "mcptemp.mcp",
                                     scrollbar <- vlOnOff.On,
                                     width <- 52,
                                     height <- 10,
                                     scratch <- vlTrueFalse.True);
                                     


      /*------------------------------------------*/
      /*     Set up the MCP Constraint Table      */
      /*------------------------------------------*/


      RISING     := cons("Rising Edge",        cons(Edge.Rising,"Rising Edge"));
      FALLING    := cons("Falling Edge",       cons(Edge.Falling,"Falling Edge"));

      DataRF     := vcRegion.Create("DataRF",
                                    vcPicklist.fac,
                                    list(list(RISING,FALLING)));

      if (S.DataEdge == Edge.Rising) 
         vcPicklist.SetByString(DataRF,car(RISING));
      else
         vcPicklist.SetByString(DataRF,car(FALLING));



      DATA_EDGE  := cons("Include Data Edge",  DataRF);
      DataEdge   := vcRegion.Create("DataEdgeRgn",
                                    vcPicklist.fac,
                                    list(list(DATA_EDGE)),
                                    exclusive <- FALSE);

      if (S.UseDataEdge == vlTrueFalse.True) 
         vcPicklist.SetPick(DataEdge,0);  /* Toggle On */
      else
         vcRegion.Enable(DataRF,vlOnOff.Off,vlTrueFalse.True);
      
      
      DataRgn    := vcTable.Make ("DataRgn", 1,2,   
                                  DataEdge,
                                  DataRF);


      ClkRF      := vcRegion.Create("ClkRF",
                                    vcPicklist.fac,
                                    list(list(RISING,FALLING)));

      if (S.ClockEdge == Edge.Rising) 
         vcPicklist.SetByString(ClkRF,car(RISING));
      else
         vcPicklist.SetByString(ClkRF,car(FALLING));

      CLOCK_EDGE := cons("Include Clock Edge", ClkRF);
      ClkEdge    := vcRegion.Create("ClkEdgeRgn",
                                     vcPicklist.fac,
                                     list(list(CLOCK_EDGE)),
                                    exclusive <- FALSE);

      if (S.UseClockEdge == vlTrueFalse.True) 
         vcPicklist.SetPick(ClkEdge,0); /* Toggle On */
      else
         vcRegion.Enable(ClkRF,vlOnOff.Off,vlTrueFalse.True);


      ClkRgn     := vcTable.Make ("ClkRgn", 1,2,   
                                  ClkEdge,
                                  ClkRF);


      ClkPinForm := vcRegion.Create("ClkPinFormRgn", 
                                       vcCmdline.fac, S.ClockPin,
                                       max_width <- 12, 
                                       width <- 9,
                                       macro_on <- TRUE);


      CLOCK_PIN  := cons("Include Clock Pin",  ClkPinForm);

      ClkPin     := vcRegion.Create("ClkPinPickRgn",
                                    vcPicklist.fac,
                                    list(list(CLOCK_PIN)),
                                    exclusive <- FALSE);



      ClkPinRgn    := vcRegion.Create ("ClkPinTblRgn", 
                                         vcMargin.fac,
                                         vcTable.Make ("", 1,2,   
                                                       ClkPin,
                                                       ClkPinForm));



      if (S.UseClockPin == vlTrueFalse.True) 
         vcPicklist.SetPick(ClkPin,0); /* Toggle on */
      else
         vcRegion.Enable(ClkPinForm,vlOnOff.Off,vlTrueFalse.True);
      

      DataClkTitleRgn := vcRegion.Create("TDBorder", 
                                         vcMargin.fac,
                                         t = vcRegion.Create("DataClkTitle",
                                                         vcLabel.fac, 
                                                         "Constrain MCP By:"));
      vcRegion.Font(t, vgFont.mediumi);

      DataClkPinRgn := vcRegion.Create ("DataClkRgn", 
                                         vcBorder.fac,
                                         vcTable.Make ("", 4,1,
                                                       DataClkTitleRgn,
                                                       DataRgn,
                                                       ClkRgn,
                                                       ClkPinRgn));
      vcCmdline.CallOnChange(ClkPinForm, SetClkPin); 
      vcPicklist.Notify     (ClkPin,    UseClkPin);
      vcPicklist.Notify     (DataEdge,  UseDataEdge);
      vcPicklist.Notify     (ClkEdge,   UseClkEdge);
      vcPicklist.Notify     (DataRF,    SetDataEdge);
      vcPicklist.Notify     (ClkRF,     SetClkEdge);


      /*---------------------------------------------*/
      /*   Set up the number of Setup/Hold Cycles    */
      /*---------------------------------------------*/

      SetupCycleLabel := vcRegion.Create("Setup_Cycles", vcLabel.fac, "Cycles for Setup");
      HoldCycleLabel  := vcRegion.Create("Hold_Cycles",  vcLabel.fac, "Cycles for Hold");

      SetupCycleRgn   := vcRegion.Create("SetupCycleScalebar", 
                                         vcScalebar.fac, 1,
                                         increment <- 1,
                                         min <- 0,
                                         max <- 10,
                                         ticks <- 5,
                                         clip <- FALSE);
      vcScalebar.CallOnChange(SetupCycleRgn, SetSetupCycles);   

      HoldCycleRgn    := vcRegion.Create("HoldCycleScalebar", 
                                         vcScalebar.fac, 0,
                                         increment <- 1,
                                         min <- 0,
                                         max <- 10,
                                         ticks <- 5,
                                         clip <- FALSE);
      vcScalebar.CallOnChange(HoldCycleRgn, SetHoldCycles);   

      SetupHoldRgn = vcRegion.Create("SetupHoldRgn", vcBorder.fac, 
                                     vcTable.Make("SetupHoldTbl", 2, 2, 
                                                  SetupCycleLabel, SetupCycleRgn,
                                                  HoldCycleLabel,  HoldCycleRgn));
      

      
      /*--------------------------------------------*/
      /*   Create the button row along the bottom   */
      /*--------------------------------------------*/

      buttons := vcRegion.Create("Buttons", vcButnRow.fac,
                                 list (list ("Accept",
                                             "Accept",
                                             Accept,
                                             list (close_window <- TRUE)),
                                       list ("Cancel",
                                             "Cancel",
                                             Cancel,
                                             list (close_window <- TRUE))));


      /*---------------------------------------------------*/
      /*     Set up the MCP Type Picklist region           */
      /*---------------------------------------------------*/

      CONST_MCP := cons("Constant MCP", list(Mcp.Constant,"Constant MCP",SetupHoldRgn));
      CYCLE_MCP := cons("Cycle MCP",    list(Mcp.Cycle,"Cycle MCP",SetupHoldRgn));  

      McpModeTitle:= vcRegion.Create("Mcp_ModeTitle", 
                                     vcLabel.fac, 
                                     "Type of MCP to Generate:");

      McpPickRgn  := vcRegion.Create("MCP_ModeMargin",
                                     vcMargin.fac,
                                     vcRegion.Create("MCP_Mode",
                                                     vcPicklist.fac,
                                                     list(list(CONST_MCP,CYCLE_MCP)),
                                                     exclusive <- vlTrueFalse.True,
                                                     font <- vgFont.large));

      if (S.McpType == Mcp.Constant) {
         vcPicklist.SetByString(McpPickRgn,car(CONST_MCP));
         vcRegion.Enable(SetupHoldRgn,vlOnOff.Off,vlTrueFalse.True);
      }
      else {
         vcPicklist.SetByString(McpPickRgn,car(CYCLE_MCP));
         vcRegion.Enable(SetupHoldRgn,vlOnOff.On,vlTrueFalse.True);
      }

      vcPicklist.Notify(McpPickRgn, SetMcpType );
      
      McpModeRgn  :=  vcTable.Make("McpModeTable", 1, 2, 
                                     McpModeTitle, McpPickRgn);


      /*----------------------------------------*/
      /*    Layout the Mcp Editor Window        */
      /*----------------------------------------*/
      LayoutRgn :=  vcRegion.Create("TheLayout", vcMargin.fac,
                                    vcTable.Make("PathAssertion", 6, 1,
                                                 Title,
                                                 McpEditRgn,
                                                 McpModeRgn,
                                                 SetupHoldRgn,
                                                 DataClkPinRgn,
                                                 buttons));

      vcTable.AlignColumns(list(DataRgn,ClkRgn,ClkPinRgn));
      VtxData = vcRegion.GetData(McpEditRgn);   

      RedoMcp();
      
      LayoutRgn;
   } /* function BuildMcpGui */
   
   
   
   
   
   S := CreateSettings(Data);
   
   TheBox := BuildMcpGui();

   vcRegion.AddChild(TopRegion,TheBox);
      
   TheBox;
} /* function _Create */


