/* ---------------------------------------------------------------------- *\
 * Module -- vtvfopen.vs
 *
 *      Facility:      Viewtime
 *
 *      Environment:   sun4, pv 5.2
 *
 *      Author:        Mani Alagappan,  Nov 1993
 *
 *      Description:   Design/Case Setup action
 *
 *      Copyright (c) 1993 by Viewlogic Systems, Inc.
 */
/* ---------------------------------------------------------------------- *\
Modification History
$Log:	vtvfopen.vs,v $
Revision 1.1  93/11/05  11:16:43  mania
Viewtime facility written

\* ---------------------------------------------------------------------- */

vcFac.Create ("vdinifo", vscmVT);
   
msg_NullBlockName := cons(2101, "Enter a valid Design name");
msg_NullCaseName :=  cons(2102, "Enter a valid Timing Case name");

/* 
   Build the Design/Case Setup dialog box 
*/

function build_fopen_dialog(d, e, inputdesign)
{
    declare w, Table, case_cmdline;
    declare block, case;
    declare case_list;
    i := 0;
    
    function AlertBox (d, e, msgnum, severity, message) 
    {
       errobj := vlErr.Create(nil, "vtime", msgnum, severity, message);
       vcErr.Report (w,nil,errobj);
    }
    function fopen_addcase(fspec){
        <device,path,fname,ext> := vscript.vsParse (fspec);
        if (ext != "")
           fname = fname + "." + ext;
        case_list = cons (fname, case_list);
    } /* function fopen_addcase */
    
    function setcaseline(timingcase, onoff){
        if (onoff == vlOnOff.On){
            case = timingcase;
            vcCmdline.Set (case_cmdline, timingcase);
        } /* if */
    } /* function setcaseline */
    
    function build_cases(blkdir, onoff)
    {        
       if (onoff == vlOnOff.On){
          block = blkdir;
          case  = nil;
            
          vcRegion.Enable (vcTable.Get (Table, 2, 0), TRUE, TRUE);
          vcCmdline.Set (case_cmdline, "");
          design := sprintf ("%s/%s/*","tim",blkdir);
          wvproject := "";
          case_list = nil;
          r_caselist := nil;

          path := vsAbsPath (".");
          /*path = vscript.vsMerge ("", path, "tim", "");*/
          name := vscript.vsMerge ("", path, design, "");
          vscript.vsLoop (name, vscript.vsDIR, fopen_addcase);
          
            r_caselist = vcRegion.Create ("cases", vcPicklist.fac, 
                                           nil , 
                                           rows <- 10, columns <- 4,
                                           macro_on <- TRUE);            
            vcPicklist.Notify (r_caselist, setcaseline);

            if (case_list != nil){
                sort_list := vscript.Sort (case_list, string_lt);
                foreach (i; vscript.list_iter (sort_list))
                    vcPicklist.Add (r_caselist, i, i);
                vcPicklist.SetPick (r_caselist, 0);
            } /* if */
            vcTable.Set (Table, 1, 0, 
                         vcPicklist.Scroll ("case_scroll", r_caselist, 1, 2, 3));
            vcPicklist.ActivateNotify (r_caselist, 
                                       func(pick,index){
                                       case = pick;
                                       fopen_accept ();});
        } /* if */
    } /* function build_cases */

    function case_notify(design, onoff)
    {
       if (w != nil)
          vcWin.ClearStatus (w);
        block = design;
        build_cases (design, onoff);
    } /* function case_notify */
    
    function block_notify(dir, onoff)
    {
       if (w != nil)
          vcWin.ClearStatus (w);
        /*printf ("block_notify: %s\n", vcTable.Get (Table, 2, 0));*/
        if (onoff == vlOnOff.On){

           if (string_eqi (dir, "tim") == TRUE)
              vdinifo.HideLibs (vcRegion.GetData (vcTable.Get (Table, 0, 0)));
           else
              vdinifo.ShowLibs (vcRegion.GetData (vcTable.Get (Table, 0, 0)));
        }
    } /* function block_notify */

    function caseline_onreturn(e, str) i := 1;

    function fopen_apply(){
        e := vcEvent.Create (0, 0, ' ', "", w, 
                             vcWin.Region (w), nil);
        block = vdinifo.GetBlockName (vcRegion.GetData (vcTable.Get (Table, 0, 0)));
        if (block == nil){
           msg := sprintf(cdr(msg_NullBlockName));
           AlertBox(d,e,car(msg_NullBlockName),vlErrSev.Error,msg);
           return (FALSE);
        }
        block = string_trim (block);
        if ((block == nil) || string_eq (block, "")){
           msg := sprintf(cdr(msg_NullBlockName));
            AlertBox(d,e,car(msg_NullBlockName),vlErrSev.Error,msg);
            return (FALSE);
        } /* if */
        case = vcRegion.Get (case_cmdline, string);
        case = string_trim (case);
        if ((case == nil) || string_eq (case, "")){
           msg := sprintf(cdr(msg_NullCaseName));
           AlertBox(d,e,car(msg_NullCaseName),vlErrSev.Error,msg);
            return (FALSE);
        } /* if */

        /* REDOTHIS */
        /*design := sprintf ("%s %s", block, case);*/
        vcCmd.Run (timing_cmd, nil, e, list (block, case), TRUE);
    } /* function fopen_apply */

    /*fopen_apply_cmd := vcWv.Cmd (fac, fopen_apply, "setup_apply");*/

    function fopen_accept()
    {
        fopen_apply ();
        vcWin.Close (w);
    } /* function fopen_accept */
    function fopen_quit(){ 
       TRUE;
    } /*function fopen_quit */
    function setup_winclosed(win)
    {
        win = nil;
    }
    function setup_wincallonclose(data) i := 1;

    Table = vcTable.Create ("Design/Case Select", 4, 1);
    vcTable.Row (Table, -1, vcTable.Border.Visible, 5, 8, 12,
                 vgAlign.MiddleCenter);

    case_cmdline = cmdline ("", caseline_onreturn);
    vcTable.Set (Table, 2, 0, entrybox ("Case Name: ", case_cmdline));

    dirs := list (list (cons ("Analyzed Designs","tim"),
                        cons ("All Designs", "sch")));
    vcTable.Set(Table, 0, 0, 
                vcRegion.Create ("", vdinifo.fac, 
                                 list(dirs, "1", case_notify, block_notify,inputdesign)));

    vcTable.Set(Table, 3, 0, 
                vcRegion.Create("Buttons", vcButnRow.fac,
                                list (list ("Accept",
                                            "Accept",
                                            fopen_accept,
                                            list (close_window <-TRUE)),
                                      list ("Apply",
                                            "Apply",
                                            fopen_apply,
                                            nil),
                                      list ("Cancel",
                                            "Cancel",
                                            fopen_quit /*vcWin.QuitCmd*/,
                                            list (close_window <- TRUE)))));
                
    w = vcWin.Create (Table);
    vcWin.CallOnClose (w, setup_wincallonclose);
    vcWin.DestroyNotify (w, setup_winclosed);
    vdinifo.SetSrchPick (vcRegion.GetData (vcTable.Get (Table,0,0)), 0);
    w;
} /* function build_fopen_dialog */
