/*
 *  $Revision: 1.9.5.1 $
 *
 *  Copyright (c) 1992,1993 by Viewlogic Systems, Inc.
 */


/*
 * This module demonstrates how to build a command dialog box.  Command
 * dialog boxes are automatically popped up when the user runs a command
 * that takes arguments.  If any arguments are not provided by the user
 * on the command line, a dialog box is created to prompt for the remaining
 * arguments.
 *
 * Core generates this dialog box from the Command object.  A facility creates
 * a command object by calling vcWv.Cmd(man).  Commands can run ViewScript
 * functions or built-in C functions, but the first two arguments to a command
 * function are always the same:
 *
 *      1.      The facility's data structure.
 *      2.      The event object that triggered the command.
 *
 * In the example, we'll write a function that takes several arguments,
 * and build a command out of it.  Since a command must be associated with
 * a facility, we'll use Core's "Window" facility to avoid explaining the
 * mechanics of building a facility in this particular demo file.  Thus the
 * first argument to the command's action function is the Window Id of the
 * window from which the command was invoked.
 */

function sample_action(window, event, string, integer, yn, string,
                       fn, fw, fr, frw, dir, cfile)
{
    /*
     * Note that, by default, the output of this printf statement will not be
     * visible in any window.  That's because ViewScript redirects the stdout
     * to /dev/null.  To change that, set the vscript.stdout preference to
     * True in your powerview.ini file or specify it as the command line
     * option: +vscript.stdout
     */
    printf ("w=%s, e=%s, str=%s, int=%d, yn=%s, str=%s\n",
            window, event, string, integer, yn, string);

    printf ("Filename:      %s\n", fn);
    printf ("FileWrite:     %s\n", fw);
    printf ("FileRead:      %s\n", fr);
    printf ("FileReadWrite: %s\n", frw);
    printf ("Directory:     %s\n", dir);
    printf ("Cwd File:      %s\n", cfile);
}

sample_cmd := vcWv.Cmd (vcWin.fac, sample_action, "sample", "Sample Command",
                 vcWv.Param ("String", vlStringVT, vcWv.Sticky, "Hello"),
                 vcWv.Param ("Integer", vlIntVT, vcWv.Sticky, 5,
                             vcWv.Secondary),
                 vcWv.Param ("YesNo", vlYesNoVT, vcWv.Sticky, vlYesNo.Yes),
                 vcWv.Param ("String2", vlStringVT, vcWv.Sticky, "Goodbye",
                             vcWv.Secondary),
                 vcWv.Param ("Filename", vlFileNameVT, vcWv.Sticky,
                             "xyz.def"),
                 vcWv.Param ("FileWrite", vlFileWriteVT, vcWv.Sticky,
                             "/tmp/asdf"),
                 vcWv.Param ("FileRead", vlFileReadVT, vcWv.Sticky,
                             "/etc/passwd"),
                 vcWv.Param ("FileReadWrite", vlFileReadWriteVT, vcWv.Sticky,
                             "newfile"),
                 vcWv.Param ("Directory", vlStringVT, vcWv.Sticky,
                             vscript.getenv ("PWD"),
                             vcFilePop.fac, file_columns <- 0,
                             dir_columns <- 2),
                 vcWv.Param ("File From Cwd", vlStringVT, vcWv.Sticky, "",
                             vcFilePop.fac, file_columns <- 2,
                             dir_columns <- 0));
                 

vcCmd.Run (sample_cmd, demo_window,
           vcEvent.Create (0, 0, 0, nil, demo_window, demo_region, nil),
           nil, FALSE);
