/*
 *  $Revision: 1.4.6.1 $
 *
 *  This is an encapsulation of the C-shell.
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

/*
 * This function is a "command generator".  Given a list of argument
 * descriptors, as documented in vcWv.Param, it builds a command that
 * appends its arguments together, separated by spaces, and sends the
 * command to the subprocess.
 */
function cmd(cmdname, abbrev, title, params) {
    function send_strings (args...) {
        declare str, sh, e, a;

        sh  = car (args); args = cdr (args);
        e   = car (args); args = cdr (args);
        for (str = abbrev; !null(args); args = cdr (args)) {
            a = car (args);
            str = str + " " + 
                (istype (a, vlStringVT) ? a : number2string (a));
        } /* for */
        vcShell.TypeIn (sh, str, TRUE);
    }
    vcCmd.Create (fac, send_strings, cmdname, title, params);
}

/*
 * Make some simple menus to encapsulate some C-Shell operations.
 */
misc := vcWv.MakeMenu ("misc",
    list ("Show environment", cmd ("printenv", "printenv", nil, nil)),
    list ("Show users", cmd ("who", "who", nil, nil)),
    list ("Find", cmd ("find", "find", "Find Files",
                       list (vcWv.Param ("Start directory", vlStringVT,
                                         vcWv.Sticky, ".",
                                         clear_on_entry <- FALSE),
                             vcWv.Param ("Pattern", vlStringVT,
                                         vcWv.Sticky, "-name ",
                                         clear_on_entry <- FALSE),
                             vcWv.Param ("Action", vlStringVT,
                                         vcWv.Sticky, "-print",
                                         clear_on_entry <- FALSE)))));

bg := vcWv.MakeMenu ("bg",
    list ("asteroid", cmd ("", "xloadimage -onroot asteroid", nil, nil)),
    list ("fracland", cmd ("", "xloadimage -onroot fracland", nil, nil)),
    list ("hand", cmd ("", "xloadimage -onroot hand", nil, nil)),
    list ("Kate Bush", cmd ("", "xloadimage -onroot kate_bush", nil, nil)));


main_menu := vcWv.MakeMenu ("vsim",
                            vcWin.RedSquareEntryNoPlot,
                            list ("Background", bg),
                            list ("Miscellaneous", misc));

vcFac.SetMenus (fac, main_menu);

/*
 * Provide the essential Create method for the facility.  In it, just
 * instantiate a Shell, run /bin/csh, and we're on our way!
 */
function _Create(arg, region) {
    r := vcRegion.Create (vcRegion.Name (region), vcShell.fac, "/bin/csh");
    vcRegion.AddChild (region, r);
    r;
}
