/*
 *  $Revision: 1.5.7.1 $
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */


/*
 * NOTE: this is not a good demonstration on how to build picklists of
 * filenames.  It's really a demonstration of how to accomplish this via the
 * vcSys.Program function.  The vcSys.Command function could have been used as
 * well, but it's slower.
 */


/*
 * This file demonstrates building a picklist of filenames from the current
 * directory.  At some point, this will become a built-in function of Core.
 * But for now, we find out the filenames by creating a subprocess to run
 * /bin/ls, keeping track of its output, and parsing the output text into a
 * list of filenames to create the picklist with.  This demonstrates some
 * basic string handling capabilities of Viewscript, in addition to the
 * subprocess interaction, plus a programmatic way of building up picklists
 * (vcPicklist.Add).
 */


filestr := "";
picklist := vcRegion.Create1 ("picklist", vcPicklist.fac, nil,
                              list (macro_on <- TRUE)); 

/*
 * Output handler for the vcSys.Program function.
 */
function parse(out_string)
{
    filestr = filestr + out_string;
    for (end := lindex (filestr, '\n'); end; end = lindex (filestr, '\n'))
    {
        str := substring (filestr, 0, end);
        vcPicklist.Add (picklist, str, str);
        if (end >= string_length (filestr) - 1)
            return (nil);
        else
            filestr = substring (filestr, end + 1, 0);
    }
}


/*
 * Exit handler for the vcSys.Program function.
 */
function make_picklist(mode, code)
{
    scroll := vcPicklist.Scroll ("Scroll", picklist, 4, 8, 12);
    vcWin.Create (scroll, vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
}


sys := vcSys.Program ("/bin/ls", list ("-1"),
                      nil, parse, nil, make_picklist, nil);

/*
 * The following code demonstrates how to handle macro recording and playback
 * when dealing with subprocesses.  If you recorded a cross-tool macro, the
 * macro system will figure it out for itself that all the tools must up and
 * running and it will put a dialog box informing the user that it's waiting
 * for tools that haven't started yet.  With subprocesses, the situation is
 * much trickier, and you have to more pro-active in this regard.  The
 * vcMac.SubrpocessHold function will register the given subprocess Id and
 * suspend macro execution until that subprocess terminates.
 */
if (vcMac.IsReplayingOn ())
    vcMac.SubprocessHold (sys, 0);
