/*
 *  $Revision: 1.4.7.1 $
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */

/*
 * This demonstrates how two regions can alternately occupy a single spot in a
 * table.  Two labeled buttons are created.  When one of them is hit, it uses
 * vcTable.Set to switch the table entry to the other one.  Note how the
 * second button, "Show me", is a little too big for the window.  When the
 * window was initially sized, only the "Hide me" button was taken into
 * account.  See the "Hide2" example for a better implementation using the
 * vcOverlay facility.
 */

table := vcTable.Create ("Hide1", 1, 1);


/*
 * Note that the two functions are written as action routines which will be
 * passed in as arguments to the vcWv.Cmd function.  Thus, both the `hide1'
 * function and the `show1' function take two required arguments, the facility
 * data object and an event object.  Both arguments are ingored in our case,
 * but they need to be there since Core *will* always pass in these arguments
 * to any action routine.
 */

function hide(win, event)
{
    vcTable.Set (table, 0, 0, button2);
    vcRegion.Reframe (table, vcRegion.Area (table));
    vcRegion.Refresh (table, vcRegion.Area (table));
}
hide_cmd := vcWv.Cmd (vcWin.fac, hide, "hide1");

    
function show(win, event)
{
    vcTable.Set (table, 0, 0, button1);
    vcRegion.Reframe (table, vcRegion.Area (table));
    vcRegion.Refresh (table, vcRegion.Area (table));
}
show_cmd := vcWv.Cmd (vcWin.fac, show, "show1");


button1 := vcWv.LabelButtonCreate ("button1", "Hide me", nil, hide_cmd, nil);
button2 := vcWv.LabelButtonCreate ("button2", "Show me", nil, show_cmd, nil);

vcTable.Add (table, button1);
vcWin.Create (table, vcWin.NoMenus + vcWin.NoCmdline + vcWin.NoStatus);
