/*
 *  $Revision: 1.8.4.1 $
 #
 *  Example facility to interface with IPC.
 *
 *  Copyright (c) 1992 by Viewlogic Systems, Inc.
 */


/* Initialize Ipc */
src := vipc.Init (vipc.TidVLOAD);


    /*
     * Create Top level table with 2 rows and 1 column
     *
     * The First row will be another table which will be a header and the
     * the second row will the vipctst input region.
     */
    table_top := vcTable.Create ("Top", 2, 1);
    vcTable.Col
        (table_top, -1, vcTable.Border.Visible, 4, 4, 4, vgAlign.MiddleLeft);


    /*
     * Lable function - for making some of the regions into lables
     */
    function label(str)
        vcRegion.Create1 (str, vcLabel.fac, str, nil);


    /*
     * Create Header table with 3 rows and 2 columns
     *
     * The first row will be the message type lable and its value.
     * The second row will be the changeable contexts and their values.
     * The third row will be the flags and their buttons.
     */
    table_header := vcTable.Create ("Header", 3, 2);
    vcTable.Col
        (table_header, 0, vcTable.Border.Invisible, 4, 4, 4,
         vgAlign.MiddleCenter);
    vcTable.Col
        (table_header, 1, vcTable.Border.Visible, 4, 4, 4, vgAlign.MiddleLeft);
    vcTable.Row
        (table_header, -1, vcTable.Border.Visible, 4, 4, 4, vgAlign.MiddleCenter);


    function cmdline (name, dflt, width)
    {
        args := list (macro_on <- TRUE, reset_clear_on_entry <- TRUE);

        if (width != nil)
            args = cons (width <- width, args);

        vcRegion.Create1 (name, vcCmdline.fac, dflt, args);


    } /* cmdline */



    /*
     * Create the message type row
     */
    mtype := 13;
    mtype_graph := vcTable.Create ("MType:", 1, 2);
    vcTable.Col
        (mtype_graph, 1, vcTable.Border.Visible, 4, 4, 4, vgAlign.MiddleLeft);
    mtype_region := cmdline ("header0:1", "13", 5);
    vcCmdline.CallOnChange
        (mtype_region, func (str) {
            mtype = convert (integer, str);
            vcTable.Set (mtype_graph, 0, 0,
                         label (sprintf (" %-45s ", vipc.MsgString (mtype))));
        });
    vcTable.Set (mtype_graph, 0, 0,
                 label (sprintf (" %-45s ", vipc.MsgString (mtype))));
    vcTable.Set (mtype_graph, 0, 1, mtype_region);
    vcTable.Set (table_header, 0, 1, mtype_graph);
    function mtype_func (data, event, val)
    {
        mtype = val;
        vcTable.Set (mtype_graph, 0, 0,
                     label (sprintf (" %-45s ", vipc.MsgString (mtype))));
        mtype_region = cmdline ("header0:1", mtype, 5);
        vcCmdline.CallOnChange
            (mtype_region, func (str) {
                mtype = convert (integer, str);
                vcTable.Set (mtype_graph, 0, 0,
                             label (sprintf (" %-45s ", vipc.MsgString (mtype))));
            });
        vcTable.Set (mtype_graph, 0, 1, mtype_region);
    };
    mtype_cmd := vcWv.Cmd
        (fac, mtype_func, "mtype", "mtype",
         vcWv.Param ("Message Type:", integer, vcWv.Sticky, 0));
    function mentry (mtype)
        list (vipc.MsgString (mtype), sprintf ("mtype %d", mtype));

    l := nil;
    function no_nil (x)
        if (x == nil)
            list ("");
        else
            x;

    function l_to_triple (l)
        if (l == nil)
            nil;
        else
            cons (list (no_nil (car (l)), no_nil (cadr (l)), no_nil (caddr (l))),
                  l_to_triple (cdddr (l)));

    for (i:=vipc.MsgLAST-1; i>0; --i) {
        l = cons (mentry (i), l);
    }
    l = vscript.Sort (l, func(x, y) {
        if (x == nil)
            TRUE;
        else if (y == nil)
            FALSE;
        else
            string_lei (car (x), car (y));
    });

    l = l_to_triple (l);
    mtype_button := apply (vcWv.make_menu, cons ("Mtype", l));
    /*vcPicklist.Notify (mtype_button, mtype_button_func ());*/
    vcTable.Set (table_header, 0, 0,
                 vcWv.PullDownMenu ("0:0", "Mtype", mtype_button));



    /*
     * Create the context row
     */
    vcTable.Set (table_header, 1, 0, label ("Contexts: "));

    table_graph := vcTable.Create ("Context", 4, 8);
    vcTable.Row
        (table_graph, -1, vcTable.Border.Visible, 4, 4, 4, vgAlign.LowerLeft);
    vcTable.Col
        (table_graph, -1, vcTable.Border.Visible, 8, 8, 8, vgAlign.LowerCenter);

    vcTable.Set (table_graph, 1, 0, label ("Source"));
    vcTable.Set (table_graph, 1, 1, label (vipc.CntxGetGid (vipc.CntxDflt)));
    vcTable.Set (table_graph, 1, 2, label (vipc.CntxGetUid (vipc.CntxDflt)));
    vcTable.Set (table_graph, 1, 3, label (vipc.CntxGetFid (vipc.CntxDflt)));
    vcTable.Set (table_graph, 1, 4, label (vipc.CntxGetTid (vipc.CntxDflt)));
    vcTable.Set (table_graph, 1, 5, label (vipc.CntxGetHnm (vipc.CntxDflt)));
    vcTable.Set (table_graph, 1, 6, label (vipc.CntxGetPid (vipc.CntxDflt)));
    vcTable.Set (table_graph, 1, 7, label (vipc.CntxGetCid (vipc.CntxDflt)));

    function setup_field(var, row, col, type, val) {
        *var = val;
        cl := cmdline (sprintf ("cl-%d:%d", row, col), val, 5);
        vcCmdline.CallOnChange (cl, func (str) *var = convert (type, str););
        vcTable.Set (table_graph, row, col, cl);
    } /* function setup_field */

    declare dest_group, dest_user, dest_frame, dest_tool, dest_host, dest_channel,
        dest_process;

    vcTable.Set (table_graph,   2, 0, label ("Destination"));
    setup_field (&dest_group,   2, 1, integer, vipc.CntxGetGid (vipc.CntxDflt));
    setup_field (&dest_user,    2, 2, integer, vipc.CntxGetUid (vipc.CntxDflt));
    setup_field (&dest_frame,   2, 3, integer, vipc.CntxGetFid (vipc.CntxDflt));
    setup_field (&dest_tool,    2, 4, integer, vipc.TidANY);
    setup_field (&dest_host,    2, 5, vlStringVT,  vipc.HnmANY);
    setup_field (&dest_process, 2, 6, integer, vipc.PidANY);
    setup_field (&dest_channel, 2, 7, integer, vipc.CidANY);

    declare match_group, match_user, match_frame, match_tool, match_host,
        match_channel, match_process;

    vcTable.Set (table_graph,    3, 0, label ("Match"));
    setup_field (&match_group,   3, 1, integer, vipc.GidANY);
    setup_field (&match_user,    3, 2, integer, vipc.UidANY);
    setup_field (&match_frame,   3, 3, integer, vipc.FidANY);
    setup_field (&match_tool,    3, 4, integer, vipc.TidANY);
    setup_field (&match_host,    3, 5, vlStringVT,  vipc.HnmANY);
    setup_field (&match_process, 3, 6, integer, vipc.PidANY);
    setup_field (&match_channel, 3, 7, integer, vipc.CidANY);

    vcTable.Set (table_graph, 0, 1, label ("Group"));
    vcTable.Set (table_graph, 0, 2, label ("User"));
    vcTable.Set (table_graph, 0, 3, label ("Framework"));
    vcTable.Set (table_graph, 0, 4, label ("Tool"));
    vcTable.Set (table_graph, 0, 5, label ("Host"));
    vcTable.Set (table_graph, 0, 6, label ("Process"));
    vcTable.Set (table_graph, 0, 7, label ("Channel"));
    vcTable.Set (table_header, 1, 1, table_graph);


    /*
     * Create and set the Flag table
     */
    vcTable.Set (table_header, 2, 0, label ("Flags: "));

    flag_table := vcTable.Create  ("Flags", 1, 4);

    flag_send_region := vcRegion.Create1
        ("flag0:0", vcPicklist.fac,
         list (list (cons ("Multicast", vipc.FlgMULTI),
                     cons ("Direct", vipc.FlgDIRECT))),
         list (macro_on <- TRUE));
    vcRegion.FocusInhibit (flag_send_region);
    send_flag := vipc.FlgMULTI;
    vcPicklist.Notify (flag_send_region, func (data, onoff_flag) send_flag = convert (integer, data););
    vcPicklist.SetPick (flag_send_region, 0);
    vcTable.Add (flag_table, label ("Send: "));
    vcTable.Add (flag_table, flag_send_region);

    flag_reply_region := vcRegion.Create1
        ("flag0:0", vcPicklist.fac,
         list (list (cons ("Many", vipc.FlgMANY),
                     cons ("Single", vipc.FlgSINGLE))),
         list (macro_on <- TRUE));
    vcRegion.FocusInhibit (flag_reply_region);
    reply_flag := vipc.FlgMULTI;
    vcPicklist.Notify (flag_reply_region, func (data, onoff_flag) reply_flag = convert (integer, data););
    vcPicklist.SetPick (flag_reply_region, 0);
    vcTable.Add (flag_table, label ("Reply: "));
    vcTable.Add (flag_table, flag_reply_region);

    vcTable.Row
        (flag_table, 0, vcTable.Border.Invisible, 10, 10, 10, vgAlign.MiddleRight);
    vcTable.Col
        (flag_table, 0, vcTable.Border.Invisible, 10, 10, 10, vgAlign.LowerLeft);
    vcTable.Col
        (flag_table, 1, vcTable.Border.Invisible, 1, 1, 1, vgAlign.LowerLeft);
    vcTable.Col
        (flag_table, 2, vcTable.Border.Invisible, 10, 10, 10, vgAlign.LowerRight);
    vcTable.Col
        (flag_table, 3, vcTable.Border.Invisible, 1, 1, 1, vgAlign.LowerLeft);
    vcTable.Set (table_header, 2, 1, flag_table);


    /*
     * Add Header to Top
     */
    vcTable.Set (table_top, 0, 0, table_header);


    /*
     * Create the input region
     */
    input_region := vcRegion.Create ("input", vcTxShell.fac, "",
                                     prompt     <- "vipctst> ",
                                     macro_on   <- TRUE);

    vcTxShell.CallOnReturn (input_region,
                            func(event, str)
                            {
                                if ((err := vcWv.ErrorCheck
                                     (func ()
                                      vcWin.RunCmdline (vcEvent.Window (event),
                                                        str, event);)) != nil)
                                    vcTxShell.PrintString (input_region, err);
                            });

    vcTable.Set (table_top, 1, 0, input_region);

    function p(str)
        if (input_region != nil)
            vcTxShell.PrintString (input_region, str);

    vcRegion.DestroyNotify (input_region, func(r) input_region = nil;);


function _Create(data, region) {


    /*
     * Add the region to the main window, and make sure the input
     * region gets the focus.
     */
    vcRegion.AddChild (region, table_top);
    vcRegion.SetFocus (region, input_region);


    /*
     * Return the input region as the data object to be associated with
     * this facility instance.
     */
    region;

} /* _Create */


/*
 * Help function
 */
function help (data, event, str)
{
    str = convert (vlStringVT, str);

    if (string_eq (str, "send")) {
        p ("  send [vlStringVT]\n");
        p ("    The send command will perform a message send\n");
        p ("    with the contents of the appropriate\n");
        p ("    variable for the current message type.\n");
    }

    else if (string_eq (str, "call")) {
        p ("  call [vlStringVT]\n");
        p ("    The call command will perform a message call\n");
        p ("    with the contents of the appropriate\n");
        p ("    variable for the current message type.\n");
    }

    else if (string_eq (str, "sys")) {
        p ("  sys [vlStringVT]\n");
        p ("    The sys command will make a system call with its\n");
        p ("    argument.  The vipctst routine will wait for the\n");
        p ("    process to complete.\n");
    }

    else if (string_eq (str, "quit")) {
        p ("  quit\n");
        p ("     Exit the vipctst program\n");
    }

    else if (string_eq (str, "msgs")) {
        p ("  msgs\n");
        for (i:=vipc.MsgACK; i<vipc.MsgLAST; i=i+1)
            p (sprintf ("    %3d (%s)\n", i, vipc.MsgString (i)));
    }

    else if (string_eq (str, "tids")) {
        p ("  tids\n");
        for (i:=vipc.TidANY; i<vipc.TidLAST; i=i+1)
            p (sprintf ("    %3d (%s)\n", i, vipc.TidString (i)));
    }

    else if (string_eq (str, "cids")) {
        p ("  cids\n");
        for (i:=vipc.CidANY; i<vipc.CidLAST; i=i+1)
            p (sprintf ("    %3d (%s)\n", i, vipc.CidString (i)));
    }

    else {
        p ("  Commands: \n");
        p ("    help [send call sys msgs tids cids]\n");
        p ("    send [vlStringVT]\n");
        p ("    call [vlStringVT]\n");
        p ("    sys [vlStringVT]\n");
        p ("    quit\n");
        p ("    msgs\n");
        p ("    tids\n");
        p ("    cids\n");
    }


} /* help */



/*
 * Send function
 */
function send (data, event, str)
{
    dest := vipc.CntxCreate
        (vipctst.dest_group, vipctst.dest_user,
         vipctst.dest_frame, vipctst.dest_tool,
         vipctst.dest_host,
         vipctst.dest_process, vipctst.dest_channel);

    p ("  Send:\n");
    p (sprintf ("    Mtype: %d (%s)\n", mtype, vipc.MsgString (mtype)));
    p (sprintf ("    Dest:   %s\n", dest));

    if (mtype == vipc.MsgSEND_CHANREGLST)
        str = vipc.CntxCreate
            (match_group, match_user, match_frame, match_tool, match_host,
             match_process, match_channel);

    p (sprintf ("    Data:   %s\n", str));
    vipc.Send (mtype, vipc.CntxDflt, dest, str, send_flag);


} /* send */



/*
 * Msg handler routine
 */
function msg_handler(mtype, src, dest, dlen, dtype, data)
{
    p ("  Ipc Message:\n");
    p (sprintf ("    Mtype: %d\n", mtype));
    p (sprintf ("    Src:   %s\n", src));
    p (sprintf ("    Dest:  %s\n", dest));
    p (sprintf ("    Data:  %s\n", data));
    if (mtype == vipc.MsgCMD)
        vcWin.RunCmdline (demo_window, data,
                          vcEvent.Create (0, 0, 0, data, demo_window,
                                          vcWin.OwnerRegion (demo_window),
                                          nil));
}


/*
 * Call function
 */
function call (data, event, str)
{
    dest := vipc.CntxCreate
        (vipctst.dest_group, vipctst.dest_user,
         vipctst.dest_frame, vipctst.dest_tool,
         vipctst.dest_host,
         vipctst.dest_process, vipctst.dest_channel);

    flag := send_flag + reply_flag;

    p ("  Call:\n");
    p (sprintf ("    Dest:   %s\n", dest));

    vipc.Call
        (mtype, vipc.CntxDflt, dest, str, vipctst.msg_handler, nil, flag);


} /* call */



/*
 * Write the session into a log file.
 */
function write_log(region, event, filename)
    vtx.Writeto (region, filename);


/*
 * Build command objects for subsequent key and menu bindings.
 */
help_cmd := vcWv.Cmd
    (fac, help, "help", "Help",
     vcWv.Param ("String:", vlStringRestOfLineVT, vcWv.Sticky, ""));
send_cmd := vcWv.Cmd
    (fac, send, "send", "Send",
     vcWv.Param ("String:", vlStringVT, vcWv.Sticky, "foo"));
call_cmd := vcWv.Cmd
    (fac, call, "call", "Call",
     vcWv.Param ("String:", vlStringVT, vcWv.Sticky, "foo"));
log_cmd := vcWv.Cmd
    (fac, write_log, "log", "Write Log File",
     vcWv.Param ("Log name", vlStringVT, vcWv.Sticky,""));


/*
 * Construct the menu line.  Note that each menu has one option, thus in
 * reality they become buttons.
 */
menus := vcWv.MakeMenu ("vscmEval",
                        vcWin.RedSquareEntryNoPlotNoQuit,
                        list ("Help", help_cmd),
                        list ("Send", send_cmd),
                        list ("Call", call_cmd),
                        list ("Log session", log_cmd));

vcFac.SetMenus (fac, menus);


vipc.Register
    (vipc.MsgACK, vipc.CntxDflt, vipc.CntxFid, msg_handler, nil, 0);
vipc.Register
    (vipc.MsgPING, vipc.CntxDflt, vipc.CntxFid, msg_handler, nil, 0);
vipc.Register
    (vipc.MsgPONG, vipc.CntxDflt, vipc.CntxFid, msg_handler, nil, 0);
vipc.Register
    (vipc.MsgCMD, vipc.CntxDflt, vipc.CntxFid, msg_handler, nil, 0);
vipc.Register
    (vipc.MsgSEND_CHANREGLST, vipc.CntxDflt, vipc.CntxFid, msg_handler,
     nil, 0);
