entity BD4CU is
generic(
         A_IO_TUP: time := 0 PS; A_IO_TDN: time := 0 PS;
         EN_IO_TUP: time := 0 PS; EN_IO_TDN: time := 0 PS;
         TN_IO_TUP: time := 0 PS; TN_IO_TDN: time := 0 PS;
         PI_IO_TUP: time := 0 PS; PI_IO_TDN: time := 0 PS;
         IO_TUP: time := 2040 PS; IO_TDN: time := 1830 PS;
         ZI_TUP: time := 500 PS; ZI_TDN: time := 570 PS;
         PO_TUP: time := 230 PS; PO_TDN: time := 170 PS;
         E_TUP: time := 410 PS; E_TDN: time := 260 PS;
         Y_TUP: time := 470 PS; Y_TDN: time := 290 PS);
port(  IO: inout vlbit := 'X';
      ZI, PO: out vlbit := 'X';
      A, EN, TN, PI : in vlbit := 'X');
end BD4CU;
architecture structural of BD4CU is
   component T_NOT
      generic ( O1_TUP, O1_TDN: time := 0 PS);
      port    ( O1: out vlbit := 'X';
                I1: in vlbit := 'X');
   end component;
   component NAND_2
      generic ( O1_TUP, O1_TDN: time := 0 PS);
      port    ( O1: out vlbit := 'X';
                I1, I2: in vlbit := 'X');
   end component;
   component OR_2
      generic ( O1_TUP, O1_TDN: time := 0 PS);
      port    ( O1: out vlbit := 'X';
                I1, I2: in vlbit := 'X');
   end component;
   component AND_3
      generic ( O1_TUP, O1_TDN: time := 0 PS);
      port    ( O1: out vlbit := 'X';
                I1, I2, I3: in vlbit := 'X');
   end component;
   component CBUF
      generic ( O1_TUP, O1_TDN: time := 0 PS);
      port    ( O1: out vlbit := 'X';
                I1, I2: in vlbit := 'X');
   end component;
   component T_BUF
      generic ( O1_TUP, O1_TDN: time := 0 PS);
      port    ( O1: out vlbit := 'X';
                I1: in vlbit := 'X');
   end component;
   -- IO connect pins 
   signal ZI_IO, A_IO, EN_IO, TN_IO, PI_IO
          : vlbit := 'X';
   -- internal pins 
   signal T_E, T_Y, T_P, T_N: vlbit := 'X';
begin
  BD4CU_0: A_IO <= transport A after A_IO_TUP when A >= A_IO else
                         A after A_IO_TDN when A <= A_IO else
                         A after A_IO_TUP when A_IO_TDN < A_IO_TUP else
                         A after A_IO_TDN;
  BD4CU_1: EN_IO <= transport EN after EN_IO_TUP when EN >= EN_IO else
                         EN after EN_IO_TDN when EN <= EN_IO else
                         EN after EN_IO_TUP when EN_IO_TDN < EN_IO_TUP else
                         EN after EN_IO_TDN;
  BD4CU_2: TN_IO <= transport TN after TN_IO_TUP when TN >= TN_IO else
                         TN after TN_IO_TDN when TN <= TN_IO else
                         TN after TN_IO_TUP when TN_IO_TDN < TN_IO_TUP else
                         TN after TN_IO_TDN;
  BD4CU_3: PI_IO <= transport PI after PI_IO_TUP when PI >= PI_IO else
                         PI after PI_IO_TDN when PI <= PI_IO else
                         PI after PI_IO_TUP when PI_IO_TDN < PI_IO_TUP else
                         PI after PI_IO_TDN;
  BD4CU_4 : T_NOT
     generic map (E_TUP, E_TDN)
     port    map ( T_E, EN_IO);
  BD4CU_5 : NAND_2
     generic map (Y_TUP, Y_TDN)
     port    map ( T_Y, A_IO, TN_IO);
  BD4CU_6 : OR_2
     generic map ( open, open)
     port    map ( T_P, EN_IO, T_Y);
  BD4CU_7 : AND_3
     generic map ( open, open)
     port    map ( T_N, T_E, T_Y, TN_IO);
  BD4CU_8 : CBUF
     generic map (IO_TUP, IO_TDN)
     port    map ( IO, T_P, T_N);
  BD4CU_9 : T_BUF
     generic map (ZI_TUP, ZI_TDN)
     port    map ( ZI_IO, IO);
  BD4CU_10 : NAND_2
     generic map (PO_TUP, PO_TDN)
     port    map ( PO, ZI_IO, PI_IO);
  BD4CU_11: ZI <= transport ZI_IO after 0 PS;
end structural;
-- end module
-- cell desc Bidirectional Buffer 4mA         Input Buffer, CMOS, pull-up 
